<?php

include APPLICATION_PATH . '/Models/RendezVous.php';

class AppointmentDAO
{

    public $entityManager;
    private $modelName = "\Application\Models\RendezVous";

    function __construct()
    {
        $registry = Zend_Registry::getInstance();
        $this->entityManager = $registry->entitymanager;
    }

    /**
     * 
     * @return \Application\Models\RendezVous
     */
    public function create()
    {
        return new \Application\Models\RendezVous();
    }

    /**
     * 
     * @param Application\Models\RendezVous $rendezVous
     */
    public function save($rendezVous)
    {
        $this->entityManager->persist($rendezVous);
        $this->entityManager->flush();
    }

    /**
     * 
     * @param Application\Models\RendezVous $rendezVous
     */
    public function update($rendezVous)
    {
        $this->entityManager->merge($rendezVous);
        $this->entityManager->flush();
    }

    /**
     * 
     * @param int $id
     * @return Application\Models\RendezVous
     */
    public function getAppointmentById($id)
    {
        return $this->entityManager->getRepository($this->modelName)->find($id);
    }

    /**
     * @param DateTime $startDate
     * @param DateTime $endDate
     * @param \Application\Models\ActiviteRendezVous $appointmentActivity
     * @param \Application\Models\PointVente $pointOfSale
     * @param \Application\Models\Personne $customer
     * @return \Application\Models\RendezVous[]
     */
    public function getAppointmentsOverPeriod($startDate, $endDate, $appointmentActivity = null, $pointOfSale = null, $customer = null)
    {
        $appointmentsQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("a")
            ->where("a.statut = 1")
            ->andWhere("a.dateDebut >= :startDate")
            ->setParameter("startDate", $startDate)
            ->andWhere("a.dateFin <= :endDate")
            ->setParameter("endDate", $endDate);
        if ($appointmentActivity != null) {
            $appointmentsQuery->andWhere("a.idActivite = :appointmentActivity")
                ->setParameter("appointmentActivity", $appointmentActivity);
        }
        if ($pointOfSale != null) {
            $appointmentsQuery->andWhere("a.idPointVente = :pointOfSale")
                ->setParameter("pointOfSale", $pointOfSale);
        }
        if ($customer != null) {
            $appointmentsQuery->andWhere("a.idClient = :customer")
                ->setParameter("customer", $customer);
        }
        return $appointmentsQuery->getQuery()->getResult();
    }

    /**
     * @param DateTime $startDate
     * @param DateTime $endDate
     * @param \Application\Models\PointVente $pointOfSale
     * @param string $gender
     * @return \Application\Models\RendezVous[]
     */
    public function getAppointmentsOverPeriodWithCustomerReadyForMessages($startDate, $endDate, $pointOfSale = null, $gender = null)
    {
        $currentDate = new DateTime();
        $appointmentsQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("a")
            ->where("a.statut = 1")
            ->andWhere("a.dateDebut >= :startDate")
            ->setParameter("startDate", $startDate)
            ->andWhere("a.dateFin <= :endDate")
            ->setParameter("endDate", $endDate)
            ->andWhere("a.dateDebut > :currentDate")
            ->setParameter("currentDate", $currentDate)
            ->join("a.idClient", "p")
            ->andWhere("p.statut = 1");
        if ($pointOfSale != null) {
            $appointmentsQuery->andWhere("a.idPointVente = :pointOfSale")
                ->setParameter("pointOfSale", $pointOfSale);
        }
        if ($gender != null) {
            $appointmentsQuery->andWhere("p.sexe = :gender")
                ->setParameter("gender", $gender);
        }
        $appointmentsQuery->andWhere($appointmentsQuery->expr()->orX(
            $appointmentsQuery->expr()->isNotNull('p.telephone'),
            $appointmentsQuery->expr()->isNotNull('p.email')
        ));
        return $appointmentsQuery->getQuery()->getResult();
    }
}
