<?php

include APPLICATION_PATH . '/Models/MvtCaisse.php';

class CashMovementDAO
{
    public $entityManager;
    private $modelName = "\Application\Models\MvtCaisse";

    function __construct()
    {
        $registry = Zend_Registry::getInstance();
        $this->entityManager = $registry->entitymanager;
    }

    public function create()
    {
        return new \Application\Models\MvtCaisse();
    }


    public function save($cashMovement)
    {
        $this->entityManager->persist($cashMovement);
        $this->entityManager->flush();
    }

    public function update($cashMovement)
    {
        $this->entityManager->merge($cashMovement);
        $this->entityManager->flush();
    }

    public function listCashMovementByCashSession($cashSession)
    {
        return $this->entityManager->getRepository($this->modelName)->findBy(array("sessionCaisse" => $cashSession,  'statut' => 1));
    }

    public function getCashMovementById($id)
    {
        return $this->entityManager->getRepository($this->modelName)->findOneBy(array('idMvtCaisse' => $id));
    }


    public function getCashMovementActiveOnCashSession($cashSession)
    {
        return $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->where("m.statut = 1")
            ->andWhere("m.valideSession = 1")
            ->andWhere("m.sessionCaisse = :cashSession")
            ->setParameter("cashSession", $cashSession)
            ->getQuery()
            ->getResult();
    }

    public function getSumOfPaymentsOnCashSession($cashSession)
    {
        $sumOfTransfersIn = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->select("SUMP(m.montant) AS Amount")
            ->where("m.valideSession = 1")
            ->andWhere("m.statut = 1")
            ->andWhere("m.idSessionCaisse = :session")
            ->setParameter("session", $cashSession)
            ->andWhere("m.modeReglement = :paymentMethod")
            ->setParameter("paymentMethod", (int)MODE_VERSEMENT_VIREMENT)
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_ENTREE)
            ->getQuery()
            ->getSingleScalarResult();

        $sumOfTransfersOut = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->select("SUMP(m.montant) AS Amount")
            ->where("m.valideSession = 1")
            ->andWhere("m.statut = 1")
            ->andWhere("m.idSessionCaisse = :session")
            ->setParameter("session", $cashSession)
            ->andWhere("m.modeReglement = :paymentMethod")
            ->setParameter("paymentMethod", (int)MODE_VERSEMENT_VIREMENT)
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_SORTIE)
            ->getQuery()
            ->getSingleScalarResult();

        $sumOfCashIn = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->select("SUMP(m.montant) AS Amount")
            ->where("m.valideSession = 1")
            ->andWhere("m.statut = 1")
            ->andWhere("m.idSessionCaisse = :session")
            ->setParameter("session", $cashSession)
            ->andWhere("m.modeReglement = :paymentMethod")
            ->setParameter("paymentMethod", (int)MODE_VERSEMENT_ESPECE)
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_ENTREE)
            ->getQuery()
            ->getSingleScalarResult();

        $sumOfCashOut = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->select("SUMP(m.montant) AS Amount")
            ->where("m.valideSession = 1")
            ->andWhere("m.statut = 1")
            ->andWhere("m.idSessionCaisse = :session")
            ->setParameter("session", $cashSession)
            ->andWhere("m.modeReglement = :paymentMethod")
            ->setParameter("paymentMethod", (int)MODE_VERSEMENT_ESPECE)
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_SORTIE)
            ->getQuery()
            ->getSingleScalarResult();

        $sumOfChecksIn = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->select("SUMP(m.montant) AS Amount")
            ->where("m.valideSession = 1")
            ->andWhere("m.statut = 1")
            ->andWhere("m.idSessionCaisse = :session")
            ->setParameter("session", $cashSession)
            ->andWhere("m.modeReglement = :paymentMethod")
            ->setParameter("paymentMethod", (int)MODE_VERSEMENT_CHEQUE)
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_ENTREE)
            ->getQuery()
            ->getSingleScalarResult();

        $sumOfChecksOut = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->select("SUMP(m.montant) AS Amount")
            ->where("m.valideSession = 1")
            ->andWhere("m.statut = 1")
            ->andWhere("m.idSessionCaisse = :session")
            ->setParameter("session", $cashSession)
            ->andWhere("m.modeReglement = :paymentMethod")
            ->setParameter("paymentMethod", (int)MODE_VERSEMENT_CHEQUE)
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_SORTIE)
            ->getQuery()
            ->getSingleScalarResult();

        $sumOfMoneyIn = $sumOfCashIn + $sumOfChecksIn + $sumOfTransfersIn;
        $sumOfMoneyOut = $sumOfCashOut + $sumOfChecksOut + $sumOfTransfersOut;

        return [
            "sumOfMoneyIn" => $sumOfMoneyIn,
            "sumOfMoneyOut" => $sumOfMoneyOut,
            "sumOfCashIn" => $sumOfCashIn,
            "sumOfCashOut" => $sumOfCashOut,
            "sumOfTransfersIn" => $sumOfTransfersIn,
            "sumOfTransfersOut" => $sumOfTransfersOut,
            "sumOfChecksIn" => $sumOfChecksIn,
            "sumOfChecksOut" => $sumOfChecksOut,
        ];
    }


    public function getCashMovementFiltered($begin = null, $end = null, $paymentMethod = null, $user = null, $type = null)
    {
        $cashMovementsList = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->where("m.statut = 1")
            ->andWhere("m.valideCaisse = 1");
        if ($begin != null) {
            $beginDate = new DateTime($begin->format("Y-m-d") . " 00:00:00");
            $cashMovementsList->andWhere(" m.dateMouvement >= :begin ")
                ->setParameter("begin", $beginDate);
        }

        if ($end != null) {
            $endDate = new DateTime($end->format("Y-m-d") . " 23:59:59");
            $cashMovementsList->andWhere(" m.dateMouvement <= :end ")
                ->setParameter("end", $endDate);
        }

        if ($paymentMethod != null) {
            $cashMovementsList->andWhere("m.modeReglement = :paymentMethod")
                ->setParameter("mode", $cashMovementsList);
        }

        if ($user != null) {
            $cashMovementsList->andWhere("m.idUtilisateur = :user")
                ->setParameter("user", $user);
        }

        if ($type != null) {
            $cashMovementsList->andWhere("m.type = :type")
                ->setParameter("type", $type);
        }
        return $cashMovementsList->getQuery()->getResult();
    }


    public function getSumOfPaymentsOnCashSessionPerDayAndPointOfSale($cashSession, $day, $pointOfSale = null)
    {
        $begin = new DateTime($day . " 00:00:00");
        $end = new DateTime($day . "23:59:59");
        $sumOfTransfersInQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->select("SUMP(m.montant) AS Amount")
            ->where("m.valideSession = 1")
            ->andWhere("m.valideCaisse = 1")
            ->andWhere("m.statut = 1")
            ->andWhere("m.idSessionCaisse = :session")
            ->setParameter("session", $cashSession)
            ->andWhere("m.modeReglement = :paymentMethod")
            ->setParameter("paymentMethod", (int)MODE_VERSEMENT_VIREMENT)
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_ENTREE)
            ->andWhere("m.dateMouvement >= :begin")
            ->setParameter("begin", $begin)
            ->andWhere("m.dateMouvement <= :end")
            ->setParameter("end", $end);
        if ($pointOfSale != null) {
            $sumOfTransfersInQuery->andWhere("m.proprietaireCaisse = :pointOfSale")
                ->setParameter("pointOfSale", $pointOfSale);
        }
        $sumOfTransfersIn = $sumOfTransfersInQuery->getQuery()->getSingleScalarResult();

        $sumOfTransfersOutQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->select("SUMP(m.montant) AS Amount")
            ->where("m.valideSession = 1")
            ->andWhere("m.valideCaisse = 1")
            ->andWhere("m.statut = 1")
            ->andWhere("m.idSessionCaisse = :session")
            ->setParameter("session", $cashSession)
            ->andWhere("m.modeReglement = :paymentMethod")
            ->setParameter("paymentMethod", (int)MODE_VERSEMENT_VIREMENT)
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_SORTIE)
            ->andWhere("m.dateMouvement >= :begin")
            ->setParameter("begin", $begin)
            ->andWhere("m.dateMouvement <= :end")
            ->setParameter("end", $end);
        if ($pointOfSale != null) {
            $sumOfTransfersOutQuery->andWhere("m.proprietaireCaisse = :pointOfSale")
                ->setParameter("pointOfSale", $pointOfSale);
        }
        $sumOfTransfersOut = $sumOfTransfersOutQuery->getQuery()->getSingleScalarResult();

        $sumOfCashInQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->select("SUMP(m.montant) AS Amount")
            ->where("m.valideSession = 1")
            ->andWhere("m.valideCaisse = 1")
            ->andWhere("m.statut = 1")
            ->andWhere("m.idSessionCaisse = :session")
            ->setParameter("session", $cashSession)
            ->andWhere("m.modeReglement = :paymentMethod")
            ->setParameter("paymentMethod", (int)MODE_VERSEMENT_ESPECE)
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_ENTREE)
            ->andWhere("m.dateMouvement >= :begin")
            ->setParameter("begin", $begin)
            ->andWhere("m.dateMouvement <= :end")
            ->setParameter("end", $end);
        if ($pointOfSale != null) {
            $sumOfCashInQuery->andWhere("m.proprietaireCaisse = :pointOfSale")
                ->setParameter("pointOfSale", $pointOfSale);
        }
        $sumOfCashIn = $sumOfCashInQuery->getQuery()->getSingleScalarResult();

        $sumOfCashOutQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->select("SUMP(m.montant) AS Amount")
            ->where("m.valideSession = 1")
            ->andWhere("m.valideCaisse = 1")
            ->andWhere("m.statut = 1")
            ->andWhere("m.idSessionCaisse = :session")
            ->setParameter("session", $cashSession)
            ->andWhere("m.modeReglement = :paymentMethod")
            ->setParameter("paymentMethod", (int)MODE_VERSEMENT_ESPECE)
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_SORTIE)
            ->andWhere("m.dateMouvement >= :begin")
            ->setParameter("begin", $begin)
            ->andWhere("m.dateMouvement <= :end")
            ->setParameter("end", $end);
        if ($pointOfSale != null) {
            $sumOfCashOutQuery->andWhere("m.proprietaireCaisse = :pointOfSale")
                ->setParameter("pointOfSale", $pointOfSale);
        }
        $sumOfCashOut = $sumOfCashOutQuery->getQuery()->getSingleScalarResult();

        $sumOfChecksInQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->select("SUMP(m.montant) AS Amount")
            ->where("m.valideSession = 1")
            ->andWhere("m.valideCaisse = 1")
            ->andWhere("m.valide")
            ->andWhere("m.statut = 1")
            ->andWhere("m.idSessionCaisse = :session")
            ->setParameter("session", $cashSession)
            ->andWhere("m.modeReglement = :paymentMethod")
            ->setParameter("paymentMethod", (int)MODE_VERSEMENT_CHEQUE)
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_ENTREE)
            ->andWhere("m.dateMouvement >= :begin")
            ->setParameter("begin", $begin)
            ->andWhere("m.dateMouvement <= :end")
            ->setParameter("end", $end);
        if ($pointOfSale != null) {
            $sumOfChecksInQuery->andWhere("m.proprietaireCaisse = :pointOfSale")
                ->setParameter("pointOfSale", $pointOfSale);
        }
        $sumOfChecksIn = $sumOfChecksInQuery->getQuery()->getSingleScalarResult();

        $sumOfChecksOutQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->select("SUMP(m.montant) AS Amount")
            ->where("m.valideSession = 1")
            ->andWhere("m.valideCaisse = 1")
            ->andWhere("m.statut = 1")
            ->andWhere("m.idSessionCaisse = :session")
            ->setParameter("session", $cashSession)
            ->andWhere("m.modeReglement = :paymentMethod")
            ->setParameter("paymentMethod", (int)MODE_VERSEMENT_CHEQUE)
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_SORTIE)
            ->andWhere("m.dateMouvement >= :begin")
            ->setParameter("begin", $begin)
            ->andWhere("m.dateMouvement <= :end")
            ->setParameter("end", $end);
        if ($pointOfSale != null) {
            $sumOfChecksOutQuery->andWhere("m.proprietaireCaisse = :pointOfSale")
                ->setParameter("pointOfSale", $pointOfSale);
        }
        $sumOfChecksOut = $sumOfChecksOutQuery->getQuery()->getSingleScalarResult();

        $sumOfMoneyIn = $sumOfCashIn + $sumOfChecksIn + $sumOfTransfersIn;
        $sumOfMoneyOut = $sumOfCashOut + $sumOfChecksOut + $sumOfTransfersOut;

        return [
            "sumOfMoneyIn" => $sumOfMoneyIn,
            "sumOfMoneyOut" => $sumOfMoneyOut,
            "sumOfCashIn" => $sumOfCashIn,
            "sumOfCashOut" => $sumOfCashOut,
            "sumOfTransfersIn" => $sumOfTransfersIn,
            "sumOfTransfersOut" => $sumOfTransfersOut,
            "sumOfChecksIn" => $sumOfChecksIn,
            "sumOfChecksOut" => $sumOfChecksOut,
        ];
    }

    public function getPeriodMoneyAndPreviousPeriodMoney(\DateTime $periodBegin, \DateTime $periodEnd, $includePreviousPeriodBalance = false, $pointOfSale = null)
    {
        $dateOfPreviousPeriod = clone $periodBegin;
        $dateOfPreviousPeriod->modify(" -1 days");
        $periodCashMovementsListQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->where("m.statut = 1")
            ->andWhere("m.valideSession = 1")
            ->andWhere("m.dateMouvement >= :begin")
            ->andWhere("m.dateMouvement < :end")
            ->setParameter("dateDebut", $periodBegin)
            ->setParameter("dateFin", $periodEnd);

        if ($pointOfSale != null) {
            $periodCashMovementsListQuery->andWhere("m.idPointVente = :pointOfSale")
                ->setParameter("pointOfSale", $pointOfSale);
        }

        $periodCashMovementsList = $periodCashMovementsListQuery->getQuery()->getResult();
        $periodTunedCashMovementsList = array();

        $previousPeriodSumOfTransfers = 0;
        $previousPeriodSumOfChecks = 0;
        $previousPeriodSumOfCash = 0;
        $previousPeriodSumOfMtnMobileMoney = 0;
        $previousPeriodSumOfMtnSimple = 0;
        $previousPeriodSumOfOrangeSimple = 0;

        $periodSumOfTransfers = 0;
        $periodSumOfChecks = 0;
        $periodSumOfCash = 0;
        $periodSumOfMtnMobileMoney = 0;
        $periodSumOfMtnSimple = 0;
        $periodSumOfOrangeSimple = 0;

        if ($includePreviousPeriodBalance) {
            $previousPeriodSumOfTransfersInQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
                ->select("SUM(m.montant) AS Amount")
                ->where("m.statut = 1")
                ->andWhere("m.valideSession = 1")
                ->andWhere("m.modeReglement = :paymentMethod")
                ->setParameter("paymentMethod", (int)MODE_VERSEMENT_VIREMENT)
                ->andWhere("m.type = :type")
                ->setParameter("type", TYPE_MVT_CAISSE_ENTREE)
                ->andWhere("m.dateMouvement < :begin")
                ->setParameter("dateDebut", $periodBegin);

            $previousPeriodSumOfTransfersOutQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
                ->select("SUM(m.montant) AS Amount")
                ->where("m.statut = 1")
                ->andWhere("m.valideSession = 1")
                ->andWhere("m.modeReglement = :paymentMethod")
                ->setParameter("paymentMethod", (int)MODE_VERSEMENT_VIREMENT)
                ->andWhere("m.type = :type")
                ->setParameter("type", TYPE_MVT_CAISSE_SORTIE)
                ->andWhere("m.dateMouvement < :begin")
                ->setParameter("dateDebut", $periodBegin);

            $previousPeriodSumOfChecksInQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
                ->select("SUM(m.montant) AS Amount")
                ->where("m.statut = 1")
                ->andWhere("m.valideSession = 1")
                ->andWhere("m.modeReglement = :paymentMethod")
                ->setParameter("paymentMethod", (int)MODE_VERSEMENT_CHEQUE)
                ->andWhere("m.type = :type")
                ->setParameter("type", TYPE_MVT_CAISSE_ENTREE)
                ->andWhere("m.dateMouvement < :begin")
                ->setParameter("dateDebut", $periodBegin);

            $previousPeriodSumOfChecksOutQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
                ->select("SUM(m.montant) AS Amount")
                ->where("m.statut = 1")
                ->andWhere("m.valideSession = 1")
                ->andWhere("m.modeReglement = :paymentMethod")
                ->setParameter("paymentMethod", (int)MODE_VERSEMENT_CHEQUE)
                ->andWhere("m.type = :type")
                ->setParameter("type", TYPE_MVT_CAISSE_SORTIE)
                ->andWhere("m.dateMouvement < :begin")
                ->setParameter("dateDebut", $periodBegin);

            $previousPeriodSumOfCashInQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
                ->select("SUM(m.montant) AS Amount")
                ->where("m.statut = 1")
                ->andWhere("m.valideSession = 1")
                ->andWhere("m.modeReglement = :paymentMethod")
                ->setParameter("paymentMethod", (int)MODE_VERSEMENT_ESPECE)
                ->andWhere("m.type = :type")
                ->setParameter("type", TYPE_MVT_CAISSE_ENTREE)
                ->andWhere("m.dateMouvement < :begin")
                ->setParameter("dateDebut", $periodBegin);

            $previousPeriodSumOfCashOutQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
                ->select("SUM(m.montant) AS Amount")
                ->where("m.statut = 1")
                ->andWhere("m.valideSession = 1")
                ->andWhere("m.modeReglement = :paymentMethod")
                ->setParameter("paymentMethod", (int)MODE_VERSEMENT_ESPECE)
                ->andWhere("m.type = :type")
                ->setParameter("type", TYPE_MVT_CAISSE_SORTIE)
                ->andWhere("m.dateMouvement < :begin")
                ->setParameter("dateDebut", $periodBegin);

            $previousPeriodSumOfMtnMobileMoneyInQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
                ->select("SUM(m.montant) AS Amount")
                ->where("m.statut = 1")
                ->andWhere("m.valideSession = 1")
                ->andWhere("m.modeReglement = :paymentMethod")
                ->setParameter("paymentMethod", (int)MODE_PAIEMENT_MOBILE_MONEY)
                ->andWhere("m.type = :type")
                ->setParameter("type", TYPE_MVT_CAISSE_ENTREE)
                ->andWhere("m.dateMouvement < :begin")
                ->setParameter("dateDebut", $periodBegin);

            $previousPeriodSumOfMtnMobileMoneyOutQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
                ->select("SUM(m.montant) AS Amount")
                ->where("m.statut = 1")
                ->andWhere("m.valideSession = 1")
                ->andWhere("m.modeReglement = :paymentMethod")
                ->setParameter("paymentMethod", (int)MODE_PAIEMENT_MOBILE_MONEY)
                ->andWhere("m.type = :type")
                ->setParameter("type", TYPE_MVT_CAISSE_SORTIE)
                ->andWhere("m.dateMouvement < :begin")
                ->setParameter("dateDebut", $periodBegin);

            $previousPeriodSumOfMtnSimpleInQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
                ->select("SUM(m.montant) AS Amount")
                ->where("m.statut = 1")
                ->andWhere("m.valideSession = 1")
                ->andWhere("m.modeReglement = :paymentMethod")
                ->setParameter("paymentMethod", (int)MODE_PAIEMENT_MOBILE_MONEY_SIMPLE)
                ->andWhere("m.type = :type")
                ->setParameter("type", TYPE_MVT_CAISSE_ENTREE)
                ->andWhere("m.dateMouvement < :begin")
                ->setParameter("dateDebut", $periodBegin);

            $previousPeriodSumOfMtnSimpleOutQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
                ->select("SUM(m.montant) AS Amount")
                ->where("m.statut = 1")
                ->andWhere("m.valideSession = 1")
                ->andWhere("m.modeReglement = :paymentMethod")
                ->setParameter("paymentMethod", (int)MODE_PAIEMENT_MOBILE_MONEY_SIMPLE)
                ->andWhere("m.type = :type")
                ->setParameter("type", TYPE_MVT_CAISSE_SORTIE)
                ->andWhere("m.dateMouvement < :begin")
                ->setParameter("dateDebut", $periodBegin);

            $previousPeriodSumOfOrangeSimpleInQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
                ->select("SUM(m.montant) AS Amount")
                ->where("m.statut = 1")
                ->andWhere("m.valideSession = 1")
                ->andWhere("m.modeReglement = :paymentMethod")
                ->setParameter("paymentMethod", (int)MODE_PAIEMENT_ORANGE_MONEY_SIMPLE)
                ->andWhere("m.type = :type")
                ->setParameter("type", TYPE_MVT_CAISSE_ENTREE)
                ->andWhere("m.dateMouvement < :begin")
                ->setParameter("dateDebut", $periodBegin);

            $previousPeriodSumOfOrangeSimpleOutQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
                ->select("SUM(m.montant) AS Amount")
                ->where("m.statut = 1")
                ->andWhere("m.valideSession = 1")
                ->andWhere("m.modeReglement = :paymentMethod")
                ->setParameter("paymentMethod", (int)MODE_PAIEMENT_ORANGE_MONEY_SIMPLE)
                ->andWhere("m.type = :type")
                ->setParameter("type", TYPE_MVT_CAISSE_SORTIE)
                ->andWhere("m.dateMouvement < :begin")
                ->setParameter("dateDebut", $periodBegin);

            if ($pointOfSale != null) {
                $previousPeriodSumOfTransfersInQuery->andWhere("m.idPointVente = :pointOfSale")
                    ->setParameter("pointOfSale", $pointOfSale);

                $previousPeriodSumOfTransfersOutQuery->andWhere("m.idPointVente = :pointOfSale")
                    ->setParameter("pointOfSale", $pointOfSale);

                $previousPeriodSumOfChecksInQuery->andWhere("m.idPointVente = :pointOfSale")
                    ->setParameter("pointOfSale", $pointOfSale);

                $previousPeriodSumOfChecksOutQuery->andWhere("m.idPointVente = :pointOfSale")
                    ->setParameter("pointOfSale", $pointOfSale);

                $previousPeriodSumOfCashInQuery->andWhere("m.idPointVente = :pointOfSale")
                    ->setParameter("pointOfSale", $pointOfSale);

                $previousPeriodSumOfCashOutQuery->andWhere("m.idPointVente = :pointOfSale")
                    ->setParameter("pointOfSale", $pointOfSale);

                $previousPeriodSumOfMtnMobileMoneyInQuery->andWhere("m.idPointVente = :pointOfSale")
                    ->setParameter("pointOfSale", $pointOfSale);

                $previousPeriodSumOfMtnMobileMoneyOutQuery->andWhere("m.idPointVente = :pointOfSale")
                    ->setParameter("pointOfSale", $pointOfSale);

                $previousPeriodSumOfMtnSimpleInQuery->andWhere("m.idPointVente = :pointOfSale")
                    ->setParameter("pointOfSale", $pointOfSale);

                $previousPeriodSumOfMtnSimpleOutQuery->andWhere("m.idPointVente = :pointOfSale")
                    ->setParameter("pointOfSale", $pointOfSale);

                $previousPeriodSumOfOrangeSimpleInQuery->andWhere("m.idPointVente = :pointOfSale")
                    ->setParameter("pointOfSale", $pointOfSale);

                $previousPeriodSumOfOrangeSimpleOutQuery->andWhere("m.idPointVente = :pointOfSale")
                    ->setParameter("pointOfSale", $pointOfSale);
            }

            $previousPeriodSumOfTransfersIn = $previousPeriodSumOfTransfersInQuery->getQuery()->getSingleScalarResult();
            $previousPeriodSumOfTransfersOut = $previousPeriodSumOfTransfersOutQuery->getQuery()->getSingleScalarResult();
            $previousPeriodSumOfChecksIn = $previousPeriodSumOfChecksInQuery->getQuery()->getSingleScalarResult();
            $previousPeriodSumOfChecksOut = $previousPeriodSumOfChecksOutQuery->getQuery()->getSingleScalarResult();
            $previousPeriodSumOfCashIn = $previousPeriodSumOfCashInQuery->getQuery()->getSingleScalarResult();
            $previousPeriodSumOfCashOut = $previousPeriodSumOfCashOutQuery->getQuery()->getSingleScalarResult();
            $previousPeriodSumOfMtnMobileMoneyIn = $previousPeriodSumOfMtnMobileMoneyInQuery->getQuery()->getSingleScalarResult();
            $previousPeriodSumOfMtnMobileMoneyOut = $previousPeriodSumOfMtnMobileMoneyOutQuery->getQuery()->getSingleScalarResult();
            $previousPeriodSumOfMtnSimpleIn = $previousPeriodSumOfMtnSimpleInQuery->getQuery()->getSingleScalarResult();
            $previousPeriodSumOfMtnSimpleOut = $previousPeriodSumOfMtnSimpleOutQuery->getQuery()->getSingleScalarResult();
            $previousPeriodSumOfOrangeSimpleIn = $previousPeriodSumOfOrangeSimpleInQuery->getQuery()->getSingleScalarResult();
            $previousPeriodSumOfOrangeSimpleOut = $previousPeriodSumOfOrangeSimpleOutQuery->getQuery()->getSingleScalarResult();

            $previousPeriodSumOfTransfers = $previousPeriodSumOfTransfersIn - $previousPeriodSumOfTransfersOut;
            $previousPeriodSumOfChecks = $previousPeriodSumOfChecksIn - $previousPeriodSumOfChecksOut;
            $previousPeriodSumOfCash = $previousPeriodSumOfCashIn - $previousPeriodSumOfCashOut;
            $previousPeriodSumOfMtnMobileMoney = $previousPeriodSumOfMtnMobileMoneyIn - $previousPeriodSumOfMtnMobileMoneyOut;
            $previousPeriodSumOfMtnSimple = $previousPeriodSumOfMtnSimpleIn - $previousPeriodSumOfMtnSimpleOut;
            $previousPeriodSumOfOrangeSimple = $previousPeriodSumOfOrangeSimpleIn - $previousPeriodSumOfOrangeSimpleOut;

            $cash = [
                "number" => 1,
                "datetime" => $dateOfPreviousPeriod->format("d/m/Y"),
                "description" => "Espèces de la veille",
                "amount" => $previousPeriodSumOfCash,
            ];
            $transfer = [
                "number" => 2,
                "datetime" => $dateOfPreviousPeriod->format("d/m/Y"),
                "description" => "Virements de la veille",
                "amount" => $previousPeriodSumOfTransfers,
            ];
            $checks = [
                "number" => 3,
                "datetime" => $dateOfPreviousPeriod->format("d/m/Y"),
                "description" => "Chèques de la veille",
                "amount" => $previousPeriodSumOfChecks,
            ];
            $orangeSimple = [
                "number" => 4,
                "datetime" => $dateOfPreviousPeriod->format("d/m/Y"),
                "description" => "Orange de la veille",
                "amount" => $previousPeriodSumOfOrangeSimple,
            ];
            $mtnSimple = [
                "number" => 5,
                "datetime" => $dateOfPreviousPeriod->format("d/m/Y"),
                "description" => "Mtn de la veille",
                "amount" => $previousPeriodSumOfMtnSimple,
            ];
            $periodTunedCashMovementsList[] = $cash;
            $periodTunedCashMovementsList[] = $transfer;
            $periodTunedCashMovementsList[] = $checks;
            $periodTunedCashMovementsList[] = $orangeSimple;
            $periodTunedCashMovementsList[] = $mtnSimple;
            $count = 6;
        } else {
            $count = 1;
        }


        foreach ($periodCashMovementsList as $cashMovement) {
            if ($cashMovement->getType() == TYPE_MVT_CAISSE_ENTREE) {
                if ($cashMovement->getModeReglement() == MODE_VERSEMENT_ESPECE) {
                    $periodSumOfCash += $cashMovement->getMontant();
                } elseif ($cashMovement->getModeReglement() == MODE_VERSEMENT_VIREMENT) {
                    $periodSumOfTransfers += $cashMovement->getMontant();
                } elseif ($cashMovement->getModeReglement() == MODE_PAIEMENT_ORANGE_MONEY_SIMPLE) {
                    $periodSumOfOrangeSimple += $cashMovement->getMontant();
                } elseif ($cashMovement->getModeReglement() == MODE_PAIEMENT_MOBILE_MONEY_SIMPLE) {
                    $periodSumOfMtnSimple += $cashMovement->getMontant();
                } else {
                    $periodSumOfChecks += $cashMovement->getMontant();
                }
            } else {
                if ($cashMovement->getModeReglement() == MODE_VERSEMENT_ESPECE) {
                    $periodSumOfCash -= $cashMovement->getMontant();
                } elseif ($cashMovement->getModeReglement() == MODE_VERSEMENT_VIREMENT) {
                    $periodSumOfTransfers -= $cashMovement->getMontant();
                } elseif ($cashMovement->getModeReglement() == MODE_PAIEMENT_ORANGE_MONEY_SIMPLE) {
                    $periodSumOfOrangeSimple -= $cashMovement->getMontant();
                } elseif ($cashMovement->getModeReglement() == MODE_PAIEMENT_MOBILE_MONEY_SIMPLE) {
                    $periodSumOfMtnSimple -= $cashMovement->getMontant();
                } else {
                    $periodSumOfChecks -= $cashMovement->getMontant();
                }
            }

            $currentCashMovement = [
                "number" => $count,
                "datetime" => $cashMovement->getDateMouvement()->format("d/m/Y H:i:s"),
                "description" => $cashMovement->getMotif(),
                "type" => $cashMovement->getType(),
                "amount" => $cashMovement->getType() == TYPE_MVT_CAISSE_ENTREE ?  $cashMovement->getMontant() : -1 * $cashMovement->getMontant(),
            ];
            $periodTunedCashMovementsList[] = $currentCashMovement;
            $count++;
        }
        $previousMoney = [
            "cash" => $previousPeriodSumOfCash,
            "transfers" => $previousPeriodSumOfTransfers,
            "checks" => $previousPeriodSumOfChecks,
            "mtnMobileMoney" => $previousPeriodSumOfMtnMobileMoney,
            "orangeSimple" => $previousPeriodSumOfOrangeSimple,
            "mtnSimple" => $previousPeriodSumOfMtnSimple,
        ];
        $currentMoney = [
            "cash" => $periodSumOfCash,
            "transfers" => $periodSumOfTransfers,
            "checks" => $periodSumOfChecks,
            "mtnMobileMoney" => $periodSumOfMtnMobileMoney,
            "orangeSimple" => $periodSumOfOrangeSimple,
            "mtnSimple" => $periodSumOfMtnSimple,
        ];
        return [
            "tunedList" => $periodTunedCashMovementsList,
            "previousPeriodMoney" => $previousMoney,
            "currentPeriodMoney" => $currentMoney
        ];
    }

    public function listEffectiveCashMovementOnDay($day)
    {
        $beginDate = new DateTime($day . " 00:00:00");
        $endDate = new DateTime($day . " 23:59:59");
        return $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->where("m.statut = 1")
            ->andWhere("m.dateMouvement >= :begin")
            ->andWhere("m.dateMouvement <= :end")
            ->setParameter("begin", $beginDate)
            ->setParameter("end", $endDate)
            ->getQuery()
            ->getResult();
    }

    public function getSumOfPaymentsInTheWholeCashRegister()
    {
        $sumOfCashIn = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->select("SUM(m.montant) AS Amount")
            ->where("m.valideSession = 1")
            ->andWhere("m.statut = 1")
            ->andWhere("m.modeReglement = :paymentMethod")
            ->setParameter("paymentMethod", MODE_VERSEMENT_ESPECE)
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_ENTREE)
            ->getQuery()
            ->getSingleScalarResult();

        $sumOfCashOut = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->select("SUM(m.montant) AS Amount")
            ->where("m.valideSession = 1")
            ->andWhere("m.statut = 1")
            ->andWhere("m.modeReglement = :paymentMethod")
            ->setParameter("paymentMethod", MODE_VERSEMENT_ESPECE)
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_SORTIE)
            ->getQuery()
            ->getSingleScalarResult();

        $sumOfTransfersIn = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->select("SUM(m.montant) AS Amount")
            ->where("m.valideSession = 1")
            ->andWhere("m.statut = 1")
            ->andWhere("m.modeReglement = :paymentMethod")
            ->setParameter("paymentMethod", MODE_VERSEMENT_VIREMENT)
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_ENTREE)
            ->getQuery()
            ->getSingleScalarResult();

        $sumOfTransfersOut = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->select("SUM(m.montant) AS Amount")
            ->where("m.valideSession = 1")
            ->andWhere("m.statut = 1")
            ->andWhere("m.modeReglement = :paymentMethod")
            ->setParameter("paymentMethod", MODE_VERSEMENT_VIREMENT)
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_SORTIE)
            ->getQuery()
            ->getSingleScalarResult();

        $sumOfChecksIn = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->select("SUM(m.montant) AS Amount")
            ->where("m.valideSession = 1")
            ->andWhere("m.statut = 1")
            ->andWhere("m.modeReglement = :paymentMethod")
            ->setParameter("paymentMethod", MODE_VERSEMENT_CHEQUE)
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_ENTREE)
            ->getQuery()
            ->getSingleScalarResult();

        $sumOfChecksOut = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->select("SUM(m.montant) AS Amount")
            ->where("m.valideSession = 1")
            ->andWhere("m.statut = 1")
            ->andWhere("m.modeReglement = :paymentMethod")
            ->setParameter("paymentMethod", MODE_VERSEMENT_CHEQUE)
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_SORTIE)
            ->getQuery()
            ->getSingleScalarResult();

        $depositCategory = $this->entityManager->getRepository("\Application\Models\CategorieDepense")->findOneBy(array("nom" => CATEGORIE_MVT_CAISSE_DEPOSIT, "statut" => 1));

        $sumOfDepositIn = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->select("SUM(m.montant) AS Amount")
            ->where("m.valideSession = 1")
            ->andWhere("m.statut = 1")
            ->andWhere("m.idCategorieDepense = :category")
            ->setParameter("category", $depositCategory)
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_ENTREE)
            ->getQuery()
            ->getSingleScalarResult();

        $sumOfDepositOut = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->select("SUM(m.montant) AS Amount")
            ->where("m.valideSession = 1")
            ->andWhere("m.statut = 1")
            ->andWhere("m.idCategorieDepense = :category")
            ->setParameter("category", $depositCategory)
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_SORTIE)
            ->getQuery()
            ->getSingleScalarResult();

        $sumOfMoneyIn = $sumOfCashIn + $sumOfTransfersIn + $sumOfChecksIn + $sumOfDepositIn;
        $sumOfMoneyOut = $sumOfCashOut + $sumOfTransfersOut + $sumOfChecksOut;
        return [
            "sumOfMoneyIn" => $sumOfMoneyIn,
            "sumOfMoneyOut" => $sumOfMoneyOut,
            "sumOfCashIn" => $sumOfCashIn,
            "sumOfCashOut" => $sumOfCashOut,
            "sumOfTransfersIn" => $sumOfTransfersIn,
            "sumOfTransfersOut" => $sumOfTransfersOut,
            "sumOfChecksIn" => $sumOfChecksIn,
            "sumOfChecksOut" => $sumOfChecksOut,
            "sumOfDepositIn" => $sumOfDepositIn,
            "sumOfDepositOut" => $sumOfDepositOut,
        ];
    }

    /**
     * 
     * @param string $day
     * @param \Application\Models\PointVente $pointOfSale
     * @return mixed[]
     */
    public function getSumOfGlobalPaymentsOnDay($day, $pointOfSale = null)
    {
        $beginDate = new DateTime("{$day} 00:00:00");
        $endDate = new DateTime("{$day} 23:59:59");
        $sumOfMoneyInQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->select("SUM(m.montant) as Amount")
            ->where("m.statut = 1")
            ->andWhere("m.valideSession = 1")
            ->andWhere("m.valideCaisse = 1")
            ->andWhere("m.dateMouvement <= :end")
            ->andWhere("m.dateMouvement >= :begin")
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_ENTREE)
            ->setParameter("end", $endDate)
            ->setParameter("begin", $beginDate);

        if ($pointOfSale != null) {
            $sumOfMoneyInQuery->andWhere("m.idPointVente = :pointOfSale")
                ->setParameter("pointOfSale", $pointOfSale);
        }

        $sumOfMoneyIn = $sumOfMoneyInQuery->getQuery()
            ->getSingleScalarResult();

        $sumOfMoneyOutQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->select("SUM(m.montant) as Amount")
            ->where("m.statut = 1")
            ->andWhere("m.valideSession = 1")
            ->andWhere("m.valideCaisse = 1")
            ->andWhere("m.dateMouvement <= :end")
            ->andWhere("m.dateMouvement >= :begin")
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_SORTIE)
            ->setParameter("end", $endDate)
            ->setParameter("begin", $beginDate);

        if ($pointOfSale != null) {
            $sumOfMoneyOutQuery->andWhere("m.idPointVente = :pointOfSale")
                ->setParameter("pointOfSale", $pointOfSale);
        }

        $sumOfMoneyOut = $sumOfMoneyOutQuery->getQuery()
            ->getSingleScalarResult();

        return [
            "sumOfMoneyIn" => (float)$sumOfMoneyIn,
            "sumOfMoneyOut" => (float)$sumOfMoneyOut,
        ];
    }

    /**
     * 
     * @param DateTime $begin
     * @param DateTime $end
     * @param \Application\Models\PointVente $pointOfSale
     * @return mixed[]
     */
    public function getSumOfGlobalPaymentsOnInterval(DateTime $begin, DateTime $end, $pointOfSale = null)
    {
        $beginDate = new DateTime($begin->format("Y-m-d") . " 00:00:00");
        $endDate = new DateTime($end->format("Y-m-d") . "23:59:59");
        $sumOfMoneyInQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->select("SUM(m.montant) as Amount")
            ->where("m.statut = 1")
            ->andWhere("m.valideCaisse = 1")
            ->andWhere("m.dateMouvement <= :end")
            ->andWhere("m.dateMouvement >= :begin")
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_ENTREE)
            ->setParameter("end", $endDate)
            ->setParameter("begin", $beginDate);

        if ($pointOfSale != null) {
            $sumOfMoneyInQuery->andWhere("m.idPointVente = :pointOfSale")
                ->setParameter("pointOfSale", $pointOfSale);
        }

        $sumOfMoneyIn = $sumOfMoneyInQuery->getQuery()
            ->getSingleScalarResult();

        $sumOfMoneyOutQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->select("SUM(m.montant) as Amount")
            ->where("m.statut = 1")
            ->andWhere("m.valideCaisse = 1")
            ->andWhere("m.dateMouvement <= :end")
            ->andWhere("m.dateMouvement >= :begin")
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_SORTIE)
            ->setParameter("end", $endDate)
            ->setParameter("begin", $beginDate);

        if ($pointOfSale != null) {
            $sumOfMoneyOutQuery->andWhere("m.idPointVente = :pointOfSale")
                ->setParameter("pointOfSale", $pointOfSale);
        }


        $sumOfMoneyOut = $sumOfMoneyOutQuery->getQuery()
            ->getSingleScalarResult();

        return [
            "sumOfMoneyIn" => (float)$sumOfMoneyIn,
            "sumOfMoneyOut" => (float)$sumOfMoneyOut,
        ];
    }

    /**
     * 
     * @param DateTime $month
     * @param \Application\Models\PointVente $pointOfSale
     * @return mixed[]
     */
    public function getSumOfGlobalPaymentsOnMonth(DateTime $month, $pointOfSale = null)
    {
        $beginDate = clone $month;
        $endDate = clone $month;
        $beginDate->modify(" first day of this month");
        $endDate->modify("last day of this month");
        return $this->getSumOfGlobalPaymentsOnInterval($beginDate, $endDate, $pointOfSale);
    }

    /**
     * 
     * @param DateTime $year
     * @param \Application\Models\PointVente $pointOfSale
     * @return mixed[]
     */
    public function getSumOfGlobalPaymentsOnYear(DateTime $year, $pointOfSale = null)
    {
        $beginDate = clone $year;
        $endDate = clone $year;
        $beginDate->modify("first day of january {$year->format(DATE_FORMAT_YEAR)}");
        $endDate->modify("last day of december {$year->format(DATE_FORMAT_YEAR)}");
        return $this->getSumOfGlobalPaymentsOnInterval($beginDate, $endDate, $pointOfSale);
    }


    public function getSumOfPaymentsOnDay($day)
    {
        $beginDate = new DateTime($day . " 00:00:00");
        $endDate = new DateTime($day . " 23:59:59");
        $sumOfCashIn = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->where("m.statut = 1")
            ->andWhere("m.valideCaisse = 1")
            ->andWhere("m.modeReglement = :paymentMethod")
            ->setParameter("paymentMethod", (int)MODE_VERSEMENT_ESPECE)
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_ENTREE)
            ->andWhere("m.dateMouvement >= :begin")
            ->andWhere("m.dateMouvement <= :end")
            ->setParameter("begin", $beginDate)
            ->setParameter("end", $endDate)
            ->getQuery()
            ->getSingleScalarResult();

        $sumOfCashOut = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->where("m.statut = 1")
            ->andWhere("m.valideCaisse = 1")
            ->andWhere("m.modeReglement = :paymentMethod")
            ->setParameter("paymentMethod", (int)MODE_VERSEMENT_ESPECE)
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_SORTIE)
            ->andWhere("m.dateMouvement >= :begin")
            ->andWhere("m.dateMouvement <= :end")
            ->setParameter("begin", $beginDate)
            ->setParameter("end", $endDate)
            ->getQuery()
            ->getSingleScalarResult();

        $sumOfTransfersIn = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->where("m.statut = 1")
            ->andWhere("m.valideCaisse = 1")
            ->andWhere("m.modeReglement = :paymentMethod")
            ->setParameter("paymentMethod", (int)MODE_VERSEMENT_VIREMENT)
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_ENTREE)
            ->andWhere("m.dateMouvement >= :begin")
            ->andWhere("m.dateMouvement <= :end")
            ->setParameter("begin", $beginDate)
            ->setParameter("end", $endDate)
            ->getQuery()
            ->getSingleScalarResult();

        $sumOfTransfersOut = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->where("m.statut = 1")
            ->andWhere("m.valideCaisse = 1")
            ->andWhere("m.modeReglement = :paymentMethod")
            ->setParameter("paymentMethod", (int)MODE_VERSEMENT_VIREMENT)
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_SORTIE)
            ->andWhere("m.dateMouvement >= :begin")
            ->andWhere("m.dateMouvement <= :end")
            ->setParameter("begin", $beginDate)
            ->setParameter("end", $endDate)
            ->getQuery()
            ->getSingleScalarResult();

        $sumOfChecksIn = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->where("m.statut = 1")
            ->andWhere("m.valideCaisse = 1")
            ->andWhere("m.modeReglement = :paymentMethod")
            ->setParameter("paymentMethod", (int)MODE_VERSEMENT_CHEQUE)
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_ENTREE)
            ->andWhere("m.dateMouvement >= :begin")
            ->andWhere("m.dateMouvement <= :end")
            ->setParameter("begin", $beginDate)
            ->setParameter("end", $endDate)
            ->getQuery()
            ->getSingleScalarResult();

        $sumOfChecksOut = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->where("m.statut = 1")
            ->andWhere("m.valideCaisse = 1")
            ->andWhere("m.modeReglement = :paymentMethod")
            ->setParameter("paymentMethod", (int)MODE_VERSEMENT_CHEQUE)
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_SORTIE)
            ->andWhere("m.dateMouvement >= :begin")
            ->andWhere("m.dateMouvement <= :end")
            ->setParameter("begin", $beginDate)
            ->setParameter("end", $endDate)
            ->getQuery()
            ->getSingleScalarResult();

        $sumOfMoneyIn = (float)$sumOfCashIn + (float)$sumOfTransfersIn + (float)$sumOfChecksIn;
        $sumOfMoneyOut = (float)$sumOfCashOut + (float)$sumOfTransfersOut + (float)$sumOfChecksOut;
        return [
            "sumOfMoneyIn" => (float)$sumOfMoneyIn,
            "sumOfMoneyOut" => (float)$sumOfMoneyOut,
            "sumOfCashIn" => (float)$sumOfCashIn,
            "sumOfCashOut" => (float)$sumOfCashOut,
            "sumOfTransfersIn" => (float)$sumOfTransfersIn,
            "sumOfTransfersOut" => (float)$sumOfTransfersOut,
            "sumOfChecksIn" => (float)$sumOfChecksIn,
            "sumOfChecksOut" => (float)$sumOfChecksOut
        ];
    }

    public function getStateOfCashRegisterOnDay($day)
    {
        $beginDate = new DateTime($day->format("Y-m-d") . " 00:00:00");
        $endDate = new DateTime($day->format("Y-m-d") . " 23:59:59");
        $sumOfCashInQuery = $this->entityManager->getRepository($this->modelName)->createQuerybuilder("m")
            ->select("SUM(m.montant)")
            ->where("m.statut = 1")
            ->andWhere("m.modeReglement = " . MODE_VERSEMENT_ESPECE)
            ->andWhere("m.valideSession = 1")
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_ENTREE);

        $sumOfCashOutQuery = $this->entityManager->getRepository($this->modelName)->createQuerybuilder("m")
            ->select("SUM(m.montant)")
            ->where("m.statut = 1")
            ->andWhere("m.modeReglement = " . MODE_VERSEMENT_ESPECE)
            ->andWhere("m.valideSession = 1")
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_SORTIE);

        $sumOfTransfersInQuery = $this->entityManager->getRepository($this->modelName)->createQuerybuilder("m")
            ->select("SUM(m.montant)")
            ->where("m.statut = 1")
            ->andWhere("m.modeReglement = " . MODE_VERSEMENT_VIREMENT)
            ->andWhere("m.valideSession = 1")
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_ENTREE);

        $sumOfTransfersOutQuery = $this->entityManager->getRepository($this->modelName)->createQuerybuilder("m")
            ->select("SUM(m.montant)")
            ->where("m.statut = 1")
            ->andWhere("m.modeReglement = " . MODE_VERSEMENT_VIREMENT)
            ->andWhere("m.valideSession = 1")
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_SORTIE);

        $sumOfChecksInQuery = $this->entityManager->getRepository($this->modelName)->createQuerybuilder("m")
            ->select("SUM(m.montant)")
            ->where("m.statut = 1")
            ->andWhere("m.modeReglement = " . MODE_VERSEMENT_CHEQUE)
            ->andWhere("m.valideSession = 1")
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_ENTREE);

        $sumOfChecksOutQuery = $this->entityManager->getRepository($this->modelName)->createQuerybuilder("m")
            ->select("SUM(m.montant)")
            ->where("m.statut = 1")
            ->andWhere("m.modeReglement = " . MODE_VERSEMENT_CHEQUE)
            ->andWhere("m.valideSession = 1")
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_SORTIE);

        $sumOfOrangeInQuery = $this->entityManager->getRepository($this->modelName)->createQuerybuilder("m")
            ->select("SUM(m.montant)")
            ->where("m.statut = 1")
            ->andWhere("m.modeReglement = " . MODE_PAIEMENT_ORANGE_MONEY_SIMPLE)
            ->andWhere("m.valideSession = 1")
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_ENTREE);

        $sumOfOrangeOutQuery = $this->entityManager->getRepository($this->modelName)->createQuerybuilder("m")
            ->select("SUM(m.montant)")
            ->where("m.statut = 1")
            ->andWhere("m.modeReglement = " . MODE_PAIEMENT_ORANGE_MONEY_SIMPLE)
            ->andWhere("m.valideSession = 1")
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_SORTIE);

        $sumOfMtnInQuery = $this->entityManager->getRepository($this->modelName)->createQuerybuilder("m")
            ->select("SUM(m.montant)")
            ->where("m.statut = 1")
            ->andWhere("m.modeReglement = " . MODE_PAIEMENT_MOBILE_MONEY_SIMPLE)
            ->andWhere("m.valideSession = 1")
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_ENTREE);

        $sumOfMtnOutQuery = $this->entityManager->getRepository($this->modelName)->createQuerybuilder("m")
            ->select("SUM(m.montant)")
            ->where("m.statut = 1")
            ->andWhere("m.modeReglement = " . MODE_PAIEMENT_MOBILE_MONEY_SIMPLE)
            ->andWhere("m.valideSession = 1")
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_SORTIE);


        if (TYPE_GESTION_CAISSE == TYPE_GESTION_CAISSE_JOURNALIERE) {

            $sumOfCashInQuery->andWhere("m.dateMouvement <= :end")
                ->setParameter("end", $endDate)
                ->andWhere("m.dateMouvement >= :begin")
                ->setParameter("begin", $beginDate);

            $sumOfCashOutQuery->andWhere("m.dateMouvement <= :end")
                ->setParameter("end", $endDate)
                ->andWhere("m.dateMouvement >= :begin")
                ->setParameter("begin", $beginDate);

            $sumOfTransfersInQuery->andWhere("m.dateMouvement <= :end")
                ->setParameter("end", $endDate)
                ->andWhere("m.dateMouvement >= :begin")
                ->setParameter("begin", $beginDate);

            $sumOfTransfersOutQuery->andWhere("m.dateMouvement <= :end")
                ->setParameter("end", $endDate)
                ->andWhere("m.dateMouvement >= :begin")
                ->setParameter("begin", $beginDate);

            $sumOfChecksInQuery->andWhere("m.dateMouvement <= :end")
                ->setParameter("end", $endDate)
                ->andWhere("m.dateMouvement >= :begin")
                ->setParameter("begin", $beginDate);

            $sumOfChecksOutQuery->andWhere("m.dateMouvement <= :end")
                ->setParameter("end", $endDate)
                ->andWhere("m.dateMouvement >= :begin")
                ->setParameter("begin", $beginDate);

            $sumOfOrangeInQuery->andWhere("m.dateMouvement <= :end")
                ->setParameter("end", $endDate)
                ->andWhere("m.dateMouvement >= :begin")
                ->setParameter("begin", $beginDate);

            $sumOfOrangeOutQuery->andWhere("m.dateMouvement <= :end")
                ->setParameter("end", $endDate)
                ->andWhere("m.dateMouvement >= :begin")
                ->setParameter("begin", $beginDate);

            $sumOfMtnInQuery->andWhere("m.dateMouvement <= :end")
                ->setParameter("end", $endDate)
                ->andWhere("m.dateMouvement >= :begin")
                ->setParameter("dateDeb", $beginDate);

            $sumOfMtnOutQuery->andWhere("m.dateMouvement <= :end")
                ->setParameter("dateFin", $endDate)
                ->andWhere("m.dateMouvement >= :begin")
                ->setParameter("dateDeb", $beginDate);
        }

        $sumOfCashIn = (float)$sumOfCashInQuery->getQuery()->getSingleScalarResult();
        $sumOfCashOut = (float)$sumOfCashOutQuery->getQuery()->getSingleScalarResult();
        $sumOfTransfersIn = (float)$sumOfTransfersInQuery->getQuery()->getSingleScalarResult();
        $sumOfTransfersOut = (float)$sumOfTransfersOutQuery->getQuery()->getSingleScalarResult();
        $sumOfChecksIn = (float)$sumOfChecksInQuery->getQuery()->getSingleScalarResult();
        $sumOfChecksOut = (float)$sumOfChecksOutQuery->getQuery()->getSingleScalarResult();
        $sumOfOrangeIn = (float)$sumOfOrangeInQuery->getQuery()->getSingleScalarResult();
        $sumOfOrangeOut = (float)$sumOfOrangeOutQuery->getQuery()->getSingleScalarResult();
        $sumOfMtnIn = (float)$sumOfMtnInQuery->getQuery()->getSingleScalarResult();
        $sumOfMtnOut = (float)$sumOfMtnOutQuery->getQuery()->getSingleScalarResult();

        return [
            "cash" => $sumOfCashIn - $sumOfCashOut,
            "orange" => $sumOfOrangeIn - $sumOfOrangeOut,
            "mtn" => $sumOfMtnIn - $sumOfMtnOut,
            "transfers" => $sumOfTransfersIn - $sumOfTransfersOut,
            "checks" => $sumOfChecksIn - $sumOfChecksOut
        ];
    }

    public function listCashMovementByOrderDateAndType($order = null, $begin = null, $end = null, $type = null)
    {

        $cashMovementList = $this->entityManager->createQueryBuilder()
            ->select('m')
            ->from('\Application\Models\MvtCaisse', 'm')
            ->innerJoin('m.idFacture', 'f')
            ->where("m.statut = 1")
            ->andWhere("f.statut = 1")
            ->andWhere("m.idFacture = f.idFacture");

        if ($order != null) {
            $cashMovementList->andWhere('m.idFacture = :order')
                ->setParameter('order', $order);
        }

        if ($type != null) {
            $cashMovementList->andWhere('m.type = :type ')
                ->setParameter('type', $type);
        }

        if ($begin != null) {
            $beginDate = new DateTime($begin->format("Y-m-d") . " 00:00:00");
            $cashMovementList->andWhere('m.dateMouvement >= :begin ')
                ->setParameter('begin', $beginDate);
        }

        if ($end != null) {
            $endDate = new DateTime($end->format("Y-m-d") . " 29:59:59");
            $cashMovementList->andWhere('m.dateMouvement <= :end ')
                ->setParameter('end', $endDate);
        }

        return $cashMovementList->getQuery()->getResult();
    }

    public function listEffectiveValidCashMovementOnCashSession($cashSession)
    {
        $cashMovementList = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->where("m.statut = 1")
            ->andWhere(' m.idSessionCaisse = :cashSession ')
            ->andWhere("m.idFacture is NULL")
            ->andWhere(' m.valideSession = 1')
            ->andWhere("m.valideCaisse = 1")
            ->setParameter('cashSession', $cashSession);

        return $cashMovementList->getQuery()->getResult();
    }

    public function getSessionCashMovementByPaymentMethod($cashSession)
    {
        $paymentList = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->join("m.idModePaiement", "mp")
            ->select("mp.code")
            ->addSelect("mp.id")
            ->addSelect("mp.designation as modePaiement")
            ->addSelect("m.type")
            ->addSelect("SUM(m.montant) as montant")
            ->where("m.statut = 1");

        $paymentList->andWhere("m.idSessionCaisse = :session")
            ->setParameter("session", $cashSession);

        $paymentList->groupBy("m.type")
            ->addGroupBy("mp.id");

        $result = $paymentList->getQuery()->getResult();

        return $result;
    }

    public function getBalanceOnSession($session)
    {
        $sumOfIn = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->select("SUM(m.montant) as Amount")
            ->where("m.statut = 1")
            ->andWhere("m.valideSession = 1")
            ->andWhere("m.idSessionCaisse = :session")
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_ENTREE)
            ->setParameter("session", $session)
            ->getQuery()
            ->getSingleScalarResult();

        $sumOfOut = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("m")
            ->select("SUM(m.montant) as Amount")
            ->where("m.statut = 1")
            ->andWhere("m.valideSession = 1")
            ->andWhere("m.idSessionCaisse = :session")
            ->andWhere("m.type = :type")
            ->setParameter("type", TYPE_MVT_CAISSE_SORTIE)
            ->setParameter("session", $session)
            ->getQuery()
            ->getSingleScalarResult();

        $balance = $sumOfIn - $sumOfOut;
        return $balance;
    }
}
