<?php

include APPLICATION_PATH . '/Models/SessionCaissier.php';

class CashSessionDAO {

    public $entityManager;
    private $modelName = "\Application\Models\SessionCaissier";
    
    function CashSessionDAO(){
        $registry = Zend_Registry::getInstance();
        $this->entityManager = $registry->entitymanager;
    }
    
    public function create()
    {
        return new \Application\Models\SessionCaissier();
    }
      
    public function save($cashSession){
        $this->entityManager->persist($cashSession);
        $this->entityManager->flush();
    }
    
    public function update($cashSession){
        $this->entityManager->merge($cashSession);
        $this->entityManager->flush();
    }

    public function getCashSessionById($id){
        $cashSession = $this->entityManager->getRepository($this->modelName)->findOneBy(array('idSessionCaisse' => $id));
        return $cashSession;
    }
    
    public function listCashSessionsByUser($user){
        $cashSessionsList = $this->entityManager->getRepository($this->modelName)->findBy(array('idUtilisateur' => $user));
        return $cashSessionsList;
    }
    
    public function listCashSessionsByDateAndUser(\DateTime $day, $user = null){
        $beginDate = new \DateTime($day->format("Y-m-d") . " 00:00:00");
        $endDate = new \DateTime($day->format("Y-m-d") . " 23:59:59");
        $cashSessionsListQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
                                      ->where("s.datePrevuCloture <= :endDate")
                                      ->andWhere("s.dateOuverture >= :beginDate")
                                      ->setParamater("endDate", $endDate)
                                      ->setParameter("beginDate" , $beginDate);
        if($user != null){
            $cashSessionsListQuery->andWhere("s.idUtilisateur = :user")
                                  ->setParameter("user" , $user);
        }

        return $cashSessionsListQuery->getQuery()->getResult();
    }
    
    public function getNotClosedCashSessionsByUserAndDate($user, \DateTime $day = null){
        $cashSessionsList = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
                                 ->andWhere("s.idUtilisateur = :user")
                                 ->andWhere("s.cloturer = :closed")
                                 ->andWhere("s.dateOuverture < :day")
                                 ->setParameter("closed" , (int)SESSION_CAISSE_NON_CLOTURE)
                                 ->setParameter("user" , $user)
                                 ->setParameter("day" , $day)
                                 ->getQuery()
                                 ->getResult();
        return $cashSessionsList;
    }
    
    public function getNotClosedCashSessionsByUser( $user = null, $pointOfSale = null ){
        $cashSessionsListQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
                                      ->where("s.cloturer = :closed")
                                      ->setParameter("closed" , (int)SESSION_CAISSE_NON_CLOTURE);

        if ( $user != null ){
            $cashSessionsListQuery->andWhere("s.idUtilisateur = :user")
                                  ->setParameter("user", $user);
        }

        if ( $pointOfSale != null ){
            $cashSessionsListQuery->andWhere("s.idPointVente = :pointvente")
                                  ->setParameter("pointvente", $pointOfSale);
        }
        return $cashSessionsListQuery->getQuery()->getResult();
    }
    
    public function checkUserCashSessionPasswordDate($day , $user , $password){
        $cashSession = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
                            ->where("s.datePrevuCloture >= :day")
                            ->andWhere("s.idUtilisateur = :user")
                            ->andWhere("s.pwdCaisse = :password")
                            ->setParameter("dateJour" , $day)
                            ->setParameter("user" , $user)
                            ->setParameter("password" , $password)
                            ->getQuery()
                            ->getOneOrNullResult();
        return $cashSession;
    }

    public function getUserClosedCashSessionOfDay(\DateTime $day, $user){
        $beginDate = new \DateTime($day->format("Y-m-d") . " 00:00:00");
        $endDate = new \DateTime($day->format("Y-m-d") . " 23:59:59");
        $cashSessionsList = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
                                 ->where("s.cloturer != :flag ")
                                 ->setParameter("flag" , (int)SESSION_CAISSE_CLOTURE)
                                 ->andWhere("s.datePrevuCloture <= :end")
                                 ->andWhere("s.dateOuverture >= :begin")
                                 ->setParameter("end" , $endDate)
                                 ->setParameter("begin", $beginDate)
                                 ->andWhere("s.idUtilisateur = :user")
                                 ->setParameter("user" , $user)
                                 ->getQuery()
                                 ->getResult();
        return $cashSessionsList;
    }
    
    public function getCashSessionsOfUser($user){
        $cashSessionsList = $this->entityManager->getRepository($this->modelName)->findBy(array("idUtilisateur" => $user));
        return $cashSessionsList;
    }

    public function getLastCashSessionOfUser($user){
      $cashSession = $this->entityManager->getRepository($this->modelName)
                          ->findOneBy(
                              array('idUtilisateur' => $user),
                              array('idSessionCaisse' => 'DESC')
                          );
      return $cashSession;
    }

    public function getNotClosedMainCashSessionsByPointOfSale( $pointOfSale = null ){
        $cashSessionsListQuery = $this->entityManager->getRepository($this->modelName)
                                      ->createQueryBuilder("s")
                                      ->join("s.idUtilisateur", "u")
                                      ->where("s.cloturer = :closed")
                                      ->setParameter("closed" , (int)SESSION_CAISSE_NON_CLOTURE)
                                      ->andWhere("u.caissierPrincipal = 1");

        if ( $pointOfSale != null ){
            $cashSessionsListQuery->andWhere("s.idPointVente = :pointvente")
                                  ->setParameter("pointvente", $pointOfSale);
        }
        return $cashSessionsListQuery->getQuery()->getResult();
    }

    public function getNotClosedUserSessionsByPointOfSale( $user, $pointOfSale = null ){
        $cashSessionsListQuery = $this->entityManager->getRepository($this->modelName)
                                      ->createQueryBuilder("s")
                                      ->join("s.idUtilisateur", "u")
                                      ->where("s.cloturer = :closed")
                                      ->setParameter("closed" , (int) SESSION_CAISSE_NON_CLOTURE)
                                      ->andWhere("s.idUtilisateur = :user")
                                      ->setParameter("user", $user);

        if ( $pointOfSale != null ){
            $cashSessionsListQuery->andWhere("s.idPointVente = :pointvente")
                                  ->setParameter("pointvente", $pointOfSale);
        }
        return $cashSessionsListQuery->getQuery()->getResult();
    }
}