<?php

include  APPLICATION_PATH. '/Models/Avoir.php';

class CreditDAO {
    
    public $entityManager;
    
    function CreditDAO(){
        $registry = Zend_Registry::getInstance();
        $this->entityManager = $registry->entitymanager;
    }
    
    /**
     * 
     * @param \Application\Models\Avoir $credit
     */
    public function save($credit){
        $this->entityManager->persist($credit);
        $this->entityManager->flush();
    }
    
    /**
     * 
     * @param \Application\Models\Avoir $credit
     */
    public function update($credit){
        $this->entityManager->merge($credit);
        $this->entityManager->flush();
    }
    
    
    public function listCredits(){
        $creditsList = $this->entityManager->getRepository("\Application\Models\Avoir")->findBy(array("statut" => 1));
        return $creditsList;
    }
    
    public function listNotValidCredits(){
        $creditsList = $this->entityManager->getRepository("\Application\Models\Avoir")->findBy(array("statut" => 1 , "valide" => (int)AVOIR_NON_VALIDE));
        return $creditsList;
    }
    
    public function listValidCredits(){
        $creditsList = $this->entityManager->getRepository("\Application\Models\Avoir")->findBy(array("statut" => 1 , "valide" => (int)AVOIR_VALIDE));
        return $creditsList;
    }
    
    public function listCreditsForOrder($order){
        $creditsList = $this->entityManager->getRepository("\Application\Models\Avoir")->findBy(array("statut" => 1 , "idFacture" => $order , "valide" => (int)AVOIR_VALIDE));
        return $creditsList;
    }
    
    public function orderHasAtLeastOneCreditAvailable( $order ){
        $credit = $this->entityManager->getRepository("\Application\Models\Avoir")->findOneBy(array("statut" => 1 , "idFacture" => $order , "valide" => (int)AVOIR_VALIDE));
        return $credit;
    }
    
    /**
     * 
     * @param integer $creditId
     * @return Application\Models\Avoir
     */
    public function getCreditById($creditId){
        $credit = $this->entityManager->getRepository("\Application\Models\Avoir")->findOneBy( array("idAvoir" => $creditId) );
        return $credit;
    }
    
    /**
     * 
     * @param \Application\Models\Facturec $order
     * @return integer
     */
    public function getSumOfCreditsOnOrder( $order ){
        $sumArray = $this->entityManager->getRepository("\Application\Models\Avoir")->createQueryBuilder("a")
                         ->select("SUM(a.montant) AS creditSum")
                         ->where("a.idFacture = :facture")
                         ->setParameter("facture" , $order)
                         ->andWhere("a.statut = 1")
                         ->andWhere("a.valide = " . AVOIR_VALIDE )
                         ->getQuery()
                         ->getScalarResult();
        return $sumArray[0]["creditSum"]? $sumArray[0]["creditSum"] : 0;
    }
    
    
    public function getSumOfCreditsOnDay( $day ){
        $beginDate = new \DateTime($day . " 00:00:00");
        $endDate = new \DateTime($day . " 23:59:59");
        $sum = $this->entityManager->getRepository("\Application\Models\Avoir")->createQueryBuilder("a")
                    ->select("SUM(a.montant) AS Amount")
                    ->where("a.statut = 1")
                    ->andWhere("a.valide = :valid")
                    ->andWhere("a.date <= :end")
                    ->andWhere("a.date >= :begin")
                    ->setParameter("end" , $endDate)
                    ->setParameter("begin" , $beginDate)
                    ->setParameter("valid" , (int)AVOIR_VALIDE)
                    ->getQuery()
                    ->getSingleScalarResult();
        return $sum;
    }
    
    public function getSumOfCreditsOnInterval( \DateTime $begin , \DateTime $end){
        $beginDate = new \DateTime($begin->format("Y-m-d") . " 00:00:00");
        $endDate = new \DateTime($end->format("Y-m-d") . " 23:59:59");
        $sum = $this->_em->getRepository("\Application\Models\Avoir")->createQueryBuilder("a")
                    ->select("SUM(a.montant) as Amount ")
                    ->where("a.statut = 1")
                    ->andWhere("a.valide = :valid")
                    ->andWhere("a.date <= :end")
                    ->andWhere("a.date >= :begin")
                    ->setParameter("end" , $endDate)
                    ->setParameter("begin" , $beginDate)
                    ->setParameter("valid" , (int)AVOIR_VALIDE)
                    ->getQuery()
                    ->getSingleScalarResult();
        return $sum;
    }
    
    public function getSumOfCreditsOnMonth( \DateTime $month ){
        $begin = clone $month;
        $end = clone $month;
        $begin->modify(" first day of this month");
        $end->modify("last day of this month");
        $sum = $this->getSumOfCreditsOnInterval($begin, $end);
        return $sum;
    }

    public function getSumOfCreditsOnYear( \DateTime $year ){
        $begin = clone $year;
        $end = clone $year;
        $begin->modify("first day of january " . $year->format("Y"));
        $end->modify("last day of december " . $year->format("y"));
        $sum = $this->getSumOfCreditsOnInterval($begin, $end);
        return $sum;
    }
    
}