<?php

include APPLICATION_PATH . '/Models/PaiementAvoir.php';

class CreditPaymentDAO {

    public $entityManager;

    function CreditPaymentDAO() {
        $registry = Zend_Registry::getInstance();
        $this->entityManager = $registry->entitymanager;
    }

    /**
     * 
     * @param \Application\Models\PaiementAvoir $creditPayment
     */
    public function save($creditPayment) {
        $this->entityManager->persist($creditPayment);
        $this->entityManager->flush();
    }

    /**
     * 
     * @param \Application\Models\PaiementAvoir $creditPayment
     */
    public function update($creditPayment) {
        $this->entityManager->merge($creditPayment);
        $this->entityManager->flush();
    }

    /**
     * 
     * @param \Application\Models\Avoir $credit
     * @return \Application\Models\PaiementAvoir
     */
    public function listPaymentsOfCredit($credit) {
        $paymentsList = $this->entityManager->getRepository('\Application\Models\PaiementAvoir')->findBy(array('idAvoir' => $credit, 'statut' => 1));
        return $paymentsList;
    }

    /**
     * 
     * @param integer $id
     * @return \Application\Models\PaiementAvoir
     */
    public function getCreditPaymentById($id) {
        $creditPayment = $this->entityManager->getRepository('\Application\Models\PaiementAvoir')->findOneBy(array('idPaiement' => $id));
        return $creditPayment;
    }
    
    /**
     * 
     * @param \Application\Models\Avoir $credit
     * @return \Application\Models\PaiementAvoir
     */
    public function getLastPaymentOnCredit($credit) {
        $creditPayment = $this->entityManager->getRepository('\Application\Models\PaiementAvoir')->findOneBy(array('idAvoir' => $credit, 'statut' => 1), array('date' => 'DESC'));
        return $creditPayment;
    }
    
    /**
     * 
     * @param \Application\Models\Avoir $credit
     * @return integer
     */
    public function getSumOfPaymentsOnCredit($credit){
        $sum = $this->entityManager->getRepository("\Application\Models\PaiementAvoir")->createQueryBuilder("p")
                    ->select("SUM(p.montant) as Amount")
                    ->where("p.statut = 1")
                    ->andWhere("p.idAvoir = :credit")
                    ->setParameter("credit" , $credit)
                    ->getQuery()
                    ->getSingleScalarResult();
        return $sum;
    }
    
    /**
     * 
     * @param integer $paymentMethod
     * @return Array of \Application\Models\PaiementAvoir
     */
    public function getPaymentsByPaymentMethod($paymentMethod) {
        $paymentsList = $this->entityManager->getRepository('\Application\Models\PaiementAvoir')->findBy(array('modePaiement' => $paymentMethod, 'statut' => 1));
        return $paymentsList;
    }
    
    public function getPaymentsByCashMovement( $cashMovement ){
        $paymentsList = $this->entityManager->getRepository("\Application\Models\PaiementAvoir")->findBy(array("idMvtCaisse" => $cashMovement));
        return $paymentsList;
    }

    /**
     * 
     * @param String $day
     * @return int
     */
    public function getSumOfPaymentsOnDay( $day ){
        $beginDate = new \DateTime($day . " 00:00:00");
        $endDate = new \DateTime($day . " 23:59:59");
        $sum = $this->entityManager->getRepository("\Application\Models\PaiementAvoir")->createQueryBuilder("p")
                    ->select("SUM(p.montant) AS Amount")
                    ->where("p.statut  = 1")
                    ->andWhere("p.datePaiement <= :end")
                    ->andWhere("p.datePaiement >= :begin")
                    ->setParameter("end" , $endDate)
                    ->setParameter("begin" , $beginDate)
                    ->getQuery()
                    ->getSingleScalarResult();
        return $sum;
    }
    
    public function getSumOfPaymentsOnInterval( \DateTime $begin , \DateTime $end){
        $beginDate = new \DateTime($begin->format("Y-m-d") . " 00:00:00");
        $endDate = new \DateTime($end->format("Y-m-d") . " 23:59:59");
        $sum = $this->entityManager->getRepository("\Application\Models\PaiementAvoir")->createQueryBuilder("p")
                    ->select("SUM(p.montant) as Amount")
                    ->where("p.statut = 1")
                    ->andWhere("p.date <= :end")
                    ->andWhere("p.date >= :begin")
                    ->setParameter("end" , $endDate)
                    ->setParameter("begin" , $beginDate)
                    ->getQuery()
                    ->getSingleScalarResult();
        return $sum;
    }

    public function getSumOfPaymentsOnMonth( \DateTime $month ){
        $beginDate = clone $month;
        $endDate = clone $month;
        $beginDate->modify(" first day of this month");
        $endDate->modify("last day of this month");
        $sum = $this->getSumOfPaymentsOnInterval($beginDate, $endDate);
        return $sum;
    }
    
    public function getSumOfPaymentsOnYear( \DateTime $year ){
        $beginDate = clone $year;
        $endDate = clone $year;
        $beginDate->modify("first day of january " . $year->format("Y"));
        $endDate->modify("last day of december " . $year->format("y"));
        $sum = $this->getSumOfPaymentsOnInterval($beginDate, $endDate);
        return $sum;
    }
    
}
