<?php

include APPLICATION_PATH . '/Models/CompteClient.php';

class CustomerAccountDAO
{
    public $entityManager;
    private $modelName = "\Application\Models\CompteClient";

    function __construct()
    {
        $registry = Zend_Registry::getInstance();
        $this->entityManager = $registry->entitymanager;
    }

    public function create()
    {
        return new \Application\Models\CompteClient();
    }

    public function save($customerAccount)
    {
        $this->entityManager->persist($customerAccount);
        $this->entityManager->flush();
    }

    public function update($customerAccount)
    {
        $this->entityManager->merge($customerAccount);
        $this->entityManager->flush();
    }

    public function getCustomerAccountById($id)
    {
        return $this->entityManager->getRepository($this->modelName)->find($id);
    }


    public function listCustomerAccounts()
    {
        return $this->entityManager->getRepository($this->modelName)->createQueryBuilder("c")
            ->where("c.statut = 1")
            ->getQuery()
            ->getResult();
    }

    public function getCustomerAccountByCustomerAndPointOfSale($customer, $pointOfSale = null)
    {
        $customerAccount = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("c")
            ->where("c.statut = 1")
            ->andWhere("c.idClient = :customer")
            ->setParameter("customer", $customer);
        if ($pointOfSale != null) {
            $customerAccount->andWhere("c.idPointVente = :pointOfSale")
                ->setParameter("pointOfSale", $pointOfSale);
        }

        return $customerAccount->getQuery()->getResult();
    }

    public function getCustomerAccountBalance($customerAccount)
    {
        $sumOfBalanceIn = $this->entityManager->getRepository("\Application\Models\MouvementCompteClient")->createQueryBuilder("mvc")
            ->select("SUM(mvc.montant) AS amount")
            ->where("mvc.statut = 1")
            ->andWhere("mvc.idCompteClient = :customerAccount")
            ->setParameter("customerAccount", $customerAccount)
            ->andWhere("mvc.typeMouvement = 0")
            ->getQuery()
            ->getSingleScalarResult();

        $sumOfBalanceOut = $this->entityManager->getRepository("\Application\Models\MouvementCompteClient")->createQueryBuilder("mvc")
            ->select("SUM(mvc.montant) AS amount")
            ->where("mvc.statut = 1")
            ->andWhere("mvc.idCompteClient = :customerAccount")
            ->setParameter("customerAccount", $customerAccount)
            ->andWhere("mvc.typeMouvement = 1")
            ->getQuery()
            ->getSingleScalarResult();

        return $sumOfBalanceIn - $sumOfBalanceOut;
    }
}
