<?php

include APPLICATION_PATH . '/Models/Facturec.php';
require_once 'Zend/Controller/Front.php';

class CustomerInvoiceDAO
{

    public $entityManager;
    private $modelName = "\Application\Models\Facturec";
    private $baseUrl;

    function __construct()
    {
        $registry = Zend_Registry::getInstance();
        $this->entityManager = $registry->entitymanager;
        $this->baseUrl = Zend_Controller_Front::getInstance()->getBaseUrl();
    }

    /**
     * @return \Application\Models\Facturec
     */
    public function create()
    {
        return new \Application\Models\Facturec();
    }

    /**
     * @param \Application\Models\Facturec $clientInvoice
     */
    public function save($clientInvoice)
    {
        $this->entityManager->persist($clientInvoice);
        $this->entityManager->flush();
    }

    /**
     * @param \Application\Models\Facturec $clientInvoice
     */
    public function update($clientInvoice)
    {
        $this->entityManager->merge($clientInvoice);
        $this->entityManager->flush();
    }

    /**
     * @param int $id
     * @return \Application\Models\Facturec
     */
    public function getCustomerInvoiceById($id)
    {
        return $this->entityManager->getRepository($this->modelName)->find($id);
    }

    public function getCustomerInvoices()
    {
        return $this->entityManager->getRepository($this->modelName)->findBy(array("statut" => 1));
    }

    public function getCountOfWaitingInvoices($currentSession = null)
    {
        $countOfWaitingInvoicesQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("f")
            ->select("COUNT(f.idFacture)")
            ->where("f.enAttente = 1")
            ->andWhere("f.statut = 1");
        if ($currentSession != null) {
            $countOfWaitingInvoicesQuery->andWhere("f.idSessionCaisser =: session")
                ->setParameter("session", $currentSession);
        }

        return $countOfWaitingInvoicesQuery->getQuery()->getSingleScalarResult();
    }

    public function getCountOfUnpaidInvoices($user = null, $session = null)
    {
        $countOfUnpaidInvoicesQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("f")
            ->select("COUNT(f.idFacture)")
            ->where("f.enAttente = 2")
            ->andWhere("f.statut = 1");
        if ($user != null) {
            $countOfUnpaidInvoicesQuery->andWhere("f.idUtilisateur = :user")
                ->setParameter("user", $user);
        }

        if ($session != null) {
            $countOfUnpaidInvoicesQuery->andWhere("f.idSessionCaissier= :session")
                ->setParameter("session", $session);
        }

        return $countOfUnpaidInvoicesQuery->getQuery()->getSingleScalarResult();
    }

    public function getUnpaidInvoices($user = null, $session = null, $customer = null)
    {
        $unpaidInvoicesListQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("f")
            ->where("f.enAttente = 2")
            ->andWhere("f.statut = 1");
        if ($user != null) {
            $unpaidInvoicesListQuery->andWhere("f.idUtilisateur = :user")
                ->setParameter("user", $user);
        }

        if ($session != null) {
            $unpaidInvoicesListQuery->andWhere("f.idSessionCaissier = :session")
                ->setParameter("session", $session);
        }

        if ($customer != null) {
            $unpaidInvoicesListQuery->andWhere("f.idPersonne = :client")
                ->setParameter("client", $customer);
        }

        return $unpaidInvoicesListQuery->getQuery()->getResult();
    }

    public function getWaitingInvoices($lastInvoiceId, $direction, $session)
    {
        $countWaitingInvoices = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("f")
            ->select("COUNT(f.idFacture)")
            ->where("f.enAttente = 1")
            ->andWhere("f.statut = 1")
            ->andWhere("f.idSessionCaissier = :session")
            ->setParameter("session", $session)
            ->getQuery()
            ->getSingleScalarResult();

        if (!$countWaitingInvoices) {
            return [
                "count" => 0,
                "current" => null,
            ];
        }

        $invoiceToShow = null;
        if (!$lastInvoiceId) {
            $invoiceToShow = $this->entityManager->getRepository($this->modelName)->findOneBy(array("enAttente" => 1, "idSessionCaissier" => $session, "statut" => 1), array("idFacture" => "ASC"));
        } else {
            if ($direction == "backward") {
                $invoiceToShow = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("f")
                    ->where("f.enAttente = 1")
                    ->andWhere("f.statut = 1")
                    ->andWhere("f.idFacture < :lastInvoiceId")
                    ->setParameter("lastInvoiceId", $lastInvoiceId)
                    ->andWhere("f.idSessionCaissier = :session")
                    ->setParameter("session", $session)
                    ->orderBy("f.idFacture", "DESC")
                    ->setMaxResults(1)
                    ->getQuery()
                    ->getSingleResult();
            } else {
                $invoiceToShow = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("f")
                    ->where("f.enAttente = 1")
                    ->andWhere("f.statut = 1")
                    ->andWhere("f.idFacture > :lastInvoiceId")
                    ->setParameter("lastInvoiceId", $lastInvoiceId)
                    ->andWhere("f.idSessionCaissier = :session")
                    ->setParameter("session", $session)
                    ->orderBy("f.idFacture", "ASC")
                    ->setMaxResults(1)
                    ->getQuery()
                    ->getSingleResult();
            }
        }

        $previousInvoice = null;
        $nextInvoice = null;
        if ($invoiceToShow != null) {
            $previousInvoice = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("f")
                ->where("f.enAttente = 1")
                ->andWhere("f.statut = 1")
                ->andWhere("f.idFacture < :invoiceToShowId")
                ->setParameter("invoiceToShowId", $invoiceToShow->getIdFacture())
                ->andWhere("f.idSessionCaissier = :session")
                ->setParameter("session", $session)
                ->orderBy("f.idFacture", "DESC")
                ->setMaxResults(1)
                ->getQuery()
                ->getSingleResult();

            $nextInvoice = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("f")
                ->where("f.enAttente = 1")
                ->andWhere("f.statut = 1")
                ->andWhere("f.idFacture > :invoiceToShowId")
                ->setParameter("invoiceToShowId", $invoiceToShow->getIdFacture())
                ->andWhere("f.idSessionCaissier = :session")
                ->setParameter("session", $session)
                ->orderBy("f.idFacture", "ASC")
                ->setMaxResults(1)
                ->getQuery()
                ->getSingleResult();
        }

        return [
            "count" => $countWaitingInvoices,
            "current" => $invoiceToShow,
            "previous" => $previousInvoice,
            "next" => $nextInvoice,
        ];
    }

    public function getWaitingReservations($lastReservationId, $user, $direction)
    {
        $countWaitingReservations = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("f")
            ->select("COUNT(f.idFacture)")
            ->where("f.valider = 0")
            ->andWhere("f.statut = 1")
            ->andWhere("f.idUtilisateur = :user")
            ->setParameter("user", $user)
            ->andWhere("f.typeVente = 2")
            ->getQuery()
            ->getSingleScalarResult();

        if (!$countWaitingReservations) {
            return [
                "count" => 0,
                "current" => null,
            ];
        }

        $reservationToShow = null;
        if (!$lastReservationId) {
            $reservationToShow = $this->entityManager->getRepository($this->modelName)->findOneBy(array("idUtilisateur" => $user, "typeVente" => 2, "valider" => 0, "statut" => 1), array("idFacture" => "ASC"));
        } else {
            if ($direction == "backward") {
                $reservationToShow = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("f")
                    ->where("f.valider = 0")
                    ->andWhere("f.statut = 1")
                    ->andWhere("f.idFacture < :lastReservationId")
                    ->setParameter("lastReservationId", $lastReservationId)
                    ->andWhere("f.typeVente = 2")
                    ->andWhere("f.idUtilisateur = :user")
                    ->setParameter("user", $user)
                    ->orderBy("f.idFacture", "DESC")
                    ->setMaxResults(1)
                    ->getQuery()
                    ->getSingleResult();
            } else {
                $reservationToShow = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("f")
                    ->where("f.valider = 0")
                    ->andWhere("f.statut = 1")
                    ->andWhere("f.idFacture > :lastReservationId")
                    ->setParameter("lastReservationId", $lastReservationId)
                    ->andWhere("f.typeVente = 2")
                    ->andWhere("f.idUtilisateur = :user")
                    ->setParameter("user", $user)
                    ->orderBy("f.idFacture", "ASC")
                    ->setMaxResults(1)
                    ->getQuery()
                    ->getSingleResult();
            }
        }

        $previousReservation = null;
        $nextReservation = null;
        if ($reservationToShow) {
            $previousReservation = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("f")
                ->where("f.valider = 0")
                ->andWhere("f.statut = 1")
                ->andWhere("f.idFacture < :reservationToShowId")
                ->setParameter("reservationToShowId", $reservationToShow->getIdFacture())
                ->andWhere("f.typeVente = 2")
                ->andWhere("f.idUtilisateur = :user")
                ->setParameter("user", $user)
                ->orderBy("f.idFacture", "DESC")
                ->setMaxResults(1)
                ->getQuery()
                ->getSingleResult();

            $nextReservation = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("f")
                ->where("f.valider = 0")
                ->andWhere("f.statut = 1")
                ->andWhere("f.idFacture > :reservationToShowId")
                ->setParameter("reservationToShowId", $reservationToShow->getIdFacture())
                ->andWhere("f.typeVente = 2")
                ->andWhere("f.idUtilisateur = :user")
                ->setParameter("user", $user)
                ->orderBy("f.idFacture", "ASC")
                ->setMaxResults(1)
                ->getQuery()
                ->getSingleResult();
        }

        return [
            "count" => $countWaitingReservations,
            "current" => $reservationToShow,
            "previous" => $previousReservation,
            "next" => $nextReservation,
        ];
    }

    /**
     * 
     * @param string $day
     * @param \Application\Models\PointVente $pointOfSale
     * @return float
     */
    public function getSumOfAmountOfInvoicesOfDay($day, $pointOfSale = null)
    {
        $begin = new DateTime($day . " 00:00:00");
        $end = new DateTime($day . " 23:59:59");
        $sumOfAmountOfInvoicesQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("f")
            ->select("SUM(f.montantNet) as Amount")
            ->where("f.statut = 1")
            ->andWhere("f.valider = 1")
            ->andWhere("f.dateFacture <= :end")
            ->andWhere("f.dateFacture >= :begin")
            ->setParameter("end", $end)
            ->setParameter("begin", $begin);

        if ($pointOfSale != null) {
            $sumOfAmountOfInvoicesQuery->andWhere("f.idPointVente = :pointOfSale")
                ->setParameter("pointOfSale", $pointOfSale);
        }

        return (float)$sumOfAmountOfInvoicesQuery->getQuery()
            ->getSingleScalarResult();
    }

    /**
     * 
     * @param string $day
     * @param \Application\Models\PointVente $pointOfSale
     * @return int
     */
    public function getInvoicesCountOfDay($day, $pointOfSale = null)
    {
        $beginDate = new DateTime("{$day} 00:00:00");
        $endDate = new DateTime("{$day} 23:59:59");
        $invoicesCountOverPeriodQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("ci")
            ->select("COUNT(ci.idFacture) AS sales_count")
            ->where("ci.statut = 1")
            ->andWhere("ci.valider = 1")
            ->andWhere("ci.dateFacture <= :end")
            ->andWhere("ci.dateFacture >= :begin")
            ->setParameter("end", $endDate)
            ->setParameter("begin", $beginDate);
        if ($pointOfSale != null) {
            $invoicesCountOverPeriodQuery->andWhere("ci.idPointVente = :pointOfSale")
                ->setParameter("pointOfSale", $pointOfSale);
        }

        return (int)$invoicesCountOverPeriodQuery->getQuery()
            ->getSingleScalarResult();
    }

    /**
     * 
     * @param string $day
     * @param \Application\Models\PointVente $pointOfSale
     * @return float
     */
    public function getSumOfPaymentsOfInvoicesOfDay($day, $pointOfSale = null)
    {
        $begin = new DateTime($day . " 00:00:00");
        $end = new DateTime($day . " 23:59:59");
        $sumOfPaymentsOfInvoicesQuery = $this->entityManager->getRepository("\Application\Models\PaiementFacture")->createQueryBuilder("p")
            ->select("SUM(p.montant) AS total")
            ->where("p.statut = 1")
            ->join("p.idFacture", "f")
            ->andWhere("f.statut = 1")
            ->andWhere("f.valider = 1")
            ->join("p.idMvtCaisse", "m")
            ->andWhere("m.statut = 1")
            ->andWhere("m.valideCaisse = 1")
            ->andWhere("p.datePaiement <= :end")
            ->andWhere("p.datePaiement >= :begin")
            ->setParameter("end", $end)
            ->setParameter("begin", $begin);

        if ($pointOfSale != null) {
            $sumOfPaymentsOfInvoicesQuery->andWhere("f.idPointVente = :pointOfSale")
                ->setParameter("pointOfSale", $pointOfSale);
        }

        return (float)$sumOfPaymentsOfInvoicesQuery->getQuery()
            ->getSingleScalarResult();
    }

    public function getSalesHistory($begin, $end)
    {
        $resultSetMapping = new ResultSetMapping();
        $resultSetMapping->addScalarResult("SALES_NUMBER", "SALES_NUMBER");
        $resultSetMapping->addScalarResult("AMOUNT", "AMOUNT");
        $resultSetMapping->addScalarResult("CURRENT_DAY", "CURRENT_DAY");
        $resultSetMapping->addScalarResult("LOGIN", "LOGIN");

        $query = $this->entityManager->createNativeQuery(
            "SELECT COUNT(ID_FACTURE) SALES_NUMBER, SUM(MONTANT_HT) AMOUNT, DATE_FORMAT(DATE_FACTURE, '%y-%m-%d') CURRENT_DAY, LOGIN
                       FROM facturec fac, utilisateur uti
                       WHERE fac.ID_UTILISATEUR = uti.ID_UTILISATEUR 
                       AND DATE_FACTURE >= '" . $begin->format('Y-m-d') . " 00:00:00'
                       AND DATE_FACTURE <= '" . $end->format('Y-m-d') . " 23:59:59' 
                       AND fac.STATUT = 1
                       AND fac.STATUT = 1
                       GROUP BY CURRENT_DAY",
            $resultSetMapping
        );
        $sales = $query->getResult();
        return [
            "content" => $sales,
        ];
    }

    /**
     * 
     * @param DateTime $begin
     * @param DateTime $end
     * @param \Application\Models\PointVente $pointOfSale
     * @return int
     */
    public function getInvoicesCountOverPeriod(DateTime $begin, DateTime $end, $pointOfSale = null)
    {
        $beginDate = new DateTime("{$begin->format(DATE_FORMAT)} 00:00:00");
        $endDate = new DateTime("{$end->format(DATE_FORMAT)} 23:59:59");
        $invoicesCountOverPeriodQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("ci")
            ->select("COUNT(ci.idFacture) AS sales_count")
            ->where("ci.statut = 1")
            ->andWhere("ci.valider = 1")
            ->andWhere("ci.dateFacture <= :end")
            ->andWhere("ci.dateFacture >= :begin")
            ->setParameter("end", $endDate)
            ->setParameter("begin", $beginDate);
        if ($pointOfSale != null) {
            $invoicesCountOverPeriodQuery->andWhere("ci.idPointVente = :pointOfSale")
                ->setParameter("pointOfSale", $pointOfSale);
        }

        return (int)$invoicesCountOverPeriodQuery->getQuery()
            ->getSingleScalarResult();
    }

    /**
     * 
     * @param DateTime $begin
     * @param DateTime $end
     * @param \Application\Models\PointVente $pointOfSale
     * @return float
     */
    public function getSumOfAmountOfInvoicesOverPeriod(DateTime $begin, DateTime $end, $pointOfSale = null)
    {
        $beginDate = new DateTime("{$begin->format(DATE_FORMAT)} 00:00:00");
        $endDate = new DateTime("{$end->format(DATE_FORMAT)} 23:59:59");
        $sumOfAmountOfInvoicesOverPeriodQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("f")
            ->select("SUM(f.montantNet) As Amount")
            ->where("f.statut = 1")
            ->andWhere("f.valider = 1")
            ->andWhere("f.dateFacture <= :end")
            ->andWhere("f.dateFacture >= :begin")
            ->setParameter("end", $endDate)
            ->setParameter("begin", $beginDate);

        if ($pointOfSale != null) {
            $sumOfAmountOfInvoicesOverPeriodQuery->andWhere("f.idPointVente = :pointOfSale")
                ->setParameter("pointOfSale", $pointOfSale);
        }

        return (float)$sumOfAmountOfInvoicesOverPeriodQuery->getQuery()
            ->getSingleScalarResult();
    }

    /**
     * 
     * @param DateTime $begin
     * @param DateTime $end
     * @param \Application\Models\PointVente $pointOfSale
     * @return float
     */
    public function getSumOfPaymentsOfInvoicesOverPeriod(DateTime $begin, DateTime $end, $pointOfSale = null)
    {
        $begin = new DateTime("{$begin->format(DATE_FORMAT)} 00:00:00");
        $end = new DateTime("{$end->format(DATE_FORMAT)} 23:59:59");
        $sumOfPaymentsOfInvoicesOverPeriodQuery = $this->entityManager->getRepository("\Application\Models\PaiementFacture")->createQueryBuilder("p")
            ->select("SUM(p.montant) AS total")
            ->where("p.statut = 1")
            ->join("p.idFacture", "f")
            ->andWhere("f.statut = 1")
            ->andWhere("f.valider = 1")
            ->join("p.idMvtCaisse", "m")
            ->andWhere("m.statut = 1")
            ->andWhere("m.valideCaisse = 1")
            ->andWhere("p.datePaiement <= :end")
            ->andWhere("p.datePaiement >= :begin")
            ->setParameter("end", $end)
            ->setParameter("begin", $begin);

        if ($pointOfSale != null) {
            $sumOfPaymentsOfInvoicesOverPeriodQuery->andWhere("f.idPointVente = :pointOfSale")
                ->setParameter("pointOfSale", $pointOfSale);
        }

        return (float)$sumOfPaymentsOfInvoicesOverPeriodQuery->getQuery()
            ->getSingleScalarResult();
    }

    /**
     * 
     * @param DateTime $month
     * @param \Application\Models\PointVente $pointOfSale
     * @return float
     */
    public function getSumOfAmountOfInvoicesOnMonth(DateTime $month, $pointOfSale = null)
    {
        $beginDate = clone $month;
        $endDate = clone $month;
        $beginDate->modify("first day of this month");
        $endDate->modify("last day of this month");
        return $this->getSumOfAmountOfInvoicesOverPeriod($beginDate, $endDate, $pointOfSale);
    }

    /**
     * 
     * @param DateTime $month
     * @param \Application\Models\PointVente $pointOfSale
     * @return float
     */
    public function getSumOfPaymentsOfInvoicesOnMonth(DateTime $month, $pointOfSale = null)
    {
        $beginDate = clone $month;
        $endDate = clone $month;
        $beginDate->modify("first day of this month");
        $endDate->modify("last day of this month");
        return $this->getSumOfPaymentsOfInvoicesOverPeriod($beginDate, $endDate, $pointOfSale);
    }

    /**
     * 
     * @param DateTime $month
     * @param \Application\Models\PointVente $pointOfSale
     * @return int
     */
    public function getInvoicesCountOnMonth(DateTime $month, $pointOfSale = null)
    {
        $beginDate = clone $month;
        $endDate = clone $month;
        $beginDate->modify("first day of this month");
        $endDate->modify("last day of this month");
        return $this->getInvoicesCountOverPeriod($beginDate, $endDate, $pointOfSale);
    }

    /**
     * 
     * @param DateTime $year
     * @param \Application\Models\PointVente $pointOfSale
     * @return float
     */
    public function getSumOfAmountOfInvoicesOnYear(DateTime $year, $pointOfSale = null)
    {
        $beginDate = clone $year;
        $endDate = clone $year;
        $beginDate->modify("first day of january {$year->format(DATE_FORMAT_YEAR)}");
        $beginDate->modify("last day of december {$year->format(DATE_FORMAT_YEAR)}");
        return $this->getSumOfAmountOfInvoicesOverPeriod($beginDate, $endDate, $pointOfSale);
    }

    /**
     * 
     * @param DateTime $year
     * @param \Application\Models\PointVente $pointOfSale
     * @return float
     */
    public function getSumOfPaymentsOfInvoicesOnYear(DateTime $year, $pointOfSale = null)
    {
        $beginDate = clone $year;
        $endDate = clone $year;
        $beginDate->modify("first day of january {$year->format(DATE_FORMAT_YEAR)}");
        $beginDate->modify("last day of december {$year->format(DATE_FORMAT_YEAR)}");
        return $this->getSumOfPaymentsOfInvoicesOverPeriod($beginDate, $endDate, $pointOfSale);
    }

    public function getUnpaidInvoicesOfCustomer($customer = null, $user = null)
    {
        $unpaidInvoicesList = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("f")
            ->where("(f.typeVente = 2 or f.typeVente = 1)")
            ->andWhere("f.statut = 1")
            ->andWhere("f.valider <> 1");
        if ($customer != null) {
            $unpaidInvoicesList->andWhere("f.idPersonne= :customer")
                ->setParameter("customer", $customer);
        }

        if ($user != null) {
            $unpaidInvoicesList->andWhere("f.idUtilisateur = :user")
                ->setParameter("user", $user);
        }

        return $unpaidInvoicesList->getQuery()->getResult();
    }

    public function getReservationsOfCustomer($customer = null, $user = null)
    {
        $reservationsList = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("f")
            ->where("(f.typeVente = 2 or f.typeVente = 1)")
            ->andWhere("f.statut = 1")
            ->andWhere("f.etatLivraison <> 2");

        if ($customer != null) {
            $reservationsList->andWhere("f.idPersonne = :customer")
                ->setParameter("customer", $customer);
        }

        if ($user != null) {
            $reservationsList->andWhere("f.idUtilisateur = :user")
                ->setParameter("user", $user);
        }

        return $reservationsList->getQuery()->getResult();
    }

    public function getInvoicesFromCredit($credit)
    {
        return $this->entityManager->getRepository($this->modelName)->findBy(array("statut" => 1, "avoirOrigine" => $credit));
    }

    public function getAllWaitingInvoices()
    {
        return $this->entityManager->getRepository($this->modelName)->findBy(array("statut" => 1, "enAttente" => 1));
    }

    public function getCountOfInvoicesOverPeriod(\DateTime $begin = null, \DateTime $end = null)
    {
        $countOfInvoicesQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("f")
            ->select("COUNT(f.idFacture)")
            ->where("f.statut = 1");

        if ($begin != null) {
            $beginDate = new \DateTime($begin->format("Y-m-d") . " 00:00:00");
            $countOfInvoicesQuery->andWhere("f.dateFacture >= :begin")
                ->setParameter("begin", $beginDate);
        }

        if ($end != null) {
            $endDate = new \DateTime($end->format("Y-m-d") . " 23:59:59");
            $countOfInvoicesQuery->andWhere("f.dateFacture <= :end")
                ->setParameter("end", $endDate);
        }

        return $countOfInvoicesQuery->getQuery()->getSingleScalarResult();
    }

    public function getInvoicesOfSession($session)
    {
        return $this->entityManager->getRepository($this->modelName)->createQueryBuilder("f")
            ->where("f.statut = 1")
            ->andWhere("f.valider != -1")
            ->andWhere("f.idSessionCaissier = :session")
            ->setParameter("session", $session)
            ->addOrderBy("f.idPointVente", "ASC")
            ->addOrderBy("f.idPointVente", "ASC")
            ->addOrderBy("f.idUtilisateur", "ASC")
            ->addOrderBy("f.dateFacture", "ASC")
            ->getQuery()
            ->getResult();
    }


    public function getInvoicesOfSellerSession($session)
    {
        return $this->entityManager->getRepository($this->modelName)->createQueryBuilder("f")
            ->where("f.statut = 1")
            ->andWhere("f.valider != -1")
            ->andWhere("f.idSessionVendeur = :session")
            ->setParameter("session", $session)
            ->addOrderBy("f.idPointVente", "ASC")
            ->addOrderBy("f.idPointVente", "ASC")
            ->addOrderBy("f.idUtilisateur", "ASC")
            ->addOrderBy("f.dateFacture", "ASC")
            ->getQuery()
            ->getResult();
    }


    public function getSumOfPaymentsOfInvoice($invoice)
    {
        return $this->entityManager->getRepository("\Application\Models\PaiementFacture")->createQueryBuilder("p")
            ->select("SUM(p.montant) AS total")
            ->where("p.idFacture = :invoice")
            ->setParameter("invoice", $invoice)
            ->andWhere("p.statut = 1")
            ->getQuery()
            ->getSingleScalarResult();
    }

    public function getAllOfPaymentsOfInvoice($invoice)
    {
        return $this->entityManager->getRepository("\Application\Models\PaiementFacture")->createQueryBuilder("p")
            ->where("p.idFacture = :invoice")
            ->setParameter("invoice", $invoice)
            ->andWhere("p.statut = 1")
            ->getQuery()
            ->getResult();
    }

    public function getSumOfPaymentsOfInvoiceByPaymentMethod($invoice, $paymentMethod)
    {
        return $this->entityManager->getRepository("\Application\Models\PaiementFacture")->createQueryBuilder("p")
            ->select("SUM(p.montant) AS total")
            ->where("p.idFacture = :invoice")
            ->andWhere("p.modePaiement = :paymentMethod")
            ->setParameter("invoice", $invoice)
            ->setParameter("paymentMethod", $paymentMethod)
            ->andWhere("p.statut = 1")
            ->getQuery()
            ->getSingleScalarResult();
    }

    public function getNotPaidInvoices($client = null, $user = null)
    {
        $notPaidInvoicesList = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("f")
            ->leftjoin("\Application\Models\PaiementFacture", "p")
            ->select('f')
            ->where("f.valider <> -1")
            ->andWhere("f.valider <> 1")
            ->andWhere("p.idFacture = f.idFacture")
            ->groupBy('f.idFacture')
            ->having('((sum(p.montant) <  f.montantHt) or sum(p.montant) is null) and f.statut = 1 and f.valider <> 1 and f.valider <> -1');
        if ($user != null) {
            $notPaidInvoicesList->andWhere("f.idUtilisateur = :user")
                ->setParameter("user", $user);
        }

        if ($client != null) {
            $notPaidInvoicesList->andWhere("f.idPersonne = :client")
                ->setParameter("client", $client);
        }

        return $notPaidInvoicesList->getQuery()->getResult();
    }

    public function lastInvoice()
    {
        return $this->entityManager->getRepository($this->modelName)->createQueryBuilder("f")
            ->select("f.dateFacture")
            ->addSelect("u.idUtilisateur")
            ->join("f.idUtilisateur", "u")
            ->where("f.statut = 1")
            ->orderBy("f.idFacture", "DESC")
            ->setMaxResults(1)
            ->getQuery()
            ->getSingleResult();
    }

    public function invoicesOfClient($client = null)
    {
        $sumOfAmountOfClientInvoices = $this->entityManager->createQueryBuilder()
            ->select("SUM(f.montantHt) as Montant")
            ->addSelect("COUNT(f.idFacture) as  nombreAchat")
            ->addSelect("Min(f.dateFacture) as dateMin")
            ->addSelect("Max(f.dateFacture) as dateMax")
            ->from('\Application\Models\Facturec', 'f')
            ->where("f.statut = 1")
            ->where("f.valider = 1");

        if ($client != null) {
            $sumOfAmountOfClientInvoices->andWhere(' f.idPersonne = :client ')
                ->setParameter('client', $client);
        }

        return $sumOfAmountOfClientInvoices->getQuery()->getResult();
    }

    public function getInvoicesFilteredOnMobile($pointOfSale = null, $user = null, $article = null, $begin = null, $end = null)
    {
        $invoicesList = $this->entityManager->createQueryBuilder()
            ->select('pv.nomPointVente')
            ->addSelect('count(a.idFacture) as nombreC')
            ->addSelect('Sum(a.montantHt) as montantC')
            ->addSelect('Sum(pf.montant) as montantE')
            ->from('\Application\Models\Facturec', 'a')
            ->innerJoin('a.idPointVente', 'pv', 'WITH', 'a.idPointVente=pv')
            ->innerJoin('\Application\Models\PaiementFacture', 'pf', 'WITH', 'pf.idFacture=a')
            ->where('1=1')
            ->groupBy('pv.nomPointVente');

        if ($pointOfSale != null) {
            $invoicesList->andWhere('a.idPointVente =:pointOfSale')
                ->setParameter('pointOfSale', $pointOfSale);
        }

        if ($article != null) {
            $invoicesList->andWhere('c.idProduit = :article')
                ->setParameter('article', $article);
        }

        if ($user != null) {
            $invoicesList->andWhere('a.idUtilisateur =:user')
                ->setParameter('user', $user);
        }

        if ($begin != null) {
            $beginDate = new DateTime($begin->format("Y-m-d"));
            $invoicesList->andWhere('a.dateFacture >= :begin')
                ->setParameter('begin', $beginDate);
        }

        if ($end != null) {
            $endDate = new DateTime($end->format("Y-m-d") . " 23:59:59");
            $invoicesList->andWhere('a.dateFacture <= :end')
                ->setParameter('end', $endDate);
        }

        return $invoicesList->getQuery()->getResult();
    }

    public function getBestSellingItemsOnMobile($begin = null, $end = null)
    {
        $bestSellingItemsList = $this->entityManager->createQueryBuilder()
            ->select('IDENTITY(a.idProduit) as idProduit')
            ->addSelect('sum(a.quantite) as quantite')
            ->addSelect('a.designation as designation')
            ->from('\Application\Models\LigneFacturec', 'a')
            ->innerJoin('a.idFacture', 'f')
            ->where('1=1')
            ->setMaxResults(5)
            ->orderBy('quantite', 'DESC')
            ->groupBy('a.idProduit');

        if ($begin != null) {
            $beginDate = new \DateTime($begin->format("Y-m-d"));
            $bestSellingItemsList->andWhere('f.dateFacture >= :begin')
                ->setParameter('begin', $beginDate);
        }

        if ($end != null) {
            $endDate = new \DateTime($end->format("Y-m-d") . " 23:59:59");
            $bestSellingItemsList->andWhere('f.dateFacture <= :end')
                ->setParameter('end', $endDate);
        }

        return $bestSellingItemsList->getQuery()->getResult();
    }

    public function getSoldItemsOnMobile($begin = null, $end = null, $user = null)
    {
        $soldItemsList = $this->entityManager->createQueryBuilder()
            ->select('IDENTITY(a.idProduit) as idProduit')
            ->addSelect('sum(a.quantite) as quantite')
            ->addSelect('a.designation as designation')
            ->addSelect('a.prixUnitaire as prix')
            ->from('\Application\Models\LigneFacturec', 'a')
            ->innerJoin('a.idFacture', 'f')
            ->where('1=1')
            ->orderBy('quantite', 'DESC')
            ->groupBy('a.idProduit');

        if ($begin != null) {
            $beginDate = new \DateTime($begin->format("Y-m-d"));
            $soldItemsList->andWhere('f.dateFacture >= :begin')
                ->setParameter('begin', $beginDate);
        }

        if ($user != null) {
            $soldItemsList->andWhere('f.idUtilisateur = :user')
                ->setParameter('user', $user);
        }

        if ($end != null) {
            $endDate = new \DateTime($end->format("Y-m-d") . " 23:59:59");
            $soldItemsList->andWhere('f.dateFacture <= :end')
                ->setParameter('end', $endDate);
        }

        return $soldItemsList->getQuery()->getResult();
    }

    public function getCashiersByPointOfSaleOnMobile($pointOfSale = null, $begin = null, $end = null)
    {
        $cashiersList = $this->entityManager->createQueryBuilder()
            ->select('p as personne')
            ->from('\Application\Models\Personne', 'p')
            ->innerJoin('\Application\Models\Utilisateur', 'u', 'WITH', 'u.idPersonne=p')
            ->innerJoin('\Application\Models\Facturec', 'f', 'WITH', 'f.idUtilisateur=u')
            ->where('1=1');

        if ($pointOfSale != null) {
            $cashiersList->andWhere('f.idPointVente = :pointOfSale')
                ->setParameter('pointOfSale', $pointOfSale);
        }

        if ($begin != null) {
            $beginDate = new \DateTime($begin->format("Y-m-d"));
            $cashiersList->andWhere('f.dateFacture >= :begin')
                ->setParameter('begin', $beginDate);
        }

        if ($end != null) {
            $endDate = new \DateTime($end->format("Y-m-d") . " 23:59:59");
            $cashiersList->andWhere('f.dateFacture <= :end')
                ->setParameter('end', $endDate);
        }

        return $cashiersList->getQuery()->getResult();
    }

    public function getSoldByCashierAndPointOfSaleOnMobile($pointOfSale = null, $person = null, $begin = null, $end = null)
    {
        $soldList = $this->entityManager->createQueryBuilder()
            ->select('f as facture')
            ->from($this->modelName, 'f')
            ->innerJoin('\Application\Models\Utilisateur', 'u', 'WITH', 'f.idUtilisateur=u')
            ->where('1=1');

        if ($pointOfSale != null) {
            $soldList->andWhere('f.idPointVente = :pointOfSale')
                ->setParameter('pointOfSale', $pointOfSale);
        }

        if ($person != null) {
            $soldList->andWhere('u.idPersonne = :person')
                ->setParameter('person', $person);
        }

        if ($begin != null) {
            $beginDate = new \DateTime($begin->format("Y-m-d"));
            $soldList->andWhere('f.dateFacture >= :begin')
                ->setParameter('begin', $beginDate);
        }

        if ($end != null) {
            $endDate = new \DateTime($end->format("Y-m-d") . " 23:59:59");
            $soldList->andWhere('f.dateFacture <= :end')
                ->setParameter('end', $endDate);
        }

        return $soldList->getQuery()->getResult();
    }

    public function getSumOfInvoicePayments($invoice)
    {
        return $this->entityManager->createQueryBuilder()
            ->select('sum(p.montant) as montant')
            ->from('\Application\Models\PaiementFacture', 'p')
            ->where('p.idFacture = :invoice')
            ->setParameter("invoice", $invoice)
            ->getQuery()
            ->getSingleScalarResult();
    }


    public function getLastInvoiceByUser($userId)
    {
        return $this->entityManager->getRepository($this->modelName)->findOneBy(array("idUtilisateur" => $userId, "statut" => 1), array("idFacture" => "DESC"));
    }

    /**
     * 
     * @param DateTime $lastDateOfService
     * @param \Application\Models\PointVente $pointOfSale
     * @param string $gender
     * @return mixed[]
     */
    public function getCustomerToFollowUpReadyForMessage($lastDateOfService, $pointOfSale = null, $gender = null)
    {
        $effectiveLastDateOfService = new DateTime("{$lastDateOfService->format(DATE_FORMAT)} 23:59:59");
        $customersToFollowUpQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("ci")
            ->select("MAX(ci.dateFacture) AS last_invoice_date")
            ->addSelect("cu.idPersonne AS customer_id")
            ->addSelect("cu.nom AS customer_last_name")
            ->addSelect("cu.prenom AS customer_first_name")
            ->addSelect("cu.telephone AS customer_phone")
            ->addSelect("cu.email AS customer_email")
            ->join("ci.idPersonne", "cu")
            ->where("cu.type = :type")
            ->setParameter("type", TYPE_PERSONNE_CLIENT)
            ->andWhere("ci.statut = 1")
            ->andWhere("ci.valider = 1");
        if ($pointOfSale != null) {
            $customersToFollowUpQuery->andWhere("ci.idPointVente = :pointOfSale")
                ->setParameter("pointOfSale", $pointOfSale);
        }
        if ($gender != null) {
            $customersToFollowUpQuery->andWhere("cu.sexe = :gender")
                ->setParameter("gender", $gender);
        }
        $customersToFollowUpQuery->andWhere($customersToFollowUpQuery->expr()->orX(
            $customersToFollowUpQuery->expr()->isNotNull("cu.telephone"),
            $customersToFollowUpQuery->expr()->isNotNull("cu.email")
        ));
        return $customersToFollowUpQuery->groupBy("ci.idPersonne")
            ->having("last_invoice_date <= :lastDateOfService")
            ->setParameter("lastDateOfService", $effectiveLastDateOfService)
            ->getQuery()
            ->getScalarResult();
    }

    /**
     * 
     * @param DateTime $startDate
     * @param DateTime $endDate
     * @param float $amount
     * @param \Application\Models\PointVente $pointOfSale
     * @param string gender
     * @return mixed[]
     */
    public function getBestCustomersReadyForMessage($startDate, $endDate, $amount, $pointOfSale = null, $gender = null)
    {
        $bestCustomersQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("ci")
            ->select("SUM(ci.montantNet) AS customer_turnover")
            ->addSelect("cu.idPersonne AS customer_id")
            ->addSelect("cu.nom AS customer_last_name")
            ->addSelect("cu.prenom AS customer_first_name")
            ->addSelect("cu.telephone AS customer_phone")
            ->addSelect("cu.email AS customer_email")
            ->join("ci.idPersonne", "cu")
            ->where("cu.type = :type")
            ->setParameter("type", TYPE_PERSONNE_CLIENT)
            ->andWhere("ci.dateFacture >= :startDate")
            ->setParameter("startDate", $startDate)
            ->andWhere("ci.dateFacture <= :endDate")
            ->setParameter("endDate", $endDate)
            ->andWhere("ci.statut = 1")
            ->andWhere("ci.valider = 1");
        if ($pointOfSale != null) {
            $bestCustomersQuery->andWhere("ci.idPointVente = :pointOfSale")
                ->setParameter("pointOfSale", $pointOfSale);
        }
        if ($gender != null) {
            $bestCustomersQuery->andWhere("cu.sexe = :gender")
                ->setParameter("gender", $gender);
        }
        $bestCustomersQuery->andWhere($bestCustomersQuery->expr()->orX(
            $bestCustomersQuery->expr()->isNotNull("cu.telephone"),
            $bestCustomersQuery->expr()->isNotNull("cu.email")
        ));
        return $bestCustomersQuery->groupBy("ci.idPersonne")
            ->having("customer_turnover >= :turnover")
            ->setParameter("turnover", $amount)
            ->getQuery()
            ->getScalarResult();
    }

    /**
     * 
     * @param DateTime $startDate
     * @param DateTime $endDate
     * @param int $visits
     * @param \Application\Models\PointVente $pointOfSale
     * @param string gender
     * @return mixed[]
     */
    public function getSatisfactionSurveyCustomersReadyForMessage($startDate, $endDate, $visits, $pointOfSale = null, $gender = null)
    {
        $satisfactionSurveyCustomersQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("ci")
            ->select("COUNT(ci.idFacture) AS customer_visits")
            ->addSelect("cu.idPersonne AS customer_id")
            ->addSelect("cu.nom AS customer_last_name")
            ->addSelect("cu.prenom AS customer_first_name")
            ->addSelect("cu.telephone AS customer_phone")
            ->addSelect("cu.email AS customer_email")
            ->join("ci.idPersonne", "cu")
            ->where("cu.type = :type")
            ->setParameter("type", TYPE_PERSONNE_CLIENT)
            ->andWhere("ci.dateFacture >= :startDate")
            ->setParameter("startDate", $startDate)
            ->andWhere("ci.dateFacture <= :endDate")
            ->setParameter("endDate", $endDate)
            ->andWhere("ci.statut = 1")
            ->andWhere("ci.valider = 1");
        if ($pointOfSale != null) {
            $satisfactionSurveyCustomersQuery->andWhere("ci.idPointVente = :pointOfSale")
                ->setParameter("pointOfSale", $pointOfSale);
        }
        if ($gender != null) {
            $satisfactionSurveyCustomersQuery->andWhere("cu.sexe = :gender")
                ->setParameter("gender", $gender);
        }
        $satisfactionSurveyCustomersQuery->andWhere($satisfactionSurveyCustomersQuery->expr()->orX(
            $satisfactionSurveyCustomersQuery->expr()->isNotNull("cu.telephone"),
            $satisfactionSurveyCustomersQuery->expr()->isNotNull("cu.email")
        ));
        return $satisfactionSurveyCustomersQuery->groupBy("ci.idPersonne")
            ->having("customer_visits >= :visits")
            ->setParameter("visits", $visits)
            ->getQuery()
            ->getScalarResult();
    }

    /**
     * 
     * @param DateTime $lastDateOfService
     * @param \Application\Models\PointVente $pointOfSale
     * @return mixed[]
     */
    public function getCustomerToFollowUp($lastDateOfService, $pointOfSale = null)
    {
        $effectiveLastDateOfService = new DateTime("{$lastDateOfService->format(DATE_FORMAT)} 23:59:59");
        $customersToFollowUpQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("ci")
            ->select("MAX(ci.dateFacture) AS last_invoice_date")
            ->addSelect("cu.idPersonne AS customer_id")
            ->addSelect("cu.nom AS customer_last_name")
            ->addSelect("cu.prenom AS customer_first_name")
            ->addSelect("cu.telephone AS customer_phone")
            ->addSelect("cu.email AS customer_email")
            ->join("ci.idPersonne", "cu")
            ->where("cu.type = :type")
            ->setParameter("type", TYPE_PERSONNE_CLIENT)
            ->andWhere("ci.statut = 1")
            ->andWhere("ci.valider = 1");
        if ($pointOfSale != null) {
            $customersToFollowUpQuery->andWhere("ci.idPointVente = :pointOfSale")
                ->setParameter("pointOfSale", $pointOfSale);
        }
        return $customersToFollowUpQuery->groupBy("ci.idPersonne")
            ->orderBy("last_invoice_date", "DESC")
            ->having("last_invoice_date <= :lastDateOfService")
            ->setParameter("lastDateOfService", $effectiveLastDateOfService)
            ->getQuery()
            ->getScalarResult();
    }

    /**
     * 
     * @param DateTime $startDate
     * @param DateTime $endDate
     * @param float $amount
     * @param \Application\Models\PointVente $pointOfSale
     * @return mixed[]
     */
    public function getBestCustomers($startDate, $endDate, $amount, $pointOfSale = null)
    {
        $bestCustomersQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("ci")
            ->select("SUM(ci.montantNet) AS customer_turnover")
            ->addSelect("cu.idPersonne AS customer_id")
            ->addSelect("cu.nom AS customer_last_name")
            ->addSelect("cu.prenom AS customer_first_name")
            ->addSelect("cu.telephone AS customer_phone")
            ->addSelect("cu.email AS customer_email")
            ->join("ci.idPersonne", "cu")
            ->where("cu.type = :type")
            ->setParameter("type", TYPE_PERSONNE_CLIENT)
            ->andWhere("ci.dateFacture >= :startDate")
            ->setParameter("startDate", $startDate)
            ->andWhere("ci.dateFacture <= :endDate")
            ->setParameter("endDate", $endDate)
            ->andWhere("ci.statut = 1")
            ->andWhere("ci.valider = 1");
        if ($pointOfSale != null) {
            $bestCustomersQuery->andWhere("ci.idPointVente = :pointOfSale")
                ->setParameter("pointOfSale", $pointOfSale);
        }
        return $bestCustomersQuery->groupBy("ci.idPersonne")
            ->orderBy("customer_turnover", "DESC")
            ->having("customer_turnover >= :turnover")
            ->setParameter("turnover", $amount)
            ->getQuery()
            ->getScalarResult();
    }

    /**
     * 
     * @param DateTime $startDate
     * @param DateTime $endDate
     * @param int $count
     * @param \Application\Models\PointVente $pointOfSale
     * @return mixed[]
     */
    public function getBestCustomersLimited($startDate, $endDate, $count, $pointOfSale = null)
    {
        $bestCustomersQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("ci")
            ->select("SUM(ci.montantNet) AS customer_turnover")
            ->addSelect("COUNT(ci.idFacture) AS sales_count")
            ->addSelect("cu.idPersonne AS customer_id")
            ->addSelect("cu.nom AS customer_last_name")
            ->addSelect("cu.prenom AS customer_first_name")
            ->join("ci.idPersonne", "cu")
            ->where("cu.type = :type")
            ->setParameter("type", TYPE_PERSONNE_CLIENT)
            ->andWhere("ci.dateFacture >= :startDate")
            ->setParameter("startDate", $startDate)
            ->andWhere("ci.dateFacture <= :endDate")
            ->setParameter("endDate", $endDate)
            ->andWhere("ci.statut = 1")
            ->andWhere("ci.valider = 1");
        if ($pointOfSale != null) {
            $bestCustomersQuery->andWhere("ci.idPointVente = :pointOfSale")
                ->setParameter("pointOfSale", $pointOfSale);
        }
        return $bestCustomersQuery->groupBy("ci.idPersonne")
            ->orderBy("customer_turnover", "DESC")
            ->setMaxResults($count)
            ->getQuery()
            ->getScalarResult();
    }

    /**
     * 
     * @param DateTime $startDate
     * @param DateTime $endDate
     * @param int $visits
     * @param \Application\Models\PointVente $pointOfSale
     * @param string gender
     * @return mixed[]
     */
    public function getSatisfactionSurveyCustomers($startDate, $endDate, $visits, $pointOfSale = null)
    {
        $satisfactionSurveyCustomersQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("ci")
            ->select("COUNT(ci.idFacture) AS customer_visits")
            ->addSelect("cu.idPersonne AS customer_id")
            ->addSelect("cu.nom AS customer_last_name")
            ->addSelect("cu.prenom AS customer_first_name")
            ->addSelect("cu.telephone AS customer_phone")
            ->addSelect("cu.email AS customer_email")
            ->join("ci.idPersonne", "cu")
            ->where("cu.type = :type")
            ->setParameter("type", TYPE_PERSONNE_CLIENT)
            ->andWhere("ci.dateFacture >= :startDate")
            ->setParameter("startDate", $startDate)
            ->andWhere("ci.dateFacture <= :endDate")
            ->setParameter("endDate", $endDate)
            ->andWhere("ci.statut = 1")
            ->andWhere("ci.valider = 1");
        if ($pointOfSale != null) {
            $satisfactionSurveyCustomersQuery->andWhere("ci.idPointVente = :pointOfSale")
                ->setParameter("pointOfSale", $pointOfSale);
        }
        return $satisfactionSurveyCustomersQuery->groupBy("ci.idPersonne")
            ->orderBy("customer_visits", "DESC")
            ->having("customer_visits >= :visits")
            ->setParameter("visits", $visits)
            ->getQuery()
            ->getScalarResult();
    }

    /**
     * 
     * @param \Application\Models\PointVente[] $pointsOfSale
     * @param \Application\Models\Utilisateur[] $cashiers
     * @param DateTime $startDate
     * @param DateTime $endDate
     * @return \Application\Models\Facturec[]
     */
    public function getSalesByPointsOfSalesAndCashiers($pointsOfSale = [], $cashiers = [], DateTime $startDate, DateTime $endDate)
    {
        $cashierSalesQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("f")
            ->where("f.statut = 1")
            ->andWhere("f.valider = 1")
            ->andWhere("f.dateFacture >= :begin")
            ->andWhere("f.dateFacture <= :end")
            ->setParameter("begin", $startDate)
            ->setParameter("end", $endDate);
        if (count($pointsOfSale) > 0) {
            $cashierSalesQuery->andWhere($cashierSalesQuery->expr()->in("f.idPointVente", ":pointsOfSale"))
                ->setParameter("pointsOfSale", $pointsOfSale);
        }
        if (count($cashiers) > 0) {
            $cashierSalesQuery->andWhere($cashierSalesQuery->expr()->in("f.idUtilisateur", ":cashiers"))
                ->setParameter("cashiers", $cashiers);
        }
        return $cashierSalesQuery->orderBy("f.idPointVente", "ASC")
            ->addOrderBy("f.idUtilisateur", "ASC")
            ->addOrderBy("f.dateFacture", "DESC")
            ->getQuery()
            ->getResult();
    }

    /**
     * 
     * @param \Application\Models\PointVente[] $pointsOfSale
     * @param DateTime $startDate
     * @param DateTime $endDate
     * @return \Application\Models\Facturec[]
     */
    public function getSalesByPointsOfSales($pointsOfSale = [], DateTime $startDate, DateTime $endDate)
    {
        $cashierSalesQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("f")
            ->where("f.statut = 1")
            ->andWhere("f.valider = 1")
            ->andWhere("f.dateFacture >= :begin")
            ->andWhere("f.dateFacture <= :end")
            ->setParameter("begin", $startDate)
            ->setParameter("end", $endDate);
        if (count($pointsOfSale) > 0) {
            $cashierSalesQuery->andWhere($cashierSalesQuery->expr()->in("f.idPointVente", ":pointsOfSale"))
                ->setParameter("pointsOfSale", $pointsOfSale);
        }
        return $cashierSalesQuery->orderBy("f.idPointVente", "ASC")
            ->addOrderBy("f.dateFacture", "DESC")
            ->getQuery()
            ->getResult();
    }

    /**
     * 
     * @param \Application\Models\Personne $customer
     * @param \Application\Models\PointVente[] $pointsOfSale
     * @param DateTime $startDate
     * @param DateTime $endDate
     * @return \Application\Models\Facturec[]
     */
    public function getSalesByCustomer($customer, $pointsOfSale = [], DateTime $startDate, DateTime $endDate)
    {
        $cashierSalesQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("f")
            ->where("f.statut = 1")
            ->andWhere("f.valider = 1")
            ->andWhere("f.dateFacture >= :begin")
            ->andWhere("f.dateFacture <= :end")
            ->setParameter("begin", $startDate)
            ->setParameter("end", $endDate)
            ->andWhere("f.idPersonne = :customer")
            ->setParameter("customer", $customer);
        if (count($pointsOfSale) > 0) {
            $cashierSalesQuery->andWhere($cashierSalesQuery->expr()->in("f.idPointVente", ":pointsOfSale"))
                ->setParameter("pointsOfSale", $pointsOfSale);
        }
        return $cashierSalesQuery->orderBy("f.idPointVente", "ASC")
            ->addOrderBy("f.dateFacture", "DESC")
            ->getQuery()
            ->getResult();
    }

    //This function is to record a new invoice, and also the related payments
    public static function registerCustomerInvoice(&$invoiceBasket, $translator)
    {   /*
            A FAIRE
            ----------
            - Gestion des prestataire ( par ligne de produit)
        */

        $userDAO = new UserDAO();
        $productDAO = new ProductDAO();
        $customerInvoiceDAO = new CustomerInvoiceDAO();
        $invoicePaymentDAO = new InvoicePaymentDAO();
        $cashSessionDAO = new CashSessionDAO();
        $customerInvoiceLineDAO = new CustomerInvoiceLineDAO();
        $pointOfSaleDAO = new PointOfSaleDAO();
        $stockDAO = new StockDAO();
        $exitDAO = new ExitDAO();
        $exitLineDAO = new ExitLineDAO();
        $custumerDAO = new PersonDAO();
        $locationDAO = new CustomerLocationDAO();
        $cashMovementDAO = new CashMovementDAO();
        $paymentMethodDAO = new PaymentMethodDAO();

        try {
            $databaseConnection = $customerInvoiceDAO->entityManager->getConnection();
            $databaseConnection->beginTransaction();

            //current date on server
            $currentDate = new DateTime();

            $invoiceNumber = (int) $invoiceBasket["invoiceNumber"];

            $sessionId = (int) $invoiceBasket["checkoutSession"]["id"];

            $sellerSessionId = (int) $invoiceBasket["sellerSession"]["id"];

            $totalPaid = (float) $invoiceBasket["paid"]["totalPaid"];

            $pendingInvoice = (int) $invoiceBasket["pending"];

            $validatedInvoice = (int) $invoiceBasket["validated"];

            //sale with destocking
            $saleWithDestocking = (int) $invoiceBasket["saleWithDestocking"] === 1 || $totalPaid > 0;

            //products
            $products = $invoiceBasket["products"];

            if ($invoiceNumber) {
                $invoice = $customerInvoiceDAO->getCustomerInvoiceById($invoiceNumber);

                if ($invoice->getValider()  ===  1) {
                    $databaseConnection->rollBack();
                    return ApiResponse::respond(101, $translator->_("sale.sell.currentInvoiceAlreadyValidatedAndCannotBeModified"));
                }
                //else, it's an update

                $amountAlreadyPay = (float) $invoicePaymentDAO->getSumOfInvoicePayments($invoice);

                if ($amountAlreadyPay > 0) {
                    $databaseConnection->rollBack();
                    return ApiResponse::respond(101, $translator->_("sale.sell.currentInvoiceIsAlreadyBeingPaidAndCannotBeModified"));
                }
                //deleting invoice
                //TODO Delete previous records
            } else {
                $invoice = $customerInvoiceDAO->create();

                $invoiceBasket["invoiceNumber"] = $invoice->getIdFacture();

                //change invoice date for instance
                $invoiceBasket["billingDate"] = $currentDate->format("Y-m-d H:i:s");
            }

            ////user
            $userId = (int) $invoiceBasket["user"]["id"];

            $user = null;

            if ($userId) {
                $user = $userDAO->getUserById($userId);
            }

            //Point of sale
            $pointOfSale = null;

            $pointOfSaleId = $invoiceBasket["pointOfSale"]["id"];

            if ($pointOfSaleId) {
                $pointOfSale = $pointOfSaleDAO->getPointOfSaleById($pointOfSaleId);
            }

            $store = $pointOfSale->getIdMagasin();

            $separateStock = GESTION_DATE_PEREMPTION_PRODUIT; //1 ou 0

            $cashoutSession = null;

            if ($sessionId) {
                $cashoutSession = $cashSessionDAO->getCashSessionById($sessionId);
            }

            $sellerSession = null;

            if ($sellerSessionId) {
                $sellerSession = $cashSessionDAO->getCashSessionById($sellerSessionId);
            }

            if (!$cashoutSession) {
                $databaseConnection->rollBack();
                return ApiResponse::respond(101, $translator->_("sale.sell.unknowCashSession"));
            }

            $cashSessionExceptedClosingDate = $cashoutSession->getDatePrevuCloture();

            $sessionLock = $cashoutSession->getCloturer() || $cashoutSession->getFerme();

            if ($cashSessionExceptedClosingDate < $currentDate || $sessionLock) {
                $databaseConnection->rollBack();
                return ApiResponse::respond(101, $translator->_("sale.sell.theCashoutSessionIsNotAvailable"));
            }

            //service provider
            $invoiceServiceProviderId = $invoiceBasket["serviceProvider"]["id"];

            $invoiceServiceProvider = null;

            if ( !$sellerSession && $invoiceServiceProviderId) {
                $invoiceServiceProvider = $userDAO->getUserById($invoiceServiceProviderId);
            }
            else{
                if($sellerSession)
                {
                    $invoiceServiceProvider = $sellerSession->getIdUtilisateur();
                }
            }

            //costomer location
            $customerLocationId = $invoiceBasket["location"]["id"];
            $groupInLocation = $invoiceBasket["location"]["groupNumber"];

            $customerLocation = $locationDAO->getLocationById($customerLocationId);

            if (!$customerLocation) {
                $customerLocation = null;

                $groupInLocation = null;
            }

            //customer
            $customerId = (int) $invoiceBasket["customer"]["id"];

            $custumerName = $invoiceBasket["customer"]["name"];

            $customer = null;

            if ($customerId) {
                $customer = $custumerDAO->getPersonById($customerId);

                if ($customer) {
                    $custumerName = $customer->getNom() . " " . $customer->getPrenom();
                }
            }

            //die("abcbon");
            //invoice registration
            $invoice->setDateFacture($currentDate)
                ->setIdPersonne($customer)
                ->setNomClient($custumerName)
                ->setIdUtilisateur($user)
                ->setServeur($invoiceServiceProvider)
                ->setIdEmplacement($customerLocation)
                ->setGroupe($groupInLocation)
                ->setIdSessionCaissier($cashoutSession)
                ->setIdSessionVendeur($sellerSession)
                ->setMontantHt(0)
                ->setMontantTtc(0)
                ->setTauxTva(0)
                ->setMontantTva(0)
                ->setTauxIr(0)
                ->setMontantIr(0)
                ->setPctRemise(0)
                ->setForfaitRemise(0)
                ->setStatut(1)
                ->setValider($validatedInvoice)
                ->setEnAttente($pendingInvoice)
                ->setIrRetenu(0)
                ->setTvaRetenu(0)
                ->setMontantNet(0)
                ->setMontantVerse(0)
                ->setIdPointVente($pointOfSale);

            $customerInvoiceDAO->save($invoice);

            if (!$invoiceNumber) {
                $invoiceNumber = $invoice->getIdFacture();

                $invoiceBasket["invoiceNumber"] = $invoiceNumber;
            }

            //Products exit
            if ($saleWithDestocking) {
                $productsExit = $exitDAO->create();

                $productsExit->setIdUtilisateur($user)
                    ->setIdMagasin($store)
                    ->setIdFacture($invoice)
                    ->setTypeSortie(TYPE_SORTIE_VENTE)
                    ->setDateSaisie($currentDate)
                    ->setDateSortie($currentDate)
                    ->setDateValidation($currentDate)
                    ->setValider(1)
                    ->setStatut(1);

                $exitDAO->save($productsExit);
            }

            //discount and invoice amounts
            $amountExcludingTax = $totalDiscountAmount = 0;

            foreach ($products as $productArray) {
                $productId = (int) $productArray["productId"];

                $batchNumber = $productArray["batchNum"];

                $productName = $productArray["productName"];

                $productPrice = (float) $productArray["productPrice"];

                $productQuantity = (float) $productArray["quantity"];

                $discountAmount = (float) $productArray["discountAmount"];

                $discountPercentage = (float) $productArray["discountPercentage"];

                $product = $productDAO->getProductById($productId);

                //service provider
                $productServiceProviderId = isset($productArray["serviceProvider"]["id"]) ? $productArray["serviceProvider"]["id"] : null;

                $productServiceProvider = null;

                if ($productServiceProviderId) {
                    $productServiceProvider = $userDAO->getUserById($productServiceProviderId);
                }

                if (!$productServiceProvider) {
                    $productServiceProvider = $invoiceServiceProvider;
                }

                //save Invoice line
                $customerInvoiceLine = $customerInvoiceLineDAO->create();

                $customerInvoiceLine->setIdFacture($invoice)
                    ->setIdProduit($product)
                    ->setNumlot($batchNumber)
                    ->setDesignation($productName)
                    ->setPrixUnitaire($productPrice)
                    ->setQuantite($productQuantity)
                    ->setPctRemise($discountPercentage)
                    ->setForfaitRemise($discountAmount)
                    ->setIdPrestataire($productServiceProvider)
                    ->setQuantiteLivree(0)
                    ->setStatut(1);

                $customerInvoiceLineDAO->save($customerInvoiceLine);

                //Destoking (only for products of type product no services)
                if ($saleWithDestocking && $product->getStockable() === 1) {
                    //NB: we suppose that there is only one stock with this batchNumber
                    $stocks = $stockDAO->getStockByBatchNumber($batchNumber);

                    if (!is_array($stocks) or count($stocks) < 1) {
                        $databaseConnection->rollBack();
                        return ApiResponse::respond(101, $translator->_("sale.sell.insufficientStock") . " " . $productName);
                    }
                    //first
                    $stock = $stocks[0];

                    $availableQty = $stock->getQuantite();

                    //if quantity needed is available!
                    if ($productQuantity <= $availableQty) {
                        $exitLine = $exitLineDAO->create();
                        $exitLine->setIdSortie($productsExit)
                            ->setIdStock($stock)
                            ->setQuantite($productQuantity)
                            ->setPrixUnitaire($productPrice)
                            ->setStatut(1);
                        $exitLineDAO->save($exitLine);

                        $finalQuantity = $availableQty - $productQuantity;

                        if ($finalQuantity < 0) {
                            $databaseConnection->rollBack();
                            return ApiResponse::respond(101, $translator->_("sale.sell.insufficientStock") . " " . $productName);
                        }

                        //decrement the quantity in stock
                        $stock->setQuantite($finalQuantity);

                        //update stock line
                        $stockDAO->update($stock);
                    } else { //needed quantity is not available with this batchNumber

                        //if it is separate stock management, we cannot continue
                        if ($separateStock) {
                            $databaseConnection->rollBack();
                            return ApiResponse::respond(101, $translator->_("sale.sell.insufficientStock") . " " . $productName);
                        } else  //get all stock of the current product in store
                        {
                            $stocks = $stockDAO->getAvailableStockByStore($product, null, $store);

                            $requestedQuantity = $productQuantity; //we will decrement requestedQuantity

                            foreach ($stocks as $stock) {
                                $availableQuantity = $stock->getQuantite();

                                if ($availableQuantity >= $requestedQuantity) {
                                    $quantityToCollected = $requestedQuantity;
                                } else {
                                    $quantityToCollected = $availableQuantity;
                                }

                                $requestedQuantity -= $quantityToCollected;

                                $exitLine = $exitLineDAO->create();
                                $exitLine->setIdSortie($productsExit)
                                    ->setIdStock($stock)
                                    ->setQuantite($quantityToCollected)
                                    ->setPrixUnitaire($productPrice)
                                    ->setStatut(1);
                                $exitLineDAO->save($exitLine);

                                $finalQuantity = $availableQuantity - $quantityToCollected;

                                if ($finalQuantity < 0) {
                                    $databaseConnection->rollBack();
                                    return ApiResponse::respond(101, $translator->_("sale.sell.insufficientStock") . " " . $productName);
                                }
                                //decrement the quantity in stock
                                $stock->setQuantite($finalQuantity);

                                //update stock line
                                $stockDAO->update($stock);


                                if ($requestedQuantity <= 0) {
                                    break;
                                }
                            }

                            if ($requestedQuantity > 0) {
                                $databaseConnection->rollBack();
                                return ApiResponse::respond(101, $translator->_("sale.sell.insufficientStock") . " " . $productName);
                            }
                        }
                    }

                    //set that all quantity has been delivrery
                    $customerInvoiceLine->setQuantiteLivree($productQuantity);

                    $customerInvoiceLineDAO->update($customerInvoiceLine);
                }

                //calculate amounts
                $amountExcludingTax += $productPrice * $productQuantity;

                $totalDiscountAmount += $discountAmount * $productQuantity;
            }

            //taxes
            $taxes =  $invoiceBasket["totals"]["taxes"];

            $vatValue = (float) $taxes["vat"]["taxeValue"];

            $itValue = (float) $taxes["it"]["taxeValue"];

            //other amounts
            $vatAmount = $itAmount = $amountIncludingTax = $netAmount = 0;

            if ($itValue > 0) {
                $itAmount = $amountExcludingTax * $itValue / 100;
                $invoice->setMontantIr($itAmount)
                    ->setTauxIr($itValue);
            }

            if ($vatValue > 0) {
                $vatAmount = $amountExcludingTax * $vatValue / 100;
                $invoice->setMontantTva($vatAmount)
                    ->setTauxTva($vatValue);
            }

            $amountIncludingTax = $amountExcludingTax + $vatAmount;

            $netAmount = $amountIncludingTax - $itAmount;

            $totalDiscountPercentage = round((100 * $totalDiscountAmount / $netAmount), 3);

            $invoice->setMontantHt($amountExcludingTax)
                ->setMontantTtc($amountIncludingTax)
                ->setForfaitRemise($totalDiscountAmount)
                ->setPctRemise($totalDiscountPercentage)
                ->setMontantNet($netAmount);

            //with compute this against
            $totalRefunded = 0;
            if ($totalPaid >= 0) {
                $invoice->setMontantVerse($totalPaid);

                if ($totalPaid > $netAmount) {
                    $totalRefunded = $totalPaid - $netAmount;
                }
            }

            if ($totalPaid > 0 && isset($invoiceBasket["paid"]["details"])) {
                $paymentDetails = $invoiceBasket["paid"]["details"];


                foreach ($paymentDetails as $payment) {
                    $paymentId = (int) $payment["id"];

                    $amount = (float) $payment["amount"];

                    $paymentMethod = $paymentMethodDAO->getPaymentMethodById($paymentId);

                    if (!$paymentMethod) {
                        $databaseConnection->rollBack();
                        return ApiResponse::respond(101, $translator->_("sale.sell.unknowPaymentMode") . " " . $payment["name"]);
                    }

                    if ((int) $paymentId == (int) MODE_VERSEMENT_ESPECE or $paymentMethod->getCode() === "ESPECES") {
                        $amount -= $totalRefunded;
                    }


                    $cashMovement =  $cashMovementDAO->create();

                    $cashMovement->setIdModePaiement($paymentMethod)
                        ->setIdFacture($invoice)
                        ->setIdSessionCaisse($cashoutSession)
                        ->setIdUtilisateur($user)
                        ->setType(TYPE_MVT_CAISSE_ENTREE)
                        ->setDateMouvement($currentDate)
                        ->setMontant($amount)
                        ->setMotif($translator->_("sale.sell.invoicePayment"))
                        ->setStatut(1)
                        ->setValideCaisse(1)
                        ->setValideSession(1)
                        ->setIdPointVente($pointOfSale);

                    $cashMovementDAO->save($cashMovement);

                    $invoicePayment =  $invoicePaymentDAO->create();

                    $invoicePayment->setIdModePaiement($paymentMethod)
                        ->setIdFacture($invoice)
                        ->setIdMvtCaisse($cashMovement)
                        ->setIdUtilisateur($user)
                        ->setDatePaiement($currentDate)
                        ->setMontant($amount)
                        ->setStatut(1);

                    $invoicePaymentDAO->save($invoicePayment);
                }

                $invoiceBasket["refunded"]["totalRefunded"] = $totalRefunded;

                $invoiceBasket["refunded"]["formattedTotalRefunded"] =  number_format($totalRefunded, 0, '', ' ');
            }
            //save update
            $customerInvoiceLineDAO->update($invoice);

            $databaseConnection->commit();

            return ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $invoiceBasket);
        } catch (Exception $ex) {

            $databaseConnection->rollBack();

            return ApiResponse::respond(102, $ex->getMessage(), null);
        }
    }

    //This function give detail on invoice identified by $invoiceId  
    public  function getDetailedInvoiceArray($invoice)
    {
        $customerInvoiceLineDAO = new CustomerInvoiceLineDAO();

        $stockDAO = new StockDAO();

        if (!is_object($invoice)) {
            $invoice = $this->getCustomerInvoiceById($invoice);
        }

        if (!is_object($invoice)) {
            return [];
        }

        $invoiceId = $invoice->getIdFacture();

        $amountExcludingTax = $invoice->getMontantHt();
        $amountIncludingTax = $invoice->getMontantTtc();
        $netAmount = $invoice->getMontantNet();
        $amoutPaid = $invoice->getMontantVerse();
        $refunded = ($amoutPaid > $netAmount) ? $amoutPaid - $netAmount : 0;
        $discountsAmount = $invoice->getForfaitRemise();
        $vat = $invoice->getTauxTva();
        $it = $invoice->getTauxIr();
        $vatAmount = $invoice->getMontantTva();
        $itAmount = $invoice->getMontantIr();
        $payments =  $this->getAllOfPaymentsOfInvoice($invoice);
        $serviceProvider = $invoice->getServeur();
        $customer = $invoice->getIdPersonne();
        $checkSession = $invoice->getIdSessionCaissier();
        $sellerSession = $invoice->getIdSessionVendeur();
        $location = $invoice->getIdEmplacement();
        $group = $invoice->getGroupe();
        $pointOfSale = $invoice->getIdPointVente();
        $invoiceLines = $customerInvoiceLineDAO->getCustomerInvoiceLinesOfInvoice($invoiceId);

        if ($checkSession) {
            $cashier = $checkSession->getIdUtilisateur();
            $checkoutSessionArray = array(
                "id" => $checkSession->getIdSessionCaisse(),
                "beginDate" => $checkSession->getDateOuverture()->format("Y-m-d H:i:s"),
                "cashierId" => $cashier->getIdUtilisateur(),
                "cashierName" => $cashier->getIdPersonne()->getNom() . " " . $cashier->getIdPersonne()->getPrenom(),
                "closed" => (int) $checkSession->getCloturer()
            );
        } else {
            $checkoutSessionArray = array(
                "id" => 0,
                "beginDate" => "",
                "cashierId" => 0,
                "cashierName" => "",
                "closed" => 0
            );
        }

        if ($sellerSession) {
            $seller = $sellerSession->getIdUtilisateur();
            $sellerSessionArray = array(
                "id" => $sellerSession->getIdSessionCaisse(),
                "beginDate" => $sellerSession->getDateOuverture()->format("Y-m-d H:i:s"),
                "cashierId" => $seller->getIdUtilisateur(),
                "cashierName" => $seller->getIdPersonne()->getNom() . " " . $seller->getIdPersonne()->getPrenom(),
                "closed" => (int) $sellerSession->getCloturer()
            );
        } else {
            $sellerSessionArray = array(
                "id" => 0,
                "beginDate" => "",
                "cashierId" => 0,
                "cashierName" => "",
                "closed" => 0
            );
        }

        $invoiceBasket = array(
            "invoiceNumber" => $invoiceId,
            "billingDate" => $invoice->getDateFacture()->format("Y-m-d H:i:s"),
            "pending" => (int) $invoice->getEnAttente(),
            "checkoutSession" => $checkoutSessionArray,
            "sellerSession" => $sellerSessionArray,
        );

        $serviceProviderArray = array(
            "id" => ($serviceProvider ? $serviceProvider->getIdPersonne()->getIdPersonne() : 0),
            "name" => ($serviceProvider ? $serviceProvider->getIdPersonne()->getNom() . " " . $serviceProvider->getIdPersonne()->getPrenom() : ""),
        );

        $customerArray = array(
            "id" => ($customer ? $customer->getIdPersonne() : 0),
            "name" => ($customer ? $customer->getNom() . " " . $customer->getPrenom() : $invoice->getNomClient()),
            "phone" => ($customer ? $customer->getTelephone() : ""),
        );

        $locationArray = array(
            "id" => ($location ? $location->getIdEmplacement() : 0),
            "name" => ($location ? $location->getDesignation() : ""),
            "groupNumber" => $group ?  $group : 0
        );

        $pointOfSaleArray = array(
            "id" => ($pointOfSale ? $pointOfSale->getIdPointVente() : 0),
            "name" => ($pointOfSale ? $pointOfSale->getNomPointVente() : ""),
        );

        $amountIncludingTaxArray = array(
            'amount' => $amountIncludingTax,
            'formattedAmount' => number_format($amountIncludingTax, 0, '', ' ')
        );

        $amountExcludingTaxArray = array(
            'amount' => $amountExcludingTax,
            'formattedAmount' => number_format($amountExcludingTax, 0, '', ' ')
        );

        $netAmountArray = array(
            "amount" => $netAmount,
            "formattedAmount" => number_format($netAmount, 0, '', ' ')
        );

        $discountsAmountArray = array(
            "amount" => $discountsAmount,
            "formattedAmount" => number_format($discountsAmount, 0, '', ' ')
        );

        $itArray = array(
            "taxeValue" =>  $it,
            "taxeAmount" => $itAmount,
            "formattedTaxeAmount" => number_format($itAmount, 0, '', ' ')
        );

        $vatArray = array(
            "taxeValue" =>  $vat,
            "taxeAmount" => $vatAmount,
            "formattedTaxeAmount" => number_format($vatAmount, 0, '', ' ')
        );

        $paymentDetailsArray = array();

        foreach ($payments as $payment) {
            $paymentMethodId = $payment->getIdModePaiement()->getId();

            $amount = $payment->getMontant();

            $paymentMethod = $payment->getIdModePaiement()->getDesignation();

            $paymentDetailsArray[] = array(
                "id" => $paymentMethodId,
                "amount" =>  $amount,
                "methodName" => $paymentMethod,
            );
        }

        $paidArray = array(
            "totalPaid" => $amoutPaid,
            "formattedTotalPaid" => number_format($amoutPaid, 0, '', ' '),
            "details" => $paymentDetailsArray
        );


        $refundedArray = array(
            "totalRefunded" => $refunded,
            "formattedTotalRefunded" => number_format($refunded, 0, '', ' '),
        );

        $invoiceBasket["serviceProvider"] = $serviceProviderArray;

        $invoiceBasket["customer"] = $customerArray;

        $invoiceBasket["location"] = $locationArray;

        $invoiceBasket["pointOfSale"] = $pointOfSaleArray;

        $invoiceBasket["considerLocation"] = ((int) GESTION_EMPLACEMENT === 1);

        $invoiceBasket["considerServiceProvider"] = ((int) GESTION_PRESTATAIRE === 1);

        $invoiceBasket["considerVAT"] = ((int) GESTION_TVA === 1);

        $invoiceBasket["considerIT"] = ((int) GESTION_IR === 1);

        $invoiceBasket["totals"] = array(
            "amountExcludingTax" => $amountExcludingTaxArray,
            "amountIncludingTax" => $amountIncludingTaxArray,
            "netAmount" => $netAmountArray,
            "discountsAmount" => $discountsAmountArray,
            "taxes" => array(
                "it" => $itArray,
                "vat" => $vatArray
            )
        );

        $invoiceBasket["paid"] = $paidArray;

        $invoiceBasket["refunded"] = $refundedArray;

        //products
        $productArray = array();

        $imageUrlBase = $this->baseUrl . '/images/products';

        $defautImage = $this->baseUrl . '/images/defaults/image_placeholder.png';

        foreach ($invoiceLines as $invoiceLine) {
            $productId = $invoiceLine->getIdProduit()->getIdProduit();

            $batchNum = $invoiceLine->getNumLot();

            $name = $invoiceLine->getDesignation();

            $price = $invoiceLine->getPrixUnitaire();

            $formattedPrice = number_format($price, 0, '', ' ');

            $quantity = $invoiceLine->getQuantite();

            $discountPercentage = round($invoiceLine->getPctRemise(), 2);

            $discountAmount = round($invoiceLine->getForfaitRemise(), 0);

            $normalSalePrice = $price + $discountAmount;

            $formattedNormalSalePrice = number_format($normalSalePrice, 0, '', ' ');

            $total = $quantity * $price;

            $formattedTotal = number_format($total, 0, '', ' ');

            $key = $productId . "_" . $batchNum;

            $stock = $stockDAO->getStockByBatchNumber($batchNum);

            $stockId = (is_array($stock) && count($stock) > 0) ? $stock[0]->getIdStock() : 0;

            //image
            $image = $defautImage;

            if ($invoiceLine->getIdProduit()->getImage() != null) {
                if (file_exists($imageUrlBase .  "/" . $invoiceLine->getIdProduit()->getImage())) {
                    $image = $imageUrlBase .  "/" . $invoiceLine->getIdProduit()->getImage();
                }
            }

            $productArray[$key] = array(

                "productName" => $name,

                "image" => $image,

                "productId" =>  $productId,

                "productPrice" => $price,

                "formattedPrice" => $formattedPrice,

                "batchNum" => $batchNum,

                "stockId" =>  $stockId,

                "quantity" => $quantity,

                "normalSalePrice" => $normalSalePrice,

                "formattedNormalSalePrice" => $formattedNormalSalePrice,

                "discountPercentage" => $discountPercentage,

                "discountAmount" => $discountAmount,

                "total" => $total,

                "formattedTotal" => $formattedTotal,

            );
        }


        $invoiceBasket["products"] = $productArray;

        return $invoiceBasket;
    }

    public function getTotalAmoutOfCashSession($cashSession)
    {

        $amount =  $this->entityManager->getRepository($this->modelName)->createQueryBuilder("f")
            ->select("SUM(f.montantHt) as Montant")
            ->where("f.statut = 1")
            ->andWhere("f.valider = 1")
            ->andWhere("f.idSessionCaissier = :session")
            ->setParameter("session", $cashSession)
            ->getQuery()
            ->getSingleScalarResult();

        return $amount;
    }

    public function getLastInvoiceOfUser($user)
    {
        $cashSession = $this->entityManager->getRepository($this->modelName)
            ->findOneBy(
                array('idUtilisateur' => $user),
                array('idFacture' => 'DESC')
            );
        return $cashSession;
    }


    public function getSessionPendingInvoices($session, $isCashier = true)
    {
        $countPendingInvoices = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("f")
            ->select("COUNT(f.idFacture)")
            ->where("f.enAttente = 1")
            ->andWhere("f.statut = 1");

            if($isCashier){
                $countPendingInvoices->andWhere("f.idSessionCaissier = :session")
                                     ->setParameter("session", $session);
            }
            else{
                $countPendingInvoices->andWhere("f.idSessionVendeur = :session")
                                     ->setParameter("session", $session);
            }
            
            $countPendingInvoices = $countPendingInvoices->getQuery()
                                            ->getSingleScalarResult();

        if (!$countPendingInvoices) {
            return [
                "count" => 0,
                "invoices" => [],
            ];
        }

        $pendingInvoices  = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("f")
            ->select("f")
            ->where("f.enAttente = 1")
            ->andWhere("f.statut = 1");

            if($isCashier){
                $pendingInvoices->andWhere("f.idSessionCaissier = :session")
                                     ->setParameter("session", $session);
            }
            else{
                $pendingInvoices->andWhere("f.idSessionVendeur = :session")
                                     ->setParameter("session", $session);
            }            

        $pendingInvoices = $pendingInvoices->orderBy("f.idFacture", "ASC")
                                           ->getQuery()
                                           ->getResult();

        return [
            "count" => count($pendingInvoices),
            "invoices" => $pendingInvoices,
        ];
    }
}
