<?php

include APPLICATION_PATH . '/Models/LigneFacturec.php';

class CustomerInvoiceLineDAO
{

    public $entityManager;
    private $modelName = "\Application\Models\LigneFacturec";

    function __construct()
    {
        $registry = Zend_Registry::getInstance();
        $this->entityManager = $registry->entitymanager;
    }

    public function create()
    {
        return new \Application\Models\LigneFacturec();
    }

    /**
     * 
     * @param Application\Models\LigneFacturec $clientInvoiceLine
     */
    public function save($clientInvoiceLine)
    {
        $this->entityManager->persist($clientInvoiceLine);
        $this->entityManager->flush();
    }

    /**
     * 
     * @param Application\Models\LigneFacturec $clientInvoiceLine
     */
    public function update($clientInvoiceLine)
    {
        $this->entityManager->merge($clientInvoiceLine);
        $this->entityManager->flush();
    }

     /**
     * 
     * @param Application\Models\LigneFacturec $clientInvoiceLine
     */
    public function delete($clientInvoiceLine)
    {
        $this->entityManager->remove($clientInvoiceLine);
        $this->entityManager->flush();
    }

    /**
     * 
     * @param integer $id
     * @return Application\Models\LigneFacturec
     */
    public function getClientInvoiceLineById($id)
    {
        $clientInvoiceLine = $this->entityManager->getRepository($this->modelName)->find($id);
        return $clientInvoiceLine;
    }

    /**
     * 
     * @param \Application\Models\Facturec $clientInvoice
     * @return \Application\Models\LigneFacturec
     */
    public function getCustomerInvoiceLinesOfInvoice($clientInvoice){
        $clientInvoiceLinesList = $this->entityManager->getRepository($this->modelName)->findBy(array("statut" => 1 , "idFacture" => $clientInvoice));
        return $clientInvoiceLinesList;
    }

    
    /**
     * 
     * @param \Application\Models\Facturec $clientInvoice
     * @return \Application\Models\LigneFacturec
     */
    public function getClientInvoiceLinesOfClientInvoice($clientInvoice)
    {
        return $this->entityManager->getRepository($this->modelName)->findBy(array("statut" => 1, "idFacture" => $clientInvoice));
    }

    /**
     * 
     * @param string $begin
     * @param string
     * @param \Application\Models\Produit $article
     * @return integer
     */
    public function getSoldQuantityOfArticleOverPeriod($begin, $end, $article)
    {
        $beginDate = new \DateTime($begin . " 00:00:00");
        $endDate = new \DateTime($end . " 23:59:59");
        $soldQuantity = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("lf")
            ->select("SUM(lf.quantite) AS Quantity")
            ->join("lf.idFacture", "f")
            ->where("f.dateFacture >= :begin")
            ->setParameter("begin", $beginDate)
            ->andWhere("f.dateFacture <= :end")
            ->setParameter("end", $endDate)
            ->andWhere("lf.idProduit = :article")
            ->setParameter("article", $article)
            ->andWhere("lf.statut = 1")
            ->getQuery()
            ->getSingleScalarResult();
        return $soldQuantity;
    }

    /**
     * 
     * @param string $day
     * @param \Application\Models\Produit $article
     * @return integer
     */
    public function getSoldQuantityOfArticleOnDay($day, $article)
    {
        return $this->getSoldQuantityOfArticleOverPeriod($day, $day, $article);
    }

    /**
     * 
     * @param DateTime $month
     * @param $article
     * @return integer
     */
    public function getSoldQuantityOfArticleOnMonth(\DateTime $month, $article)
    {
        $beginDate = clone $month;
        $endDate = clone $month;
        $beginDate->modify(" first day of this month");
        $endDate->modify("last day of this month");
        return $this->getSoldQuantityOfArticleOverPeriod($beginDate->format("Y-m-d"), $endDate->format("Y-m-d"), $article);
    }

    /**
     * 
     * @param DateTime $year
     * @param $article
     * @return integer
     */
    public function getSoldQuantityOfArticleOnYear(\DateTime $year, $article)
    {
        $beginDate = clone $year;
        $endDate = clone $year;
        $beginDate->modify("first day of january " . $year->format("Y"));
        $endDate->modify("last day of december " . $year->format("y"));
        return $this->getSoldQuantityOfArticleOverPeriod($beginDate->format("Y-m-d"), $endDate->format("Y-m-d"), $article);
    }

    /**
     * 
     * @param \Application\Models\PointVente[] $pointsOfSale
     * @param \Application\Models\Utilisateur[] $cashiers
     * @param DateTime $startDate
     * @param DateTime $endDate
     * @return mixed[]
     */
    public function getSoldArticleCountByPointsOfSaleAndCashiers($pointsOfSale = [], $cashiers = [], DateTime $startDate, DateTime $endDate)
    {
        $soldArticlesCountQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("lf")
            ->select("SUM(lf.quantite) AS sold_quantity")
            ->addSelect("SUM(lf.quantite * lf.prixUnitaire) AS sold_amount")
            ->addSelect("AVG(lf.prixUnitaire) AS unit_price")
            ->addSelect("pro.prixAchat AS buying_price")
            ->addSelect("(SUM(lf.quantite * lf.prixUnitaire) - SUM(lf.quantite * pro.prixAchat)) AS profit")
            ->addSelect("pro.designation AS product_name")
            ->addSelect("pro.idProduit AS product_id")
            ->addSelect("cat.idCategorie AS category_id")
            ->addSelect("cat.nom AS category_name")
            ->addSelect("pos.nomPointVente AS point_of_sale_name")
            ->addSelect("pos.idPointVente AS point_of_sale_id")
            ->addSelect("cas.idUtilisateur AS cashier_id")
            ->join("lf.idProduit", "pro")
            ->join("pro.idCategorieProduit", "cat")
            ->join("lf.idFacture", "f")
            ->join("f.idPointVente", "pos")
            ->join("f.idUtilisateur", "cas")
            ->where("lf.statut = 1")
            ->andWhere("f.statut = 1")
            ->andWhere("f.valider = 1")
            ->andWhere("f.dateFacture >= :begin")
            ->andWhere("f.dateFacture <= :end")
            ->setParameter("begin", $startDate)
            ->setParameter("end", $endDate);
        if (count($pointsOfSale) > 0) {
            $soldArticlesCountQuery->andWhere($soldArticlesCountQuery->expr()->in("f.idPointVente", ":pointsOfSale"))
                ->setParameter("pointsOfSale", $pointsOfSale);
        }
        if (count($cashiers) > 0) {
            $soldArticlesCountQuery->andWhere($soldArticlesCountQuery->expr()->in("f.idUtilisateur", ":cashiers"))
                ->setParameter("cashiers", $cashiers);
        }
        return $soldArticlesCountQuery->orderBy("point_of_sale_id", "ASC")
            ->addOrderBy("cashier_id", "ASC")
            ->addOrderBy("category_id", "ASC")
            ->addOrderBy("product_name", "ASC")
            ->groupBy("f.idPointVente")
            ->addGroupBy("f.idUtilisateur")
            ->addGroupBy("lf.idProduit")
            ->getQuery()
            ->getResult();
    }

    /**
     * 
     * @param \Application\Models\PointVente[] $pointsOfSale
     * @param DateTime $startDate
     * @param DateTime $endDate
     * @return mixed[]
     */
    public function getSoldArticleCountByPointsOfSale($pointsOfSale = [], DateTime $startDate, DateTime $endDate)
    {
        $soldArticlesCountQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("lf")
            ->select("SUM(lf.quantite) AS sold_quantity")
            ->addSelect("SUM(lf.quantite * lf.prixUnitaire) AS sold_amount")
            ->addSelect("AVG(lf.prixUnitaire) AS unit_price")
            ->addSelect("pro.prixAchat AS buying_price")
            ->addSelect("(SUM(lf.quantite * lf.prixUnitaire) - SUM(lf.quantite * pro.prixAchat)) AS profit")
            ->addSelect("pro.designation AS product_name")
            ->addSelect("pro.idProduit AS product_id")
            ->addSelect("cat.idCategorie AS category_id")
            ->addSelect("cat.nom AS category_name")
            ->addSelect("pos.nomPointVente AS point_of_sale_name")
            ->addSelect("pos.idPointVente AS point_of_sale_id")
            ->join("lf.idProduit", "pro")
            ->join("pro.idCategorieProduit", "cat")
            ->join("lf.idFacture", "f")
            ->join("f.idPointVente", "pos")
            ->where("lf.statut = 1")
            ->andWhere("f.statut = 1")
            ->andWhere("f.valider = 1")
            ->andWhere("f.dateFacture >= :begin")
            ->andWhere("f.dateFacture <= :end")
            ->setParameter("begin", $startDate)
            ->setParameter("end", $endDate);
        if (count($pointsOfSale) > 0) {
            $soldArticlesCountQuery->andWhere($soldArticlesCountQuery->expr()->in("f.idPointVente", ":pointsOfSale"))
                ->setParameter("pointsOfSale", $pointsOfSale);
        }
        return $soldArticlesCountQuery->orderBy("point_of_sale_id", "ASC")
            ->addOrderBy("category_id", "ASC")
            ->addOrderBy("product_name", "ASC")
            ->groupBy("f.idPointVente")
            ->addGroupBy("lf.idProduit")
            ->getQuery()
            ->getResult();
    }

    /**
     * 
     * @param \Application\Models\PointVente[] $pointsOfSale
     * @param \Application\Models\CategorieProduit[] $productCategories
     * @param \Application\Models\Produit[] $products
     * @param DateTime $startDate
     * @param DateTime $endDate
     * @return mixed[]
     */
    public function getSoldArticleCount($pointsOfSale = [], $productCategories = [], $products = [], DateTime $startDate, DateTime $endDate)
    {
        $soldArticlesCountQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("lf")
            ->select("SUM(lf.quantite) AS sold_quantity")
            ->addSelect("SUM(lf.quantite * lf.prixUnitaire) AS sold_amount")
            ->addSelect("AVG(lf.prixUnitaire) AS unit_price")
            ->addSelect("pro.prixAchat AS buying_price")
            ->addSelect("(SUM(lf.quantite * lf.prixUnitaire) - SUM(lf.quantite * pro.prixAchat)) AS profit")
            ->addSelect("pro.designation AS product_name")
            ->addSelect("pro.idProduit AS product_id")
            ->addSelect("cat.idCategorie AS category_id")
            ->addSelect("cat.nom AS category_name")
            ->addSelect("pos.nomPointVente AS point_of_sale_name")
            ->addSelect("pos.idPointVente AS point_of_sale_id")
            ->join("lf.idProduit", "pro")
            ->join("pro.idCategorieProduit", "cat")
            ->join("lf.idFacture", "f")
            ->join("f.idPointVente", "pos")
            ->where("lf.statut = 1")
            ->andWhere("f.statut = 1")
            ->andWhere("f.valider = 1")
            ->andWhere("f.dateFacture >= :begin")
            ->andWhere("f.dateFacture <= :end")
            ->setParameter("begin", $startDate)
            ->setParameter("end", $endDate);
        if (count($pointsOfSale) > 0) {
            $soldArticlesCountQuery->andWhere($soldArticlesCountQuery->expr()->in("f.idPointVente", ":pointsOfSale"))
                ->setParameter("pointsOfSale", $pointsOfSale);
        }
        if (count($productCategories) > 0) {
            $soldArticlesCountQuery->andWhere($soldArticlesCountQuery->expr()->in("pro.idCategorieProduit", ":productCategories"))
                ->setParameter("productCategories", $productCategories);
        }
        if (count($products) > 0) {
            $soldArticlesCountQuery->andWhere($soldArticlesCountQuery->expr()->in("lf.idProduit", ":products"))
                ->setParameter("products", $products);
        }
        return $soldArticlesCountQuery->orderBy("point_of_sale_id", "ASC")
            ->addOrderBy("category_id", "ASC")
            ->addOrderBy("product_name", "ASC")
            ->groupBy("f.idPointVente")
            ->addGroupBy("lf.idProduit")
            ->getQuery()
            ->getResult();
    }

    /**
     * 
     * @param \Application\Models\Personne $customer
     * @param \Application\Models\PointVente[] $pointsOfSale
     * @param DateTime $startDate
     * @param DateTime $endDate
     * @return mixed[]
     */
    public function getSoldArticleCountByCustomer($customer, $pointsOfSale = [], DateTime $startDate, DateTime $endDate)
    {
        $soldArticlesCountQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("lf")
            ->select("SUM(lf.quantite) AS sold_quantity")
            ->addSelect("SUM(lf.quantite * lf.prixUnitaire) AS sold_amount")
            ->addSelect("AVG(lf.prixUnitaire) AS unit_price")
            ->addSelect("pro.prixAchat AS buying_price")
            ->addSelect("(SUM(lf.quantite * lf.prixUnitaire) - SUM(lf.quantite * pro.prixAchat)) AS profit")
            ->addSelect("pro.designation AS product_name")
            ->addSelect("pro.idProduit AS product_id")
            ->addSelect("cat.idCategorie AS category_id")
            ->addSelect("cat.nom AS category_name")
            ->addSelect("pos.nomPointVente AS point_of_sale_name")
            ->addSelect("pos.idPointVente AS point_of_sale_id")
            ->join("lf.idProduit", "pro")
            ->join("pro.idCategorieProduit", "cat")
            ->join("lf.idFacture", "f")
            ->join("f.idPointVente", "pos")
            ->where("lf.statut = 1")
            ->andWhere("f.statut = 1")
            ->andWhere("f.valider = 1")
            ->andWhere("f.dateFacture >= :begin")
            ->andWhere("f.dateFacture <= :end")
            ->setParameter("begin", $startDate)
            ->setParameter("end", $endDate)
            ->andWhere("f.idPersonne = :customer")
            ->setParameter("customer", $customer);
        if (count($pointsOfSale) > 0) {
            $soldArticlesCountQuery->andWhere($soldArticlesCountQuery->expr()->in("f.idPointVente", ":pointsOfSale"))
                ->setParameter("pointsOfSale", $pointsOfSale);
        }
        return $soldArticlesCountQuery->orderBy("point_of_sale_id", "ASC")
            ->addOrderBy("category_id", "ASC")
            ->addOrderBy("product_name", "ASC")
            ->groupBy("f.idPointVente")
            ->addGroupBy("lf.idProduit")
            ->getQuery()
            ->getResult();
    }

    /**
     * 
     * @param DateTime $startDate
     * @param DateTime $endDate
     * @param int $count
     * @param \Application\Models\PointVente $pointOfSale
     * @return mixed[]
     */
    public function getBestSoldArticles($startDate, $endDate, $count, $pointOfSale = null)
    {
        $bestSoldArticlesCountQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("lf")
            ->select("SUM(lf.quantite) AS sold_quantity")
            ->addSelect("SUM(lf.quantite * lf.prixUnitaire) AS sold_amount")
            ->addSelect("pro.designation AS product_name")
            ->addSelect("pro.idProduit AS product_id")
            ->join("lf.idProduit", "pro")
            ->join("pro.idCategorieProduit", "cat")
            ->join("lf.idFacture", "f")
            ->join("f.idPointVente", "pos")
            ->where("lf.statut = 1")
            ->andWhere("f.statut = 1")
            ->andWhere("f.valider = 1")
            ->andWhere("f.dateFacture >= :begin")
            ->andWhere("f.dateFacture <= :end")
            ->setParameter("begin", $startDate)
            ->setParameter("end", $endDate);
        if ($pointOfSale != null) {
            $bestSoldArticlesCountQuery->andWhere("f.idPointVente = :pointsOfSale")
                ->setParameter("pointsOfSale", $pointOfSale);
        }
        return $bestSoldArticlesCountQuery->orderBy("sold_quantity", "DESC")
            ->addGroupBy("lf.idProduit")
            ->setMaxResults($count)
            ->getQuery()
            ->getResult();
    }

    public function getArticlesBySalesSession($session, $isCashierSession = true)
    {
        try {
            $allSessionSales = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("lv")
                ->select("a.idProduit as idArticle")
                ->addselect("lv.designation as designation")
                ->addselect("u.idUtilisateur as idVendeur")
                ->addselect("p.idPointVente as idPointVente")
                ->addselect("p.nomPointVente as nomPointVente")
                ->addselect("sum(lv.quantite) as quantite")
                ->addselect("sum(lv.quantite * lv.prixUnitaire) as montant")
                ->addselect("c.idCategorie as idCategorie")
                ->addselect("c.nom as nomCategorie")
                ->join("lv.idFacture", "v")
                ->join("lv.idProduit", "a")
                ->join("a.idCategorieProduit", "c")
                ->join("v.idUtilisateur", "u")
                ->join("v.idPointVente", "p")
                ->where("v.statut = 1")
                ->andWhere("lv.statut = 1")
                ->andWhere("v.valider = 1");

                if($isCashierSession)
                {
                    $allSessionSales->andWhere("v.idSessionCaissier = :session")
                                    ->setParameter("session", $session);
                }
                else{
                    $allSessionSales->andWhere("v.idSessionVendeur = :session")
                                    ->setParameter("session", $session);
                }
                
            $allSessionSales->addGroupBy("idArticle");
            $allSessionSales->addGroupBy("v.idPointVente");

            $allSessionSales->addOrderBy("idPointVente", "ASC");
            $allSessionSales->addOrderBy("c.idCategorie", "ASC");
            $allSessionSales->addOrderBy("idArticle", "ASC");
        } catch (Exception $ex) {
            die("Error: " . $ex->getMessage());
        }

        return $allSessionSales->getQuery()->getResult();
    }
}
