<?php

include APPLICATION_PATH . '/Models/Sortie.php';

class ExitDAO {
    
    public $entityManager;
    private $modelName = "\Application\Models\Sortie";
    
    function __construct(){
        $registry = Zend_Registry::getInstance();
        $this->entityManager = $registry->entitymanager;
    }

    public function create()
    {
        return new \Application\Models\Sortie();
    }
    
    /**
     * 
     * @param Application\Models\Sortie $exit
     */
    public function save($exit){
        $this->entityManager->persist($exit);
        $this->entityManager->flush();
        
    }
    
    /**
     * 
     * @param Application\Models\Sortie $exit
     */
    public function update($exit){
        $this->entityManager->merge($exit);
        $this->entityManager->flush();
    }
    
    
    /**
     * 
     * @param integer $id
     * @return Application\Models\Sortie
     */
    public function getExitById($id){
        return $this->entityManager->getRepository($this->modelName)->find($id);
    }
    
	
    public function listExits(){
        return $this->entityManager->getRepository($this->modelName)->findBy(array("statut" => 1));
    }
    
	public function getExitByInvoice($invoice){
        return $this->entityManager->getRepository($this->modelName)->findOneBy(array("idFacture" => $invoice, "statut" => 1));
    }
	
    public function listNotValidExits(){
        return $this->entityManager->getRepository($this->modelName)->findBy(array("statut" => 1 , "valide" => (int)SORTIE_NON_VALIDE));
    }
    
    public function getExitsFiltered($user = null, $exitType = null, $begin = null, $end = null, $store = null, $withInterStore = true )
    {
        $exitsList = $this->entityManager->createQueryBuilder()
                             ->select('r')
                             ->from($this->modelName,'r')
                             ->where("r.statut = 1")
                             ->andWhere("r.typeSortie <> '".TYPE_SORTIE_VENTE."'");

        if( !$withInterStore ){
            $exitsList->andWhere("a.typeSortie <> '".TYPE_SORTIE_MVT_INTERMAGASIN."'");
        }

        if($user != null){
            $exitsList->andWhere('a.idUtilisateur =:user')
                         ->setParameter('user', $user);
        }

        if($begin != null)
        {
            $beginDate = new \DateTime($begin->format("Y-m-d"));
            $exitsList->andWhere('a.dateSaisie >= :begin ')
                         ->setParameter('begin', $beginDate);
        }

        if($end != null)
        {
            $endDate = new \DateTime($end->format("Y-m-d") . " 23:59:59");
            $exitsList->andWhere('a.dateSaisie <= :end ')
                         ->setParameter('end', $endDate);
        }
              
        if($exitType != null)
        {
            $exitsList->andWhere('a.typeSortie =:type')
                         ->setParameter('type', $exitType);
        }

        if($store != null)
        {
          $exitsList->andWhere('a.idMagasin =:store')
            ->setParameter('store', $store);
        }
        return $exitsList->orderBy("a.idSortie" , "ASC")->getQuery()->getResult();
    }
}
