<?php

include APPLICATION_PATH . '/Models/LigneSortie.php';

class ExitLineDAO
{

    public $entityManager;
    private $modelName = "\Application\Models\LigneSortie";

    function __construct()
    {
        $registry = Zend_Registry::getInstance();
        $this->entityManager = $registry->entitymanager;
    }

    /**
     * 
     * @return \Application\Models\LigneSortie
     */
    public function create()
    {
        return new \Application\Models\LigneSortie();
    }

    /**
     * 
     * @param \Application\Models\LigneSortie $exitLine
     */
    public function save($exitLine)
    {
        $this->entityManager->persist($exitLine);
        $this->entityManager->flush();
    }

    /**
     * 
     * @param \Application\Models\LigneSortie $exitLine
     */
    public function update($exitLine)
    {
        $this->entityManager->merge($exitLine);
        $this->entityManager->flush();
    }

    /**
     * 
     * @param \Application\Models\LigneSortie $exitLine
     */
    public function delete($exitLine)
    {
        $this->entityManager->remove($exitLine);
        $this->entityManager->flush();
    }

    /**
     * 
     * @param int $id
     * @return \Application\Models\Sortie
     */
    public function getExitLineById($id)
    {
        $exitLine = $this->entityManager->getRepository($this->modelName)->find($id);
        return $exitLine;
    }

    /**
     * 
     * @param \Application\Models\Sortie $exit
     * @return \Application\Models\LigneSortie[]
     */
    public function getExitLinesOfExit($exit)
    {
        return $this->entityManager->getRepository($this->modelName)->findBy(array("idSortie" => $exit, "statut" => 1));
    }

    public function getExitLinesOfProductOverPeriod($product = null, $begin = null, $end = null, $store = null)
    {
        $exitLinesQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("ls")
            ->join("ls.idSortie", "so")
            ->join("ls.idStock", "st")
            ->where("s.statut = 1");

        if ($product != null) {
            $exitLinesQuery->andWhere("st.idProduit = :product")
                ->setParameter("product", $product);
        }

        if ($begin != null) {
            $beginDate = new \DateTime($begin->format("Y-m-d") . " 00:00:00");
            $exitLinesQuery->andWhere("so.dateValidation >= :begin")
                ->setParameter("begin", $beginDate);
        }

        if ($end != null) {
            $endDate = new \DateTime($end->format("Y-m-d") . " 23:59:59");
            $exitLinesQuery->andWhere("so.dateValidation <= :end")
                ->setParameter("end", $endDate);
        }

        if ($store != null) {
            $exitLinesQuery->andWhere("so.idMagasin = :store")
                ->setParameter("store", $store);
        }

        return $exitLinesQuery->getQuery()->getResult();
    }

    public function getExitLinesOfProductBeforeDate($product = null, $date = null)
    {
        $exitLinesQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("ls")
            ->join("ls.idSortie", "so")
            ->join("ls.idStock", "st")
            ->where("so.statut = 1");

        if ($product != null) {
            $exitLinesQuery->andWhere("st.idProdui = :product")
                ->setParameter("product", $product);
        }

        if ($date != null) {
            $effectiveDate = new \DateTime($date->format("Y-m-d") . " 23:59:59");
            $exitLinesQuery->andWhere("so.dateValidation <= :date")
                ->setParameter("date", $effectiveDate);
        }

        return $exitLinesQuery->getQuery()->getResult();
    }

    public function getSumOfQuantityOfProductAtDate($product, $date, $store = null)
    {
        $effectiveDate = new \DateTime($date->format("Y-m-d") . " 29:59:59");
        $sumOfQuantityQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("ls")
            ->select("SUM(ls.quantite) AS Quantity")
            ->join("ls.idSortie", "so")
            ->join("ls.idStock", 'st')
            ->where("so.statut = 1")
            ->andWhere("st.idProduit = :product")
            ->setParameter("product", $product)
            ->andWhere("so.dateValidation <= :date")
            ->setParameter("date", $effectiveDate);

        if ($store != null) {
            $sumOfQuantityQuery->andWhere("so.idMagasin = :store")
                ->setParameter("store", $store);
        }

        return $sumOfQuantityQuery->getQuery()->getSingleScalarResult();
    }

    public function getSumOfQuantityOfProductOverPeriod($product, $begin, $end, $store = null)
    {
        $beginDate = new \DateTime($begin->format("Y-m-d") . " 00:00:00");
        $endDate = new \DateTime($end->format("Y-m-d") . " 23:59:59");
        $sumOfQuantityQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("ls")
            ->select("SUM(ls.quantite) AS Quantity")
            ->join("ls.idSortie", "so")
            ->join("ls.idStock", "st")
            ->where("so.statut = 1")
            ->andWhere("st.idProduit = :product")
            ->setParameter("product", $product)
            ->andWhere("so.dateValidation >= :begin")
            ->setParameter("begin", $beginDate)
            ->andWhere("so.dateValidation <= :end")
            ->setParameter("end", $endDate);

        if ($store != null) {
            $sumOfQuantityQuery->andWhere("so.idMagasin = :store")
                ->setParameter("store", $store);
        }

        return $sumOfQuantityQuery->getQuery()->getSingleScalarResult();
    }

    public function getSumOfQuantityOfProductBeforeDate($product, $date, $store = null)
    {
        $sumOfQuantityQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("ls")
            ->select("SUM(ls.quantite) as Quantity")
            ->join("ls.idSortie", "so")
            ->join("ls.idStock", "st")
            ->where("so.statut = 1")
            ->andWhere("st.idProduit = :product")
            ->setParameter("product", $product)
            ->andWhere("so.dateValidation < :date")
            ->setParameter("date", $date);

        if ($store != null) {
            $sumOfQuantityQuery->andWhere("so.idMagasin = :store")
                ->setParameter("store", $store);
        }

        return $sumOfQuantityQuery->getQuery()->getSingleScalarResult();
    }

    public function getSumOfQuantityOfProductOfDate($product, $date, $store = null)
    {
        $sumOfQuantityQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("ls")
            ->select("SUM(ls.quantite) as Quantity")
            ->join("ls.idSortie", "so")
            ->join("ls.idStock", "st")
            ->where("so.statut = 1")
            ->andWhere("st.idProduit = :product")
            ->setParameter("product", $product)
            ->andWhere("so.dateValidation = :date")
            ->setParameter("date", $date);

        if ($store != null) {
            $sumOfQuantityQuery->andWhere("so.idMagasin = :store")
                ->setParameter("store", $store);
        }

        return $sumOfQuantityQuery->getQuery()->getSingleScalarResult();
    }

    /**
     * 
     * @param \Application\Models\Produit $product
     * @return \Application\Models\LigneSortie
     */
    public function getLastExitLineOfProduct($product) {
        $lastExitLineQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("ls")
            ->join("ls.idStock", "st")
            ->where("st.statut = 1")
            ->andWhere("st.idProduit = :product")
            ->setParameter("product", $product)
            ->join("ls.idSortie", "so")
            ->andWhere("so.statut = 1")
            ->andWhere("so.valider = 1")
            ->orderBy("so.dateValidation", "DESC")
            ->setMaxResults(1)
            ->getQuery()
            ->getResult();
        return count($lastExitLineQuery) > 0 ? $lastExitLineQuery[0] : null;
    }
}
