<?php

include APPLICATION_PATH . '/Models/PaiementFacture.php';

class InvoicePaymentDAO
{

    public $entityManager;

    function __construct()
    {
        $registry = Zend_Registry::getInstance();
        $this->entityManager = $registry->entitymanager;
    }

    public function create(){
        return new \Application\Models\PaiementFacture();
    }

    /**
     * 
     * @param \Application\Models\PaiementFacture $invoicePayment
     */
    public function save($invoicePayment)
    {
        $this->entityManager->persist($invoicePayment);
        $this->entityManager->flush();
    }

    /**
     * 
     * @param \Application\Models\PaiementFacture $invoicePayment
     */
    public function update($invoicePayment)
    {
        $this->entityManager->merge($invoicePayment);
        $this->entityManager->flush();
    }

    /**
     * 
     * @param \Application\Models\Facture $invoice
     * @return \Application\Models\PaiementFacture
     */
    public function listPaymentsOfInvoice($invoice)
    {
        return $this->entityManager->getRepository('\Application\Models\PaiementFacture')->findBy(array('idFacture' => $invoice, 'statut' => 1));
    }

    /**
     * 
     * @param integer $id
     * @return Array of \Application\Models\PaiementFacture
     */
    public function getInvoicePaymentById($id)
    {
        return $this->entityManager->getRepository('\Application\Models\PaiementFacture')->findOneBy(array('idPaiementFacture' => $id));
    }

    /**
     * 
     * @param \Application\Models\Facture $invoice
     * @return \Application\Models\PaiementFacture
     */
    public function getLastInvoicePayment($invoice)
    {
        return $this->entityManager->getRepository('\Application\Models\PaiementFacture')->findOneBy(array('idFacture' => $invoice, 'statut' => 1), array('date' => 'DESC'));
    }

    /**
     * 
     * @param \Application\Models\Facture $invoice
     * @return float
     */
    public function getSumOfInvoicePayments($invoice)
    {
        return (float)$this->entityManager->getRepository("\Application\Models\PaiementFacture")->createQueryBuilder("p")
            ->select("SUM(p.montant) as amount")
            ->where("p.statut = 1")
            ->join("p.idMvtCaisse", "mvc")
            ->andWhere("mvc.statut = 1")
            ->andWhere("mvc.valideCaisse = 1")
            ->andWhere("p.idFacture = :invoice")
            ->setParameter("invoice", $invoice)
            ->getQuery()
            ->getSingleScalarResult();
    }

    /**
     * 
     * @param integer $paymentMethod
     * @return \Application\Models\PaiementFacture
     */
    public function getInvoicePaymentByPaymentMethod($paymentMethod)
    {
        return $this->entityManager->getRepository('\Application\Models\PaiementFacture')->findBy(array('modePaiement' => $paymentMethod, 'statut' => 1));
    }


    public function getInvoicePaymentByCashMovement($cashMovement)
    {
        return $this->entityManager->getRepository("\Application\Models\PaiementFacture")->findBy(array("idMvtCaisse" => $cashMovement, "statut" => 1));
    }

    /**
     * 
     * @param string $day
     * @return int
     */
    public function getSumOfInvoicePaymentsByDay($day)
    {
        $beginDate = new \DateTime($day . " 00:00:00");
        $endDate = new \DateTime($day . " 23:59:59");
        return $this->entityManager->getRepository("\Application\Models\PaiementFacture")->createQueryBuilder("p")
            ->select("SUM(p.montant) AS amount")
            ->where("p.statut  = 1")
            ->join("p.idMvtCaisse", "mvc")
            ->andWhere("mvc.statut = 1")
            ->andWhere("mvc.valideCaisse = 1")
            ->andWhere("p.datePaiement <= :end")
            ->andWhere("p.datePaiement >= :begin")
            ->setParameter("dateFin", $endDate)
            ->setParameter("dateDebut", $beginDate)
            ->getQuery()
            ->getSingleScalarResult();
    }

    public function getSumOfInvoicePaymentsOnInterval(\DateTime $begin, \DateTime $end)
    {
        $beginDate = new \DateTime($begin->format("Y-m-d") . " 00:00:00");
        $endDate = new \DateTime($end->format("Y-m-d") . " 23:59:59");
        $sumOfInvoicePayments = $this->entityManager->getRepository("\Application\Models\PaiementFacture")->createQueryBuilder("p")
            ->select("SUM(p.montant) as Montant")
            ->where("p.statut = 1")
            ->andWhere("p.datePaiement <= :end")
            ->andWhere("p.datePaiement >= :begin")
            ->setParameter("dateFin", $endDate)
            ->setParameter("dateDebut", $beginDate)
            ->getQuery()
            ->getSingleScalarResult();
        return $sumOfInvoicePayments;
    }

    /**
     * 
     * @param \DateTime $month
     * @return float
     */
    public function getSumOfInvoicePaymentsOnMonth(\DateTime $month)
    {
        $beginDate = clone $month;
        $endDate = clone $month;
        $beginDate->modify(" first day of this month");
        $endDate->modify("last day of this month");
        $sumOfInvoicePayments = $this->getSumOfInvoicePaymentsOnInterval($beginDate, $endDate);
        return $sumOfInvoicePayments;
    }

    public function getSumOfInvoicePaymentsOnYear(\DateTime $year)
    {
        $beginDate = clone $year;
        $endDate = clone $year;
        $beginDate->modify("first day of january " . $year->format("Y"));
        $endDate->modify("last day of december " . $year->format("y"));
        $sumOfInvoicePayments = $this->getSumOfInvoicePaymentsOnInterval($beginDate, $endDate);
        return $sumOfInvoicePayments;
    }
}
