<?php

include APPLICATION_PATH . '/Models/CarteFidelite.php';

class LoyaltyCardDAO
{
    public $entityManager;
    private $modelName = "\Application\Models\CarteFidelite";

    function __construct()
    {
        $registry = Zend_Registry::getInstance();
        $this->entityManager = $registry->entitymanager;
    }

    /**
     * 
     * @return \Application\Models\CarteFidelite
     */
    public function create()
    {
        return new \Application\Models\CarteFidelite();
    }

    /**
     * 
     * @param \Application\Models\CarteFidelite $loyaltyCard
     */
    public function save($loyaltyCard)
    {
        $this->entityManager->persist($loyaltyCard);
        $this->entityManager->flush();
    }

    /**
     * 
     * @param \Application\Models\CarteFidelite $loyaltyCard
     */
    public function update($loyaltyCard)
    {
        $this->entityManager->merge($loyaltyCard);
        $this->entityManager->flush();
    }

    /**
     * 
     * @param \Application\Models\PointVente $pointOfSale
     * @return \Application\Models\CarteFidelite[]
     */
    public function getLoyaltyCardsByPointOfSale($pointOfSale)
    {
        return $this->entityManager->getRepository($this->modelName)
            ->findBy(array("statut" => 1, "idPointVente" => $pointOfSale));
    }

    /**
     * 
     * @param \Application\Models\PointVente $pointOfSale
     * @return \Application\Models\CarteFidelite[]
     */
    public function getActiveLoyaltyCardsByPointOfSale($pointOfSale) {
        $currentDate = new DateTime();
        return $this->entityManager->getRepository($this->modelName)->createQueryBuilder("c")
            ->where("c.statut = 1")
            ->andWhere("c.dateExpiration >= :date")
            ->setParameter("date", $currentDate)
            ->andWhere("c.idPointVente = :pointOfSale")
            ->setParameter("pointOfSale", $pointOfSale)
            ->getQuery()
            ->getResult();
    }

    /**
     * 
     * @param int $id
     * @return \Application\Models\CarteFidelite
     */
    public function getLoyaltyCardById($id)
    {
        return $this->entityManager->getRepository($this->modelName)
            ->findOneBy(array("idCarteFidelite" => $id));
    }

    /**
     * 
     * @param \Application\Models\Personne $customer
     * @param \Application\Models\PointVente $pointOfSale
     * @param \Application\Models\CarteFidelite
     */
    public function getCustomerActiveCard($customer, $pointOfSale)
    {
        $currentDate = new DateTime();
        return $this->entityManager->getRepository($this->modelName)->createQueryBuilder("c")
            ->where("c.statut = 1")
            ->andWhere("c.idPersonne = :customer")
            ->andWhere("c.dateExpiration >= :expirationDate")
            ->andWhere("c.idPointVente = :pointOfSale")
            ->setParameter("customer", $customer)
            ->setParameter("expirationDate", $currentDate)
            ->setParameter("pointOfSale", $pointOfSale)
            ->getQuery()
            ->getOneOrNullResult();
    }

    /**
     * 
     * @param string $code
     * @return \Application\Models\CarteFidelite
     */
    public function getLoyaltyCardByCode($code)
    {
        $currentDate = new \DateTime();
        return $this->entityManager->getRepository($this->modelName)->createQueryBuilder("c")
            ->where("c.statut = 1")
            ->andWhere("c.dateExpiration >= :date")
            ->setParameter("date", $currentDate)
            ->andWhere("c.code = :code")
            ->setParameter("code", $code)
            ->getQuery()
            ->getOneOrNullResult();
    }

    /**
     * 
     * @param DateTime $begin
     * @param DateTime $end
     * @param \Application\Models\Personne $customer
     * @return mixed[]
     */
    public function getMostUsedCategoriesByPeriodAndCustomer(DateTime $begin, DateTime $end, $customer)
    {
        $begin = new DateTime($begin->format("Y-m-d") . " 00:00:00");
        $end = new DateTime($end->format("Y-m-d") . " 23:59:59");
        return $this->entityManager->getRepository("\Application\Models\LigneFacturec")->createQueryBuilder("l")
            ->select("c.idCategorie AS idCategorie")
            ->addSelect("COUNT(c.idCategorie) AS occurrences")
            ->addSelect("c.nom AS nom")
            ->where("l.statut = 1")
            ->join("l.idFacture", "f")
            ->andWhere("f.statut = 1")
            ->andWhere("f.valider = 1")
            ->andWhere("f.idPersonne = :customer")
            ->setParameter("customer", $customer)
            ->andWhere("f.dateFacture >= :begin")
            ->andWhere("f.dateFacture <= :end")
            ->setParameter("begin", $begin)
            ->setParameter("end", $end)
            ->join("l.idProduit", "p")
            ->join("p.idCategorieProduit", "c")
            ->groupBy("p.idCategorieProduit")
            ->orderBy("occurrences", "DESC")
            ->setMaxResults(5)
            ->getQuery()
            ->getResult();
    }

    /**
     * 
     * @return int
     */
    public function getLastGroupNumber()
    {
        return $this->entityManager->getRepository("\Application\Models\CarteFidelite")->createQueryBuilder("c")
            ->select("MAX(c.numeroGroupe) AS lastGroup")
            ->getQuery()
            ->getSingleScalarResult();
    }

    /**
     * 
     * @param int $groupNumber
     * @param \Application\Models\CarteFidelite[]
     */
    public function getLoyaltyCardsByGroupNumber($groupNumber)
    {
        return $this->entityManager->getRepository("\Application\Models\CarteFidelite")
            ->findBy(array("statut" => 1, "numeroGroupe" => $groupNumber));
    }

}