<?php

include APPLICATION_PATH . '/Models/DetailPaieMensuel.php';

class MonthlyPayDetailDAO
{
    public $entityManager;
    private $modelName = "\Application\Models\DetailPaieMensuel";

    function __construct()
    {
        $registry = Zend_Registry::getInstance();
        $this->entityManager = $registry->entitymanager;
    }

    /**
     * 
     * @return \Application\Models\DetailPaieMensuel
     */
    public function create()
    {
        return new \Application\Models\DetailPaieMensuel();
    }

    /**
     * 
     * @param \Application\Models\DetailPaieMensuel $monthlyPayDetail
     */
    public function save($monthlyPayDetail)
    {
        $this->entityManager->persist($monthlyPayDetail);
        $this->entityManager->flush();
    }

    /**
     * 
     * @param \Application\Models\DetailPaieMensuel $montlyPayDetail
     */
    public function update($monthlyPayDetail)
    {
        $this->entityManager->merge($monthlyPayDetail);
        $this->entityManager->flush();
    }

    /**
     * 
     * @param Application\Models\DetailPaieMensuel $montlyPayDetail
     */
    public function delete($montlyPayDetail)
    {
        $this->entityManager->remove($montlyPayDetail);
        $this->entityManager->flush();
    }

    /**
     * 
     * @param int $id
     * @return Application\Models\DetailPaieMensuel
     */
    public function getMonthlyPayDetailById($id)
    {
        return $this->entityManager->getRepository($this->modelName)->find($id);
    }

    /**
     * 
     * @param \Application\Models\PaieMensuel $monthlyPay
     * @return \Application\Models\DetailPaieMensuel[]
     */
    public function getMonthlyPayDetailOfMonthlyPay($monthlyPay)
    {
        return $this->entityManager->getRepository($this->modelName)->findBy(array("idPaieMensuel" => $monthlyPay, "statut" => 1));
    }

    /**
     * 
     * @param \Application\Models\PaieMensuel $montlyPay
     * @param \Application\Models\RubriquePaie $payrollSection
     * @return Application\Models\DetailPaieMensuel
     */
    public function getMonthlyPayDetailOfMonthlyPayAndPayrollSection($monthlyPay, $payrollSection)
    {
        return $this->entityManager->getRepository($this->modelName)->findOneBy(array("idPaieMensuel" => $monthlyPay, "idRubriquePaie" => $payrollSection, "statut" => 1));
    }

    /**
     * 
     * @param \Application\Models\Mois $month
     * @return mixed[]
     */
    public function getAllPayrollSectionsOfAMonth($month) {
        return $this->entityManager->getRepository($this->modelName)->createQueryBuilder("d")
            ->select("r.idRubriquePaie As payrollSectionId")
            ->where("d.statut = 1")
            ->join("d.idRubriquePaie", "r")
            ->join("d.idPaieMensuel", "p")
            ->join("p.idMois", "m")
            ->andWhere("p.statut = 1")
            ->andWhere("m.idMois = :monthId")
            ->setParameter("monthId", $month->getIdMois())
            ->groupBy("d.idRubriquePaie")
            ->getQuery()
            ->getScalarResult();
    }
}
