<?php

include APPLICATION_PATH . '/Models/Commande.php';

class OrderDAO
{
    public $entityManager;
    private $modelName = "\Application\Models\Commande";

    function __construct()
    {
        $registry = Zend_Registry::getInstance();
        $this->entityManager = $registry->entitymanager;
    }

    public function create()
    {
        return new \Application\Models\Commande();
    }

    public function save($order)
    {
        $this->entityManager->persist($order);
        $this->entityManager->flush();
    }

    public function update($order)
    {
        $this->entityManager->merge($order);
        $this->entityManager->flush();
    }

    /**
     * 
     * @return \Application\Models\Commande
     */
    public function getOrderById($id)
    {
        return $this->entityManager->getRepository($this->modelName)->find($id);
    }

    public function listOrders()
    {
        return $this->entityManager->getRepository($this->modelName)->findBy(array("statut" => 1));
    }

    /**
     * 
     * @param \Application\Models\Fournisseur $supplier
     * @param DateTime $begin
     * @param DateTime $end
     * @param \Application\Models\PointVente $pointOfSale
     * @return \Application\Models\Commande[]
     */
    public function getOrdersFiltered($supplier = null, $begin = null, $end = null, $pointOfSale = null)
    {
        $ordersQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("o")
            ->where("o.statut = 1");

        if ($supplier != null) {
            $ordersQuery->andWhere('o.idFournisseur = :supplier')
                ->setParameter('supplier', $supplier);
        }

        if ($begin != null) {
            $beginDate = new \DateTime($begin->format("Y-m-d") . " 00:00:00");
            $ordersQuery->andWhere('o.dateCommande >= :begin')
                ->setParameter('begin', $beginDate);
        }

        if ($end != null) {
            $endDate = new \DateTime($end->format("Y-m-d") . " 23:59:59");
            $ordersQuery->andWhere('o.dateCommande <= :end')
                ->setParameter('end', $endDate);
        }

        if ($pointOfSale != null) {
            $ordersQuery->andWhere('o.idPointVente = :pointOfSale')
                ->setParameter('pointOfSale', $pointOfSale);
        }

        return $ordersQuery->orderBy("o.idFournisseur, o.idCommande", "ASC")->getQuery()->getResult();
    }

    public function listOrdersOfSupplier($supplier)
    {
        return $this->entityManager->getRepository($this->modelName)->createQueryBuilder("o")
            ->where("o.statut = 1")
            ->andWhere("o.idFournisseur = :provider")
            ->setParameter("supplier", $supplier)
            ->getQuery()
            ->getResult();
    }
}
