<?php

include APPLICATION_PATH . '/Models/Personne.php';

class PersonDAO
{
    public $entityManager;
    private $modelName = "\Application\Models\Personne";

    function __construct()
    {
        $registry = Zend_Registry::getInstance();
        $this->entityManager = $registry->entitymanager;
    }

    /**
     * 
     * @return \Application\Models\Personne
     */
    public function create()
    {
        return new \Application\Models\Personne();
    }

    public function save($person)
    {
        $this->entityManager->persist($person);
        $this->entityManager->flush();
    }

    public function update($person)
    {
        $this->entityManager->merge($person);
        $this->entityManager->flush();
    }

    /**
     * 
     * @param int $id
     * @return \Application\Models\Personne
     */
    public function getPersonById($id)
    {
        return $this->entityManager->getRepository($this->modelName)->find($id);
    }

    /**
     * 
     * @return \Application\Models\Personne[]
     */
    public function listCustomers()
    {
        return $this->entityManager->getRepository($this->modelName)
            ->findBy(array('statut' => 1, 'type' => TYPE_PERSONNE_CLIENT), array('idPersonne' => 'ASC'));
    }

    /**
     * 
     * @param string $gender
     * @return \Application\Models\Personne[]
     */
    public function listCustomersReadyForMessages($gender = null)
    {
        $customersQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("p")
            ->where("p.statut = 1")
            ->andWhere("p.type = :type")
            ->setParameter("type", TYPE_PERSONNE_CLIENT);
        if ($gender != null) {
            $customersQuery->andWhere("p.sexe = :gender")
                ->setParameter("gender", $gender);
        }
        $customersQuery->andWhere($customersQuery->expr()->orX(
            $customersQuery->expr()->isNotNull('p.telephone'),
            $customersQuery->expr()->isNotNull('p.email')
        ));
        return $customersQuery->getQuery()->getResult();
    }

    /**
     * 
     * @param string $gender
     * @return \Application\Models\Personne[]
     */
    public function listCustomersBornTodayReadyForMessages($gender = null)
    {
        $currentDate = new DateTime();
        $currentDateAtStart = new DateTime("{$currentDate->format(DATE_FORMAT)} 00:00:00");
        $currentDateAtEnd = new DateTime("{$currentDate->format(DATE_FORMAT)} 23:59:59");
        $customersQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("p")
            ->where("p.statut = 1")
            ->andWhere("p.type = :type")
            ->setParameter("type", TYPE_PERSONNE_CLIENT)
            ->andWhere("p.dateNaissance >= :startDate")
            ->setParameter("startDate", $currentDateAtStart)
            ->andWhere("p.dateNaissance <= :endDate")
            ->setParameter("endDate", $currentDateAtEnd);
        if ($gender != null) {
            $customersQuery->andWhere("p.sexe = :gender")
                ->setParameter("gender", $gender);
        }
        $customersQuery->andWhere($customersQuery->expr()->orX(
            $customersQuery->expr()->isNotNull('p.telephone'),
            $customersQuery->expr()->isNotNull('p.email')
        ));
        return $customersQuery->getQuery()->getResult();
    }

    /**
     * 
     * @return \Application\Models\Personne[]
     */
    public function listUsers()
    {
        return $this->entityManager->getRepository($this->modelName)
            ->findBy(array('statut' => 1, 'type' => TYPE_PERSONNE_UTILISATEUR), array('idPersonne' => 'ASC'));
    }

    public function getCustomerByPhoneNumber($phoneNumber)
    {
        return $this->entityManager->getRepository($this->modelName)->findBy(array("type" => TYPE_PERSONNE_CLIENT, "telephone" => $phoneNumber, "statut" => 1));
    }

    public function customerPhoneNumberExists($phoneNumber, $customer = null)
    {
        $customerList = $this->entityManager->getRepository($this->modelName)->findBy(array("type" => TYPE_PERSONNE_CLIENT, "telephone" => $phoneNumber, "statut" => 1));
        return $customer != null ?
            count($customerList) > 0 && $customerList[0] != $customer
            : count($customerList) > 0;
    }

    /**
     * 
     * @param string $name
     * @return \Application\Models\Personne[]
     */
    public function searchClientByName($name)
    {
        //We will optimize a little bit the search here by splitting the search name query
        $nameParts = explode(" ", $name);
        $matchingCustomersQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("p")
            ->where("p.type = :type")
            ->setParameter("type", TYPE_PERSONNE_CLIENT);
        $countNamePart = 1;
        foreach ($nameParts as $namePart) {
            $uppercaseNamePart = strtoupper($namePart);
            $matchingCustomersQuery->andWhere("UPPER(CONCAT(p.nom, ' ', COALESCE(p.prenom, ''))) LIKE :namePart{$countNamePart}")
                ->setParameter("namePart{$countNamePart}", "%{$uppercaseNamePart}%");
            $countNamePart++;
        }
        return $matchingCustomersQuery->andWhere("p.statut = 1")
            ->getQuery()
            ->getResult();
    }

    public function checkClientOfClientProfileExist($clientProfile)
    {
        $this->entityManager->getRepository($this->modelName)->findBy(array("type" => TYPE_PERSONNE_CLIENT, "idProfilClient" => $clientProfile, "statut" => 1));
    }
}
