<?php

include  APPLICATION_PATH . '/Models/IntervallePoint.php';

class PointIntervalDAO
{
    public $entityManager;
    private $modelName = "\Application\Models\IntervallePoint";

    function __construct()
    {
        $registry = Zend_Registry::getInstance();
        $this->entityManager = $registry->entitymanager;
    }

    /**
     * 
     * @return \Application\Models\IntervallePoint
     */
    public function create()
    {
        return new \Application\Models\IntervallePoint();
    }

    /**
     * 
     * @param \Application\Models\IntervallePoint $pointInterval
     */
    public function save($pointInterval)
    {
        $this->entityManager->persist($pointInterval);
        $this->entityManager->flush();
    }

    /**
     * 
     * @param \Application\Models\IntervallePoint $pointInterval
     */
    public function update($pointInterval)
    {
        $this->entityManager->merge($pointInterval);
        $this->entityManager->flush();
    }

    /**
     * 
     * @param int $id
     * @return \Application\Models\IntervallePoint
     */
    public function getPointIntervalById($id)
    {
        return $this->entityManager->getRepository($this->modelName)->find($id);
    }

    /**
     * 
     * @param float $minimumAmount
     * @param float $maximumAmount
     * @param int $pointIntervalId
     * @return \Application\Models\IntervallePoint
     */
    public function overlappingCheck($minimumAmount, $maximumAmount, $pointIntervalId = null)
    {
        $overlappingQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("p")
            ->where("p.statut = 1");
        $overlappingQuery->andWhere(
            $overlappingQuery->expr()->orX(
                $overlappingQuery->expr()->andX(
                    $overlappingQuery->expr()->gte('p.sommeMinimale', ':min'),
                    $overlappingQuery->expr()->lte('p.sommeMinimale', ':max')
                ),
                $overlappingQuery->expr()->andX(
                    $overlappingQuery->expr()->gte('p.sommeMaximale', ':min'),
                    $overlappingQuery->expr()->lte('p.sommeMaximale', ':max')
                ),
                $overlappingQuery->expr()->andX(
                    $overlappingQuery->expr()->gte(':min', 'p.sommeMinimale'),
                    $overlappingQuery->expr()->lte(':min', 'p.sommeMaximale')
                ),
                $overlappingQuery->expr()->andX(
                    $overlappingQuery->expr()->gte(':max', 'p.sommeMinimale'),
                    $overlappingQuery->expr()->lte(':max', 'p.sommeMaximale')
                )
            )
        );

        $overlappingQuery->setParameter("min", $minimumAmount)
            ->setParameter("max", $maximumAmount);

        if ($pointIntervalId != null) {
            $overlappingQuery->andWhere("p.idIntervallePoint <> :pointIntervalId")
                ->setParameter("pointIntervalId", $pointIntervalId);
        }

        return $overlappingQuery->setMaxResults(1)
            ->getQuery()
            ->getOneOrNullResult();
    }

    /**
     * 
     * @param float $amount
     * @return \Application\Models\IntervallePoint
     */
    public function getPointIntervalIncluingAmount($amount)
    {
        return $this->entityManager->getRepository($this->modelName)->createQueryBuilder("p")
            ->where("p.statut = 1")
            ->andWhere("p.sommeMinimale <= :amount")
            ->andWhere("p.sommeMaximale >= :amount")
            ->setParameter("amount", $amount)
            ->setMaxResults(1)
            ->getQuery()
            ->getOneOrNullResult();
    }
}
