<?php

include  APPLICATION_PATH . '/Models/PeriodeValiditePoint.php';

class PointValidityPeriodDAO
{
    public $entityManager;
    private $modelName = "\Application\Models\PeriodeValiditePoint";

    function __construct()
    {
        $registry = Zend_Registry::getInstance();
        $this->entityManager = $registry->entitymanager;
    }

    /**
     * 
     * @return \Application\Models\PeriodeValiditePoint
     */
    public function create()
    {
        return new \Application\Models\PeriodeValiditePoint();
    }

    /**
     * 
     * @param \Application\Models\PeriodeValiditePoint $pointValidityPeriod
     */
    public function save($pointValidityPeriod)
    {
        $this->entityManager->persist($pointValidityPeriod);
        $this->entityManager->flush();
    }

    /**
     * 
     * @param \Application\Models\PeriodeValiditePoint $pointValidityPeriod
     */
    public function update($pointValidityPeriod)
    {
        $this->entityManager->merge($pointValidityPeriod);
        $this->entityManager->flush();
    }

    /**
     * 
     * @param int $pointValidityPeriodId
     * @return \Application\Models\PeriodeValiditePoint
     */
    public function getPointValidityPeriodById($pointValidityPeriodId)
    {
        return $this->entityManager->getRepository($this->modelName)->findOneBy(array("idPeriodeValiditePoint" => $pointValidityPeriodId));
    }

    /**
     * 
     * @param DateTime $startDate
     * @param DateTime $endDate
     * @param int $pointValidityPeriodId
     * @return \Application\Models\PeriodeValiditePoint[]
     */
    public function getOverlappingDiscounts($startDate, $endDate, $pointValidityPeriodId = null)
    {
        $overlappingQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("p")
            ->where("p.statut = 1");
        $overlappingQuery->andWhere(
            $overlappingQuery->expr()->orX(
                $overlappingQuery->expr()->andX(
                    $overlappingQuery->expr()->gte('p.dateDebut', ':start'),
                    $overlappingQuery->expr()->lte('p.dateDebut', ':end')
                ),
                $overlappingQuery->expr()->andX(
                    $overlappingQuery->expr()->gte('p.dateFin', ':start'),
                    $overlappingQuery->expr()->lte('p.dateFin', ':end')
                ),
                $overlappingQuery->expr()->andX(
                    $overlappingQuery->expr()->gte(':start', 'p.dateDebut'),
                    $overlappingQuery->expr()->lte(':start', 'p.dateFin')
                ),
                $overlappingQuery->expr()->andX(
                    $overlappingQuery->expr()->gte(':end', 'p.dateDebut'),
                    $overlappingQuery->expr()->lte(':end', 'p.dateFin')
                )
            )
        );

        $overlappingQuery->setParameter("start", $startDate)
            ->setParameter("end", $endDate);

        if ($pointValidityPeriodId != null) {
            $overlappingQuery->andWhere("p.idPeriodeValiditePoint <> :pointValidityPeriodId")
                ->setParameter("pointValidityPeriodId", $pointValidityPeriodId);
        }

        return $overlappingQuery->getQuery()
            ->getResult();
    }

    /**
     * 
     * @param DateTime $date
     * @return \Application\Models\PeriodeValiditePoint[]
     */
    public function getPointValidityPeriodIncluingDate(DateTime $date = null)
    {
        if ($date == null) $date = new DateTime();
        return $this->entityManager->getRepository($this->modelName)->createQueryBuilder("p")
            ->where("p.statut = 1")
            ->andWhere("p.dateDebut <= :date")
            ->andWhere("p.dateFin >= :date")
            ->setParameter("date", $date)
            ->getQuery();
    }
}
