<?php

include APPLICATION_PATH . '/Models/ParametrePrix.php';

class PriceSettingDAO
{
    public $entityManager;
    private $modelName = "\Application\Models\ParametrePrix";

    function __construct()
    {
        $registry = Zend_Registry::getInstance();
        $this->entityManager = $registry->entitymanager;
    }

    /**
     * 
     * @return \Application\Models\ParametrePrix
     */
    public function create()
    {
        return new \Application\Models\ParametrePrix();
    }

    /**
     * 
     * @param int $dayNumber
     * @return DateTime
     */
    public function getLastDayOfNumber($dayNumber)
    {
        $currentDate = new DateTime();
        $currentDate->modify("next monday");
        switch ($dayNumber) {
            case 1:
                return $currentDate->modify("last monday");
            case 2:
                return $currentDate->modify("last tuesday");
            case 3:
                return $currentDate->modify("last wednesday");
            case 4:
                return $currentDate->modify("last thursday");
            case 5:
                return $currentDate->modify("last friday");
            case 6:
                return $currentDate->modify("last saturday");
            case 7:
                return $currentDate->modify("last sunday");
        }
    }

    /**
     * 
     * @param \Application\Models\ParametrePrix $priceSetting
     */
    public function save($priceSetting)
    {
        $this->entityManager->persist($priceSetting);
        $this->entityManager->flush();
    }

    /**
     * 
     * @param \Application\Models\ParametrePrix $priceSetting
     */
    public function update($priceSetting)
    {
        $this->entityManager->merge($priceSetting);
        $this->entityManager->flush();
    }

    /**
     * 
     * @param int $id
     * @return \Application\Models\ParametrePrix
     */
    public function getPriceSettingById($id)
    {
        return $this->entityManager->getRepository($this->modelName)->find($id);
    }

    /**
     * 
     * @return \Application\Models\ParametrePrix[]
     */
    public function listPriceSettings()
    {
        return $this->entityManager->getRepository($this->modelName)->findBy(array("statut" => 1));
    }

    /**
     * 
     * @param \Application\Models\PointVente $pointOfSale
     * @return \Application\Models\ParametrePrix[]
     */
    public function getPriceSettingsByPointOfSale($pointOfSale = null)
    {
        $priceSettings = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("p")
            ->where("p.statut = 1");
        if ($pointOfSale != null) {
            $priceSettings->andWhere("p.idPointVente = :pointOfSale")
                ->setParameter("pointOfSale", $pointOfSale);
        }
        return $priceSettings->getQuery()
            ->getResult();
    }

    /**
     * 
     * @param \Application\Models\PointVente $pointOfSale
     * @param DateTime $dateTime
     * @return int
     */
    public function getConsideredPriceByPointOfSale($pointOfSale, $dateTime = null)
    {
        if ($dateTime == null) {
            $dateTime = new \DateTime();
        }
        $targetDay = $dateTime->format("N");
        $allPriceSettingsOfPointOfSale = $this->getPriceSettingsByPointOfSale($pointOfSale);
        $consideredPrice = 0;
        foreach ($allPriceSettingsOfPointOfSale as $priceSetting) {
            $beginDayHour = $priceSetting->getJourDebut();
            $endDayHour = $priceSetting->getJourFin();
            $explodeBegin = explode(" ", $beginDayHour);
            $explodeEnd = explode(" ", $endDayHour);
            $beginDay = (int)$explodeBegin[0];
            $beginHour = $explodeBegin[1];
            $endDay = (int)$explodeEnd[0];
            $endHour = $explodeEnd[1];
            if ($targetDay >= $beginDay && $targetDay <= $endDay) {
                $lastBeginDay = $this->getLastDayOfNumber($beginDay);
                $lastEndDay = $this->getLastDayOfNumber($endDay);
                $lastBeginDay = new DateTime("{$lastBeginDay->format(DATE_FORMAT)} {$beginHour}");
                $lastEndDay = new DateTime("{$lastEndDay->format(DATE_FORMAT)} {$endHour}");
                $toTimeBegin = strtotime($lastBeginDay->format(DATE_FORMAT_WITH_TIME));
                $toTimeEnd = strtotime($lastEndDay->format(DATE_FORMAT_WITH_TIME));
                $toTimeCurrent = strtotime($dateTime->format(DATE_FORMAT_WITH_TIME));
                if ($toTimeCurrent >= $toTimeBegin && $toTimeCurrent <= $toTimeEnd) {
                    $consideredPrice = $priceSetting->getPrixApplicable();
                    break;
                }
            }
        }
        return $consideredPrice;
    }
}
