<?php

include APPLICATION_PATH . '/Models/CategorieProduit.php';

class ProductCategoryDAO
{
    public $entityManager;
    private $modelName = "\Application\Models\CategorieProduit";

    function __construct()
    {
        $registry = Zend_Registry::getInstance();
        $this->entityManager = $registry->entitymanager;
    }

    /**
     * 
     * @return \Application\Models\CategorieProduit
     */
    public function create()
    {
        return new \Application\Models\CategorieProduit();
    }

    public function save($productCategory)
    {
        $this->entityManager->persist($productCategory);
        $this->entityManager->flush();
    }

    public function update($productCategory)
    {
        $this->entityManager->merge($productCategory);
        $this->entityManager->flush();
    }

    /**
     * 
     * @param int $id
     * @return \Application\Models\CategorieProduit
     */
    public function getProductCategoryById($id)
    {
        return $this->entityManager->getRepository($this->modelName)->find($id);
    }

    public function getProductCategoryByName($name)
    {
        return $this->entityManager->getRepository($this->modelName)->findOneBy(array("nom" => $name, "statut" => 1));
    }

    public function getAllProductCategories()
    {
        return $this->entityManager->getRepository($this->modelName)->findBy(array("statut" => 1), array("nom" => "ASC"));
    }

    public function getProductCategoriesHavingProductsInStock()
    {
        return $this->entityManager->getRepository($this->modelName)->createQueryBuilder("c")
            ->where("c.statut = 1")
            ->innerJoin(\Application\Models\Produit::class,
                'p',
                'with',
                'c.idCategorie = p.idCategorieProduit and p.statut = 1'
            )
            ->innerJoin(\Application\Models\Stock::class,
                's',
                'with',
                'p.idProduit = s.idProduit and s.statut= 1 and s.quantite > 0'
            )
            ->getQuery()
            ->getResult();
    }

    public function productCategoryNameAlreadyExists($name, $productCategory = null)
    {
        $productCategoryList = $this->entityManager->getRepository($this->modelName)->findBy(array("nom" => $name, "statut" => 1));
        return $productCategory != null ?
            count($productCategoryList) > 0 && $productCategoryList[0] != $productCategory
            : count($productCategoryList) > 0;
    }

    public function findProductCategoryByName($name)
    {
        return $this->entityManager->getRepository($this->modelName)->createQueryBuilder("c")
            ->where("c.statut = 1")
            ->andWhere("UPPER(c.nom) LIKE :name")
            ->setParameter("name", '%' . strtoupper($name) . '%')
            ->getQuery()
            ->getResult();
    }

    public function getSubCategoriesOfProductCategory($productCategory)
    {
        return $this->entityManager->getRepository($this->modelName)->createQueryBuilder("c")
            ->where("c.statut = 1")
            ->andWhere("c.idParent = :category")
            ->setParameter("category", $productCategory)
            ->getQuery()
            ->getResult();
    }

    public function getParentCategories()
    {
        return $this->entityManager->getRepository($this->modelName)->createQueryBuilder('c')
            ->where("c.statut = 1")
            ->andWhere("c.idParent is NULL")
            ->getQuery()
            ->getResult();
    }
}
