<?php

include APPLICATION_PATH . '/Models/Produit.php';

class ProductDAO
{
    public $entityManager;
    private $modelName = "\Application\Models\Produit";

    function __construct()
    {
        $registry = Zend_Registry::getInstance();
        $this->entityManager = $registry->entitymanager;
    }

    /**
     * 
     * @return \Application\Models\Produit
     */
    public function create()
    {
        return new \Application\Models\Produit();
    }

    public function save($product)
    {
        $this->entityManager->persist($product);
        $this->entityManager->flush();
    }

    public function update($product)
    {
        $this->entityManager->merge($product);
        $this->entityManager->flush();
    }

    /**
     * 
     * @param int $id
     * @return \Application\Models\Produit
     */
    public function getProductById($id)
    {
        return $this->entityManager->getRepository($this->modelName)->find($id);
    }

    public function getProductByCode($code)
    {
        return $this->entityManager->getRepository($this->modelName)->findOneBy(array("code" => $code));
    }

    public function getProducts()
    {
        return $this->entityManager->getRepository($this->modelName)->findBy(array("statut" => 1), array("designation" => "ASC"));
    }

    public function getUsedProductsByStore($storeId = null)
    {
        $productsListQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("p")
            ->select("DISTINCT(p.idProduit) as idProduit")
            ->where("1 = 1");

        if ($storeId != null) {
            $productsListQuery->andWhere("p.idMagasin = :storeId")
                ->setParameter("storeId", $storeId);
        }

        $productsListQuery->orderBy("p.designation", "ASC");
        return $productsListQuery->getQuery()->getResult();
    }

    public function checkProductNameExist($name, $product = null)
    {
        $possibleProduct = $this->entityManager->getRepository($this->modelName)->findOneBy(array("designation" => $name, "statut" => 1));
        return $product != null ?
            $possibleProduct != null && $possibleProduct != $product
            : $possibleProduct != null;
    }

    public function checkProductReferenceExist($reference, $product = null)
    {
        $possibleProduct = $this->entityManager->getRepository($this->modelName)->findOneBy(array("referenceProduit" => $reference, "statut" => 1));
        return $product != null ?
            $possibleProduct != null && $possibleProduct != $product
            : $possibleProduct != null;
    }

    public function checkProductBarCodeExist($barCode, $product = null)
    {
        $possibleProduct = $this->entityManager->getRepository($this->modelName)->findOneBy(array("codeBarre" => $barCode, "statut" => 1));
        return $product != null ?
            $possibleProduct != null && $possibleProduct != $product
            : $possibleProduct != null;
    }

    public function getProductsFiltered($name = null, $category = null, $type = null, $offset = 0, $limit = 100)
    {
        $productsListQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("p")
            ->where("p.statut = 1");
        if ($name != null) {
            $productsListQuery->andWhere("UPPER(p.designation) LIKE :name")
                ->setParameter("name", '%' . strtoupper(trim($name)) . '%');
        }

        if ($category) {
            $productsListQuery->andWhere("p.idCategorieProduit = :category")
                ->setParameter("category", $category);
        }

        if (!is_null($type)) {
            $productsListQuery->andWhere("p.stockable = {$type}");
        }

        return $productsListQuery
                    ->orderBy("p.designation", "ASC")
                    ->setFirstResult($offset)
                    ->setMaxResults($limit)
                    ->getQuery()
                    ->getResult();
    }

    public function getServicesByName($name = null)
    {
        $servicesListQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("p")
            ->where("p.statut = 1")
            ->andWhere("p.stockable = 0");

        if ($name != null) {
            $servicesListQuery->andWhere("UPPER(p.designation) LIKE :name")
                ->setParameter("name", '%' . strtoupper(trim($name)) . '%');
        }

        return $servicesListQuery->orderBy("p.designation", "ASC")->getQuery()->getResult();
    }

    public function getProductsByBrand($brand)
    {
        return $this->entityManager->getRepository($this->modelName)->createQueryBuilder("p")
            ->where("p.statut = 1")
            ->join("p.idMarque", "m")
            ->andWhere("UPPER(m.nomMarque) like :name")
            ->setParameter("name", '%' . strtoupper(trim($brand)) . '%')
            ->getQuery()
            ->getResult();
    }

    public function getProductsOfFamily($family)
    {
        return $this->entityManager->getRepository($this->modelName)->findOneBy(array("statut" => 1, "idFamille" => $family));
    }

    public function getProductsOfUnit($unit)
    {
        return $this->entityManager->getRepository($this->modelName)->findOneBy(array("statut" => 1, "idUniteGestion" => $unit));
    }

    public function getProductsOfType($type)
    {
        return $this->entityManager->getRepository($this->modelName)->findOneBy(array("statut" => 1, "idType" => $type));
    }

    public function getProductAvailableByCriteria($store = null, $name = null, $category = null, $stockSepare = true, $offset = 0, $limit = 100)
    {
        $productInStock = $this->entityManager->getRepository("\Application\Models\Stock")->createQueryBuilder("s")
            ->join("s.idProduit", "p")
            ->join("p.idCategorieProduit", "c")
            ->select("SUM(s.quantite) as quantity")
            ->addSelect("s.idStock as idStock")
            ->addSelect("p.idProduit as idProduit")
            ->addSelect("s.designation as designation")
            ->addSelect("c.idCategorie as idCategorie")
            ->addSelect("p.image as image")
            ->addSelect("s.pourcentageMaxRemise as pourcentageMaxRemise")
            ->addSelect("s.numLot as numLot")
            ->addSelect("s.prixVente as prixVente")
            ->addSelect("s.prixVente2 as prixVente2")
            ->addSelect("s.prixVente3 as prixVente3")
            ->addSelect("s.datePeremption as datePeremption")
            ->where("s.statut = 1")
            ->andWhere("p.statut = 1")
            ->andWhere("s.quantite > 0");

            if ($store) {
                $productInStock->andWhere("s.idMagasin = :mag")
                    ->setParameter("mag", $store);
            }

            if ($category) {
                $productInStock->andWhere("p.idCategorieProduit = :category")
                    ->setParameter("category", $category);
            }

            if ($name) {
                $productInStock->andWhere("p.designation like :name")
                    ->setParameter("name", "%" . trim($name) . "%");
            }

            $productInStock->groupBy("s.idProduit");

            if ($stockSepare) {
                $productInStock->addGroupBy("s.idStock");
            }

            $productInStock->orderBy("p.designation", "ASC");
            

            return $productInStock
                    ->setFirstResult($offset)
                    
                    ->setMaxResults($limit)

                    ->getQuery()

                    ->getResult();
    }

    public function getProductsByBrandAndProvider($brandName, $providerId = null)
    {
        $productsList = $this->entityManager->getRepository("\Application\Models\ProduitFournisseur")->createQueryBuilder("pf")
            ->select("p.idProduit")
            ->addSelect("p.designation")
            ->addSelect("m.nomMarque")
            ->addSelect("pf.prixAchat")
            ->where("pf.statut = 1")
            ->join("pf.idProduit", "p")
            ->join("p.idMarque", "m");
        if ($brandName != null) {
            $productsList->andWhere("m.nomMarque like :brand")
                ->setParameter("brand", "%" . $brandName . "%");
        }

        if ($providerId != null) {
            $productsList->andWhere("pf.idFournisseur = :providerId ")
                ->setParameter("providerId", $providerId);
        }

        $productsList->groupBy("pf.idProduit");
        return $productsList->getQuery()->getResult();
    }

    public function getNearProducts($productId)
    {
        $previousProduct = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("a")
            ->where("a.statut = 1")
            ->andWhere("a.idProduit < :productId")
            ->setParameter("productId", $productId)
            ->setMaxResults(1)
            ->orderBy("a.idProduit", "DESC")
            ->getQuery()
            ->getResult();

        $nextProduct = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("a")
            ->where("a.statut = 1")
            ->andWhere("a.idProduit > :productId")
            ->setParameter("productId", $productId)
            ->setMaxResults(1)
            ->orderBy("a.idProduit", "ASC")
            ->getQuery()
            ->getResult();

        $previous = $previousProduct != null ? $previousProduct : null;
        $next = $nextProduct != null ? $nextProduct : null;

        return array("previous" => $previous, "next" => $next);
    }

    public function listProductsByFamily($family = null)
    {

        $productsList = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("p")
            ->select("p")
            ->where("p.statut = 1");
        if ($family != null) {
            $productsList->andWhere("p.idFamille = :family")
                ->setParameter("family", $family);
        }

        $productsList->orderBy("p.idFamille", "ASC");

        return $productsList->getQuery()->getResult();
    }


    public function getProductsByCategory($category)
    {
        return $this->entityManager->getRepository($this->modelName)->findBy(array("idCategorieProduit" => $category, "statut" => 1), array("designation" => "ASC"));
    }

    public function getProductsByNature($nature)
    {
        return $this->entityManager->getRepository($this->modelName)->findBy(array("idNature" => $nature, "statut" => 1), array("designation" => "ASC"));
    }

    /**
     * 
     * @return int
     */
    public function getProductsCount()
    {
        return $this->entityManager->getRepository($this->modelName)->createQueryBuilder("p")
            ->select("COUNT(p.idProduit)")
            ->where("p.statut = 1")
            ->getQuery()
            ->getSingleScalarResult();
    }

    /**
     * 
     * @param \Application\Models\CategorieProduit[] $productCategories
     * @return |Application\Models\Produit[]
     */
    public function getProductsOfCategories($productCategories = []) {
        $productsOfCategoriesQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("p")
            ->where("p.statut = 1");
        if (count($productCategories) > 0) {
            $productsOfCategoriesQuery->andWhere($productsOfCategoriesQuery->expr()->in("p.idCategorieProduit", ":productCategories"))
                ->setParameter("productCategories", $productCategories);
        }
        return $productsOfCategoriesQuery->orderBy("p.designation", "ASC")
            ->getQuery()
            ->getResult();
    }
}
