<?php

include APPLICATION_PATH . '/Models/Promotion.php';

class PromotionDAO
{
    public $entityManager;
    private $modelName = "\Application\Models\Promotion";

    function __construct()
    {
        $registry = Zend_Registry::getInstance();
        $this->entityManager = $registry->entitymanager;
    }

    /**
     * 
     * @return \Application\Models\Promotion
     */
    public function create()
    {
        return new \Application\Models\Promotion();
    }

    /**
     *
     * @param Application\Models\Promotion $promotion
     */
    public function save($promotion){
        $this->entityManager->persist($promotion);
        $this->entityManager->flush();
    }

    /**
     *
     * @param Application\Models\Promotion $promotion
     */
    public function update($promotion){
        $this->entityManager->merge($promotion);
        $this->entityManager->flush();
    }

    /**
     *
     * @param int $id
     * @return Application\Models\Promotion
     */
    public function getPromotionById($id){
        return $this->entityManager->getRepository($this->modelName)->find($id);
    }

    /**
     * 
     * @param DateTime $startDate
     * @param DateTime $endDate
     * @param int $promotionId
     * @return \Application\Models\Promotion[]
     */
    public function getOverlappingPromotions($startDate, $endDate, $promotionId = null)
    {
        $overlappingQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("p")
            ->where("p.statut = 1");
        $overlappingQuery->andWhere(
            $overlappingQuery->expr()->orX(
                $overlappingQuery->expr()->andX(
                    $overlappingQuery->expr()->gte('p.dateDebut', ':start'),
                    $overlappingQuery->expr()->lte('p.dateDebut', ':end')
                ),
                $overlappingQuery->expr()->andX(
                    $overlappingQuery->expr()->gte('p.dateFin', ':start'),
                    $overlappingQuery->expr()->lte('p.dateFin', ':end')
                ),
                $overlappingQuery->expr()->andX(
                    $overlappingQuery->expr()->gte(':start', 'p.dateDebut'),
                    $overlappingQuery->expr()->lte(':start', 'p.dateFin')
                ),
                $overlappingQuery->expr()->andX(
                    $overlappingQuery->expr()->gte(':end', 'p.dateDebut'),
                    $overlappingQuery->expr()->lte(':end', 'p.dateFin')
                )
            )
        );

        $overlappingQuery->setParameter("start", $startDate)
            ->setParameter("end", $endDate);

        if ($promotionId != null) {
            $overlappingQuery->andWhere("p.idPromotion <> :promotionId")
                ->setParameter("promotionId", $promotionId);
        }

        return $overlappingQuery->getQuery()
            ->getResult();
    }

    /**
     * 
     * @param DateTime $date
     * @return \Application\Models\Promotion[]
     */
    public function getPromotionsIncluingDate(DateTime $date = null)
    {
        if ($date == null) $date = new DateTime();
        return $this->entityManager->getRepository($this->modelName)->createQueryBuilder("p")
            ->where("p.statut = 1")
            ->andWhere("p.dateDebut <= :date")
            ->andWhere("p.dateFin >= :date")
            ->setParameter("date", $date)
            ->getQuery();
    }


    public function currentPromotion($currentDate){
        $promotions = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("p")
            ->where("p.dateDebut <= :date")
            ->andWhere("p.dateFin >= :date")
            ->andWhere("p.statut = 1")
            ->andWhere("p.valide = 1")
            ->setParameter("date", $currentDate)
            ->getQuery()
            ->getResult();
        return $promotions;
    }

}