<?php

include APPLICATION_PATH . '/Models/Reception.php';

class ReceptionDAO
{
    public $entityManager;
    private $modelName = "\Application\Models\Reception";

    function __construct()
    {
        $registry = Zend_Registry::getInstance();
        $this->entityManager = $registry->entitymanager;
    }

    /**
     * 
     * @return \Application\Models\Reception
     */
    public function create()
    {
        return new \Application\Models\Reception();
    }

    /**
     * 
     * @param \Application\Models\Reception $reception
     */
    public function save($reception)
    {
        $this->entityManager->persist($reception);
        $this->entityManager->flush();
    }

    /**
     * 
     * @param \Application\Models\Reception $reception
     */
    public function update($reception)
    {
        $this->entityManager->merge($reception);
        $this->entityManager->flush();
    }

    /**
     * 
     * @param int $id
     * @return \Application\Models\Reception
     */
    public function getReceptionById($id)
    {
        return $this->entityManager->getRepository($this->modelName)->find($id);
    }

    /**
     * 
     * @return \Application\Models\Reception[]
     */
    public function listReceptions()
    {
        return $this->entityManager->getRepository($this->modelName)->findBy(array("statut" => 1));
    }

    /**
     * 
     * @return \Application\Models\Reception[]
     */
    public function listNotValidReceptions()
    {
        return $this->entityManager->getRepository($this->modelName)->findBy(array("statut" => 1, "valide" => (int)RECEPTION_NON_VALIDE));
    }

    /**
     * 
     * @param DateTime $begin
     * @param DateTime $end
     * @return \Application\Models\Reception[]
     */
    public function getReceptionsByDate($begin, $end)
    {
        $receptionsList = $this->entityManager->createQueryBuilder()
            ->select('r')
            ->from($this->modelName, 'r')
            ->where("r.statut = 1");

        if ($begin != null) {
            $receptionsList->andWhere(' a.dateReception >= :begin ')
                ->setParameter('begin', $begin);
        }

        if ($begin != null) {
            $receptionsList->andWhere(' a.dateReception <= :end ')
                ->setParameter('end', $end);
        }

        return $receptionsList->getQuery()->getResult();
    }

    /**
     * 
     * @param \Application\Models\Utilisateur $user
     * @param \Application\Models\Fournisseur $supplier
     * @return \Application\Models\Reception[]
     */
    public function getReceptionByUserAndSupplier($user = null, $supplier = null)
    {
        $receptionsList = $this->entityManager->createQueryBuilder()
            ->select('r')
            ->from($this->modelName, 'r')
            ->innerJoin('r.idUtilisateur', 'u')
            ->innerJoin('r.idFournisseur', 'p')
            ->where("r.statut = 1");

        if ($supplier != null) {
            $receptionsList->andWhere("r.idFournisseur = p.idFournisseur")
                ->andWhere(' r.idFournisseur = :supplier ')
                ->setParameter('provider', $supplier);
        }

        if ($user != null) {
            $receptionsList->andWhere("r.idUtilisateur = u.idUtilisateur")
                ->andWhere(' r.idUtilisateur = :utilisateur ')
                ->setParameter('utilisateur', $user);

        }

        return $receptionsList->getQuery()->getResult();
    }

    /**
     * 
     * @param string $deliveryNoteNumber
     * @param \Application\Models\Reception $reception
     * @return boolean
     */
    public function receptionDeliveryNoteNumberAlreadyExists($deliveryNoteNumber, $reception = null)
    {
        $receptionList = $this->entityManager->getRepository($this->modelName)->findBy(array("numBl" => $deliveryNoteNumber, "statut" => 1));
        return $reception != null ?
            count($receptionList) > 0 && $receptionList[0] != $reception
            : count($receptionList) > 0;
    }

    /**
     * 
     * @param \Application\Models\Magasin $store
     * @param \Application\Models\Fournisseur $supplier
     * @param DateTime $begin
     * @param DateTime $end
     * @return \Application\Models\Reception[]
     */
    public function getReceptionsFiltered($store = null, $supplier = null, $begin = null, $end = null) {
        $receptionsList = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("r")
            ->where("r.statut = 1")
            ->andWhere("r.idFournisseur IS NOT NULL");
        
        if($store != null) {
            $receptionsList->andWhere("r.idMagasin = :store")
                ->setParameter("store", $store);
        }

        if($supplier != null) {
            $receptionsList->andWhere("r.idFournisseur = :supplier")
                ->setParameter("supplier", $supplier);
        }

        if($begin != null) {
            $receptionsList->andWhere("r.dateReception >= :begin")
                ->setParameter("begin", $begin);
        }

        if($end != null) {
            $receptionsList->andWhere("r.dateReception <= :end")
                ->setParameter("end", $end);
        }

        return $receptionsList->orderBy("r.idMagasin, r.idFournisseur, r.idReception", "ASC")->getQuery()->getResult();
    }
}
