<?php

include APPLICATION_PATH . '/Models/LigneReception.php';

class ReceptionLineDAO
{
    public $entityManager;
    private $modelName = "\Application\Models\LigneReception";

    function __construct()
    {
        $registry = Zend_Registry::getInstance();
        $this->entityManager = $registry->entitymanager;
    }

    /**
     * 
     * @return \Application\Models\LigneReception
     */
    public function create()
    {
        return new \Application\Models\LigneReception();
    }

    public function save($receptionLine)
    {
        $this->entityManager->persist($receptionLine);
        $this->entityManager->flush();
    }

    public function update($receptionLine)
    {
        $this->entityManager->merge($receptionLine);
        $this->entityManager->flush();
    }

    public function delete($receptionLine)
    {
        $this->entityManager->remove($receptionLine);
        $this->entityManager->flush();
    }

    public function getReceptionLineById($id)
    {
        return $this->entityManager->getRepository($this->modelName)->find($id);
    }

    /**
     * 
     * @param \Application\Models\Reception
     * @return \Application\Models\LigneReception[]
     */
    public function getReceptionLinesOfReception($reception)
    {
        return $this->entityManager->getRepository($this->modelName)->findBy(array("idReception" => $reception));
    }

    public function getReceptionLinesOfProduct($product)
    {
        return $this->entityManager->getRepository($this->modelName)->findBy(array("idProduit" => $product));
    }

    public function getReceptionLinesOfProductOverPeriod($product = null, \DateTime $begin = null, \DateTime $end = null, $store = null)
    {
        $receptionLinesQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("l")
            ->join("l.idReception", "r")
            ->where("r.statut = 1");

        if ($product != null) {
            $receptionLinesQuery->andWhere("l.idProduit = :product")
                ->setParameter("product", $product);
        }

        if ($begin != null) {
            $beginDate = new DateTime($begin->format("Y-m-d") . " 00:00:00");
            $receptionLinesQuery->andWhere("r.dateValidation >= :begin")
                ->setParameter("begin", $beginDate);
        }

        if ($end != null) {
            $endDate = new DateTime($end->format("Y-m-d") . " 23:59:59");
            $receptionLinesQuery->andWhere("r.dateValidation <= :end")
                ->setParameter('end', $endDate);
        }

        if ($store != null) {
            $receptionLinesQuery->andWhere("r.idMagasin = :store")
                ->setParameter('store', $store);
        }

        return $receptionLinesQuery->getQuery()->getResult();
    }

    public function getReceptionLinesOfProductBeforeDate($product = null, $date = null)
    {
        $receptionLinesQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("l")
            ->join("l.idReception", "r")
            ->where("r.statut = 1");

        if ($product != null) {
            $receptionLinesQuery->andWhere("a.idProduit = :product")
                ->setParameter("product", $product);
        }

        if ($date != null) {
            $effectiveDate = new DateTime($date->format("Y-m-d") . " 23:59:59");
            $receptionLinesQuery->andWhere("r.dateValidation <= :date")
                ->setParameter("date", $effectiveDate);
        }

        return $receptionLinesQuery->getQuery()->getResult();
    }

    public function getSumOfQuantifyOfProductAtDate($product, \DateTime $date, $store = null)
    {
        $effectiveDate = new DateTime($date->format("Y-m-d") . " 23:59:59");
        $sumOfQuantityOfProductQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("l")
            ->select("SUM(l.quantite) AS quantity")
            ->join("l.idReception", "r")
            ->where("r.statut = 1")
            ->andWhere("l.idProduit = :product")
            ->setParameter("product", $product)
            ->andWhere("r.dateValidation <= :date")
            ->setParameter("date", $effectiveDate);

        if ($store != null) {
            $sumOfQuantityOfProductQuery->andWhere("r.idMagasin = :store")
                ->setParameter("store", $store);
        }

        return $sumOfQuantityOfProductQuery->getQuery()->getSingleScalarResult();
    }

    public function getSumOfQuantityOfProductOverPeriod($product, \DateTime $begin, \DateTime $end, $store = null)
    {
        $beginDate = new DateTime($begin->format("Y-m-d") . " 00:00:00");
        $endDate = new DateTime($end->format("Y-m-d") . " 23:59:59");
        $sumOfQuantityOfProductQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("l")
            ->join("l.idReception", "r")
            ->where("r.statut = 1")
            ->andWhere("l.idProduit = :product")
            ->setParameter("product", $product)
            ->andWhere("r.dateValidation >= :begin")
            ->setParameter("begin", $beginDate)
            ->andWhere("r.dateValidation <= :end")
            ->setParameter("end", $endDate);

        if ($store != null) {
            $sumOfQuantityOfProductQuery->andWhere("r.idMagasin = :store")
                ->setParameter("store", $store);
        }

        return $sumOfQuantityOfProductQuery->getQuery()->getSingleScalarResult();
    }

    public function getSumOfQuantityOfProductBeforeDate($product, $date, $store = null)
    {
        $sumOfQuantityOfProductQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("l")
            ->join("l.idReception", "r")
            ->where("r.statut = 1")
            ->andWhere("l.idProduit = :product")
            ->setParameter("product", $product)
            ->andWhere("r.dateValidation < :date")
            ->setParameter("date", $date);

        if ($store != null) {
            $sumOfQuantityOfProductQuery->andWhere("r.idMagasin = :store")
                ->setParameter("store", $store);
        }

        return $sumOfQuantityOfProductQuery->getQuery()->getSingleScalarResult();
    }

    public function getOfSumOfQuantityOfProductOfDate($product, $date, $store = null)
    {
        $sumOfQuantityOfProductQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("l")
            ->join("l.idReception", "r")
            ->where("r.statut = 1")
            ->andWhere("l.idProduit = :product")
            ->setParameter("product", $product)
            ->andWhere("r.dateValidation = :date")
            ->setParameter("date", $date);

        if ($store != null) {
            $sumOfQuantityOfProductQuery->andWhere("r.idMagasin = :store")
                ->setParameter("store", $store);
        }

        return $sumOfQuantityOfProductQuery->getQuery()->getSingleScalarResult();
    }
}
