<?php

include APPLICATION_PATH . '/Models/Stock.php';

class StockDAO
{
    public $entityManager;
    private $modelName = "\Application\Models\Stock";

    function __construct()
    {
        $registry = Zend_Registry::getInstance();
        $this->entityManager = $registry->entitymanager;
    }

    /**
     * 
     * @return \Application\Models\Stock
     */
    public function create()
    {
        return new \Application\Models\Stock();
    }

    /**
     * 
     * @param \Application\Models\Stock
     */
    public function save($stock)
    {
        $this->entityManager->persist($stock);
        $this->entityManager->flush();
    }

    /**
     * 
     * @param \Application\Models\Stock
     */
    public function update($stock)
    {
        $this->entityManager->merge($stock);
        $this->entityManager->flush();
    }

    /**
     * 
     * @param int $id
     * @return \Application\Models\Stock
     */
    public function getStockById($id)
    {
        return $this->entityManager->getRepository($this->modelName)->find($id);
    }

    /**
     * 
     * @return \Application\Models\Stock[]
     */
    public function stockList()
    {
        return $this->entityManager->getRepository($this->modelName)->findBy(array("statut" => 1));
    }

    /**
     * 
     * @param \Application\Models\Magasin $stock
     * @return \Application\Models\Stock[]
     */
    public function getStockByStore($store)
    {
        return $this->entityManager->getRepository($this->modelName)->findBy(array("statut" => 1, "idMagasin" => $store));
    }

    /**
     * 
     * @param String $batchNumber
     * @return \Application\Models\Stock[]
     */
    public function getStockByBatchNumber($batchNumber)
    {
        return $this->entityManager->getRepository($this->modelName)->findBy(array("statut" => 1, "numLot" => $batchNumber));
    }

    /**
     * 
     * @param \Application\Models\Produit $product
     * @param String $batchNumber
     * @return \Application\Models\Stock[]
     */
    public function getStockByProductAndBatchNumber($product, $batchNumber)
    {
        return $this->entityManager->getRepository($this->modelName)->findBy(array("statut" => 1, "idProduit" => $product, "numLot" => $batchNumber));
    }

    /**
     * 
     * @param \Application\Models\Produit $product
     * @param String $batchNumber
     * @param \Application\Models\Magasin $store
     * @return \Application\Models\Stock
     */
    public function getStockLineByStoreProductAndBatchNumber($product, $batchNumber, $store)
    {
        return $this->entityManager->getRepository($this->modelName)->findOneBy(array("idProduit" => $product, "idMagasin" => $store, "numLot" => $batchNumber, "statut" => 1));
    }

    /**
     * 
     * @param \Application\Models\Magasin $store
     * @param String $batchNumber
     * @return \Application\Models\Stock
     */
    public function getStockByStoreAndBatchNumber($store, $batchNumber)
    {
        return $this->entityManager->getRepository($this->modelName)->findOneBy(array("idMagasin" => $store, "numLot" => $batchNumber, "statut" => 1));
    }

    public function getAvailableStock($product = null, \DateTime $expirationDate = null)
    {
        if ($expirationDate == null) {
            $expirationDate = new \DateTime();
        }

        $stockList = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
            ->where("s.quantite > 0")
            ->andWhere("s.datePeremption >= :expirationDate")
            ->setParameter("expirationDate", $expirationDate);

        if ($product) {
            $stockList->andWhere("s.idProduit = :product")
                ->setParameter("product", $product);
        }

        return $stockList->getQuery()->getResult();
    }

    /**
     * 
     * @param \Application\Models\Produit $product
     * @param DateTime $expirationDate
     * @param \Application\Models\Magasin $store
     * @return \Application\Models\Stock[]
     */
    public function getAvailableStockByStore($product = null, \DateTime $expirationDate = null, $store = null)
    {
        $stockList = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
            ->where("s.quantite > 0")
            ->andWhere("s.statut = 1");

        if ($expirationDate) {
            $stockList->andWhere("s.datePeremption = :expirationDate")
                ->setParameter("expirationDate", $expirationDate);
        }

        if ($product) {
            $stockList->andWhere("s.idProduit = :product")
                ->setParameter("product", $product);
        }

        if ($store) {
            $stockList->andWhere("s.idMagasin = :store")
                ->setParameter("store", $store);
        }

        return $stockList->getQuery()->getResult();
    }

    public function getCumulativeStockByProviderBatchNumber($providerBatchNumber)
    {
        return $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
            ->select("SUM(s.quantite) AS quantite")
            ->addSelect("p.designation")
            ->addSelect("AVG(s.prixVente) AS prixVente")
            ->addSelect("s.numLotFournisseur")
            ->where("s.quantite > 0")
            ->andWhere("s.numLotFournisseur = :providerBatchNumber")
            ->setParameter("providerBatchNumber", $providerBatchNumber)
            ->join("s.idProduit", "p")
            ->groupBy("s.numLotFournisseur")
            ->getQuery()
            ->getScalarResult();
    }

    public function getCumulativeStockByBatchNumber($batchNumber)
    {
        return $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
            ->select("SUM(s.quantite) AS quantite")
            ->addSelect("p.designation")
            ->addSelect("AVG(s.prixVente) AS prixVente")
            ->addSelect("s.numLot as numLotFournisseur")
            ->where("s.quantite > 0")
            ->andWhere("s.numLot = :batchNumber")
            ->setParameter("batchNumber", $batchNumber)
            ->join("s.idProduit", "p")
            ->groupBy("s.numLot")
            ->getQuery()
            ->getScalarResult();
    }

    public function getCumulativeStockWithAllPriceByBatchNumber($batchNumber)
    {
        return $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
            ->select("SUM(s.quantite) AS quantite")
            ->addSelect("s.idStock AS idSock")
            ->addSelect("p.designation")
            ->addSelect("p.stockable")
            ->addSelect("s.prixVente AS prixVente")
            ->addSelect("s.prixVente2 AS prixVente2")
            ->addSelect("s.prixVente3 AS prixVente3")
            ->addSelect("s.numLot as numLotFournisseur")
            ->addSelect("p.idProduit AS idProduit")
            ->addSelect("c.idCategorie AS idCategorie")
            ->where("s.numLot = :batchNumber")
            ->setParameter("batchNumber", $batchNumber)
            ->join("s.idProduit", "p")
            ->join("p.idCategorieProduit", "c")
            ->groupBy("s.numLot")
            ->getQuery()
            ->getScalarResult();
    }

    public function getNotFinishedStockByProviderBatchNumber($providerBatchNumber)
    {
        return $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
            ->where("s.quantite > 0")
            ->andWhere("s.numLotFournisseur = :providerBatchNumber")
            ->setParameter("providerBatchNumber", $providerBatchNumber)
            ->getQuery()
            ->getResult();
    }

    public function getNotFinishedStockByBatchNumber($batchNumber)
    {
        return $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
            ->where("s.quantite > 0")
            ->andWhere("s.numLot = :batchNumber")
            ->setParameter("batchNumber", $batchNumber)
            ->getQuery()
            ->getResult();
    }

    /**
     * 
     * @param \Application\Models\Produit $product
     * @param \Application\Models\Magasin $store
     * @return []
     */
    public function getCumulativeStockByProductAndStore($product, $store = null)
    {
        $cumulativeStock = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
            ->select("SUM(s.quantite) AS quantite")
            ->addSelect("p.designation")
            ->addSelect("AVG(s.prixVente) AS prixVente")
            ->where("s.quantite > 0")
            ->andWhere("s.idProduit = :product")
            ->setParameter("product", $product);
        if ($store != null) {
            $cumulativeStock->andWhere("s.idMagasin = :store")
                ->setParameter("store", $store);
        }

        return $cumulativeStock->join("s.idProduit", "p")
            ->andWhere("p.statut = 1")
            ->groupBy("s.idProduit")
            ->getQuery()
            ->getScalarResult();
    }

    public function getAvailableStockByDesignationAndStore($designation, $store = null)
    {
        $stockList = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
            ->where("s.quantite > 0")
            ->andWhere("s.statut = 1")
            ->join("s.idProduit", "p")
            ->andWhere("p.statut = 1")
            ->andWhere("s.designation like :designation or p.referenceProduit like :reference")
            ->setParameter("designation", "%" . $designation . "%")
            ->setParameter("reference", "%" . $designation . "%");
        if ($store) {
            $stockList->andWhere("s.idMagasin = :store")
                ->setParameter("store", $store);
        }

        return $stockList->orderBy("p.designation", "ASC")->getQuery()->getResult();
    }

    public function getAvailableStockByDesignationCategoryAndStore($designation, $category = null, $store = null)
    {
        $stockList = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
            ->where("s.quantite > 0 ")
            ->andWhere("s.statut = 1")
            ->join("s.idProduit", "p")
            ->andWhere("p.statut = 1")
            ->andWhere("s.designation like :designation")
            ->setParameter("designation", "%" . $designation . "%");

        if ($category) {
            $stockList->andWhere("p.idCategorieProduit = :category")
                ->setParameter("category", $category);
        }

        if ($store) {
            $stockList->andWhere("s.idMagasin = :store")
                ->setParameter("store", $store);
        }

        return $stockList->orderBy("p.designation", "ASC")->getQuery()->getResult();
    }

    public function getStockByDesignation($designation)
    {
        return $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
            ->where("s.statut = 1")
            ->join("s.idProduit", "p")
            ->andWhere("p.statut = 1")
            ->andWhere("p.designation like :designation")
            ->setParameter("designation", "%" . $designation . "%")
            ->orderBy("p.designation", "ASC")
            ->getQuery()
            ->getResult();
    }

    public function getNotFinishedStockByDesignation($designation)
    {
        return $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
            ->where("s.quantite > 0 ")
            ->andWhere("s.statut = 1")
            ->join("s.idProduit", "p")
            ->andWhere("p.statut = 1")
            ->andWhere("p.designation like :designation")
            ->setParameter("designation", "%" . $designation . "%")
            ->getQuery()
            ->getResult();
    }

    public function getNotFinishedStockByDesignationAndStore($designation, $store)
    {
        return $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
            ->where("s.quantite > 0 ")
            ->andWhere("s.statut = 1")
            ->join("s.idProduit", "p")
            ->andWhere("p.statut = 1")
            ->andWhere("p.designation like :designation")
            ->setParameter("designation", "%" . $designation . "%")
            ->andWhere("s.idMagasin = :store")
            ->setParameter("store", $store)
            ->getQuery()
            ->getResult();
    }

    public function getStockHavingReachedTheThreshold($category = null)
    {
        $stockList = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
            ->join("s.idProduit", "p")
            ->where("s.statut = 1")
            ->andWhere("p.statut = 1")
            ->andWhere("s.quantite > 0")
            ->andWhere("s.quantite <= p.quantiteAlerte");

        if ($category) {
            $stockList->andWhere("p.idCategorieProduit = :category")
                ->setParameter("category", $category);
        }

        return $stockList->getQuery()->getResult();
    }

    public function getStockExpired($category = null)
    {
        $stockList = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
            ->join("s.idProduit", "p")
            ->where("s.statut = 1")
            ->andWhere("s.datePeremption <= '" . date("Y-m-d") . "'")
            ->andWhere("s.quantite > 0")
            ->andWhere("p.statut = 1")
            ->orderBy("s.idMagasin", "ASC");

        if ($category) {
            $stockList->andWhere("p.idCategorieProduit = :category")
                ->setParameter("category", $category);
        }

        return $stockList->getQuery()->getResult();
    }

    public function getStockExpiredByCategoryAndStore($category = null, $store = null)
    {
        $stockList = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
            ->join("s.idProduit", "p")
            ->where("s.statut = 1")
            ->andWhere("s.datePeremption <= '" . date("Y-m-d") . "'")
            ->andWhere("s.quantite > 0")
            ->andWhere("p.statut = 1");

        if ($category) {
            $stockList->andWhere("s.idCategorieProduit = :category")
                ->setParameter("category", $category);
        }

        if ($store) {
            $stockList->andWhere("s.idMagasin", ":store")
                ->setParameter("store", $store);
        }

        return $stockList->getQuery()->getResult();
    }


    public function getProductInStockWithPrice($store = null)
    {
        $productsInStockQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
            ->select("DISTINCT p.idProduit AS idProduit")
            ->addSelect("s.prixVente AS prixVente")
            ->join("s.idProduit", "p")
            ->where("s.statut = 1")
            ->andWhere("p.statut = 1")
            ->andWhere("s.quantite > 0")
            ->orderBy("p.designation", "ASC");

        if ($store != null) {
            $productsInStockQuery->andWhere("s.idMagasin = :store")
                ->setParameter("store", $store);
        }

        return $productsInStockQuery->getQuery()
            ->getScalarResult();
    }

    public function getDistinctProductInStockByCategory($category, $store = null)
    {
        $productsInStockQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
            ->select("p.idProduit AS idProduit")
            ->addSelect("AVG(s.prixVente) AS prixVente")
            ->join("s.idProduit", "p")
            ->where("s.statut = 1")
            ->andWhere("p.statut = 1")
            ->andWhere("s.quantite > 0")
            ->andWhere("p.idCategorieProduit = :category")
            ->setParameter("category", $category);

        if ($store != null) {
            $productsInStockQuery->andWhere("s.idMagasin = :store")
                ->setParameter("store", $store);
        }

        return $productsInStockQuery->orderBy("p.designation", "ASC")
            ->groupBy("p.idProduit")
            ->getQuery()
            ->getResult();
    }


    /**
     * 
     * @param \Application\Models\Magasin $store
     * @param int $perishable
     * @param \Application\Models\CategorieProduit $productCategory
     * @param \Application\Models\Produit $product
     * @return int[]
     */
    public function getAllProductsInStock($store = null, $perishable = null, $productCategory = null, $product = null)
    {
        $productsInStock = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
            ->select('DISTINCT p.idProduit AS idProduit')
            ->join("s.idProduit", "p")
            ->where("s.statut = 1")
            ->andWhere("s.quantite > 0")
            ->andWhere("p.statut = 1");

        if ($store !== null) {
            $productsInStock->andWhere("s.idMagasin = :store")
                ->setParameter("store", $store);
        }

        if ($perishable !== null) {
            $productsInStock->andWhere("p.perissable = :perishable")
                ->setParameter("perishable", $perishable);
        }

        if ($productCategory != null) {
            $productsInStock->andWhere("p.idCategorieProduit = :productCategory")
                ->setParameter("productCategory", $productCategory);
        }

        if ($product != null) {
            $productsInStock->andWhere("p.idProduit = :productId")
                ->setParameter("productId", $product->getIdProduit());
        }

        return $productsInStock->getQuery()
            ->getScalarResult();
    }

    /**
     * 
     * @param \Application\Models\Magasin $store
     * @param int $perishable
     * @param \Application\Models\CategorieProduit $productCategory
     * @param \Application\Models\Produit $product
     * @return int[]
     */
    public function getAllProductsEverInStock($store = null, $perishable = null, $productCategory = null, $product = null)
    {
        $productsInStock = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
            ->select('DISTINCT p.idProduit AS idProduit')
            ->join("s.idProduit", "p")
            ->where("s.statut = 1")
            ->andWhere("p.statut = 1");

        if ($store !== null) {
            $productsInStock->andWhere("s.idMagasin = :store")
                ->setParameter("store", $store);
        }

        if ($perishable !== null) {
            $productsInStock->andWhere("p.perissable = :perishable")
                ->setParameter("perishable", $perishable);
        }

        if ($productCategory != null) {
            $productsInStock->andWhere("p.idCategorieProduit = :productCategory")
                ->setParameter("productCategory", $productCategory);
        }

        if ($product != null) {
            $productsInStock->andWhere("p.idProduit = :productId")
                ->setParameter("productId", $product->getIdProduit());
        }

        return $productsInStock->getQuery()
            ->getScalarResult();
    }

    public function getDistinctProductInStockByStoreAndDate($store, $date)
    {
        return $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
            ->select('DISTINCT p.idProduit AS productId')
            ->join("s.idProduit", "p")
            ->where("s.statut = 1")
            ->andWhere("p.statut = 1")
            ->andWhere("s.dateEntree <= :entryDate")
            ->setParameter("entryDate", $date)
            ->andWhere("s.idMagasin = :store")
            ->setParameter("store", $store)
            ->getQuery()
            ->getScalarResult();
    }

    public function getStockExpiredOnPeriod()
    {
        $endDate = new \DateTime();
        $endDate->modify(" + " . NOMBRE_JOUR_POUR_ALERTE_PEREMPTION . " days");
        $currentDate = new \DateTime();
        return $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
            ->where("s.quantite > 0")
            ->andWhere("s.datePeremption > :currentDate")
            ->andWhere("s.datePeremption <= :endDate")
            ->andWhere("s.quantite > 0")
            ->andWhere("s.statut = 1")
            ->setParameter("currentDate", $currentDate)
            ->setParameter("endDate", $endDate)
            ->orderBy("s.idMagasin", "ASC")
            ->getQuery()
            ->getResult();
    }

    public function getStockWithMinimalPrice($category = null, $store = null)
    {
        $stockList = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
            ->addSelect('sum(s.quantite) as quantite')
            ->addSelect('sum(s.quantiteEntree) as quantiteEntree')
            ->addSelect('MIN(s.prixVente) AS prixVente')
            ->innerJoin('s.idProduit', 'p')
            ->innerJoin('p.idCategorieProduit', 'c')
            ->where("s.statut = 1")
            ->andWhere("s.idProduit = p.idProduit")
            ->andWhere("p.statut = 1")
            ->andWhere("p.idCategorieProduit = c.idCategorie");

        if ($category) {
            $stockList->andWhere('p.idCategorieProduit = :category')
                ->setParameter('category', $category);
        }

        if ($store) {
            $stockList->andWhere('s.idMagasin = :store')
                ->setParameter('store', $store);
        }

        return $stockList->groupBy('p.idProduit')
            ->getQuery()
            ->getResult();
    }

    /**
     * 
     * @param \Application\Models\Produit $product
     * @param \Application\Models\Magasin $store
     * @return \Application\Models\Stock[]
     */
    public function getAllStockByProductAndStore($product = null, $store = null)
    {
        $stockList = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
            ->innerJoin('s.idProduit', 'p')
            ->where("s.statut = 1")
            ->andWhere("p.statut = 1")
            ->andWhere("s.idProduit = p.idProduit")
            ->andWhere("s.quantite > 0");

        if ($product) {
            $stockList->andWhere(' s.idProduit = :product')
                ->setParameter('product', $product);
        }

        if ($store) {
            $stockList->andWhere('s.idMagasin = :store')
                ->setParameter('store', $store);
        }

        return $stockList->orderBy('s.quantite', 'DESC')
            ->getQuery()
            ->getResult();
    }

    public function getAllAvailableStockByProductAndStore($product = null, $store = null)
    {
        $stockList = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
            ->innerJoin('s.idProduit', 'p')
            ->where("s.statut = 1")
            ->andWhere("p.statut = 1")
            ->andWhere("s.quantite > 0")
            ->andWhere("s.idProduit = p.idProduit");

        if ($product) {
            $stockList->andWhere(' s.idProduit = :product')
                ->setParameter('product', $product);
        }

        if ($store) {
            $stockList->andWhere('s.idMagasin = :store')
                ->setParameter('store', $store);
        }

        return $stockList->orderBy('s.quantite', 'DESC')
            ->getQuery()
            ->getResult();
    }

    public function getStockCharacterized($criteria, $values, $store = null)
    {
        $index = 0;
        $resultsArray = [];
        foreach ($criteria as $singleCriteria) {
            $stockList = $this->entityManager->getRepository("\Application\Models\CaracteristiqueStock")->createQueryBuilder("c")
                ->select("s.idStock as idStock")
                ->join("c.idStock", "s")
                ->where("s.statut = 1")
                ->andWhere("s.quantite > 0")
                ->andWhere("c.idCritereStock = :criteria")
                ->setParameter("criteria", $singleCriteria)
                ->andWhere("c.valeurOuverte like :value")
                ->setParameter("value", "%" . $values[$index] . "%");

            if ($store) {
                $stockList->andWhere("s.idMagasin = :store")
                    ->setParameter("store", $store);
            }

            $currentResults = [];
            foreach ($stockList->getQuery()->getResult() as $stockEntry) {
                $currentResults[] = $stockEntry["idStock"];
            }
            $resultsArray[] = $currentResults;
            $index++;
        }

        if (count($resultsArray) >= 2) {
            $firstEntry = $resultsArray[0];
            $secondEntry = $resultsArray[1];
            $interResult = array_intersect($firstEntry, $secondEntry);
            if (count($resultsArray) > 2) {
                for ($cpt = 2; $cpt < count($resultsArray); $cpt++) {
                    $interResult = array_intersect($interResult, $resultsArray[$cpt]);
                }
            }
            return $interResult;
        } else {
            return $resultsArray[0];
        }
    }

    public function getAllCategoriesAvailableInStockByStore($store = null)
    {
        $stockList = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
            ->select("c.idCategorie AS idCategorie")
            ->where("s.quantite > 0 ")
            ->andWhere("s.statut = 1")
            ->join("s.idProduit", "p")
            ->join("p.idCategorieProduit", "c")
            ->andWhere("p.statut = 1");

        if ($store) {
            $stockList->andWhere("s.idMagasin = :store")
                ->setParameter("store", $store);
        }

        return $stockList->groupBy("c.idCategorie")->orderBy("c.nom", "ASC")->getQuery()->getResult();
    }

    /**
     * 
     * @param \Application\Models\Magasin $store
     * @param DateTime $fromExpirationDate
     * @param DateTime $toExpirationDate
     * @return \Application\Models\Stock[]
     */
    public function getAvailableStockByStoreAndExpirationDateInterval($store = null, $fromExpirationDate = null, $toExpirationDate = null)
    {
        $stockList = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
            ->join("s.idProduit", "p")
            ->where("s.quantite > 0")
            ->andWhere("s.statut = 1")
            ->andWhere("p.statut = 1");

        if ($store != null) {
            $stockList->andWhere("s.idMagasin = :store")
                ->setParameter("store", $store);
        }

        if ($fromExpirationDate != null) {
            $stockList->andWhere("s.datePeremption >= :fromExpirationDate")
                ->setParameter("fromExpirationDate", $fromExpirationDate);
        }

        if ($toExpirationDate != null) {
            $stockList->andWhere("s.datePeremption <= :toExpirationDate")
                ->setParameter("toExpirationDate", $toExpirationDate);
        }

        return $stockList->orderBy("s.idMagasin", "ASC")->getQuery()->getResult();
    }

    /**
     * 
     * @param \Application\Models\Produit $product
     */
    public function deleteAllAvailableStockOfProduct($product)
    {
        $queryBuilder = $this->entityManager->createQueryBuilder();
        $updateQuery = $queryBuilder->update($this->modelName, "s")
            ->set("s.statut", ":status")
            ->where("s.idProduit = :product")
            ->setParameter("status", 0)
            ->setParameter("product", $product)
            ->getQuery();
        $updateQuery->execute();
    }

    /**
     * 
     * @param \Application\Models\Produit $product
     * @param \Application\Models\Magasin $store
     * @return \Application\Models\Stock $stock
     */
    public function getMustRecentStockOfProducAtStore($product, $store)
    {
        return $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
            ->where("s.statut = 1")
            ->andWhere("s.quantite > 0")
            ->andWhere("s.idProduit = :product")
            ->andWhere("s.idMagasin = :store")
            ->setParameter("product", $product)
            ->setParameter("store", $store)
            ->orderBy("s.dateEntree", "DESC")
            ->setMaxResults(1)
            ->getQuery()
            ->getOneOrNullResult();
    }

    /**
     * 
     * @param \Application\Models\Produit $product
     * @param DateTime $targetDate
     * @param \Application\Models\Magasin $store
     */
    public function getProductQuantityAtDate($product, $targetDate, $store = null)
    {
        $eveningOfTargetDate = new DateTime("{$targetDate->format(DATE_FORMAT)} 23:59:59");
        $entryQuantityQuery = $this->entityManager->getRepository("Application\Models\LigneReception")->createQueryBuilder("l")
            ->select("SUM(l.quantite)")
            ->join("l.idReception", "r")
            ->where("r.statut = 1")
            ->andWhere("r.valider = 1")
            ->andWhere("l.idProduit = :product")
            ->setParameter("product", $product)
            ->andWhere("r.dateValidation <= :targetDate")
            ->setParameter("targetDate", $eveningOfTargetDate);
        $exitQuantityQuery = $this->entityManager->getRepository("Application\Models\LigneSortie")->createQueryBuilder("l")
            ->select("SUM(l.quantite)")
            ->join("l.idSortie", "s")
            ->join("l.idStock", "st")
            ->where("s.statut = 1")
            ->andWhere("st.idProduit = :product")
            ->setParameter("product", $product)
            ->andWhere("s.valider = 1")
            ->andWhere("s.dateValidation <= :targetDate")
            ->setParameter("targetDate", $eveningOfTargetDate);
        if ($store != null) {
            $entryQuantityQuery->andWhere("r.idMagasin = :store")
                ->setParameter("store", $store);
            $exitQuantityQuery->andWhere("s.idMagasin = :store")
                ->setParameter("store", $store);
        }
        return $entryQuantityQuery->getQuery()->getSingleScalarResult() - $exitQuantityQuery->getQuery()->getSingleScalarResult();
    }

    /**
     * 
     * @param \Application\Models\Produit $product
     * @param DateTime $startDate
     * @param DateTime $endDate
     * @param \Application\Models\Magasin $store
     */
    public function getProductQuantityEnteredOverPeriod($product, $startDate, $endDate, $store = null)
    {
        $eveningOfEndDate = new DateTime("{$endDate->format(DATE_FORMAT)} 23:59:59");
        $entryQuantityQuery = $this->entityManager->getRepository("Application\Models\LigneReception")->createQueryBuilder("l")
            ->select("SUM(l.quantite)")
            ->join("l.idReception", "r")
            ->where("r.statut = 1")
            ->andWhere("r.valider = 1")
            ->andWhere("l.idProduit = :product")
            ->setParameter("product", $product)
            ->andWhere("r.dateValidation >= :startDate")
            ->andWhere("r.dateValidation <= :endDate")
            ->setParameter("startDate", $startDate)
            ->setParameter("endDate", $eveningOfEndDate);
        if ($store != null) {
            $entryQuantityQuery->andWhere("r.idMagasin = :store")
                ->setParameter("store", $store);
        }
        return (int)$entryQuantityQuery->getQuery()->getSingleScalarResult();
    }

    /**
     * 
     * @param \Application\Models\Produit $product
     * @param DateTime $startDate
     * @param DateTime $endDate
     * @param \Application\Models\Magasin $store
     */
    public function getProductQuantityIssuedOverPeriod($product, $startDate, $endDate, $store = null)
    {
        $eveningOfEndDate = new DateTime("{$endDate->format(DATE_FORMAT)} 23:59:59");
        $exitQuantityQuery = $this->entityManager->getRepository("Application\Models\LigneSortie")->createQueryBuilder("l")
            ->select("SUM(l.quantite)")
            ->join("l.idSortie", "s")
            ->join("l.idStock", "st")
            ->where("s.statut = 1")
            ->andWhere("st.idProduit = :product")
            ->setParameter("product", $product)
            ->andWhere("s.valider = 1")
            ->andWhere("s.dateValidation >= :startDate")
            ->andWhere("s.dateValidation <= :endDate")
            ->setParameter("startDate", $startDate)
            ->setParameter("endDate", $eveningOfEndDate);
        if ($store != null) {
            $exitQuantityQuery->andWhere("s.idMagasin = :store")
                ->setParameter("store", $store);
        }
        return (int)$exitQuantityQuery->getQuery()->getSingleScalarResult();
    }

    protected static function sortProductMovementDetailsArray($productMovementDetail1, $productMovementDetail2)
    {
        if (strtotime($productMovementDetail1["date"]) == strtotime($productMovementDetail2["date"]))
            return 0;
        else if (strtotime($productMovementDetail1["date"]) > strtotime($productMovementDetail2["date"]))
            return 1;
        else
            return -1;
    }

    /**
     * 
     * @param \Application\Models\Produit $product
     * @param DateTime $startDate
     * @param DateTime $endDate
     * @param mixed $translator
     * @param \Application\Models\Magasin $store
     */
    public function getProductMovementResumeOnPeriod($product, $startDate, $endDate, $translator, $store = null)
    {
        $entryLinesQuery =  $this->entityManager->getRepository("Application\Models\LigneReception")->createQueryBuilder("l")
            ->join("l.idReception", "r")
            ->where("r.statut = 1")
            ->andWhere("r.valider = 1")
            ->andWhere("l.idProduit = :product")
            ->setParameter("product", $product)
            ->andWhere("r.dateValidation >= :startDate")
            ->andWhere("r.dateValidation <= :endDate")
            ->setParameter("startDate", $startDate)
            ->setParameter("endDate", $endDate);
        $exitLinesQuery = $this->entityManager->getRepository("Application\Models\LigneSortie")->createQueryBuilder("l")
            ->join("l.idSortie", "s")
            ->join("l.idStock", "st")
            ->where("s.statut = 1")
            ->andWhere("st.idProduit = :product")
            ->setParameter("product", $product)
            ->andWhere("s.dateValidation >= :startDate")
            ->andWhere("s.dateValidation <= :endDate")
            ->setParameter("startDate", $startDate)
            ->setParameter("endDate", $endDate);
        if ($store != null) {
            $entryLinesQuery->andWhere("r.idMagasin = :store")
                ->setParameter("store", $store);
            $exitLinesQuery->andWhere("s.idMagasin = :store")
                ->setParameter("store", $store);
        }

        $entryLines = $entryLinesQuery->getQuery()->getResult();
        $exitLines = $exitLinesQuery->getQuery()->getResult();
        $definitiveResume = [];
        foreach ($entryLines as $entryLine) {
            $definitiveResume[] = [
                "date" => $entryLine->getIdReception()->getDateValidation()->format(DATE_FORMAT),
                "sign" => "+",
                "quantity" => $entryLine->getQuantite(),
                "reason" => $entryLine->getIdReception()->getObservation(),
                "store" => $entryLine->getIdReception()->getIdMagasin()->getNomMagasin(),
            ];
        }
        foreach ($exitLines as $exitLine) {
            $definitiveResume[] = [
                "date" => $exitLine->getIdSortie()->getDateValidation()->format(DATE_FORMAT),
                "sign" => "-",
                "quantity" => $exitLine->getQuantite(),
                "reason" => $exitLine->getIdSortie()->getIdFacture() != null ? "{$translator->_("view.productPath.saleCompleted")} {$exitLine->getIdSortie()->getIdFacture()->getIdFacture()}" : $exitLine->getIdSortie()->getMotif(),
                "store" => $exitLine->getIdSortie()->getIdMagasin()->getNomMagasin(),
            ];
        }
        usort($definitiveResume, "StockDAO::sortProductMovementDetailsArray");
        return $definitiveResume;
    }

    /**
     * 
     * @param \Application\Models\Magasin $store
     * @param \Application\Models\CategorieProduit $category
     * @return mixed[]
     */
    public function getProductsOutOfStock($store, $category = null)
    {
        $outOfStockProductsQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("s")
            ->select("p.idProduit as product_id")
            ->where("s.statut = 1")
            ->join("s.idProduit", "p")
            ->andWhere("p.statut = 1")
            ->andWhere("s.idMagasin = :store")
            ->setParameter("store", $store);
        if ($category != null) {
            $outOfStockProductsQuery->andWhere("p.idCategorieProduit = :category")
                ->setParameter("category", $category);
        }
        return $outOfStockProductsQuery->groupBy("p.idProduit")
            ->having("SUM(s.quantite) = 0")
            ->getQuery()
            ->getScalarResult();
    }
}
