<?php

include APPLICATION_PATH . '/Models/Utilisateur.php';

class UserDAO
{
    public $entityManager;
    private $modelName = "\Application\Models\Utilisateur";

    function __construct()
    {
        $registry = Zend_Registry::getInstance();
        $this->entityManager = $registry->entitymanager;
    }

    /**
     * 
     * @return \Application\Models\Utilisateur
     */
    public function create()
    {
        return new \Application\Models\Utilisateur();
    }

    /**
     * 
     * @param \Application\Models\Utilisateur $user
     */
    public function save($user)
    {
        $this->entityManager->persist($user);
        $this->entityManager->flush();
    }

    /**
     * 
     * @param \Application\Models\Utilisateur $user
     */
    public function update($user)
    {
        $this->entityManager->merge($user);
        $this->entityManager->flush();
    }

    /**
     * 
     * @param int $id
     * @return \Application\Models\Utilisateur
     */
    public function getUserById($id)
    {
        return $this->entityManager->getRepository($this->modelName)->findOneBy(array('idUtilisateur' => $id));
    }

    /**
     * 
     * @param string $login
     * @param string $password
     * @return \Application\Models\Utilisateur
     */
    public function getUserByLoginPassword($login, $password)
    {
        return $this->entityManager->getRepository($this->modelName)->findOneBy(array('login' => $login, 'password' => $password, "statut" => 1));
    }

    /**
     * 
     * @param \Application\Models\PointVente $pointOfSale
     * @return \Application\Models\Utilisateur[]
     */
    public function listUsers($pointOfSale = null)
    {
        $protectedLogin = PROTECTED_APP_LOGIN;
        $usersQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("u")
            ->where("u.statut = 1")
            ->andWhere("u.login <> :protectedLogin")
            ->setParameter("protectedLogin", $protectedLogin);

        if ($pointOfSale != null) {
            $usersQuery->andWhere("u.idPointVente = :pointOfSale")
                ->setParameter("pointOfSale", $pointOfSale);
        }

        return $usersQuery->getQuery()
            ->getResult();
    }

    /**
     * 
     * @param \Application\Models\PointVente $pointOfSale
     * @return \Application\Models\Utilisateur[]
     */
    public function listEmployees($pointOfSale = null)
    {
        $protectedLogin = PROTECTED_APP_LOGIN;
        $employeesQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("u")
            ->where("u.statut = 1")
            ->andWhere("u.employe = 1")
            ->andWhere("u.login <> :protectedLogin")
            ->setParameter("protectedLogin", $protectedLogin);

        if ($pointOfSale != null) {
            $employeesQuery->andWhere("u.idPointVente = :pointOfSale")
                ->setParameter("pointOfSale", $pointOfSale);
        }

        return $employeesQuery->getQuery()
            ->getResult();
    }

    /**
     * 
     * @param \Application\Models\PointVente $pointOfSale
     * @return \Application\Models\Utilisateur[]
     */
    public function listCashiers($pointOfSale = null)
    {
        $protectedLogin = PROTECTED_APP_LOGIN;
        $cashiersQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("u")
            ->where("u.statut = 1")
            ->andWhere("u.caissier = 1")
            ->andWhere("u.login <> :protectedLogin")
            ->setParameter("protectedLogin", $protectedLogin);

        if ($pointOfSale != null) {
            $cashiersQuery->andWhere("u.idPointVente = :pointOfSale")
                ->setParameter("pointOfSale", $pointOfSale);
        }

        return $cashiersQuery->getQuery()
            ->getResult();
    }

    /**
     * 
     * @param \Application\Models\PointVente[] $pointsOfSale
     * @return \Application\Models\Utilisateur[]
     */
    public function listCashiersOfPointsOfSale($pointsOfSale)
    {
        $protectedLogin = PROTECTED_APP_LOGIN;
        $cashiersQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("u")
            ->where("u.statut = 1")
            ->andWhere("u.caissier = 1")
            ->andWhere("u.login <> :protectedLogin")
            ->setParameter("protectedLogin", $protectedLogin);
        $cashiersQuery->andWhere($cashiersQuery->expr()->in("u.idPointVente", ":pointsOfSale"))
            ->setParameter("pointsOfSale", $pointsOfSale);
        return $cashiersQuery->getQuery()
            ->getResult();
    }

    /**
     * 
     * @param string $login
     * @return \Application\Models\Utilisateur
     */
    public function getUserByLogin($login)
    {
        return $this->entityManager->getRepository($this->modelName)->findOneBy(array("login" => $login, "statut" => 1));
    }

    /**
     * 
     * @param string $login
     * @param string $password
     * @return \Application\Models\Utilisateur
     */
    public function checkUserCashierPassword($login, $password)
    {
        return $this->entityManager->getRepository($this->modelName)->findOneBy(array("login" => $login, "pwdCaisse" => $password));
    }

    /**
     * 
     * @param string $login
     * @param string $password
     * @return \Application\Models\Utilisateur
     */
    public function checkUserValidationPassword($login, $password)
    {
        return $this->entityManager->getRepository($this->modelName)->findOneBy(array("login" => $login, "password" => $password));
    }

    /**
     * 
     * @return int
     */
    public function getUsersCount()
    {
        $protectedLogin = PROTECTED_APP_LOGIN;
        return $this->entityManager->getRepository($this->modelName)->createQueryBuilder("u")
            ->select("COUNT(u.idUtilisateur)")
            ->where("u.statut = 1")
            ->andWhere("u.login <> :protectedLogin")
            ->setParameter("protectedLogin", $protectedLogin)
            ->getQuery()
            ->getSingleScalarResult();
    }
}
