<?php

include  APPLICATION_PATH . '/Models/IntervalleBonAchat.php';

class VoucherIntervalDAO
{
    public $entityManager;
    private $modelName = "\Application\Models\IntervalleBonAchat";

    function __construct()
    {
        $registry = Zend_Registry::getInstance();
        $this->entityManager = $registry->entitymanager;
    }

    /**
     * 
     * @return \Application\Models\IntervalleBonAchat
     */
    public function create()
    {
        return new \Application\Models\IntervalleBonAchat();
    }

    /**
     * 
     * @param \Application\Models\IntervalleBonAchat $voucherInterval
     */
    public function save($voucherInterval)
    {
        $this->entityManager->persist($voucherInterval);
        $this->entityManager->flush();
    }

    /**
     * 
     * @param \Application\Models\IntervalleBonAchat $voucherInterval
     */
    public function update($voucherInterval)
    {
        $this->entityManager->merge($voucherInterval);
        $this->entityManager->flush();
    }

    /**
     * 
     * @param int $id
     * @return \Application\Models\IntervalleBonAchat
     */
    public function getVoucherIntervalById($id)
    {
        return $this->entityManager->getRepository($this->modelName)->find($id);
    }

    /**
     * 
     * @param int $minimumPoint
     * @param int $maximumPoint
     * @param int $voucherIntervalId
     * @return \Application\Models\IntervalleBonAchat
     */
    public function overlappingCheck($minimumPoint, $maximumPoint, $voucherIntervalId = null)
    {
        $overlappingQuery = $this->entityManager->getRepository($this->modelName)->createQueryBuilder("v")
            ->where("v.statut = 1");
        $overlappingQuery->andWhere(
            $overlappingQuery->expr()->orX(
                $overlappingQuery->expr()->andX(
                    $overlappingQuery->expr()->gte('v.pointMinimum', ':min'),
                    $overlappingQuery->expr()->lte('v.pointMinimum', ':max')
                ),
                $overlappingQuery->expr()->andX(
                    $overlappingQuery->expr()->gte('v.pointMaximum', ':min'),
                    $overlappingQuery->expr()->lte('v.pointMaximum', ':max')
                ),
                $overlappingQuery->expr()->andX(
                    $overlappingQuery->expr()->gte(':min', 'v.pointMinimum'),
                    $overlappingQuery->expr()->lte(':min', 'v.pointMaximum')
                ),
                $overlappingQuery->expr()->andX(
                    $overlappingQuery->expr()->gte(':max', 'v.pointMinimum'),
                    $overlappingQuery->expr()->lte(':max', 'v.pointMaximum')
                )
            )
        );

        $overlappingQuery->setParameter("min", $minimumPoint)
            ->setParameter("max", $maximumPoint);

        if ($voucherIntervalId != null) {
            $overlappingQuery->andWhere("v.idIntervalleBonAchat <> :voucherIntervalId")
                ->setParameter("voucherIntervalId", $voucherIntervalId);
        }

        return $overlappingQuery->setMaxResults(1)
            ->getQuery()
            ->getOneOrNullResult();
    }

    /**
     * 
     * @param int $points
     * @return \Application\Models\IntervalleBonAchat
     */
    public function getVoucherIntervalIncluingPoints($points)
    {
        return $this->entityManager->getRepository($this->modelName)->createQueryBuilder("v")
            ->where("v.statut = 1")
            ->andWhere("v.pointMinimum <= :points")
            ->andWhere("v.pointMaximum >= :points")
            ->setParameter("points", $points)
            ->setMaxResults(1)
            ->getQuery()
            ->getOneOrNullResult();
    }
}
