<?php

namespace Application\Models;

use Doctrine\ORM\Mapping as ORM;

/**
 * Utilisateur
 *
 * @ORM\Table(name="utilisateur")
 * @ORM\Entity
 */
class Utilisateur
{
    /**
     * @var integer
     *
     * @ORM\Column(name="ID_UTILISATEUR", type="bigint", nullable=false)
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="IDENTITY")
     */
    private $idUtilisateur;

    /**
     * @var string
     *
     * @ORM\Column(name="LOGIN", type="string", length=255, nullable=true)
     */
    private $login;

    /**
     * @var string
     *
     * @ORM\Column(name="PASSWORD", type="text", nullable=true)
     */
    private $password;

    /**
     * @var string
     *
     * @ORM\Column(name="PWD_CAISSE", type="text", nullable=true)
     */
    private $pwdCaisse;

    /**
     * @var integer
     *
     * @ORM\Column(name="CAISSIER", type="smallint", nullable=true)
     */
    private $caissier;

    /**
     * @var integer
     *
     * @ORM\Column(name="STATUT", type="smallint", nullable=true)
     */
    private $statut;

    /**
     * @var integer
     *
     * @ORM\Column(name="DROIT_REDUCTION", type="integer", nullable=true)
     */
    private $droitReduction;

    /**
     * @var integer
     *
     * @ORM\Column(name="CAISSIER_PRINCIPAL", type="smallint", nullable=true)
     */
    private $caissierPrincipal;

    /**
     * @var integer
     *
     * @ORM\Column(name="EMPLOYE", type="smallint", nullable=true)
     */
    private $employe;

    /**
     * @var string
     *
     * @ORM\Column(name="PROFILE_IMAGE", type="string", length=255, nullable=true)
     */
    private $profileImage;

    /**
     * @var string
     *
     * @ORM\Column(name="RIB", type="string", length=255, nullable=true)
     */
    private $rib;

    /**
     * @var string
     *
     * @ORM\Column(name="BANK", type="string", length=255, nullable=true)
     */
    private $bank;

    /**
     * @var string
     *
     * @ORM\Column(name="BANK_AGENCY", type="string", length=255, nullable=true)
     */
    private $bankAgency;

    /**
     * @var \DateTime
     *
     * @ORM\Column(name="LAST_LOGIN_DATE", type="datetime", nullable=true)
     */
    private $lastLoginDate;

    /**
     * @var \DateTime
     *
     * @ORM\Column(name="LAST_LOGOUT_DATE", type="datetime", nullable=true)
     */
    private $lastLogoutDate;

    /**
     * @var \DateTime
     *
     * @ORM\Column(name="LAST_ACTION_DATE", type="datetime", nullable=true)
     */
    private $lastActionDate;

    /**
     * @var integer
     *
     * @ORM\Column(name="PASSWORD_IS_RESET", type="smallint", nullable=true)
     */
    private $passwordIsReset;

    /**
     * @var integer
     *
     * @ORM\Column(name="TWO_FACTOR_ENABLED", type="smallint", nullable=true)
     */
    private $twoFactorEnabled;

    /**
     * @var string
     *
     * @ORM\Column(name="TWO_FACTOR_OTP", type="string", length=255, nullable=true)
     */
    private $twoFactorOtp;

    /**
     * @var \Application\Models\Profil
     *
     * @ORM\ManyToOne(targetEntity="Application\Models\Profil")
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="ID_PROFIL", referencedColumnName="ID_PROFIL")
     * })
     */
    private $idProfil;

    /**
     * @var \Application\Models\Personne
     *
     * @ORM\ManyToOne(targetEntity="Application\Models\Personne")
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="ID_PERSONNE", referencedColumnName="ID_PERSONNE")
     * })
     */
    private $idPersonne;

    /**
     * @var \Application\Models\PointVente
     *
     * @ORM\ManyToOne(targetEntity="Application\Models\PointVente")
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="ID_POINT_VENTE", referencedColumnName="ID_POINT_VENTE")
     * })
     */
    private $idPointVente;

    /**
     * @var \Application\Models\ModelePaieMensuel
     *
     * @ORM\ManyToOne(targetEntity="Application\Models\ModelePaieMensuel")
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="ID_MODELE_PAIE", referencedColumnName="id_modele_paie")
     * })
     */
    private $idModelePaie;



    /**
     * Get idUtilisateur
     *
     * @return integer 
     */
    public function getIdUtilisateur()
    {
        return $this->idUtilisateur;
    }

    /**
     * Set login
     *
     * @param string $login
     * @return Utilisateur
     */
    public function setLogin($login)
    {
        $this->login = $login;
    
        return $this;
    }

    /**
     * Get login
     *
     * @return string 
     */
    public function getLogin()
    {
        return $this->login;
    }

    /**
     * Set password
     *
     * @param string $password
     * @return Utilisateur
     */
    public function setPassword($password)
    {
        $this->password = $password;
    
        return $this;
    }

    /**
     * Get password
     *
     * @return string 
     */
    public function getPassword()
    {
        return $this->password;
    }

    /**
     * Set pwdCaisse
     *
     * @param string $pwdCaisse
     * @return Utilisateur
     */
    public function setPwdCaisse($pwdCaisse)
    {
        $this->pwdCaisse = $pwdCaisse;
    
        return $this;
    }

    /**
     * Get pwdCaisse
     *
     * @return string 
     */
    public function getPwdCaisse()
    {
        return $this->pwdCaisse;
    }

    /**
     * Set caissier
     *
     * @param integer $caissier
     * @return Utilisateur
     */
    public function setCaissier($caissier)
    {
        $this->caissier = $caissier;
    
        return $this;
    }

    /**
     * Get caissier
     *
     * @return integer 
     */
    public function getCaissier()
    {
        return $this->caissier;
    }

    /**
     * Set statut
     *
     * @param integer $statut
     * @return Utilisateur
     */
    public function setStatut($statut)
    {
        $this->statut = $statut;
    
        return $this;
    }

    /**
     * Get statut
     *
     * @return integer 
     */
    public function getStatut()
    {
        return $this->statut;
    }

    /**
     * Set droitReduction
     *
     * @param integer $droitReduction
     * @return Utilisateur
     */
    public function setDroitReduction($droitReduction)
    {
        $this->droitReduction = $droitReduction;
    
        return $this;
    }

    /**
     * Get droitReduction
     *
     * @return integer 
     */
    public function getDroitReduction()
    {
        return $this->droitReduction;
    }

    /**
     * Set caissierPrincipal
     *
     * @param integer $caissierPrincipal
     * @return Utilisateur
     */
    public function setCaissierPrincipal($caissierPrincipal)
    {
        $this->caissierPrincipal = $caissierPrincipal;
    
        return $this;
    }

    /**
     * Get caissierPrincipal
     *
     * @return integer 
     */
    public function getCaissierPrincipal()
    {
        return $this->caissierPrincipal;
    }

    /**
     * Set employe
     *
     * @param integer $employe
     * @return Utilisateur
     */
    public function setEmploye($employe)
    {
        $this->employe = $employe;
    
        return $this;
    }

    /**
     * Get employe
     *
     * @return integer 
     */
    public function getEmploye()
    {
        return $this->employe;
    }

    /**
     * Set profileImage
     *
     * @param string $profileImage
     * @return Utilisateur
     */
    public function setProfileImage($profileImage)
    {
        $this->profileImage = $profileImage;
    
        return $this;
    }

    /**
     * Get profileImage
     *
     * @return string 
     */
    public function getProfileImage()
    {
        return $this->profileImage;
    }

    /**
     * Set rib
     *
     * @param string $rib
     * @return Utilisateur
     */
    public function setRib($rib)
    {
        $this->rib = $rib;
    
        return $this;
    }

    /**
     * Get rib
     *
     * @return string 
     */
    public function getRib()
    {
        return $this->rib;
    }

    /**
     * Set bank
     *
     * @param string $bank
     * @return Utilisateur
     */
    public function setBank($bank)
    {
        $this->bank = $bank;
    
        return $this;
    }

    /**
     * Get bank
     *
     * @return string 
     */
    public function getBank()
    {
        return $this->bank;
    }

    /**
     * Set bankAgency
     *
     * @param string $bankAgency
     * @return Utilisateur
     */
    public function setBankAgency($bankAgency)
    {
        $this->bankAgency = $bankAgency;
    
        return $this;
    }

    /**
     * Get bankAgency
     *
     * @return string 
     */
    public function getBankAgency()
    {
        return $this->bankAgency;
    }

    /**
     * Set lastLoginDate
     *
     * @param \DateTime $lastLoginDate
     * @return Utilisateur
     */
    public function setLastLoginDate($lastLoginDate)
    {
        $this->lastLoginDate = $lastLoginDate;
    
        return $this;
    }

    /**
     * Get lastLoginDate
     *
     * @return \DateTime 
     */
    public function getLastLoginDate()
    {
        return $this->lastLoginDate;
    }

    /**
     * Set lastLogoutDate
     *
     * @param \DateTime $lastLogoutDate
     * @return Utilisateur
     */
    public function setLastLogoutDate($lastLogoutDate)
    {
        $this->lastLogoutDate = $lastLogoutDate;
    
        return $this;
    }

    /**
     * Get lastLogoutDate
     *
     * @return \DateTime 
     */
    public function getLastLogoutDate()
    {
        return $this->lastLogoutDate;
    }

    /**
     * Set lastActionDate
     *
     * @param \DateTime $lastActionDate
     * @return Utilisateur
     */
    public function setLastActionDate($lastActionDate)
    {
        $this->lastActionDate = $lastActionDate;
    
        return $this;
    }

    /**
     * Get lastActionDate
     *
     * @return \DateTime 
     */
    public function getLastActionDate()
    {
        return $this->lastActionDate;
    }

    /**
     * Set passwordIsReset
     *
     * @param integer $passwordIsReset
     * @return Utilisateur
     */
    public function setPasswordIsReset($passwordIsReset)
    {
        $this->passwordIsReset = $passwordIsReset;
    
        return $this;
    }

    /**
     * Get passwordIsReset
     *
     * @return integer 
     */
    public function getPasswordIsReset()
    {
        return $this->passwordIsReset;
    }

    /**
     * Set twoFactorEnabled
     *
     * @param integer $twoFactorEnabled
     * @return Utilisateur
     */
    public function setTwoFactorEnabled($twoFactorEnabled)
    {
        $this->twoFactorEnabled = $twoFactorEnabled;
    
        return $this;
    }

    /**
     * Get twoFactorEnabled
     *
     * @return integer 
     */
    public function getTwoFactorEnabled()
    {
        return $this->twoFactorEnabled;
    }

    /**
     * Set twoFactorOtp
     *
     * @param string $twoFactorOtp
     * @return Utilisateur
     */
    public function setTwoFactorOtp($twoFactorOtp)
    {
        $this->twoFactorOtp = $twoFactorOtp;
    
        return $this;
    }

    /**
     * Get twoFactorOtp
     *
     * @return string 
     */
    public function getTwoFactorOtp()
    {
        return $this->twoFactorOtp;
    }

    /**
     * Set idProfil
     *
     * @param \Application\Models\Profil $idProfil
     * @return Utilisateur
     */
    public function setIdProfil(\Application\Models\Profil $idProfil = null)
    {
        $this->idProfil = $idProfil;
    
        return $this;
    }

    /**
     * Get idProfil
     *
     * @return \Application\Models\Profil 
     */
    public function getIdProfil()
    {
        return $this->idProfil;
    }

    /**
     * Set idPersonne
     *
     * @param \Application\Models\Personne $idPersonne
     * @return Utilisateur
     */
    public function setIdPersonne(\Application\Models\Personne $idPersonne = null)
    {
        $this->idPersonne = $idPersonne;
    
        return $this;
    }

    /**
     * Get idPersonne
     *
     * @return \Application\Models\Personne 
     */
    public function getIdPersonne()
    {
        return $this->idPersonne;
    }

    /**
     * Set idPointVente
     *
     * @param \Application\Models\PointVente $idPointVente
     * @return Utilisateur
     */
    public function setIdPointVente(\Application\Models\PointVente $idPointVente = null)
    {
        $this->idPointVente = $idPointVente;
    
        return $this;
    }

    /**
     * Get idPointVente
     *
     * @return \Application\Models\PointVente 
     */
    public function getIdPointVente()
    {
        return $this->idPointVente;
    }

    /**
     * Set idModelePaie
     *
     * @param \Application\Models\ModelePaieMensuel $idModelePaie
     * @return Utilisateur
     */
    public function setIdModelePaie(\Application\Models\ModelePaieMensuel $idModelePaie = null)
    {
        $this->idModelePaie = $idModelePaie;
    
        return $this;
    }

    /**
     * Get idModelePaie
     *
     * @return \Application\Models\ModelePaieMensuel 
     */
    public function getIdModelePaie()
    {
        return $this->idModelePaie;
    }
}