<?php

include APPLICATION_PATH . '/includes/daos.php';

class AppointmentController extends Zend_Controller_Action
{
    private $translator;

    public function init()
    {
        session_start();
        SubscriptionChecker::checkSubscriptionValidity($this);
        if (isset($_SESSION[SESSION_USER_ID]) && $_SESSION[SESSION_USER_ID] != null) {
            $_SESSION[LAST_ACTION_DATE] = new DateTime();
            LogDAO::updateLastActionDateForCurrentUser();
            $this->translator = Translator::init();
        } else {
            if ($this->getRequest()->isPost()) {
                echo ApiResponse::respond(SESSION_EXPIRED_CODE);
                exit();
            } else {
                $this->_helper->redirector('index', 'Index');
            }
        }
    }

    public function activitiesAction()
    {
        AccessRightChecker::checkAccessRight($this, 97);
        $this->view->translator = $this->translator;
    }

    public function datatableactivitiesAction()
    {
        try {
            $title = $_POST["title"] != "" ? $_POST["title"] : null;
            $whereTitle = "";
            if ($title != null) {
                $whereTitle .= " AND a.titre LIKE '%{$title}%'";
            }
            $where = "a.statut = 1{$whereTitle}";
            $activitiesTable = "
                (
                    SELECT a.id_activite as activity_id, a.titre as activity_title, a.description AS activity_description,
                    a.background AS activity_background
                    FROM activite_rendez_vous a
                    WHERE {$where}
                ) AS activitiesTable
            ";
            $primaryKey = 'activity_id';
            $columns = [
                ['db' => 'activity_title', 'dt' => 0],
                ['db' => 'activity_description', 'dt' => 1],
                ['db' => 'activity_background', 'dt' => 2],
                ['db' => 'activity_id', 'dt' => 3],
            ];
            $computeDatatableResults = SSP::customSimple($_POST, $activitiesTable, $primaryKey, $columns);
            $datatableResult = $computeDatatableResults["datatable"];
            $finalResult = $computeDatatableResults["custom"];
            foreach ($datatableResult["data"] as $row) {
                $activityTitle = $row[0];
                $activityDescription = $row[1];
                $background = $row[2];
                $activityId = $row[3];
                $actions = "
                    <button title='{$this->translator->_("shared.edit")}' about='{$activityId}' class='mini ui icon button app-secondary-button app-edit-action'>
                        <i class='edit icon'></i>
                    </button>
                    <button title='{$this->translator->_("shared.delete")}' about='{$activityId}' class='mini ui red icon button app-delete-action'>
                        <i class='trash icon'></i>
                    </button>
                ";
                $tableRow = [
                    $activityTitle,
                    $activityDescription,
                    $background != null ? $background : APP_SECONDARY_BRAND,
                    $actions,
                ];
                $finalResult["data"][] = $tableRow;
            }
            echo json_encode($finalResult);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 duplicate
     * 102 unknown error
     */
    public function createactivityAction()
    {
        try {
            $title = $_POST["title"];
            $description = $_POST["description"];
            $background = $_POST["background"];
            $appointmentActivityDao = new AppointmentActivityDAO();
            if ($appointmentActivityDao->appointmentActivityTitleAlreadyExists($title)) {
                echo ApiResponse::respond(101);
                exit();
            }
            $appointmentActivity = $appointmentActivityDao->create();
            $appointmentActivity->setTitre($title);
            $appointmentActivity->setDescription($description);
            $appointmentActivity->setBackground($background);
            $appointmentActivity->setStatut(1);
            $appointmentActivityDao->save($appointmentActivity);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("appointment.activities.activitySuccessfullyRegistered")} : N°{$appointmentActivity->getIdActivite()}({$appointmentActivity->getTitre()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 duplicate
     * 102 unknown error
     */
    public function updateactivityAction()
    {
        try {
            $id = (int)$_POST["id"];
            $title = $_POST["title"];
            $description = $_POST["description"];
            $background = $_POST["background"];
            $appointmentActivityDao = new AppointmentActivityDAO();
            $appointmentActivity = $appointmentActivityDao->getAppointmentActivityById($id);
            if ($appointmentActivityDao->appointmentActivityTitleAlreadyExists($title, $appointmentActivity)) {
                echo ApiResponse::respond(101);
                exit();
            }
            $appointmentActivity->setTitre($title);
            $appointmentActivity->setDescription($description);
            $appointmentActivity->setBackground($background);
            $appointmentActivityDao->update($appointmentActivity);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("appointment.activities.activityUpdatedSuccessfully")} : N°{$appointmentActivity->getIdActivite()}({$appointmentActivity->getTitre()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function deleteactivityAction()
    {
        try {
            $id = (int)$_POST["id"];
            $appointmentActivityDao = new AppointmentActivityDAO();
            $appointmentActivity = $appointmentActivityDao->getAppointmentActivityById($id);
            if ($appointmentActivity->getStatut() == 0) {
                echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE);
                exit();
            }
            $appointmentActivity->setStatut(0);
            $appointmentActivityDao->update($appointmentActivity);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("appointment.activities.activityDeletedSuccessfully")} : N°{$appointmentActivity->getIdActivite()}({$appointmentActivity->getTitre()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    public function indexAction()
    {
        AccessRightChecker::checkAccessRight($this, 96);
        $userDao = new UserDAO();
        $appointmentActivityDao = new AppointmentActivityDAO();
        $pointOfSaleDao = new PointOfSaleDAO();
        $users = $userDao->listUsers();
        $appointmentActivities = $appointmentActivityDao->getAllAppointmentActivities();
        $currentUser = LogDAO::getCurrentUser();
        $pointOfSales = [];
        if ($currentUser->getIdPointVente() == null || in_array($currentUser->getIdProfil()->getCode(), IDENTIFIED_PROFILES_CODES)) {
            $pointOfSales = $pointOfSaleDao->listPointOfSales();
        } else {
            $pointOfSales[] = $currentUser->getIdPointVente();
        }
        $this->view->users = $users;
        $this->view->appointmentActivities = $appointmentActivities;
        $this->view->pointOfSales = $pointOfSales;
        $this->view->translator = $this->translator;
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function loadappointmentsAction()
    {
        try {
            $appointmentDao = new AppointmentDAO();
            $appointmentActivityDao = new AppointmentActivityDAO();
            $pointOfSaleDao = new PointOfSaleDAO();
            $personDao = new PersonDAO();
            $startDate = new DateTime($_POST["start"]);
            $endDate = new DateTime($_POST["end"]);
            $effectiveEndDate = new DateTime("{$endDate->format(DATE_FORMAT)} 23:59:59");
            $activityId = isset($_POST["activity"]) && $_POST["activity"] != "" ? (int)$_POST["activity"] : null;
            $pointOfSaleId = isset($_POST["pointOfSale"]) && $_POST["pointOfSale"] != "" ? (int)$_POST["pointOfSale"] : null;
            $customerId = isset($_POST["customer"]) && $_POST["customer"] != "" ? (int)$_POST["customer"] : null;
            $activity = $activityId != null ? $appointmentActivityDao->getAppointmentActivityById($activityId) : null;
            $pointOfSale = $pointOfSaleId != null ? $pointOfSaleDao->getPointOfSaleById($pointOfSaleId) : null;
            $customer = $customerId != null ? $personDao->getPersonById($customerId) : null;
            $currentUser = LogDAO::getCurrentUser();
            $targetPointOfSale = $pointOfSale != null ? $pointOfSale : $currentUser->getIdPointVente();
            $appointments = $appointmentDao->getAppointmentsOverPeriod($startDate, $effectiveEndDate, $activity, $targetPointOfSale, $customer);
            $result = [];
            $currentDate = new DateTime();
            foreach ($appointments as $appointment) {
                $doneIndicator = strtotime($currentDate->format(DATE_FORMAT_WITH_TIME)) > strtotime($appointment->getDateFin()->format(DATE_FORMAT_WITH_TIME)) ? 1 : 0;
                $result[] = [
                    "title" => "{$appointment->getIdActivite()->getTitre()} - {$appointment->getIdClient()->getNom()} {$appointment->getIdClient()->getPrenom()}",
                    "start" => $appointment->getDateDebut()->format(DATE_FORMAT_WITH_TIME),
                    "end" => $appointment->getDateFin()->format(DATE_FORMAT_WITH_TIME),
                    "extendedProps" => [
                        "appointmentId" => $appointment->getIdRendezVous(),
                        "doneIndicator" => $doneIndicator != 0 ? ($appointment->getActiviteRealisee() == 1 ? 2 : 3) : $doneIndicator,
                    ],
                    "color" => $appointment->getIdActivite()->getBackground() != null ? $appointment->getIdActivite()->getBackground() : APP_SECONDARY_BRAND,
                ];
            }
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function createAction()
    {
        try {
            $appointmentDao = new AppointmentDAO();
            $appointmentActivityDao = new AppointmentActivityDAO();
            $personDao = new PersonDAO();
            $pointOfSaleDao = new PointOfSaleDAO();
            $startDate = new DateTime($_POST["startDate"]);
            $endDate = new DateTime($_POST["endDate"]);
            $activityId = (int)$_POST["activity"];
            $customerId = (int)$_POST["customer"];
            $pointOfSaleId = (int)$_POST["pointOfSale"];
            $description = $_POST["description"];
            $staffMembers = implode(";", json_decode($_POST["staffMembers"], true));
            $activity = $appointmentActivityDao->getAppointmentActivityById($activityId);
            $customer = $personDao->getPersonById($customerId);
            $pointOfSale = $pointOfSaleDao->getPointOfSaleById($pointOfSaleId);
            $appointment = $appointmentDao->create();
            $appointment->setObservation($description);
            $appointment->setDateDebut($startDate);
            $appointment->setDateFin($endDate);
            $appointment->setIdActivite($activity);
            $appointment->setIdClient($customer);
            $appointment->setListeIdUtilisateurs($staffMembers);
            $appointment->setIdPointVente($pointOfSale);
            $appointment->setActiviteRealisee(0);
            $appointment->setStatut(1);
            $appointmentDao->save($appointment);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("appointment.index.appointmentSuccessfullyRegistered")} : {$appointment->getIdClient()->getNom()} {$appointment->getIdClient()->getPrenom()}:{$appointment->getIdActivite()->getTitre()},({$appointment->getIdPointVente()->getNomPointVente()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 sucess
     * 101 unkwown error
     */
    public function detailAction()
    {
        try {
            $appointmentDao = new AppointmentDAO();
            $userDao = new UserDAO();
            $id = (int)$_POST["id"];
            $appointment = $appointmentDao->getAppointmentById($id);
            $staffMembersIds = explode(";", $appointment->getListeIdUtilisateurs());
            $staffMembersNames = [];
            foreach ($staffMembersIds as $staffMemberId) {
                $staffUser = $userDao->getUserById((int)$staffMemberId);
                if($staffUser != null) {
                    $staffMembersNames[] = "{$staffUser->getIdPersonne()->getNom()} {$staffUser->getIdPersonne()->getPrenom()}";
                }
            }
            $result = [
                "startDate" => $appointment->getDateDebut()->format(DATE_FORMAT_WITH_TIME),
                "endDate" => $appointment->getDateFin()->format(DATE_FORMAT_WITH_TIME),
                "activity" => $appointment->getIdActivite()->getIdActivite(),
                "activityName" => $appointment->getIdActivite()->getTitre(),
                "customer" => $appointment->getIdClient()->getIdPersonne(),
                "customerName" => "{$appointment->getIdClient()->getNom()} {$appointment->getIdClient()->getPrenom()}",
                "pointOfSale" => $appointment->getIdPointVente()->getIdPointVente(),
                "pointOfSaleName" => $appointment->getIdPointVente()->getNomPointVente(),
                "description" => $appointment->getObservation(),
                "staffInvolded" => $staffMembersIds,
                "staffInvolvedNames" => $staffMembersNames,
                "honored" => $appointment->getActiviteRealisee(),
            ];
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function declarehonoredAction()
    {
        try {
            $appointmentDao = new AppointmentDAO();
            $id = (int)$_POST["id"];
            $appointment = $appointmentDao->getAppointmentById($id);
            $appointment->setActiviteRealisee(1);
            $appointmentDao->update($appointment);
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function updateAction()
    {
        try {
            $appointmentDao = new AppointmentDAO();
            $appointmentActivityDao = new AppointmentActivityDAO();
            $personDao = new PersonDAO();
            $pointOfSaleDao = new PointOfSaleDAO();
            $id = (int)$_POST["id"];
            $startDate = new DateTime($_POST["startDate"]);
            $endDate = new DateTime($_POST["endDate"]);
            $activityId = (int)$_POST["activity"];
            $customerId = (int)$_POST["customer"];
            $pointOfSaleId = (int)$_POST["pointOfSale"];
            $description = $_POST["description"];
            $staffMembers = implode(";", json_decode($_POST["staffMembers"], true));
            $activity = $appointmentActivityDao->getAppointmentActivityById($activityId);
            $customer = $personDao->getPersonById($customerId);
            $pointOfSale = $pointOfSaleDao->getPointOfSaleById($pointOfSaleId);
            $appointment = $appointmentDao->getAppointmentById($id);
            $appointment->setObservation($description);
            $appointment->setDateDebut($startDate);
            $appointment->setDateFin($endDate);
            $appointment->setIdActivite($activity);
            $appointment->setIdClient($customer);
            $appointment->setListeIdUtilisateurs($staffMembers);
            $appointment->setIdPointVente($pointOfSale);
            $appointmentDao->update($appointment);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("appointment.index.appointmentUpdatedSuccessfully")} : {$appointment->getIdClient()->getNom()} {$appointment->getIdClient()->getPrenom()}:{$appointment->getIdActivite()->getTitre()},({$appointment->getIdPointVente()->getNomPointVente()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function deleteAction()
    {
        try {
            $id = (int)$_POST["id"];
            $appointmentDao = new AppointmentDAO();
            $appointment = $appointmentDao->getAppointmentById($id);
            if ($appointment->getStatut() == 0) {
                echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE);
                exit();
            }
            $appointment->setStatut(0);
            $appointmentDao->update($appointment);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("appointment.index.appointmentDeletedSuccessfully")} : {$appointment->getIdClient()->getNom()} {$appointment->getIdClient()->getPrenom()}:{$appointment->getIdActivite()->getTitre()},({$appointment->getIdPointVente()->getNomPointVente()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }
}
