<?php

include APPLICATION_PATH . '/includes/daos.php';

class CustomerController extends Zend_Controller_Action
{
    private $translator;

    public function init()
    {
        session_start();
        SubscriptionChecker::checkSubscriptionValidity($this);
        if (isset($_SESSION[SESSION_USER_ID]) && $_SESSION[SESSION_USER_ID] != null) {
            $_SESSION[LAST_ACTION_DATE] = new DateTime();
            LogDAO::updateLastActionDateForCurrentUser();
            $this->translator = Translator::init();
        } else {
            if ($this->getRequest()->isPost()) {
                echo ApiResponse::respond(SESSION_EXPIRED_CODE);
                exit();
            } else {
                $this->_helper->redirector('index', 'Index');
            }
        }
    }

    public function indexAction()
    {
        AccessRightChecker::checkAccessRight($this, 11);
        $pointOfSaleDao = new PointOfSaleDAO();
        $userDao = new UserDAO();
        $currentUser = $userDao->getUserById($_SESSION[SESSION_USER_ID]);
        $pointOfSales = [];
        if ($currentUser->getIdPointVente() != null) {
            $pointOfSales[] = $currentUser->getIdPointVente();
        } else {
            $pointOfSales = $pointOfSaleDao->listPointOfSales();
        }
        $this->view->translator = $this->translator;
        $this->view->pointOfSales = $pointOfSales;
    }

    /**
     * @throws Exception
     */
    public function datatableAction()
    {
        $name = $_POST["name"] != "" ? $_POST["name"] : null;
        $phone = $_POST["phone"] != "" ? $_POST["phone"] : null;
        $email = $_POST["email"] != "" ? $_POST["email"] : null;
        $whereName = "";
        $wherePhone = "";
        $whereEmail = "";
        if ($name != null) {
            $whereName .= " AND CONCAT(p.nom,' ',COALESCE(p.prenom, '')) LIKE '%{$name}%'";
        }
        if ($phone != null) {
            $wherePhone .= " AND p.telephone LIKE '%{$phone}%'";
        }
        if ($email != null) {
            $whereEmail .= " AND p.email LIKE '%{$email}%'";
        }
        $customerType = TYPE_PERSONNE_CLIENT;
        $where = "p.type = '{$customerType}' AND p.statut = 1{$whereName}{$wherePhone}{$whereEmail}";
        $customersTable = "
                (
                    SELECT p.id_personne AS customer_id,
                    CONCAT(p.nom,' ',COALESCE(p.prenom, '')) AS customer_name, p.email as customer_email,
                    p.telephone AS customer_phone, p.date_creation as customer_creation_date,
                    p.ville as customer_city
                    FROM personne p
                    WHERE {$where}
                ) AS customersTable
        ";
        $primaryKey = 'customer_id';
        $columns = [
            ['db' => 'customer_name', 'dt' => 0],
            ['db' => 'customer_creation_date', 'dt' => 1],
            ['db' => 'customer_phone', 'dt' => 2],
            ['db' => 'customer_email', 'dt' => 3],
            ['db' => 'customer_city', 'dt' => 4],
            ['db' => 'customer_id', 'dt' => 5],
        ];
        $computeDatatableResults = SSP::customSimple($_POST, $customersTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        foreach ($datatableResult["data"] as $row) {
            $customerName = $row[0];
            $creationDate = new DateTime($row[1]);
            $customerCreationDate = $creationDate->format(DATE_FORMAT);
            $customerPhone = $row[2];
            $customerEmail = $row[3];
            $customerCity = $row[4];
            $customerId = $row[5];
            $actions = "
                <button title='{$this->translator->_("shared.consult")}' about='{$customerId}' class='mini ui icon button app-primary-button app-consult-action'>
                    <i class='eye icon'></i>
                </button>
                <button title='{$this->translator->_("shared.edit")}' about='{$customerId}' class='mini ui icon button app-secondary-button app-edit-action'>
                    <i class='edit icon'></i>
                </button>
                <button title='{$this->translator->_("shared.delete")}' about='{$customerId}' class='mini ui red icon button app-delete-action'>
                    <i class='trash icon'></i>
                </button>
            ";
            $tableRow = [
                $customerName,
                $customerCreationDate,
                $customerPhone,
                $customerEmail,
                $customerCity,
                $actions,
            ];
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    public function datatablebalancesAction()
    {
        try {
            $customerType = TYPE_PERSONNE_CLIENT;
            $pointOfSaleTargetId = (int)$_POST["pointOfSale"] != 0 ? (int)$_POST["pointOfSale"] : null;
            $customerName = $_POST["customerName"] != "" ? $_POST["customerName"] : null;
            $wherePointOfSale = "";
            $whereCustomerName = "";
            if ($pointOfSaleTargetId != null) {
                $wherePointOfSale = " AND pv.id_point_vente = {$pointOfSaleTargetId}";
            }
            if ($customerName != null) {
                $whereCustomerName = " AND CONCAT(pe.nom, ' ', COALESCE(pe.prenom, '')) LIKE '%{$customerName}%'";
            }
            $where = "
                pe.statut = 1 AND pe.type='{$customerType}' AND pv.statut = 1 AND (cc.statut = 1 OR cc.statut IS NULL)
                {$wherePointOfSale}{$whereCustomerName}
            ";
            $balancesTable = "
                (
                    SELECT CONCAT(pe.id_personne,pv.id_point_vente) AS customer_balance_id,
                    pv.nom_point_vente AS point_of_sale_name, CONCAT(pe.nom, ' ', COALESCE(pe.prenom, '')) AS customer_name,
                    (
                        SELECT COUNT(fc.id_facture) FROM facturec fc 
                        WHERE fc.statut = 1 AND fc.valider = 1 AND
                        fc.id_personne = pe.id_personne AND fc.id_point_vente = pv.id_point_vente
                    ) AS number_of_purchases,
                    (
                        SELECT SUM(fc.montant_ht) FROM facturec fc 
                        WHERE fc.statut = 1 AND fc.valider = 1 AND
                        fc.id_personne = pe.id_personne AND fc.id_point_vente = pv.id_point_vente
                        GROUP BY fc.id_personne
                    ) AS accumulated_purchases,
                    (
                        SELECT MIN(fc.date_facture) FROM facturec fc 
                        WHERE fc.statut = 1 AND fc.valider = 1 AND
                        fc.id_personne = pe.id_personne AND fc.id_point_vente = pv.id_point_vente
                    ) AS first_purchase_date,
                    (
                        SELECT MAX(fc.date_facture) FROM facturec fc 
                        WHERE fc.statut = 1 AND fc.valider = 1 AND
                        fc.id_personne = pe.id_personne AND fc.id_point_vente = pv.id_point_vente
                    ) AS last_purchase_date,
                    cc.id_compte_client AS customer_account_id,
                    pe.id_personne AS customer_id, pv.id_point_vente AS point_of_sale_id
                    FROM personne pe LEFT OUTER JOIN point_vente pv ON 1 = 1 
                    LEFT OUTER JOIN compte_client cc ON 
                    ((pe.id_personne = cc.id_client AND pv.id_point_vente = cc.id_point_vente) OR cc.id_client IS NULL)
                    WHERE {$where}
                ) AS balancesTable
            ";
            $primaryKey = 'customer_balance_id';
            $columns = [
                ['db' => 'point_of_sale_name', 'dt' => 0],
                ['db' => 'customer_name', 'dt' => 1],
                ['db' => 'number_of_purchases', 'dt' => 2],
                ['db' => 'accumulated_purchases', 'dt' => 3],
                ['db' => 'first_purchase_date', 'dt' => 4],
                ['db' => 'last_purchase_date', 'dt' => 5],
                ['db' => 'customer_account_id', 'dt' => 6],
                ['db' => 'customer_id', 'dt' => 7],
                ['db' => 'point_of_sale_id', 'dt' => 8],
            ];
            $computeDatatableResults = SSP::customSimple($_POST, $balancesTable, $primaryKey, $columns);
            $datatableResult = $computeDatatableResults["datatable"];
            $finalResult = $computeDatatableResults["custom"];
            foreach ($datatableResult["data"] as $row) {
                $pointOfSaleName = $row[0];
                $customerName = $row[1];
                $numberOfPurchases = $row[2];
                $accumulatedPurchases = number_format((float)$row[3], 0, '', ' ');
                $firstPurchaseAtDate = new DateTime($row[4]);
                $lastPurchaseAtDate = new DateTime($row[5]);
                $firstPurchaseDate = $row[4] != null ? $firstPurchaseAtDate->format(DATE_FORMAT) : $this->translator->_("shared.undefined");
                $lastPurchaseDate = $row[5] != null ? $lastPurchaseAtDate->format(DATE_FORMAT) : $this->translator->_("shared.undefined");
                $customerAccountId = (int)$row[6];
                $customerId = (int)$row[7];
                $pointOfSaleId = (int)$row[8];
                $customerAccount = null;
                $customerAccountDao = new CustomerAccountDAO();
                $personDao = new PersonDAO();
                $pointOfSaleDao = new PointOfSaleDAO();
                $pointOfSale = $pointOfSaleDao->getPointOfSaleById($pointOfSaleId);
                $customer = $personDao->getPersonById($customerId);
                if ($customerAccountId == null) {
                    $customerAccount = $customerAccountDao->create();
                    $customerAccount->setDateCreation(new DateTime());
                    $customerAccount->setIdClient($customer);
                    $customerAccount->setIdPointVente($pointOfSale);
                    $customerAccount->setStatut(1);
                    $customerAccountDao->save($customerAccount);
                } else {
                    $customerAccount = $customerAccountDao->getCustomerAccountById($customerAccountId);
                }
                $currentBalance = number_format((float)$customerAccountDao->getCustomerAccountBalance($customerAccount), 0, '', ' ');
                $numberOfPoints = 0;
                $loyaltyCardDao = new LoyaltyCardDAO();
                $loyaltyCard = $loyaltyCardDao->getCustomerActiveCard($customer, $pointOfSale);
                if ($loyaltyCard != null) {
                    $loyaltyCardPointDao = new LoyaltyCardPointDAO();
                    $numberOfPoints = $loyaltyCardPointDao->getPointsOfLoyaltyCard($loyaltyCard);
                }
                $tableRow = [
                    $pointOfSaleName,
                    $customerName,
                    $currentBalance,
                    $numberOfPurchases,
                    $accumulatedPurchases,
                    $numberOfPoints,
                    $firstPurchaseDate,
                    $lastPurchaseDate,
                ];
                $finalResult["data"][] = $tableRow;
            }
            echo json_encode($finalResult);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 duplicate phone number
     * 102 unknown error
     */
    public function createAction()
    {
        try {
            $lastName = $_POST["lastName"];
            $firstName = $_POST["firstName"] != "" ? $_POST["firstName"] : null;
            $type = $_POST["type"];
            $gender = isset($_POST["gender"]) ? $_POST["gender"] : null;
            $phoneNumber = $_POST["phoneNumber"];
            $email = $_POST["email"] != "" ? $_POST["email"] : null;
            $city = $_POST["city"];
            $dateOfBirth = $_POST["dateOfBirth"] != "" ? $_POST["dateOfBirth"] : null;
            $address = $_POST["address"] != "" ? $_POST["address"] : null;
            $creditRight = (int)$_POST["creditRight"];
            $personDao = new PersonDAO();
            if ($personDao->customerPhoneNumberExists($phoneNumber)) {
                echo ApiResponse::respond(101);
                exit();
            }
            $customer = $personDao->create();
            $customer->setNom($lastName);
            if ($firstName != null) {
                $customer->setPrenom($firstName);
            }
            $customer->setNature($type);
            if ($gender != null) {
                $customer->setSexe($gender);
            }
            $customer->setTelephone($phoneNumber);
            if ($email != null) {
                $customer->setEmail($email);
            }
            $customer->setVille($city);
            if ($dateOfBirth != null) {
                $customer->setDateNaissance(new DateTime($dateOfBirth));
            }
            if ($address != null) {
                $customer->setAdresse($address);
            }
            $customer->setDroitCredit($creditRight);
            $customer->setStatut(1);
            $customer->setType(TYPE_PERSONNE_CLIENT);
            $customer->setDateCreation(new DateTime());
            $personDao->save($customer);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("customer.index.customerRegisteredSuccessfully")} : N°{$customer->getIdPersonne()}({$customer->getNom()} {$customer->getPrenom()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function detailAction()
    {
        try {
            $id = (int)$_POST["id"];
            $personDao = new PersonDAO();
            $customer = $personDao->getPersonById($id);
            $result = [
                "lastName" => $customer->getNom(),
                "firstName" => $customer->getPrenom(),
                "type" => $customer->getNature(),
                "phoneNumber" => $customer->getTelephone(),
                "email" => $customer->getEmail(),
                "city" => $customer->getVille(),
                "dateOfBirth" => $customer->getDateNaissance() != null ? $customer->getDateNaissance()->format(DATE_FORMAT) : "",
                "address" => $customer->getAdresse(),
                "creditRight" => $customer->getDroitCredit(),
            ];
            if ($customer->getSexe() != null) {
                $result["gender"] = $customer->getSexe();
            }
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 duplicate phone number
     * 102 unknown error
     */
    public function updateAction()
    {
        try {
            $id = (int)$_POST["id"];
            $lastName = $_POST["lastName"];
            $firstName = $_POST["firstName"] != "" ? $_POST["firstName"] : null;
            $type = $_POST["type"];
            $gender = isset($_POST["gender"]) ? $_POST["gender"] : null;
            $phoneNumber = $_POST["phoneNumber"];
            $email = $_POST["email"] != "" ? $_POST["email"] : null;
            $city = $_POST["city"];
            $dateOfBirth = $_POST["dateOfBirth"] != "" ? $_POST["dateOfBirth"] : null;
            $address = $_POST["address"] != "" ? $_POST["address"] : null;
            $creditRight = (int)$_POST["creditRight"];
            $personDao = new PersonDAO();
            $customer = $personDao->getPersonById($id);
            if ($personDao->customerPhoneNumberExists($phoneNumber, $customer)) {
                echo ApiResponse::respond(101);
                exit();
            }
            $customer->setNom($lastName);
            if ($firstName != null) {
                $customer->setPrenom($firstName);
            }
            $customer->setNature($type);
            if ($gender != null) {
                $customer->setSexe($gender);
            }
            $customer->setTelephone($phoneNumber);
            if ($email != null) {
                $customer->setEmail($email);
            }
            $customer->setVille($city);
            if ($dateOfBirth != null) {
                $customer->setDateNaissance(new DateTime($dateOfBirth));
            }
            if ($address != null) {
                $customer->setAdresse($address);
            }
            $customer->setDroitCredit($creditRight);
            $personDao->update($customer);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("customer.index.customerUpdatedSuccessfully")} : N°{$customer->getIdPersonne()}({$customer->getNom()} {$customer->getPrenom()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 impossible deletion
     * 102 unknown error
     */
    public function deleteAction()
    {
        try {
            $id = (int)$_POST["id"];
            $personDao = new PersonDAO();
            $customerAccountDao = new CustomerAccountDAO();
            $customer = $personDao->getPersonById($id);
            if ($customer->getStatut() == 0) {
                echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE);
                exit();
            }
            $customerAccounts = $customerAccountDao->getCustomerAccountByCustomerAndPointOfSale($customer);
            $canDelete = true;
            foreach ($customerAccounts as $customerAccount) {
                if ($customerAccountDao->getCustomerAccountBalance($customerAccount) > 0) {
                    $canDelete = false;
                    break;
                }
            }
            if (!$canDelete) {
                echo ApiResponse::respond(101);
                exit();
            }
            $customer->setStatut(0);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("customer.index.customerDeletedSuccessfully")} : N°{$customer->getIdPersonne()}({$customer->getNom()} {$customer->getPrenom()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * true success
     * false unknown error
     */
    public function semanticsearchAction() {
        try {
            $likeQuery = $_GET["query"];
            if($likeQuery == "") {
                echo ApiResponse::semantic(true, []);
                exit();
            }

            $personDao = new PersonDAO();
            $possibleCustomers = $personDao->searchClientByName($likeQuery);
            $result = [];
            foreach($possibleCustomers as $possibleCustomer) {
                $result[] = [
                    "name" => "{$possibleCustomer->getNom()} {$possibleCustomer->getPrenom()}",
                    "value" => $possibleCustomer->getIdPersonne(),
                ];
            }
            echo ApiResponse::semantic(true, $result);
        } catch (Exception $ex) {
            echo ApiResponse::semantic(false, ["message" => $ex->getMessage()]);
        }
        exit();
    }
    /*
     * 100 success
     * 101 unknown error
     */
    public function getcustomersAction()
    {
        $like = $_GET["q"];
        $personDAO = new PersonDAO();
        $customers = $personDAO->searchClientByName($like);
        $tabCustomer = array();
        foreach( $customers as $customer ){
            $tabCustomer[] = array("name" => $customer->getNom() . " " . $customer->getPrenom(), "id" => $customer->getIdPersonne(),  "phone" => $customer->getTelephone());
        }
        echo json_encode(array("results" => $tabCustomer));
        exit();
    }
}
