<?php

include APPLICATION_PATH . '/includes/daos.php';

class ExitController extends Zend_Controller_Action
{
    private $translator;

    public function init()
    {
        session_start();
        SubscriptionChecker::checkSubscriptionValidity($this);
        if (isset($_SESSION[SESSION_USER_ID]) && $_SESSION[SESSION_USER_ID] != null) {
            $_SESSION[LAST_ACTION_DATE] = new DateTime();
            LogDAO::updateLastActionDateForCurrentUser();
            $this->translator = Translator::init();
        } else {
            if ($this->getRequest()->isPost()) {
                echo ApiResponse::respond(SESSION_EXPIRED_CODE);
                exit();
            } else {
                $this->_helper->redirector('index', 'Index');
            }
        }
    }

    public function typeAction()
    {
        AccessRightChecker::checkAccessRight($this, 132);
        $this->view->translator = $this->translator;
    }

    public function datatabletypesAction()
    {
        $name = $_POST["name"] != "" ? $_POST["name"] : null;
        $code = $_POST["code"] != "" ? $_POST["code"] : null;
        $whereName = "";
        $whereCode = "";
        if ($name != null) {
            $whereName .= " AND t.nom LIKE '%{$name}%'";
        }
        if ($code != null) {
            $whereCode .= " AND t.code LIKE '%{$code}%'";
        }
        $where = "t.statut = 1{$whereName}{$whereCode}";
        $exitTypesTable = "
                (
                    SELECT t.id_type_sortie AS exit_type_id, t.nom AS exit_type_name, t.code AS exit_type_code
                    FROM type_sortie_exceptionnelle t
                    WHERE {$where}
                ) AS exitTypesTable
        ";
        $primaryKey = 'exit_type_id';
        $columns = [
            ['db' => 'exit_type_name', 'dt' => 0],
            ['db' => 'exit_type_code', 'dt' => 1],
            ['db' => 'exit_type_id', 'dt' => 2],
        ];
        $computeDatatableResults = SSP::customSimple($_POST, $exitTypesTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        foreach ($datatableResult["data"] as $row) {
            $exitTypeName = $row[0];
            $exitTypeCode = $row[1];
            $exitTypeId = (int)$row[2];
            $actions = "
                <button title='{$this->translator->_("shared.edit")}' about='{$exitTypeId}' class='mini ui icon button app-secondary-button app-edit-action'>
                    <i class='edit icon'></i>
                </button>
                <button title='{$this->translator->_("shared.delete")}' about='{$exitTypeId}' class='mini ui red icon button app-delete-action'>
                    <i class='trash icon'></i>
                </button>
            ";
            $tableRow = [
                $exitTypeName,
                $exitTypeCode,
                $actions,
            ];
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    /**
     * 100 success
     * 101 duplicate on name
     * 102 duplicate on code
     * 103 unknown error
     */
    public function createtypeAction()
    {
        try {
            $name = $_POST["name"];
            $code = $_POST["code"];
            $exceptionalOutingTypeDao = new ExceptionalOutingTypeDAO();
            if ($exceptionalOutingTypeDao->checkExceptionalOutingTypeNameExist($name)) {
                echo ApiResponse::respond(101);
                exit();
            }
            if ($exceptionalOutingTypeDao->checkExceptionalOutingTypeCodeExist($code)) {
                echo ApiResponse::respond(102);
                exit();
            }
            $exceptionalOutingType = $exceptionalOutingTypeDao->create();
            $exceptionalOutingType->setNom($name);
            $exceptionalOutingType->setCode($code);
            $exceptionalOutingType->setStatut(1);
            $exceptionalOutingTypeDao->save($exceptionalOutingType);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("exit.type.exceptionalOutingTypeSuccessfullyRegistered")} : N°{$exceptionalOutingType->getIdTypeSortie()}({$exceptionalOutingType->getNom()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(103, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 duplicate on name
     * 102 duplicate on code
     * 103 unknown error
     */
    public function updatetypeAction()
    {
        try {
            $id = (int)$_POST["id"];
            $name = $_POST["name"];
            $code = $_POST["code"];
            $exceptionalOutingTypeDao = new ExceptionalOutingTypeDAO();
            $exceptionalOutingType = $exceptionalOutingTypeDao->getExceptionalOutingTypeById($id);
            if ($exceptionalOutingTypeDao->checkExceptionalOutingTypeNameExist($name, $exceptionalOutingType)) {
                echo ApiResponse::respond(101);
                exit();
            }
            if ($exceptionalOutingTypeDao->checkExceptionalOutingTypeCodeExist($code, $exceptionalOutingType)) {
                echo ApiResponse::respond(102);
                exit();
            }
            $exceptionalOutingType->setNom($name);
            $exceptionalOutingType->setCode($code);
            $exceptionalOutingTypeDao->update($exceptionalOutingType);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("exit.type.exceptionalOutingTypeUpdatedSuccessfully")} : N°{$exceptionalOutingType->getIdTypeSortie()}({$exceptionalOutingType->getNom()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(103, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function deletetypeAction()
    {
        try {
            $id = (int)$_POST["id"];
            $exceptionalOutingTypeDao = new ExceptionalOutingTypeDAO();
            $exceptionalOutingType = $exceptionalOutingTypeDao->getExceptionalOutingTypeById($id);
            if ($exceptionalOutingType->getStatut() == 0) {
                echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE);
                exit();
            }
            $exceptionalOutingType->setStatut(0);
            $exceptionalOutingTypeDao->update($exceptionalOutingType);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("exit.type.exceptionalOutingTypeDeletedSuccessfully")} : N°{$exceptionalOutingType->getIdTypeSortie()}({$exceptionalOutingType->getNom()})"
            );
            echo ApiResponse::respond();
            exit();
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    public function indexAction()
    {
        AccessRightChecker::checkAccessRight($this, 21);
        $currentUser = LogDAO::getCurrentUser();
        $storeDao = new StoreDAO();
        $stores = [];
        if ($currentUser->getIdPointVente() == null || in_array($currentUser->getIdProfil()->getCode(), IDENTIFIED_PROFILES_CODES)) {
            $stores = $storeDao->getAllStores();
        } else {
            $stores[] = $currentUser->getIdPointVente()->getIdMagasin();
        }
        $exceptionalOutingTypeDao = new ExceptionalOutingTypeDAO();
        $this->view->stores = $stores;
        $this->view->exceptionalOutingTypes = $exceptionalOutingTypeDao->getAllExceptionalOutingTypes();
        $this->view->translator = $this->translator;
    }

    public function datatableAction()
    {
        $store = (int)$_POST["store"] != 0 ? (int)$_POST["store"] : null;
        $exitType = $_POST["exitType"] != "" ? (int)$_POST["exitType"] : null;
        $from = $_POST["from"] != "" ? $_POST["from"] : null;
        $to = $_POST["to"] != "" ? $_POST["to"] : null;
        $whereStore = "";
        $whereExitType = "";
        $whereFrom = "";
        $whereTo = "";
        if ($store != null) {
            $whereStore .= " AND s.id_magasin = {$store}";
        }
        if ($exitType != null) {
            $exceptionalOutingTypeDao = new ExceptionalOutingTypeDAO();
            $exceptionOutingType = $exceptionalOutingTypeDao->getExceptionalOutingTypeById($exitType);
            $whereExitType .= " AND s.type_sortie = '{$exceptionOutingType->getCode()}'";
        }
        if ($from != null) {
            $whereFrom .= " AND s.date_sortie >= '{$from} 00:00:00'";
        }
        if ($to != null) {
            $whereTo .= " AND s.date_sortie <= '{$to} 23:59:59'";
        }
        $where = "s.statut = 1 AND s.type_sortie <> '" . TYPE_SORTIE_VENTE . "'{$whereStore}{$whereExitType}{$whereFrom}{$whereTo}";
        $exceptionalOutingsTable = "
                (
                    SELECT s.id_sortie AS exit_id, m.nom_magasin AS exit_store, tse.nom AS exit_type, s.date_saisie AS exit_entry_date,
                    s.date_sortie AS exit_outing_date, s.motif AS exit_comment, CONCAT(pe.nom, ' ', COALESCE(pe.prenom, '')) AS exit_user,
                    s.valider AS exit_validated
                    FROM sortie s JOIN magasin m ON s.id_magasin = m.id_magasin JOIN type_sortie_exceptionnelle tse ON s.type_sortie = tse.code
                    JOIN utilisateur u ON s.id_utilisateur = u.id_utilisateur JOIN personne pe ON u.id_personne = pe.id_personne 
                    WHERE {$where}
                ) AS exceptionalOutingsTable
        ";
        $primaryKey = 'exit_id';
        $columns = [
            ['db' => 'exit_id', 'dt' => 0],
            ['db' => 'exit_store', 'dt' => 1],
            ['db' => 'exit_type', 'dt' => 2],
            ['db' => 'exit_entry_date', 'dt' => 3],
            ['db' => 'exit_outing_date', 'dt' => 4],
            ['db' => 'exit_comment', 'dt' => 5],
            ['db' => 'exit_user', 'dt' => 6],
            ['db' => 'exit_validated', 'dt' => 7],
        ];
        $computeDatatableResults = SSP::customSimple($_POST, $exceptionalOutingsTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        foreach ($datatableResult["data"] as $row) {
            $exitId = (int)$row[0];
            $exitStore = $row[1];
            $exitType = $row[2];
            $exitEntryDate = new DateTime($row[3]);
            $exitOutingDate = new DateTime($row[4]);
            $exitComment = $row[5];
            $exitUser = $row[6];
            $exitValidated = (int)$row[7];
            $actions = "
                <button title='{$this->translator->_("shared.consult")}' about='{$exitId}' class='mini ui icon button app-primary-button app-consult-action'>
                    <i class='eye icon'></i>
                </button>
            ";
            if ($exitValidated == 0) {
                $actions .= "
                    <button title='{$this->translator->_("shared.edit")}' about='{$exitId}' class='mini ui icon button app-secondary-button app-edit-action'>
                        <i class='edit icon'></i>
                    </button>
                    <button title='{$this->translator->_("shared.delete")}' about='{$exitId}' class='mini ui red icon button app-delete-action'>
                        <i class='trash icon'></i>
                    </button>
                ";
            } else {
                $actions .= "
                    <button title='{$this->translator->_("shared.print")}' about='{$exitId}' class='mini ui purple icon button app-print-action'>
                        <i class='print icon'></i>
                    </button>
                ";
            }
            $tableRow = [
                $exitId,
                $exitStore,
                $exitType,
                $exitEntryDate->format(DATE_FORMAT),
                $exitOutingDate->format(DATE_FORMAT),
                $exitComment,
                $exitUser,
                $actions,
            ];
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    public function exitstocksdatatableAction()
    {
        $store = (int)$_POST["store"];
        $batchNumber = $_POST["batchNumber"] != "" ? $_POST["batchNumber"] : null;
        $name = $_POST["name"] != "" ? $_POST["name"] : null;
        $whereStore = " AND s.id_magasin = {$store}";
        $whereBatchNumber = "";
        $whereName = "";
        if ($batchNumber != null) {
            $whereBatchNumber .= " AND s.num_lot LIKE '%{$batchNumber}%'";
        }
        if ($name != null) {
            $whereName .= " AND s.designation LIKE '%{$name}%'";
        }
        $where = "s.statut = 1 AND s.quantite > 0 AND p.statut = 1{$whereStore}{$whereBatchNumber}{$whereName} order by s.date_entree ASC";
        $exitStocksTable = "
                (
                    SELECT s.id_stock AS stock_id, s.quantite AS stock_quantity, s.prix_vente AS stock_price, s.num_lot AS stock_batch_number,
                    s.designation AS stock_designation, s.date_peremption AS expiration_date, p.image AS product_image,
                    p.perissable AS product_perishable
                    FROM stock s JOIN produit p ON s.id_produit = p.id_produit
                    WHERE {$where}
                ) AS exitStocksTable
        ";
        $primaryKey = 'stock_id';
        $columns = [
            ['db' => 'stock_id', 'dt' => 0],
            ['db' => 'stock_quantity', 'dt' => 1],
            ['db' => 'stock_price', 'dt' => 2],
            ['db' => 'stock_batch_number', 'dt' => 3],
            ['db' => 'stock_designation', 'dt' => 4],
            ['db' => 'product_image', 'dt' => 5],
            ['db' => 'expiration_date', 'dt' => 6],
            ['db' => 'product_perishable', 'dt' => 7],
        ];
        $computeDatatableResults = SSP::customSimple($_POST, $exitStocksTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        foreach ($datatableResult["data"] as $row) {
            $stockId = $row[0];
            $stockQuantity = $row[1];
            $stockPrice = (float)$row[2];
            $stockBatchNumber = $row[3];
            $stockDesignation = $row[4];
            $productImage = $row[5] != null ? $row[5] : "";
            $expirationDate = $row[6];
            $productPerishable = (int)$row[7];
            $tableRow = [
                [
                    $stockId,
                    $stockQuantity,
                    $stockPrice,
                    $stockBatchNumber,
                    $stockDesignation,
                    $productImage,
                    $expirationDate,
                    $productPerishable,
                ],
            ];
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    /**
     * 100 success
     * 101 insufficient quantity on stock
     * 102 unknown error
     */
    public function createAction()
    {
        $exitDao = new ExitDAO();
        $databaseConnection = $exitDao->entityManager->getConnection();
        $databaseConnection->beginTransaction();
        try {
            $storeId = (int)$_POST["store"];
            $exitTypeId = (int)$_POST["exitType"];
            $outingDate = new DateTime($_POST["outingDate"]);
            $commentOnExit = $_POST["commentOnExit"];
            $content = json_decode($_POST["content"], true);
            $exit = $exitDao->create();
            $exit->setIdUtilisateur(LogDAO::getCurrentUser());
            $storeDao = new StoreDAO();
            $store = $storeDao->getStoreById($storeId);
            $exit->setIdMagasin($store);
            $exceptionalOutingTypeDao = new ExceptionalOutingTypeDAO();
            $exceptionalOutingType = $exceptionalOutingTypeDao->getExceptionalOutingTypeById($exitTypeId);
            $exit->setTypeSortie($exceptionalOutingType->getCode());
            $exit->setDateSaisie(new DateTime());
            $exit->setDateSortie($outingDate);
            $exit->setMotif($commentOnExit);
            $exit->setValider(0);
            $exit->setStatut(1);
            $exitDao->save($exit);
            $stockDao = new StockDAO();
            $exitLineDao = new ExitLineDAO();
            foreach ($content as $exitDetail) {
                $stock = $stockDao->getStockById((int)$exitDetail["id"]);
                if ($stock->getQuantite() < (int)$exitDetail["quantityToOut"]) {
                    $databaseConnection->rollBack();
                    echo ApiResponse::respond(101, null, ["batchNumber" => $exitDetail["batchNumber"]]);
                    exit();
                }
                $exitLine = $exitLineDao->create();
                $exitLine->setIdSortie($exit);
                $exitLine->setIdStock($stock);
                $exitLine->setQuantite((int)$exitDetail["quantityToOut"]);
                $exitLine->setPrixUnitaire($stock->getPrixVente());
                $exitLine->setStatut(1);
                $exitLineDao->save($exitLine);
            }
            $databaseConnection->commit();
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("exit.index.exitSuccessfullyRegistered")} : N°{$exit->getIdSortie()}"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            $databaseConnection->rollBack();
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function detailAction()
    {
        try {
            $id = (int)$_POST["id"];
            $exitDao = new ExitDAO();
            $exceptionalOutingTypeDao = new ExceptionalOutingTypeDAO();
            $exit = $exitDao->getExitById($id);
            $exceptionalOutingType = $exceptionalOutingTypeDao->getExceptionalOutingTypeByCode($exit->getTypeSortie());
            $result = [
                "store" => $exit->getIdMagasin()->getIdMagasin(),
                "storeName" => $exit->getIdMagasin()->getNomMagasin(),
                "exitType" => $exceptionalOutingType->getIdTypeSortie(),
                "exitTypeName" => $exceptionalOutingType->getNom(),
                "outingDate" => $exit->getDateSortie()->format(DATE_FORMAT),
                "commentOnExit" => $exit->getMotif(),
                "content" => [],
            ];
            $exitLineDao = new ExitLineDAO();
            $exitLines = $exitLineDao->getExitLinesOfExit($exit);
            foreach ($exitLines as $exitLine) {
                $result["content"][] = [
                    "id" => $exitLine->getIdStock()->getIdStock(),
                    "designation" => $exitLine->getIdStock()->getDesignation(),
                    "batchNumber" => $exitLine->getIdStock()->getNumLot(),
                    "remainingQuantity" => $exitLine->getIdStock()->getQuantite(),
                    "sellingPrice" => $exitLine->getIdStock()->getPrixVente(),
                    "quantityToOut" => $exitLine->getQuantite(),
                ];
            }
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 insufficient quantity on stock
     * 102 unknown error
     */
    public function updateAction()
    {
        $exitDao = new ExitDAO();
        $databaseConnection = $exitDao->entityManager->getConnection();
        $databaseConnection->beginTransaction();
        try {
            $id = (int)$_POST["id"];
            $storeId = (int)$_POST["store"];
            $exitTypeId = (int)$_POST["exitType"];
            $outingDate = new DateTime($_POST["outingDate"]);
            $commentOnExit = $_POST["commentOnExit"];
            $content = json_decode($_POST["content"], true);
            $exit = $exitDao->getExitById($id);
            $storeDao = new StoreDAO();
            $store = $storeDao->getStoreById($storeId);
            $exit->setIdMagasin($store);
            $exceptionalOutingTypeDao = new ExceptionalOutingTypeDAO();
            $exceptionalOutingType = $exceptionalOutingTypeDao->getExceptionalOutingTypeById($exitTypeId);
            $exit->setTypeSortie($exceptionalOutingType->getCode());
            $exit->setDateSortie($outingDate);
            $exit->setMotif($commentOnExit);
            $exitDao->update($exit);
            $stockDao = new StockDAO();
            $exitLineDao = new ExitLineDAO();
            $exitLines = $exitLineDao->getExitLinesOfExit($exit);
            foreach ($exitLines as $exitLine) {
                $exitLineDao->delete($exitLine);
            }
            foreach ($content as $exitDetail) {
                $stock = $stockDao->getStockById((int)$exitDetail["id"]);
                if ($stock->getQuantite() < (int)$exitDetail["quantityToOut"]) {
                    $databaseConnection->rollBack();
                    echo ApiResponse::respond(101, null, ["batchNumber" => $exitDetail["batchNumber"]]);
                    exit();
                }
                $exitLine = $exitLineDao->create();
                $exitLine->setIdSortie($exit);
                $exitLine->setIdStock($stock);
                $exitLine->setQuantite((int)$exitDetail["quantityToOut"]);
                $exitLine->setPrixUnitaire($stock->getPrixVente());
                $exitLine->setStatut(1);
                $exitLineDao->save($exitLine);
            }
            $databaseConnection->commit();
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("exit.index.exitUpdatedSuccessfully")} : N°{$exit->getIdSortie()}"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            $databaseConnection->rollBack();
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function deleteAction()
    {
        try {
            $id = (int)$_POST["id"];
            $exitDao = new ExitDAO();
            $exit = $exitDao->getExitById($id);
            if ($exit->getStatut() == 0) {
                echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE);
                exit();
            }
            if ($exit->getValider() == 1) {
                echo ApiResponse::respond(ALREADY_VALIDATED_ISSUE_CODE);
                exit();
            }
            $exit->setStatut(0);
            $exitDao->update($exit);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("exit.index.exitDeletedSuccessfully")} : N°{$exit->getIdSortie()}"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    public function validationAction()
    {
        AccessRightChecker::checkAccessRight($this, 47);
        $exceptionalOutingTypeDao = new ExceptionalOutingTypeDAO();
        $storeDao = new StoreDAO();
        $currentUser = LogDAO::getCurrentUser();
        $stores = [];
        if ($currentUser->getIdPointVente() == null || in_array($currentUser->getIdProfil()->getCode(), IDENTIFIED_PROFILES_CODES)) {
            $stores = $storeDao->getAllStores();
        } else {
            $stores[] = $currentUser->getIdPointVente()->getIdMagasin();
        }
        $this->view->stores = $stores;
        $this->view->exceptionalOutingTypes = $exceptionalOutingTypeDao->getAllExceptionalOutingTypes();
        $this->view->translator = $this->translator;
    }

    public function datatablevalidationAction()
    {
        $store = (int)$_POST["store"] != 0 ? (int)$_POST["store"] : null;
        $exitType = $_POST["exitType"] != "" ? (int)$_POST["exitType"] : null;
        $from = $_POST["from"] != "" ? $_POST["from"] : null;
        $to = $_POST["to"] != "" ? $_POST["to"] : null;
        $whereStore = "";
        $whereExitType = "";
        $whereFrom = "";
        $whereTo = "";
        if ($store != null) {
            $whereStore .= " AND s.id_magasin = {$store}";
        }
        if ($exitType != null) {
            $exceptionalOutingTypeDao = new ExceptionalOutingTypeDAO();
            $exceptionOutingType = $exceptionalOutingTypeDao->getExceptionalOutingTypeById($exitType);
            $whereExitType .= " AND s.type_sortie = '{$exceptionOutingType->getCode()}'";
        }
        if ($from != null) {
            $whereFrom .= " AND s.date_sortie >= '{$from} 00:00:00'";
        }
        if ($to != null) {
            $whereTo .= " AND s.date_sortie <= '{$to} 23:59:59'";
        }
        $where = "s.statut = 1 AND s.valider = 0 AND s.type_sortie <> '" . TYPE_SORTIE_VENTE . "'{$whereStore}{$whereExitType}{$whereFrom}{$whereTo}";
        $exceptionalOutingsValidationTable = "
                (
                    SELECT s.id_sortie AS exit_id, m.nom_magasin AS exit_store, tse.nom AS exit_type, s.date_saisie AS exit_entry_date,
                    s.date_sortie AS exit_outing_date, s.motif AS exit_comment, CONCAT(pe.nom, ' ', COALESCE(pe.prenom, '')) AS exit_user,
                    s.valider AS exit_validated
                    FROM sortie s JOIN magasin m ON s.id_magasin = m.id_magasin JOIN type_sortie_exceptionnelle tse ON s.type_sortie = tse.code
                    JOIN utilisateur u ON s.id_utilisateur = u.id_utilisateur JOIN personne pe ON u.id_personne = pe.id_personne 
                    WHERE {$where}
                ) AS exceptionalOutingsValidationTable
        ";
        $primaryKey = 'exit_id';
        $columns = [
            ['db' => 'exit_id', 'dt' => 0],
            ['db' => 'exit_store', 'dt' => 1],
            ['db' => 'exit_type', 'dt' => 2],
            ['db' => 'exit_entry_date', 'dt' => 3],
            ['db' => 'exit_outing_date', 'dt' => 4],
            ['db' => 'exit_comment', 'dt' => 5],
            ['db' => 'exit_user', 'dt' => 6],
            ['db' => 'exit_validated', 'dt' => 7],
        ];
        $computeDatatableResults = SSP::customSimple($_POST, $exceptionalOutingsValidationTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        foreach ($datatableResult["data"] as $row) {
            $exitId = (int)$row[0];
            $exitStore = $row[1];
            $exitType = $row[2];
            $exitEntryDate = new DateTime($row[3]);
            $exitOutingDate = new DateTime($row[4]);
            $exitComment = $row[5];
            $exitUser = $row[6];
            $actions = "
                <button title='{$this->translator->_("shared.consult")}' about='{$exitId}' class='mini ui icon button app-primary-button app-consult-action'>
                    <i class='eye icon'></i>
                </button>
                <button title='{$this->translator->_("shared.validate")}' about='{$exitId}' class='mini ui green icon button app-validate-action'>
                    <i class='check icon'></i>
                </button>
            ";
            $tableRow = [
                $exitId,
                $exitStore,
                $exitType,
                $exitEntryDate->format(DATE_FORMAT),
                $exitOutingDate->format(DATE_FORMAT),
                $exitComment,
                $exitUser,
                $actions,
            ];
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    /**
     * 100 success
     * 101 insufficient quantity on stock
     * 102 unknown error
     */
    public function validateAction()
    {
        $exitDao = new ExitDAO();
        $databaseConnection = $exitDao->entityManager->getConnection();
        $databaseConnection->beginTransaction();
        try {
            $id = (int)$_POST["id"];
            $exit = $exitDao->getExitById($id);
            if ($exit->getStatut() == 0) {
                echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE);
                exit();
            }
            if ($exit->getValider() == 1) {
                echo ApiResponse::respond(ALREADY_VALIDATED_ISSUE_CODE);
                exit();
            }
            $exit->setValider(1);
            $exit->setDateValidation(new DateTime());
            $exitDao->update($exit);
            $stockDao = new StockDAO();
            $exitLineDao = new ExitLineDAO();
            $exitLines = $exitLineDao->getExitLinesOfExit($exit);
            foreach ($exitLines as $exitLine) {
                $stock = $exitLine->getIdStock();
                if ($stock->getQuantite() < $exitLine->getQuantite()) {
                    $databaseConnection->rollBack();
                    echo ApiResponse::respond(101, null, ["batchNumber" => $stock->getNumLot()]);
                    exit();
                }
                $stock->setQuantite($stock->getQuantite() - $exitLine->getQuantite());
                $stockDao->update($stock);
            }
            $databaseConnection->commit();
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("exit.validation.exitValidatedSuccessfully")} : N°{$exit->getIdSortie()}"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            $databaseConnection->rollBack();
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }
}
