<?php

include APPLICATION_PATH . '/includes/daos.php';

class ExpensecategoryController extends Zend_Controller_Action
{
    private $translator;

    public function init()
    {
        session_start();
        SubscriptionChecker::checkSubscriptionValidity($this);
        if (isset($_SESSION[SESSION_USER_ID]) && $_SESSION[SESSION_USER_ID] != null) {
            $_SESSION[LAST_ACTION_DATE] = new DateTime();
            LogDAO::updateLastActionDateForCurrentUser();
            $this->translator = Translator::init();
        } else {
            if ($this->getRequest()->isPost()) {
                echo ApiResponse::respond(SESSION_EXPIRED_CODE);
                exit();
            } else {
                $this->_helper->redirector('index', 'Index');
            }
        }
    }

    public function indexAction()
    {
        AccessRightChecker::checkAccessRight($this, 69);
        $this->view->translator = $this->translator;
    }

    public function datatableAction()
    {
        try {
            $name = $_POST["name"] != "" ? $_POST["name"] : null;
            $whereName = "";
            if ($name != null) {
                $whereName .= " AND c.nom LIKE '%{$name}%'";
            }
            $where = "c.statut = 1{$whereName}";
            $expenseCategoriesTable = "
                (
                    SELECT c.id_categorie_depense as expense_category_id, c.nom as expense_category_name
                    FROM categorie_depense c
                    WHERE {$where}
                ) AS expenseCategoriesTable
            ";
            $primaryKey = 'expense_category_id';
            $columns = [
                ['db' => 'expense_category_name', 'dt' => 0],
                ['db' => 'expense_category_id', 'dt' => 1],
            ];
            $computeDatatableResults = SSP::customSimple($_POST, $expenseCategoriesTable, $primaryKey, $columns);
            $datatableResult = $computeDatatableResults["datatable"];
            $finalResult = $computeDatatableResults["custom"];
            foreach ($datatableResult["data"] as $row) {
                $expenseCategoryName = $row[0];
                $expenseCategoryId = $row[1];
                $actions = "
                    <button title='{$this->translator->_("shared.edit")}' about='{$expenseCategoryId}' class='mini ui icon button app-secondary-button app-edit-action'>
                        <i class='edit icon'></i>
                    </button>
                    <button title='{$this->translator->_("shared.delete")}' about='{$expenseCategoryId}' class='mini ui red icon button app-delete-action'>
                        <i class='trash icon'></i>
                    </button>
                ";
                $tableRow = [
                    $expenseCategoryName,
                    $actions,
                ];
                $finalResult["data"][] = $tableRow;
            }
            echo json_encode($finalResult);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 duplicate
     * 102 unknown error
     */
    public function createAction()
    {
        try {
            $name = $_POST["name"];
            $expenseCategoryDao = new ExpenseCategoryDAO();
            if ($expenseCategoryDao->expenseCategoryNameAlreadyExists($name)) {
                echo ApiResponse::respond(101);
                exit();
            }
            $expenseCategory = $expenseCategoryDao->create();
            $expenseCategory->setNom($name);
            $expenseCategory->setStatut(1);
            $expenseCategoryDao->save($expenseCategory);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("expenseCategory.index.expenseCategorySuccessfullyRegistered")} : N°{$expenseCategory->getIdCategorieDepense()}({$expenseCategory->getNom()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 duplicate
     * 102 unknown error
     */
    public function updateAction()
    {
        try {
            $id = (int)$_POST["id"];
            $name = $_POST["name"];
            $expenseCategoryDao = new ExpenseCategoryDAO();
            $expenseCategory = $expenseCategoryDao->getExpenseCategoryById($id);
            if ($expenseCategoryDao->expenseCategoryNameAlreadyExists($name, $expenseCategory)) {
                echo ApiResponse::respond(101);
                exit();
            }
            $expenseCategory->setNom($name);
            $expenseCategoryDao->update($expenseCategory);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("expenseCategory.index.expenseCategoryUpdatedSuccessfully")} : N°{$expenseCategory->getIdCategorieDepense()}({$expenseCategory->getNom()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function deleteAction()
    {
        try {
            $id = (int)$_POST["id"];
            $expenseCategoryDao = new ExpenseCategoryDAO();
            $expenseCategory = $expenseCategoryDao->getExpenseCategoryById($id);
            if ($expenseCategory->getStatut() == 0) {
                echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE);
                exit();
            }
            $expenseCategory->setStatut(0);
            $expenseCategoryDao->update($expenseCategory);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("expenseCategory.index.expenseCategoryDeletedSuccessfully")} : N°{$expenseCategory->getIdCategorieDepense()}({$expenseCategory->getNom()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }
}
