<?php

include APPLICATION_PATH . '/includes/daos.php';

class FundentrycategoryController extends Zend_Controller_Action
{
    private $translator;

    public function init()
    {
        session_start();
        SubscriptionChecker::checkSubscriptionValidity($this);
        if (isset($_SESSION[SESSION_USER_ID]) && $_SESSION[SESSION_USER_ID] != null) {
            $_SESSION[LAST_ACTION_DATE] = new DateTime();
            LogDAO::updateLastActionDateForCurrentUser();
            $this->translator = Translator::init();
        } else {
            if ($this->getRequest()->isPost()) {
                echo ApiResponse::respond(SESSION_EXPIRED_CODE);
                exit();
            } else {
                $this->_helper->redirector('index', 'Index');
            }
        }
    }

    public function indexAction()
    {
        AccessRightChecker::checkAccessRight($this, 134);
        $this->view->translator = $this->translator;
    }

    public function datatableAction()
    {
        try {
            $name = $_POST["name"] != "" ? $_POST["name"] : null;
            $whereName = "";
            if ($name != null) {
                $whereName .= " AND c.nom LIKE '%{$name}%'";
            }
            $where = "c.statut = 1{$whereName}";
            $fundEntryCategoriesTable = "
                (
                    SELECT c.id_categorie_entree AS fund_entry_category_id, c.nom AS fund_entry_category_name
                    FROM categorie_entree c
                    WHERE {$where}
                ) AS fundEntryCategoriesTable
            ";
            $primaryKey = 'fund_entry_category_id';
            $columns = [
                ['db' => 'fund_entry_category_name', 'dt' => 0],
                ['db' => 'fund_entry_category_id', 'dt' => 1],
            ];
            $computeDatatableResults = SSP::customSimple($_POST, $fundEntryCategoriesTable, $primaryKey, $columns);
            $datatableResult = $computeDatatableResults["datatable"];
            $finalResult = $computeDatatableResults["custom"];
            foreach ($datatableResult["data"] as $row) {
                $fundEntryCategoryName = $row[0];
                $fundEntryCategoryId = $row[1];
                $actions = "
                    <button title='{$this->translator->_("shared.edit")}' about='{$fundEntryCategoryId}' class='mini ui icon button app-secondary-button app-edit-action'>
                        <i class='edit icon'></i>
                    </button>
                    <button title='{$this->translator->_("shared.delete")}' about='{$fundEntryCategoryId}' class='mini ui red icon button app-delete-action'>
                        <i class='trash icon'></i>
                    </button>
                ";
                $tableRow = [
                    $fundEntryCategoryName,
                    $actions,
                ];
                $finalResult["data"][] = $tableRow;
            }
            echo json_encode($finalResult);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 duplicate
     * 102 unknown error
     */
    public function createAction()
    {
        try {
            $name = $_POST["name"];
            $entryCategoryDao = new EntryCategoryDAO();
            if ($entryCategoryDao->entryCategoryNameAlreadyExists($name)) {
                echo ApiResponse::respond(101);
                exit();
            }
            $entryCategory = $entryCategoryDao->create();
            $entryCategory->setNom($name);
            $entryCategory->setStatut(1);
            $entryCategoryDao->save($entryCategory);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("fundEntryCategory.index.fundEntryCategorySuccessfullyRegistered")} : N°{$entryCategory->getIdCategorieEntree()}({$entryCategory->getNom()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 duplicate
     * 102 unknown error
     */
    public function updateAction()
    {
        try {
            $id = (int)$_POST["id"];
            $name = $_POST["name"];
            $entryCategoryDao = new EntryCategoryDAO();
            $entryCategory = $entryCategoryDao->getEntryCategoryById($id);
            if ($entryCategoryDao->entryCategoryNameAlreadyExists($name, $entryCategory)) {
                echo ApiResponse::respond(101);
                exit();
            }
            $entryCategory->setNom($name);
            $entryCategoryDao->update($entryCategory);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("fundEntryCategory.index.fundEntryCategoryUpdatedSuccessfully")} : N°{$entryCategory->getIdCategorieEntree()}({$entryCategory->getNom()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function deleteAction()
    {
        try {
            $id = (int)$_POST["id"];
            $entryCategoryDao = new EntryCategoryDAO();
            $entryCategory = $entryCategoryDao->getEntryCategoryById($id);
            if ($entryCategory->getStatut() == 0) {
                echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE);
                exit();
            }
            $entryCategory->setStatut(0);
            $entryCategoryDao->update($entryCategory);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("fundEntryCategory.index.fundEntryCategoryDeletedSuccessfully")} : N°{$entryCategory->getIdCategorieEntree()}({$entryCategory->getNom()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }
}
