<?php

include APPLICATION_PATH . '/includes/daos.php';

class HomeController extends Zend_Controller_Action
{
    private $translator;

    public function init()
    {
        session_start();
        SubscriptionChecker::checkSubscriptionValidity($this);
        if (isset($_SESSION[SESSION_USER_ID]) && $_SESSION[SESSION_USER_ID] != null) {
            $_SESSION[LAST_ACTION_DATE] = new DateTime();
            LogDAO::updateLastActionDateForCurrentUser();
            $this->translator = Translator::init();
        } else {
            if ($this->getRequest()->isPost()) {
                echo ApiResponse::respond(SESSION_EXPIRED_CODE);
                exit();
            } else {
                $this->_helper->redirector('index', 'Index');
            }
        }
    }

    public function indexAction()
    {
        $currentUser = LogDAO::getCurrentUser();
        $currentUserPointOfSale = $currentUser->getIdPointVente();
        $customerInvoiceDao = new CustomerInvoiceDAO();
        $cashMovementDao = new CashMovementDAO();
        $appointmentActivityDao = new AppointmentActivityDAO();
        $dataForInvoice = [];
        $dataForInvoicePayment = [];
        $dataForCashEntry = [];
        $dataForCashExit = [];
        $fundMovementsTitle = "";
        $salesTitle = "";

        $currentDate = new DateTime();
        $beginDate = null;
        $displayChartType = TYPE_AFFICHAGE_CHART;
        if ($displayChartType) {
            if ($displayChartType == TYPE_PERIODE_CHART_JOURNALIER) {
                $beginDate = clone $currentDate;
                $fundMovementsTitle = $this->translator->_("home.index.todayFundMovements");
                $salesTitle = $this->translator->_("home.index.todaySales");
                $invoiceOccurrence = [];
                $cashEntryOccurrence = [];
                $cashOutOccurrence = [];
                $invoicePaymentOccurrence = [];
                $invoiceOccurrence["x"] = $beginDate->format(DATE_FORMAT);
                $invoicePaymentOccurrence["x"] = $beginDate->format(DATE_FORMAT);
                $cashEntryOccurrence["x"] = $beginDate->format(DATE_FORMAT);
                $cashOutOccurrence["x"] = $beginDate->format(DATE_FORMAT);
                $sumOfAmountOfInvoicesOfDay = $customerInvoiceDao->getSumOfAmountOfInvoicesOfDay($beginDate->format(DATE_FORMAT), $currentUserPointOfSale);
                $invoiceOccurrence["y"] = $sumOfAmountOfInvoicesOfDay ? $sumOfAmountOfInvoicesOfDay : 0;
                $sumOfPaymentOfInvoicesOnDay = $customerInvoiceDao->getSumOfPaymentsOfInvoicesOfDay($beginDate->format(DATE_FORMAT), $currentUserPointOfSale);
                $invoicePaymentOccurrence["y"] = $sumOfPaymentOfInvoicesOnDay ? $sumOfPaymentOfInvoicesOnDay : 0;
                $cashMovement = $cashMovementDao->getSumOfGlobalPaymentsOnDay($beginDate->format(DATE_FORMAT), $currentUserPointOfSale);
                $cashEntryOccurrence["y"] = $cashMovement["sumOfMoneyIn"] ? $cashMovement["sumOfMoneyIn"] : 0;
                $cashOutOccurrence["y"] = $cashMovement["sumOfMoneyOut"] ? $cashMovement["sumOfMoneyOut"] : 0;
                $dataForInvoice[] = $invoiceOccurrence;
                $dataForInvoicePayment[] = $invoicePaymentOccurrence;
                $dataForCashEntry[] = $cashEntryOccurrence;
                $dataForCashExit[] = $cashOutOccurrence;
            }

            if ($displayChartType == TYPE_PERIODE_CHART_HEBDOMADAIRE || $displayChartType == TYPE_PERIODE_CHART_MENSUEL) {
                if ($displayChartType == TYPE_PERIODE_CHART_HEBDOMADAIRE) {
                    $beginDate = clone $currentDate;
                    $previousSundayDate = clone $beginDate;
                    $previousSundayDate->modify("last sunday");
                    $previousSundayDate->modify("next monday");
                    $beginDate = clone $previousSundayDate;
                    $currentDate = clone $previousSundayDate->modify("next sunday");
                    $fundMovementsTitle = "{$this->translator->_("home.fundMovementsOfTheWeek")}({$beginDate->format(DATE_FORMAT_DAY_NUMERIC)} {$beginDate->format(DATE_FORMAT_MONTH_MINI)} {$beginDate->format(DATE_FORMAT_YEAR)}-{$currentDate->format(DATE_FORMAT_DAY_NUMERIC)} {$currentDate->format(DATE_FORMAT_MONTH_MINI)} {$currentDate->format(DATE_FORMAT_YEAR)})";
                    $salesTitle = "{$this->translator->_("home.salesOfTheWeek")} ({$beginDate->format(DATE_FORMAT_DAY_NUMERIC)} {$beginDate->format(DATE_FORMAT_MONTH_MINI)} {$beginDate->format(DATE_FORMAT_YEAR)}-{$currentDate->format(DATE_FORMAT_DAY_NUMERIC)} {$currentDate->format(DATE_FORMAT_MONTH_MINI)} {$currentDate->format(DATE_FORMAT_YEAR)})";
                } else {
                    $beginDate = clone $currentDate;
                    $beginDate->modify("first day of this month");
                    $currentDate->modify("last day of this month");
                    $fundMovementsTitle = "{$this->translator->_("home.fundMovementsOf")} {$beginDate->format(DATE_FORMAT_MONTH)} {$beginDate->format(DATE_FORMAT_YEAR)}";
                    $salesTitle = "{$this->translator->_("home.salesOf")} {$beginDate->format(DATE_FORMAT_MONTH)} {$beginDate->format(DATE_FORMAT_YEAR)}";
                }

                $currentTime = strtotime($currentDate->format(DATE_FORMAT));
                $beginTime = strtotime($beginDate->format(DATE_FORMAT));

                while ($beginTime <= $currentTime) {
                    $invoiceOccurrence = [];
                    $cashEntryOccurrence = [];
                    $cashOutOccurrence = [];
                    $invoicePaymentOccurrence = [];
                    $invoiceOccurrence["x"] = $beginDate->format(DATE_FORMAT);
                    $invoicePaymentOccurrence["x"] = $beginDate->format(DATE_FORMAT);
                    $cashEntryOccurrence["x"] = $beginDate->format(DATE_FORMAT);
                    $cashOutOccurrence["x"] = $beginDate->format(DATE_FORMAT);
                    $sumOfAmountOfInvoicesOfDay = $customerInvoiceDao->getSumOfAmountOfInvoicesOfDay($beginDate->format(DATE_FORMAT), $currentUserPointOfSale);
                    $invoiceOccurrence["y"] = $sumOfAmountOfInvoicesOfDay ? $sumOfAmountOfInvoicesOfDay : 0;
                    $sumOfPaymentOfInvoicesOnDay = $customerInvoiceDao->getSumOfPaymentsOfInvoicesOfDay($beginDate->format(DATE_FORMAT), $currentUserPointOfSale);
                    $invoicePaymentOccurrence["y"] = $sumOfPaymentOfInvoicesOnDay ? $sumOfPaymentOfInvoicesOnDay : 0;
                    $cashMovement = $cashMovementDao->getSumOfGlobalPaymentsOnDay($beginDate->format(DATE_FORMAT), $currentUserPointOfSale);
                    $cashEntryOccurrence["y"] = $cashMovement["sumOfMoneyIn"] ? $cashMovement["sumOfMoneyIn"] : 0;
                    $cashOutOccurrence["y"] = $cashMovement["sumOfMoneyOut"] ? $cashMovement["sumOfMoneyOut"] : 0;
                    $dataForInvoice[] = $invoiceOccurrence;
                    $dataForInvoicePayment[] = $invoicePaymentOccurrence;
                    $dataForCashEntry[] = $cashEntryOccurrence;
                    $dataForCashExit[] = $cashOutOccurrence;
                    $beginDate->modify("+1 day");
                    $beginTime = strtotime($beginDate->format(DATE_FORMAT));
                }
            }

            if ($displayChartType == TYPE_PERIODE_CHART_TRIMESTRIEL) {
                $month = (int)$currentDate->format(DATE_FORMAT_MONTH_NUMERIC);
                $year = $currentDate->format(DATE_FORMAT_YEAR);
                if ($month >= 1 && $month <= 3) {
                    $firstMonth = new DateTime("{$year}-01-01");
                    $firstMonthMoment = new \Moment\Moment($firstMonth->format(DATE_FORMAT));
                    $secondMonth = new DateTime("{$year}-02-01");
                    $secondMonthMoment = new \Moment\Moment($secondMonth->format(DATE_FORMAT));
                    $thirdMonth = new DateTime("{$year}-03-01");
                    $thirdMonthMoment = new \Moment\Moment($thirdMonth->format(DATE_FORMAT));
                    $salesTitle = "{$this->translator->_("home.salesOfTheFirstQuarterOf")} {$year}";
                    $fundMovementsTitle = "{$this->translator->_("home.fundMovementsOfTheFirstQuarterOf")} {$year}";
                }

                if ($month >= 4 && $month <= 6) {
                    $firstMonth = new DateTime("{$year}-04-01");
                    $firstMonthMoment = new \Moment\Moment($firstMonth->format(DATE_FORMAT));
                    $secondMonth = new DateTime("{$year}-05-01");
                    $secondMonthMoment = new \Moment\Moment($secondMonth->format(DATE_FORMAT));
                    $thirdMonth = new DateTime("{$year}-06-01");
                    $thirdMonthMoment = new \Moment\Moment($thirdMonth->format(DATE_FORMAT));
                    $salesTitle = "{$this->translator->_("home.salesOfTheSecondQuarterOf")} {$year}";
                    $fundMovementsTitle = "{$this->translator->_("home.fundMovementsOfTheSecondQuarterOf")} {$year}";
                }

                if ($month >= 7 && $month <= 9) {
                    $firstMonth = new DateTime("{$year}-07-01");
                    $firstMonthMoment = new \Moment\Moment($firstMonth->format(DATE_FORMAT));
                    $secondMonth = new DateTime("{$year}-08-01");
                    $secondMonthMoment = new \Moment\Moment($secondMonth->format(DATE_FORMAT));
                    $thirdMonth = new DateTime("{$year}-09-01");
                    $thirdMonthMoment = new \Moment\Moment($thirdMonth->format(DATE_FORMAT));
                    $salesTitle = "{$this->translator->_("home.salesOfTheThirdQuarterOf")} {$year}";
                    $fundMovementsTitle = "{$this->translator->_("home.fundMovementsOfTheThirdQuarterOf")} {$year}";
                }

                if ($month >= 10 && $month <= 12) {
                    $firstMonth = new DateTime("{$year}-10-01");
                    $firstMonthMoment = new \Moment\Moment($firstMonth->format(DATE_FORMAT));
                    $secondMonth = new DateTime("{$year}-11-01");
                    $secondMonthMoment = new \Moment\Moment($secondMonth->format(DATE_FORMAT));
                    $thirdMonth = new DateTime("{$year}-12-01");
                    $thirdMonthMoment = new \Moment\Moment($thirdMonth->format(DATE_FORMAT));
                    $salesTitle = "{$this->translator->_("home.salesOfTheFourthQuarterOf")} {$year}";
                    $fundMovementsTitle = "{$this->translator->_("home.fundMovementsOfTheFourthQuarterOf")} {$year}";
                }

                $firstMonthInvoiceOccurrence = [];
                $firstMonthInvoiceOccurrence["x"] = "{$firstMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $secondMonthInvoiceOccurrence = [];
                $secondMonthInvoiceOccurrence["x"] = "{$secondMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $thirdMonthInvoiceOccurrence = [];
                $thirdMonthInvoiceOccurrence["x"] = "{$thirdMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";

                $firstMonthInvoicePaymentOccurrence = [];
                $firstMonthInvoicePaymentOccurrence["x"] = "{$firstMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $secondMonthInvoicePaymentOccurrence = [];
                $secondMonthInvoicePaymentOccurrence["x"] = "{$secondMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $thirdMonthInvoicePaymentOccurrence = [];
                $thirdMonthInvoicePaymentOccurrence["x"] = "{$thirdMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";

                $firstMonthCashEntryOccurrence = [];
                $firstMonthCashEntryOccurrence["x"] = "{$firstMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $secondMonthCashEntryOccurrence = [];
                $secondMonthCashEntryOccurrence["x"] = "{$secondMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $thirdMonthCashEntryOccurrence = [];
                $thirdMonthCashEntryOccurrence["x"] = "{$thirdMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";

                $firstMonthCashOutOccurrence = [];
                $firstMonthCashOutOccurrence["x"] = "{$firstMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $secondMonthCashOutOccurrence = [];
                $secondMonthCashOutOccurrence["x"] = "{$secondMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $thirdMonthCashOutOccurrence = [];
                $thirdMonthCashOutOccurrence["x"] = "{$thirdMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";

                $sumOfAmountOfInvoicesOfFirstMonth = $customerInvoiceDao->getSumOfAmountOfInvoicesOnMonth($firstMonth, $currentUserPointOfSale);
                $sumOfAmountOfInvoicesOfSecondMonth = $customerInvoiceDao->getSumOfAmountOfInvoicesOnMonth($secondMonth, $currentUserPointOfSale);
                $sumOfAmountOfInvoicesOfThirdMonth = $customerInvoiceDao->getSumOfAmountOfInvoicesOnMonth($thirdMonth, $currentUserPointOfSale);
                $firstMonthInvoiceOccurrence["y"] = $sumOfAmountOfInvoicesOfFirstMonth ? $sumOfAmountOfInvoicesOfFirstMonth : 0;
                $secondMonthInvoiceOccurrence["y"] = $sumOfAmountOfInvoicesOfSecondMonth ? $sumOfAmountOfInvoicesOfSecondMonth : 0;
                $thirdMonthInvoiceOccurrence["y"] = $sumOfAmountOfInvoicesOfThirdMonth ? $sumOfAmountOfInvoicesOfThirdMonth : 0;

                $sumOfPaymentOfInvoicesOfFirstMonth = $customerInvoiceDao->getSumOfPaymentsOfInvoicesOnMonth($firstMonth, $currentUserPointOfSale);
                $sumOfPaymentOfInvoicesOfSecondMonth = $customerInvoiceDao->getSumOfPaymentsOfInvoicesOnMonth($secondMonth, $currentUserPointOfSale);
                $sumOfPaymentOfInvoicesOfThirdMonth = $customerInvoiceDao->getSumOfPaymentsOfInvoicesOnMonth($thirdMonth, $currentUserPointOfSale);
                $firstMonthInvoicePaymentOccurrence["y"] = $sumOfPaymentOfInvoicesOfFirstMonth ? $sumOfPaymentOfInvoicesOfFirstMonth : 0;
                $secondMonthInvoicePaymentOccurrence["y"] = $sumOfPaymentOfInvoicesOfSecondMonth ? $sumOfPaymentOfInvoicesOfSecondMonth : 0;
                $thirdMonthInvoicePaymentOccurrence["y"] = $sumOfPaymentOfInvoicesOfThirdMonth ? $sumOfPaymentOfInvoicesOfThirdMonth : 0;

                $cashMovementOfFirstMonth = $cashMovementDao->getSumOfGlobalPaymentsOnMonth($firstMonth, $currentUserPointOfSale);
                $cashMovementOfSecondMonth = $cashMovementDao->getSumOfGlobalPaymentsOnMonth($secondMonth, $currentUserPointOfSale);
                $cashMovementOfThirdMonth = $cashMovementDao->getSumOfGlobalPaymentsOnMonth($thirdMonth, $currentUserPointOfSale);
                $firstMonthCashEntryOccurrence["y"] = $cashMovementOfFirstMonth["sumOfMoneyIn"] ? $cashMovementOfFirstMonth["sumOfMoneyIn"] : 0;
                $firstMonthCashOutOccurrence["y"] = $cashMovementOfFirstMonth["sumOfMoneyOut"] ? $cashMovementOfFirstMonth["sumOfMoneyOut"] : 0;
                $secondMonthCashEntryOccurrence["y"] = $cashMovementOfSecondMonth["sumOfMoneyIn"] ? $cashMovementOfSecondMonth["sumOfMoneyIn"] : 0;
                $secondMonthCashOutOccurrence["y"] = $cashMovementOfSecondMonth["sumOfMoneyOut"] ? $cashMovementOfSecondMonth["sumOfMoneyOut"] : 0;
                $thirdMonthCashEntryOccurrence["y"] = $cashMovementOfThirdMonth["sumOfMoneyIn"] ? $cashMovementOfThirdMonth["sumOfMoneyIn"] : 0;
                $thirdMonthCashOutOccurrence["y"] = $cashMovementOfThirdMonth["sumOfMoneyOut"] ? $cashMovementOfThirdMonth["sumOfMoneyOut"] : 0;

                $dataForInvoice[] = $firstMonthInvoiceOccurrence;
                $dataForInvoice[] = $secondMonthInvoiceOccurrence;
                $dataForInvoice[] = $thirdMonthInvoiceOccurrence;

                $dataForInvoicePayment[] = $firstMonthInvoicePaymentOccurrence;
                $dataForInvoicePayment[] = $secondMonthInvoicePaymentOccurrence;
                $dataForInvoicePayment[] = $thirdMonthInvoicePaymentOccurrence;

                $dataForCashEntry[] = $firstMonthCashEntryOccurrence;
                $dataForCashEntry[] = $secondMonthCashEntryOccurrence;
                $dataForCashEntry[] = $thirdMonthCashEntryOccurrence;

                $dataForCashExit[] = $firstMonthCashOutOccurrence;
                $dataForCashExit[] = $secondMonthCashOutOccurrence;
                $dataForCashExit[] = $thirdMonthCashOutOccurrence;
            }

            if ($displayChartType == TYPE_PERIODE_CHART_SEMESTRIEL) {
                $month = (int)$currentDate->format(DATE_FORMAT_MONTH_NUMERIC);
                $year = $currentDate->format(DATE_FORMAT_YEAR);
                if ($month >= 1 && $month <= 6) {
                    $firstMonth = new DateTime("{$year}-01-01");
                    $firstMonthMoment = new \Moment\Moment($firstMonth->format(DATE_FORMAT));
                    $secondMonth = new DateTime("{$year}-02-01");
                    $secondMonthMoment = new \Moment\Moment($secondMonth->format(DATE_FORMAT));
                    $thirdMonth = new DateTime("{$year}-03-01");
                    $thirdMonthMoment = new \Moment\Moment($thirdMonth->format(DATE_FORMAT));
                    $fourthMonth = new DateTime("{$year}-04-01");
                    $fourthMonthMoment = new \Moment\Moment($fourthMonth->format(DATE_FORMAT));
                    $fifthMonth = new DateTime("{$year}-05-01");
                    $fifthMonthMoment = new \Moment\Moment($fifthMonth->format(DATE_FORMAT));
                    $sixthMonth = new DateTime("{$year}-06-01");
                    $sixthMonthMoment = new \Moment\Moment($sixthMonth->format(DATE_FORMAT));
                    $salesTitle = "{$this->translator->_("home.salesOfTheFirstHalfOf")} {$year}";
                    $fundMovementsTitle = "{$this->translator->_("home.fundMovementsOfTheFirstHalfOf")} {$year}";
                } else {
                    $firstMonth = new DateTime("{$year}-07-01");
                    $firstMonthMoment = new \Moment\Moment($firstMonth->format(DATE_FORMAT));
                    $secondMonth = new DateTime("{$year}-08-01");
                    $secondMonthMoment = new \Moment\Moment($secondMonth->format(DATE_FORMAT));
                    $thirdMonth = new DateTime("{$year}-09-01");
                    $thirdMonthMoment = new \Moment\Moment($thirdMonth->format(DATE_FORMAT));
                    $fourthMonth = new DateTime("{$year}-10-01");
                    $fourthMonthMoment = new \Moment\Moment($fourthMonth->format(DATE_FORMAT));
                    $fifthMonth = new DateTime("{$year}-11-01");
                    $fifthMonthMoment = new \Moment\Moment($fifthMonth->format(DATE_FORMAT));
                    $sixthMonth = new DateTime("{$year}-12-01");
                    $sixthMonthMoment = new \Moment\Moment($sixthMonth->format(DATE_FORMAT));
                    $salesTitle = "{$this->translator->_("home.salesOfTheSecondHalfOf")} {$year}";
                    $fundMovementsTitle = "{$this->translator->_("home.fundMovementsOfTheSecondHalfOf")} {$year}";
                }

                $firstMonthInvoiceOccurrence = [];
                $firstMonthInvoiceOccurrence["x"] = "{$firstMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $secondMonthInvoiceOccurrence = [];
                $secondMonthInvoiceOccurrence["x"] = "{$secondMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $thirdMonthInvoiceOccurrence = [];
                $thirdMonthInvoiceOccurrence["x"] = "{$thirdMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $fourthMonthInvoiceOccurrence = [];
                $fourthMonthInvoiceOccurrence["x"] = "{$fourthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $fifthMonthInvoiceOccurrence = [];
                $fifthMonthInvoiceOccurrence["x"] = "{$fifthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $sixthMonthInvoiceOccurrence = [];
                $sixthMonthInvoiceOccurrence["x"] = "{$sixthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";

                $firstMonthInvoicePaymentOccurrence = [];
                $firstMonthInvoicePaymentOccurrence["x"] = "{$firstMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $secondMonthInvoicePaymentOccurrence = [];
                $secondMonthInvoicePaymentOccurrence["x"] = "{$secondMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $thirdMonthInvoicePaymentOccurrence = [];
                $thirdMonthInvoicePaymentOccurrence["x"] = "{$thirdMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $fourthMonthInvoicePaymentOccurrence = [];
                $fourthMonthInvoicePaymentOccurrence["x"] = "{$fourthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $fifthMonthInvoicePaymentOccurrence = [];
                $fifthMonthInvoicePaymentOccurrence["x"] = "{$fifthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $sixthMonthInvoicePaymentOccurrence = [];
                $sixthMonthInvoicePaymentOccurrence["x"] = "{$sixthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";

                $firstMonthCashEntryOccurrence = [];
                $firstMonthCashEntryOccurrence["x"] = "{$firstMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $secondMonthCashEntryOccurrence = [];
                $secondMonthCashEntryOccurrence["x"] = "{$secondMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $thirdMonthCashEntryOccurrence = [];
                $thirdMonthCashEntryOccurrence["x"] = "{$thirdMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $fourthMonthCashEntryOccurrence = [];
                $fourthMonthCashEntryOccurrence["x"] = "{$fourthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $fifthMonthCashEntryOccurrence = [];
                $fifthMonthCashEntryOccurrence["x"] = "{$fifthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $sixthMonthCashEntryOccurrence = [];
                $sixthMonthCashEntryOccurrence["x"] = "{$sixthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";

                $firstMonthCashOutOccurrence = [];
                $firstMonthCashOutOccurrence["x"] = "{$firstMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $secondMonthCashOutOccurrence = [];
                $secondMonthCashOutOccurrence["x"] = "{$secondMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $thirdMonthCashOutOccurrence = [];
                $thirdMonthCashOutOccurrence["x"] = "{$thirdMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $fourthMonthCashOutOccurrence = [];
                $fourthMonthCashOutOccurrence["x"] = "{$fourthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $fifthMonthCashOutOccurrence = [];
                $fifthMonthCashOutOccurrence["x"] = "{$fifthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $sixthMonthCashOutOccurrence = [];
                $sixthMonthCashOutOccurrence["x"] = "{$sixthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";

                $sumOfAmountOfInvoicesOfFirstMonth = $customerInvoiceDao->getSumOfAmountOfInvoicesOnMonth($firstMonth, $currentUserPointOfSale);
                $sumOfAmountOfInvoicesOfSecondMonth = $customerInvoiceDao->getSumOfAmountOfInvoicesOnMonth($secondMonth, $currentUserPointOfSale);
                $sumOfAmountOfInvoicesOfThirdMonth = $customerInvoiceDao->getSumOfAmountOfInvoicesOnMonth($thirdMonth, $currentUserPointOfSale);
                $sumOfAmountOfInvoicesOfFourthMonth = $customerInvoiceDao->getSumOfAmountOfInvoicesOnMonth($fourthMonth, $currentUserPointOfSale);
                $sumOfAmountOfInvoicesOfFifthMonth = $customerInvoiceDao->getSumOfAmountOfInvoicesOnMonth($fifthMonth, $currentUserPointOfSale);
                $sumOfAmountOfInvoicesOfSixthMonth = $customerInvoiceDao->getSumOfAmountOfInvoicesOnMonth($sixthMonth, $currentUserPointOfSale);
                $firstMonthInvoiceOccurrence["y"] = $sumOfAmountOfInvoicesOfFirstMonth ? $sumOfAmountOfInvoicesOfFirstMonth : 0;
                $secondMonthInvoiceOccurrence["y"] = $sumOfAmountOfInvoicesOfSecondMonth ? $sumOfAmountOfInvoicesOfSecondMonth : 0;
                $thirdMonthInvoiceOccurrence["y"] = $sumOfAmountOfInvoicesOfThirdMonth ? $sumOfAmountOfInvoicesOfThirdMonth : 0;
                $fourthMonthInvoiceOccurrence["y"] = $sumOfAmountOfInvoicesOfFourthMonth ? $sumOfAmountOfInvoicesOfFourthMonth : 0;
                $fifthMonthInvoiceOccurrence["y"] = $sumOfAmountOfInvoicesOfFifthMonth ? $sumOfAmountOfInvoicesOfFifthMonth : 0;
                $sixthMonthInvoiceOccurrence["y"] = $sumOfAmountOfInvoicesOfSixthMonth ? $sumOfAmountOfInvoicesOfSixthMonth : 0;

                $sumOfPaymentOfInvoicesOfFirstMonth = $customerInvoiceDao->getSumOfPaymentsOfInvoicesOnMonth($firstMonth, $currentUserPointOfSale);
                $sumOfPaymentOfInvoicesOfSecondMonth = $customerInvoiceDao->getSumOfPaymentsOfInvoicesOnMonth($secondMonth, $currentUserPointOfSale);
                $sumOfPaymentOfInvoicesOfThirdMonth = $customerInvoiceDao->getSumOfPaymentsOfInvoicesOnMonth($thirdMonth, $currentUserPointOfSale);
                $sumOfPaymentOfInvoicesOfFourthMonth = $customerInvoiceDao->getSumOfPaymentsOfInvoicesOnMonth($fourthMonth, $currentUserPointOfSale);
                $sumOfPaymentOfInvoicesOfFifthMonth = $customerInvoiceDao->getSumOfPaymentsOfInvoicesOnMonth($fifthMonth, $currentUserPointOfSale);
                $sumOfPaymentOfInvoicesOfSixthMonth = $customerInvoiceDao->getSumOfPaymentsOfInvoicesOnMonth($sixthMonth, $currentUserPointOfSale);
                $firstMonthInvoicePaymentOccurrence["y"] = $sumOfPaymentOfInvoicesOfFirstMonth ? $sumOfPaymentOfInvoicesOfFirstMonth : 0;
                $secondMonthInvoicePaymentOccurrence["y"] = $sumOfPaymentOfInvoicesOfSecondMonth ? $sumOfPaymentOfInvoicesOfSecondMonth : 0;
                $thirdMonthInvoicePaymentOccurrence["y"] = $sumOfPaymentOfInvoicesOfThirdMonth ? $sumOfPaymentOfInvoicesOfThirdMonth : 0;
                $fourthMonthInvoicePaymentOccurrence["y"] = $sumOfPaymentOfInvoicesOfFourthMonth ? $sumOfPaymentOfInvoicesOfFourthMonth : 0;
                $fifthMonthInvoicePaymentOccurrence["y"] = $sumOfPaymentOfInvoicesOfFifthMonth ? $sumOfPaymentOfInvoicesOfFifthMonth : 0;
                $sixthMonthInvoicePaymentOccurrence["y"] = $sumOfPaymentOfInvoicesOfSixthMonth ? $sumOfPaymentOfInvoicesOfSixthMonth : 0;

                $cashMovementOfFirstMonth = $cashMovementDao->getSumOfGlobalPaymentsOnMonth($firstMonth, $currentUserPointOfSale);
                $cashMovementOfSecondMonth = $cashMovementDao->getSumOfGlobalPaymentsOnMonth($secondMonth, $currentUserPointOfSale);
                $cashMovementOfThirdMonth = $cashMovementDao->getSumOfGlobalPaymentsOnMonth($thirdMonth, $currentUserPointOfSale);
                $cashMovementOfFourthMonth = $cashMovementDao->getSumOfGlobalPaymentsOnMonth($fourthMonth, $currentUserPointOfSale);
                $cashMovementOfFifthMonth = $cashMovementDao->getSumOfGlobalPaymentsOnMonth($fifthMonth, $currentUserPointOfSale);
                $cashMovementOfSixthMonth = $cashMovementDao->getSumOfGlobalPaymentsOnMonth($sixthMonth, $currentUserPointOfSale);
                $firstMonthCashEntryOccurrence["y"] = $cashMovementOfFirstMonth["sumOfMoneyIn"] ? $cashMovementOfFirstMonth["sumOfMoneyIn"] : 0;
                $firstMonthCashOutOccurrence["y"] = $cashMovementOfFirstMonth["sumOfMoneyOut"] ? $cashMovementOfFirstMonth["sumOfMoneyOut"] : 0;
                $secondMonthCashEntryOccurrence["y"] = $cashMovementOfSecondMonth["sumOfMoneyIn"] ? $cashMovementOfSecondMonth["sumOfMoneyIn"] : 0;
                $secondMonthCashOutOccurrence["y"] = $cashMovementOfSecondMonth["sumOfMoneyOut"] ? $cashMovementOfSecondMonth["sumOfMoneyOut"] : 0;
                $thirdMonthCashEntryOccurrence["y"] = $cashMovementOfThirdMonth["sumOfMoneyIn"] ? $cashMovementOfThirdMonth["sumOfMoneyIn"] : 0;
                $thirdMonthCashOutOccurrence["y"] = $cashMovementOfThirdMonth["sumOfMoneyOut"] ? $cashMovementOfThirdMonth["sumOfMoneyOut"] : 0;
                $fourthMonthCashEntryOccurrence["y"] = $cashMovementOfFourthMonth["sumOfMoneyIn"] ? $cashMovementOfFourthMonth["sumOfMoneyIn"] : 0;
                $fourthMonthCashOutOccurrence["y"] = $cashMovementOfFourthMonth["sumOfMoneyOut"] ? $cashMovementOfFourthMonth["sumOfMoneyOut"] : 0;
                $fifthMonthCashEntryOccurrence["y"] = $cashMovementOfFifthMonth["sumOfMoneyIn"] ? $cashMovementOfFifthMonth["sumOfMoneyIn"] : 0;
                $fifthMonthCashOutOccurrence["y"] = $cashMovementOfFifthMonth["sumOfMoneyOut"] ? $cashMovementOfFifthMonth["sumOfMoneyOut"] : 0;
                $sixthMonthCashEntryOccurrence["y"] = $cashMovementOfSixthMonth["sumOfMoneyIn"] ? $cashMovementOfSixthMonth["sumOfMoneyIn"] : 0;
                $sixthMonthCashOutOccurrence["y"] = $cashMovementOfSixthMonth["sumOfMoneyOut"] ? $cashMovementOfSixthMonth["sumOfMoneyOut"] : 0;

                $dataForInvoice[] = $firstMonthInvoiceOccurrence;
                $dataForInvoice[] = $secondMonthInvoiceOccurrence;
                $dataForInvoice[] = $thirdMonthInvoiceOccurrence;
                $dataForInvoice[] = $fourthMonthInvoiceOccurrence;
                $dataForInvoice[] = $fifthMonthInvoiceOccurrence;
                $dataForInvoice[] = $sixthMonthInvoiceOccurrence;

                $dataForInvoicePayment[] = $firstMonthInvoicePaymentOccurrence;
                $dataForInvoicePayment[] = $secondMonthInvoicePaymentOccurrence;
                $dataForInvoicePayment[] = $thirdMonthInvoicePaymentOccurrence;
                $dataForInvoicePayment[] = $fourthMonthInvoicePaymentOccurrence;
                $dataForInvoicePayment[] = $fifthMonthInvoicePaymentOccurrence;
                $dataForInvoicePayment[] = $sixthMonthInvoicePaymentOccurrence;

                $dataForCashEntry[] = $firstMonthCashEntryOccurrence;
                $dataForCashEntry[] = $secondMonthCashEntryOccurrence;
                $dataForCashEntry[] = $thirdMonthCashEntryOccurrence;
                $dataForCashEntry[] = $fourthMonthCashEntryOccurrence;
                $dataForCashEntry[] = $fifthMonthCashEntryOccurrence;
                $dataForCashEntry[] = $sixthMonthCashEntryOccurrence;

                $dataForCashExit[] = $firstMonthCashOutOccurrence;
                $dataForCashExit[] = $secondMonthCashOutOccurrence;
                $dataForCashExit[] = $thirdMonthCashOutOccurrence;
                $dataForCashExit[] = $fourthMonthCashOutOccurrence;
                $dataForCashExit[] = $fifthMonthCashOutOccurrence;
                $dataForCashExit[] = $sixthMonthCashOutOccurrence;
            }

            if ($displayChartType == TYPE_PERIODE_CHART_ANNUEL) {
                $year = $currentDate->format(DATE_FORMAT_YEAR);
                $firstMonth = new DateTime("{$year}-01-01");
                $firstMonthMoment = new \Moment\Moment($firstMonth->format(DATE_FORMAT));
                $secondMonth = new DateTime("{$year}-02-01");
                $secondMonthMoment = new \Moment\Moment($secondMonth->format(DATE_FORMAT));
                $thirdMonth = new DateTime("{$year}-03-01");
                $thirdMonthMoment = new \Moment\Moment($thirdMonth->format(DATE_FORMAT));
                $fourthMonth = new DateTime("{$year}-04-01");
                $fourthMonthMoment = new \Moment\Moment($fourthMonth->format(DATE_FORMAT));
                $fifthMonth = new DateTime("{$year}-05-01");
                $fifthMonthMoment = new \Moment\Moment($fifthMonth->format(DATE_FORMAT));
                $sixthMonth = new DateTime("{$year}-06-01");
                $sixthMonthMoment = new \Moment\Moment($sixthMonth->format(DATE_FORMAT));
                $seventhMonth = new DateTime("{$year}-07-01");
                $seventhMonthMoment = new \Moment\Moment($seventhMonth->format(DATE_FORMAT));
                $eigthMonth = new DateTime("{$year}-08-01");
                $eigthMonthMoment = new \Moment\Moment($eigthMonth->format(DATE_FORMAT));
                $ninthMonth = new DateTime("{$year}-09-01");
                $ninthMonthMoment = new \Moment\Moment($ninthMonth->format(DATE_FORMAT));
                $tenthMonth = new DateTime("{$year}-10-01");
                $tenthMonthMoment = new \Moment\Moment($tenthMonth->format(DATE_FORMAT));
                $eleventhMonth = new DateTime("{$year}-11-01");
                $eleventhMonthMoment = new \Moment\Moment($eleventhMonth->format(DATE_FORMAT));
                $twelfthMonth = new DateTime("{$year}-12-01");
                $twelfthMonthMoment = new \Moment\Moment($twelfthMonth->format(DATE_FORMAT));
                $salesTitle = "{$this->translator->_("home.salesOfYear")} {$year}";
                $fundMovementsTitle = "{$this->translator->_("home.fundMovementsOfYear")} {$year}";

                $firstMonthInvoiceOccurrence = [];
                $firstMonthInvoiceOccurrence["x"] = "{$firstMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $secondMonthInvoiceOccurrence = [];
                $secondMonthInvoiceOccurrence["x"] = "{$secondMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $thirdMonthInvoiceOccurrence = [];
                $thirdMonthInvoiceOccurrence["x"] = "{$thirdMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $fourthMonthInvoiceOccurrence = [];
                $fourthMonthInvoiceOccurrence["x"] = "{$fourthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $fifthMonthInvoiceOccurrence = [];
                $fifthMonthInvoiceOccurrence["x"] = "{$fifthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $sixthMonthInvoiceOccurrence = [];
                $sixthMonthInvoiceOccurrence["x"] = "{$sixthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $seventhMonthInvoiceOccurrence = [];
                $seventhMonthInvoiceOccurrence["x"] = "{$seventhMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $eigthMonthInvoiceOccurrence = [];
                $eigthMonthInvoiceOccurrence["x"] = "{$eigthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $ninthMonthInvoiceOccurrence = [];
                $ninthMonthInvoiceOccurrence["x"] = "{$ninthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $tenthMonthInvoiceOccurrence = [];
                $tenthMonthInvoiceOccurrence["x"] = "{$tenthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $eleventhMonthInvoiceOccurrence = [];
                $eleventhMonthInvoiceOccurrence["x"] = "{$eleventhMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $twelfthMonthInvoiceOccurrence = [];
                $twelfthMonthInvoiceOccurrence["x"] = "{$twelfthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";

                $firstMonthInvoicePaymentOccurrence = [];
                $firstMonthInvoicePaymentOccurrence["x"] = "{$firstMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $secondMonthInvoicePaymentOccurrence = [];
                $secondMonthInvoicePaymentOccurrence["x"] = "{$secondMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $thirdMonthInvoicePaymentOccurrence = [];
                $thirdMonthInvoicePaymentOccurrence["x"] = "{$thirdMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $fourthMonthInvoicePaymentOccurrence = [];
                $fourthMonthInvoicePaymentOccurrence["x"] = "{$fourthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $fifthMonthInvoicePaymentOccurrence = [];
                $fifthMonthInvoicePaymentOccurrence["x"] = "{$fifthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $sixthMonthInvoicePaymentOccurrence = [];
                $sixthMonthInvoicePaymentOccurrence["x"] = "{$sixthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $seventhMonthInvoicePaymentOccurrence = [];
                $seventhMonthInvoicePaymentOccurrence["x"] = "{$seventhMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $eigthMonthInvoicePaymentOccurrence = [];
                $eigthMonthInvoicePaymentOccurrence["x"] = "{$eigthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $ninthMonthInvoicePaymentOccurrence = [];
                $ninthMonthInvoicePaymentOccurrence["x"] = "{$ninthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $tenthMonthInvoicePaymentOccurrence = [];
                $tenthMonthInvoicePaymentOccurrence["x"] = "{$tenthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $eleventhMonthInvoicePaymentOccurrence = [];
                $eleventhMonthInvoicePaymentOccurrence["x"] = "{$eleventhMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $twelfthMonthInvoicePaymentOccurrence = [];
                $twelfthMonthInvoicePaymentOccurrence["x"] = "{$twelfthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";

                $firstMonthCashEntryOccurrence = [];
                $firstMonthCashEntryOccurrence["x"] = "{$firstMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $secondMonthCashEntryOccurrence = [];
                $secondMonthCashEntryOccurrence["x"] = "{$secondMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $thirdMonthCashEntryOccurrence = [];
                $thirdMonthCashEntryOccurrence["x"] = "{$thirdMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $fourthMonthCashEntryOccurrence = [];
                $fourthMonthCashEntryOccurrence["x"] = "{$fourthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $fifthMonthCashEntryOccurrence = [];
                $fifthMonthCashEntryOccurrence["x"] = "{$fifthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $sixthMonthCashEntryOccurrence = [];
                $sixthMonthCashEntryOccurrence["x"] = "{$sixthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $seventhMonthCashEntryOccurrence = [];
                $seventhMonthCashEntryOccurrence["x"] = "{$seventhMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $eigthMonthCashEntryOccurrence = [];
                $eigthMonthCashEntryOccurrence["x"] = "{$eigthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $ninthMonthCashEntryOccurrence = [];
                $ninthMonthCashEntryOccurrence["x"] = "{$ninthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $tenthMonthCashEntryOccurrence = [];
                $tenthMonthCashEntryOccurrence["x"] = "{$tenthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $eleventhMonthCashEntryOccurrence = [];
                $eleventhMonthCashEntryOccurrence["x"] = "{$eleventhMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $twelfthMonthCashEntryOccurrence = [];
                $twelfthMonthCashEntryOccurrence["x"] = "{$twelfthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";

                $firstMonthCashOutOccurrence = [];
                $firstMonthCashOutOccurrence["x"] = "{$firstMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $secondMonthCashOutOccurrence = [];
                $secondMonthCashOutOccurrence["x"] = "{$secondMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $thirdMonthCashOutOccurrence = [];
                $thirdMonthCashOutOccurrence["x"] = "{$thirdMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $fourthMonthCashOutOccurrence = [];
                $fourthMonthCashOutOccurrence["x"] = "{$fourthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $fifthMonthCashOutOccurrence = [];
                $fifthMonthCashOutOccurrence["x"] = "{$fifthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $sixthMonthCashOutOccurrence = [];
                $sixthMonthCashOutOccurrence["x"] = "{$sixthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $seventhMonthCashOutOccurrence = [];
                $seventhMonthCashOutOccurrence["x"] = "{$seventhMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $eigthMonthCashOutOccurrence = [];
                $eigthMonthCashOutOccurrence["x"] = "{$eigthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $ninthMonthCashOutOccurrence = [];
                $ninthMonthCashOutOccurrence["x"] = "{$ninthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $tenthMonthCashOutOccurrence = [];
                $tenthMonthCashOutOccurrence["x"] = "{$tenthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $eleventhMonthCashOutOccurrence = [];
                $eleventhMonthCashOutOccurrence["x"] = "{$eleventhMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                $twelfthMonthCashOutOccurrence = [];
                $twelfthMonthCashOutOccurrence["x"] = "{$twelfthMonthMoment->format(DATE_FORMAT_MONTH)} {$year}";

                $sumOfAmountOfInvoicesOfFirstMonth = $customerInvoiceDao->getSumOfAmountOfInvoicesOnMonth($firstMonth, $currentUserPointOfSale);
                $sumOfAmountOfInvoicesOfSecondMonth = $customerInvoiceDao->getSumOfAmountOfInvoicesOnMonth($secondMonth, $currentUserPointOfSale);
                $sumOfAmountOfInvoicesOfThirdMonth = $customerInvoiceDao->getSumOfAmountOfInvoicesOnMonth($thirdMonth, $currentUserPointOfSale);
                $sumOfAmountOfInvoicesOfFourthMonth = $customerInvoiceDao->getSumOfAmountOfInvoicesOnMonth($fourthMonth, $currentUserPointOfSale);
                $sumOfAmountOfInvoicesOfFifthMonth = $customerInvoiceDao->getSumOfAmountOfInvoicesOnMonth($fifthMonth, $currentUserPointOfSale);
                $sumOfAmountOfInvoicesOfSixthMonth = $customerInvoiceDao->getSumOfAmountOfInvoicesOnMonth($sixthMonth, $currentUserPointOfSale);
                $sumOfAmountOfInvoicesOfSeventhMonth = $customerInvoiceDao->getSumOfAmountOfInvoicesOnMonth($seventhMonth, $currentUserPointOfSale);
                $sumOfAmountOfInvoicesOfEigthMonth = $customerInvoiceDao->getSumOfAmountOfInvoicesOnMonth($eigthMonth, $currentUserPointOfSale);
                $sumOfAmountOfInvoicesOfNinthMonth = $customerInvoiceDao->getSumOfAmountOfInvoicesOnMonth($ninthMonth, $currentUserPointOfSale);
                $sumOfAmountOfInvoicesOfTenthMonth = $customerInvoiceDao->getSumOfAmountOfInvoicesOnMonth($tenthMonth, $currentUserPointOfSale);
                $sumOfAmountOfInvoicesOfEleventhMonth = $customerInvoiceDao->getSumOfAmountOfInvoicesOnMonth($eleventhMonth, $currentUserPointOfSale);
                $sumOfAmountOfInvoicesOfTwelfthMonth = $customerInvoiceDao->getSumOfAmountOfInvoicesOnMonth($twelfthMonth, $currentUserPointOfSale);
                $firstMonthInvoiceOccurrence["y"] = $sumOfAmountOfInvoicesOfFirstMonth ? $sumOfAmountOfInvoicesOfFirstMonth : 0;
                $secondMonthInvoiceOccurrence["y"] = $sumOfAmountOfInvoicesOfSecondMonth ? $sumOfAmountOfInvoicesOfSecondMonth : 0;
                $thirdMonthInvoiceOccurrence["y"] = $sumOfAmountOfInvoicesOfThirdMonth ? $sumOfAmountOfInvoicesOfThirdMonth : 0;
                $fourthMonthInvoiceOccurrence["y"] = $sumOfAmountOfInvoicesOfFourthMonth ? $sumOfAmountOfInvoicesOfFourthMonth : 0;
                $fifthMonthInvoiceOccurrence["y"] = $sumOfAmountOfInvoicesOfFifthMonth ? $sumOfAmountOfInvoicesOfFifthMonth : 0;
                $sixthMonthInvoiceOccurrence["y"] = $sumOfAmountOfInvoicesOfSixthMonth ? $sumOfAmountOfInvoicesOfSixthMonth : 0;
                $seventhMonthInvoiceOccurrence["y"] = $sumOfAmountOfInvoicesOfSeventhMonth ? $sumOfAmountOfInvoicesOfSeventhMonth : 0;
                $eigthMonthInvoiceOccurrence["y"] = $sumOfAmountOfInvoicesOfEigthMonth ? $sumOfAmountOfInvoicesOfEigthMonth : 0;
                $ninthMonthInvoiceOccurrence["y"] = $sumOfAmountOfInvoicesOfNinthMonth ? $sumOfAmountOfInvoicesOfNinthMonth : 0;
                $tenthMonthInvoiceOccurrence["y"] = $sumOfAmountOfInvoicesOfTenthMonth ? $sumOfAmountOfInvoicesOfTenthMonth : 0;
                $eleventhMonthInvoiceOccurrence["y"] = $sumOfAmountOfInvoicesOfEleventhMonth ? $sumOfAmountOfInvoicesOfEleventhMonth : 0;
                $twelfthMonthInvoiceOccurrence["y"] = $sumOfAmountOfInvoicesOfTwelfthMonth ? $sumOfAmountOfInvoicesOfTwelfthMonth : 0;

                $sumOfPaymentOfInvoicesOfFirstMonth = $customerInvoiceDao->getSumOfPaymentsOfInvoicesOnMonth($firstMonth, $currentUserPointOfSale);
                $sumOfPaymentOfInvoicesOfSecondMonth = $customerInvoiceDao->getSumOfPaymentsOfInvoicesOnMonth($secondMonth, $currentUserPointOfSale);
                $sumOfPaymentOfInvoicesOfThirdMonth = $customerInvoiceDao->getSumOfPaymentsOfInvoicesOnMonth($thirdMonth, $currentUserPointOfSale);
                $sumOfPaymentOfInvoicesOfFourthMonth = $customerInvoiceDao->getSumOfPaymentsOfInvoicesOnMonth($fourthMonth, $currentUserPointOfSale);
                $sumOfPaymentOfInvoicesOfFifthMonth = $customerInvoiceDao->getSumOfPaymentsOfInvoicesOnMonth($fifthMonth, $currentUserPointOfSale);
                $sumOfPaymentOfInvoicesOfSixthMonth = $customerInvoiceDao->getSumOfPaymentsOfInvoicesOnMonth($sixthMonth, $currentUserPointOfSale);
                $sumOfPaymentOfInvoicesOfSeventhMonth = $customerInvoiceDao->getSumOfPaymentsOfInvoicesOnMonth($seventhMonth, $currentUserPointOfSale);
                $sumOfPaymentOfInvoicesOfEigthMonth = $customerInvoiceDao->getSumOfPaymentsOfInvoicesOnMonth($eigthMonth, $currentUserPointOfSale);
                $sumOfPaymentOfInvoicesOfNinthMonth = $customerInvoiceDao->getSumOfPaymentsOfInvoicesOnMonth($ninthMonth, $currentUserPointOfSale);
                $sumOfPaymentOfInvoicesOfTenthMonth = $customerInvoiceDao->getSumOfPaymentsOfInvoicesOnMonth($tenthMonth, $currentUserPointOfSale);
                $sumOfPaymentOfInvoicesOfEleventhMonth = $customerInvoiceDao->getSumOfPaymentsOfInvoicesOnMonth($eleventhMonth, $currentUserPointOfSale);
                $sumOfPaymentOfInvoicesOfTwelfthMonth = $customerInvoiceDao->getSumOfPaymentsOfInvoicesOnMonth($twelfthMonth, $currentUserPointOfSale);
                $firstMonthInvoicePaymentOccurrence["y"] = $sumOfPaymentOfInvoicesOfFirstMonth ? $sumOfPaymentOfInvoicesOfFirstMonth : 0;
                $secondMonthInvoicePaymentOccurrence["y"] = $sumOfPaymentOfInvoicesOfSecondMonth ? $sumOfPaymentOfInvoicesOfSecondMonth : 0;
                $thirdMonthInvoicePaymentOccurrence["y"] = $sumOfPaymentOfInvoicesOfThirdMonth ? $sumOfPaymentOfInvoicesOfThirdMonth : 0;
                $fourthMonthInvoicePaymentOccurrence["y"] = $sumOfPaymentOfInvoicesOfFourthMonth ? $sumOfPaymentOfInvoicesOfFourthMonth : 0;
                $fifthMonthInvoicePaymentOccurrence["y"] = $sumOfPaymentOfInvoicesOfFifthMonth ? $sumOfPaymentOfInvoicesOfFifthMonth : 0;
                $sixthMonthInvoicePaymentOccurrence["y"] = $sumOfPaymentOfInvoicesOfSixthMonth ? $sumOfPaymentOfInvoicesOfSixthMonth : 0;
                $seventhMonthInvoicePaymentOccurrence["y"] = $sumOfPaymentOfInvoicesOfSeventhMonth ? $sumOfPaymentOfInvoicesOfSeventhMonth : 0;
                $eigthMonthInvoicePaymentOccurrence["y"] = $sumOfPaymentOfInvoicesOfEigthMonth ? $sumOfPaymentOfInvoicesOfEigthMonth : 0;
                $ninthMonthInvoicePaymentOccurrence["y"] = $sumOfPaymentOfInvoicesOfNinthMonth ? $sumOfPaymentOfInvoicesOfNinthMonth : 0;
                $tenthMonthInvoicePaymentOccurrence["y"] = $sumOfPaymentOfInvoicesOfTenthMonth ? $sumOfPaymentOfInvoicesOfTenthMonth : 0;
                $eleventhMonthInvoicePaymentOccurrence["y"] = $sumOfPaymentOfInvoicesOfEleventhMonth ? $sumOfPaymentOfInvoicesOfEleventhMonth : 0;
                $twelfthMonthInvoicePaymentOccurrence["y"] = $sumOfPaymentOfInvoicesOfTwelfthMonth ? $sumOfPaymentOfInvoicesOfTwelfthMonth : 0;

                $cashMovementOfFirstMonth = $cashMovementDao->getSumOfGlobalPaymentsOnMonth($firstMonth, $currentUserPointOfSale);
                $cashMovementOfSecondMonth = $cashMovementDao->getSumOfGlobalPaymentsOnMonth($secondMonth, $currentUserPointOfSale);
                $cashMovementOfThirdMonth = $cashMovementDao->getSumOfGlobalPaymentsOnMonth($thirdMonth, $currentUserPointOfSale);
                $cashMovementOfFourthMonth = $cashMovementDao->getSumOfGlobalPaymentsOnMonth($fourthMonth, $currentUserPointOfSale);
                $cashMovementOfFifthMonth = $cashMovementDao->getSumOfGlobalPaymentsOnMonth($fifthMonth, $currentUserPointOfSale);
                $cashMovementOfSixthMonth = $cashMovementDao->getSumOfGlobalPaymentsOnMonth($sixthMonth, $currentUserPointOfSale);
                $cashMovementOfSeventhMonth = $cashMovementDao->getSumOfGlobalPaymentsOnMonth($seventhMonth, $currentUserPointOfSale);
                $cashMovementOfEigthMonth = $cashMovementDao->getSumOfGlobalPaymentsOnMonth($eigthMonth, $currentUserPointOfSale);
                $cashMovementOfNinthMonth = $cashMovementDao->getSumOfGlobalPaymentsOnMonth($ninthMonth, $currentUserPointOfSale);
                $cashMovementOfTenthMonth = $cashMovementDao->getSumOfGlobalPaymentsOnMonth($tenthMonth, $currentUserPointOfSale);
                $cashMovementOfEleventhMonth = $cashMovementDao->getSumOfGlobalPaymentsOnMonth($eleventhMonth, $currentUserPointOfSale);
                $cashMovementOfTwelfthMonth = $cashMovementDao->getSumOfGlobalPaymentsOnMonth($twelfthMonth, $currentUserPointOfSale);
                $firstMonthCashEntryOccurrence["y"] = $cashMovementOfFirstMonth["sumOfMoneyIn"] ? $cashMovementOfFirstMonth["sumOfMoneyIn"] : 0;
                $firstMonthCashOutOccurrence["y"] = $cashMovementOfFirstMonth["sumOfMoneyOut"] ? $cashMovementOfFirstMonth["sumOfMoneyOut"] : 0;
                $secondMonthCashEntryOccurrence["y"] = $cashMovementOfSecondMonth["sumOfMoneyIn"] ? $cashMovementOfSecondMonth["sumOfMoneyIn"] : 0;
                $secondMonthCashOutOccurrence["y"] = $cashMovementOfSecondMonth["sumOfMoneyOut"] ? $cashMovementOfSecondMonth["sumOfMoneyOut"] : 0;
                $thirdMonthCashEntryOccurrence["y"] = $cashMovementOfThirdMonth["sumOfMoneyIn"] ? $cashMovementOfThirdMonth["sumOfMoneyIn"] : 0;
                $thirdMonthCashOutOccurrence["y"] = $cashMovementOfThirdMonth["sumOfMoneyOut"] ? $cashMovementOfThirdMonth["sumOfMoneyOut"] : 0;
                $fourthMonthCashEntryOccurrence["y"] = $cashMovementOfFourthMonth["sumOfMoneyIn"] ? $cashMovementOfFourthMonth["sumOfMoneyIn"] : 0;
                $fourthMonthCashOutOccurrence["y"] = $cashMovementOfFourthMonth["sumOfMoneyOut"] ? $cashMovementOfFourthMonth["sumOfMoneyOut"] : 0;
                $fifthMonthCashEntryOccurrence["y"] = $cashMovementOfFifthMonth["sumOfMoneyIn"] ? $cashMovementOfFifthMonth["sumOfMoneyIn"] : 0;
                $fifthMonthCashOutOccurrence["y"] = $cashMovementOfFifthMonth["sumOfMoneyOut"] ? $cashMovementOfFifthMonth["sumOfMoneyOut"] : 0;
                $sixthMonthCashEntryOccurrence["y"] = $cashMovementOfSixthMonth["sumOfMoneyIn"] ? $cashMovementOfSixthMonth["sumOfMoneyIn"] : 0;
                $sixthMonthCashOutOccurrence["y"] = $cashMovementOfSixthMonth["sumOfMoneyOut"] ? $cashMovementOfSixthMonth["sumOfMoneyOut"] : 0;
                $seventhMonthCashEntryOccurrence["y"] = $cashMovementOfSeventhMonth["sumOfMoneyIn"] ? $cashMovementOfSeventhMonth["sumOfMoneyIn"] : 0;
                $seventhMonthCashOutOccurrence["y"] = $cashMovementOfSeventhMonth["sumOfMoneyOut"] ? $cashMovementOfSeventhMonth["sumOfMoneyOut"] : 0;
                $eigthMonthCashEntryOccurrence["y"] = $cashMovementOfEigthMonth["sumOfMoneyIn"] ? $cashMovementOfEigthMonth["sumOfMoneyIn"] : 0;
                $eigthMonthCashOutOccurrence["y"] = $cashMovementOfEigthMonth["sumOfMoneyOut"] ? $cashMovementOfEigthMonth["sumOfMoneyOut"] : 0;
                $ninthMonthCashEntryOccurrence["y"] = $cashMovementOfNinthMonth["sumOfMoneyIn"] ? $cashMovementOfNinthMonth["sumOfMoneyIn"] : 0;
                $ninthMonthCashOutOccurrence["y"] = $cashMovementOfNinthMonth["sumOfMoneyOut"] ? $cashMovementOfNinthMonth["sumOfMoneyOut"] : 0;
                $tenthMonthCashEntryOccurrence["y"] = $cashMovementOfTenthMonth["sumOfMoneyIn"] ? $cashMovementOfTenthMonth["sumOfMoneyIn"] : 0;
                $tenthMonthCashOutOccurrence["y"] = $cashMovementOfTenthMonth["sumOfMoneyOut"] ? $cashMovementOfTenthMonth["sumOfMoneyOut"] : 0;
                $eleventhMonthCashEntryOccurrence["y"] = $cashMovementOfEleventhMonth["sumOfMoneyIn"] ? $cashMovementOfEleventhMonth["sumOfMoneyIn"] : 0;
                $eleventhMonthCashOutOccurrence["y"] = $cashMovementOfEleventhMonth["sumOfMoneyOut"] ? $cashMovementOfEleventhMonth["sumOfMoneyOut"] : 0;
                $twelfthMonthCashEntryOccurrence["y"] = $cashMovementOfTwelfthMonth["sumOfMoneyIn"] ? $cashMovementOfTwelfthMonth["sumOfMoneyIn"] : 0;
                $twelfthMonthCashOutOccurrence["y"] = $cashMovementOfTwelfthMonth["sumOfMoneyOut"] ? $cashMovementOfTwelfthMonth["sumOfMoneyOut"] : 0;

                $dataForInvoice[] = $firstMonthInvoiceOccurrence;
                $dataForInvoice[] = $secondMonthInvoiceOccurrence;
                $dataForInvoice[] = $thirdMonthInvoiceOccurrence;
                $dataForInvoice[] = $fourthMonthInvoiceOccurrence;
                $dataForInvoice[] = $fifthMonthInvoiceOccurrence;
                $dataForInvoice[] = $sixthMonthInvoiceOccurrence;
                $dataForInvoice[] = $seventhMonthInvoiceOccurrence;
                $dataForInvoice[] = $eigthMonthInvoiceOccurrence;
                $dataForInvoice[] = $ninthMonthInvoiceOccurrence;
                $dataForInvoice[] = $tenthMonthInvoiceOccurrence;
                $dataForInvoice[] = $eleventhMonthInvoiceOccurrence;
                $dataForInvoice[] = $twelfthMonthInvoiceOccurrence;

                $dataForInvoicePayment[] = $firstMonthInvoicePaymentOccurrence;
                $dataForInvoicePayment[] = $secondMonthInvoicePaymentOccurrence;
                $dataForInvoicePayment[] = $thirdMonthInvoicePaymentOccurrence;
                $dataForInvoicePayment[] = $fourthMonthInvoicePaymentOccurrence;
                $dataForInvoicePayment[] = $fifthMonthInvoicePaymentOccurrence;
                $dataForInvoicePayment[] = $sixthMonthInvoicePaymentOccurrence;
                $dataForInvoicePayment[] = $seventhMonthInvoicePaymentOccurrence;
                $dataForInvoicePayment[] = $eigthMonthInvoicePaymentOccurrence;
                $dataForInvoicePayment[] = $ninthMonthInvoicePaymentOccurrence;
                $dataForInvoicePayment[] = $tenthMonthInvoicePaymentOccurrence;
                $dataForInvoicePayment[] = $eleventhMonthInvoicePaymentOccurrence;
                $dataForInvoicePayment[] = $twelfthMonthInvoicePaymentOccurrence;

                $dataForCashEntry[] = $firstMonthCashEntryOccurrence;
                $dataForCashEntry[] = $secondMonthCashEntryOccurrence;
                $dataForCashEntry[] = $thirdMonthCashEntryOccurrence;
                $dataForCashEntry[] = $fourthMonthCashEntryOccurrence;
                $dataForCashEntry[] = $fifthMonthCashEntryOccurrence;
                $dataForCashEntry[] = $sixthMonthCashEntryOccurrence;
                $dataForCashEntry[] = $seventhMonthCashEntryOccurrence;
                $dataForCashEntry[] = $eigthMonthCashEntryOccurrence;
                $dataForCashEntry[] = $ninthMonthCashEntryOccurrence;
                $dataForCashEntry[] = $tenthMonthCashEntryOccurrence;
                $dataForCashEntry[] = $eleventhMonthCashEntryOccurrence;
                $dataForCashEntry[] = $twelfthMonthCashEntryOccurrence;

                $dataForCashExit[] = $firstMonthCashOutOccurrence;
                $dataForCashExit[] = $secondMonthCashOutOccurrence;
                $dataForCashExit[] = $thirdMonthCashOutOccurrence;
                $dataForCashExit[] = $fourthMonthCashOutOccurrence;
                $dataForCashExit[] = $fifthMonthCashOutOccurrence;
                $dataForCashExit[] = $sixthMonthCashOutOccurrence;
                $dataForCashExit[] = $seventhMonthCashOutOccurrence;
                $dataForCashExit[] = $eigthMonthCashOutOccurrence;
                $dataForCashExit[] = $ninthMonthCashOutOccurrence;
                $dataForCashExit[] = $tenthMonthCashOutOccurrence;
                $dataForCashExit[] = $eleventhMonthCashOutOccurrence;
                $dataForCashExit[] = $twelfthMonthCashOutOccurrence;
            }
        } else {
            $beginDate = clone $currentDate;
            $previousSundayDate = clone $beginDate;
            $previousSundayDate->modify("last sunday");
            $previousSundayDate->modify("next monday");
            $beginDate = clone $previousSundayDate;
            $currentDate = clone $previousSundayDate->modify("next sunday");

            $currentTime = strtotime($currentDate->format(DATE_FORMAT));
            $beginTime = strtotime($beginDate->format(DATE_FORMAT));

            while ($beginTime <= $currentTime) {
                $invoiceOccurrence = [];
                $cashEntryOccurrence = [];
                $cashOutOccurrence = [];
                $invoicePaymentOccurrence = [];
                $invoiceOccurrence["x"] = $beginDate->format(DATE_FORMAT);
                $invoicePaymentOccurrence["x"] = $beginDate->format(DATE_FORMAT);
                $cashEntryOccurrence["x"] = $beginDate->format(DATE_FORMAT);
                $cashOutOccurrence["x"] = $beginDate->format(DATE_FORMAT);
                $sumOfAmountOfInvoicesOfDay = $customerInvoiceDao->getSumOfAmountOfInvoicesOfDay($beginDate->format(DATE_FORMAT), $currentUserPointOfSale);
                $invoiceOccurrence["y"] = $sumOfAmountOfInvoicesOfDay ? $sumOfAmountOfInvoicesOfDay : 0;
                $sumOfPaymentOfInvoicesOnDay = $customerInvoiceDao->getSumOfPaymentsOfInvoicesOfDay($beginDate->format(DATE_FORMAT), $currentUserPointOfSale);
                $invoicePaymentOccurrence["y"] = $sumOfPaymentOfInvoicesOnDay ? $sumOfPaymentOfInvoicesOnDay : 0;
                $cashMovement = $cashMovementDao->getSumOfGlobalPaymentsOnDay($beginDate->format(DATE_FORMAT), $currentUserPointOfSale);
                $cashEntryOccurrence["y"] = $cashMovement["sumOfMoneyIn"] ? $cashMovement["sumOfMoneyIn"] : 0;
                $cashOutOccurrence["y"] = $cashMovement["sumOfMoneyOut"] ? $cashMovement["sumOfMoneyOut"] : 0;
                $dataForInvoice[] = $invoiceOccurrence;
                $dataForInvoicePayment[] = $invoicePaymentOccurrence;
                $dataForCashEntry[] = $cashEntryOccurrence;
                $dataForCashExit[] = $cashOutOccurrence;
                $beginDate->modify("+1 day");
                $beginTime = strtotime($beginDate->format(DATE_FORMAT));
            }
        }

        $storeDao = new StoreDAO();
        $stores = [];
        if ($currentUser->getIdPointVente() == null || in_array($currentUser->getIdProfil()->getCode(), IDENTIFIED_PROFILES_CODES)) {
            $stores = $storeDao->getAllStores();
        } else {
            $stores[] = $currentUser->getIdPointVente()->getIdMagasin();
        }
        $superData = [];
        $superData["dataForCashEntry"] = json_encode($dataForCashEntry);
        $superData["dataForCashExit"] = json_encode($dataForCashExit);
        $superData["dataForInvoice"] = json_encode($dataForInvoice);
        $superData["dataForInvoicePayment"] = json_encode($dataForInvoicePayment);
        $appointmentActivities = $appointmentActivityDao->getAllAppointmentActivities();
        $this->view->superData = $superData;
        $this->view->stores = $stores;
        $this->view->fundMovementsTitle = $fundMovementsTitle;
        $this->view->salesTitle = $salesTitle;
        $this->view->appointmentActivities = $appointmentActivities;
        $this->view->translator = $this->translator;
    }

    public function statistiquesAction()
    {
        if (!empty($_POST)) {
            $type = $_POST["type"];
            $typePeriode = $_POST["typePeriode"];
            $debut = $_POST["debut"];
            $fin = $_POST["fin"];

            if ($type == "vente") {
                $factureDao = new CustomerInvoiceDAO();
                $paiementDao = new PaiementFactureDAO();
                $avoirDao = new CreditDAO();
                $paiementAvoirDao = new CreditPaymentDAO();
                $datasFacture = array();
                $datasPaiement = array();
                $datasAvoirs = array();
                $datasPaiementAvoir = array();
                $modePaiement = !empty($_POST["modepaiement"]) ? $_POST["modepaiement"] : null;
                if ($typePeriode == "jour") {
                    if ($debut == $fin) {
                        $title = "Ventes du " . $debut;
                    } else {
                        $title = " Ventes du " . $debut . " au " . $fin;
                    }
                    $dateDebut = new \DateTime($debut);
                    $dateFin = new \DateTime($fin);
                    $timeDebut = strtotime($debut);
                    $timeFin = strtotime($fin);
                    while ($timeDebut <= $timeFin) {
                        $factureConcerne = array();
                        $paiementConcerne = array();
                        $avoirConcerne = array();
                        $paiementAvoirConcerne = array();
                        $factureConcerne["label"] = $dateDebut->format("Y-m-d");
                        $paiementConcerne["label"] = $dateDebut->format("Y-m-d");
                        $avoirConcerne["label"] = $dateDebut->format("Y-m-d");
                        $paiementAvoirConcerne["label"] = $dateDebut->format("Y-m-d");
                        $factureConcerne["y"] = !is_null($factureDao->getSommeMontantFactureParJour($dateDebut->format("Y-m-d"))) ? $factureDao->getSommeMontantFactureParJour($dateDebut->format("Y-m-d")) : 0;
                        $paiementConcerne["y"] = !is_null($paiementDao->getSommePaiementFactureParJour($dateDebut->format("Y-m-d"), $modePaiement)) ? $paiementDao->getSommePaiementFactureParJour($dateDebut->format("Y-m-d"), $modePaiement) : 0;
                        $avoirConcerne["y"] = !is_null($avoirDao->getSommeMontantAvoirParJour($dateDebut->format("Y-m-d"))) ? $avoirDao->getSommeMontantAvoirParJour($dateDebut->format("Y-m-d")) : 0;
                        $paiementAvoirConcerne["y"] = !is_null($paiementAvoirDao->getSommePaiementAvoirParJour($dateDebut->format("Y-m-d"))) ? $paiementAvoirDao->getSommePaiementAvoirParJour($dateDebut->format("Y-m-d")) : 0;
                        $datasFacture[] = $factureConcerne;
                        $datasPaiement[] = $paiementConcerne;
                        $datasAvoirs[] = $avoirConcerne;
                        $datasPaiementAvoir[] = $paiementAvoirConcerne;
                        $dateDebut->modify("+1 day");
                        //$timeDebut = strtotime($dateDebut->format("Y-m-d"));
                    }
                }

                if ($typePeriode == "mois") {
                    $dateDebut = new \DateTime($debut);
                    $dateFin = new \DateTime($fin);
                    if ($dateDebut->format("Y-m") == $dateFin->format("Y-m")) {
                        $title = " Ventes de " . $this->mois[$dateDebut->format("m")] . " " . $dateDebut->format("Y");
                    } else {
                        $title = " Ventes de " . $this->mois[$dateDebut->format("m")] . " " . $dateDebut->format("Y") . " A " . $this->mois[$dateFin->format("m")] . " " . $dateFin->format("Y");
                    }
                    $dateDebut->modify("first day of this month");
                    $dateDebut->modify(" +6 days ");
                    $dateFin->modify("last day of this month");
                    $timeDebut = strtotime($dateDebut->format("Y-m-d"));
                    $timeFin = strtotime($dateFin->format("Y-m-d"));
                    while ($timeDebut <= $timeFin) {
                        $factureConcerne = array();
                        $paiementConcerne = array();
                        $avoirConcerne = array();
                        $paiementAvoirConcerne = array();
                        $factureConcerne["label"] = $dateDebut->format("Y-m");
                        $paiementConcerne["label"] = $dateDebut->format("Y-m");
                        $avoirConcerne["label"] = $dateDebut->format("Y-m");
                        $paiementAvoirConcerne["label"] = $dateDebut->format("Y-m");
                        $factureConcerne["y"] = !is_null($factureDao->getSommeMontantFactureParMois($dateDebut)) ? $factureDao->getSommeMontantFactureParMois($dateDebut) : 0;
                        $paiementConcerne["y"] = !is_null($paiementDao->getSommePaiementFactureParMois($dateDebut)) ? $paiementDao->getSommePaiementFactureParMois($dateDebut) : 0;
                        $avoirConcerne["y"] = !is_null($avoirDao->getSommeMontantAvoirParMois($dateDebut)) ? $avoirDao->getSommeMontantAvoirParMois($dateDebut) : 0;
                        $paiementAvoirConcerne["y"] = !is_null($paiementAvoirDao->getSommePaiementAvoirParMois($dateDebut)) ? $paiementAvoirDao->getSommePaiementAvoirParMois($dateDebut) : 0;
                        $datasFacture[] = $factureConcerne;
                        $datasPaiement[] = $paiementConcerne;
                        $datasAvoirs[] = $avoirConcerne;
                        $datasPaiementAvoir[] = $paiementAvoirConcerne;
                        $dateDebut->modify(" +1 month");
                        $timeDebut = strtotime($dateDebut->format("Y-m-d"));
                    }
                }

                if ($typePeriode == "annee") {
                    $dateDebut = new \DateTime($debut);
                    $dateFin = new \DateTime($fin);
                    if ($dateDebut->format("Y") == $dateFin->format("Y")) {
                        $title = " Ventes de " . $dateDebut->format("Y");
                    } else {
                        $title = " Ventes de " . $dateDebut->format("Y") . " A " . $dateFin->format("Y");
                    }
                    $anneeDebut = (int)$dateDebut->format("Y");
                    $anneeFin = (int)$dateFin->format("Y");
                    while ($anneeDebut <= $anneeFin) {
                        $factureConcerne = array();
                        $paiementConcerne = array();
                        $avoirConcerne = array();
                        $paiementAvoirConcerne = array();
                        $factureConcerne["label"] = $anneeDebut;
                        $paiementConcerne["label"] = $anneeDebut;
                        $avoirConcerne["label"] = $anneeDebut;
                        $avoirConcerne["label"] = $anneeDebut;
                        $factureConcerne["y"] = !is_null($factureDao->getSomeMontantFactureParAn(new \DateTime($anneeDebut . "-01-01"))) ? $factureDao->getSomeMontantFactureParAn(new \DateTime($anneeDebut . "-01-01")) : 0;
                        $paiementConcerne["y"] = !is_null($paiementDao->getSommePaiementFactureParAn(new \DateTime($anneeDebut . "-01-01"))) ? $paiementDao->getSommePaiementFactureParAn(new \DateTime($anneeDebut . "-01-01")) : 0;
                        $avoirConcerne["y"] = !is_null($avoirDao->getSomeMontantAvoirParAn(new \DateTime($anneeDebut . "-01-01"))) ? $avoirDao->getSomeMontantAvoirParAn(new \DateTime($anneeDebut . "-01-01")) : 0;
                        $paiementAvoirConcerne["y"] = !is_null($paiementAvoirDao->getSommePaiementAvoirParAn(new \DateTime($anneeDebut . "-01-01"))) ? $paiementAvoirDao->getSommePaiementAvoirParAn(new \DateTime($anneeDebut . "-01-01")) : 0;
                        $datasFacture[] = $factureConcerne;
                        $datasPaiement[] = $paiementConcerne;
                        $datasAvoirs[] = $avoirConcerne;
                        $datasPaiementAvoir[] = $paiementAvoirConcerne;
                        $anneeDebut++;
                    }
                }
                echo json_encode(array("datasFacture" => $datasFacture, "datasPaiement" => $datasPaiement, "datasPaiementAvoir" => $datasPaiementAvoir, "datasAvoir" => $datasAvoirs, "title" => $title));
            }

            if ($type == "caisse") {

                $datasCaisseEntrees = array();
                $datasCaisseSorties = array();
                $datasCaisseEspeces = array();
                $mvtCaisseDao = new CashMovementDAO();

                if ($typePeriode == "jour") {
                    if ($debut == $fin) {
                        $title = "Mouvements de fonds du " . $debut;
                    } else {
                        $title = "Mouvements de fonds du " . $debut . " au " . $fin;
                    }
                    $dateDebut = new \DateTime($debut);
                    $dateFin = new \DateTime($fin);
                    $timeDebut = strtotime($debut);
                    $timeFin = strtotime($fin);
                    while ($timeDebut <= $timeFin) {
                        $caisseEntreesConcerne = array();
                        $caisseEspecesConcerne = array();
                        $caisseSortiesConcerne = array();
                        $caisseEntreesConcerne["label"] = $dateDebut->format("Y-m-d");
                        $caisseEspecesConcerne["label"] = $dateDebut->format("Y-m-d");
                        $caisseSortiesConcerne["label"] = $dateDebut->format("Y-m-d");
                        $infosCaisse = !is_null($mvtCaisseDao->getSommePaiementCaisseParJour($dateDebut->format("Y-m-d"))) ? $mvtCaisseDao->getSommePaiementCaisseParJour($dateDebut->format("Y-m-d")) : array("entrees" => 0, "sorties" => 0);
                        $caisseEntreesConcerne["y"] = $infosCaisse["entrees"] ? $infosCaisse["entrees"] : 0;
                        //                               $caisseEspecesConcerne["y"] = $infosCaisse["especes"];
                        $caisseSortiesConcerne["y"] = $infosCaisse["sorties"] ? $infosCaisse["sorties"] : 0;
                        $datasCaisseEntrees[] = $caisseEntreesConcerne;
                        $datasCaisseSorties[] = $caisseSortiesConcerne;
                        //                               $datasCaisseEspeces[] = $caisseEspecesConcerne;
                        $dateDebut->modify("+1 day");
                        $timeDebut = strtotime($dateDebut->format("Y-m-d"));
                    }
                }

                if ($typePeriode == "mois") {
                    $dateDebut = new \DateTime($debut);
                    $dateFin = new \DateTime($fin);
                    if ($dateDebut->format("Y-m") == $dateFin->format("Y-m")) {
                        $title = " mouvements de fonds  de " . $this->mois[$dateDebut->format("m")] . " " . $dateDebut->format("Y");
                    } else {
                        $title = " mouvements de fonds  de " . $this->mois[$dateDebut->format("m")] . " " . $dateDebut->format("Y") . " A " . $this->mois[$dateFin->format("m")] . " " . $dateFin->format("Y");
                    }
                    $dateDebut->modify("first day of this month");
                    $dateDebut->modify(" +6 days ");
                    $dateFin->modify("last day of this month");
                    $timeDebut = strtotime($dateDebut->format("Y-m-d"));
                    $timeFin = strtotime($dateFin->format("Y-m-d"));
                    while ($timeDebut <= $timeFin) {
                        $caisseEntreesConcerne = array();
                        $caisseEspecesConcerne = array();
                        $caisseSortiesConcerne = array();
                        $caisseEntreesConcerne["label"] = $dateDebut->format("Y-m");
                        $caisseEspecesConcerne["label"] = $dateDebut->format("Y-m");
                        $caisseSortiesConcerne["label"] = $dateDebut->format("Y-m");
                        $infosCaisse = !is_null($mvtCaisseDao->getSommePaiementsCaisseParMois($dateDebut)) ? $mvtCaisseDao->getSommePaiementsCaisseParMois($dateDebut) : 0;
                        $caisseEntreesConcerne["y"] = $infosCaisse["entrees"];
                        //                               $caisseEspecesConcerne["y"] = $infosCaisse["especes"];
                        $caisseSortiesConcerne["y"] = $infosCaisse["sorties"];
                        $datasCaisseEntrees[] = $caisseEntreesConcerne;
                        $datasCaisseSorties[] = $caisseSortiesConcerne;
                        //                               $datasCaisseEspeces[] = $caisseEspecesConcerne;
                        $dateDebut->modify(" +1 month");
                        $timeDebut = strtotime($dateDebut->format("Y-m-d"));
                    }
                }

                if ($typePeriode == "annee") {
                    $dateDebut = new \DateTime($debut);
                    $dateFin = new \DateTime($fin);
                    if ($dateDebut->format("Y") == $dateFin->format("Y")) {
                        $title = " mouvements de fonds de " . $dateDebut->format("Y");
                    } else {
                        $title = " mouvements de fonds de " . $dateDebut->format("Y") . " A " . $dateFin->format("Y");
                    }
                    $anneeDebut = (int)$dateDebut->format("Y");
                    $anneeFin = (int)$dateFin->format("Y");
                    while ($anneeDebut <= $anneeFin) {
                        $caisseEntreesConcerne = array();
                        $caisseEspecesConcerne = array();
                        $caisseSortiesConcerne = array();
                        $caisseEntreesConcerne["label"] = $anneeDebut;
                        $caisseEspecesConcerne["label"] = $anneeDebut;
                        $caisseSortiesConcerne["label"] = $anneeDebut;
                        //$infosCaisse = !is_null($mvtCaisseDao->ObtenirSommePaiementsCaisseParAnnee(new \DateTime($anneeDebut . "-01-01")))?$mvtCaisseDao->ObtenirSommePaiementsCaisseParAnnee(new \DateTime($anneeDebut . "-01-01")) : 0;
                        $infosCaisse = !is_null($mvtCaisseDao->getSommePaiementCaisseParAn(new \DateTime($anneeDebut . "-01-01"))) ? $mvtCaisseDao->getSommePaiementCaisseParAn(new \DateTime($anneeDebut . "-01-01")) : 0;
                        $caisseEntreesConcerne["y"] = $infosCaisse["entrees"];
                        //                               $caisseEspecesConcerne["y"] = $infosCaisse["especes"];
                        $caisseSortiesConcerne["y"] = $infosCaisse["sorties"];
                        $datasCaisseEntrees[] = $caisseEntreesConcerne;
                        $datasCaisseSorties[] = $caisseSortiesConcerne;
                        //                               $datasCaisseEspeces[] = $caisseEspecesConcerne;
                        $anneeDebut++;
                    }
                }

                echo json_encode(array("datasCaisseEntrees" => $datasCaisseEntrees, "datasCaisseSorties" => $datasCaisseSorties, "datasCaisseEspeces" => $datasCaisseEspeces, "title" => $title));
            }

            if ($type == "article") {
                $ligneFactureDao = new CustomerInvoiceLineDAO();
                //$articleDao = new ArticleDAO();
                $articleDao = new ProductDAO();
                $stockDAO = new StockDAO();
                $datasArticles = array();
                if ($_POST["article"] == "tous") {
                    $listeArticle = $stockDAO->listerProduitEnStock();
                    // $listeArticle = $articleDao->listerProduit();
                    foreach ($listeArticle as $article) {
                        $articleConcerne = array();
                        //$articleConcerne["label"] = $article->getCode();
                        $article = $articleDao->getProduit((int)$article);
                        $articleConcerne["label"] = utf8_encode($article->getDesignation());
                        if ($typePeriode == "jour") {
                            if ($debut == $fin) {
                                $title = "Ventes de produit du " . $debut;
                            } else {
                                $title = "Ventes de produit du " . $debut . " au " . $fin;
                            }
                            $articleConcerne["y"] = count($ligneFactureDao->obtenirQteArticleVenduIntervalle($debut, $fin, $article));
                        }

                        if ($typePeriode == "mois") {
                            $dateDebut = new \DateTime($debut);
                            $dateFin = new \DateTime($fin);
                            if ($dateDebut->format("Y-m") == $dateFin->format("Y-m")) {
                                $title = " Ventes de produits de " . $this->mois[$dateDebut->format("m")] . " " . $dateDebut->format("Y");
                            } else {
                                $title = " Ventes de produits de " . $this->mois[$dateDebut->format("m")] . " " . $dateDebut->format("Y") . " A " . $this->mois[$dateFin->format("m")] . " " . $dateFin->format("Y");
                            }
                            $dateDebut->modify("first day of this month");
                            $dateFin->modify("last day of this month");
                            $articleConcerne["y"] = count($ligneFactureDao->obtenirQteArticleVenduIntervalle($dateDebut->format("Y-m-d"), $dateFin->format("Y-m-d"), $article));
                        }

                        if ($typePeriode == "annee") {
                            $dateDebut = new \DateTime($debut);
                            $dateFin = new \DateTime($fin);
                            if ($dateDebut->format("Y") == $dateFin->format("Y")) {
                                $title = " Ventes de produit du " . $dateDebut->format("Y");
                            } else {
                                $title = " Ventes de produit de " . $dateDebut->format("Y") . " A " . $dateFin->format("Y");
                            }
                            $articleConcerne["y"] = 0;
                            $anneeDebut = (int)$dateDebut->format("Y");
                            $anneeFin = (int)$dateFin->format("Y");
                            while ($anneeDebut <= $anneeFin) {
                                $articleConcerne["y"] += count($ligneFactureDao->obtenirQteArticleVenduAnnee(new \DateTime($anneeDebut . "-01-01"), $article));
                                $anneeDebut++;
                            }
                        }
                        $datasArticles[] = $articleConcerne;
                    }
                } else {
                    $article = $articleDao->getProduit((int)$_POST["article"]);
                    if ($typePeriode == "jour") {
                        if ($debut == $fin) {
                            $title = " Ventes de produit du " . $debut;
                        } else {
                            $title = " Ventes de produit du " . $debut . " au " . $fin;
                        }
                        $dateDebut = new \DateTime($debut);
                        $dateFin = new \DateTime($fin);
                        $timeDebut = strtotime($debut);
                        $timeFin = strtotime($fin);
                        while ($timeDebut <= $timeFin) {
                            $articleConcerne = array();
                            $articleConcerne["label"] = $dateDebut->format("Y-m-d");
                            $articleConcerne["y"] = count($ligneFactureDao->obtenirQteArticleVenduJour($dateDebut->format("Y-m-d"), $article));
                            $datasArticles[] = $articleConcerne;
                            $dateDebut->modify("+1 day");
                            $timeDebut = strtotime($dateDebut->format("Y-m-d"));
                        }
                    }

                    if ($typePeriode == "mois") {
                        $dateDebut = new \DateTime($debut);
                        $dateFin = new \DateTime($fin);
                        if ($dateDebut->format("Y-m") == $dateFin->format("Y-m")) {
                            $title = " Ventes de produit de " . $this->mois[$dateDebut->format("m")] . " " . $dateDebut->format("Y");
                        } else {
                            $title = " Ventes de produit de " . $this->mois[$dateDebut->format("m")] . " " . $dateDebut->format("Y") . " A " . $this->mois[$dateFin->format("m")] . " " . $dateFin->format("Y");
                        }
                        $dateDebut->modify("first day of this month");
                        $dateDebut->modify(" +6 days ");
                        $dateFin->modify("last day of this month");
                        $timeDebut = strtotime($dateDebut->format("Y-m-d"));
                        $timeFin = strtotime($dateFin->format("Y-m-d"));
                        while ($timeDebut <= $timeFin) {
                            $articleConcerne = array();
                            $articleConcerne["label"] = $dateDebut->format("Y-m");
                            $articleConcerne["y"] = count($ligneFactureDao->obtenirQteArticleVenduMois($dateDebut, $article));
                            $datasArticles[] = $articleConcerne;
                            $dateDebut->modify(" +1 month");
                            $timeDebut = strtotime($dateDebut->format("Y-m-d"));
                        }
                    }

                    if ($typePeriode == "annee") {
                        $dateDebut = new \DateTime($debut);
                        $dateFin = new \DateTime($fin);
                        if ($dateDebut->format("Y") == $dateFin->format("Y")) {
                            $title = " Ventes de produit du " . $dateDebut->format("Y");
                        } else {
                            $title = " Ventes de produit du " . $dateDebut->format("Y") . " Au " . $dateFin->format("Y");
                        }
                        $anneeDebut = (int)$dateDebut->format("Y");
                        $anneeFin = (int)$dateFin->format("Y");
                        while ($anneeDebut <= $anneeFin) {
                            $articleConcerne = array();
                            $articleConcerne["label"] = $anneeDebut;
                            $articleConcerne["y"] = count($ligneFactureDao->obtenirQteArticleVenduAnnee(new \DateTime($anneeDebut . "-01-01"), $article));
                            $datasArticles[] = $articleConcerne;
                            $anneeDebut++;
                        }
                    }
                }
                echo json_encode(array("datasArticles" => $datasArticles, "title" => $title));
            }

            if ($type == "stock") {
                //$detailMouvementDao = new DetailMouvementDAO();
                $articleDao = new ProductDAO();
                $stockDAO = new StockDAO();
                $ligneReceptDAO = new ReceptionLineDAO();
                $ligneSortieDAO = new ExitLineDAO();

                $dateDebut = new \DateTime($debut);
                $dateFin = new \DateTime($fin);


                $datasStockEntree = array();
                $datasStockSortie = array();
                $datasStockStock = array();
                $stockJour = 0;
                if ($_POST["article"] == "tous") {
                    //$listeArticle = $articleDao->listerProduit();
                    $listeArticle = $stockDAO->listerProduitStock(); //Je recupère exclusivement la liste des produits en stock
                    foreach ($listeArticle as $article) {
                        $articleConcerneEntree = array();
                        $articleConcerneSortie = array();
                        $idProduitConcerne = $article["idProduit"];
                        $leProduitConcerne = $articleDao->getProduit($idProduitConcerne);
                        $articleConcerneEntree["label"] = utf8_encode($leProduitConcerne->getDesignation());
                        $articleConcerneSortie["label"] = utf8_encode($leProduitConcerne->getDesignation());
                        $articleConcerneEntree["y"] = 0;
                        $articleConcerneSortie["y"] = 0;
                        if ($typePeriode == "jour") {
                            if ($debut == $fin) {
                                $title = "Mouvement de stock du " . $debut;
                            } else {
                                $title = " Mouvement de stock du " . $debut . " au " . $fin;
                            }
                            //                                     $current = $detailMouvementDao->obtenirQteArticleStockEntreSortieIntervalle($debut, $fin, $article);
                            //                                     $articleConcerneEntree["y"] += $current["entree"];
                            //                                     $articleConcerneSortie["y"] += abs($current["sortie"]);
                            $QteReceptionProduit = $ligneReceptDAO->obtenirQuantiteTotaleReceptionProduitIntervalle($leProduitConcerne, $dateDebut, $dateFin);
                            $QteSortieProduit = $ligneSortieDAO->obtenirQuantiteTotaleSortieProduitIntervalle($leProduitConcerne, $dateDebut, $dateFin);
                            $articleConcerneEntree["y"] += $QteReceptionProduit;
                            $articleConcerneSortie["y"] += $QteSortieProduit;
                        }

                        if ($typePeriode == "mois") {
                            $dateDebut = new \DateTime($debut);
                            $dateFin = new \DateTime($fin);
                            if ($dateDebut->format("Y-m") == $dateFin->format("Y-m")) {
                                $title = " Mouvements de stock de " . $this->mois[$dateDebut->format("m")] . " " . $dateDebut->format("Y");
                            } else {
                                $title = " Mouvements de stock de " . $this->mois[$dateDebut->format("m")] . " " . $dateDebut->format("Y") . " A " . $this->mois[$dateFin->format("m")] . " " . $dateFin->format("Y");
                            }
                            $dateDebut->modify("first day of this month");
                            $dateFin->modify("last day of this month");
                            //                                           $current = $detailMouvementDao->obtenirQteArticleStockEntreSortieIntervalle($dateDebut->format("Y-m-d"), $dateFin->format("Y-m-d"), $article);
                            //                                           $articleConcerneEntree["y"] += $current["entree"];
                            //                                           $articleConcerneSortie["y"] += abs($current["sortie"]);

                            $QteReceptionProduit = $ligneReceptDAO->obtenirQuantiteTotaleReceptionProduitIntervalle($leProduitConcerne, $dateDebut, $dateFin);
                            $QteSortieProduit = $ligneSortieDAO->obtenirQuantiteTotaleSortieProduitIntervalle($leProduitConcerne, $dateDebut, $dateFin);
                            $articleConcerneEntree["y"] += $QteReceptionProduit;
                            $articleConcerneSortie["y"] += $QteSortieProduit;
                        }

                        if ($typePeriode == "annee") {
                            $dateDebut = new \DateTime($debut);
                            $dateFin = new \DateTime($fin);
                            if ($dateDebut->format("Y") == $dateFin->format("Y")) {
                                $title = " Mouvement de stock de " . $dateDebut->format("Y");
                            } else {
                                $title = " Mouvement de stock de " . $dateDebut->format("Y") . " A " . $dateFin->format("Y");
                            }
                            $anneeDebut = (int)$dateDebut->format("Y");
                            $anneeFin = (int)$dateFin->format("Y");
                            while ($anneeDebut <= $anneeFin) {
                                //                                          $current = $detailMouvementDao->obtenirQteArticleStockEntreSortieAnnee(new \DateTime($anneeDebut . "-01-01"), $article);
                                //                                          $articleConcerneEntree["y"] += $current["entree"];
                                //                                          $articleConcerneSortie["y"] += abs($current["sortie"]);
                                //                                          $anneeDebut++;

                            }
                        }
                        $datasStockEntree[] = $articleConcerneEntree;
                        $datasStockSortie[] = $articleConcerneSortie;
                    }
                } else {
                    $article = $articleDao->getProduit((int)$_POST["article"]);
                    if ($typePeriode == "jour") {
                        if ($debut == $fin) {
                            $title = " Mouvements de stock du " . $debut;
                        } else {
                            $title = " Mouvements de stock du " . $debut . " au " . $fin;
                        }
                        $stockJour = 1;
                        $dateDebut = new \DateTime($debut);
                        $dateFin = new \DateTime($fin);
                        $timeDebut = strtotime($debut);
                        $timeFin = strtotime($fin);
                        while ($timeDebut <= $timeFin) {
                            $articleConcerneEntree = array();
                            $articleConcerneSortie = array();
                            $articleConcerneStock = array();
                            $articleConcerneStock["label"] = $dateDebut->format("Y-m-d");
                            $articleConcerneEntree["label"] = $dateDebut->format("Y-m-d");
                            $articleConcerneSortie["label"] = $dateDebut->format("Y-m-d");
                            //                                   $current = $detailMouvementDao->obtenirQteArticleStockEntreSortieJour($dateDebut->format("Y-m-d"), $article);
                            $QteReceptionProduit = $ligneReceptDAO->obtenirQuantiteTotaleReceptionProduitIntervalle($article, $dateDebut, $dateFin);
                            $QteSortieProduit = $ligneSortieDAO->obtenirQuantiteTotaleSortieProduitIntervalle($article, $dateDebut, $dateFin);

                            //                                   $articleConcerneEntree["y"] = $current["entree"];
                            //                                   $articleConcerneSortie["y"] = abs($current["sortie"]);
                            //                                   $articleConcerneStock["y"] = $current["stock"];
                            //                                   
                            $articleConcerneEntree["y"] = $QteReceptionProduit;
                            $articleConcerneSortie["y"] = $QteSortieProduit;
                            $articleConcerneStock["y"] = abs($QteReceptionProduit - $QteSortieProduit);
                            //var_dump($QteSortieProduit); exit;
                            //                                   $datasStockStock[] = $articleConcerneStock;
                            //                                   $datasStockEntree[] = $articleConcerneEntree;
                            //                                   $datasStockSortie[] = $articleConcerneSortie;
                            //                                   
                            $datasStockStock[] = $articleConcerneStock;
                            $datasStockEntree[] = $articleConcerneEntree;
                            $datasStockSortie[] = $articleConcerneSortie;

                            $dateDebut->modify("+1 day");
                            $timeDebut = strtotime($dateDebut->format("Y-m-d"));
                        }
                    }

                    if ($typePeriode == "mois") {
                        $dateDebut = new \DateTime($debut);
                        $dateFin = new \DateTime($fin);
                        if ($dateDebut->format("Y-m") == $dateFin->format("Y-m")) {
                            $title = " Mouvements de stock  de " . $this->mois[$dateDebut->format("m")] . " " . $dateDebut->format("Y");
                        } else {
                            $title = " Mouvements de stock de " . $this->mois[$dateDebut->format("m")] . " " . $dateDebut->format("Y") . " A " . $this->mois[$dateFin->format("m")] . " " . $dateFin->format("Y");
                        }
                        $dateDebut->modify("first day of this month");
                        $dateDebut->modify(" +6 days ");
                        $dateFin->modify("last day of this month");
                        $timeDebut = strtotime($dateDebut->format("Y-m-d"));
                        $timeFin = strtotime($dateFin->format("Y-m-d"));
                        while ($timeDebut <= $timeFin) {
                            $articleConcerneEntree = array();
                            $articleConcerneSortie = array();
                            $articleConcerneEntree["label"] = $dateDebut->format("Y-m");
                            $articleConcerneSortie["label"] = $dateDebut->format("Y-m");
                            //                               $current = $detailMouvementDao->obtenirQteArticleStockEntreSortieMois($dateDebut, $article);
                            //                               $articleConcerneEntree["y"] = $current["entree"];
                            //                               $articleConcerneSortie["y"] = abs($current["sortie"]);
                            //                               $datasStockEntree[] = $articleConcerneEntree;
                            //                               $datasStockSortie[] = $articleConcerneSortie;
                            $QteReceptionProduit = $ligneReceptDAO->obtenirQuantiteTotaleReceptionProduitIntervalle($article, $dateDebut, $dateFin);
                            $QteSortieProduit = $ligneSortieDAO->obtenirQuantiteTotaleSortieProduitIntervalle($article, $dateDebut, $dateFin);
                            $articleConcerneEntree["y"] = $QteReceptionProduit;
                            $articleConcerneSortie["y"] = $QteSortieProduit;
                            $datasStockEntree[] = $articleConcerneEntree;
                            $datasStockSortie[] = $articleConcerneSortie;
                            $dateDebut->modify(" +1 month");
                            $timeDebut = strtotime($dateDebut->format("Y-m-d"));
                        }
                    }

                    if ($typePeriode == "annee") {
                        $dateDebut = new \DateTime($debut);
                        $dateFin = new \DateTime($fin);
                        if ($dateDebut->format("Y") == $dateFin->format("Y")) {
                            $title = " Mouvements de stock de " . $dateDebut->format("Y");
                        } else {
                            $title = " Mouvements de stock de " . $dateDebut->format("Y") . " A " . $dateFin->format("Y");
                        }
                        $anneeDebut = (int)$dateDebut->format("Y");
                        $anneeFin = (int)$dateFin->format("Y");
                        while ($anneeDebut <= $anneeFin) {
                            $articleConcerneEntree = array();
                            $articleConcerneSortie = array();
                            $articleConcerneEntree["label"] = $anneeDebut;
                            $articleConcerneSortie["label"] = $anneeDebut;
                            //                                     $current = $detailMouvementDao->obtenirQteArticleStockEntreSortieAnnee(new \DateTime($anneeDebut."-01-01"), $article);
                            //                                     $articleConcerneEntree["y"] = abs($current["entree"]);
                            //                                     $articleConcerneSortie["y"] = abs($current["sortie"]);
                            //                                     $datasStockEntree[] = $articleConcerneEntree;
                            //                                     $datasStockSortie[] = $articleConcerneSortie;
                            //                                     $anneeDebut++;
                        }
                    }
                }
                echo json_encode(array("datasStockEntrees" => $datasStockEntree, "datasStockSorties" => $datasStockSortie, "datasStockStock" => $datasStockStock, "stockJour" => $stockJour, "title" => $title));
            }

            exit();
        }

        $factureDao = new CustomerInvoiceDAO();
        $avoirDao = new CreditDAO();
        $mvtCaisseDao = new CashMovementDAO();
        $paiementFactureDao = new PaiementFactureDAO();
        $paiementAvoirDao = new CreditPaymentDAO();
        $articleDao = new ProductDAO();
        $stockDAO = new StockDAO();

        //$articleDao = new ArticleDAO();
        //$detailMouvementDao = new DetailMouvementDAO();
        $ligneFactureDao = new CustomerInvoiceLineDAO();
        // $listeArticles = $articleDao->listerProduit();
        $listeArticles = $stockDAO->listerProduitEnStock();
        $biAvoir = array();
        $biFacture = array();
        $biArticles = array();
        $biStockEntree = array();
        $biStockSortie = array();
        $biPaiementAvoir = array();
        $biPaiementFacture = array();
        $biCaisseEntree = array();
        $biCaisseSortie = array();
        $listeVente = array();
        $listeStock = array();
        foreach ($listeArticles as $article) {
            $produit = $articleDao->getProduit($article["idProduit"]);
            $listeStock[$produit->getIdProduit()]["article"] = $article;
            $listeStock[$produit->getIdProduit()]["entree"] = 0;
            $listeStock[$produit->getIdProduit()]["sortie"] = 0;
        }

        foreach ($listeArticles as $article) {
            $produit = $articleDao->getProduit($article["idProduit"]);
            $listeVente[$produit->getIdProduit()]["article"] = $article;
            $listeVente[$produit->getIdProduit()]["qte"] = 0;
        }

        $dateCourante = new \DateTime();
        $typePeriodeChart = TYPE_AFFICHAGE_CHART;
        if ($typePeriodeChart) {
            if ($typePeriodeChart == TYPE_PERIODE_CHART_JOURNALIER) {
                $dateDebut = clone $dateCourante;
            }

            if ($typePeriodeChart == TYPE_PERIODE_CHART_HEBDOMADAIRE) {
                $dateDebut = clone $dateCourante;
                $dateDimanchePrecedent = clone $dateDebut;
                $dateDimanchePrecedent->modify("last sunday");
                $dateDimanchePrecedent->modify("next monday");
                $dateDebutClone = clone $dateDimanchePrecedent;
                $dateDebut = clone $dateDimanchePrecedent;
                $dateCourante = clone $dateDimanchePrecedent->modify("next sunday");
            }

            if ($typePeriodeChart == TYPE_PERIODE_CHART_MENSUEL) {
                $dateDebut = clone $dateCourante;
                $dateDebut->modify("first day of this month");
                $dateCourante->modify("last day of this month");
            }

            if ($typePeriodeChart == TYPE_PERIODE_CHART_TRIMESTRIEL) {
                $mois = (int)$dateCourante->format("m");
                $annee = $dateCourante->format("Y");
                if ($mois >= 1 && $mois <= 3) {
                    $dateDebut = new \DateTime($annee . "-01-01");
                    $dateCourante = new \DateTime($annee . "-03-01");
                    $dateCourante->modify("last day of this month");
                }

                if ($mois >= 4 && $mois <= 6) {
                    $dateDebut = new \DateTime($annee . "-04-01");
                    $dateCourante = new \DateTime($annee . "-06-01");
                    $dateCourante->modify("last day of this month");
                }

                if ($mois >= 7 && $mois <= 9) {
                    $dateDebut = new \DateTime($annee . "-07-01");
                    $dateCourante = new \DateTime($annee . "-09-01");
                    $dateCourante->modify("last day of this month");
                }

                if ($mois >= 10 && $mois <= 12) {
                    $dateDebut = new \DateTime($annee . "-10-01");
                    $dateCourante = new \DateTime($annee . "-12-01");
                    $dateCourante->modify("last day of this month");
                }
            }

            if ($typePeriodeChart == TYPE_PERIODE_CHART_SEMESTRIEL) {
                $mois = (int)$dateCourante->format("m");
                $annee = $dateCourante->format("Y");
                if ($mois >= 1 && $mois <= 6) {
                    $dateDebut = new \DateTime($annee . "-01-01");
                    $dateCourante = new \DateTime($annee . "-06-01");
                    $dateCourante->modify("last day of this month");
                } else {
                    $dateDebut = new \DateTime($annee . "-07-01");
                    $dateCourante = new \DateTime($annee . "-12-01");
                    $dateCourante->modify("last day of this month");
                }
            }

            if ($typePeriodeChart == TYPE_PERIODE_CHART_ANNUEL) {
                $annee = $dateCourante->format("Y");
                $dateCourante = new \DateTime($annee . "-12-01");
                $dateDebut = new \DateTime($annee . "-01-01");
                $dateCourante->modify("last day of this month");
            }

            $dateDebutClone = clone $dateDebut;
        } else {
            $dateDebut = clone $dateCourante;
            $dateDimanchePrecedent = clone $dateDebut;
            $dateDimanchePrecedent->modify("last sunday");
            $dateDimanchePrecedent->modify("next monday");
            $dateDebutClone = clone $dateDimanchePrecedent;
            $dateDebut = clone $dateDimanchePrecedent;
            $dateCourante = clone $dateDimanchePrecedent->modify("next sunday");
        }
        $timeCourant = strtotime($dateCourante->format("Y-m-d"));
        $timeDebut = strtotime($dateDebut->format("Y-m-d"));
        while ($timeDebut <= $timeCourant) {
            $occurrenceAvoir = array();
            $occurrenceFacture = array();
            $occurrenceCaisseEntree = array();
            $occurrenceCaisseSortie = array();
            $occurrencePaiementAvoir = array();
            $occurrencePaiementFacture = array();
            $occurrenceAvoir["label"] = $dateDebut->format("d/m/Y");
            $occurrenceFacture["label"] = $dateDebut->format("d/m/Y");
            $occurrencePaiementAvoir["label"] = $dateDebut->format("d/m/Y");
            $occurrencePaiementFacture["label"] = $dateDebut->format("d/m/Y");
            $occurrenceCaisseEntree["label"] = $dateDebut->format("d/m/Y");
            $occurrenceCaisseSortie["label"] = $dateDebut->format("d/m/Y");

            $occurrenceAvoir["y"] = !is_null($avoirDao->getSommeMontantAvoirParJour($dateDebut->format("Y-m-d"))) ? $avoirDao->getSommeMontantAvoirParJour($dateDebut->format("Y-m-d")) : 0;
            $occurrenceFacture["y"] = !is_null($factureDao->getSommeMontantFactureParJour($dateDebut->format("Y-m-d"))) ? $factureDao->getSommeMontantFactureParJour($dateDebut->format("Y-m-d")) : 0;
            $occurrencePaiementAvoir["y"] = !is_null($paiementAvoirDao->getSommePaiementAvoirParJour($dateDebut->format("Y-m-d"))) ? $paiementAvoirDao->getSommePaiementAvoirParJour($dateDebut->format("Y-m-d")) : 0;
            $occurrencePaiementFacture["y"] = !is_null($paiementFactureDao->getSommePaiementFactureParJour($dateDebut->format("Y-m-d"))) ? $paiementFactureDao->getSommePaiementFactureParJour($dateDebut->format("Y-m-d")) : 0;
            $caisse = $mvtCaisseDao->getSommePaiementCaisseParJour($dateDebut->format("Y-m-d"));
            $occurrenceCaisseEntree["y"] = !is_null($caisse["entrees"]) ? $caisse["entrees"] : 0;
            $occurrenceCaisseSortie["y"] = !is_null($caisse["sorties"]) ? $caisse["sorties"] : 0;
            $biAvoir[] = $occurrenceAvoir;
            $biFacture[] = $occurrenceFacture;
            $biPaiementAvoir[] = $occurrencePaiementAvoir;
            $biPaiementFacture[] = $occurrencePaiementFacture;
            $biCaisseEntree[] = $occurrenceCaisseEntree;
            $biCaisseSortie[] = $occurrenceCaisseSortie;
            //            foreach ( $listeStock as $key => $val ){
            //                    $current = $detailMouvementDao->obtenirQteArticleStockEntreSortieJour($dateDebut->format("Y-m-d"), $val["article"]);
            //                    $listeStock[$key]["entree"] += $current["entree"];
            //                    $listeStock[$key]["sortie"] += abs($current["sortie"]);
            //            }
            foreach ($listeVente as $key => $val) {
                $listeVente[$key]["qte"] += count($ligneFactureDao->obtenirQteArticleVenduJour($dateDebut->format("Y-m-d"), $val["article"]["idProduit"]));
            }
            $dateDebut->modify(" +1 day");
            $timeDebut = strtotime($dateDebut->format("Y-m-d"));
        }

        foreach ($listeStock as $key => $val) {
            $stockConcerneEntree = array();
            $stockConcerneSortie = array();
            $prodDAO = new ProductDAO();
            $leProduit = $prodDAO->getProduit($key);

            //$stockConcerneEntree["label"] = $key;
            $stockConcerneEntree["label"] = utf8_encode($leProduit->getDesignation());
            $stockConcerneEntree["y"] = $val["entree"];
            //$stockConcerneSortie["label"] = $key;
            $stockConcerneSortie["label"] = utf8_encode($leProduit->getDesignation());
            $stockConcerneSortie["y"] = $val["sortie"];
            $biStockEntree[] = $stockConcerneEntree;
            $biStockSortie[] = $stockConcerneSortie;
        }

        foreach ($listeVente as $key => $val) {
            $venteConcerne = array();
            $prodDAO = new ProductDAO();
            $leProduit = $prodDAO->getProduit($key);
            $venteConcerne["label"] = utf8_encode($leProduit->getDesignation());
            $venteConcerne["y"] = $val["qte"];
            $biArticles[] = $venteConcerne;
        }

        $superDatas = array();
        $superDatas["biCaisseEntree"] = json_encode($biCaisseEntree);
        $superDatas["biCaisseSortie"] = json_encode($biCaisseSortie);
        $superDatas["biFacture"] = json_encode($biFacture);
        $superDatas["biAvoir"] = json_encode($biAvoir);
        $superDatas["biPaiementFacture"] = json_encode($biPaiementFacture);
        $superDatas["biPaiementAvoir"] = json_encode($biPaiementAvoir);
        $superDatas["entrees"] = json_encode($biStockEntree);
        $superDatas["sorties"] = json_encode($biStockSortie);
        $superDatas["articles"] = json_encode($biArticles);
        $this->view->superDatas = $superDatas;
        $this->view->debut = $dateDebutClone->format("d/m/Y");
        $this->view->fin = $dateCourante->format("d/m/Y");
        $this->view->debutDateTime = $dateDebutClone->format("d-m-Y");
        $this->view->finDateTime = $dateCourante->format("d-m-Y");
        $this->view->articles = $listeArticles;
    }
}
