<?php

include APPLICATION_PATH . '/includes/daos.php';

class IndexController extends Zend_Controller_Action
{

    private $translator;

    public function init()
    {
        session_start();
        SubscriptionChecker::checkSubscriptionValidity($this);
        $this->translator = Translator::init();
    }

    public function indexAction()
    {
        if (isset($_SESSION[SESSION_USER_ID]) && $_SESSION[SESSION_USER_ID] != null) {
            $userId = $_SESSION[SESSION_USER_ID];
            $userDAO = new UserDAO();
            $user = $userDAO->getUserById($userId);
            if ($user != null) {
                $this->_helper->redirector('', 'Home');
            }
        }
        $this->view->translator = $this->translator;
    }

    /**
     * 100 success
     * 101 login or password incorrect
     * 102 password is reset
     * 103 two factor enabled
     * 104 unknown error
     */
    public function signinAction()
    {
        error_reporting(E_ERROR | E_PARSE);
        try {
            $login = isset($_POST["login"]) ? $_POST["login"] : null;
            $password = isset($_POST['password']) ? sha1(md5($_POST["password"])) : null;
            $userDao = new UserDAO();
            if ($user = $userDao->getUserByLoginPassword($login, $password)) {
                if ($user->getPasswordIsReset() == 1) {
                    echo ApiResponse::respond(102);
                    exit();
                }
                if ($user->getTwoFactorEnabled() == 1) {
                    $generatedOtp = Utility::generateOtp(6);
                    $user->setTwoFactorOtp($generatedOtp);
                    $userDao->update($user);
                    $primaryBrand = APP_PRIMARY_BRAND;
                    $secondaryBrand = APP_SECONDARY_BRAND;
                    $thirdBrand = APP_THIRD_BRAND;
                    $currentDate = new DateTime();
                    $currentYear = $currentDate->format("Y");
                    $applicationName = NOM_APPLICATION;
                    $mail = new PHPMailer\PHPMailer\PHPMailer();
                    $mail->isSMTP();
                    $mail->Host = CRESCITA_DEFAULT_MAIL_DOMAIN;
                    $mail->SMTPAuth = true;
                    $mail->Username = CRESCITA_DEFAULT_MAIL_EMAIL;
                    $mail->Password = CRESCITA_DEFAULT_MAIL_APP_PASSWORD;
                    $mail->SMTPSecure = PHPMailer\PHPMailer\PHPMailer::ENCRYPTION_SMTPS;
                    $mail->Port = (int)CRESCITA_DEFAULT_MAIL_PORT;
                    $mail->setFrom(CRESCITA_DEFAULT_MAIL_EMAIL, $applicationName);
                    $mail->addAddress($user->getIdPersonne()->getEmail(), "{$user->getIdPersonne()->getNom()} {$user->getIdPersonne()->getPrenom()}");
                    $mail->isHTML(true);
                    $mail->CharSet = "UTF-8";
                    $mail->Subject = $this->translator->_("shared.doubleAuthentication");
                    $emailHtml = "
                        <html lang='en'>
                            <head>
                                <style>
                                    .main-table {
                                        border: none; 
                                        margin: 50px; 
                                        background-color: {$primaryBrand}55;
                                        min-width: 90%;
                                    }

                                    @media screen and (max-width:600px) {
                                        .main-table {
                                            margin: 0;
                                        }
                                    }
                                </style>
                            </head>
                            <body>
                                <div style='background-image: url(cid:app_background); display: flex; justify-content: center;'>
                                    <table class='main-table'>
                                        <tr>
                                            <div style='border-width: 2px; border-style: solid; border-color: {$secondaryBrand}; padding: 1em 1em; border-radius: 4px;'>
                                                <p style='text-align: center;'>
                                                    <i style='color: {$thirdBrand}; background-color: {$primaryBrand}; font-style: italic; font-size: 16px; padding: 5px; border-radius: 6px;'>
                                                        {$applicationName}
                                                    </i>
                                                </p>
                                                <p style='text-align: center;'>
                                                    <img style='width: 125px; height: 100px; border-radius: 6px;' src='/images/assets/app_logo.png' />
                                                </p>
                                                <p style='color: {$thirdBrand}; text-align: center; font-weight: bold; font-size: 16px;'>{$this->translator->_("index.index.youAreReceivingThisEmailBecauseYouHaveActivatedDoubleAuthenticationOnYourAccount")}<p>
                                                <p style='color: {$thirdBrand}; text-align: center; font-weight: bold; font-size: 16px;'>{$this->translator->_("shared.theVerificationCodeIs")}:</p>
                                                <br/>
                                                <p style='color: {$secondaryBrand}; text-align: center; font-weight: bold; font-size: 18px;'>{$generatedOtp}</p>
                                            </div>
                                        </tr>
                                        <tr>
                                            <div style='border-width: 1.5px; border-style: solid; border-color: {$primaryBrand}; border-radius: 6px;'></div>
                                            <p style='text-align: center;'>
                                                <i style='font-weight: bold; font-size: 20px; color: {$thirdBrand};'>Crescita</i>
                                                <br/>
                                                <i style='color: {$thirdBrand}'>&copy; 2022-{$currentYear} Crescita</i>
                                            </p>
                                        </tr>
                                    </table>
                                </div>
                            </body>
                        </html>
                    ";
                    $mail->msgHTML($emailHtml, PUBLIC_PATH);
                    $mail->addEmbeddedImage(PUBLIC_PATH . '/images/assets/app_background.png', 'app_background');
                    $mail->AltBody = "{$this->translator->_("index.index.youAreReceivingThisEmailBecauseYouHaveActivatedDoubleAuthenticationOnYourAccount")} {$this->translator->_("shared.theVerificationCodeIs")}: {$generatedOtp}";
                    $mail->send();
                    echo ApiResponse::respond(103);
                    exit();
                }
                session_cache_limiter('private_no_expire');
                session_cache_expire(DEFAULT_USER_SESSION_DURATION);
                $_SESSION[SESSION_USER_ID] = $user->getIdUtilisateur();
                $_SESSION[SESSION_USER_LAST_NAME] = $user->getIdPersonne()->getNom();
                $_SESSION[SESSION_USER_FIRST_NAME] = $user->getIdPersonne()->getPrenom();
                $_SESSION[SESSION_USER_PROFILE_NAME] = $user->getIdProfil()->getNom();
                $_SESSION[SESSION_USER_LOGIN] = $user->getLogin();
                $_SESSION[SESSION_USER_POINT_OF_SALE] = $user->getIdPointVente() != null ? $user->getIdPointVente()->getNomPointVente() : null;
                $_SESSION[SESSION_USER_PROFILE_IMAGE] = $user->getProfileImage() != null ? $user->getProfileImage() : null;
                $_SESSION[SESSION_USER_PROFILE_CODE] = $user->getIdProfil()->getCode();
                $_SESSION[SESSION_USER_TWO_FACTOR_ENABLED] = $user->getTwoFactorEnabled() == 1 ? 1 : 0;
                $currentDate = new DateTime();
                LogDAO::registerOperationForCurrentUser(
                    $this->translator->_("index.signIn.signInOfUser") . " " . $user->getIdPersonne()->getNom() . " " . $user->getIdPersonne()->getPrenom() . " " . $this->translator->_("index.shared.whichLoginIs") . "(" . $user->getLogin() . ")",
                    $currentDate
                );
                LogDAO::updateLastLoginDateForCurrentUser($currentDate);
                echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["firstName" => $user->getIdPersonne()->getPrenom()]);
            } else {
                echo ApiResponse::respond(101);
            }
        } catch (Exception $ex) {
            echo ApiResponse::respond(104, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 login or password incorrect
     * 102 unknown error
     */
    public function definenewpasswordAction()
    {
        try {
            $login = isset($_POST["login"]) ? $_POST["login"] : null;
            $password = isset($_POST['password']) ? sha1(md5($_POST["password"])) : null;
            $newPassword = isset($_POST['newPassword']) ? sha1(md5($_POST["newPassword"])) : null;
            $userDao = new UserDAO();
            if ($user = $userDao->getUserByLoginPassword($login, $password)) {
                $user->setPassword($newPassword);
                $user->setPasswordIsReset(0);
                $userDao->update($user);
                LogDAO::registerOperationForUser($user, $this->translator->_("index.index.newPasswordDefined"));
                echo ApiResponse::respond();
            } else {
                echo ApiResponse::respond(101);
            }
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 incorrect verification code
     * 102 unknown error
     */
    public function checktwofactorcodeAction() {
        error_reporting(E_ERROR | E_PARSE);
        try {
            $login = isset($_POST["login"]) ? $_POST["login"] : null;
            $password = isset($_POST['password']) ? sha1(md5($_POST["password"])) : null;
            $verificationCode = $_POST["verificationCode"];
            $userDao = new UserDAO();
            if ($user = $userDao->getUserByLoginPassword($login, $password)) {
                if ($user->getTwoFactorOtp() != $verificationCode) {
                    echo ApiResponse::respond(101);
                    exit();
                }
                session_cache_limiter('private_no_expire');
                session_cache_expire(DEFAULT_USER_SESSION_DURATION);
                $_SESSION[SESSION_USER_ID] = $user->getIdUtilisateur();
                $_SESSION[SESSION_USER_LAST_NAME] = $user->getIdPersonne()->getNom();
                $_SESSION[SESSION_USER_FIRST_NAME] = $user->getIdPersonne()->getPrenom();
                $_SESSION[SESSION_USER_PROFILE_NAME] = $user->getIdProfil()->getNom();
                $_SESSION[SESSION_USER_LOGIN] = $user->getLogin();
                $_SESSION[SESSION_USER_POINT_OF_SALE] = $user->getIdPointVente() != null ? $user->getIdPointVente()->getNomPointVente() : null;
                $_SESSION[SESSION_USER_PROFILE_IMAGE] = $user->getProfileImage() != null ? $user->getProfileImage() : null;
                $_SESSION[SESSION_USER_PROFILE_CODE] = $user->getIdProfil()->getCode();
                $_SESSION[SESSION_USER_TWO_FACTOR_ENABLED] = $user->getTwoFactorEnabled() == 1 ? 1 : 0;
                $currentDate = new DateTime();
                LogDAO::registerOperationForCurrentUser(
                    $this->translator->_("index.signIn.signInOfUser") . " " . $user->getIdPersonne()->getNom() . " " . $user->getIdPersonne()->getPrenom() . " " . $this->translator->_("index.shared.whichLoginIs") . "(" . $user->getLogin() . ")",
                    $currentDate
                );
                LogDAO::updateLastLoginDateForCurrentUser($currentDate);
                echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["firstName" => $user->getIdPersonne()->getPrenom()]);
            } else {
                echo ApiResponse::respond(101);
            }
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    public function signoutAction()
    {
        if (isset($_SESSION[SESSION_USER_ID])) {
            $userDao = new UserDAO();
            $user = $userDao->getUserById($_SESSION[SESSION_USER_ID]);
            $currentDate = new DateTime();
            LogDAO::registerOperationForCurrentUser(
                $this->translator->_("index.signOut.signOutOfUser") . " " . $user->getIdPersonne()->getNom() . " " . $user->getIdPersonne()->getPrenom() . " " . $this->translator->_("index.shared.whichLoginIs") . "(" . $user->getLogin() . ")",
                $currentDate
            );
            LogDAO::updateLastLogoutDateForCurrentUser($currentDate);
            unset($_SESSION[SESSION_USER_ID]);
            unset($_SESSION[SESSION_USER_LAST_NAME]);
            unset($_SESSION[SESSION_USER_FIRST_NAME]);
            unset($_SESSION[SESSION_USER_PROFILE_NAME]);
            unset($_SESSION[SESSION_USER_LOGIN]);
            unset($_SESSION[SESSION_USER_POINT_OF_SALE]);
            unset($_SESSION[SESSION_USER_PROFILE_IMAGE]);
            unset($_SESSION[SESSION_USER_PROFILE_CODE]);
            unset($_SESSION[SESSION_USER_TWO_FACTOR_ENABLED]);
        }
        $this->_helper->redirector('', '');
    }
}
