<?php

include APPLICATION_PATH . '/includes/daos.php';

class LoyaltyController extends Zend_Controller_Action
{
    private $translator;
    private $days;

    public function init()
    {
        session_start();
        SubscriptionChecker::checkSubscriptionValidity($this);
        if (isset($_SESSION[SESSION_USER_ID]) && $_SESSION[SESSION_USER_ID] != null) {
            $_SESSION[LAST_ACTION_DATE] = new DateTime();
            LogDAO::updateLastActionDateForCurrentUser();
            $this->translator = Translator::init();
            $this->days = [
                "0" => $this->translator->_("shared.days.monday"),
                "1" => $this->translator->_("shared.days.tuesday"),
                "2" => $this->translator->_("shared.days.wednesday"),
                "3" => $this->translator->_("shared.days.thursday"),
                "4" => $this->translator->_("shared.days.friday"),
                "5" => $this->translator->_("shared.days.saturday"),
                "6" => $this->translator->_("shared.days.sunday"),
            ];
        } else {
            if ($this->getRequest()->isPost()) {
                echo ApiResponse::respond(SESSION_EXPIRED_CODE);
                exit();
            } else {
                $this->_helper->redirector('index', 'Index');
            }
        }
    }

    public function indexAction()
    {
        AccessRightChecker::checkAccessRight($this, 83);
        $pointOfSaleDao = new PointOfSaleDAO();
        $currentUser = LogDAO::getCurrentUser();
        $pointsOfSale = [];
        if ($currentUser->getIdPointVente() == null || in_array($currentUser->getIdProfil()->getCode(), IDENTIFIED_PROFILES_CODES)) {
            $pointsOfSale = $pointOfSaleDao->listPointOfSales();
        } else {
            $pointsOfSale[] = $currentUser->getIdPointVente();
        }
        $this->view->pointsOfSale = $pointsOfSale;
        $this->view->translator = $this->translator;
    }

    public function datatableAction()
    {
        $currentDate = new DateTime();
        $currentDateFormat = $currentDate->format(DATE_FORMAT_WITH_SECONDS);
        $pointOfSale = (int)$_POST["pointOfSale"] != 0 ? (int)$_POST["pointOfSale"] : null;
        $customer = (int)$_POST["customer"] != 0 ? (int)$_POST["customer"] : null;
        $status = (int)$_POST["status"] != 0 ? (int)$_POST["status"] : null;
        $wherePointOfSale = "";
        $whereCustomer = "";
        $whereStatus = "";
        if ($pointOfSale != null) {
            $wherePointOfSale .= " AND c.id_point_vente = {$pointOfSale}";
        }
        if ($customer != null) {
            $whereCustomer .= " AND c.id_personne = {$customer}";
        }
        if ($status != null) {
            if ($status == 1) {
                $whereStatus .= " AND c.id_personne IS NULL";
            } else {
                $whereStatus .= " AND c.id_personne IS NOT NULL";
            }
        }
        $where = "c.statut = 1 AND c.date_expiration > '{$currentDateFormat}'{$wherePointOfSale}{$whereCustomer}{$whereStatus}";
        $storesTable = "
                (
                    SELECT c.id_carte_fidelite AS card_id, c.id_personne AS card_holder_id, c.code AS card_code,
                    c.date_expiration AS card_expiration_date, p.nom_point_vente AS card_point_of_sale
                    FROM carte_fidelite c JOIN point_vente p ON c.id_point_vente = p.id_point_vente
                    WHERE {$where}
                ) AS cardsTable
        ";
        $primaryKey = 'card_id';
        $columns = [
            ['db' => 'card_code', 'dt' => 0],
            ['db' => 'card_point_of_sale', 'dt' => 1],
            ['db' => 'card_holder_id', 'dt' => 2],
            ['db' => 'card_expiration_date', 'dt' => 3],
            ['db' => 'card_id', 'dt' => 4],
        ];
        $computeDatatableResults = SSP::customSimple($_POST, $storesTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        foreach ($datatableResult["data"] as $row) {
            $personDao = new PersonDAO();
            $cardCode = $row[0];
            $cardPointOfSale = $row[1];
            $customerId = $row[2];
            $customer = $customerId != null ? $personDao->getPersonById($customerId) : null;
            $expirationDate = new DateTime($row[3]);
            $cardId = $row[4];
            $actions = "";
            if ($customer != null) {
                $actions = "
                    <button title='{$this->translator->_("shared.print")}' about='{$cardId}' class='mini ui purple icon button app-print-action'>
                        <i class='print icon'></i>
                    </button>
                ";
            } else {
                $actions = "
                    <button title='{$this->translator->_("loyalty.index.linkTheCardToACustomer")}' about='{$cardId}' class='mini ui icon button app-primary-button app-link-action'>
                        <i class='user plus icon'></i>
                    </button>
                    <button title='{$this->translator->_("shared.delete")}' about='{$cardId}' class='mini ui red icon button app-delete-action'>
                        <i class='trash icon'></i>
                    </button>
                ";
            }
            $tableRow = [
                $cardCode,
                $cardPointOfSale,
                $customer != null ?
                    "{$customer->getNom()} {$customer->getPrenom()}" :
                    "<a class='ui label app-secondary-label'>{$this->translator->_("loyalty.index.notAttributed")}</a>",
                $expirationDate->format(DATE_FORMAT),
                $actions,
            ];
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function createcardsAction()
    {
        $loyaltyCardDao = new LoyaltyCardDAO();
        $databaseConnection = $loyaltyCardDao->entityManager->getConnection();
        $databaseConnection->beginTransaction();
        try {
            $pointOfSaleDao = new PointOfSaleDAO();
            $count = (int)$_POST["count"];
            $pointOfSaleId = (int)$_POST["pointOfSale"];
            $pointOfSale = $pointOfSaleDao->getPointOfSaleById($pointOfSaleId);
            $expirationDate = new DateTime($_POST["expirationDate"]);
            $lastGroupNumber = $loyaltyCardDao->getLastGroupNumber();
            $currentDate = new DateTime();
            $lastGroupNumber = $lastGroupNumber ? $lastGroupNumber + 1 : 1;
            $lastGroupNumberString = "{$lastGroupNumber}";
            if (strlen($lastGroupNumberString) % 2 != 0) {
                $lastGroupNumberString = "0{$lastGroupNumberString}";
            }
            while ($count >= 1) {
                $countString = "{$count}";
                if (strlen($countString) % 2 != 0) {
                    $countString = "0{$countString}";
                }
                $loyaltyCard = $loyaltyCardDao->create();
                $loyaltyCard->setCode("{$currentDate->format(DATE_FORMAT_YEAR)}{$currentDate->format(DATE_FORMAT_MONTH_NUMERIC)}{$currentDate->format(DATE_FORMAT_DAY_NUMERIC)}{$countString}{$lastGroupNumberString}");
                $loyaltyCard->setDateExpiration($expirationDate);
                $loyaltyCard->setIdPointVente($pointOfSale);
                $loyaltyCard->setNumeroGroupe($lastGroupNumber);
                $loyaltyCard->setStatut(1);
                $loyaltyCardDao->save($loyaltyCard);
                $count--;
            }
            $databaseConnection->commit();
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("loyalty.index.loyaltyCardsRegisteredSuccessfully")} : {$this->translator->_("loyalty.index.group")} N°{$lastGroupNumber}"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            $databaseConnection->rollBack();
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 customer already has a card
     * 102 unknown error
     */
    public function linkcardtocustomerAction()
    {
        $loyaltyCardDao = new LoyaltyCardDAO();
        $databaseConnection = $loyaltyCardDao->entityManager->getConnection();
        $databaseConnection->beginTransaction();
        try {
            $personDao = new PersonDAO();
            $loyaltyCardId = (int)$_POST["card"];
            $loyaltyCard = $loyaltyCardDao->getLoyaltyCardById($loyaltyCardId);
            $customerId = (int)$_POST["customer"];
            $customer = $personDao->getPersonById($customerId);
            if ($loyaltyCardDao->getCustomerActiveCard($customer, $loyaltyCard->getIdPointVente()) != null) {
                echo ApiResponse::respond(101, null, ["pointOfSale" => $loyaltyCard->getIdPointVente()->getNomPointVente()]);
                exit();
            }
            $loyaltyCard->setIdPersonne($customer);
            $loyaltyCardDao->update($loyaltyCard);
            $databaseConnection->commit();
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            $databaseConnection->rollBack();
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 card alreay linked
     * 102 unknown error
     */
    public function deletecardAction()
    {
        try {
            $loyaltyCardDao = new LoyaltyCardDAO();
            $id = (int)$_POST["id"];
            $loyaltyCard = $loyaltyCardDao->getLoyaltyCardById($id);
            if ($loyaltyCard->getStatut() == 0) {
                echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE);
                exit();
            }
            if ($loyaltyCard->getIdPersonne() != null) {
                echo ApiResponse::respond(101);
                exit();
            }
            $loyaltyCard->setStatut(0);
            $loyaltyCardDao->update($loyaltyCard);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("loyalty.index.loyaltyCardDeletedSuccessfully")} : N°{$loyaltyCard->getCode()}"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    public function purchaseintervalsAction()
    {
        AccessRightChecker::checkAccessRight($this, 80);
        $this->view->translator = $this->translator;
    }

    public function datatablepurchaseintervalsAction()
    {
        $minimumAmount = $_POST["minimumAmount"] != "" ? (float)$_POST["minimumAmount"] : null;
        $maximumAmount = $_POST["maximumAmount"] != "" ? (float)$_POST["maximumAmount"] : null;
        $whereMinimumAmount = "";
        $whereMaximumAmount = "";
        if ($minimumAmount != null) {
            $whereMinimumAmount .= " AND i.somme_minimale >= {$minimumAmount}";
        }
        if ($maximumAmount != null) {
            $whereMaximumAmount .= " AND i.somme_maximale <= {$maximumAmount}";
        }
        $where = "i.statut = 1{$whereMinimumAmount}{$whereMaximumAmount}";
        $intervalsTable = "
                (
                    SELECT i.id_intervalle_point AS interval_id, i.somme_minimale AS interval_minimum, 
                    i.somme_maximale AS interval_maximum, i.nombre_point AS interval_points
                    FROM intervalle_point i
                    WHERE {$where}
                ) AS intervalsTable
        ";
        $primaryKey = 'interval_id';
        $columns = [
            ['db' => 'interval_minimum', 'dt' => 0],
            ['db' => 'interval_maximum', 'dt' => 1],
            ['db' => 'interval_points', 'dt' => 2],
            ['db' => 'interval_id', 'dt' => 3],
        ];
        $computeDatatableResults = SSP::customSimple($_POST, $intervalsTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        foreach ($datatableResult["data"] as $row) {
            $intervalMinimum = (float)$row[0];
            $intervalMaximum = (float)$row[1];
            $intervalPoints = $row[2];
            $intervalId = $row[3];
            $actions = "
                <button title='{$this->translator->_("shared.edit")}' about='{$intervalId}' class='mini ui icon button app-secondary-button app-edit-action'>
                    <i class='edit icon'></i>
                </button>
                <button title='{$this->translator->_("shared.delete")}' about='{$intervalId}' class='mini ui red icon button app-delete-action'>
                    <i class='trash icon'></i>
                </button>
            ";
            $tableRow = [
                number_format($intervalMinimum, 0, '', ' '),
                number_format($intervalMaximum, 0, '', ' '),
                $intervalPoints,
                $actions,
            ];
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    /**
     * 100 success
     * 101 overlapping interval
     * 102 unknown error
     */
    public function createpurchaseintervalAction()
    {
        try {
            $pointIntervalDao = new PointIntervalDAO();
            $minimumAmount = (float)$_POST["minimumAmount"];
            $maximumAmount = (float)$_POST["maximumAmount"];
            $numberOfPoints = (int)$_POST["numberOfPoints"];
            if ($pointIntervalDao->overlappingCheck($minimumAmount, $maximumAmount)) {
                echo ApiResponse::respond(101);
                exit();
            }
            $pointInterval = $pointIntervalDao->create();
            $pointInterval->setSommeMinimale($minimumAmount);
            $pointInterval->setSommeMaximale($maximumAmount);
            $pointInterval->setNombrePoint($numberOfPoints);
            $pointInterval->setStatut(1);
            $pointIntervalDao->save($pointInterval);
            $minimumAmountFormat = number_format($minimumAmount, 0, '', ' ');
            $maximumAmountFormat = number_format($maximumAmount, 0, '', ' ');
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("loyalty.purchaseIntervals.purchaseIntervalCreatedSuccessfully")} : {$minimumAmountFormat} - {$maximumAmountFormat} ($numberOfPoints)"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 overlapping interval
     * 102 unknown error
     */
    public function updatepurchaseintervalAction()
    {
        try {
            $pointIntervalDao = new PointIntervalDAO();
            $id = (int)$_POST["id"];
            $minimumAmount = (float)$_POST["minimumAmount"];
            $maximumAmount = (float)$_POST["maximumAmount"];
            $numberOfPoints = (int)$_POST["numberOfPoints"];
            if ($pointIntervalDao->overlappingCheck($minimumAmount, $maximumAmount, $id)) {
                echo ApiResponse::respond(101);
                exit();
            }
            $pointInterval = $pointIntervalDao->getPointIntervalById($id);
            $pointInterval->setSommeMinimale($minimumAmount);
            $pointInterval->setSommeMaximale($maximumAmount);
            $pointInterval->setNombrePoint($numberOfPoints);
            $pointIntervalDao->update($pointInterval);
            $minimumAmountFormat = number_format($minimumAmount, 0, '', ' ');
            $maximumAmountFormat = number_format($maximumAmount, 0, '', ' ');
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("loyalty.purchaseIntervals.purchaseIntervalUpdatedSuccessfully")} : {$minimumAmountFormat} - {$maximumAmountFormat} ($numberOfPoints)"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function deletepurchaseintervalAction()
    {
        try {
            $pointIntervalDao = new PointIntervalDAO();
            $id = (int)$_POST["id"];
            $pointInterval = $pointIntervalDao->getPointIntervalById($id);
            if ($pointInterval->getStatut() == 0) {
                echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE);
                exit();
            }
            $pointInterval->setStatut(0);
            $pointIntervalDao->update($pointInterval);
            $minimumAmountFormat = number_format($pointInterval->getSommeMinimale(), 0, '', ' ');
            $maximumAmountFormat = number_format($pointInterval->getSommeMaximale(), 0, '', ' ');
            $numberOfPoints = $pointInterval->getNombrePoint();
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("loyalty.purchaseIntervals.purchaseIntervalDeletedSuccessfully")} : {$minimumAmountFormat} - {$maximumAmountFormat} ($numberOfPoints)"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    public function pointsintervalsAction()
    {
        AccessRightChecker::checkAccessRight($this, 82);
        $this->view->translator = $this->translator;
    }

    public function datatablepointsintervalsAction()
    {
        $minimumNumberOfPoints = $_POST["minimumNumberOfPoints"] != "" ? (float)$_POST["minimumNumberOfPoints"] : null;
        $maximumNumberOfPoints = $_POST["maximumNumberOfPoints"] != "" ? (float)$_POST["maximumNumberOfPoints"] : null;
        $whereMinimumNumberOfPoints = "";
        $whereMaximumNumberOfPoints = "";
        if ($minimumNumberOfPoints != null) {
            $whereMinimumNumberOfPoints .= " AND i.point_minimum >= {$minimumNumberOfPoints}";
        }
        if ($maximumNumberOfPoints != null) {
            $whereMaximumNumberOfPoints .= " AND i.point_maximum <= {$maximumNumberOfPoints}";
        }
        $where = "i.statut = 1{$whereMinimumNumberOfPoints}{$whereMaximumNumberOfPoints}";
        $intervalsTable = "
                (
                    SELECT i.id_intervalle_bon_achat AS interval_id, i.point_minimum AS interval_minimum, 
                    i.point_maximum AS interval_maximum, i.montant_bon_achat AS interval_voucher_amount
                    FROM intervalle_bon_achat i
                    WHERE {$where}
                ) AS intervalsTable
        ";
        $primaryKey = 'interval_id';
        $columns = [
            ['db' => 'interval_minimum', 'dt' => 0],
            ['db' => 'interval_maximum', 'dt' => 1],
            ['db' => 'interval_voucher_amount', 'dt' => 2],
            ['db' => 'interval_id', 'dt' => 3],
        ];
        $computeDatatableResults = SSP::customSimple($_POST, $intervalsTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        foreach ($datatableResult["data"] as $row) {
            $intervalMinimum = $row[0];
            $intervalMaximum = $row[1];
            $intervalVoucherAmount = (float)$row[2];
            $intervalId = $row[3];
            $actions = "
                <button title='{$this->translator->_("shared.edit")}' about='{$intervalId}' class='mini ui icon button app-secondary-button app-edit-action'>
                    <i class='edit icon'></i>
                </button>
                <button title='{$this->translator->_("shared.delete")}' about='{$intervalId}' class='mini ui red icon button app-delete-action'>
                    <i class='trash icon'></i>
                </button>
            ";
            $tableRow = [
                $intervalMinimum,
                $intervalMaximum,
                number_format($intervalVoucherAmount, 0, '', ' '),
                $actions,
            ];
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    /**
     * 100 success
     * 101 overlapping interval
     * 102 unknown error
     */
    public function createpointsintervalAction()
    {
        try {
            $voucherIntervalDao = new VoucherIntervalDAO();
            $minimumNumberOfPoints = (int)$_POST["minimumNumberOfPoints"];
            $maximumNumberOfPoints = (int)$_POST["maximumNumberOfPoints"];
            $voucherAmount = (float)$_POST["voucherAmount"];
            if ($voucherIntervalDao->overlappingCheck($minimumNumberOfPoints, $maximumNumberOfPoints)) {
                echo ApiResponse::respond(101);
                exit();
            }
            $voucherInterval = $voucherIntervalDao->create();
            $voucherInterval->setPointMinimum($minimumNumberOfPoints);
            $voucherInterval->setPointMaximum($maximumNumberOfPoints);
            $voucherInterval->setMontantBonAchat($voucherAmount);
            $voucherInterval->setStatut(1);
            $voucherIntervalDao->save($voucherInterval);
            $voucherAmountFormat = number_format($voucherAmount, 0, '', ' ');
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("loyalty.pointsIntervals.pointsIntervalCreatedSuccessfully")} : {$minimumNumberOfPoints} - {$maximumNumberOfPoints} ($voucherAmountFormat)"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 overlapping interval
     * 102 unknown error
     */
    public function updatepointsintervalAction()
    {
        try {
            $voucherIntervalDao = new VoucherIntervalDAO();
            $id = (int)$_POST["id"];
            $minimumNumberOfPoints = (int)$_POST["minimumNumberOfPoints"];
            $maximumNumberOfPoints = (int)$_POST["maximumNumberOfPoints"];
            $voucherAmount = (float)$_POST["voucherAmount"];
            if ($voucherIntervalDao->overlappingCheck($minimumNumberOfPoints, $maximumNumberOfPoints, $id)) {
                echo ApiResponse::respond(101);
                exit();
            }
            $voucherInterval = $voucherIntervalDao->getVoucherIntervalById($id);
            $voucherInterval->setPointMinimum($minimumNumberOfPoints);
            $voucherInterval->setPointMaximum($maximumNumberOfPoints);
            $voucherInterval->setMontantBonAchat($voucherAmount);
            $voucherIntervalDao->update($voucherInterval);
            $voucherAmountFormat = number_format($voucherAmount, 0, '', ' ');
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("loyalty.pointsIntervals.pointsIntervalUpdatedSuccessfully")} : {$minimumNumberOfPoints} - {$maximumNumberOfPoints} ($voucherAmountFormat)"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function deletepointsintervalAction()
    {
        try {
            $voucherIntervalDao = new VoucherIntervalDAO();
            $id = (int)$_POST["id"];
            $voucherInterval = $voucherIntervalDao->getVoucherIntervalById($id);
            if ($voucherInterval->getStatut() == 0) {
                echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE);
                exit();
            }
            $voucherInterval->setStatut(0);
            $voucherIntervalDao->update($voucherInterval);
            $minimumNumberOfPoints = $voucherInterval->getPointMinimum();
            $maximumNumberOfPoints = $voucherInterval->getPointMaximum();
            $voucherAmountFormat = number_format($voucherInterval->getMontantBonAchat(), 0, '', ' ');
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("loyalty.pointsIntervals.pointsIntervalDeletedSuccessfully")} : {$minimumNumberOfPoints} - {$maximumNumberOfPoints} ($voucherAmountFormat)"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    public function carddiscountsAction()
    {
        AccessRightChecker::checkAccessRight($this, 84);
        $productCategoryDao = new ProductCategoryDAO();
        $productCategories = $productCategoryDao->getAllProductCategories();
        $this->view->productCategories = $productCategories;
        $this->view->days = $this->days;
        $this->view->translator = $this->translator;
    }

    public function datatablecarddiscountsAction()
    {
        $startDate = $_POST["startDate"] != "" ? $_POST["startDate"] : null;
        $endDate = $_POST["endDate"] != "" ? $_POST["endDate"] : null;
        $whereStartDate = "";
        $whereEndDate = "";
        if ($startDate != null) {
            $whereStartDate .= " AND p.date_debut >= '{$startDate}'";
        }
        if ($endDate != null) {
            $whereEndDate .= " AND p.date_fin <= '{$endDate}'";
        }
        $where = "p.statut = 1{$whereStartDate}{$whereEndDate}";
        $pointValidityPeriodsTable = "
                (
                    SELECT p.id_periode_validite_point AS point_validity_period_id, p.date_debut AS point_validity_period_start, 
                    p.date_fin AS point_validity_period_end, p.pourcentage_reduction_carte AS point_validity_period_discount_percentage,
                    p.liste_jours_reduction AS point_validity_period_discount_days, p.categories_concernees AS point_validity_period_categories_involved
                    FROM periode_validite_point p
                    WHERE {$where}
                ) AS pointValidityPeriodsTable
        ";
        $primaryKey = 'point_validity_period_id';
        $columns = [
            ['db' => 'point_validity_period_start', 'dt' => 0],
            ['db' => 'point_validity_period_end', 'dt' => 1],
            ['db' => 'point_validity_period_discount_percentage', 'dt' => 2],
            ['db' => 'point_validity_period_discount_days', 'dt' => 3],
            ['db' => 'point_validity_period_categories_involved', 'dt' => 4],
            ['db' => 'point_validity_period_id', 'dt' => 5],
        ];
        $computeDatatableResults = SSP::customSimple($_POST, $pointValidityPeriodsTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        $currentDate = new DateTime();
        $currentTime = strtotime($currentDate->format(DATE_FORMAT));
        foreach ($datatableResult["data"] as $row) {
            $productCategoryDao = new ProductCategoryDAO();
            $startDate = new DateTime($row[0]);
            $startDateMoment = new \Moment\Moment($startDate->format(DATE_FORMAT));
            $endDate = new DateTime($row[1]);
            $endDateMoment = new \Moment\Moment($endDate->format(DATE_FORMAT));
            $discountPercentage = $row[2];
            $discountDays = explode(" ", $row[3]);
            $discountDaysString = "";
            foreach ($discountDays as $discountDay) {
                $discountDaysString .= "
                    <a style='margin-bottom: 5px;' class='ui label app-secondary-label'>{$this->days[$discountDay]}</a>
                ";
            }
            $categoriesInvolved = explode(" ", $row[4]);
            $categoriesInvolvedString = "";
            foreach ($categoriesInvolved as $categoryInvolved) {
                if ($categoryInvolved == "") {
                    $categoriesInvolvedString .= "<a style='margin-bottom: 5px;' class='ui label app-primary-label'>{$this->translator->_("loyalty.cardDiscounts.allCategories")}</a>";
                    break;
                }
                $productCategory = $productCategoryDao->getProductCategoryById((int)$categoryInvolved);
                $categoriesInvolvedString .= "
                    <a style='margin-bottom: 5px;' class='ui label app-secondary-label'>{$productCategory->getNom()}</a>
                ";
            }
            $pointValidityPeriodId = $row[5];
            $actions = "";
            if (strtotime($startDate->format(DATE_FORMAT)) > $currentTime) {
                $actions .= "
                    <button title='{$this->translator->_("shared.edit")}' about='{$pointValidityPeriodId}' class='mini ui icon button app-secondary-button app-edit-action'>
                        <i class='edit icon'></i>
                    </button>
                    <button title='{$this->translator->_("shared.delete")}' about='{$pointValidityPeriodId}' class='mini ui red icon button app-delete-action'>
                        <i class='trash icon'></i>
                    </button>
                ";
            }
            $tableRow = [
                $startDateMoment->format(DATE_FORMAT_MIDDLE),
                $endDateMoment->format(DATE_FORMAT_MIDDLE),
                $discountPercentage,
                $discountDaysString,
                $categoriesInvolvedString,
                $actions,
            ];
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    /**
     * 100 success
     * 101 category conflict
     * 102 unknown error
     */
    public function createcarddiscountAction()
    {
        try {
            $pointValidityPeriodDao = new PointValidityPeriodDAO();
            $startDate = new DateTime($_POST["startDate"]);
            $endDate = new DateTime($_POST["endDate"]);
            $discountPercentage = (float)$_POST["discountPercentage"];
            $discountDays = json_decode($_POST["discountDays"], true);
            $categoriesInvolved = json_decode($_POST["categoriesInvolved"], true);
            $discountsOverSamePeriod = $pointValidityPeriodDao->getOverlappingDiscounts($startDate, $endDate);
            $sameCategoryId = null;
            $sameDay = null;
            foreach ($discountsOverSamePeriod as $discountOverSamePeriod) {
                $periodCategories = explode(" ", $discountOverSamePeriod->getCategoriesConcernees());
                $sameSubCategoryId = null;
                $sameSubDay = null;
                if (count($categoriesInvolved) == 0) {
                    $sameSubCategoryId = 0;
                } else {
                    foreach ($periodCategories as $periodCategoryId) {
                        if ($periodCategoryId == "") {
                            $sameSubCategoryId = 0;
                            break;
                        }
                        if (in_array($periodCategoryId, $categoriesInvolved)) {
                            $sameSubCategoryId = $periodCategoryId;
                            break;
                        }
                    }
                }
                if (!is_null($sameSubCategoryId)) {
                    $periodDiscountDays = explode(" ", $discountOverSamePeriod->getListeJoursReduction());
                    foreach ($periodDiscountDays as $periodDiscountDay) {
                        if (in_array($periodDiscountDay, $discountDays)) {
                            $sameSubDay = $periodDiscountDay;
                            break;
                        }
                    }
                }
                if (!is_null($sameSubCategoryId) && !is_null($sameSubDay)) {
                    $sameCategoryId = $sameSubCategoryId;
                    $sameDay = $sameSubDay;
                    break;
                }
            }
            if (!is_null($sameCategoryId) && !is_null($sameDay)) {
                $categoryInvolvedLabel = "";
                if ((int)$sameCategoryId != 0) {
                    $productCategoryDao = new ProductCategoryDAO();
                    $productCategory = $productCategoryDao->getProductCategoryById((int)$sameCategoryId);
                    $categoryInvolvedLabel = $productCategory->getNom();
                } else {
                    $categoryInvolvedLabel = $this->translator->_("loyalty.cardDiscounts.allCategories");
                }
                $discountDayLabel = $this->days[$sameDay];
                echo ApiResponse::respond(101, null, ["category" => $categoryInvolvedLabel, "day" => $discountDayLabel]);
                exit();
            }
            $categoriesInvolvedString = "";
            $countCategoriesInvolved = 0;
            foreach ($categoriesInvolved as $categoryInvolved) {
                if ($countCategoriesInvolved == 0) {
                    $categoriesInvolvedString .= "{$categoryInvolved}";
                } else {
                    $categoriesInvolvedString .= " {$categoryInvolved}";
                }
                $countCategoriesInvolved++;
            }
            $discountDaysString = "";
            $countDiscountDays = 0;
            foreach ($discountDays as $discountDay) {
                if ($countDiscountDays == 0) {
                    $discountDaysString .= "{$discountDay}";
                } else {
                    $discountDaysString .= " {$discountDay}";
                }
                $countDiscountDays++;
            }
            $pointValidityPeriod = $pointValidityPeriodDao->create();
            $pointValidityPeriod->setDateDebut($startDate);
            $pointValidityPeriod->setDateFin($endDate);
            $pointValidityPeriod->setPourcentageReductionCarte($discountPercentage);
            $pointValidityPeriod->setListeJoursReduction($discountDaysString);
            $pointValidityPeriod->setCategoriesConcernees($categoriesInvolvedString);
            $pointValidityPeriod->setStatut(1);
            $pointValidityPeriodDao->save($pointValidityPeriod);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("loyalty.cardDiscounts.periodicDiscountCreatedSuccessfully")} : {$startDate->format(DATE_FORMAT)} - {$endDate->format(DATE_FORMAT)}"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function detailcarddiscountAction()
    {
        try {
            $pointValidityPeriodDao = new PointValidityPeriodDAO();
            $id = (int)$_POST["id"];
            $pointValidityPeriod = $pointValidityPeriodDao->getPointValidityPeriodById($id);
            $result = [
                "startDate" => $pointValidityPeriod->getDateDebut()->format(DATE_FORMAT),
                "endDate" => $pointValidityPeriod->getDateFin()->format(DATE_FORMAT),
                "discountPercentage" => $pointValidityPeriod->getPourcentageReductionCarte(),
                "discountDays" => explode(" ", $pointValidityPeriod->getListeJoursReduction()),
                "categoriesInvolved" => [],
            ];
            $concernedCategories = explode(" ", $pointValidityPeriod->getCategoriesConcernees());
            foreach ($concernedCategories as $concernedCategory) {
                if ($concernedCategory == "") {
                    break;
                }
                $result["categoriesInvolved"][] = $concernedCategory;
            }
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 category conflict
     * 102 unknown error
     */
    public function updatecarddiscountAction()
    {
        try {
            $pointValidityPeriodDao = new PointValidityPeriodDAO();
            $id = (int)$_POST["id"];
            $startDate = new DateTime($_POST["startDate"]);
            $endDate = new DateTime($_POST["endDate"]);
            $discountPercentage = (float)$_POST["discountPercentage"];
            $discountDays = json_decode($_POST["discountDays"], true);
            $categoriesInvolved = json_decode($_POST["categoriesInvolved"], true);
            $discountsOverSamePeriod = $pointValidityPeriodDao->getOverlappingDiscounts($startDate, $endDate, $id);
            $sameCategoryId = null;
            $sameDay = null;
            foreach ($discountsOverSamePeriod as $discountOverSamePeriod) {
                $periodCategories = explode(" ", $discountOverSamePeriod->getCategoriesConcernees());
                $sameSubCategoryId = null;
                $sameSubDay = null;
                if (count($categoriesInvolved) == 0) {
                    $sameSubCategoryId = 0;
                } else {
                    foreach ($periodCategories as $periodCategoryId) {
                        if ($periodCategoryId == "") {
                            $sameSubCategoryId = 0;
                            break;
                        }
                        if (in_array($periodCategoryId, $categoriesInvolved)) {
                            $sameSubCategoryId = $periodCategoryId;
                            break;
                        }
                    }
                }
                if (!is_null($sameSubCategoryId)) {
                    $periodDiscountDays = explode(" ", $discountOverSamePeriod->getListeJoursReduction());
                    foreach ($periodDiscountDays as $periodDiscountDay) {
                        if (in_array($periodDiscountDay, $discountDays)) {
                            $sameSubDay = $periodDiscountDay;
                            break;
                        }
                    }
                }
                if (!is_null($sameSubCategoryId) && !is_null($sameSubDay)) {
                    $sameCategoryId = $sameSubCategoryId;
                    $sameDay = $sameSubDay;
                    break;
                }
            }
            if (!is_null($sameCategoryId) && !is_null($sameDay)) {
                $categoryInvolvedLabel = "";
                if ((int)$sameCategoryId != 0) {
                    $productCategoryDao = new ProductCategoryDAO();
                    $productCategory = $productCategoryDao->getProductCategoryById((int)$sameCategoryId);
                    $categoryInvolvedLabel = $productCategory->getNom();
                } else {
                    $categoryInvolvedLabel = $this->translator->_("loyalty.cardDiscounts.allCategories");
                }
                $discountDayLabel = $this->days[$sameDay];
                echo ApiResponse::respond(101, null, ["category" => $categoryInvolvedLabel, "day" => $discountDayLabel]);
                exit();
            }
            $categoriesInvolvedString = "";
            $countCategoriesInvolved = 0;
            foreach ($categoriesInvolved as $categoryInvolved) {
                if ($countCategoriesInvolved == 0) {
                    $categoriesInvolvedString .= "{$categoryInvolved}";
                } else {
                    $categoriesInvolvedString .= " {$categoryInvolved}";
                }
                $countCategoriesInvolved++;
            }
            $discountDaysString = "";
            $countDiscountDays = 0;
            foreach ($discountDays as $discountDay) {
                if ($countDiscountDays == 0) {
                    $discountDaysString .= "{$discountDay}";
                } else {
                    $discountDaysString .= " {$discountDay}";
                }
                $countDiscountDays++;
            }
            $pointValidityPeriod = $pointValidityPeriodDao->getPointValidityPeriodById($id);
            $pointValidityPeriod->setDateDebut($startDate);
            $pointValidityPeriod->setDateFin($endDate);
            $pointValidityPeriod->setPourcentageReductionCarte($discountPercentage);
            $pointValidityPeriod->setListeJoursReduction($discountDaysString);
            $pointValidityPeriod->setCategoriesConcernees($categoriesInvolvedString);
            $pointValidityPeriodDao->update($pointValidityPeriod);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("loyalty.cardDiscounts.periodicDiscountUpdatedSuccessfully")} : {$startDate->format(DATE_FORMAT)} - {$endDate->format(DATE_FORMAT)}"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function deletecarddiscountAction()
    {
        try {
            $pointValidityPeriodDao = new PointValidityPeriodDAO();
            $id = (int)$_POST["id"];
            $pointValidityPeriod = $pointValidityPeriodDao->getPointValidityPeriodById($id);
            if ($pointValidityPeriod->getStatut() == 0) {
                echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE);
                exit();
            }
            $pointValidityPeriod->setStatut(0);
            $pointValidityPeriodDao->update($pointValidityPeriod);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("loyalty.cardDiscounts.periodicDiscountDeletedSuccessfully")} : {$pointValidityPeriod->getDateDebut()->format(DATE_FORMAT)} - {$pointValidityPeriod->getDateFin()->format(DATE_FORMAT)}"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    public function currentpointsAction()
    {
        AccessRightChecker::checkAccessRight($this, 105);
        $pointOfSaleDao = new PointOfSaleDAO();
        $currentUser = LogDAO::getCurrentUser();
        $pointsOfSale = [];
        if ($currentUser->getIdPointVente() == null || in_array($currentUser->getIdProfil()->getCode(), IDENTIFIED_PROFILES_CODES)) {
            $pointsOfSale = $pointOfSaleDao->listPointOfSales();
        } else {
            $pointsOfSale[] = $currentUser->getIdPointVente();
        }
        $this->view->pointsOfSale = $pointsOfSale;
        $this->view->translator = $this->translator;
    }

    public function datatablecurrentpointsAction()
    {
        $currentDate = new DateTime();
        $currentDateFormat = $currentDate->format(DATE_FORMAT_WITH_SECONDS);
        $customer = $_POST["customer"] != "" ? (int)$_POST["customer"] : null;
        $pointOfSale = $_POST["pointOfSale"] != "" ? (int)$_POST["pointOfSale"] : null;
        $minimumNumberOfPoints = $_POST["minimumNumberOfPoints"] != "" ? (int)$_POST["minimumNumberOfPoints"] : null;
        $maximumNumberOfPoints = $_POST["maximumNumberOfPoints"] != "" ? (int)$_POST["maximumNumberOfPoints"] : null;
        $whereCustomer = "";
        $wherePointOfSale = "";
        $havingMinimumNumberOfPoints = "";
        $havingMaximumNumberOfPoints = "";
        if ($customer != null) {
            $whereCustomer .= " AND c.id_personne = {$customer}";
        }
        if ($pointOfSale != null) {
            $wherePointOfSale .= " AND c.id_point_vente = {$pointOfSale}";
        }
        if ($minimumNumberOfPoints != null) {
            $havingMinimumNumberOfPoints .= " AND customer_points >= {$minimumNumberOfPoints}";
        }
        if ($maximumNumberOfPoints != null) {
            $havingMaximumNumberOfPoints .= " AND customer_points <= {$maximumNumberOfPoints}";
        }
        $where = "(p.date_expiration > '{$currentDateFormat}' OR p.id_point_carte_fidelite IS NULL) AND c.statut = 1 AND c.date_expiration > '{$currentDateFormat}'{$whereCustomer}{$wherePointOfSale}";
        $having = "customer_points >= 0{$havingMinimumNumberOfPoints}{$havingMaximumNumberOfPoints}";
        $currentPointsTable = "
                (
                    SELECT c.id_carte_fidelite AS card_id, c.code AS card_code, CONCAT(pe.nom,' ',COALESCE(pe.prenom, '')) AS customer_name,
                    ifnull(SUM(p.quantite_point), 0) AS customer_points, pv.nom_point_vente AS point_of_sale_name, c.date_expiration AS card_expiration_date
                    FROM carte_fidelite c JOIN point_vente pv ON c.id_point_vente = pv.id_point_vente JOIN personne pe 
                    ON c.id_personne = pe.id_personne LEFT OUTER JOIN point_carte_fidelite p 
                    ON (c.id_carte_fidelite = p.id_carte_fidelite OR p.id_point_carte_fidelite IS NULL)
                    WHERE {$where}
                    GROUP BY c.id_carte_fidelite
                    HAVING {$having}
                ) AS currentPointsTable
        ";
        $primaryKey = 'card_id';
        $columns = [
            ['db' => 'customer_name', 'dt' => 0],
            ['db' => 'point_of_sale_name', 'dt' => 1],
            ['db' => 'card_code', 'dt' => 2],
            ['db' => 'customer_points', 'dt' => 3],
            ['db' => 'card_expiration_date', 'dt' => 4],
        ];
        $computeDatatableResults = SSP::customSimple($_POST, $currentPointsTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        foreach ($datatableResult["data"] as $row) {
            $customerName = $row[0];
            $pointOfSale = $row[1];
            $cardCode = $row[2];
            $customerPoints = $row[3];
            $expirationDate = $row[4];
            $expirationDateMoment = new \Moment\Moment($expirationDate);
            $tableRow = [
                $customerName,
                $pointOfSale,
                $cardCode,
                $customerPoints,
                $expirationDateMoment->format(DATE_FORMAT_LONG),
            ];
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    public function vouchersAction()
    {
        AccessRightChecker::checkAccessRight($this, 89);
        $pointOfSaleDao = new PointOfSaleDAO();
        $currentUser = LogDAO::getCurrentUser();
        $pointsOfSale = [];
        if ($currentUser->getIdPointVente() == null || in_array($currentUser->getIdProfil()->getCode(), IDENTIFIED_PROFILES_CODES)) {
            $pointsOfSale = $pointOfSaleDao->listPointOfSales();
        } else {
            $pointsOfSale[] = $currentUser->getIdPointVente();
        }
        $this->view->pointsOfSale = $pointsOfSale;
        $this->view->translator = $this->translator;
    }

    public function datatablevouchersAction()
    {
        try {
            $currentDate = new DateTime();
            $currentTime = strtotime($currentDate->format(DATE_FORMAT_WITH_SECONDS)) . "vouchers";
            $pointsOfSale = json_decode($_POST["pointsOfSale"], true);
            $customer = (int)$_POST["customer"] != 0 ? (int)$_POST["customer"] : null;
            $startDate = $_POST["startDate"] != "" ? $_POST["startDate"] : null;
            $endDate = $_POST["endDate"] != "" ? $_POST["endDate"] : null;
            $wherePointsOfSale = "";
            $whereCustomer = "";
            $whereStartDate = "";
            $whereEndDate = "";
            if (count($pointsOfSale) > 0) {
                //We need to create a stored procedure here to explode the concerned points of sale string
                $customerVoucherDao = new CustomerVoucherDAO();
                $databaseConnection = $customerVoucherDao->entityManager->getConnection();
                $explodeProcedureSql = "
                    CREATE PROCEDURE explodeString_{$currentTime}(
                        IN inputString TEXT,
                        IN delimiterChar CHAR(1),
                        IN pointOfSale VARCHAR(7),
                        OUT isSelected TINYINT
                    )
                    BEGIN
                        DROP TEMPORARY TABLE IF EXISTS temp_string_{$currentTime};
                        CREATE TEMPORARY TABLE temp_string_{$currentTime}(vals text); 
                        WHILE LOCATE(delimiterChar,inputString) > 1 DO
                            INSERT INTO temp_string_{$currentTime} 
                            SELECT SUBSTRING_INDEX(inputString,delimiterChar,1);
                            SET inputString = SUBSTRING(inputString, INSTR(inputString, delimiterChar) + 1);
                        END WHILE;
                        INSERT INTO temp_string_{$currentTime}(vals) VALUES(inputString);
                        SELECT COUNT(*) INTO isSelected FROM temp_string_{$currentTime} WHERE vals = pointOfSale;
                    END;
                ";
                $explodeProcedureRequest = $databaseConnection->prepare($explodeProcedureSql);
                $explodeProcedureRequest->execute();
                $checkPointOfSaleSelectionFunctionSql = "
                    CREATE FUNCTION checkPointOfSaleIsSelected_{$currentTime}(
                        pointsOfSale TEXT,
                        pointOfSale VARCHAR(7),
                        delimiterChar CHAR(1)
                    )
                    RETURNS TINYINT
                    DETERMINISTIC
                    BEGIN
                        DECLARE pointOfSaleIsSelected TINYINT;
                        CALL explodeString_{$currentTime}(pointsOfSale, delimiterChar, pointOfSale, pointOfSaleIsSelected);
                        RETURN (pointOfSaleIsSelected);
                    END;
                ";
                $checkPointOfSaleSelectionFunctionRequest = $databaseConnection->prepare($checkPointOfSaleSelectionFunctionSql);
                $checkPointOfSaleSelectionFunctionRequest->execute();
                $countPointOfSale = 0;
                $wherePointsOfSale .= "  AND";
                if (count($pointsOfSale) > 1) {
                    $wherePointsOfSale .= " (";
                }
                foreach ($pointsOfSale as $pointOfSale) {
                    if (count($pointsOfSale) == 1) {
                        $wherePointsOfSale .= " checkPointOfSaleIsSelected_{$currentTime}(REPLACE(v.liste_points_vente_cible, ' ', '>'), {$pointOfSale}, '>') = 1";
                    } else {
                        if ($countPointOfSale == 0) {
                            $wherePointsOfSale .= " ( checkPointOfSaleIsSelected_{$currentTime}(REPLACE(v.liste_points_vente_cible, ' ', '>'), {$pointOfSale}, '>') = 1 )";
                        } else {
                            $wherePointsOfSale .= " AND ( checkPointOfSaleIsSelected_{$currentTime}(REPLACE(v.liste_points_vente_cible, ' ', '>'), {$pointOfSale}, '>') = 1 )";
                        }
                        $countPointOfSale++;
                    }
                }
                if (count($pointsOfSale) > 1) {
                    $wherePointsOfSale .= " )";
                }
            }
            if ($customer != null) {
                $whereCustomer .= " AND v.id_personne >= {$customer}";
            }
            if ($startDate != null) {
                $whereStartDate .= " AND v.date_emission >= '{$startDate}'";
            }
            if ($endDate != null) {
                $whereEndDate .= " AND v.date_emission <= '{$endDate}'";
            }
            $where = "v.statut = 1{$wherePointsOfSale}{$whereCustomer}{$whereStartDate}{$whereEndDate}";
            $vouchersTable = "
                (
                    SELECT v.id_bon_achat_client AS voucher_id, v.code_bon_achat_client AS voucher_code, CONCAT(p.nom,' ',COALESCE(p.prenom, '')) AS customer_name,
                    v.date_emission AS voucher_issue_date, v.date_expiration AS voucher_expiration_date, v.liste_points_vente_cible AS voucher_points_of_sale,
                    v.montant AS voucher_amount, v.valider AS voucher_validated, v.utiliser AS voucher_used, v.id_carte_fidelite AS voucher_loyalty_card_id
                    FROM bon_achat_client v JOIN personne p ON v.id_personne = p.id_personne
                    WHERE {$where}
                ) AS vouchersTable
            ";
            $primaryKey = 'voucher_id';
            $columns = [
                ['db' => 'voucher_code', 'dt' => 0],
                ['db' => 'voucher_points_of_sale', 'dt' => 1],
                ['db' => 'customer_name', 'dt' => 2],
                ['db' => 'voucher_issue_date', 'dt' => 3],
                ['db' => 'voucher_expiration_date', 'dt' => 4],
                ['db' => 'voucher_amount', 'dt' => 5],
                ['db' => 'voucher_validated', 'dt' => 6],
                ['db' => 'voucher_used', 'dt' => 7],
                ['db' => 'voucher_loyalty_card_id', 'dt' => 8],
                ['db' => 'voucher_id', 'dt' => 9],
            ];
            $computeDatatableResults = SSP::customSimple($_POST, $vouchersTable, $primaryKey, $columns);
            $datatableResult = $computeDatatableResults["datatable"];
            $finalResult = $computeDatatableResults["custom"];
            $pointOfSaleDao = new PointOfSaleDAO();
            foreach ($datatableResult["data"] as $row) {
                $voucherCode = $row[0];
                $voucherPointsOfSale = explode(" ", $row[1]);
                $voucherPointsOfSaleString = "";
                foreach ($voucherPointsOfSale as $voucherPointOfSale) {
                    $pointOfSale = $pointOfSaleDao->getPointOfSaleById((int)$voucherPointOfSale);
                    $voucherPointsOfSaleString .= "
                        <a style='margin-bottom: 5px;' class='ui label app-secondary-label'>{$pointOfSale->getNomPointVente()}</a>
                    ";
                }
                $customerName = $row[2];
                $voucherIssueDate = new DateTime($row[3]);
                $voucherExpirationDate = new DateTime($row[4]);
                $voucherAmount = number_format((float)$row[5], 0, '', ' ');
                $voucherValidated = (int)$row[6];
                $voucherUsed = (int)$row[7];
                $voucherLoyaltyCardId = $row[8];
                $voucherId = (int)$row[9];
                $actions = "";
                if ($voucherValidated == 0) {
                    $actions .= "
                        <button title='{$this->translator->_("shared.edit")}' about='{$voucherId}' class='mini ui icon button app-secondary-button app-edit-action'>
                            <i class='edit icon'></i>
                        </button>
                        <button title='{$this->translator->_("shared.delete")}' about='{$voucherId}' class='mini ui red icon button app-delete-action'>
                            <i class='trash icon'></i>
                        </button>
                    ";
                } else {
                    $actions .= "
                        <button title='{$this->translator->_("shared.print")}' about='{$voucherId}' class='mini ui purple icon button app-print-action'>
                            <i class='print icon'></i>
                        </button>
                    ";
                }
                $tableRow = [
                    $voucherCode,
                    $voucherPointsOfSaleString,
                    $customerName,
                    $voucherIssueDate->format(DATE_FORMAT),
                    $voucherExpirationDate->format(DATE_FORMAT),
                    $voucherAmount,
                    $voucherUsed == 1 ?
                        "<a class='ui label app-secondary-label'>{$this->translator->_("loyalty.vouchers.used")}</a>" :
                        "<a class='ui label app-primary-label'>{$this->translator->_("loyalty.vouchers.unused")}</a>",
                    $voucherLoyaltyCardId != null ?
                        "<a class='ui label app-secondary-label'>{$this->translator->_("loyalty.vouchers.loyaltyCard")}</a>" :
                        "<a class='ui label app-primary-label'>{$this->translator->_("loyalty.vouchers.freelyCreated")}</a>",
                    $actions,
                ];
                $finalResult["data"][] = $tableRow;
            }
            if (count($pointsOfSale) > 0) {
                $deleteTemporaryFunctionProcedureTableSql = "
                    DROP PROCEDURE IF EXISTS explodeString_{$currentTime};
                    DROP TEMPORARY TABLE IF EXISTS temp_string_{$currentTime};
                    DROP FUNCTION IF EXISTS checkPointOfSaleIsSelected_{$currentTime};
                ";
                $deleteTemporaryFunctionProcedureTableRequest = $databaseConnection->prepare($deleteTemporaryFunctionProcedureTableSql);
                $deleteTemporaryFunctionProcedureTableRequest->execute();
            }
            echo json_encode($finalResult);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function createvoucherAction()
    {
        $customerVoucherDao = new CustomerVoucherDAO();
        $databaseConnection = $customerVoucherDao->entityManager->getConnection();
        $databaseConnection->beginTransaction();
        try {
            $pointOfSaleDao = new PointOfSaleDAO();
            $personDao = new PersonDAO();
            $pointsOfSale = json_decode($_POST["pointsOfSale"], true);
            $customerId = (int)$_POST["customer"];
            $customer = $personDao->getPersonById($customerId);
            $amount = (float)$_POST["amount"];
            $expirationDate = new DateTime($_POST["expirationDate"]);
            $customerVoucher = $customerVoucherDao->create();
            $currentDate = new DateTime();
            $customerVoucher->setCodeBonAchatClient("{$currentDate->format(DATE_FORMAT_YEAR)}{$currentDate->format(DATE_FORMAT_MONTH_NUMERIC)}{$currentDate->format(DATE_FORMAT_DAY_NUMERIC)}");
            $customerVoucher->setDateEmission($currentDate);
            $customerVoucher->setDateExpiration($expirationDate);
            $customerVoucher->setIdPersonne($customer);
            $countPointOfSale = 0;
            $pointsOfSaleString = "";
            foreach ($pointsOfSale as $pointOfSaleId) {
                if ($countPointOfSale == 0) {
                    $pointOfSale = $pointOfSaleDao->getPointOfSaleById((int)$pointOfSaleId);
                    $customerVoucher->setIdPointVente($pointOfSale);
                    $pointsOfSaleString .= "{$pointOfSaleId}";
                } else {
                    $pointsOfSaleString .= " {$pointOfSaleId}";
                }
                $countPointOfSale++;
            }
            $customerVoucher->setListePointsVenteCible($pointsOfSaleString);
            $customerVoucher->setMontant($amount);
            $customerVoucher->setStatut(1);
            $customerVoucher->setUtiliser(0);
            $customerVoucher->setValider(0);
            $customerVoucher->setIdUtilisateur(LogDAO::getCurrentUser());
            $customerVoucherDao->save($customerVoucher);
            $customerVoucherSuffix = "{$customerVoucher->getIdBonAchatClient()}";
            if (strlen($customerVoucherSuffix) % 2 != 0) {
                $customerVoucherSuffix = "0{$customerVoucherSuffix}";
            }
            $customerVoucher->setCodeBonAchatClient("{$customerVoucher->getCodeBonAchatClient()}{$customerVoucherSuffix}");
            $customerVoucherDao->update($customerVoucher);
            $databaseConnection->commit();
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("loyalty.vouchers.voucherCreatedSuccessfully")} : N° {$customerVoucher->getCodeBonAchatClient()}"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            $databaseConnection->rollBack();
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function detailvoucherAction()
    {
        try {
            $customerVoucherDao = new CustomerVoucherDAO();
            $id = (int)$_POST["id"];
            $customerVoucher = $customerVoucherDao->getCustomerVoucherById($id);
            $result = [
                "pointsOfSale" => explode(" ", $customerVoucher->getListePointsVenteCible()),
                "customer" => $customerVoucher->getIdPersonne()->getIdPersonne(),
                "customerName" => "{$customerVoucher->getIdPersonne()->getNom()} {$customerVoucher->getIdPersonne()->getPrenom()}",
                "amount" => $customerVoucher->getMontant(),
                "expirationDate" => $customerVoucher->getDateExpiration()->format(DATE_FORMAT),
            ];
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function updatevoucherAction()
    {
        $customerVoucherDao = new CustomerVoucherDAO();
        $databaseConnection = $customerVoucherDao->entityManager->getConnection();
        $databaseConnection->beginTransaction();
        try {
            $pointOfSaleDao = new PointOfSaleDAO();
            $personDao = new PersonDAO();
            $id = (int)$_POST["id"];
            $pointsOfSale = json_decode($_POST["pointsOfSale"], true);
            $customerId = (int)$_POST["customer"];
            $customer = $personDao->getPersonById($customerId);
            $amount = (float)$_POST["amount"];
            $expirationDate = new DateTime($_POST["expirationDate"]);
            $customerVoucher = $customerVoucherDao->getCustomerVoucherById($id);
            $customerVoucher->setDateExpiration($expirationDate);
            $customerVoucher->setIdPersonne($customer);
            $countPointOfSale = 0;
            $pointsOfSaleString = "";
            foreach ($pointsOfSale as $pointOfSaleId) {
                if ($countPointOfSale == 0) {
                    $pointOfSale = $pointOfSaleDao->getPointOfSaleById((int)$pointOfSaleId);
                    $customerVoucher->setIdPointVente($pointOfSale);
                    $pointsOfSaleString .= "{$pointOfSaleId}";
                } else {
                    $pointsOfSaleString .= " {$pointOfSaleId}";
                }
                $countPointOfSale++;
            }
            $customerVoucher->setListePointsVenteCible($pointsOfSaleString);
            $customerVoucher->setMontant($amount);
            $customerVoucherDao->update($customerVoucher);
            $databaseConnection->commit();
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("loyalty.vouchers.voucherUpdatedSuccessfully")} : N° {$customerVoucher->getCodeBonAchatClient()}"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            $databaseConnection->rollBack();
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function deletevoucherAction()
    {
        try {
            $customerVoucherDao = new CustomerVoucherDAO();
            $id = (int)$_POST["id"];
            $customerVoucher = $customerVoucherDao->getCustomerVoucherById($id);
            if ($customerVoucher->getStatut() == 0) {
                echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE);
                exit();
            }
            if ($customerVoucher->getValider() == 1) {
                echo ApiResponse::respond(ALREADY_VALIDATED_ISSUE_CODE);
                exit();
            }
            $customerVoucher->setStatut(0);
            $customerVoucherDao->update($customerVoucher);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("loyalty.vouchers.voucherDeletedSuccessfully")} : N°{$customerVoucher->getCodeBonAchatClient()}"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    public function vouchervalidationAction()
    {
        AccessRightChecker::checkAccessRight($this, 155);
        $pointOfSaleDao = new PointOfSaleDAO();
        $currentUser = LogDAO::getCurrentUser();
        $pointsOfSale = [];
        if ($currentUser->getIdPointVente() == null || in_array($currentUser->getIdProfil()->getCode(), IDENTIFIED_PROFILES_CODES)) {
            $pointsOfSale = $pointOfSaleDao->listPointOfSales();
        } else {
            $pointsOfSale[] = $currentUser->getIdPointVente();
        }
        $this->view->pointsOfSale = $pointsOfSale;
        $this->view->translator = $this->translator;
    }

    public function datatablevouchervalidationAction()
    {
        try {
            $currentDate = new DateTime();
            $currentTime = strtotime($currentDate->format(DATE_FORMAT_WITH_SECONDS));
            $pointsOfSale = json_decode($_POST["pointsOfSale"], true);
            $customer = (int)$_POST["customer"] != 0 ? (int)$_POST["customer"] : null;
            $startDate = $_POST["startDate"] != "" ? $_POST["startDate"] : null;
            $endDate = $_POST["endDate"] != "" ? $_POST["endDate"] : null;
            $wherePointsOfSale = "";
            $whereCustomer = "";
            $whereStartDate = "";
            $whereEndDate = "";
            if (count($pointsOfSale) > 0) {
                //We need to create a stored procedure here to explode the concerned points of sale string
                $customerVoucherDao = new CustomerVoucherDAO();
                $databaseConnection = $customerVoucherDao->entityManager->getConnection();
                $explodeProcedureSql = "
                    CREATE PROCEDURE explodeString_{$currentTime}(
                        IN inputString TEXT,
                        IN delimiterChar CHAR(1),
                        IN pointOfSale VARCHAR(7),
                        OUT isSelected TINYINT
                    )
                    BEGIN
                        DROP TEMPORARY TABLE IF EXISTS temp_string_{$currentTime};
                        CREATE TEMPORARY TABLE temp_string_{$currentTime}(vals text); 
                        WHILE LOCATE(delimiterChar,inputString) > 1 DO
                            INSERT INTO temp_string_{$currentTime} 
                            SELECT SUBSTRING_INDEX(inputString,delimiterChar,1);
                            SET inputString = SUBSTRING(inputString, INSTR(inputString, delimiterChar) + 1);
                        END WHILE;
                        INSERT INTO temp_string_{$currentTime}(vals) VALUES(inputString);
                        SELECT COUNT(*) INTO isSelected FROM temp_string_{$currentTime} WHERE vals = pointOfSale;
                    END;
                ";
                $explodeProcedureRequest = $databaseConnection->prepare($explodeProcedureSql);
                $explodeProcedureRequest->execute();
                $checkPointOfSaleSelectionFunctionSql = "
                    CREATE FUNCTION checkPointOfSaleIsSelected_{$currentTime}(
                        pointsOfSale TEXT,
                        pointOfSale VARCHAR(7),
                        delimiterChar CHAR(1)
                    )
                    RETURNS TINYINT
                    DETERMINISTIC
                    BEGIN
                        DECLARE pointOfSaleIsSelected TINYINT;
                        CALL explodeString_{$currentTime}(pointsOfSale, delimiterChar, pointOfSale, pointOfSaleIsSelected);
                        RETURN (pointOfSaleIsSelected);
                    END;
                ";
                $checkPointOfSaleSelectionFunctionRequest = $databaseConnection->prepare($checkPointOfSaleSelectionFunctionSql);
                $checkPointOfSaleSelectionFunctionRequest->execute();
                $countPointOfSale = 0;
                $wherePointsOfSale .= "  AND";
                if (count($pointsOfSale) > 1) {
                    $wherePointsOfSale .= " (";
                }
                foreach ($pointsOfSale as $pointOfSale) {
                    if (count($pointsOfSale) == 1) {
                        $wherePointsOfSale .= " checkPointOfSaleIsSelected_{$currentTime}(REPLACE(v.liste_points_vente_cible, ' ', '>'), {$pointOfSale}, '>') = 1";
                    } else {
                        if ($countPointOfSale == 0) {
                            $wherePointsOfSale .= " ( checkPointOfSaleIsSelected_{$currentTime}(REPLACE(v.liste_points_vente_cible, ' ', '>'), {$pointOfSale}, '>') = 1 )";
                        } else {
                            $wherePointsOfSale .= " AND ( checkPointOfSaleIsSelected_{$currentTime}(REPLACE(v.liste_points_vente_cible, ' ', '>'), {$pointOfSale}, '>') = 1 )";
                        }
                        $countPointOfSale++;
                    }
                }
                if (count($pointsOfSale) > 1) {
                    $wherePointsOfSale .= " )";
                }
            }
            if ($customer != null) {
                $whereCustomer .= " AND v.id_personne >= {$customer}";
            }
            if ($startDate != null) {
                $whereStartDate .= " AND v.date_emission >= '{$startDate}'";
            }
            if ($endDate != null) {
                $whereEndDate .= " AND v.date_emission <= '{$endDate}'";
            }
            $where = "v.statut = 1 AND v.valider = 0{$wherePointsOfSale}{$whereCustomer}{$whereStartDate}{$whereEndDate}";
            $vouchersValidationTable = "
                (
                    SELECT v.id_bon_achat_client AS voucher_id, v.code_bon_achat_client AS voucher_code, CONCAT(p.nom,' ',COALESCE(p.prenom, '')) AS customer_name,
                    v.date_emission AS voucher_issue_date, v.date_expiration AS voucher_expiration_date, v.liste_points_vente_cible AS voucher_points_of_sale,
                    v.montant AS voucher_amount, v.valider AS voucher_validated, v.utiliser AS voucher_used
                    FROM bon_achat_client v JOIN personne p ON v.id_personne = p.id_personne
                    WHERE {$where}
                ) AS vouchersValidationTable
            ";
            $primaryKey = 'voucher_id';
            $columns = [
                ['db' => 'voucher_code', 'dt' => 0],
                ['db' => 'voucher_points_of_sale', 'dt' => 1],
                ['db' => 'customer_name', 'dt' => 2],
                ['db' => 'voucher_issue_date', 'dt' => 3],
                ['db' => 'voucher_expiration_date', 'dt' => 4],
                ['db' => 'voucher_amount', 'dt' => 5],
                ['db' => 'voucher_id', 'dt' => 6],
            ];
            $computeDatatableResults = SSP::customSimple($_POST, $vouchersValidationTable, $primaryKey, $columns);
            $datatableResult = $computeDatatableResults["datatable"];
            $finalResult = $computeDatatableResults["custom"];
            $pointOfSaleDao = new PointOfSaleDAO();
            foreach ($datatableResult["data"] as $row) {
                $voucherCode = $row[0];
                $voucherPointsOfSale = explode(" ", $row[1]);
                $voucherPointsOfSaleString = "";
                foreach ($voucherPointsOfSale as $voucherPointOfSale) {
                    $pointOfSale = $pointOfSaleDao->getPointOfSaleById((int)$voucherPointOfSale);
                    $voucherPointsOfSaleString .= "
                        <a style='margin-bottom: 5px;' class='ui label app-secondary-label'>{$pointOfSale->getNomPointVente()}</a>
                    ";
                }
                $customerName = $row[2];
                $voucherIssueDate = new DateTime($row[3]);
                $voucherExpirationDate = new DateTime($row[4]);
                $voucherAmount = number_format((float)$row[5], 0, '', ' ');
                $voucherId = (int)$row[6];
                $actions = "
                    <button title='{$this->translator->_("shared.validate")}' about='{$voucherId}' class='mini ui green icon button app-validate-action'>
                        <i class='check icon'></i>
                    </button>
                ";
                $tableRow = [
                    $voucherCode,
                    $voucherPointsOfSaleString,
                    $customerName,
                    $voucherIssueDate->format(DATE_FORMAT),
                    $voucherExpirationDate->format(DATE_FORMAT),
                    $voucherAmount,
                    $actions,
                ];
                $finalResult["data"][] = $tableRow;
            }
            if (count($pointsOfSale) > 0) {
                $deleteTemporaryFunctionProcedureTableSql = "
                    DROP PROCEDURE IF EXISTS explodeString_{$currentTime};
                    DROP TEMPORARY TABLE IF EXISTS temp_string_{$currentTime};
                    DROP FUNCTION IF EXISTS checkPointOfSaleIsSelected_{$currentTime};
                ";
                $deleteTemporaryFunctionProcedureTableRequest = $databaseConnection->prepare($deleteTemporaryFunctionProcedureTableSql);
                $deleteTemporaryFunctionProcedureTableRequest->execute();
            }
            echo json_encode($finalResult);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function validatevoucherAction()
    {
        try {
            $id = (int)$_POST["id"];
            $customerVoucherDao = new CustomerVoucherDAO();;
            $customerVoucher = $customerVoucherDao->getCustomerVoucherById($id);
            if ($customerVoucher->getStatut() == 0) {
                echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE);
                exit();
            }
            if ($customerVoucher->getValider() == 1) {
                echo ApiResponse::respond(ALREADY_VALIDATED_ISSUE_CODE);
                exit();
            }
            $customerVoucher->setValider(1);
            $customerVoucher->setDateValidation(new DateTime());
            $customerVoucherDao->update($customerVoucher);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("loyalty.voucherValidation.voucherValidatedSuccessfully")} : N°{$customerVoucher->getCodeBonAchatClient()}"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    public function convertpointsAction()
    {
        AccessRightChecker::checkAccessRight($this, 156);
        $pointOfSaleDao = new PointOfSaleDAO();
        $currentUser = LogDAO::getCurrentUser();
        $pointsOfSale = [];
        if ($currentUser->getIdPointVente() == null || in_array($currentUser->getIdProfil()->getCode(), IDENTIFIED_PROFILES_CODES)) {
            $pointsOfSale = $pointOfSaleDao->listPointOfSales();
        } else {
            $pointsOfSale[] = $currentUser->getIdPointVente();
        }
        $this->view->pointsOfSale = $pointsOfSale;
        $this->view->translator = $this->translator;
    }

    public function datatableconvertpointsAction()
    {
        $currentDate = new DateTime();
        $currentDateFormat = $currentDate->format(DATE_FORMAT_WITH_SECONDS);
        $customer = $_POST["customer"] != "" ? (int)$_POST["customer"] : null;
        $pointOfSale = $_POST["pointOfSale"] != "" ? (int)$_POST["pointOfSale"] : null;
        $whereCustomer = "";
        $wherePointOfSale = "";
        if ($customer != null) {
            $whereCustomer .= " AND c.id_personne = {$customer}";
        }
        if ($pointOfSale != null) {
            $wherePointOfSale .= " AND c.id_point_vente = {$pointOfSale}";
        }
        $where = "(p.date_expiration > '{$currentDateFormat}' OR p.id_point_carte_fidelite IS NULL) AND c.statut = 1 AND c.date_expiration > '{$currentDateFormat}'{$whereCustomer}{$wherePointOfSale}";
        $convertPointsTable = "
                (
                    SELECT c.id_carte_fidelite AS card_id, c.code AS card_code, CONCAT(pe.nom,' ',COALESCE(pe.prenom, '')) AS customer_name,
                    ifnull(SUM(p.quantite_point), 0) AS customer_points, pv.nom_point_vente AS point_of_sale_name, c.date_expiration AS card_expiration_date
                    FROM carte_fidelite c JOIN point_vente pv ON c.id_point_vente = pv.id_point_vente JOIN personne pe 
                    ON c.id_personne = pe.id_personne LEFT OUTER JOIN point_carte_fidelite p 
                    ON (c.id_carte_fidelite = p.id_carte_fidelite OR p.id_point_carte_fidelite IS NULL)
                    WHERE {$where}
                    GROUP BY c.id_carte_fidelite
                    HAVING customer_points > 0
                ) AS convertPointsTable
        ";
        $primaryKey = 'card_id';
        $columns = [
            ['db' => 'customer_name', 'dt' => 0],
            ['db' => 'point_of_sale_name', 'dt' => 1],
            ['db' => 'card_code', 'dt' => 2],
            ['db' => 'customer_points', 'dt' => 3],
            ['db' => 'card_expiration_date', 'dt' => 4],
            ['db' => 'card_id', 'dt' => 5],
        ];
        $computeDatatableResults = SSP::customSimple($_POST, $convertPointsTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        foreach ($datatableResult["data"] as $row) {
            $customerName = $row[0];
            $pointOfSale = $row[1];
            $cardCode = $row[2];
            $customerPoints = $row[3];
            $expirationDate = $row[4];
            $expirationDateMoment = new \Moment\Moment($expirationDate);
            $cardId = (int)$row[5];
            $actions = "
                <button title='{$this->translator->_("loyalty.convertPoints.convertThePoints")}' about='{$cardId}' class='mini ui violet icon button convert-points-action'>
                    <i class='share square icon'></i>
                </button>
            ";
            $tableRow = [
                $customerName,
                $pointOfSale,
                $cardCode,
                $customerPoints,
                $expirationDateMoment->format(DATE_FORMAT_LONG),
                $actions,
            ];
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    /**
     * 100 success
     * 101 insufficient points
     * 102 unknown error
     */
    public function convertpointsofcardAction()
    {
        $voucherDefaultDuration = (int)DUREE_DEFAULT_BON_ACHAT_CLIENT;
        $currentDate = new DateTime();
        $expirationDate = clone $currentDate;
        $expirationDate->modify(" + {$voucherDefaultDuration} days");
        $loyaltyCardDao = new LoyaltyCardDAO();
        $loyaltyCardPointDao = new LoyaltyCardPointDAO();
        $voucherIntervalDao = new VoucherIntervalDAO();
        $customerVoucherDao = new CustomerVoucherDAO();
        $databaseConnection = $loyaltyCardDao->entityManager->getConnection();
        $databaseConnection->beginTransaction();
        try {
            $loyaltyCardId = (int)$_POST["id"];
            $loyaltyCard = $loyaltyCardDao->getLoyaltyCardById($loyaltyCardId);
            $weCouldConvertSomePoints = false;
            $pointsOfLoyaltyCard = $loyaltyCardPointDao->getPointsOfLoyaltyCard($loyaltyCard);
            $possibleVoucherInterval = $voucherIntervalDao->getVoucherIntervalIncluingPoints($pointsOfLoyaltyCard);
            $customerVoucher = null;
            if ($possibleVoucherInterval != null) {
                $customerVoucher = $customerVoucherDao->create();
                $customerVoucher->setCodeBonAchatClient("{$currentDate->format(DATE_FORMAT_YEAR)}{$currentDate->format(DATE_FORMAT_MONTH_NUMERIC)}{$currentDate->format(DATE_FORMAT_DAY_NUMERIC)}");
                $customerVoucher->setDateEmission($currentDate);
                $customerVoucher->setDateExpiration($expirationDate);
                $customerVoucher->setIdPersonne($loyaltyCard->getIdPersonne());
                $customerVoucher->setIdPointVente($loyaltyCard->getIdPointVente());
                $customerVoucher->setListePointsVenteCible("{$loyaltyCard->getIdPointVente()->getIdPointVente()}");
                $customerVoucher->setMontant($possibleVoucherInterval->getMontantBonAchat());
                $customerVoucher->setStatut(1);
                $customerVoucher->setUtiliser(0);
                $customerVoucher->setValider(1);
                $customerVoucher->setIdUtilisateur(LogDAO::getCurrentUser());
                $customerVoucher->setIdCarteFidelite($loyaltyCard);
                $customerVoucher->setDateValidation($currentDate);
                $customerVoucherDao->save($customerVoucher);
                $customerVoucherSuffix = "{$customerVoucher->getIdBonAchatClient()}";
                if (strlen($customerVoucherSuffix) % 2 != 0) {
                    $customerVoucherSuffix = "0{$customerVoucherSuffix}";
                }
                $customerVoucher->setCodeBonAchatClient("{$customerVoucher->getCodeBonAchatClient()}{$customerVoucherSuffix}");
                $customerVoucherDao->update($customerVoucher);
                $availableLoyaltyCardPointRecords = $loyaltyCardPointDao->getAllPointsOfLoyaltyCard($loyaltyCard);
                $pointsConsumedCovered = 0;
                foreach ($availableLoyaltyCardPointRecords as $loyaltyCardPoint) {
                    $pointsPossibleToCover = $loyaltyCardPoint->getQuantitePoint() >= ($pointsOfLoyaltyCard - $pointsConsumedCovered) ? ($pointsOfLoyaltyCard - $pointsConsumedCovered) : $loyaltyCardPoint->getQuantitePoint();
                    $loyaltyCardPoint->setDateExpiration($currentDate);
                    if ($pointsPossibleToCover < $loyaltyCardPoint->getQuantitePoint()) {
                        $restOfLoyaltyCardPoint = $loyaltyCardPointDao->create();
                        $restOfLoyaltyCardPoint->setDateAssignation($loyaltyCardPoint->getDateAssignation());
                        $restOfLoyaltyCardPoint->setDateExpiration($loyaltyCard->getDateExpiration());
                        $restOfLoyaltyCardPoint->setIdCarteFidelite($loyaltyCard);
                        $restOfLoyaltyCardPoint->setQuantitePoint($loyaltyCardPoint->getQuantitePoint() - $pointsPossibleToCover);
                        $loyaltyCardPoint->setQuantitePoint($pointsPossibleToCover);
                        $loyaltyCardPointDao->save($restOfLoyaltyCardPoint);
                    }
                    $loyaltyCardPointDao->update($loyaltyCardPoint);
                    $pointsConsumedCovered += $pointsPossibleToCover;
                    if ($pointsConsumedCovered == $pointsOfLoyaltyCard) break;
                }
                $weCouldConvertSomePoints = true;
            } else {
                if ($pointsOfLoyaltyCard >= (int)NOMBRE_POINT_MIN_POUR_BON_ACHAT) {
                    $timesMinimumNumberOfPoints = (int)($pointsOfLoyaltyCard / (int)NOMBRE_POINT_MIN_POUR_BON_ACHAT);
                    $customerVoucherAmount = (float)($timesMinimumNumberOfPoints * (float)MONTANT_BON_ACHAT_PAR_NOMBRE_POINT_MIN);
                    $customerVoucher = $customerVoucherDao->create();
                    $customerVoucher->setCodeBonAchatClient("{$currentDate->format(DATE_FORMAT_YEAR)}{$currentDate->format(DATE_FORMAT_MONTH_NUMERIC)}{$currentDate->format(DATE_FORMAT_DAY_NUMERIC)}");
                    $customerVoucher->setDateEmission($currentDate);
                    $customerVoucher->setDateExpiration($expirationDate);
                    $customerVoucher->setIdPersonne($loyaltyCard->getIdPersonne());
                    $customerVoucher->setIdPointVente($loyaltyCard->getIdPointVente());
                    $customerVoucher->setListePointsVenteCible("{$loyaltyCard->getIdPointVente()->getIdPointVente()}");
                    $customerVoucher->setMontant($customerVoucherAmount);
                    $customerVoucher->setStatut(1);
                    $customerVoucher->setUtiliser(0);
                    $customerVoucher->setValider(1);
                    $customerVoucher->setIdUtilisateur(LogDAO::getCurrentUser());
                    $customerVoucher->setIdCarteFidelite($loyaltyCard);
                    $customerVoucher->setDateValidation($currentDate);
                    $customerVoucherDao->save($customerVoucher);
                    $customerVoucherSuffix = "{$customerVoucher->getIdBonAchatClient()}";
                    if (strlen($customerVoucherSuffix) % 2 != 0) {
                        $customerVoucherSuffix = "0{$customerVoucherSuffix}";
                    }
                    $customerVoucher->setCodeBonAchatClient("{$customerVoucher->getCodeBonAchatClient()}{$customerVoucherSuffix}");
                    $customerVoucherDao->update($customerVoucher);
                    $availableLoyaltyCardPointRecords = $loyaltyCardPointDao->getAllPointsOfLoyaltyCard($loyaltyCard);
                    $pointsToBeCovered = $timesMinimumNumberOfPoints * (int)NOMBRE_POINT_MIN_POUR_BON_ACHAT;
                    $pointsConsumedCovered = 0;
                    foreach ($availableLoyaltyCardPointRecords as $loyaltyCardPoint) {
                        $pointsPossibleToCover = $loyaltyCardPoint->getQuantitePoint() >= ($pointsToBeCovered - $pointsConsumedCovered) ? ($pointsToBeCovered - $pointsConsumedCovered) : $loyaltyCardPoint->getQuantitePoint();
                        $loyaltyCardPoint->setDateExpiration($currentDate);
                        if ($pointsPossibleToCover < $loyaltyCardPoint->getQuantitePoint()) {
                            $restOfLoyaltyCardPoint = $loyaltyCardPointDao->create();
                            $restOfLoyaltyCardPoint->setDateAssignation($loyaltyCardPoint->getDateAssignation());
                            $restOfLoyaltyCardPoint->setDateExpiration($loyaltyCard->getDateExpiration());
                            $restOfLoyaltyCardPoint->setIdCarteFidelite($loyaltyCard);
                            $restOfLoyaltyCardPoint->setQuantitePoint($loyaltyCardPoint->getQuantitePoint() - $pointsPossibleToCover);
                            $loyaltyCardPoint->setQuantitePoint($pointsPossibleToCover);
                            $loyaltyCardPointDao->save($restOfLoyaltyCardPoint);
                        }
                        $loyaltyCardPointDao->update($loyaltyCardPoint);
                        $pointsConsumedCovered += $pointsPossibleToCover;
                        if ($pointsConsumedCovered == $pointsToBeCovered) break;
                    }
                    $weCouldConvertSomePoints = true;
                }
            }
            $databaseConnection->commit();
            if ($weCouldConvertSomePoints) {
                LogDAO::registerOperationForCurrentUser(
                    "{$this->translator->_("loyalty.convertPoints.pointsSuccessfullyConverted")} : {$this->translator->_("loyalty.convertPoints.loyaltyCardNumber")} {$loyaltyCard->getCode()}({$loyaltyCard->getIdPersonne()->getNom()} {$loyaltyCard->getIdPersonne()->getPrenom()}) => {$this->translator->_("loyalty.convertPoints.voucherNumber")} {$customerVoucher->getCodeBonAchatClient()}"
                );
                echo ApiResponse::respond();
            } else {
                echo ApiResponse::respond(101);
            }
        } catch (Exception $ex) {
            $databaseConnection->rollBack();
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function convertpointsofallcardsAction()
    {
        $voucherDefaultDuration = (int)DUREE_DEFAULT_BON_ACHAT_CLIENT;
        $currentDate = new DateTime();
        $expirationDate = clone $currentDate;
        $expirationDate->modify(" + {$voucherDefaultDuration} days");
        $loyaltyCardDao = new LoyaltyCardDAO();
        $loyaltyCardPointDao = new LoyaltyCardPointDAO();
        $voucherIntervalDao = new VoucherIntervalDAO();
        $customerVoucherDao = new CustomerVoucherDAO();
        $pointOfSaleDao = new PointOfSaleDAO();
        $databaseConnection = $loyaltyCardDao->entityManager->getConnection();
        $databaseConnection->beginTransaction();
        try {
            $pointOfSaleId = (int)$_POST["pointOfSale"];
            $pointOfSale = $pointOfSaleDao->getPointOfSaleById($pointOfSaleId);
            $loyaltyCardsOfPointOfSale = $loyaltyCardDao->getActiveLoyaltyCardsByPointOfSale($pointOfSale);
            foreach ($loyaltyCardsOfPointOfSale as $loyaltyCard) {
                $pointsOfLoyaltyCard = $loyaltyCardPointDao->getPointsOfLoyaltyCard($loyaltyCard);
                $possibleVoucherInterval = $voucherIntervalDao->getVoucherIntervalIncluingPoints($pointsOfLoyaltyCard);
                $customerVoucher = null;
                if ($possibleVoucherInterval != null) {
                    $customerVoucher = $customerVoucherDao->create();
                    $customerVoucher->setCodeBonAchatClient("{$currentDate->format(DATE_FORMAT_YEAR)}{$currentDate->format(DATE_FORMAT_MONTH_NUMERIC)}{$currentDate->format(DATE_FORMAT_DAY_NUMERIC)}");
                    $customerVoucher->setDateEmission($currentDate);
                    $customerVoucher->setDateExpiration($expirationDate);
                    $customerVoucher->setIdPersonne($loyaltyCard->getIdPersonne());
                    $customerVoucher->setIdPointVente($loyaltyCard->getIdPointVente());
                    $customerVoucher->setListePointsVenteCible("{$loyaltyCard->getIdPointVente()->getIdPointVente()}");
                    $customerVoucher->setMontant($possibleVoucherInterval->getMontantBonAchat());
                    $customerVoucher->setStatut(1);
                    $customerVoucher->setUtiliser(0);
                    $customerVoucher->setValider(1);
                    $customerVoucher->setIdUtilisateur(LogDAO::getCurrentUser());
                    $customerVoucher->setIdCarteFidelite($loyaltyCard);
                    $customerVoucher->setDateValidation($currentDate);
                    $customerVoucherDao->save($customerVoucher);
                    $customerVoucherSuffix = "{$customerVoucher->getIdBonAchatClient()}";
                    if (strlen($customerVoucherSuffix) % 2 != 0) {
                        $customerVoucherSuffix = "0{$customerVoucherSuffix}";
                    }
                    $customerVoucher->setCodeBonAchatClient("{$customerVoucher->getCodeBonAchatClient()}{$customerVoucherSuffix}");
                    $customerVoucherDao->update($customerVoucher);
                    $availableLoyaltyCardPointRecords = $loyaltyCardPointDao->getAllPointsOfLoyaltyCard($loyaltyCard);
                    $pointsConsumedCovered = 0;
                    foreach ($availableLoyaltyCardPointRecords as $loyaltyCardPoint) {
                        $pointsPossibleToCover = $loyaltyCardPoint->getQuantitePoint() >= ($pointsOfLoyaltyCard - $pointsConsumedCovered) ? ($pointsOfLoyaltyCard - $pointsConsumedCovered) : $loyaltyCardPoint->getQuantitePoint();
                        $loyaltyCardPoint->setDateExpiration($currentDate);
                        if ($pointsPossibleToCover < $loyaltyCardPoint->getQuantitePoint()) {
                            $restOfLoyaltyCardPoint = $loyaltyCardPointDao->create();
                            $restOfLoyaltyCardPoint->setDateAssignation($loyaltyCardPoint->getDateAssignation());
                            $restOfLoyaltyCardPoint->setDateExpiration($loyaltyCard->getDateExpiration());
                            $restOfLoyaltyCardPoint->setIdCarteFidelite($loyaltyCard);
                            $restOfLoyaltyCardPoint->setQuantitePoint($loyaltyCardPoint->getQuantitePoint() - $pointsPossibleToCover);
                            $loyaltyCardPoint->setQuantitePoint($pointsPossibleToCover);
                            $loyaltyCardPointDao->save($restOfLoyaltyCardPoint);
                        }
                        $loyaltyCardPointDao->update($loyaltyCardPoint);
                        $pointsConsumedCovered += $pointsPossibleToCover;
                        if ($pointsConsumedCovered == $pointsOfLoyaltyCard) break;
                        LogDAO::registerOperationForCurrentUser(
                            "{$this->translator->_("loyalty.convertPoints.pointsSuccessfullyConverted")} : {$this->translator->_("loyalty.convertPoints.loyaltyCardNumber")} {$loyaltyCard->getCode()}({$loyaltyCard->getIdPersonne()->getNom()} {$loyaltyCard->getIdPersonne()->getPrenom()}) => {$this->translator->_("loyalty.convertPoints.voucherNumber")} {$customerVoucher->getCodeBonAchatClient()}"
                        );
                    }
                } else {
                    if ($pointsOfLoyaltyCard >= (int)NOMBRE_POINT_MIN_POUR_BON_ACHAT) {
                        $timesMinimumNumberOfPoints = (int)($pointsOfLoyaltyCard / (int)NOMBRE_POINT_MIN_POUR_BON_ACHAT);
                        $customerVoucherAmount = (float)($timesMinimumNumberOfPoints * (float)MONTANT_BON_ACHAT_PAR_NOMBRE_POINT_MIN);
                        $customerVoucher = $customerVoucherDao->create();
                        $customerVoucher->setCodeBonAchatClient("{$currentDate->format(DATE_FORMAT_YEAR)}{$currentDate->format(DATE_FORMAT_MONTH_NUMERIC)}{$currentDate->format(DATE_FORMAT_DAY_NUMERIC)}");
                        $customerVoucher->setDateEmission($currentDate);
                        $customerVoucher->setDateExpiration($expirationDate);
                        $customerVoucher->setIdPersonne($loyaltyCard->getIdPersonne());
                        $customerVoucher->setIdPointVente($loyaltyCard->getIdPointVente());
                        $customerVoucher->setListePointsVenteCible("{$loyaltyCard->getIdPointVente()->getIdPointVente()}");
                        $customerVoucher->setMontant($customerVoucherAmount);
                        $customerVoucher->setStatut(1);
                        $customerVoucher->setUtiliser(0);
                        $customerVoucher->setValider(1);
                        $customerVoucher->setIdUtilisateur(LogDAO::getCurrentUser());
                        $customerVoucher->setIdCarteFidelite($loyaltyCard);
                        $customerVoucher->setDateValidation($currentDate);
                        $customerVoucherDao->save($customerVoucher);
                        $customerVoucherSuffix = "{$customerVoucher->getIdBonAchatClient()}";
                        if (strlen($customerVoucherSuffix) % 2 != 0) {
                            $customerVoucherSuffix = "0{$customerVoucherSuffix}";
                        }
                        $customerVoucher->setCodeBonAchatClient("{$customerVoucher->getCodeBonAchatClient()}{$customerVoucherSuffix}");
                        $customerVoucherDao->update($customerVoucher);
                        $availableLoyaltyCardPointRecords = $loyaltyCardPointDao->getAllPointsOfLoyaltyCard($loyaltyCard);
                        $pointsToBeCovered = $timesMinimumNumberOfPoints * (int)NOMBRE_POINT_MIN_POUR_BON_ACHAT;
                        $pointsConsumedCovered = 0;
                        foreach ($availableLoyaltyCardPointRecords as $loyaltyCardPoint) {
                            $pointsPossibleToCover = $loyaltyCardPoint->getQuantitePoint() >= ($pointsToBeCovered - $pointsConsumedCovered) ? ($pointsToBeCovered - $pointsConsumedCovered) : $loyaltyCardPoint->getQuantitePoint();
                            $loyaltyCardPoint->setDateExpiration($currentDate);
                            if ($pointsPossibleToCover < $loyaltyCardPoint->getQuantitePoint()) {
                                $restOfLoyaltyCardPoint = $loyaltyCardPointDao->create();
                                $restOfLoyaltyCardPoint->setDateAssignation($loyaltyCardPoint->getDateAssignation());
                                $restOfLoyaltyCardPoint->setDateExpiration($loyaltyCard->getDateExpiration());
                                $restOfLoyaltyCardPoint->setIdCarteFidelite($loyaltyCard);
                                $restOfLoyaltyCardPoint->setQuantitePoint($loyaltyCardPoint->getQuantitePoint() - $pointsPossibleToCover);
                                $loyaltyCardPoint->setQuantitePoint($pointsPossibleToCover);
                                $loyaltyCardPointDao->save($restOfLoyaltyCardPoint);
                            }
                            $loyaltyCardPointDao->update($loyaltyCardPoint);
                            $pointsConsumedCovered += $pointsPossibleToCover;
                            if ($pointsConsumedCovered == $pointsToBeCovered) break;
                        }
                        LogDAO::registerOperationForCurrentUser(
                            "{$this->translator->_("loyalty.convertPoints.pointsSuccessfullyConverted")} : {$this->translator->_("loyalty.convertPoints.loyaltyCardNumber")} {$loyaltyCard->getCode()}({$loyaltyCard->getIdPersonne()->getNom()} {$loyaltyCard->getIdPersonne()->getPrenom()}) => {$this->translator->_("loyalty.convertPoints.voucherNumber")} {$customerVoucher->getCodeBonAchatClient()}"
                        );
                    }
                }
            }
            $databaseConnection->commit();
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            $databaseConnection->rollBack();
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }
}
