<?php

include APPLICATION_PATH . '/includes/daos.php';

class MessageController extends Zend_Controller_Action
{
    private $translator;

    public function init()
    {
        session_start();
        SubscriptionChecker::checkSubscriptionValidity($this);
        if (isset($_SESSION[SESSION_USER_ID]) && $_SESSION[SESSION_USER_ID] != null) {
            $_SESSION[LAST_ACTION_DATE] = new DateTime();
            LogDAO::updateLastActionDateForCurrentUser();
            $this->translator = Translator::init();
        } else {
            if ($this->getRequest()->isPost()) {
                echo ApiResponse::respond(SESSION_EXPIRED_CODE);
                exit();
            } else {
                $this->_helper->redirector('index', 'Index');
            }
        }
    }

    public function templatesAction()
    {
        AccessRightChecker::checkAccessRight($this, 147);
        $this->view->translator = $this->translator;
    }

    public function datatabletemplatesAction()
    {
        try {
            $title = $_POST["title"] != "" ? $_POST["title"] : null;
            $whereTitle = "";
            if ($title != null) {
                $whereTitle .= " AND m.titre LIKE '%{$title}%'";
            }
            $where = "m.statut = 1{$whereTitle}";
            $messageTemplatesTable = "
                (
                    SELECT m.id_modele as template_id, m.titre as template_title, m.message AS template_message
                    FROM modele_message m
                    WHERE {$where}
                ) AS messagesTemplatesTable
            ";
            $primaryKey = 'template_id';
            $columns = [
                ['db' => 'template_title', 'dt' => 0],
                ['db' => 'template_message', 'dt' => 1],
                ['db' => 'template_id', 'dt' => 2],
            ];
            $computeDatatableResults = SSP::customSimple($_POST, $messageTemplatesTable, $primaryKey, $columns);
            $datatableResult = $computeDatatableResults["datatable"];
            $finalResult = $computeDatatableResults["custom"];
            foreach ($datatableResult["data"] as $row) {
                $templateTitle = $row[0];
                $templateMessage = $row[1];
                $templateId = $row[2];
                $actions = "
                    <button title='{$this->translator->_("shared.edit")}' about='{$templateId}' class='mini ui icon button app-secondary-button app-edit-action'>
                        <i class='edit icon'></i>
                    </button>
                    <button title='{$this->translator->_("shared.delete")}' about='{$templateId}' class='mini ui red icon button app-delete-action'>
                        <i class='trash icon'></i>
                    </button>
                ";
                //Replace tags with corresponding language values
                $templateMessage = str_replace("{{CUSTOMER_LAST_NAME}}", $this->translator->_("message.{{CUSTOMER_LAST_NAME}}"), $templateMessage);
                $templateMessage = str_replace("{{CUSTOMER_FIRST_NAME}}", $this->translator->_("message.{{CUSTOMER_FIRST_NAME}}"), $templateMessage);
                $templateMessage = str_replace("{{CUSTOMER_CIVILITY}}", $this->translator->_("message.{{CUSTOMER_CIVILITY}}"), $templateMessage);
                $templateMessage = str_replace("{{CURRENT_DATE_WITH_LETTERS}}", $this->translator->_("message.{{CURRENT_DATE_WITH_LETTERS}}"), $templateMessage);
                $templateMessage = str_replace("{{CURRENT_DATE_WITH_NUMBERS}}", $this->translator->_("message.{{CURRENT_DATE_WITH_NUMBERS}}"), $templateMessage);
                $templateMessage = str_replace("{{COMPANY_NAME}}", $this->translator->_("message.{{COMPANY_NAME}}"), $templateMessage);
                $templateMessage = str_replace("{{NOM_DU_CLIENT}}", $this->translator->_("message.{{CUSTOMER_LAST_NAME}}"), $templateMessage);
                $templateMessage = str_replace("{{PRENOM_DU_CLIENT}}", $this->translator->_("message.{{CUSTOMER_FIRST_NAME}}"), $templateMessage);
                $templateMessage = str_replace("{{CIVILITE_DU_CLIENT}}", $this->translator->_("message.{{CUSTOMER_CIVILITY}}"), $templateMessage);
                $templateMessage = str_replace("{{DATE_COURANTE_AVEC_LETTRES}}", $this->translator->_("message.{{CURRENT_DATE_WITH_LETTERS}}"), $templateMessage);
                $templateMessage = str_replace("{{DATE_COURANTE_AVEC_CHIFFRES}}", $this->translator->_("message.{{CURRENT_DATE_WITH_NUMBERS}}"), $templateMessage);
                $templateMessage = str_replace("{{NOM_DE_LA_STRUCTURE}}", $this->translator->_("message.{{COMPANY_NAME}}"), $templateMessage);
                $potentialMatches = [];
                preg_match_all('/(\{\{[A-Z_]+\}\})/s', $templateMessage, $potentialMatches);
                if (count($potentialMatches) > 0) {
                    $matches = $potentialMatches[0];
                    foreach ($matches as $match) {
                        $templateMessage = str_replace($match, "<mark>{$match}</mark>", $templateMessage);
                    }
                }
                $tableRow = [
                    $templateTitle,
                    $templateMessage,
                    $actions,
                ];
                $finalResult["data"][] = $tableRow;
            }
            echo json_encode($finalResult);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 duplicate
     * 102 unknown error
     */
    public function createtemplateAction()
    {
        try {
            $title = $_POST["title"];
            $message = $_POST["message"];
            $messageTemplateDao = new MessageTemplateDAO();
            if ($messageTemplateDao->messageTemplateTitleAlreadyExists($title)) {
                echo ApiResponse::respond(101);
                exit();
            }
            $messageTemplate = $messageTemplateDao->create();
            $messageTemplate->setTitre($title);
            $messageTemplate->setMessage($message);
            $messageTemplate->setStatut(1);
            $messageTemplateDao->save($messageTemplate);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("message.templates.messageTemplateSuccessfullyRegistered")} : N°{$messageTemplate->getIdModele()}({$messageTemplate->getTitre()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 duplicate
     * 102 unknown error
     */
    public function updatetemplateAction()
    {
        try {
            $id = (int)$_POST["id"];
            $title = $_POST["title"];
            $message = $_POST["message"];
            $messageTemplateDao = new MessageTemplateDAO();
            $messageTemplate = $messageTemplateDao->getMessageTemplateById($id);
            if ($messageTemplateDao->messageTemplateTitleAlreadyExists($title, $messageTemplate)) {
                echo ApiResponse::respond(101);
                exit();
            }
            $messageTemplate->setTitre($title);
            $messageTemplate->setMessage($message);
            $messageTemplateDao->update($messageTemplate);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("message.templates.messageTemplateUpdatedSuccessfully")} : N°{$messageTemplate->getIdModele()}({$messageTemplate->getTitre()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function deletetemplateAction()
    {
        try {
            $id = (int)$_POST["id"];
            $messageTemplateDao = new MessageTemplateDAO();
            $messageTemplate = $messageTemplateDao->getMessageTemplateById($id);
            if ($messageTemplate->getStatut() == 0) {
                echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE);
                exit();
            }
            $messageTemplate->setStatut(0);
            $messageTemplateDao->update($messageTemplate);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("message.templates.messageTemplateDeletedSuccessfully")} : N°{$messageTemplate->getIdModele()}({$messageTemplate->getTitre()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    public function indexAction()
    {
        AccessRightChecker::checkAccessRight($this, 78);
        $messageTemplateDao = new MessageTemplateDAO();
        $pointOfSaleDao = new PointOfSaleDAO();
        $pointOfSales = [];
        $currentUser = LogDAO::getCurrentUser();
        if ($currentUser->getIdPointVente() == null || in_array($currentUser->getIdProfil()->getCode(), IDENTIFIED_PROFILES_CODES)) {
            $pointOfSales = $pointOfSaleDao->listPointOfSales();
        } else {
            $pointOfSales[] = $currentUser->getIdPointVente();
        }
        $messageTemplates = $messageTemplateDao->getAllMessageTemplates();
        $translatedTemplates = [];
        foreach ($messageTemplates as $messageTemplate) {
            $messageTemplate->setMessage(str_replace("{{CUSTOMER_LAST_NAME}}", $this->translator->_("message.{{CUSTOMER_LAST_NAME}}"), $messageTemplate->getMessage()));
            $messageTemplate->setMessage(str_replace("{{CUSTOMER_FIRST_NAME}}", $this->translator->_("message.{{CUSTOMER_FIRST_NAME}}"), $messageTemplate->getMessage()));
            $messageTemplate->setMessage(str_replace("{{CUSTOMER_CIVILITY}}", $this->translator->_("message.{{CUSTOMER_CIVILITY}}"), $messageTemplate->getMessage()));
            $messageTemplate->setMessage(str_replace("{{CURRENT_DATE_WITH_LETTERS}}", $this->translator->_("message.{{CURRENT_DATE_WITH_LETTERS}}"), $messageTemplate->getMessage()));
            $messageTemplate->setMessage(str_replace("{{CURRENT_DATE_WITH_NUMBERS}}", $this->translator->_("message.{{CURRENT_DATE_WITH_NUMBERS}}"), $messageTemplate->getMessage()));
            $messageTemplate->setMessage(str_replace("{{COMPANY_NAME}}", $this->translator->_("message.{{COMPANY_NAME}}"), $messageTemplate->getMessage()));
            $messageTemplate->setMessage(str_replace("{{NOM_DU_CLIENT}}", $this->translator->_("message.{{CUSTOMER_LAST_NAME}}"), $messageTemplate->getMessage()));
            $messageTemplate->setMessage(str_replace("{{PRENOM_DU_CLIENT}}", $this->translator->_("message.{{CUSTOMER_FIRST_NAME}}"), $messageTemplate->getMessage()));
            $messageTemplate->setMessage(str_replace("{{CIVILITE_DU_CLIENT}}", $this->translator->_("message.{{CUSTOMER_CIVILITY}}"), $messageTemplate->getMessage()));
            $messageTemplate->setMessage(str_replace("{{DATE_COURANTE_AVEC_LETTRES}}", $this->translator->_("message.{{CURRENT_DATE_WITH_LETTERS}}"), $messageTemplate->getMessage()));
            $messageTemplate->setMessage(str_replace("{{DATE_COURANTE_AVEC_CHIFFRES}}", $this->translator->_("message.{{CURRENT_DATE_WITH_NUMBERS}}"), $messageTemplate->getMessage()));
            $messageTemplate->setMessage(str_replace("{{NOM_DE_LA_STRUCTURE}}", $this->translator->_("message.{{COMPANY_NAME}}"), $messageTemplate->getMessage()));
            $translatedTemplates[] = $messageTemplate;
        }
        $this->view->messageTemplates = $translatedTemplates;
        $this->view->pointOfSales = $pointOfSales;
        $this->view->translator = $this->translator;
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function loadpossiblereceiversAction()
    {
        try {
            $receiversType = $_POST["receiversType"];
            $gender = $_POST["gender"] != "0" ? $_POST["gender"] : null;
            $sendMean = $_POST["sendMean"] != "0" ? (int)$_POST["sendMean"] : null;
            switch ($receiversType) {
                case "GENERIC":
                    $personDao = new PersonDAO();
                    $result = [];
                    $allCustomers = $personDao->listCustomersReadyForMessages($gender);
                    foreach ($allCustomers as $customer) {
                        if (($customer->getTelephone() == "" && $customer->getEmail() == null) || ($customer->getEmail() == "" && $customer->getTelephone() == null)) continue;
                        if ($sendMean == 1 && ($customer->getTelephone() == "" || $customer->getTelephone() == null)) continue;
                        if ($sendMean == 2 && ($customer->getEmail() == "" || $customer->getEmail() == null)) continue;
                        $result[] = [
                            "id" => $customer->getIdPersonne(),
                            "name" => "{$customer->getNom()} {$customer->getPrenom()}",
                            "phone" => $customer->getTelephone(),
                            "email" => $customer->getEmail(),
                        ];
                    }
                    echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
                    exit();
                case "HAPPY_BIRTHDAY":
                    $personDao = new PersonDAO();
                    $result = [];
                    $allCustomers = $personDao->listCustomersBornTodayReadyForMessages($gender);
                    foreach ($allCustomers as $customer) {
                        if (($customer->getTelephone() == "" && $customer->getEmail() == null) || ($customer->getEmail() == "" && $customer->getTelephone() == null)) continue;
                        if ($sendMean == 1 && ($customer->getTelephone() == "" || $customer->getTelephone() == null)) continue;
                        if ($sendMean == 2 && ($customer->getEmail() == "" || $customer->getEmail() == null)) continue;
                        $result[] = [
                            "id" => $customer->getIdPersonne(),
                            "name" => "{$customer->getNom()} {$customer->getPrenom()}",
                            "phone" => $customer->getTelephone(),
                            "email" => $customer->getEmail(),
                        ];
                    }
                    echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
                    exit();
                case "REMIND_APPOINTMENT":
                    $pointOfSaleDao = new PointOfSaleDAO();
                    $appointmentDao = new AppointmentDAO();
                    $pointOfSaleId = (int)$_POST["pointOfSale"] != 0 ? (int)$_POST["pointOfSale"] : null;
                    $pointOfSale = $pointOfSaleId != null ? $pointOfSaleDao->getPointOfSaleById($pointOfSaleId) : null;
                    $remindType = $_POST["remindType"];
                    $startDate = null;
                    $endDate = null;
                    if ($remindType == "REMIND_ON_DATE") {
                        $startDate = new DateTime("{$_POST["remindDate"]} 00:00:00");
                        $endDate = new DateTime("{$_POST["remindDate"]} 23:59:59");
                    } else {
                        $startDate = new DateTime("{$_POST["remindStartDate"]} 00:00:00");
                        $endDate = new DateTime("{$_POST["remindEndDate"]} 23:59:59");
                    }
                    $result = [];
                    $appointments = $appointmentDao->getAppointmentsOverPeriodWithCustomerReadyForMessages($startDate, $endDate, $pointOfSale, $gender);
                    foreach ($appointments as $appointment) {
                        $customer = $appointment->getIdClient();
                        if (($customer->getTelephone() == "" && $customer->getEmail() == null) || ($customer->getEmail() == "" && $customer->getTelephone() == null)) continue;
                        if ($sendMean == 1 && ($customer->getTelephone() == "" || $customer->getTelephone() == null)) continue;
                        if ($sendMean == 2 && ($customer->getEmail() == "" || $customer->getEmail() == null)) continue;
                        $result[] = [
                            "id" => $appointment->getIdRendezVous(),
                            "name" => "{$customer->getNom()} {$customer->getPrenom()}",
                            "phone" => $customer->getTelephone(),
                            "email" => $customer->getEmail(),
                            "appointmentDate" => $appointment->getDateDebut()->format(DATE_FORMAT_WITH_TIME),
                            "appointmentActivity" => $appointment->getIdActivite()->getTitre(),
                            "appointmentPointOfSale" => $appointment->getIdPointVente()->getNomPointVente(),
                        ];
                    }
                    echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
                    exit();
                case "FOLLOW_UP":
                    $pointOfSaleDao = new PointOfSaleDAO();
                    $pointOfSaleId = (int)$_POST["pointOfSale"] != 0 ? (int)$_POST["pointOfSale"] : null;
                    $pointOfSale = $pointOfSaleId != null ? $pointOfSaleDao->getPointOfSaleById($pointOfSaleId) : null;
                    $daysCount = (int)$_POST["daysCount"];
                    $currentDate = new DateTime();
                    $currentDate->modify("- {$daysCount} days");
                    $customerInvoiceDao = new CustomerInvoiceDAO();
                    $result = [];
                    $allCustomers = $customerInvoiceDao->getCustomerToFollowUpReadyForMessage($currentDate, $pointOfSale, $gender);
                    foreach ($allCustomers as $customer) {
                        if (($customer["customer_phone"] == "" && $customer["customer_email"] == null) || ($customer["customer_email"] == "" && $customer["customer_phone"] == null)) continue;
                        if ($sendMean == 1 && ($customer["customer_phone"] == "" || $customer["customer_phone"] == null)) continue;
                        if ($sendMean == 2 && ($customer["customer_email"] == "" || $customer["customer_email"] == null)) continue;
                        $result[] = [
                            "id" => $customer["customer_id"],
                            "name" => "{$customer["customer_last_name"]} {$customer["customer_first_name"]}",
                            "phone" => $customer["customer_phone"],
                            "email" => $customer["customer_email"],
                            "lastVisitDate" => $customer["last_invoice_date"],
                        ];
                    }
                    echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
                    exit();
                case "PROMOTION_FOR_BEST_CUSTOMERS":
                    $pointOfSaleDao = new PointOfSaleDAO();
                    $pointOfSaleId = (int)$_POST["pointOfSale"] != 0 ? (int)$_POST["pointOfSale"] : null;
                    $pointOfSale = $pointOfSaleId != null ? $pointOfSaleDao->getPointOfSaleById($pointOfSaleId) : null;
                    $startDate = new DateTime("{$_POST["startDate"]} 00:00:00");
                    $endDate = new DateTime("{$_POST["endDate"]} 23:59:59");
                    $amount = (float)$_POST["amount"];
                    $customerInvoiceDao = new CustomerInvoiceDAO();
                    $result = [];
                    $allCustomers = $customerInvoiceDao->getBestCustomersReadyForMessage($startDate, $endDate, $amount, $pointOfSale, $gender);
                    foreach ($allCustomers as $customer) {
                        if (($customer["customer_phone"] == "" && $customer["customer_email"] == null) || ($customer["customer_email"] == "" && $customer["customer_phone"] == null)) continue;
                        if ($sendMean == 1 && ($customer["customer_phone"] == "" || $customer["customer_phone"] == null)) continue;
                        if ($sendMean == 2 && ($customer["customer_email"] == "" || $customer["customer_email"] == null)) continue;
                        $result[] = [
                            "id" => $customer["customer_id"],
                            "name" => "{$customer["customer_last_name"]} {$customer["customer_first_name"]}",
                            "phone" => $customer["customer_phone"],
                            "email" => $customer["customer_email"],
                            "turnover" => $customer["customer_turnover"],
                        ];
                    }
                    echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
                    exit();
                case "SATISFACTION_SURVEY":
                    $pointOfSaleDao = new PointOfSaleDAO();
                    $pointOfSaleId = (int)$_POST["pointOfSale"] != 0 ? (int)$_POST["pointOfSale"] : null;
                    $pointOfSale = $pointOfSaleId != null ? $pointOfSaleDao->getPointOfSaleById($pointOfSaleId) : null;
                    $startDate = new DateTime("{$_POST["startDate"]} 00:00:00");
                    $endDate = new DateTime("{$_POST["endDate"]} 23:59:59");
                    $visits = (int)$_POST["visits"];
                    $customerInvoiceDao = new CustomerInvoiceDAO();
                    $result = [];
                    $allCustomers = $customerInvoiceDao->getSatisfactionSurveyCustomersReadyForMessage($startDate, $endDate, $visits, $pointOfSale, $gender);
                    foreach ($allCustomers as $customer) {
                        if (($customer["customer_phone"] == "" && $customer["customer_email"] == null) || ($customer["customer_email"] == "" && $customer["customer_phone"] == null)) continue;
                        if ($sendMean == 1 && ($customer["customer_phone"] == "" || $customer["customer_phone"] == null)) continue;
                        if ($sendMean == 2 && ($customer["customer_email"] == "" || $customer["customer_email"] == null)) continue;
                        $result[] = [
                            "id" => $customer["customer_id"],
                            "name" => "{$customer["customer_last_name"]} {$customer["customer_first_name"]}",
                            "phone" => $customer["customer_phone"],
                            "email" => $customer["customer_email"],
                            "visits" => $customer["customer_visits"],
                        ];
                    }
                    echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
                    exit();
            }
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 
     * @param string $message
     * @param int $id
     * @param bool $isForAppointment
     * @return string
     */
    protected function completemessageTags($message, $id, $isForAppointment = false)
    {
        $appointmentDao = new AppointmentDAO();
        $personDao = new PersonDAO();
        $customer = [];
        if ($isForAppointment) {
            $appointment = $appointmentDao->getAppointmentById($id);
            $customer = [
                "id" => $id,
                "lastName" => $appointment->getIdClient()->getNom(),
                "firstName" =>  $appointment->getIdClient()->getPrenom(),
                "civility" => $appointment->getIdClient()->getSexe() != null ?
                    ($appointment->getIdClient()->getSexe() == MALE_GENDER ? $this->translator->_("message.index.mr") : $this->translator->_("message.index.ms"))
                    : $this->translator->_("message.index.mr/ms"),
                "phone" => $appointment->getIdClient()->getTelephone(),
                "email" => $appointment->getIdClient()->getEmail(),
                "appointmentPointOfSale" => $appointment->getIdPointVente()->getNomPointVente(),
                "appointmentActivity" => $appointment->getIdActivite()->getTitre(),
                "appointmentDateWithNumbers" => $appointment->getDateDebut()->format(DATE_FORMAT_WITH_TIME),
            ];
            $momentOfAppointment = new \Moment\Moment($appointment->getDateDebut()->format(DATE_FORMAT_WITH_TIME));
            $customer["appointmentDateWithLetters"] = $momentOfAppointment->format(DATE_FORMAT_LONG_WITH_TIME);
        } else {
            $person = $personDao->getPersonById($id);
            $customer = [
                "id" => $id,
                "lastName" => $person->getNom(),
                "firstName" => $person->getPrenom(),
                "civility" => $person->getSexe() != null ?
                    ($person->getSexe() == "M" ? $this->translator->_("message.index.mr") : $this->translator->_("message.index.ms"))
                    : $this->translator->_("message.index.mr/ms"),
                "phone" => $person->getTelephone(),
                "email" => $person->getEmail(),
            ];
        }
        $currentDate = new DateTime();
        $momentCurrentDate = new \Moment\Moment($currentDate->format(DATE_FORMAT));
        $formatCurrentDate = $currentDate->format(DATE_FORMAT);
        $formatMomentCurrentDate = $momentCurrentDate->format(DATE_FORMAT_LONG);
        $completedMessage = $message;
        $completedMessage = str_replace("{{CUSTOMER_LAST_NAME}}", $customer["lastName"], $completedMessage);
        $completedMessage = str_replace("{{CUSTOMER_FIRST_NAME}}", $customer["firstName"], $completedMessage);
        $completedMessage = str_replace("{{CUSTOMER_CIVILITY}}", $customer["civility"], $completedMessage);
        $completedMessage = str_replace("{{CURRENT_DATE_WITH_LETTERS}}", $formatMomentCurrentDate, $completedMessage);
        $completedMessage = str_replace("{{CURRENT_DATE_WITH_NUMBERS}}", $formatCurrentDate, $completedMessage);
        $completedMessage = str_replace("{{COMPANY_NAME}}", NOM_STRUCTURE, $completedMessage);
        $completedMessage = str_replace("{{NOM_DU_CLIENT}}", $customer["lastName"], $completedMessage);
        $completedMessage = str_replace("{{PRENOM_DU_CLIENT}}", $customer["firstName"], $completedMessage);
        $completedMessage = str_replace("{{CIVILITE_DU_CLIENT}}", $customer["civility"], $completedMessage);
        $completedMessage = str_replace("{{DATE_COURANTE_AVEC_LETTRES}}", $formatMomentCurrentDate, $completedMessage);
        $completedMessage = str_replace("{{DATE_COURANTE_AVEC_CHIFFRES}}", $formatCurrentDate, $completedMessage);
        $completedMessage = str_replace("{{NOM_DE_LA_STRUCTURE}}", NOM_STRUCTURE, $completedMessage);

        if ($isForAppointment) {
            $completedMessage = str_replace("{{APPOINTMENT_DATETIME_WITH_LETTERS}}", $customer["appointmentDateWithLetters"], $completedMessage);
            $completedMessage = str_replace("{{APPOINTMENT_DATETIME_WITH_NUMBERS}}", $customer["appointmentDateWithNumbers"], $completedMessage);
            $completedMessage = str_replace("{{APPOINTMENT_ACTIVITY}}", $customer["appointmentActivity"], $completedMessage);
            $completedMessage = str_replace("{{APPOINTMENT_POINT_OF_SALE}}", $customer["appointmentPointOfSale"], $completedMessage);
            $completedMessage = str_replace("{{DATE_HEURE_DU_RENDEZ_AVEC_LETTRES}}", $customer["appointmentDateWithLetters"], $completedMessage);
            $completedMessage = str_replace("{{DATE_HEURE_DU_RENDEZ_VOUS_AVEC_CHIFFRES}}", $customer["appointmentDateWithNumbers"], $completedMessage);
            $completedMessage = str_replace("{{ACTIVITE_DU_RENDEZ_VOUS}}", $customer["appointmentActivity"], $completedMessage);
            $completedMessage = str_replace("{{POINT_DE_VENTE_DU_RENDEZ_VOUS}}", $customer["appointmentPointOfSale"], $completedMessage);
        }

        return [
            "customer" => $customer,
            "message" => $completedMessage
        ];
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function getmessagetosendforcustomerAction()
    {
        try {
            $id = (int)$_POST["id"];
            $template = $_POST["template"];
            $forAppointmentMessages = $template == "APPOINTMENT" ? true : false;
            $message = $_POST["message"];
            $messageToSend = $this->completemessageTags($message, $id, $forAppointmentMessages)["message"];
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $messageToSend);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function sendmessagesAction()
    {
        try {
            $template = $_POST["template"];
            $sentToAll = (int)$_POST["sendToAll"];
            $message = $_POST["message"];
            $gender = $_POST["gender"] != "0" ? $_POST["gender"] : null;
            $sendMean = (int)$_POST["sendMean"] != 0 ? (int)$_POST["sendMean"] : null;
            $mailSubject = $_POST["mailSubject"];
            $recipientsIds = $sentToAll == 0 ? json_decode($_POST["recipientIds"], true) : [];
            $isForAppointment = $template == "REMIND_APPOINTMENT" ? true : false;
            if ($sentToAll == 1) {
                switch ($template) {
                    case "GENERIC":
                        $personDao = new PersonDAO();
                        $allCustomers = $personDao->listCustomersReadyForMessages($gender);
                        foreach ($allCustomers as $customer) {
                            if (($customer->getTelephone() == "" && $customer->getEmail() == null) || ($customer->getEmail() == "" && $customer->getTelephone() == null)) continue;
                            if ($sendMean == 1 && ($customer->getTelephone() == "" || $customer->getTelephone() == null)) continue;
                            if ($sendMean == 2 && ($customer->getEmail() == "" || $customer->getEmail() == null)) continue;
                            $recipientsIds[] = $customer->getIdPersonne();
                        }
                        break;
                    case "HAPPY_BIRTHDAY":
                        $personDao = new PersonDAO();
                        $allCustomers = $personDao->listCustomersBornTodayReadyForMessages($gender);
                        foreach ($allCustomers as $customer) {
                            if (($customer->getTelephone() == "" && $customer->getEmail() == null) || ($customer->getEmail() == "" && $customer->getTelephone() == null)) continue;
                            if ($sendMean == 1 && ($customer->getTelephone() == "" || $customer->getTelephone() == null)) continue;
                            if ($sendMean == 2 && ($customer->getEmail() == "" || $customer->getEmail() == null)) continue;
                            $recipientsIds[] = $customer->getIdPersonne();
                        }
                        break;
                    case "REMIND_APPOINTMENT":
                        $pointOfSaleDao = new PointOfSaleDAO();
                        $appointmentDao = new AppointmentDAO();
                        $pointOfSaleId = (int)$_POST["pointOfSale"] != 0 ? (int)$_POST["pointOfSale"] : null;
                        $pointOfSale = $pointOfSaleId != null ? $pointOfSaleDao->getPointOfSaleById($pointOfSaleId) : null;
                        $remindType = $_POST["remindType"];
                        $startDate = null;
                        $endDate = null;
                        if ($remindType == "REMIND_ON_DATE") {
                            $startDate = new DateTime("{$_POST["remindDate"]} 00:00:00");
                            $endDate = new DateTime("{$_POST["remindDate"]} 23:59:59");
                        } else {
                            $startDate = new DateTime("{$_POST["remindStartDate"]} 00:00:00");
                            $endDate = new DateTime("{$_POST["remindEndDate"]} 23:59:59");
                        }
                        $appointments = $appointmentDao->getAppointmentsOverPeriodWithCustomerReadyForMessages($startDate, $endDate, $pointOfSale, $gender);
                        foreach ($appointments as $appointment) {
                            $customer = $appointment->getIdClient();
                            if (($customer->getTelephone() == "" && $customer->getEmail() == null) || ($customer->getEmail() == "" && $customer->getTelephone() == null)) continue;
                            if ($sendMean == 1 && ($customer->getTelephone() == "" || $customer->getTelephone() == null)) continue;
                            if ($sendMean == 2 && ($customer->getEmail() == "" || $customer->getEmail() == null)) continue;
                            $recipientsIds[] = $appointment->getIdRendezVous();
                        }
                        break;
                    case "FOLLOW_UP":
                        $pointOfSaleDao = new PointOfSaleDAO();
                        $pointOfSaleId = (int)$_POST["pointOfSale"] != 0 ? (int)$_POST["pointOfSale"] : null;
                        $pointOfSale = $pointOfSaleId != null ? $pointOfSaleDao->getPointOfSaleById($pointOfSaleId) : null;
                        $daysCount = (int)$_POST["daysCount"];
                        $currentDate = new DateTime();
                        $currentDate->modify("- {$daysCount} days");
                        $customerInvoiceDao = new CustomerInvoiceDAO();
                        $allCustomers = $customerInvoiceDao->getCustomerToFollowUpReadyForMessage($currentDate, $pointOfSale, $gender);
                        foreach ($allCustomers as $customer) {
                            if (($customer["customer_phone"] == "" && $customer["customer_email"] == null) || ($customer["customer_email"] == "" && $customer["customer_phone"] == null)) continue;
                            if ($sendMean == 1 && ($customer["customer_phone"] == "" || $customer["customer_phone"] == null)) continue;
                            if ($sendMean == 2 && ($customer["customer_email"] == "" || $customer["customer_email"] == null)) continue;
                            $recipientsIds[] = (int)$customer["customer_id"];
                        }
                        break;
                    case "PROMOTION_FOR_BEST_CUSTOMERS":
                        $pointOfSaleDao = new PointOfSaleDAO();
                        $pointOfSaleId = (int)$_POST["pointOfSale"] != 0 ? (int)$_POST["pointOfSale"] : null;
                        $pointOfSale = $pointOfSaleId != null ? $pointOfSaleDao->getPointOfSaleById($pointOfSaleId) : null;
                        $startDate = new DateTime("{$_POST["startDate"]} 00:00:00");
                        $endDate = new DateTime("{$_POST["endDate"]} 23:59:59");
                        $amount = (float)$_POST["amount"];
                        $customerInvoiceDao = new CustomerInvoiceDAO();
                        $allCustomers = $customerInvoiceDao->getBestCustomersReadyForMessage($startDate, $endDate, $amount, $pointOfSale, $gender);
                        foreach ($allCustomers as $customer) {
                            if (($customer["customer_phone"] == "" && $customer["customer_email"] == null) || ($customer["customer_email"] == "" && $customer["customer_phone"] == null)) continue;
                            if ($sendMean == 1 && ($customer["customer_phone"] == "" || $customer["customer_phone"] == null)) continue;
                            if ($sendMean == 2 && ($customer["customer_email"] == "" || $customer["customer_email"] == null)) continue;
                            $recipientsIds[] = (int)$customer["customer_id"];
                        }
                        break;
                    case "SATISFACTION_SURVEY":
                        $pointOfSaleDao = new PointOfSaleDAO();
                        $pointOfSaleId = (int)$_POST["pointOfSale"] != 0 ? (int)$_POST["pointOfSale"] : null;
                        $pointOfSale = $pointOfSaleId != null ? $pointOfSaleDao->getPointOfSaleById($pointOfSaleId) : null;
                        $startDate = new DateTime("{$_POST["startDate"]} 00:00:00");
                        $endDate = new DateTime("{$_POST["endDate"]} 23:59:59");
                        $visits = (int)$_POST["visits"];
                        $customerInvoiceDao = new CustomerInvoiceDAO();
                        $allCustomers = $customerInvoiceDao->getSatisfactionSurveyCustomersReadyForMessage($startDate, $endDate, $visits, $pointOfSale, $gender);
                        foreach ($allCustomers as $customer) {
                            if (($customer["customer_phone"] == "" && $customer["customer_email"] == null) || ($customer["customer_email"] == "" && $customer["customer_phone"] == null)) continue;
                            if ($sendMean == 1 && ($customer["customer_phone"] == "" || $customer["customer_phone"] == null)) continue;
                            if ($sendMean == 2 && ($customer["customer_email"] == "" || $customer["customer_email"] == null)) continue;
                            $recipientsIds[] = (int)$customer["customer_id"];
                        }
                        break;
                }
            }
            $completeRecipientsWithInformationsAndMessage = [];
            foreach ($recipientsIds as $recipientId) {
                $completeRecipientsWithInformationsAndMessage[] = $this->completemessageTags($message, (int)$recipientId, $isForAppointment);
            }

            $sendStatistics = [
                "totalSms" => ($sendMean == null || $sendMean == 1) ? count($completeRecipientsWithInformationsAndMessage) : 0,
                "totalEmail" => ($sendMean == null || $sendMean == 2) ? count($completeRecipientsWithInformationsAndMessage) : 0,
                "totalSmsSent" => 0,
                "totalEmailSent" => 0,
                "failure" => [],
            ];

            $currentUser = LogDAO::getCurrentUser();
            $messageLogDao = new MessageLogDAO();
            $messageLog = $messageLogDao->create();
            $messageLog->setIdUtilisateur($currentUser);
            $messageLog->setSendDate(new DateTime());
            $messageLog->setTotalEmail(count($completeRecipientsWithInformationsAndMessage));
            $messageLog->setTotalSms(count($completeRecipientsWithInformationsAndMessage));
            $messageLog->setTotalEmailSent(0);
            $messageLog->setTotalSmsSent(0);
            $messageLog->setStatut(1);
            $messageLogDao->save($messageLog);
            $effectiveSmsCountToSend = 0;
            $effectiveEmailCountToSend = 0;
            $messageLogDetailDao = new MessageLogDetailDAO();
            $personDao = new PersonDAO();
            $appointmentDao = new AppointmentDAO();

            $primaryBrand = APP_PRIMARY_BRAND;
            $secondaryBrand = APP_SECONDARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $companyName = NOM_STRUCTURE;
            $includeStoreLogoInMail = (int)LOGO_PRESENT_DANS_LES_MAILS;
            $companyLogo = LOGO_STRUCTURE;
            $currentDate = new DateTime();
            $currentYear = $currentDate->format("Y");
            //Then we send messages from $completeRecipientsWithInformationsAndMessage
            foreach ($completeRecipientsWithInformationsAndMessage as $messageInstance) {
                $messageLogDetail = $messageLogDetailDao->create();
                $customer = $personDao->getPersonById((int)$messageInstance["customer"]["id"]);
                if ($isForAppointment) {
                    $appointment = $appointmentDao->getAppointmentById((int)$messageInstance["customer"]["id"]);
                    $customer = $appointment->getIdClient();
                }
                $messageLogDetail->setIdClient($customer);
                $messageLogDetail->setIdMessageLog($messageLog);
                $messageLogDetail->setMessage($messageInstance["message"]);
                $messageLogDetail->setEmailSendStatus(0);
                $messageLogDetail->setSmsSendStatus(0);
                $messageLogDetail->setStatut(1);
                $smsSent = false;
                $emailSent = false;
                if (($sendMean == null || $sendMean == 1) && $messageInstance["customer"]["phone"] != null && $messageInstance["customer"]["phone"] != "") {
                    $effectiveSmsCountToSend++;
                    //TODO send sms $messageInstance["message"] here $messageInstance["customer"]["phone"]
                    //TODO if success increment $sendStatistics["totalSmsSent"]++; 
                    //TODO if success set $smsSent = true;
                    //TODO if send failure $sendStatistics["failure"][$messageInstance["customer"]["id"]]["smsSent"] = 0;
                } else {
                    if ($sendMean == null || $sendMean == 1) {
                        $effectiveSmsCountToSend++;
                        $sendStatistics["failure"][$messageInstance["customer"]["id"]]["smsSent"] = 0;
                    } else {
                        $smsSent = true;
                    }
                }

                if (($sendMean == null || $sendMean == 2) && $messageInstance["customer"]["email"] != null && $messageInstance["customer"]["email"] != "") {
                    $effectiveEmailCountToSend++;
                    try {
                        $mail = new PHPMailer\PHPMailer\PHPMailer();
                        $mail->isSMTP();
                        $mail->Host = PARAMETRE_MESSAGERIE_NOM_DOMAINE;
                        $mail->SMTPAuth = true;
                        $mail->Username = PARAMETRE_MESSAGERIE_EMAIL;
                        $mail->Password = PARAMETRE_MESSAGERIE_PASS_EMAIL;
                        $mail->SMTPSecure = PHPMailer\PHPMailer\PHPMailer::ENCRYPTION_SMTPS;
                        $mail->Port = (int)PARAMETRE_MESSAGERIE_PORT;
                        $mail->setFrom(PARAMETRE_MESSAGERIE_EMAIL, $companyName);
                        $mail->addAddress($messageInstance["customer"]["email"], "{$messageInstance["customer"]["lastName"]} {$messageInstance["customer"]["firstName"]}");
                        $mail->isHTML(true);
                        $mail->CharSet = "UTF-8";
                        $mail->Subject = $mailSubject;
                        $companyLogoHtml = "";
                        if ($includeStoreLogoInMail == 1) {
                            $companyLogoHtml = "
                                <p style='text-align: center;'>
                                    <img style='width: 125px; height: 100px; border-radius: 6px;' src='/images/business/{$companyLogo}' />
                                </p>
                            ";
                        }
                        $emailHtml = "
                            <html lang='en'>
                                <head>
                                    <style>
                                        .main-table {
                                            border: none; 
                                            margin: 50px; 
                                            background-color: {$primaryBrand}55;
                                            min-width: 90%;
                                        }

                                        @media screen and (max-width:600px) {
                                            .main-table {
                                                margin: 0;
                                            }
                                        }
                                    </style>
                                </head>
                                <body>
                                    <div style='background-image: url(cid:app_background);'>
                                        <table class='main-table'>
                                            <tr>
                                                <div style='border-width: 2px; border-style: solid; border-color: {$secondaryBrand}; padding: 1em 1em; border-radius: 4px;'>
                                                    <p style='text-align: center;'>
                                                        <i style='color: {$thirdBrand}; background-color: {$primaryBrand}; font-style: italic; font-size: 16px; padding: 5px; border-radius: 6px;'>
                                                            {$companyName}
                                                        </i>
                                                    </p>
                                                    {$companyLogoHtml}
                                                    <p style='color: {$thirdBrand}; text-align: center; font-weight: bold; font-size: 16px;'>{$messageInstance["message"]}<p>
                                                </div>
                                            </tr>
                                            <tr>
                                                <div style='border-width: 1.5px; border-style: solid; border-color: {$primaryBrand}; border-radius: 6px;'></div>
                                                <p style='text-align: center;'>
                                                    <i style='font-weight: bold; font-size: 15px; color: {$secondaryBrand};'>{$this->translator->_("message.thisMailWasSentWith")}</i>
                                                    <br/>
                                                    <i style='font-weight: bold; font-size: 20px; color: {$thirdBrand};'>Crescita</i>
                                                    <br/>
                                                    <img style='width: 125px; height: 100px; border-radius: 6px;' src='/images/assets/app_logo.png' />
                                                    <br/>
                                                    <i style='color: {$thirdBrand}'>&copy; 2022-{$currentYear} Crescita</i>
                                                </p>
                                            </tr>
                                        </table>
                                    </div>
                                </body>
                            </html>
                        ";
                        $mail->msgHTML($emailHtml, PUBLIC_PATH);
                        $mail->addEmbeddedImage(PUBLIC_PATH . '/images/assets/app_background.png', 'app_background');
                        $mail->AltBody = $messageInstance["message"];
                        $mail->send();
                        $sendStatistics["totalEmailSent"]++;
                        $emailSent = true;
                    } catch (Exception $ex) {
                        $sendStatistics["failure"][$messageInstance["customer"]["id"]]["emailSent"] = 0;
                    }
                } else {
                    if ($sendMean == null || $sendMean == 2) {
                        $effectiveEmailCountToSend++;
                        $sendStatistics["failure"][$messageInstance["customer"]["id"]]["emailSent"] = 0;
                    } else {
                        $emailSent = true;
                    }
                }
                if (!$smsSent || !$emailSent) {
                    $sendStatistics["failure"][$messageInstance["customer"]["id"]]["name"] = "{$messageInstance["customer"]["lastName"]} {$messageInstance["customer"]["firstName"]}";
                    $sendStatistics["failure"][$messageInstance["customer"]["id"]]["phone"] = $messageInstance["customer"]["phone"];
                    $sendStatistics["failure"][$messageInstance["customer"]["id"]]["email"] = $messageInstance["customer"]["email"];
                    $sendStatistics["failure"][$messageInstance["customer"]["id"]]["message"] = $messageInstance["message"];
                }
                if ($smsSent) $messageLogDetail->setSmsSendStatus(1);
                if ($emailSent) $messageLogDetail->setEmailSendStatus(1);
                $messageLogDetailDao->save($messageLogDetail);
            }
            $messageLog->setTotalEmail($effectiveEmailCountToSend);
            $messageLog->setTotalSms($effectiveSmsCountToSend);
            $messageLog->setTotalEmailSent($sendStatistics["totalEmailSent"]);
            $messageLog->setTotalSmsSent($sendStatistics["totalSmsSent"]);
            $messageLogDao->update($messageLog);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $sendStatistics);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }
}
