<?php

include APPLICATION_PATH . '/includes/daos.php';

class OperationsController extends Zend_Controller_Action
{
    private $translator;

    public function init()
    {
        session_start();
        SubscriptionChecker::checkSubscriptionValidity($this);
        if (isset($_SESSION[SESSION_USER_ID]) && $_SESSION[SESSION_USER_ID] != null) {
            $_SESSION[LAST_ACTION_DATE] = new DateTime();
            LogDAO::updateLastActionDateForCurrentUser();
            $this->translator = Translator::init();
        } else {
            if ($this->getRequest()->isPost()) {
                echo ApiResponse::respond(SESSION_EXPIRED_CODE);
                exit();
            } else {
                $this->_helper->redirector('index', 'Index');
            }
        }
    }

    public function indexAction()
    {
        AccessRightChecker::checkAccessRight($this, 44);
        $userDao = new UserDAO();
        $users = $userDao->listUsers();
        $this->view->users = $users;
        $this->view->translator = $this->translator;
    }

    public function datatableAction()
    {
        try {
            $user = (int)$_POST["user"] != 0 ? (int)$_POST["user"] : null;
            $beginDate = "{$_POST["beginDate"]} 00:00:00";
            $endDate = "{$_POST["endDate"]} 23:59:59";
            $whereUser = "";
            if ($user != null) {
                $whereUser .= " AND o.id_utilisateur = {$user}";
            }
            $protectedLogin = PROTECTED_APP_LOGIN;
            $where = "o.date_enreg >= '{$beginDate}' AND o.date_enreg <= '{$endDate}' AND u.login <> '{$protectedLogin}'{$whereUser}";
            $operationsTable = "
                (
                    SELECT o.id_log AS operation_id, o.date_enreg AS operation_date,
                    o.description_log AS operation_description, CONCAT(pe.nom, ' ', COALESCE(pe.prenom, '')) AS user_name
                    FROM log o JOIN utilisateur u ON o.id_utilisateur = u.id_utilisateur 
                    JOIN personne pe ON u.id_personne = pe.id_personne
                    WHERE {$where}
                ) AS operationsTable
            ";
            $primaryKey = 'operation_id';
            $columns = [
                ['db' => 'user_name', 'dt' => 0],
                ['db' => 'operation_description', 'dt' => 1],
                ['db' => 'operation_date', 'dt' => 2],
            ];
            $computeDatatableResults = SSP::customSimple($_POST, $operationsTable, $primaryKey, $columns);
            $datatableResult = $computeDatatableResults["datatable"];
            $finalResult = $computeDatatableResults["custom"];
            foreach ($datatableResult["data"] as $row) {
                $userName = $row[0];
                $operationDescription = $row[1];
                $operationDate = $row[2];
                $tableRow = [
                    $userName,
                    $operationDescription,
                    $operationDate,
                ];
                $finalResult["data"][] = $tableRow;
            }
            echo json_encode($finalResult);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    public function printAction()
    {
        AccessRightChecker::checkAccessRight($this, 148);
        $userDao = new UserDAO();
        $users = $userDao->listUsers();
        $this->view->users = $users;
        $this->view->translator = $this->translator;
    }

    public function datatableprintAction()
    {
        try {
            $user = (int)$_POST["user"] != 0 ? (int)$_POST["user"] : null;
            $code = $_POST["code"] != "" ? $_POST["code"] : null;
            $beginDate = "{$_POST["beginDate"]} 00:00:00";
            $endDate = "{$_POST["endDate"]} 23:59:59";
            $whereUser = "";
            $whereCode = "";
            if ($user != null) {
                $whereUser .= " AND p.id_utilisateur = {$user}";
            }
            if ($code != null) {
                $whereCode .= " AND p.code LIKE '%{$code}%'";
            }
            $protectedLogin = PROTECTED_APP_LOGIN;
            $where = "p.date >= '{$beginDate}' AND p.date <= '{$endDate}' AND u.login <> '{$protectedLogin}'{$whereUser}{$whereCode}";
            $printLogsTable = "
                (
                    SELECT p.id_print_log AS print_log_id, p.date AS print_log_date,
                    p.code AS print_log_code, p.title AS print_log_title, p.filters AS print_log_filters,
                    CONCAT(pe.nom, ' ', COALESCE(pe.prenom, '')) AS user_name
                    FROM print_log p JOIN utilisateur u ON p.id_utilisateur = u.id_utilisateur 
                    JOIN personne pe ON u.id_personne = pe.id_personne
                    WHERE {$where}
                ) AS printLogsTable
            ";
            $primaryKey = 'print_log_id';
            $columns = [
                ['db' => 'user_name', 'dt' => 0],
                ['db' => 'print_log_code', 'dt' => 1],
                ['db' => 'print_log_title', 'dt' => 2],
                ['db' => 'print_log_filters', 'dt' => 3],
                ['db' => 'print_log_date', 'dt' => 4],
            ];
            $computeDatatableResults = SSP::customSimple($_POST, $printLogsTable, $primaryKey, $columns);
            $datatableResult = $computeDatatableResults["datatable"];
            $finalResult = $computeDatatableResults["custom"];
            foreach ($datatableResult["data"] as $row) {
                $userName = $row[0];
                $printLogCode = $row[1];
                $printLogTitle = $row[2];
                $printLogFilters = $row[3];
                $printLogDate = $row[4];
                $tableRow = [
                    $userName,
                    $printLogCode,
                    $printLogTitle,
                    $printLogFilters,
                    $printLogDate,
                ];
                $finalResult["data"][] = $tableRow;
            }
            echo json_encode($finalResult);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    public function messageAction()
    {
        AccessRightChecker::checkAccessRight($this, 149);
        $userDao = new UserDAO();
        $users = $userDao->listUsers();
        $this->view->users = $users;
        $this->view->translator = $this->translator;
    }

    public function datatablemessageAction()
    {
        try {
            $user = (int)$_POST["user"] != 0 ? (int)$_POST["user"] : null;
            $beginDate = "{$_POST["beginDate"]} 00:00:00";
            $endDate = "{$_POST["endDate"]} 23:59:59";
            $whereUser = "";
            if ($user != null) {
                $whereUser .= " AND m.id_utilisateur = {$user}";
            }
            $protectedLogin = PROTECTED_APP_LOGIN;
            $where = "m.send_date >= '{$beginDate}' AND m.send_date <= '{$endDate}' AND u.login <> '{$protectedLogin}'{$whereUser}";
            $messageLogsTable = "
                (
                    SELECT m.id_message_log AS message_log_id, m.send_date AS message_log_send_date,
                    m.total_sms AS message_log_total_sms, m.total_email AS message_log_total_email,
                    m.total_sms_sent AS message_log_total_sms_sent, m.total_email_sent AS message_log_total_email_sent,
                    CONCAT(pe.nom, ' ', COALESCE(pe.prenom, '')) AS user_name
                    FROM message_log m JOIN utilisateur u ON m.id_utilisateur = u.id_utilisateur 
                    JOIN personne pe ON u.id_personne = pe.id_personne
                    WHERE {$where}
                ) AS messageLogsTable
            ";
            $primaryKey = 'message_log_id';
            $columns = [
                ['db' => 'user_name', 'dt' => 0],
                ['db' => 'message_log_total_sms', 'dt' => 1],
                ['db' => 'message_log_total_sms_sent', 'dt' => 2],
                ['db' => 'message_log_total_email', 'dt' => 3],
                ['db' => 'message_log_total_email_sent', 'dt' => 4],
                ['db' => 'message_log_send_date', 'dt' => 5],
                ['db' => 'message_log_id', 'dt' => 6],
            ];
            $computeDatatableResults = SSP::customSimple($_POST, $messageLogsTable, $primaryKey, $columns);
            $datatableResult = $computeDatatableResults["datatable"];
            $finalResult = $computeDatatableResults["custom"];
            foreach ($datatableResult["data"] as $row) {
                $userName = $row[0];
                $messageLogTotalSms = (int)$row[1] != 0 ? $row[1] : $this->translator->_("operations.message.notConsidered");
                $messageLogTotalSmsSent = (int)$row[1] != 0 ? $row[2] : $this->translator->_("operations.message.notConsidered");
                $messageLogTotalEmail = (int)$row[3] != 0 ? $row[3] : $this->translator->_("operations.message.notConsidered");
                $messageLogTotalEmailSent = (int)$row[3] != 0 ? $row[4] : $this->translator->_("operations.message.notConsidered");
                $messageLogSendDate = $row[5];
                $messageLogId = $row[6];
                $actions = "
                    <button title='{$this->translator->_("operations.message.viewMessages")}' about='{$messageLogId}' class='mini ui icon button app-primary-button app-consult-action'>
                        <i class='list alternate icon'></i>
                    </button>
                ";
                $tableRow = [
                    $userName,
                    $messageLogTotalSms,
                    $messageLogTotalSmsSent,
                    $messageLogTotalEmail,
                    $messageLogTotalEmailSent,
                    $messageLogSendDate,
                    $actions,
                ];
                $finalResult["data"][] = $tableRow;
            }
            echo json_encode($finalResult);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function messagesoflogAction() {
        try {
            $id = (int)$_POST["id"];
            $messageLogDao = new MessageLogDAO();
            $messageLog = $messageLogDao->getMessageLogById($id);
            $messageLogDetailDao = new MessageLogDetailDAO();
            $messageLogDetails = $messageLogDetailDao->getDetailsOfMessageLog($messageLog);
            $result = [];
            foreach($messageLogDetails as $messageLogDetail) {
                $result[] = [
                    "customer" => "{$messageLogDetail->getIdClient()->getNom()} {$messageLogDetail->getIdClient()->getPrenom()}",
                    "phone" => $messageLogDetail->getIdClient()->getTelephone(),
                    "email" => $messageLogDetail->getIdClient()->getEmail(),
                    "smsSent" => $messageLog->getTotalSms() > 0 ? $messageLogDetail->getSmsSendStatus() : 2,
                    "emailSent" => $messageLog->getTotalEmail() > 0 ? $messageLogDetail->getEmailSendStatus() : 2,
                    "message" => $messageLogDetail->getMessage(),
                ];
            }
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }
}
