<?php

include APPLICATION_PATH . '/includes/daos.php';

class OrderController extends Zend_Controller_Action
{
    private $translator;

    public function init()
    {
        session_start();
        SubscriptionChecker::checkSubscriptionValidity($this);
        if (isset($_SESSION[SESSION_USER_ID]) && $_SESSION[SESSION_USER_ID] != null) {
            $_SESSION[LAST_ACTION_DATE] = new DateTime();
            LogDAO::updateLastActionDateForCurrentUser();
            $this->translator = Translator::init();
        } else {
            if ($this->getRequest()->isPost()) {
                echo ApiResponse::respond(SESSION_EXPIRED_CODE);
                exit();
            } else {
                $this->_helper->redirector('index', 'Index');
            }
        }
    }

    public function indexAction()
    {
        AccessRightChecker::checkAccessRight($this, 29);
        $supplierDao = new SupplierDAO();
        $productCategoryDao = new ProductCategoryDAO();
        $currentUser = LogDAO::getCurrentUser();
        $storeDao = new StoreDAO();
        $stores = [];
        if ($currentUser->getIdPointVente() == null || in_array($currentUser->getIdProfil()->getCode(), IDENTIFIED_PROFILES_CODES)) {
            $stores = $storeDao->getAllStores();
        } else {
            $stores[] = $currentUser->getIdPointVente()->getIdMagasin();
        }
        $this->view->suppliers = $supplierDao->listSuppliers();
        $this->view->productCategories = $productCategoryDao->getAllProductCategories();
        $this->view->stores = $stores;
        $this->view->translator = $this->translator;
    }

    public function datatableAction()
    {
        $supplier = (int)$_POST["supplier"] != 0 ? (int)$_POST["supplier"] : null;
        $from = $_POST["from"] != "" ? $_POST["from"] : null;
        $to = $_POST["to"] != "" ? $_POST["to"] : null;
        $whereSupplier = "";
        $whereFrom = "";
        $whereTo = "";
        if ($supplier != null) {
            $whereSupplier .= " AND c.id_fournisseur = {$supplier}";
        }
        if ($from != null) {
            $whereFrom .= " AND c.date_commande >= '{$from} 00:00:00'";
        }
        if ($to != null) {
            $whereTo .= " AND c.date_commande <= '{$to} 23:59:59'";
        }
        $where = "c.statut = 1{$whereSupplier}{$whereFrom}{$whereTo}";
        $ordersTable = "
                (
                    SELECT c.id_commande AS order_id, f.nom_fournisseur AS order_supplier,
                    c.date_commande AS order_date, c.date_livraison AS order_delivery_date,
                    c.date_saisie AS order_entry_date, c.montant_ht AS order_total,
                    c.valider AS order_validated, c.facturer AS order_invoiced,
                    c.receptionner AS order_received
                    FROM commande c JOIN fournisseur f ON c.id_fournisseur = f.id_fournisseur
                    WHERE {$where}
                ) AS ordersTable
        ";
        $primaryKey = 'order_id';
        $columns = [
            ['db' => 'order_id', 'dt' => 0],
            ['db' => 'order_supplier', 'dt' => 1],
            ['db' => 'order_date', 'dt' => 2],
            ['db' => 'order_entry_date', 'dt' => 3],
            ['db' => 'order_total', 'dt' => 4],
            ['db' => 'order_validated', 'dt' => 5],
            ['db' => 'order_received', 'dt' => 6],
            ['db' => 'order_delivery_date', 'dt' => 7],
        ];
        $computeDatatableResults = SSP::customSimple($_POST, $ordersTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        foreach ($datatableResult["data"] as $row) {
            $orderId = (int)$row[0];
            $orderSupplier = $row[1];
            $orderDate = new DateTime($row[2]);
            $orderEntryDate = new DateTime($row[3]);
            $orderTotal = number_format((float)$row[4], 0, '', ' ');
            $orderValidated = (int)$row[5];
            $orderReceived = (int)$row[6];
            $orderDeliveryDate = $row[7] != null ? new DateTime($row[7]) : null;
            $actions = "
                <button title='{$this->translator->_("shared.consult")}' about='{$orderId}' class='mini ui icon button app-primary-button app-consult-action'>
                    <i class='eye icon'></i>
                </button>
            ";
            if ($orderValidated == 0) {
                $actions .= "
                    <button title='{$this->translator->_("shared.edit")}' about='{$orderId}' class='mini ui icon button app-secondary-button app-edit-action'>
                        <i class='edit icon'></i>
                    </button>
                    <button title='{$this->translator->_("shared.delete")}' about='{$orderId}' class='mini ui red icon button app-delete-action'>
                        <i class='trash icon'></i>
                    </button>
                ";
            } else {
                if ($orderReceived == 0) {
                    $actions .= "
                        <button title='{$this->translator->_("order.index.receive")}' about='{$orderId}' class='mini ui green icon button app-receive-action'>
                            <i class='truck icon'></i>
                        </button>
                    ";
                }
                $actions .= "
                    <button title='{$this->translator->_("shared.print")}' about='{$orderId}' class='mini ui purple icon button app-print-action'>
                        <i class='print icon'></i>
                    </button>
                ";
            }
            $tableRow = [
                $orderId,
                $orderSupplier,
                $orderDate->format(DATE_FORMAT),
                $orderDeliveryDate != null ? $orderDeliveryDate->format(DATE_FORMAT) : "",
                $orderEntryDate->format(DATE_FORMAT_WITH_SECONDS),
                $orderTotal,
                $actions,
            ];
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    public function orderproductsdatatableAction()
    {
        $name = $_POST["name"] != "" ? $_POST["name"] : null;
        $category = (int)$_POST["category"] != 0 ? (int)$_POST["category"] : null;
        $whereName = "";
        $whereCategory = "";
        if ($name != null) {
            $whereName .= " AND p.designation LIKE '%{$name}%'";
        }
        if ($category != null) {
            $whereCategory .= " AND p.id_categorie_produit = {$category}";
        }
        $where = "p.statut = 1 AND p.stockable = 1{$whereName}{$whereCategory} order by p.designation ASC";
        $orderProductsTable = "
                (
                    SELECT p.id_produit AS product_id,
                    p.designation AS product_designation, p.prix_vente AS product_sale_price, 
                    p.prix_vente2 AS product_sale_price_2, p.prix_vente3 AS product_sale_price_3, 
                    p.prix_achat AS product_buying_price, p.reference_produit AS product_reference,
                    p.image AS product_image
                    FROM produit p
                    WHERE {$where}
                ) AS orderProductsTable
        ";
        $primaryKey = 'product_id';
        $columns = [
            ['db' => 'product_designation', 'dt' => 0],
            ['db' => 'product_buying_price', 'dt' => 1],
            ['db' => 'product_image', 'dt' => 2],
            ['db' => 'product_id', 'dt' => 3],
        ];
        $computeDatatableResults = SSP::customSimple($_POST, $orderProductsTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        foreach ($datatableResult["data"] as $row) {
            $productDesignation = $row[0];
            $productBuyingPrice = $row[1];
            $productImage = $row[2] != null ? $row[2] : "";
            $productId = $row[3];
            $tableRow = [
                [
                    $productDesignation,
                    $productBuyingPrice,
                    $productImage,
                    $productId,
                ],
            ];
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function createAction()
    {
        $orderDao = new OrderDAO();
        $databaseConnection = $orderDao->entityManager->getConnection();
        $databaseConnection->beginTransaction();
        try {
            $supplierId = (int)$_POST["supplier"];
            $orderDate = new DateTime($_POST["orderDate"]);
            $deliveryDate = new DateTime($_POST["deliveryDate"]);
            $currentDate = new DateTime();
            $content = json_decode($_POST["content"], true);
            $supplierDao = new SupplierDAO();
            $order = $orderDao->create();
            $order->setDateCommande($orderDate);
            $order->setDateSaisie($currentDate);
            $supplier = $supplierDao->getSupplierById($supplierId);
            $order->setIdFournisseur($supplier);
            $orderTotal = 0;
            foreach ($content as $detail) {
                $orderTotal += (int)$detail["quantity"] * (float)$detail["price"];
            }
            $order->setMontantHt($orderTotal);
            $order->setTauxTva(0);
            $order->setMontantTva(0);
            $order->setMontantTtc($orderTotal);
            $order->setFacturer(0);
            $order->setReceptionner(0);
            $order->setValider(0);
            $order->setDateLivraison($deliveryDate);
            $order->setStatut(1);
            $orderDao->save($order);
            $orderLineDao = new OrderLineDAO();
            $productDao = new ProductDAO();
            foreach ($content as $detail) {
                $orderLine = $orderLineDao->create();
                $orderLine->setIdCommande($order);
                $product = $productDao->getProductById($detail["id"]);
                $orderLine->setIdProduit($product);
                $orderLine->setPrixUnitaire((float)$detail["price"]);
                $orderLine->setQuantite((int)$detail["quantity"]);
                $orderLine->setStatut(1);
                $orderLineDao->save($orderLine);
            }
            $databaseConnection->commit();
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("order.index.orderSuccessfullyRegistered")} : N°{$order->getIdCommande()}"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            $databaseConnection->rollBack();
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function detailAction()
    {
        try {
            $id = (int)$_POST["id"];
            $orderDao = new OrderDAO();
            $orderLineDao = new OrderLineDAO();
            $order = $orderDao->getOrderById($id);
            $orderLines = $orderLineDao->getOrderLinesByOrder($order);
            $content = [];
            foreach ($orderLines as $orderLine) {
                $content[] = [
                    "id" => $orderLine->getIdProduit()->getIdProduit(),
                    "quantity" => $orderLine->getQuantite(),
                    "price" => $orderLine->getPrixUnitaire(),
                    "total" => $orderLine->getQuantite() * $orderLine->getPrixUnitaire(),
                    "designation" => $orderLine->getIdProduit()->getDesignation(),
                    "defaultSellingPrice" => $orderLine->getIdProduit()->getPrixVente(),
                    "defaultMaxReductionPercent" => $orderLine->getIdProduit()->getPourcentageMaxRemise(),
                    "isPerishable" => $orderLine->getIdProduit()->getPerissable(),
                ];
            }
            $result = [
                "content" => $content,
                "supplier" => $order->getIdFournisseur()->getIdFournisseur(),
                "supplierName" => $order->getIdFournisseur()->getNomFournisseur(),
                "orderDate" => $order->getDateCommande()->format(DATE_FORMAT),
                "total" => $order->getMontantHt(),
                "entryDate" => $order->getDateSaisie()->format(DATE_FORMAT_WITH_SECONDS),
                "deliveryDate" => $order->getDateLivraison() != null ? $order->getDateLivraison()->format(DATE_FORMAT) : "",
            ];
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function updateAction()
    {
        $orderDao = new OrderDAO();
        $databaseConnection = $orderDao->entityManager->getConnection();
        $databaseConnection->beginTransaction();
        try {
            $id = (int)$_POST["id"];
            $order = $orderDao->getOrderById($id);
            if ($order->getValider() == 1) {
                echo ApiResponse::respond();
                exit();
            }
            $supplierId = (int)$_POST["supplier"];
            $orderDate = new DateTime($_POST["orderDate"]);
            $deliveryDate = new DateTime($_POST["deliveryDate"]);
            $content = json_decode($_POST["content"], true);
            $supplierDao = new SupplierDAO();
            $order->setDateCommande($orderDate);
            $supplier = $supplierDao->getSupplierById($supplierId);
            $order->setIdFournisseur($supplier);
            $orderTotal = 0;
            foreach ($content as $detail) {
                $orderTotal += (int)$detail["quantity"] * (float)$detail["price"];
            }
            $order->setMontantHt($orderTotal);
            $order->setMontantTtc($orderTotal);
            $order->setDateLivraison($deliveryDate);
            $orderDao->update($order);
            $orderLineDao = new OrderLineDAO();
            $orderLines = $orderLineDao->getOrderLinesByOrder($order);
            foreach ($orderLines as $orderLine) {
                $orderLineDao->delete($orderLine);
            }
            $productDao = new ProductDAO();
            foreach ($content as $detail) {
                $orderLine = $orderLineDao->create();
                $orderLine->setIdCommande($order);
                $product = $productDao->getProductById($detail["id"]);
                $orderLine->setIdProduit($product);
                $orderLine->setPrixUnitaire((float)$detail["price"]);
                $orderLine->setQuantite((int)$detail["quantity"]);
                $orderLine->setStatut(1);
                $orderLineDao->save($orderLine);
            }
            $databaseConnection->commit();
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("order.index.orderUpdatedSuccessfully")} : N°{$order->getIdCommande()}"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            $databaseConnection->rollBack();
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function deleteAction()
    {
        try {
            $id = (int)$_POST["id"];
            $orderDao = new OrderDAO();
            $order = $orderDao->getOrderById($id);
            if ($order->getStatut() == 0) {
                echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE);
                exit();
            }
            if ($order->getValider() == 1) {
                echo ApiResponse::respond(ALREADY_VALIDATED_ISSUE_CODE);
                exit();
            }
            $order->setStatut(0);
            $orderDao->update($order);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("order.index.orderDeletedSuccessfully")} : N°{$order->getIdCommande()}"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    public function validationAction()
    {
        AccessRightChecker::checkAccessRight($this, 32);
        $supplierDao = new SupplierDAO();
        $productCategoryDao = new ProductCategoryDAO();
        $this->view->suppliers = $supplierDao->listSuppliers();
        $this->view->productCategories = $productCategoryDao->getAllProductCategories();
        $this->view->translator = $this->translator;
    }

    public function datatablevalidationAction()
    {
        $supplier = (int)$_POST["supplier"] != 0 ? (int)$_POST["supplier"] : null;
        $from = $_POST["from"] != "" ? $_POST["from"] : null;
        $to = $_POST["to"] != "" ? $_POST["to"] : null;
        $whereSupplier = "";
        $whereFrom = "";
        $whereTo = "";
        if ($supplier != null) {
            $whereSupplier .= " AND c.id_fournisseur = {$supplier}";
        }
        if ($from != null) {
            $whereFrom .= " AND c.date_commande >= '{$from} 00:00:00'";
        }
        if ($to != null) {
            $whereTo .= " AND c.date_commande <= '{$to} 23:59:59'";
        }
        $where = "c.statut = 1 AND c.valider = 0{$whereSupplier}{$whereFrom}{$whereTo}";
        $ordersToValidateTable = "
                (
                    SELECT c.id_commande AS order_id, f.nom_fournisseur AS order_supplier,
                    c.date_commande AS order_date,
                    c.date_saisie AS order_entry_date, c.montant_ht AS order_total
                    FROM commande c JOIN fournisseur f ON c.id_fournisseur = f.id_fournisseur
                    WHERE {$where}
                ) AS ordersToValidateTable
        ";
        $primaryKey = 'order_id';
        $columns = [
            ['db' => 'order_id', 'dt' => 0],
            ['db' => 'order_supplier', 'dt' => 1],
            ['db' => 'order_date', 'dt' => 2],
            ['db' => 'order_entry_date', 'dt' => 3],
            ['db' => 'order_total', 'dt' => 4],
        ];
        $computeDatatableResults = SSP::customSimple($_POST, $ordersToValidateTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        foreach ($datatableResult["data"] as $row) {
            $orderId = (int)$row[0];
            $orderSupplier = $row[1];
            $orderDate = new DateTime($row[2]);
            $orderEntryDate = new DateTime($row[3]);
            $orderTotal = number_format((float)$row[4], 0, '', ' ');
            $actions = "
                <button title='{$this->translator->_("shared.consult")}' about='{$orderId}' class='mini ui icon button app-primary-button app-consult-action'>
                    <i class='eye icon'></i>
                </button>
                <button title='{$this->translator->_("shared.validate")}' about='{$orderId}' class='mini ui green icon button app-validate-action'>
                    <i class='check icon'></i>
                </button>
            ";
            $tableRow = [
                $orderId,
                $orderSupplier,
                $orderDate->format(DATE_FORMAT),
                $orderEntryDate->format(DATE_FORMAT_WITH_SECONDS),
                $orderTotal,
                $actions,
            ];
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function validateAction()
    {
        try {
            $id = (int)$_POST["id"];
            $orderDao = new OrderDAO();
            $order = $orderDao->getOrderById($id);
            if ($order->getStatut() == 0) {
                echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE);
                exit();
            }
            if ($order->getValider() == 1) {
                echo ApiResponse::respond(ALREADY_VALIDATED_ISSUE_CODE);
                exit();
            }
            $order->setValider(1);
            $orderDao->update($order);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("order.validation.orderValidatedSuccessfully")} : N°{$order->getIdCommande()}"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }
}
