<?php

include APPLICATION_PATH . '/includes/daos.php';

class PasswordController extends Zend_Controller_Action
{
    private $translator;

    public function init()
    {
        session_start();
        SubscriptionChecker::checkSubscriptionValidity($this);
        if (isset($_SESSION[SESSION_USER_ID]) && $_SESSION[SESSION_USER_ID] != null) {
            $_SESSION[LAST_ACTION_DATE] = new DateTime();
            LogDAO::updateLastActionDateForCurrentUser();
            $this->translator = Translator::init();
        } else {
            if ($this->getRequest()->getActionName() != "reset") {
                if ($this->getRequest()->isPost()) {
                    echo ApiResponse::respond(SESSION_EXPIRED_CODE);
                    exit();
                } else {
                    $this->_helper->redirector('index', 'Index');
                }
            } else {
                $this->translator = Translator::init();
            }
        }
    }

    /**
     * 100 success
     * 101 current password incorrect
     * 102 unknown error
     */
    public function changeAction()
    {
        try {
            $currentPassword = $_POST["currentPassword"];
            $newPassword = $_POST["newPassword"];
            $userDao = new UserDAO();
            $currentUser = $userDao->getUserById($_SESSION[SESSION_USER_ID]);
            if (sha1(md5($currentPassword)) == $currentUser->getPassword()) {
                $currentUser->setPassword(sha1(md5($newPassword)));
                $userDao->update($currentUser);
                LogDAO::registerOperationForCurrentUser(
                    $this->translator->_("shared.userWithLogin") . " " . $currentUser->getLogin() . " " . $this->translator->_("shared.changedHisPassword")
                );
                echo ApiResponse::respond();
            } else {
                echo ApiResponse::respond(101);
            }
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 user login not found
     * 102 email not correspond to login
     * 103 unknown error
     */
    public function resetAction()
    {
        try {
            $login = $_POST["login"];
            $email = $_POST["email"];
            $userDao = new UserDAO();
            $targetUser = $userDao->getUserByLogin($login);
            if ($targetUser != null) {
                if ($targetUser->getIdPersonne()->getEmail() == $email) {
                    $generatedPassword = Utility::generatePassword(16);
                    $targetUser->setPassword(sha1(md5($generatedPassword)));
                    $targetUser->setPasswordIsReset(1);
                    $userDao->update($targetUser);
                    $primaryBrand = APP_PRIMARY_BRAND;
                    $secondaryBrand = APP_SECONDARY_BRAND;
                    $thirdBrand = APP_THIRD_BRAND;
                    $currentDate = new DateTime();
                    $currentYear = $currentDate->format("Y");
                    $applicationName = NOM_APPLICATION;
                    $mail = new PHPMailer\PHPMailer\PHPMailer();
                    $mail->isSMTP();
                    $mail->Host = CRESCITA_DEFAULT_MAIL_DOMAIN;
                    $mail->SMTPAuth = true;
                    $mail->Username = CRESCITA_DEFAULT_MAIL_EMAIL;
                    $mail->Password = CRESCITA_DEFAULT_MAIL_APP_PASSWORD;
                    $mail->SMTPSecure = PHPMailer\PHPMailer\PHPMailer::ENCRYPTION_SMTPS;
                    $mail->Port = (int)CRESCITA_DEFAULT_MAIL_PORT;
                    $mail->setFrom(CRESCITA_DEFAULT_MAIL_EMAIL, $applicationName);
                    $mail->addAddress($targetUser->getIdPersonne()->getEmail(), "{$targetUser->getIdPersonne()->getNom()} {$targetUser->getIdPersonne()->getPrenom()}");
                    $mail->isHTML(true);
                    $mail->CharSet = "UTF-8";
                    $mail->Subject = $this->translator->_("password.reset.passwordReset");
                    $emailHtml = "
                        <html lang='en'>
                            <head>
                                <style>
                                    .main-table {
                                        border: none; 
                                        margin: 50px; 
                                        background-color: {$primaryBrand}55;
                                        min-width: 90%;
                                    }

                                    @media screen and (max-width:600px) {
                                        .main-table {
                                            margin: 0;
                                        }
                                    }
                                </style>
                            </head>
                            <body>
                                <div style='background-image: url(cid:app_background); display: flex; justify-content: center;'>
                                    <table class='main-table'>
                                        <tr>
                                            <div style='border-width: 2px; border-style: solid; border-color: {$secondaryBrand}; padding: 1em 1em; border-radius: 4px;'>
                                                <p style='text-align: center;'>
                                                    <i style='color: {$thirdBrand}; background-color: {$primaryBrand}; font-style: italic; font-size: 16px; padding: 5px; border-radius: 6px;'>
                                                        {$applicationName}
                                                    </i>
                                                </p>
                                                <p style='text-align: center;'>
                                                    <img style='width: 125px; height: 100px; border-radius: 6px;' src='/images/assets/app_logo.png' />
                                                </p>
                                                <p style='color: {$thirdBrand}; text-align: center; font-weight: bold; font-size: 16px;'>{$this->translator->_("password.reset.youAreReceivingThisEmailBecauseYouHaveRequestedAPasswordReset")}<p>
                                                <p style='color: {$thirdBrand}; text-align: center; font-weight: bold; font-size: 16px;'>{$this->translator->_("password.reset.theResetPasswordIs")}:</p>
                                                <br/>
                                                <p style='color: {$secondaryBrand}; text-align: center; font-weight: bold; font-size: 18px;'>{$generatedPassword}</p>
                                            </div>
                                        </tr>
                                        <tr>
                                            <div style='border-width: 1.5px; border-style: solid; border-color: {$primaryBrand}; border-radius: 6px;'></div>
                                            <p style='text-align: center;'>
                                                <i style='font-weight: bold; font-size: 20px; color: {$thirdBrand};'>Crescita</i>
                                                <br/>
                                                <i style='color: {$thirdBrand}'>&copy; 2022-{$currentYear} Crescita</i>
                                            </p>
                                        </tr>
                                    </table>
                                </div>
                            </body>
                        </html>
                    ";
                    $mail->msgHTML($emailHtml, PUBLIC_PATH);
                    $mail->addEmbeddedImage(PUBLIC_PATH . '/images/assets/app_background.png', 'app_background');
                    $mail->AltBody = "{$this->translator->_("password.reset.youAreReceivingThisEmailBecauseYouHaveRequestedAPasswordReset")} {$this->translator->_("password.reset.theResetPasswordIs")}: {$generatedPassword}";
                    $mail->send();
                    LogDAO::registerOperationForUser($targetUser, $this->translator->_("password.reset.passwordReset"));
                    echo ApiResponse::respond();
                } else {
                    echo ApiResponse::respond(102);
                }
            } else {
                echo ApiResponse::respond(101);
            }
        } catch (Exception $ex) {
            echo ApiResponse::respond(103, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function passwordisloginAction()
    {
        try {
            $userDao = new UserDAO();
            $currentUser = $userDao->getUserById($_SESSION[SESSION_USER_ID]);
            if ($currentUser->getPassword() == sha1(md5($currentUser->getLogin()))) {
                echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["same" => 1]);
                exit();
            }
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["same" => 0]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }
}
