<?php

include APPLICATION_PATH . '/includes/daos.php';

class PointofsaleController extends Zend_Controller_Action
{
    private $translator;

    public function init()
    {
        session_start();
        SubscriptionChecker::checkSubscriptionValidity($this);
        if (isset($_SESSION[SESSION_USER_ID]) && $_SESSION[SESSION_USER_ID] != null) {
            $_SESSION[LAST_ACTION_DATE] = new DateTime();
            LogDAO::updateLastActionDateForCurrentUser();
            $this->translator = Translator::init();
        } else {
            if ($this->getRequest()->isPost()) {
                echo ApiResponse::respond(SESSION_EXPIRED_CODE);
                exit();
            } else {
                $this->_helper->redirector('index', 'Index');
            }
        }
    }

    public function indexAction()
    {
        AccessRightChecker::checkAccessRight($this, 86);
        $personDao = new PersonDAO();
        $storeDao = new StoreDAO();
        $subscriptionPackageDao = new SubscriptionPackageDAO();
        $pointOfSaleDao = new PointOfSaleDAO();
        $subscriptionPackage = $subscriptionPackageDao->getSubscriptionPackageById((int)SUBSCRIPTION_PACKAGE);
        $this->view->maxPointsOfSale = $subscriptionPackage->getCountSalePoints() + SUBSCRIPTION_COUNT_MORE_SALE_POINTS;
        $this->view->currentPointsOfSale = $pointOfSaleDao->getPointsOfSaleCount();
        $this->view->translator = $this->translator;
        $this->view->users = $personDao->listUsers();
        $this->view->stores = $storeDao->getAllStores();
    }

    public function datatableAction()
    {
        $name = $_POST["name"] != "" ? $_POST["name"] : null;
        $manager = (int)$_POST["manager"] != 0 ? (int)$_POST["manager"] : null;
        $store = (int)$_POST["store"] != 0 ? (int)$_POST["store"] : null;
        $whereName = "";
        $whereManager = "";
        $whereStore = "";
        if ($name != null) {
            $whereName .= " AND p.nom_point_vente LIKE '%{$name}%'";
        }
        if ($manager != null) {
            $whereManager .= " AND p.gerant_point_vente = {$manager}";
        }
        if ($store != null) {
            $whereStore .= " AND p.id_magasin = {$store}";
        }
        $where = "p.statut = 1{$whereName}{$whereManager}{$whereStore}";
        $pointOfSalesTable = "
                (
                    SELECT p.id_point_vente as point_of_sale_id,
                    p.nom_point_vente as point_of_sale_name, CONCAT(pe.nom, ' ', pe.prenom) as manager,
                    p.localisation as location, m.nom_magasin as store_name
                    FROM point_vente p JOIN magasin m ON p.id_magasin = m.id_magasin
                    LEFT JOIN personne pe ON (p.gerant_point_vente = pe.id_personne OR pe.id_personne IS NULL)
                    WHERE {$where}
                ) AS pointOfSalesTable
        ";
        $primaryKey = 'point_of_sale_id';
        $columns = [
            ['db' => 'point_of_sale_name', 'dt' => 0],
            ['db' => 'manager', 'dt' => 1],
            ['db' => 'store_name', 'dt' => 2],
            ['db' => 'location', 'dt' => 3],
            ['db' => 'point_of_sale_id', 'dt' => 4],
        ];
        $computeDatatableResults = SSP::customSimple($_POST, $pointOfSalesTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        foreach ($datatableResult["data"] as $row) {
            $pointOfSaleName = $row[0];
            $manager = $row[1] != null ? $row[1] : "
                <div class='ui label app-secondary-label'>
                    {$this->translator->_("pointOfSale.index.noAssociatedManager")}
                </div>
            ";
            $storeName = $row[2];
            $location = $row[3];
            $pointOfSaleId = $row[4];
            $actions = "
                <button title='{$this->translator->_("shared.edit")}' about='{$pointOfSaleId}' class='mini ui icon button app-secondary-button app-edit-action'>
                    <i class='edit icon'></i>
                </button>
                <button title='{$this->translator->_("shared.delete")}' about='{$pointOfSaleId}' class='mini ui red icon button app-delete-action'>
                    <i class='trash icon'></i>
                </button>
            ";
            $tableRow = [
                $pointOfSaleName,
                $manager,
                $storeName,
                $location,
                $actions,
            ];
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    /**
     * 100 success
     * 101 duplicate
     * 102 Subscription limits reached
     * 103 unknown error
     */
    public function createAction()
    {
        $subscriptionPackageDao = new SubscriptionPackageDAO();
        $pointOfSaleDao = new PointOfSaleDAO();
        $subscriptionPackage = $subscriptionPackageDao->getSubscriptionPackageById((int)SUBSCRIPTION_PACKAGE);
        $maxPointsOfSale = $subscriptionPackage->getCountSalePoints() + SUBSCRIPTION_COUNT_MORE_SALE_POINTS;
        $currentPointsOfSale = $pointOfSaleDao->getPointsOfSaleCount();
        if ($currentPointsOfSale >= $maxPointsOfSale) {
            echo ApiResponse::respond(102, null, ["maxPointsOfSale" => $maxPointsOfSale, "currentPointsOfSale" => $currentPointsOfSale]);
            exit();
        }
        try {
            $name = $_POST["name"];
            $managerId = (int)$_POST["manager"];
            $storeId = (int)$_POST["store"];
            $location = $_POST["location"];
            if ($pointOfSaleDao->pointOfSaleNameAlreadyExists($name)) {
                echo ApiResponse::respond(101);
                exit();
            }
            $storeDao = new StoreDAO();
            $personDao = new PersonDAO();
            $pointOfSale = $pointOfSaleDao->create();
            $pointOfSale->setNomPointVente($name);
            $pointOfSale->setLocalisation($location);
            $store = $storeDao->getStoreById($storeId);
            if ($managerId != null) {
                $user = $personDao->getPersonById($managerId);
                $pointOfSale->setGerantPointVente($user);
            }
            $pointOfSale->setIdMagasin($store);
            $pointOfSale->setStatut(1);
            $pointOfSaleDao->save($pointOfSale);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("pointOfSale.index.pointOfSaleSuccessfullyRegistered")} : N°{$pointOfSale->getIdPointVente()}({$pointOfSale->getNomPointVente()})"
            );
            $currentPointsOfSale = $pointOfSaleDao->getPointsOfSaleCount();
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["maxPointsOfSale" => $maxPointsOfSale, "currentPointsOfSale" => $currentPointsOfSale]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(103, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function detailAction()
    {
        try {
            $id = (int)$_POST["id"];
            $pointOfSaleDao = new PointOfSaleDAO();
            $pointOfSale = $pointOfSaleDao->getPointOfSaleById($id);
            $result = [
                "name" => $pointOfSale->getNomPointVente(),
                "manager" => $pointOfSale->getGerantPointVente() != null ? $pointOfSale->getGerantPointVente()->getIdPersonne() : "",
                "store" => $pointOfSale->getIdMagasin()->getIdMagasin(),
                "location" => $pointOfSale->getLocalisation(),
            ];
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 duplicate
     * 102 unknown error
     */
    public function updateAction()
    {
        try {
            $id = (int)$_POST["id"];
            $name = $_POST["name"];
            $managerId = (int)$_POST["manager"];
            $storeId = (int)$_POST["store"];
            $location = $_POST["location"];
            $pointOfSaleDao = new PointOfSaleDAO();
            $pointOfSale = $pointOfSaleDao->getPointOfSaleById($id);
            if ($pointOfSaleDao->pointOfSaleNameAlreadyExists($name, $pointOfSale)) {
                echo ApiResponse::respond(101);
                exit();
            }
            $storeDao = new StoreDAO();
            $personDao = new PersonDAO();
            $pointOfSale->setNomPointVente($name);
            $pointOfSale->setLocalisation($location);
            $store = $storeDao->getStoreById($storeId);
            if ($managerId != null) {
                $user = $personDao->getPersonById($managerId);
                $pointOfSale->setGerantPointVente($user);
            } else {
                $pointOfSale->setGerantPointVente(null);
            }
            $pointOfSale->setIdMagasin($store);
            $pointOfSaleDao->update($pointOfSale);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("pointOfSale.index.pointOfSaleUpdatedSuccessfully")} : N°{$pointOfSale->getIdPointVente()}({$pointOfSale->getNomPointVente()})"
            );
            $subscriptionPackageDao = new SubscriptionPackageDAO();
            $subscriptionPackage = $subscriptionPackageDao->getSubscriptionPackageById((int)SUBSCRIPTION_PACKAGE);
            $maxPointsOfSale = $subscriptionPackage->getCountSalePoints() + SUBSCRIPTION_COUNT_MORE_SALE_POINTS;
            $currentPointsOfSale = $pointOfSaleDao->getPointsOfSaleCount();
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["maxPointsOfSale" => $maxPointsOfSale, "currentPointsOfSale" => $currentPointsOfSale]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function deleteAction()
    {
        try {
            $id = (int)$_POST["id"];
            $pointOfSaleDao = new PointOfSaleDAO();
            $pointOfSale = $pointOfSaleDao->getPointOfSaleById($id);
            if ($pointOfSale->getStatut() == 0) {
                echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE);
                exit();
            }
            $pointOfSale->setStatut(0);
            $pointOfSaleDao->update($pointOfSale);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("pointOfSale.index.pointOfSaleDeletedSuccessfully")} : N°{$pointOfSale->getIdPointVente()}({$pointOfSale->getNomPointVente()})"
            );
            $subscriptionPackageDao = new SubscriptionPackageDAO();
            $subscriptionPackage = $subscriptionPackageDao->getSubscriptionPackageById((int)SUBSCRIPTION_PACKAGE);
            $maxPointsOfSale = $subscriptionPackage->getCountSalePoints() + SUBSCRIPTION_COUNT_MORE_SALE_POINTS;
            $currentPointsOfSale = $pointOfSaleDao->getPointsOfSaleCount();
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["maxPointsOfSale" => $maxPointsOfSale, "currentPointsOfSale" => $currentPointsOfSale]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }
}
