<?php

include APPLICATION_PATH . '/includes/daos.php';

class ProductController extends Zend_Controller_Action
{
    private $translator;

    public function init()
    {
        session_start();
        SubscriptionChecker::checkSubscriptionValidity($this);
        if (isset($_SESSION[SESSION_USER_ID]) && $_SESSION[SESSION_USER_ID] != null) {
            $_SESSION[LAST_ACTION_DATE] = new DateTime();
            LogDAO::updateLastActionDateForCurrentUser();
            $this->translator = Translator::init();
        } else {
            if ($this->getRequest()->isPost()) {
                echo ApiResponse::respond(SESSION_EXPIRED_CODE);
                exit();
            } else {
                $this->_helper->redirector('index', 'Index');
            }
        }
    }

    public function indexAction()
    {
        AccessRightChecker::checkAccessRight($this, 9);
        $genderDao = new GenderDAO();
        $productCategoryDao = new ProductCategoryDAO();
        $productNatureDao = new ProductNatureDAO();
        $productDao = new ProductDAO();
        $subscriptionPackageDao = new SubscriptionPackageDAO();
        $subscriptionPackage = $subscriptionPackageDao->getSubscriptionPackageById((int)SUBSCRIPTION_PACKAGE);
        $this->view->maxProducts = $subscriptionPackage->getCountProducts() + SUBSCRIPTION_MORE_COUNT_PRODUCTS;
        $this->view->currentProducts = $productDao->getProductsCount();
        $this->view->genders = $genderDao->getGenders();
        $this->view->categories = $productCategoryDao->getAllProductCategories();
        $this->view->natures = $productNatureDao->getProductNatures();
        $this->view->translator = $this->translator;
    }

    public function datatableAction()
    {
        $name = $_POST["name"] != "" ? $_POST["name"] : null;
        $category = (int)$_POST["category"] != 0 ? (int)$_POST["category"] : null;
        $reference = $_POST["reference"] != "" ? $_POST["reference"] : null;
        $type = $_POST["type"] != "" ? (int)$_POST["type"] : null;
        $whereName = " AND p.designation LIKE '%{$name}%'";
        $whereCategory = "";
        $whereReference = "";
        $whereType = "";
        if (!is_null($name)) {
            $whereName .= " AND p.designation LIKE '%{$name}%'";
        }
        if (!is_null($category)) {
            $whereCategory .= " AND p.id_categorie_produit = {$category}";
        }
        if (!is_null($reference)) {
            $whereReference .= " AND p.reference_produit LIKE '%{$reference}%'";
        }
        if (!is_null($type)) {
            $whereType .= " AND p.stockable = {$type}";
        }
        $where = "p.statut = 1{$whereName}{$whereCategory}{$whereReference}{$whereType} order by p.designation ASC";
        $productsTable = "
                (
                    SELECT p.id_produit AS product_id,
                    p.designation AS product_designation, p.prix_vente AS product_sale_price, 
                    p.prix_vente2 AS product_sale_price_2, p.prix_vente3 AS product_sale_price_3, 
                    p.prix_achat AS product_buying_price, p.reference_produit AS product_reference, 
                    c.nom AS category_name, p.stockable AS product_type, p.image AS product_image
                    FROM produit p JOIN categorie_produit c ON p.id_categorie_produit = c.id_categorie 
                    WHERE {$where}
                ) AS productsTable
        ";
        $primaryKey = 'product_id';
        $columns = [
            ['db' => 'product_designation', 'dt' => 0],
            ['db' => 'category_name', 'dt' => 1],
            ['db' => 'product_sale_price', 'dt' => 2],
            ['db' => 'product_sale_price_2', 'dt' => 3],
            ['db' => 'product_sale_price_3', 'dt' => 4],
            ['db' => 'product_buying_price', 'dt' => 5],
            ['db' => 'product_id', 'dt' => 6],
            ['db' => 'product_reference', 'dt' => 7],
            ['db' => 'product_type', 'dt' => 8],
            ['db' => 'product_image', 'dt' => 9],
        ];
        $computeDatatableResults = SSP::customSimple($_POST, $productsTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        foreach ($datatableResult["data"] as $row) {
            $productDesignation = $row[0];
            $categoryName = $row[1];
            $productSalePrice = number_format((float)$row[2], 0, '', ' ');
            $productSalePrice2 = number_format((float)$row[3], 0, '', ' ');
            $productSalePrice3 = number_format((float)$row[4], 0, '', ' ');
            $productBuyingPrice = number_format((float)$row[5], 0, '', ' ');
            $productId = $row[6];
            $productReference = $row[7];
            $productType = $this->translator->_((int)$row[8] == 1 ? "product.index.product" : "product.index.service");
            $productImage = $row[9];
            $actions = "
                <button title='{$this->translator->_("shared.consult")}' about='{$productId}' class='mini ui icon button app-primary-button app-consult-action'>
                    <i class='eye icon'></i>
                </button>
                <button title='{$this->translator->_("shared.edit")}' about='{$productId}' class='mini ui icon button app-secondary-button app-edit-action'>
                    <i class='edit icon'></i>
                </button>
                <button title='{$this->translator->_("shared.delete")}' about='{$productId}' class='mini ui red icon button app-delete-action'>
                    <i class='trash icon'></i>
                </button>
            ";
            $tableRow = [
                [
                    $productImage,
                    $productId,
                ],
                $productDesignation,
                $categoryName,
                $productType,
            ];
            if (GESTION_REFERENCE_PRODUIT == 1) {
                $tableRow[] = $productReference;
            }
            $tableRow[] = $productSalePrice;
            $tableRow[] = $productSalePrice2;
            $tableRow[] = $productSalePrice3;
            $tableRow[] = $productBuyingPrice;
            $tableRow[] = $actions;
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    /**
     * 100 success
     * 101 duplicate reference
     * 102 duplicate name
     * 103 duplicate bar code
     * 104 subscription limits reached
     * 105 unknown error
     */
    public function createAction()
    {
        $productDao = new ProductDAO();
        $subscriptionPackageDao = new SubscriptionPackageDAO();
        $subscriptionPackage = $subscriptionPackageDao->getSubscriptionPackageById((int)SUBSCRIPTION_PACKAGE);
        $maxProducts = $subscriptionPackage->getCountProducts() + SUBSCRIPTION_MORE_COUNT_PRODUCTS;
        $currentProducts = $productDao->getProductsCount();
        if ($currentProducts >= $maxProducts) {
            echo ApiResponse::respond(104, null, ["maxProducts" => $maxProducts, "currentProducts" => $currentProducts]);
            exit();
        }
        $databaseConnection = $productDao->entityManager->getConnection();
        $databaseConnection->beginTransaction();
        try {
            $gender = isset($_POST["gender"]) ? $_POST["gender"] : null;
            $reference = isset($_POST["reference"]) ? $_POST["reference"] : null;
            if ($reference != null && $productDao->checkProductReferenceExist($reference)) {
                echo ApiResponse::respond(101);
                exit();
            }
            $name = $_POST["name"];
            if ($productDao->checkProductNameExist($name)) {
                echo ApiResponse::respond(102);
                exit();
            }
            $reductionPercent = $_POST["reductionPercent"] != "" ? (int)$_POST["reductionPercent"] : null;
            $productIsAService = (int)$_POST["productIsAService"];
            $barCode = $_POST["barCode"] != "" ? $_POST["barCode"] : null;
            if ($barCode != null && $productDao->checkProductBarCodeExist($barCode)) {
                echo ApiResponse::respond(103);
                exit();
            }
            $productIsPerishable = isset($_POST["productIsPerishable"]) ? (int)$_POST["productIsPerishable"] : 0; //if not sent product is not perishable at all
            $category = (int)$_POST["category"];
            $subCategory = (int)$_POST["subCategory"];
            $description = $_POST["description"];
            $productWillBeInventoried = (int)$_POST["productWillBeInventoried"];
            $nature = (int)$_POST["nature"];
            $priority = $_POST["priority"] != "" ? (int)$_POST["priority"] : null;
            $stockAlertQuantity = $_POST["stockAlertQuantity"] != "" ? (int)$_POST["stockAlertQuantity"] : null;
            $buyingPrice = (float)$_POST["buyingPrice"];
            $defaultSalePrice = (float)$_POST["defaultSalePrice"];
            $salePrice2 = (float)$_POST["salePrice2"];
            $salePrice3 = (float)$_POST["salePrice3"];
            $product = $productDao->create();
            if ($gender != null) {
                $genderDao = new GenderDAO();
                $effectiveGender = $genderDao->getGenderById($gender);
                $product->setIdGenre($effectiveGender);
            }
            if ($reference != null) {
                $product->setReferenceProduit($reference);
            }
            $product->setDesignation($name);
            if ($reductionPercent != null) {
                $product->setPourcentageMaxRemise($reductionPercent);
            }
            $product->setStockable($productIsAService);
            $productCategoryDao = new ProductCategoryDAO();
            $effectiveCategory = $productCategoryDao->getProductCategoryById($category);
            if ($subCategory != 0) {
                $effectiveCategory = $productCategoryDao->getProductCategoryById($subCategory);
            }
            $product->setIdCategorieProduit($effectiveCategory);
            $product->setDescription($description);
            $product->setInventorier($productWillBeInventoried);
            if ($nature != 0) {
                $productNatureDao = new ProductNatureDAO();
                $effectiveNature = $productNatureDao->getProductNatureById($nature);
                $product->setIdNature($effectiveNature);
            }
            if ($priority != null) {
                $product->setPriorite($priority);
            }
            if ($stockAlertQuantity != null) {
                $product->setQuantiteAlerte($stockAlertQuantity);
            }
            $product->setPrixAchat($buyingPrice);
            $taxesPercent = (float)VALEUR_TVA;
            $product->setPrixVente($defaultSalePrice);
            $product->setPrixTtc($defaultSalePrice + $defaultSalePrice * $taxesPercent / 100);
            $product->setPrixVente2($salePrice2);
            $product->setPrixTtc2($salePrice2 + $salePrice2 * $taxesPercent / 100);
            $product->setPrixVente3($salePrice3);
            $product->setPrixTtc3($salePrice3 + $salePrice3 * $taxesPercent / 100);
            $product->setMarge($defaultSalePrice - $buyingPrice);
            $product->setCodeBarre($barCode);
            $product->setPerissable($productIsPerishable);
            $product->setStatut(1);
            $productDao->save($product);
            if (isset($_FILES["newProductImageInput"]) && $_FILES["newProductImageInput"]["name"] != "") {
                $uploadedFilename = $_FILES["newProductImageInput"]["name"];
                $uploadFilenameArray = explode(".", $uploadedFilename);
                $extension = end($uploadFilenameArray);
                $currentDate = new DateTime();
                $randomName = $currentDate->format("YmdHis");
                $filename = PUBLIC_PATH . "/images/products/{$randomName}_{$product->getIdProduit()}.{$extension}";
                $product->setImage("{$randomName}_{$product->getIdProduit()}.{$extension}");
                move_uploaded_file($_FILES["newProductImageInput"]["tmp_name"], $filename);
                $productDao->update($product);
            }
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("product.index.productSuccessfullyRegistered")} : N°{$product->getIdProduit()}({$product->getDesignation()})"
            );
            $databaseConnection->commit();
            $currentProducts = $productDao->getProductsCount();
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["maxProducts" => $maxProducts, "currentProducts" => $currentProducts]);
        } catch (Exception $ex) {
            $databaseConnection->rollBack();
            echo ApiResponse::respond(105, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function detailAction()
    {
        try {
            $id = (int)$_POST["id"];
            $productDao = new ProductDAO();
            $product = $productDao->getProductById($id);
            $result = [
                "gender" => $product->getIdGenre() != null ? $product->getIdGenre()->getIdGenre() : "",
                "genderName" => $product->getIdGenre() != null ? $this->translator->_($product->getIdGenre()->getNom()) : "/",
                "reference" => $product->getReferenceProduit(),
                "name" => $product->getDesignation(),
                "reductionPercent" => $product->getPourcentageMaxRemise(),
                "productIsAService" => $product->getStockable(),
                "productIsAServiceLabel" => ($product->getStockable() == 1)? $this->translator->_("shared.no"):$this->translator->_("shared.yes"),
                "image" => $product->getImage() != null ? $product->getImage() : "",
                "category" => $product->getIdCategorieProduit()->getIdCategorie(),
                "categoryName" => $product->getIdCategorieProduit()->getNom(),
                "description" => $product->getDescription(),
                "productWillBeInventoried" => $product->getInventorier(),
                "productWillBeInventoriedLabel" => ($product->getInventorier() == 1)? $this->translator->_("shared.yes"):$this->translator->_("shared.no"),
                "nature" => $product->getIdNature() != null ? $product->getIdNature()->getIdNature() : "",
                "natureName" => $product->getIdNature() != null ? $product->getIdNature()->getNomNature() : "",
                "priority" => $product->getPriorite(),
                "stockAlertQuantity" => $product->getQuantiteAlerte(),
                "buyingPrice" => $product->getPrixAchat(),
                "defaultSalePrice" => $product->getPrixVente(),
                "salePrice2" => $product->getPrixVente2(),
                "salePrice3" => $product->getPrixVente3(),
                "barCode" => $product->getCodeBarre(),
                "productIsPerishable" => $product->getPerissable(),
                "productIsPerishableLabel" => $product->getPerissable() == 1 ? $this->translator->_("shared.yes") : $this->translator->_("shared.no"),
            ];
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 duplicate reference
     * 102 duplicate name
     * 103 duplicate bar code
     * 104 unknown error
     */
    public function updateAction()
    {
        $productDao = new ProductDAO();
        $databaseConnection = $productDao->entityManager->getConnection();
        $databaseConnection->beginTransaction();
        try {
            $id = (int)$_POST["id"];
            $product = $productDao->getProductById($id);
            $gender = isset($_POST["gender"]) ? $_POST["gender"] : null;
            $reference = isset($_POST["reference"]) ? $_POST["reference"] : null;
            if ($reference != null && $productDao->checkProductReferenceExist($reference, $product)) {
                echo ApiResponse::respond(101);
                exit();
            }
            $name = $_POST["name"];
            if ($productDao->checkProductNameExist($name, $product)) {
                echo ApiResponse::respond(102);
                exit();
            }
            $reductionPercent = $_POST["reductionPercent"] != "" ? (int)$_POST["reductionPercent"] : null;
            $productIsAService = (int)$_POST["productIsAService"];
            $barCode = $_POST["barCode"] != "" ? $_POST["barCode"] : null;
            if ($barCode != null && $productDao->checkProductBarCodeExist($barCode, $product)) {
                echo ApiResponse::respond(103);
                exit();
            }
            $productIsPerishable = isset($_POST["productIsPerishable"]) ? (int)$_POST["productIsPerishable"] : 0; //if not sent product is not perishable at all
            $category = (int)$_POST["category"];
            $subCategory = (int)$_POST["subCategory"];
            $description = $_POST["description"];
            $productWillBeInventoried = (int)$_POST["productWillBeInventoried"];
            $nature = (int)$_POST["nature"];
            $priority = $_POST["priority"] != "" ? (int)$_POST["priority"] : null;
            $stockAlertQuantity = $_POST["stockAlertQuantity"] != "" ? (int)$_POST["stockAlertQuantity"] : null;
            $buyingPrice = (float)$_POST["buyingPrice"];
            $defaultSalePrice = (float)$_POST["defaultSalePrice"];
            $salePrice2 = (float)$_POST["salePrice2"];
            $salePrice3 = (float)$_POST["salePrice3"];
            $applyPrice = (int)$_POST["applyPrice"];
            if ($gender != null) {
                $genderDao = new GenderDAO();
                $effectiveGender = $genderDao->getGenderById($gender);
                $product->setIdGenre($effectiveGender);
            } else {
                $product->setIdGenre(null);
            }
            if ($reference != null) {
                $product->setReferenceProduit($reference);
            } else {
                $product->setReferenceProduit(null);
            }
            $product->setDesignation($name);
            if ($reductionPercent != null) {
                $product->setPourcentageMaxRemise($reductionPercent);
            } else {
                $product->setPourcentageMaxRemise(null);
            }
            $product->setStockable($productIsAService);
            $productCategoryDao = new ProductCategoryDAO();
            $effectiveCategory = $productCategoryDao->getProductCategoryById($category);
            if ($subCategory != 0) {
                $effectiveCategory = $productCategoryDao->getProductCategoryById($subCategory);
            }
            $product->setIdCategorieProduit($effectiveCategory);
            $product->setDescription($description);
            $product->setInventorier($productWillBeInventoried);
            if ($nature != 0) {
                $productNatureDao = new ProductNatureDAO();
                $effectiveNature = $productNatureDao->getProductNatureById($nature);
                $product->setIdNature($effectiveNature);
            } else {
                $product->setIdNature(null);
            }
            if ($priority != null) {
                $product->setPriorite($priority);
            } else {
                $product->setPriorite(null);
            }
            if ($stockAlertQuantity != null) {
                $product->setQuantiteAlerte($stockAlertQuantity);
            } else {
                $product->setQuantiteAlerte(null);
            }
            $product->setPrixAchat($buyingPrice);
            $taxesPercent = (float)VALEUR_TVA;
            $product->setPrixVente($defaultSalePrice);
            $product->setPrixTtc($defaultSalePrice + $defaultSalePrice * $taxesPercent / 100);
            $product->setPrixVente2($salePrice2);
            $product->setPrixTtc2($salePrice2 + $salePrice2 * $taxesPercent / 100);
            $product->setPrixVente3($salePrice3);
            $product->setPrixTtc3($salePrice3 + $salePrice3 * $taxesPercent / 100);
            $product->setMarge($defaultSalePrice - $buyingPrice);
            $product->setCodeBarre($barCode);
            $product->setPerissable($productIsPerishable);
            if (isset($_FILES["editProductImageInput"]) && $_FILES["editProductImageInput"]["name"] != "") {
                $uploadedFilename = $_FILES["editProductImageInput"]["name"];
                $uploadFilenameArray = explode(".", $uploadedFilename);
                $extension = end($uploadFilenameArray);
                $currentDate = new DateTime();
                $randomName = $currentDate->format("YmdHis");
                $filename = PUBLIC_PATH . "/images/products/{$randomName}_{$product->getIdProduit()}.{$extension}";
                if ($product->getImage() != null) {
                    error_reporting(E_ERROR | E_PARSE);
                    unlink(PUBLIC_PATH . "/images/products/{$product->getImage()}");
                }
                $product->setImage("{$randomName}_{$product->getIdProduit()}.{$extension}");
                move_uploaded_file($_FILES["editProductImageInput"]["tmp_name"], $filename);
            }
            $productDao->update($product);
            //Update existing stocks price if accepted and product is not as service
            if ($applyPrice == 1 && $productIsAService == 1) {
                $stockDao = new StockDAO();
                $userDao = new UserDAO();
                $currentUser = $userDao->getUserById($_SESSION[SESSION_USER_ID]);
                $specificStore = null;
                if ($currentUser->getIdPointVente() != null) {
                    $specificStore = $currentUser->getIdPointVente()->getIdMagasin();
                }
                $allStockAvailableForProduct = $stockDao->getAllAvailableStockByProductAndStore($product, $specificStore);
                foreach ($allStockAvailableForProduct as $stock) {
                    $stock->setPrixVente($defaultSalePrice);
                    $stock->setPrixVente2($salePrice2);
                    $stock->setPrixVente3($salePrice3);
                    $stock->setPrixTtc($defaultSalePrice + $defaultSalePrice * $taxesPercent / 100);
                    $stock->setPrixTtc2($salePrice2 + $salePrice2 * $taxesPercent / 100);
                    $stock->setPrixTtc3($salePrice3 + $salePrice3 * $taxesPercent / 100);
                    $stock->setPrixAchat($buyingPrice);
                    $stock->setMarge($defaultSalePrice - $buyingPrice);
                    $stockDao->update($stock);
                }
            }
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("product.index.productUpdatedSuccessfully")} : N°{$product->getIdProduit()}({$product->getDesignation()})"
            );
            $databaseConnection->commit();
            $subscriptionPackageDao = new SubscriptionPackageDAO();
            $subscriptionPackage = $subscriptionPackageDao->getSubscriptionPackageById((int)SUBSCRIPTION_PACKAGE);
            $maxProducts = $subscriptionPackage->getCountProducts() + SUBSCRIPTION_MORE_COUNT_PRODUCTS;
            $currentProducts = $productDao->getProductsCount();
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["maxProducts" => $maxProducts, "currentProducts" => $currentProducts]);
        } catch (Exception $ex) {
            $databaseConnection->rollBack();
            echo ApiResponse::respond(104, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function deleteAction()
    {
        $productDao = new ProductDAO();
        $databaseConnection = $productDao->entityManager->getConnection();
        $databaseConnection->beginTransaction();
        try {
            $id = (int)$_POST["id"];
            $product = $productDao->getProductById($id);
            if ($product->getStatut() == 0) {
                echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE);
                exit();
            }
            $product->setStatut(0);
            $productDao->update($product);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("product.index.productDeletedSuccessfully")} : N°{$product->getIdProduit()}({$product->getDesignation()})"
            );
            //If product is storable delete all stocks available
            if ($product->getStockable() == 1) {
                $stockDao = new StockDAO();
                $stockDao->deleteAllAvailableStockOfProduct($product);
            }
            $databaseConnection->commit();
            $subscriptionPackageDao = new SubscriptionPackageDAO();
            $subscriptionPackage = $subscriptionPackageDao->getSubscriptionPackageById((int)SUBSCRIPTION_PACKAGE);
            $maxProducts = $subscriptionPackage->getCountProducts() + SUBSCRIPTION_MORE_COUNT_PRODUCTS;
            $currentProducts = $productDao->getProductsCount();
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["maxProducts" => $maxProducts, "currentProducts" => $currentProducts]);
        } catch (Exception $ex) {
            $databaseConnection->rollBack();
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    public function tariffsAction()
    {
        AccessRightChecker::checkAccessRight($this, 55);
        $stockDao = new StockDAO();
        $storeDao = new StoreDAO();
        $productCategoryDao = new ProductCategoryDao();
        $currentUser = LogDAO::getCurrentUser();
        $stores = [];
        $productCategories = [];
        $productCategoriesIds = null;
        if ($currentUser->getIdPointVente() == null || in_array($currentUser->getIdProfil()->getCode(), IDENTIFIED_PROFILES_CODES)) {
            $stores = $storeDao->getAllStores();
            $productCategoriesIds = $stockDao->getAllCategoriesAvailableInStockByStore();
        } else {
            $stores[] = $currentUser->getIdPointVente()->getIdMagasin();
            $productCategoriesIds = $stockDao->getAllCategoriesAvailableInStockByStore($stores[0]);
        }
        foreach ($productCategoriesIds as $productCategoryId) {
            $productCategory = $productCategoryDao->getProductCategoryById($productCategoryId["idCategorie"]);
            $productCategories[] = $productCategory;
        }
        $this->view->stores = $stores;
        $this->view->productCategories = $productCategories;
        $this->view->translator = $this->translator;
    }

    public function datatabletariffsAction()
    {
        try {
            $store = (int)$_POST["store"] != 0 ? (int)$_POST["store"] : null;
            $category = (int)$_POST["category"] != 0 ? (int)$_POST["category"] : null;
            $name = $_POST["name"] != "" ? $_POST["name"] : null;
            $batchNumber = $_POST["batchNumber"] != "" ? $_POST["batchNumber"] : null;
            $whereCategory = "";
            $whereStore = "";
            $whereName = "";
            $whereBatchNumber = "";
            if ($store != null) {
                $whereStore .= " AND s.id_magasin = {$store}";
            }
            if ($category != null) {
                $whereCategory .= " AND p.id_categorie_produit = {$category}";
            }
            if ($name != null) {
                $whereName .= " AND s.designation LIKE '%{$name}%'";
            }
            if ($batchNumber != null) {
                $whereBatchNumber .= " AND s.num_lot_fournisseur LIKE '%{$batchNumber}%'";
            }
            $where = "s.statut = 1 AND s.quantite > 0 AND p.statut = 1{$whereStore}{$whereCategory}{$whereName}{$whereBatchNumber}";
            $tariffsTable = "
                (
                    SELECT s.id_stock AS stock_id, s.id_magasin AS store_id, s.id_produit AS product_id,
                    p.id_categorie_produit AS product_category_id, s.designation AS designation,
                    s.date_entree AS entry_date, s.quantite AS quantity, s.quantite_entree AS entry_quantity,
                    s.prix_achat AS buying_price, s.prix_vente AS sale_price, s.prix_vente2 AS sale_price_2,
                    s.prix_vente3 AS sale_price_3, s.num_lot AS batch_number, 
                    s.num_lot_fournisseur AS provider_batch_number, s.date_peremption AS expiration_date,
                    s.taux_tva_vente AS sale_vat_rate, s.taux_tva_achat AS buy_vat_rate,
                    p.quantite_alerte AS alert_quantity, m.nom_magasin AS store_name
                    FROM stock s JOIN magasin m ON s.id_magasin = m.id_magasin JOIN produit p ON s.id_produit = p.id_produit 
                    WHERE {$where}
                ) AS tariffsTable
            ";
            $primaryKey = 'stock_id';
            if (GESTION_DATE_PEREMPTION_PRODUIT == 1) {
                $columns = [
                    ['db' => 'designation', 'dt' => 0],
                    ['db' => 'store_name', 'dt' => 1],
                    ['db' => 'batch_number', 'dt' => 2],
                    ['db' => 'quantity', 'dt' => 3],
                    ['db' => 'expiration_date', 'dt' => 4],
                    ['db' => 'buying_price', 'dt' => 5],
                    ['db' => 'sale_price', 'dt' => 6],
                    ['db' => 'sale_price_2', 'dt' => 7],
                    ['db' => 'sale_price_3', 'dt' => 8],
                    ['db' => 'stock_id', 'dt' => 9],
                    ['db' => 'alert_quantity', 'dt' => 10],
                ];
            } else {
                $columns = [
                    ['db' => 'designation', 'dt' => 0],
                    ['db' => 'store_name', 'dt' => 1],
                    ['db' => 'batch_number', 'dt' => 2],
                    ['db' => 'quantity', 'dt' => 3],
                    ['db' => 'buying_price', 'dt' => 4],
                    ['db' => 'sale_price', 'dt' => 5],
                    ['db' => 'sale_price_2', 'dt' => 6],
                    ['db' => 'sale_price_3', 'dt' => 7],
                    ['db' => 'stock_id', 'dt' => 8],
                    ['db' => 'alert_quantity', 'dt' => 9],
                ];
            }
            $computeDatatableResults = SSP::customSimple($_POST, $tariffsTable, $primaryKey, $columns);
            $datatableResult = $computeDatatableResults["datatable"];
            $finalResult = $computeDatatableResults["custom"];
            $stockDao = new StockDAO();
            foreach ($datatableResult["data"] as $row) {
                $designation = $row[0];
                $storeName = $row[1];
                $batchNumber = $row[2];
                $quantity = (int)$row[3];
                $stockId = GESTION_DATE_PEREMPTION_PRODUIT == 1 ? $row[9] : $row[8];
                $stock = $stockDao->getStockById($stockId);
                $expirationDate = GESTION_DATE_PEREMPTION_PRODUIT == 1 && $stock->getIdProduit()->getPerissable() == 1 && $row[4] != null ? $row[4] : null;
                $buyingPrice = number_format(GESTION_DATE_PEREMPTION_PRODUIT == 1 ? (float)$row[5] : (float)$row[4], 0, '', ' ');
                $salePrice = number_format(GESTION_DATE_PEREMPTION_PRODUIT == 1 ? (float)$row[6] : (float)$row[5], 0, '', ' ');
                $salePrice2 = number_format(GESTION_DATE_PEREMPTION_PRODUIT == 1 ? (float)$row[7] : (float)$row[6], 0, '', ' ');
                $salePrice3 = number_format(GESTION_DATE_PEREMPTION_PRODUIT == 1 ? (float)$row[8] : (float)$row[7], 0, '', ' ');
                $alertQuantity = GESTION_DATE_PEREMPTION_PRODUIT == 1 ? $row[10] : $row[9];
                $alertQuantity = $alertQuantity != null ? (int)$alertQuantity : (int)QUANTITE_SEUIL_PRODUIT_PAR_DEFAUT;
                $buyingPriceHtmlContent = "
                    <form class='ui form fluid price-form'>
                        <div class='ui field input-field buying-price hidden' about='{$stockId}'>
                            <div class='ui action input'>
                                <input class='buying-price' about='{$stockId}' type='text' placeholder='{$this->translator->_("product.tariffs.price")}'>
                                <button class='ui icon button app-secondary-button buying-price save-button' about='{$stockId}'>
                                    <i class='check icon'></i>
                                </button>
                            </div>
                        </div>
                        <div class='ui field'>
                            <button class='ui right labeled icon button app-primary-button buying-price edit-button' about='{$stockId}'>
                                <i class='edit icon edit-icon'></i>
                                <b class='buying-price' about='{$stockId}'>{$buyingPrice}</b>
                            </button>
                        </div>
                    </form>
                ";
                $salePriceHtmlContent = "
                    <form class='ui form fluid price-form'>
                        <div class='ui field input-field sale-price hidden' about='{$stockId}'>
                            <div class='ui action input'>
                                <input class='sale-price' about='{$stockId}' type='text' placeholder='{$this->translator->_("product.tariffs.price")}'>
                                <button class='ui icon button app-secondary-button sale-price save-button' about='{$stockId}'>
                                    <i class='check icon'></i>
                                </button>
                            </div>
                        </div>
                        <div class='ui field'>
                            <button class='ui right labeled icon button app-primary-button sale-price edit-button' about='{$stockId}'>
                                <i class='edit icon edit-icon'></i>
                                <b class='sale-price' about='{$stockId}'>{$salePrice}</b>
                            </button>
                        </div>
                    </form>
                ";
                $salePrice2HtmlContent = "
                    <form class='ui form fluid price-form'>
                        <div class='ui field input-field sale-price-2 hidden' about='{$stockId}'>
                            <div class='ui action input'>
                                <input class='sale-price-2' about='{$stockId}' type='text' placeholder='{$this->translator->_("product.tariffs.price")}'>
                                <button class='ui icon button app-secondary-button sale-price-2 save-button' about='{$stockId}'>
                                    <i class='check icon'></i>
                                </button>
                            </div>
                        </div>
                        <div class='ui field'>
                            <button class='ui right labeled icon button app-primary-button sale-price-2 edit-button' about='{$stockId}'>
                                <i class='edit icon edit-icon'></i>
                                <b class='sale-price-2' about='{$stockId}'>{$salePrice2}</b>
                            </button>
                        </div>
                    </form>
                ";
                $salePrice3HtmlContent = "
                    <form class='ui form fluid price-form'>
                        <div class='ui field input-field sale-price-3 hidden' about='{$stockId}'>
                            <div class='ui action input'>
                                <input class='sale-price-3' about='{$stockId}' type='text' placeholder='{$this->translator->_("product.tariffs.price")}'>
                                <button class='ui icon button app-secondary-button sale-price-3 save-button' about='{$stockId}'>
                                    <i class='check icon'></i>
                                </button>
                            </div>
                        </div>
                        <div class='ui field'>
                            <button class='ui right labeled icon button app-primary-button sale-price-3 edit-button' about='{$stockId}'>
                                <i class='edit icon edit-icon'></i>
                                <b class='sale-price-3' about='{$stockId}'>{$salePrice3}</b>
                            </button>
                        </div>
                    </form>
                ";
                $backgroundColor = "";
                if (GESTION_DATE_PEREMPTION_PRODUIT == 1) {
                    if ($expirationDate != null) {
                        $effectiveExpirationDate = new DateTime($expirationDate);
                        $currentDate = new DateTime();
                        $differenceInterval = $effectiveExpirationDate->diff($currentDate);
                        $daysCount = -1 * $differenceInterval->format('%R%a');
                        if ($daysCount <= 0) {
                            $backgroundColor = COULEUR_PRODUIT_PERIME;
                            if ($quantity <= $alertQuantity) {
                                $backgroundColor = "linear-gradient(to bottom, " . COULEUR_PRODUIT_SEUIL_ATTEINT . " 10%, " . COULEUR_PRODUIT_PERIME . " 70%)";
                            }
                        } else {
                            if ($quantity <= $alertQuantity) {
                                $backgroundColor = COULEUR_PRODUIT_SEUIL_ATTEINT;
                            } else {
                                if ($daysCount <= NOMBRE_JOUR_POUR_ALERTE_PEREMPTION) {
                                    $backgroundColor = COULEUR_PRODUIT_PROCHE_DATE_PEREMPTION;
                                }
                            }
                        }
                    } else {
                        if ($quantity <= $alertQuantity) {
                            $backgroundColor = COULEUR_PRODUIT_SEUIL_ATTEINT;
                        }
                    }
                } else {
                    if ($quantity <= $alertQuantity) {
                        $backgroundColor = COULEUR_PRODUIT_SEUIL_ATTEINT;
                    }
                }
                $tableRow = [];
                if (GESTION_DATE_PEREMPTION_PRODUIT == 1) {
                    $effectiveExpirationDate = $expirationDate != null ? new DateTime($expirationDate) : null;
                    $tableRow = [
                        $designation,
                        $storeName,
                        $batchNumber,
                        $quantity,
                        $effectiveExpirationDate != null ? $effectiveExpirationDate->format(DATE_FORMAT) : null,
                        $buyingPriceHtmlContent,
                        $salePriceHtmlContent,
                        $salePrice2HtmlContent,
                        $salePrice3HtmlContent,
                        $backgroundColor,
                    ];
                } else {
                    $tableRow = [
                        $designation,
                        $storeName,
                        $batchNumber,
                        $quantity,
                        $buyingPriceHtmlContent,
                        $salePriceHtmlContent,
                        $salePrice2HtmlContent,
                        $salePrice3HtmlContent,
                        $backgroundColor,
                    ];
                }
                $finalResult["data"][] = $tableRow;
            }
            echo json_encode($finalResult);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function updatepriceAction()
    {
        try {
            $stockDao = new StockDAO();
            $stockId = (int)$_POST["stockId"];
            $buyingPrice = isset($_POST["buyingPrice"]) ? (float)$_POST["buyingPrice"] : null;
            $salePrice = isset($_POST["salePrice"]) ? (float)$_POST["salePrice"] : null;
            $salePrice2 = isset($_POST["salePrice2"]) ? (float)$_POST["salePrice2"] : null;
            $salePrice3 = isset($_POST["salePrice3"]) ? (float)$_POST["salePrice3"] : null;
            $stock = $stockDao->getStockById($stockId);
            if ($buyingPrice != null) {
                $stock->setPrixAchat($buyingPrice);
            }
            if ($salePrice != null) {
                $stock->setPrixVente($salePrice);
            }
            if ($salePrice2 != null) {
                $stock->setPrixVente2($salePrice2);
            }
            if ($salePrice3 != null) {
                $stock->setPrixVente3($salePrice3);
            }
            $stockDao->update($stock);
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function productsofcategoryAction()
    {
        try {
            $productCategoryDao = new ProductCategoryDAO();
            $productDao = new ProductDAO();
            $id = (int)$_POST["id"];
            $productCategory = $productCategoryDao->getProductCategoryById($id);
            $productsOfCategory = $productDao->getProductsByCategory($productCategory);
            $result = [];
            foreach ($productsOfCategory as $product) {
                $result[] = [
                    "id" => $product->getIdProduit(),
                    "name" => $product->getDesignation(),
                ];
            }
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function productsofcategoriesAction()
    {
        try {
            $productCategoryDao = new ProductCategoryDAO();
            $productDao = new ProductDAO();
            $productCategoriesIds = json_decode($_POST["categories"], true);
            $productCategories = [];
            foreach($productCategoriesIds as $productCategoryId) {
                $productCategories[] = $productCategoryDao->getProductCategoryById((int)$productCategoryId);
            }
            $productsOfCategories = $productDao->getProductsOfCategories($productCategories);
            $result = [];
            foreach ($productsOfCategories as $product) {
                $result[] = [
                    "id" => $product->getIdProduit(),
                    "name" => $product->getDesignation(),
                ];
            }
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }
}
