<?php

include APPLICATION_PATH . '/includes/daos.php';

class ProductcategoryController extends Zend_Controller_Action
{
    private $translator;

    public function init()
    {
        session_start();
        SubscriptionChecker::checkSubscriptionValidity($this);
        if (isset($_SESSION[SESSION_USER_ID]) && $_SESSION[SESSION_USER_ID] != null) {
            $_SESSION[LAST_ACTION_DATE] = new DateTime();
            LogDAO::updateLastActionDateForCurrentUser();
            $this->translator = Translator::init();
        } else {
            if ($this->getRequest()->isPost()) {
                echo ApiResponse::respond(SESSION_EXPIRED_CODE);
                exit();
            } else {
                $this->_helper->redirector('index', 'Index');
            }
        }
    }

    public function indexAction()
    {
        AccessRightChecker::checkAccessRight($this, 42);
        $productCategoryDao = new ProductCategoryDAO();
        $productCategories = $productCategoryDao->getParentCategories();
        $this->view->productCategories = $productCategories;
        $this->view->translator = $this->translator;
    }

    public function datatableAction()
    {
        $name = $_POST["name"] != "" ? $_POST["name"] : null;
        $parent = (int)$_POST["parent"] != 0 ? (int)$_POST["parent"] : null;
        $whereName = "";
        $whereParent = "";
        if ($name != null) {
            $whereName .= " AND c.nom LIKE '%{$name}%'";
        }
        if ( $parent != null){
            $whereParent = " AND c.id_parent = {$parent}";
        }
        $where = "c.statut = 1{$whereName}{$whereParent}";
        $productCategoriesTable = "
                (
                    SELECT c.nom AS product_category_name , c.id_categorie AS product_category_id,
                    c2.nom AS parent_category_name, c2.id_categorie AS parent_category_id
                    FROM categorie_produit c LEFT JOIN categorie_produit c2 on c.id_parent=c2.id_categorie
                    WHERE {$where}
                ) AS productCategoriesTable
        ";
        $primaryKey = 'product_category_id';
        $columns = [
            ['db' => 'product_category_name', 'dt' => 0],
            ['db' => 'parent_category_name', 'dt' => 1],
            ['db' => 'product_category_id', 'dt' => 2],
        ];
        $computeDatatableResults = SSP::customSimple($_POST, $productCategoriesTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        foreach ($datatableResult["data"] as $row) {
            $productCategoryName = $row[0];
            $parentCategoryName = $row[1];
            $productCategoryId = $row[2];
            $actions = "
                <button title='{$this->translator->_("shared.edit")}' about='{$productCategoryId}' class='mini ui icon button app-secondary-button app-edit-action'>
                    <i class='edit icon'></i>
                </button>
                <button title='{$this->translator->_("shared.delete")}' about='{$productCategoryId}' class='mini ui red icon button app-delete-action'>
                    <i class='trash icon'></i>
                </button>
            ";
            $tableRow = [
                $productCategoryName,
                $parentCategoryName,
                $actions,
            ];
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    /**
     * 100 success
     * 101 duplicate
     * 102 unknown error
     */
    public function createAction()
    {
        try {
            $name = $_POST["name"];
            $parent = (int)$_POST["parent"];
            $productCategoryDao = new ProductCategoryDAO();
            if ($productCategoryDao->productCategoryNameAlreadyExists($name)) {
                echo ApiResponse::respond(101);
                exit();
            }
            $productCategory = $productCategoryDao->create();
            $productCategory->setNom($name);
            $productCategory->setStatut(1);
            if ($parent != 0) {
                $parentCategory = $productCategoryDao->getProductCategoryById($parent);
                $productCategory->setIdParent($parentCategory);
            }
            $productCategoryDao->save($productCategory);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("productCategory.index.productCategorySuccessfullyRegistered")} : N°{$productCategory->getIdCategorie()}({$productCategory->getNom()})"
            );
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["id" => $productCategory->getIdCategorie()]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function detailAction()
    {
        try {
            $id = (int)$_POST["id"];
            $productCategoryDao = new ProductCategoryDAO();
            $productCategory = $productCategoryDao->getProductCategoryById($id);
            echo ApiResponse::respond(
                REQUEST_SUCCESS_CODE,
                null,
                [
                    "name" => $productCategory->getNom(),
                    "parent" => $productCategory->getIdParent() ? $productCategory->getIdParent()->getIdCategorie() : 0,
                ]
            );
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 duplicate
     * 102 unknown error
     */
    public function updateAction()
    {
        try {
            $id = (int)$_POST["id"];
            $name = $_POST["name"];
            $parent = (int)$_POST["parent"];
            $productCategoryDao = new ProductCategoryDAO();
            $productCategory = $productCategoryDao->getProductCategoryById($id);
            if ($productCategoryDao->productCategoryNameAlreadyExists($name, $productCategory)) {
                echo ApiResponse::respond(101);
                exit();
            }
            $productCategory->setNom($name);
            if ($parent != 0) {
                $parentCategory = $productCategoryDao->getProductCategoryById($parent);
                $productCategory->setIdParent($parentCategory);
            } else {
                $productCategory->setIdParent(null);
            }
            $productCategoryDao->update($productCategory);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("productCategory.index.productCategoryUpdatedSuccessfully")} : N°{$productCategory->getIdCategorie()}({$productCategory->getNom()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 impossible deletion
     * 102 unknown error
     */
    public function deleteAction()
    {
        try {
            $id = (int)$_POST["id"];
            $productCategoryDao = new ProductCategoryDAO();
            $productDao = new ProductDAO();
            $productCategory = $productCategoryDao->getProductCategoryById($id);
            if ($productCategory->getStatut() == 0) {
                echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE);
                exit();
            }
            if (count($productCategoryDao->getSubCategoriesOfProductCategory($productCategory)) > 0 || count($productDao->getProductsByCategory($productCategory)) > 0) {
                echo ApiResponse::respond(101);
                exit();
            }
            $productCategory->setStatut(0);
            $productCategoryDao->update($productCategory);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("productCategory.index.productCategoryDeletedSuccessfully")} : N°{$productCategory->getIdCategorie()}({$productCategory->getNom()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function subcategoriesAction()
    {
        try{
            $id = (int)$_POST["id"];
            $productCategoryDao = new ProductCategoryDAO();
            $productCategory = $productCategoryDao->getProductCategoryById($id);
            $result = [];
            $subCategories = $productCategoryDao->getSubCategoriesOfProductCategory($productCategory);
            foreach($subCategories as $subCategory){
                $result[] = [
                  "id" => $subCategory->getIdCategorie(),
                  "name" => $subCategory->getNom(),
                ];
            }
            echo ApiResponse::respond(100, null, $result);
        } catch (Exception $ex){
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }
}