<?php

include APPLICATION_PATH . '/includes/daos.php';

class ProductnatureController extends Zend_Controller_Action
{
    private $translator;

    public function init()
    {
        session_start();
        SubscriptionChecker::checkSubscriptionValidity($this);
        if (isset($_SESSION[SESSION_USER_ID]) && $_SESSION[SESSION_USER_ID] != null) {
            $_SESSION[LAST_ACTION_DATE] = new DateTime();
            LogDAO::updateLastActionDateForCurrentUser();
            $this->translator = Translator::init();
        } else {
            if ($this->getRequest()->isPost()) {
                echo ApiResponse::respond(SESSION_EXPIRED_CODE);
                exit();
            } else {
                $this->_helper->redirector('index', 'Index');
            }
        }
    }

    public function indexAction()
    {
        AccessRightChecker::checkAccessRight($this, 107);
        $this->view->translator = $this->translator;
    }

    public function datatableAction()
    {
        $name = $_POST["name"] != "" ? $_POST["name"] : null;
        $whereName = "";
        if ($name != null) {
            $whereName .= " AND n.nom_nature LIKE '%{$name}%'";
        }
        $where = "n.statut = 1{$whereName}";
        $productNaturesTable = "
                (
                    SELECT n.nom_nature AS product_nature_name , n.id_nature AS product_nature_id
                    FROM nature_produit n
                    WHERE {$where}
                ) AS productNaturesTable
        ";
        $primaryKey = 'product_nature_id';
        $columns = [
            ['db' => 'product_nature_name', 'dt' => 0],
            ['db' => 'product_nature_id', 'dt' => 1],
        ];
        $computeDatatableResults = SSP::customSimple($_POST, $productNaturesTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        foreach ($datatableResult["data"] as $row) {
            $productNatureName = $row[0];
            $productNatureId = $row[1];
            $actions = "
                <button title='{$this->translator->_("shared.edit")}' about='{$productNatureId}' class='mini ui icon button app-secondary-button app-edit-action'>
                    <i class='edit icon'></i>
                </button>
                <button title='{$this->translator->_("shared.delete")}' about='{$productNatureId}' class='mini ui red icon button app-delete-action'>
                    <i class='trash icon'></i>
                </button>
            ";
            $tableRow = [
                $productNatureName,
                $actions,
            ];
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    /**
     * 100 success
     * 101 duplicate
     * 102 unknown error
     */
    public function createAction()
    {
        try {
            $name = $_POST["name"];
            $productNatureDao = new ProductNatureDAO();
            if ($productNatureDao->productNatureNameAlreadyExists($name)) {
                echo ApiResponse::respond(101);
                exit();
            }
            $productNature = $productNatureDao->create();
            $productNature->setNomNature($name);
            $productNature->setStatut(1);
            $productNatureDao->save($productNature);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("productNature.index.productNatureSuccessfullyRegistered")} : N°{$productNature->getIdNature()}({$productNature->getNomNature()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 duplicate
     * 102 unknown error
     */
    public function updateAction()
    {
        try {
            $id = (int)$_POST["id"];
            $name = $_POST["name"];
            $productNatureDao = new ProductNatureDAO();
            $productNature = $productNatureDao->getProductNatureById($id);
            if ($productNatureDao->productNatureNameAlreadyExists($name, $productNature)) {
                echo ApiResponse::respond(101);
                exit();
            }
            $productNature->setNomNature($name);
            $productNatureDao->update($productNature);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("productNature.index.productNatureUpdatedSuccessfully")} : N°{$productNature->getIdNature()}({$productNature->getNomNature()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 impossible deletion
     * 102 unknown error
     */
    public function deleteAction()
    {
        try {
            $id = (int)$_POST["id"];
            $productNatureDao = new ProductNatureDAO();
            $productDao = new ProductDAO();
            $productNature = $productNatureDao->getProductNatureById($id);
            if($productNature->getStatut() == 0){
                echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE);
                exit();
            }
            if (count($productDao->getProductsByNature($productNature)) > 0) {
                echo ApiResponse::respond(101);
                exit();
            }
            $productNature->setStatut(0);
            $productNatureDao->update($productNature);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("productNature.index.productNatureDeletedSuccessfully")} : N°{$productNature->getIdNature()}({$productNature->getNomNature()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }
}