<?php

include APPLICATION_PATH . '/includes/daos.php';

class ProfileController extends Zend_Controller_Action
{
    private $translator;

    public function init()
    {
        session_start();
        SubscriptionChecker::checkSubscriptionValidity($this);
        if (isset($_SESSION[SESSION_USER_ID]) && $_SESSION[SESSION_USER_ID] != null) {
            $_SESSION[LAST_ACTION_DATE] = new DateTime();
            LogDAO::updateLastActionDateForCurrentUser();
            $this->translator = Translator::init();
        } else {
            if ($this->getRequest()->isPost()) {
                echo ApiResponse::respond(SESSION_EXPIRED_CODE);
                exit();
            } else {
                $this->_helper->redirector('index', 'Index');
            }
        }
    }

    public function indexAction()
    {
        AccessRightChecker::checkAccessRight($this, 37);
        $this->view->translator = $this->translator;
    }

    public function datatableAction()
    {
        $name = $_POST["name"] != "" ? $_POST["name"] : null;
        $whereName = "";
        if ($name != null) {
            $whereName .= " AND p.nom LIKE '%{$name}%'";
        }
        $where = "p.statut = 1{$whereName}";
        $profilesTable = "
                (
                    SELECT p.id_profil AS profile_id, p.nom AS profile_name, p.code AS profile_code
                    FROM profil p
                    WHERE {$where}
                ) AS profilesTable
        ";
        $primaryKey = 'profile_id';
        $columns = [
            ['db' => 'profile_name', 'dt' => 0],
            ['db' => 'profile_id', 'dt' => 1],
            ['db' => 'profile_code', 'dt' => 2],
        ];
        $computeDatatableResults = SSP::customSimple($_POST, $profilesTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        foreach ($datatableResult["data"] as $row) {
            $profileName = $row[0];
            $profileId = $row[1];
            $profileCode = $row[2];
            $actions = "
                <button title='{$this->translator->_("shared.consult")}' about='{$profileId}' class='mini ui icon button app-primary-button app-consult-action'>
                    <i class='eye icon'></i>
                </button>
                <button title='{$this->translator->_("shared.edit")}' about='{$profileId}' class='mini ui icon button app-secondary-button app-edit-action'>
                    <i class='edit icon'></i>
                </button>
            ";
            if (!in_array($profileCode, IDENTIFIED_PROFILES_CODES)) {
                $actions .= "
                    <button title='{$this->translator->_("shared.delete")}' about='{$profileId}' class='mini ui red icon button app-delete-action'>
                        <i class='trash icon'></i>
                    </button>
                ";
            }
            $tableRow = [
                !in_array($profileCode, IDENTIFIED_PROFILES_CODES) ? $profileName : "
                    <div class='ui label app-primary-label'>
                        {$profileName}
                    </div>
                ",
                $actions,
            ];
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    /**
     * 100 success
     * 101 duplicate
     * 102 unknown error
     */
    public function createAction()
    {
        $profileDao = new ProfileDAO();
        $databaseConnection = $profileDao->entityManager->getConnection();
        $databaseConnection->beginTransaction();
        try {
            $profileName = $_POST["profileName"];
            $selections = json_decode($_POST["selections"]);
            if ($profileDao->profileNameAlreadyExists($profileName)) {
                echo ApiResponse::respond(101);
                exit();
            }
            $profile = $profileDao->create();
            $profile->setNom($profileName);
            $profile->setStatut(1);
            $profileDao->save($profile);
            $profileLineDao = new ProfileLineDAO();
            $menuDao = new MenuDAO();
            foreach ($selections as $selection) {
                $menu = $menuDao->getMenuById((int)$selection);
                $profileLine = $profileLineDao->create();
                $profileLine->setIdProfil($profile);
                $profileLine->setIdMenu($menu);
                $profileLine->setStatut(1);
                $profileLineDao->save($profileLine);
            }
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("profile.index.profileSuccessfullyRegistered")} : N°{$profile->getIdProfil()}({$profile->getNom()})"
            );
            $databaseConnection->commit();
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            $databaseConnection->rollBack();
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function detailAction()
    {
        try {
            $id = (int)$_POST["id"];
            $profileDao = new ProfileDAO();
            $profileLineDao = new ProfileLineDAO();
            $profile = $profileDao->getProfileById($id);
            $profileLines = $profileLineDao->getProfileLinesByProfile($profile);
            $result = [
                "profileName" => $profile->getNom(),
                "selections" => [],
            ];
            foreach ($profileLines as $profileLine) {
                $result["selections"][] = $profileLine->getIdMenu()->getIdMenu();
            }
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 duplicate
     * 102 unknown error
     */
    public function updateAction()
    {
        $profileDao = new ProfileDAO();
        $databaseConnection = $profileDao->entityManager->getConnection();
        $databaseConnection->beginTransaction();
        try {
            $id = (int)$_POST["id"];
            $profileName = $_POST["profileName"];
            $selections = json_decode($_POST["selections"]);
            $profile = $profileDao->getProfileById($id);
            if ($profileDao->profileNameAlreadyExists($profileName, $profile)) {
                echo ApiResponse::respond(101);
                exit();
            };
            $profile->setNom($profileName);
            $profileDao->update($profile);
            $profileLineDao = new ProfileLineDAO();
            $currentProfileLines = $profileLineDao->getProfileLinesByProfile($profile);
            foreach ($currentProfileLines as $currentProfileLine) {
                $profileLineDao->delete($currentProfileLine);
            }
            $menuDao = new MenuDAO();
            foreach ($selections as $selection) {
                $menu = $menuDao->getMenuById((int)$selection);
                $profileLine = $profileLineDao->create();
                $profileLine->setIdProfil($profile);
                $profileLine->setIdMenu($menu);
                $profileLine->setStatut(1);
                $profileLineDao->save($profileLine);
            }
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("profile.index.profileUpdatedSuccessfully")} : N°{$profile->getIdProfil()}({$profile->getNom()})"
            );
            $databaseConnection->commit();
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            $databaseConnection->rollBack();
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 cannot delete
     * 102 unknown error
     */
    public function deleteAction()
    {
        try {
            $id = (int)$_POST["id"];
            $profileDao = new ProfileDAO();
            $profile = $profileDao->getProfileById($id);
            if ($profile->getStatut() == 0) {
                echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE);
                exit();
            }
            if ($profileDao->someUsersHaveThisProfile($profile)) {
                echo ApiResponse::respond(101);
                exit();
            }
            $profile->setStatut(0);
            $profileDao->update($profile);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("profile.index.profileDeletedSuccessfully")} : N°{$profile->getIdProfil()}({$profile->getNom()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }
}
