<?php

include APPLICATION_PATH . '/includes/daos.php';

class PromotionController extends Zend_Controller_Action
{
    private $translator;

    public function init()
    {
        session_start();
        SubscriptionChecker::checkSubscriptionValidity($this);
        if (isset($_SESSION[SESSION_USER_ID]) && $_SESSION[SESSION_USER_ID] != null) {
            $_SESSION[LAST_ACTION_DATE] = new DateTime();
            LogDAO::updateLastActionDateForCurrentUser();
            $this->translator = Translator::init();
            $this->days = [
                "0" => $this->translator->_("shared.days.monday"),
                "1" => $this->translator->_("shared.days.tuesday"),
                "2" => $this->translator->_("shared.days.wednesday"),
                "3" => $this->translator->_("shared.days.thursday"),
                "4" => $this->translator->_("shared.days.friday"),
                "5" => $this->translator->_("shared.days.saturday"),
                "6" => $this->translator->_("shared.days.sunday"),
            ];
        } else {
            if ($this->getRequest()->isPost()) {
                echo ApiResponse::respond(SESSION_EXPIRED_CODE);
                exit();
            } else {
                $this->_helper->redirector('index', 'Index');
            }
        }
    }

    public function indexAction()
    {
        AccessRightChecker::checkAccessRight($this, 143);
        $pointOfSaleDao = new PointOfSaleDAO();
        $productCategoryDao = new ProductCategoryDAO();
        $currentUser = LogDAO::getCurrentUser();
        $pointsOfSale = [];
        if ($currentUser->getIdPointVente() == null || in_array($currentUser->getIdProfil()->getCode(), IDENTIFIED_PROFILES_CODES)) {
            $pointsOfSale = $pointOfSaleDao->listPointOfSales();
        } else {
            $pointsOfSale[] = $currentUser->getIdPointVente();
        }
        $this->view->pointsOfSale = $pointsOfSale;
        $this->view->productCategories = $productCategoryDao->getAllProductCategories();
        $this->view->translator = $this->translator;
    }

    public function datatableAction()
    {
        try {
            $currentDate = new DateTime();
            $currentTime = strtotime($currentDate->format(DATE_FORMAT_WITH_SECONDS)) . "promotions";
            $pointsOfSale = json_decode($_POST["pointsOfSale"], true);
            $startDate = $_POST["startDate"] != "" ? $_POST["startDate"] : null;
            $endDate = $_POST["endDate"] != "" ? $_POST["endDate"] : null;
            $wherePointsOfSale = "";
            $whereStartDate = "";
            $whereEndDate = "";
            if (count($pointsOfSale) > 0) {
                //We need to create a stored procedure here to explode the concerned points of sale string
                $promotionDao = new PromotionDAO();
                $databaseConnection = $promotionDao->entityManager->getConnection();
                $explodeProcedureSql = "
                    CREATE PROCEDURE explodeString_{$currentTime}(
                        IN inputString TEXT,
                        IN delimiterChar CHAR(1),
                        IN pointOfSale VARCHAR(7),
                        OUT isSelected TINYINT
                    )
                    BEGIN
                        DROP TEMPORARY TABLE IF EXISTS temp_string_{$currentTime};
                        CREATE TEMPORARY TABLE temp_string_{$currentTime}(vals text); 
                        WHILE LOCATE(delimiterChar,inputString) > 1 DO
                            INSERT INTO temp_string_{$currentTime} 
                            SELECT SUBSTRING_INDEX(inputString,delimiterChar,1);
                            SET inputString = SUBSTRING(inputString, INSTR(inputString, delimiterChar) + 1);
                        END WHILE;
                        INSERT INTO temp_string_{$currentTime}(vals) VALUES(inputString);
                        SELECT COUNT(*) INTO isSelected FROM temp_string_{$currentTime} WHERE vals = pointOfSale;
                    END;
                ";
                $explodeProcedureRequest = $databaseConnection->prepare($explodeProcedureSql);
                $explodeProcedureRequest->execute();
                $checkPointOfSaleSelectionFunctionSql = "
                    CREATE FUNCTION checkPointOfSaleIsSelected_{$currentTime}(
                        pointsOfSale TEXT,
                        pointOfSale VARCHAR(7),
                        delimiterChar CHAR(1)
                    )
                    RETURNS TINYINT
                    DETERMINISTIC
                    BEGIN
                        DECLARE pointOfSaleIsSelected TINYINT;
                        CALL explodeString_{$currentTime}(pointsOfSale, delimiterChar, pointOfSale, pointOfSaleIsSelected);
                        RETURN (pointOfSaleIsSelected);
                    END;
                ";
                $checkPointOfSaleSelectionFunctionRequest = $databaseConnection->prepare($checkPointOfSaleSelectionFunctionSql);
                $checkPointOfSaleSelectionFunctionRequest->execute();
                $countPointOfSale = 0;
                $wherePointsOfSale .= "  AND";
                if (count($pointsOfSale) > 1) {
                    $wherePointsOfSale .= " (";
                }
                foreach ($pointsOfSale as $pointOfSale) {
                    if (count($pointsOfSale) == 1) {
                        $wherePointsOfSale .= " p.liste_points_vente_cibles = '' OR checkPointOfSaleIsSelected_{$currentTime}(REPLACE(p.liste_points_vente_cibles, ' ', '>'), {$pointOfSale}, '>') = 1";
                    } else {
                        if ($countPointOfSale == 0) {
                            $wherePointsOfSale .= " ( p.liste_points_vente_cibles = '' OR checkPointOfSaleIsSelected_{$currentTime}(REPLACE(p.liste_points_vente_cibles, ' ', '>'), {$pointOfSale}, '>') = 1 )";
                        } else {
                            $wherePointsOfSale .= " AND ( p.liste_points_vente_cibles = '' OR checkPointOfSaleIsSelected_{$currentTime}(REPLACE(p.liste_points_vente_cibles, ' ', '>'), {$pointOfSale}, '>') = 1 )";
                        }
                        $countPointOfSale++;
                    }
                }
                if (count($pointsOfSale) > 1) {
                    $wherePointsOfSale .= " )";
                }
            }
            if ($startDate != null) {
                $whereStartDate .= " AND p.date_debut >= '{$startDate}'";
            }
            if ($endDate != null) {
                $whereEndDate .= " AND p.date_fin <= '{$endDate}'";
            }
            $where = "p.statut = 1{$wherePointsOfSale}{$whereStartDate}{$whereEndDate}";
            $promotionsTable = "
                (
                    SELECT p.id_promotion AS promotion_id, p.liste_points_vente_cibles AS promotion_points_of_sale, p.date_debut AS promotion_start_date,
                    p.date_fin AS promotion_end_date, p.date_creation AS promotion_entry_date, p.valide AS promotion_validated
                    FROM promotion p
                    WHERE {$where}
                ) AS promotionsTable
            ";
            $primaryKey = 'promotion_id';
            $columns = [
                ['db' => 'promotion_points_of_sale', 'dt' => 0],
                ['db' => 'promotion_start_date', 'dt' => 1],
                ['db' => 'promotion_end_date', 'dt' => 2],
                ['db' => 'promotion_entry_date', 'dt' => 3],
                ['db' => 'promotion_validated', 'dt' => 4],
                ['db' => 'promotion_id', 'dt' => 5],
            ];
            $computeDatatableResults = SSP::customSimple($_POST, $promotionsTable, $primaryKey, $columns);
            $datatableResult = $computeDatatableResults["datatable"];
            $finalResult = $computeDatatableResults["custom"];
            $pointOfSaleDao = new PointOfSaleDAO();
            foreach ($datatableResult["data"] as $row) {
                $promotionPointsOfSale = explode(" ", $row[0]);
                $promotionPointsOfSaleString = "";
                foreach ($promotionPointsOfSale as $promotionPointOfSale) {
                    if ($promotionPointOfSale == "") {
                        $promotionPointsOfSaleString .= "
                            <a class='ui label app-primary-label'>{$this->translator->_("promotion.index.allPointsOfSale")}</a>
                        ";
                        break;
                    }
                    $pointOfSale = $pointOfSaleDao->getPointOfSaleById((int)$promotionPointOfSale);
                    $promotionPointsOfSaleString .= "
                        <a style='margin-bottom: 5px;' class='ui label app-secondary-label'>{$pointOfSale->getNomPointVente()}</a>
                    ";
                }
                $promotionStartDate = new DateTime($row[1]);
                $promotionStartDateMoment = new \Moment\Moment($promotionStartDate->format(DATE_FORMAT_WITH_TIME));
                $promotionEndDate = new DateTime($row[2]);
                $promotionEndDateMoment = new \Moment\Moment($promotionEndDate->format(DATE_FORMAT_WITH_TIME));
                $promotionEntryDate = new DateTime($row[3]);
                $promotionEntryDateMoment = new \Moment\Moment($promotionEntryDate->format(DATE_FORMAT_WITH_TIME));
                $promotionValidated = (int)$row[4];
                $promotionId = (int)$row[5];
                $actions = "
                    <button title='{$this->translator->_("shared.consult")}' about='{$promotionId}' class='mini ui icon button app-primary-button app-consult-action'>
                        <i class='eye icon'></i>
                    </button>
                ";
                if ($promotionValidated == 0) {
                    $actions .= "
                        <button title='{$this->translator->_("shared.edit")}' about='{$promotionId}' class='mini ui icon button app-secondary-button app-edit-action'>
                            <i class='edit icon'></i>
                        </button>
                        <button title='{$this->translator->_("shared.delete")}' about='{$promotionId}' class='mini ui red icon button app-delete-action'>
                            <i class='trash icon'></i>
                        </button>
                    ";
                }
                $tableRow = [
                    $promotionPointsOfSaleString,
                    $promotionStartDateMoment->format(DATE_FORMAT_MIDDLE_WITH_TIME),
                    $promotionEndDateMoment->format(DATE_FORMAT_MIDDLE_WITH_TIME),
                    $promotionEntryDateMoment->format(DATE_FORMAT_MIDDLE_WITH_TIME),
                    $actions,
                ];
                $finalResult["data"][] = $tableRow;
            }
            if (count($pointsOfSale) > 0) {
                $deleteTemporaryFunctionProcedureTableSql = "
                    DROP PROCEDURE IF EXISTS explodeString_{$currentTime};
                    DROP TEMPORARY TABLE IF EXISTS temp_string_{$currentTime};
                    DROP FUNCTION IF EXISTS checkPointOfSaleIsSelected_{$currentTime};
                ";
                $deleteTemporaryFunctionProcedureTableRequest = $databaseConnection->prepare($deleteTemporaryFunctionProcedureTableSql);
                $deleteTemporaryFunctionProcedureTableRequest->execute();
            }
            echo json_encode($finalResult);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    public function promotionproductsdatatableAction()
    {
        $name = $_POST["name"] != "" ? $_POST["name"] : null;
        $category = (int)$_POST["category"] != 0 ? (int)$_POST["category"] : null;
        $whereName = "";
        $whereCategory = "";
        if ($name != null) {
            $whereName .= " AND p.designation LIKE '%{$name}%'";
        }
        if ($category != null) {
            $whereCategory .= " AND p.id_categorie_produit = {$category}";
        }
        $where = "p.statut = 1 AND p.stockable = 1{$whereName}{$whereCategory} order by p.designation ASC";
        $promotionProductsTable = "
                (
                    SELECT p.id_produit AS product_id,
                    p.designation AS product_designation, p.prix_vente AS product_sale_price, 
                    p.prix_vente2 AS product_sale_price_2, p.prix_vente3 AS product_sale_price_3, 
                    p.prix_achat AS product_buying_price, p.reference_produit AS product_reference,
                    p.image AS product_image
                    FROM produit p
                    WHERE {$where}
                ) AS promotionProductsTable
        ";
        $primaryKey = 'product_id';
        $columns = [
            ['db' => 'product_designation', 'dt' => 0],
            ['db' => 'product_sale_price', 'dt' => 1],
            ['db' => 'product_image', 'dt' => 2],
            ['db' => 'product_id', 'dt' => 3],
        ];
        $computeDatatableResults = SSP::customSimple($_POST, $promotionProductsTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        foreach ($datatableResult["data"] as $row) {
            $productDesignation = $row[0];
            $productSalePrice = $row[1];
            $productImage = $row[2] != null ? $row[2] : "";
            $productId = $row[3];
            $tableRow = [
                [
                    $productDesignation,
                    $productSalePrice,
                    $productImage,
                    $productId,
                ],
            ];
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    /**
     * 100 success
     * 101 overlapping promotion conflict
     * 102 unknown error
     */
    public function createAction()
    {
        $promotionDao = new PromotionDAO();
        $promotionLineDao = new PromotionLineDAO();
        $productDao = new ProductDAO();
        $databaseConnection = $promotionDao->entityManager->getConnection();
        $databaseConnection->beginTransaction();
        try {
            $pointsOfSaleIds = json_decode($_POST["pointsOfSale"], true);
            $startDate = new DateTime($_POST["startDate"]);
            $endDate = new DateTime($_POST["endDate"]);
            $commentOnPromotion = $_POST["commentOnPromotion"];
            $content = json_decode($_POST["content"], true);
            $productIds = [];
            foreach ($content as $detail) {
                $productIds[] = (int)$detail["id"];
            }
            $promotionsOverSamePeriod = $promotionDao->getOverlappingPromotions($startDate, $endDate);
            $samePointOfSale = null;
            $sameProduct = null;
            $conflictPromotion = null;
            foreach ($promotionsOverSamePeriod as $promotionOverSamePeriod) {
                $periodPointsOfSale = explode(" ", $promotionOverSamePeriod->getListePointsVenteCibles());
                $sameSubPointOfSale = null;
                $sameSubProduct = null;
                if (count($pointsOfSaleIds) == 0) {
                    $sameSubPointOfSale = 0;
                } else {
                    foreach ($periodPointsOfSale as $pointOfSale) {
                        if ($pointOfSale == "") {
                            $sameSubPointOfSale = 0;
                            break;
                        }
                        if (in_array($pointOfSale, $pointsOfSaleIds)) {
                            $sameSubPointOfSale = $pointOfSale;
                            break;
                        }
                    }
                }
                if (!is_null($sameSubPointOfSale)) {
                    $periodPromotionLines = $promotionLineDao->getPromotionLinesOfPromotion($promotionOverSamePeriod);
                    foreach ($periodPromotionLines as $periodPromotionLine) {
                        if (in_array($periodPromotionLine->getIdProduit()->getIdProduit(), $productIds)) {
                            $sameSubProduct = $periodPromotionLine->getIdProduit();
                            break;
                        }
                    }
                }
                if (!is_null($sameSubPointOfSale) && !is_null($sameSubProduct)) {
                    $samePointOfSale = $sameSubPointOfSale;
                    $sameProduct = $sameSubProduct;
                    $conflictPromotion = $promotionOverSamePeriod;
                    break;
                }
            }
            if (!is_null($samePointOfSale) && !is_null($sameProduct) && !is_null($conflictPromotion)) {
                echo ApiResponse::respond(101, null, [
                    "startDate" => $conflictPromotion->getDateDebut()->format(DATE_FORMAT_WITH_TIME),
                    "endDate" => $conflictPromotion->getDateFin()->format(DATE_FORMAT_WITH_TIME),
                    "product" => $sameProduct->getDesignation(),
                ]);
                exit();
            }
            $promotion = $promotionDao->create();
            $pointsOfSaleConcernedString = "";
            $countPointOfSale = 0;
            foreach ($pointsOfSaleIds as $pointOfSaleId) {
                if ($countPointOfSale == 0) {
                    $pointsOfSaleConcernedString .= "{$pointOfSaleId}";
                } else {
                    $pointsOfSaleConcernedString .= " {$pointOfSaleId}";
                }
                $countPointOfSale++;
            }
            $promotion->setListePointsVenteCibles($pointsOfSaleConcernedString);
            $promotion->setDateDebut($startDate);
            $promotion->setDateFin($endDate);
            $promotion->setDateCreation(new DateTime());
            $promotion->setIdUtilisateur(LogDAO::getCurrentUser());
            $promotion->setObjet($commentOnPromotion);
            $promotion->setValide(0);
            $promotion->setStatut(1);
            $promotionDao->save($promotion);
            foreach ($content as $detail) {
                $promotionLine = $promotionLineDao->create();
                $product = $productDao->getProductById((int)$detail["id"]);
                $promotionLine->setIdProduit($product);
                $promotionLine->setIdPromotion($promotion);
                $promotionLine->setPourcentage((float)$detail["percentage"]);
                $promotionLine->setStatut(1);
                $promotionLineDao->save($promotionLine);
            }
            $databaseConnection->commit();
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("promotion.index.promotionCreatedSuccessfully")} : N°{$promotion->getIdPromotion()}({$startDate->format(DATE_FORMAT_WITH_TIME)} - {$endDate->format(DATE_FORMAT_WITH_TIME)})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            $databaseConnection->rollBack();
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function detailAction()
    {
        try {
            $id = (int)$_POST["id"];
            $promotionDao = new PromotionDAO();
            $promotionLineDao = new PromotionLineDAO();
            $promotion = $promotionDao->getPromotionById($id);
            $result = [
                "pointsOfSale" => $promotion->getListePointsVenteCibles() == "" ? [] : explode(" ", $promotion->getListePointsVenteCibles()),
                "pointsOfSaleStrings" => [],
                "startDate" => $promotion->getDateDebut()->format(DATE_FORMAT_WITH_TIME),
                "endDate" => $promotion->getDateFin()->format(DATE_FORMAT_WITH_TIME),
                "commentOnPromotion" => $promotion->getObjet(),
                "content" => [],
            ];
            $promotionLines = $promotionLineDao->getPromotionLinesOfPromotion($promotion);
            foreach($promotionLines as $promotionLine) {
                $result["content"][] = [
                    "id" => $promotionLine->getIdProduit()->getIdProduit(),
                    "price" => $promotionLine->getIdProduit()->getPrixVente(),
                    "designation" => $promotionLine->getIdProduit()->getDesignation(),
                    "percentage" => $promotionLine->getPourcentage(),
                ];
            }
            if ($promotion->getListePointsVenteCibles() != "") {
                $pointOfSaleDao = new PointOfSaleDAO();
                $pointsOfSaleIds = explode(" ", $promotion->getListePointsVenteCibles());
                foreach($pointsOfSaleIds as $pointOfSaleId) {
                    $pointOfSale = $pointOfSaleDao->getPointOfSaleById((int)$pointOfSaleId);
                    $result["pointsOfSaleStrings"][] = $pointOfSale->getNomPointVente();
                }
            }
            echo ApiResponse::respond(100, null, $result);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 overlapping promotion conflict
     * 102 unknown error
     */
    public function updateAction()
    {
        $promotionDao = new PromotionDAO();
        $promotionLineDao = new PromotionLineDAO();
        $productDao = new ProductDAO();
        $databaseConnection = $promotionDao->entityManager->getConnection();
        $databaseConnection->beginTransaction();
        try {
            $promotionId = (int)$_POST["id"];
            $pointsOfSaleIds = json_decode($_POST["pointsOfSale"], true);
            $startDate = new DateTime($_POST["startDate"]);
            $endDate = new DateTime($_POST["endDate"]);
            $commentOnPromotion = $_POST["commentOnPromotion"];
            $content = json_decode($_POST["content"], true);
            $productIds = [];
            foreach ($content as $detail) {
                $productIds[] = (int)$detail["id"];
            }
            $promotionsOverSamePeriod = $promotionDao->getOverlappingPromotions($startDate, $endDate, $promotionId);
            $samePointOfSale = null;
            $sameProduct = null;
            $conflictPromotion = null;
            foreach ($promotionsOverSamePeriod as $promotionOverSamePeriod) {
                $periodPointsOfSale = explode(" ", $promotionOverSamePeriod->getListePointsVenteCibles());
                $sameSubPointOfSale = null;
                $sameSubProduct = null;
                if (count($pointsOfSaleIds) == 0) {
                    $sameSubPointOfSale = 0;
                } else {
                    foreach ($periodPointsOfSale as $pointOfSale) {
                        if ($pointOfSale == "") {
                            $sameSubPointOfSale = 0;
                            break;
                        }
                        if (in_array($pointOfSale, $pointsOfSaleIds)) {
                            $sameSubPointOfSale = $pointOfSale;
                            break;
                        }
                    }
                }
                if (!is_null($sameSubPointOfSale)) {
                    $periodPromotionLines = $promotionLineDao->getPromotionLinesOfPromotion($promotionOverSamePeriod);
                    foreach ($periodPromotionLines as $periodPromotionLine) {
                        if (in_array($periodPromotionLine->getIdProduit()->getIdProduit(), $productIds)) {
                            $sameSubProduct = $periodPromotionLine->getIdProduit();
                            break;
                        }
                    }
                }
                if (!is_null($sameSubPointOfSale) && !is_null($sameSubProduct)) {
                    $samePointOfSale = $sameSubPointOfSale;
                    $sameProduct = $sameSubProduct;
                    $conflictPromotion = $promotionOverSamePeriod;
                    break;
                }
            }
            if (!is_null($samePointOfSale) && !is_null($sameProduct) && !is_null($conflictPromotion)) {
                echo ApiResponse::respond(101, null, [
                    "startDate" => $conflictPromotion->getDateDebut()->format(DATE_FORMAT_WITH_TIME),
                    "endDate" => $conflictPromotion->getDateFin()->format(DATE_FORMAT_WITH_TIME),
                    "product" => $sameProduct->getDesignation(),
                ]);
                exit();
            }
            $promotion = $promotionDao->getPromotionById($promotionId);
            $pointsOfSaleConcernedString = "";
            $countPointOfSale = 0;
            foreach ($pointsOfSaleIds as $pointOfSaleId) {
                if ($countPointOfSale == 0) {
                    $pointsOfSaleConcernedString .= "{$pointOfSaleId}";
                } else {
                    $pointsOfSaleConcernedString .= " {$pointOfSaleId}";
                }
                $countPointOfSale++;
            }
            $promotion->setListePointsVenteCibles($pointsOfSaleConcernedString);
            $promotion->setDateDebut($startDate);
            $promotion->setDateFin($endDate);
            $promotion->setObjet($commentOnPromotion);
            $promotionDao->update($promotion);
            $promotionLines = $promotionLineDao->getPromotionLinesOfPromotion($promotion);
            foreach($promotionLines as $promotionLine) {
                $promotionLineDao->delete($promotionLine);
            }
            foreach ($content as $detail) {
                $promotionLine = $promotionLineDao->create();
                $product = $productDao->getProductById((int)$detail["id"]);
                $promotionLine->setIdProduit($product);
                $promotionLine->setIdPromotion($promotion);
                $promotionLine->setPourcentage((float)$detail["percentage"]);
                $promotionLine->setStatut(1);
                $promotionLineDao->save($promotionLine);
            }
            $databaseConnection->commit();
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("promotion.index.promotionUpdatedSuccessfully")} : N°{$promotion->getIdPromotion()}({$startDate->format(DATE_FORMAT_WITH_TIME)} - {$endDate->format(DATE_FORMAT_WITH_TIME)})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            $databaseConnection->rollBack();
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function deleteAction()
    {
        try {
            $promotionDao = new PromotionDAO();
            $id = (int)$_POST["id"];
            $promotion = $promotionDao->getPromotionById($id);
            if ($promotion->getStatut() == 0) {
                echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE);
                exit();
            }
            if ($promotion->getValide() == 1) {
                echo ApiResponse::respond(ALREADY_VALIDATED_ISSUE_CODE);
                exit();
            }
            $promotion->setStatut(0);
            $promotionDao->update($promotion);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("promotion.index.promotionDeletedSuccessfully")} : N°{$promotion->getIdPromotion()}({$promotion->getDateDebut()->format(DATE_FORMAT_WITH_TIME)} - {$promotion->getDateFin()->format(DATE_FORMAT_WITH_TIME)})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    public function validationAction()
    {
        AccessRightChecker::checkAccessRight($this, 144);
        $pointOfSaleDao = new PointOfSaleDAO();
        $currentUser = LogDAO::getCurrentUser();
        $pointsOfSale = [];
        if ($currentUser->getIdPointVente() == null || in_array($currentUser->getIdProfil()->getCode(), IDENTIFIED_PROFILES_CODES)) {
            $pointsOfSale = $pointOfSaleDao->listPointOfSales();
        } else {
            $pointsOfSale[] = $currentUser->getIdPointVente();
        }
        $this->view->pointsOfSale = $pointsOfSale;
        $this->view->translator = $this->translator;
    }

    public function datatablevalidationAction()
    {
        try {
            $currentDate = new DateTime();
            $currentTime = strtotime($currentDate->format(DATE_FORMAT_WITH_SECONDS)) . "promotions";
            $pointsOfSale = json_decode($_POST["pointsOfSale"], true);
            $startDate = $_POST["startDate"] != "" ? $_POST["startDate"] : null;
            $endDate = $_POST["endDate"] != "" ? $_POST["endDate"] : null;
            $wherePointsOfSale = "";
            $whereStartDate = "";
            $whereEndDate = "";
            if (count($pointsOfSale) > 0) {
                //We need to create a stored procedure here to explode the concerned points of sale string
                $promotionDao = new PromotionDAO();
                $databaseConnection = $promotionDao->entityManager->getConnection();
                $explodeProcedureSql = "
                    CREATE PROCEDURE explodeString_{$currentTime}(
                        IN inputString TEXT,
                        IN delimiterChar CHAR(1),
                        IN pointOfSale VARCHAR(7),
                        OUT isSelected TINYINT
                    )
                    BEGIN
                        DROP TEMPORARY TABLE IF EXISTS temp_string_{$currentTime};
                        CREATE TEMPORARY TABLE temp_string_{$currentTime}(vals text); 
                        WHILE LOCATE(delimiterChar,inputString) > 1 DO
                            INSERT INTO temp_string_{$currentTime} 
                            SELECT SUBSTRING_INDEX(inputString,delimiterChar,1);
                            SET inputString = SUBSTRING(inputString, INSTR(inputString, delimiterChar) + 1);
                        END WHILE;
                        INSERT INTO temp_string_{$currentTime}(vals) VALUES(inputString);
                        SELECT COUNT(*) INTO isSelected FROM temp_string_{$currentTime} WHERE vals = pointOfSale;
                    END;
                ";
                $explodeProcedureRequest = $databaseConnection->prepare($explodeProcedureSql);
                $explodeProcedureRequest->execute();
                $checkPointOfSaleSelectionFunctionSql = "
                    CREATE FUNCTION checkPointOfSaleIsSelected_{$currentTime}(
                        pointsOfSale TEXT,
                        pointOfSale VARCHAR(7),
                        delimiterChar CHAR(1)
                    )
                    RETURNS TINYINT
                    DETERMINISTIC
                    BEGIN
                        DECLARE pointOfSaleIsSelected TINYINT;
                        CALL explodeString_{$currentTime}(pointsOfSale, delimiterChar, pointOfSale, pointOfSaleIsSelected);
                        RETURN (pointOfSaleIsSelected);
                    END;
                ";
                $checkPointOfSaleSelectionFunctionRequest = $databaseConnection->prepare($checkPointOfSaleSelectionFunctionSql);
                $checkPointOfSaleSelectionFunctionRequest->execute();
                $countPointOfSale = 0;
                $wherePointsOfSale .= "  AND";
                if (count($pointsOfSale) > 1) {
                    $wherePointsOfSale .= " (";
                }
                foreach ($pointsOfSale as $pointOfSale) {
                    if (count($pointsOfSale) == 1) {
                        $wherePointsOfSale .= " p.liste_points_vente_cibles = '' OR checkPointOfSaleIsSelected_{$currentTime}(REPLACE(p.liste_points_vente_cibles, ' ', '>'), {$pointOfSale}, '>') = 1";
                    } else {
                        if ($countPointOfSale == 0) {
                            $wherePointsOfSale .= " ( p.liste_points_vente_cibles = '' OR checkPointOfSaleIsSelected_{$currentTime}(REPLACE(p.liste_points_vente_cibles, ' ', '>'), {$pointOfSale}, '>') = 1 )";
                        } else {
                            $wherePointsOfSale .= " AND ( p.liste_points_vente_cibles = '' OR checkPointOfSaleIsSelected_{$currentTime}(REPLACE(p.liste_points_vente_cibles, ' ', '>'), {$pointOfSale}, '>') = 1 )";
                        }
                        $countPointOfSale++;
                    }
                }
                if (count($pointsOfSale) > 1) {
                    $wherePointsOfSale .= " )";
                }
            }
            if ($startDate != null) {
                $whereStartDate .= " AND p.date_debut >= '{$startDate}'";
            }
            if ($endDate != null) {
                $whereEndDate .= " AND p.date_fin <= '{$endDate}'";
            }
            $where = "p.statut = 1 AND p.valide = 0{$wherePointsOfSale}{$whereStartDate}{$whereEndDate}";
            $promotionsValidationTable = "
                (
                    SELECT p.id_promotion AS promotion_id, p.liste_points_vente_cibles AS promotion_points_of_sale, p.date_debut AS promotion_start_date,
                    p.date_fin AS promotion_end_date, p.date_creation AS promotion_entry_date
                    FROM promotion p
                    WHERE {$where}
                ) AS promotionsValidationTable
            ";
            $primaryKey = 'promotion_id';
            $columns = [
                ['db' => 'promotion_points_of_sale', 'dt' => 0],
                ['db' => 'promotion_start_date', 'dt' => 1],
                ['db' => 'promotion_end_date', 'dt' => 2],
                ['db' => 'promotion_entry_date', 'dt' => 3],
                ['db' => 'promotion_id', 'dt' => 4],
            ];
            $computeDatatableResults = SSP::customSimple($_POST, $promotionsValidationTable, $primaryKey, $columns);
            $datatableResult = $computeDatatableResults["datatable"];
            $finalResult = $computeDatatableResults["custom"];
            $pointOfSaleDao = new PointOfSaleDAO();
            foreach ($datatableResult["data"] as $row) {
                $promotionPointsOfSale = explode(" ", $row[0]);
                $promotionPointsOfSaleString = "";
                foreach ($promotionPointsOfSale as $promotionPointOfSale) {
                    if ($promotionPointOfSale == "") {
                        $promotionPointsOfSaleString .= "
                            <a class='ui label app-primary-label'>{$this->translator->_("promotion.index.allPointsOfSale")}</a>
                        ";
                        break;
                    }
                    $pointOfSale = $pointOfSaleDao->getPointOfSaleById((int)$promotionPointOfSale);
                    $promotionPointsOfSaleString .= "
                        <a style='margin-bottom: 5px;' class='ui label app-secondary-label'>{$pointOfSale->getNomPointVente()}</a>
                    ";
                }
                $promotionStartDate = new DateTime($row[1]);
                $promotionStartDateMoment = new \Moment\Moment($promotionStartDate->format(DATE_FORMAT_WITH_TIME));
                $promotionEndDate = new DateTime($row[2]);
                $promotionEndDateMoment = new \Moment\Moment($promotionEndDate->format(DATE_FORMAT_WITH_TIME));
                $promotionEntryDate = new DateTime($row[3]);
                $promotionEntryDateMoment = new \Moment\Moment($promotionEntryDate->format(DATE_FORMAT_WITH_TIME));
                $promotionId = (int)$row[4];
                $actions = "
                    <button title='{$this->translator->_("shared.consult")}' about='{$promotionId}' class='mini ui icon button app-primary-button app-consult-action'>
                        <i class='eye icon'></i>
                    </button>
                    <button title='{$this->translator->_("shared.validate")}' about='{$promotionId}' class='mini ui green icon button app-validate-action'>
                        <i class='check icon'></i>
                    </button>
                ";
                $tableRow = [
                    $promotionPointsOfSaleString,
                    $promotionStartDateMoment->format(DATE_FORMAT_MIDDLE_WITH_TIME),
                    $promotionEndDateMoment->format(DATE_FORMAT_MIDDLE_WITH_TIME),
                    $promotionEntryDateMoment->format(DATE_FORMAT_MIDDLE_WITH_TIME),
                    $actions,
                ];
                $finalResult["data"][] = $tableRow;
            }
            if (count($pointsOfSale) > 0) {
                $deleteTemporaryFunctionProcedureTableSql = "
                    DROP PROCEDURE IF EXISTS explodeString_{$currentTime};
                    DROP TEMPORARY TABLE IF EXISTS temp_string_{$currentTime};
                    DROP FUNCTION IF EXISTS checkPointOfSaleIsSelected_{$currentTime};
                ";
                $deleteTemporaryFunctionProcedureTableRequest = $databaseConnection->prepare($deleteTemporaryFunctionProcedureTableSql);
                $deleteTemporaryFunctionProcedureTableRequest->execute();
            }
            echo json_encode($finalResult);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function validateAction()
    {
        try {
            $id = (int)$_POST["id"];
            $promotionDao = new PromotionDao();
            $promotion = $promotionDao->getPromotionById($id);
            if ($promotion->getStatut() == 0) {
                echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE);
                exit();
            }
            if ($promotion->getValide() == 1) {
                echo ApiResponse::respond(ALREADY_VALIDATED_ISSUE_CODE);
                exit();
            }
            $promotion->setValide(1);
            $promotionDao->update($promotion);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("promotion.index.promotionValidatedSuccessfully")} : N°{$promotion->getIdPromotion()}({$promotion->getDateDebut()->format(DATE_FORMAT_WITH_TIME)} - {$promotion->getDateFin()->format(DATE_FORMAT_WITH_TIME)})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }
}
