<?php

include APPLICATION_PATH . '/includes/daos.php';

class ReceptionController extends Zend_Controller_Action
{
    private $translator;

    public function init()
    {
        session_start();
        SubscriptionChecker::checkSubscriptionValidity($this);
        if (isset($_SESSION[SESSION_USER_ID]) && $_SESSION[SESSION_USER_ID] != null) {
            $_SESSION[LAST_ACTION_DATE] = new DateTime();
            LogDAO::updateLastActionDateForCurrentUser();
            $this->translator = Translator::init();
        } else {
            if ($this->getRequest()->isPost()) {
                echo ApiResponse::respond(SESSION_EXPIRED_CODE);
                exit();
            } else {
                $this->_helper->redirector('index', 'Index');
            }
        }
    }

    public function indexAction()
    {
        AccessRightChecker::checkAccessRight($this, 13);
        $supplierDao = new SupplierDAO();
        $productCategoryDao = new ProductCategoryDAO();
        $currentUser = LogDAO::getCurrentUser();
        $storeDao = new StoreDAO();
        $stores = [];
        if ($currentUser->getIdPointVente() == null || in_array($currentUser->getIdProfil()->getCode(), IDENTIFIED_PROFILES_CODES)) {
            $stores = $storeDao->getAllStores();
        } else {
            $stores[] = $currentUser->getIdPointVente()->getIdMagasin();
        }
        $this->view->suppliers = $supplierDao->listSuppliers();
        $this->view->productCategories = $productCategoryDao->getAllProductCategories();
        $this->view->stores = $stores;
        $this->view->translator = $this->translator;
    }

    public function datatableAction()
    {
        $store = (int)$_POST["store"] != 0 ? (int)$_POST["store"] : null;
        $supplier = (int)$_POST["supplier"] != 0 ? (int)$_POST["supplier"] : null;
        $from = $_POST["from"] != "" ? $_POST["from"] : null;
        $to = $_POST["to"] != "" ? $_POST["to"] : null;
        $whereStore = "";
        $whereSupplier = "";
        $whereFrom = "";
        $whereTo = "";
        if ($store != null) {
            $whereStore .= " AND r.id_magasin = {$store}";
        }
        if ($supplier != null) {
            $whereSupplier .= " AND r.id_fournisseur = {$supplier}";
        }
        if ($from != null) {
            $whereFrom .= " AND r.date_reception >= '{$from} 00:00:00'";
        }
        if ($to != null) {
            $whereTo .= " AND r.date_reception <= '{$to} 23:59:59'";
        }
        $where = "r.statut = 1{$whereStore}{$whereSupplier}{$whereFrom}{$whereTo}";
        $receptionsTable = "
                (
                    SELECT r.id_reception AS reception_id, m.nom_magasin AS reception_store, f.nom_fournisseur AS reception_supplier,
                    r.date_saisie AS reception_entry_date, r.date_reception AS reception_receiving_date, r.num_bl AS reception_delivery_note_number, 
                    r.valider AS reception_validated
                    FROM reception r JOIN magasin m ON r.id_magasin = m.id_magasin JOIN fournisseur f ON r.id_fournisseur = f.id_fournisseur
                    WHERE {$where}
                ) AS receptionsTable
        ";
        $primaryKey = 'reception_id';
        $columns = [
            ['db' => 'reception_id', 'dt' => 0],
            ['db' => 'reception_store', 'dt' => 1],
            ['db' => 'reception_supplier', 'dt' => 2],
            ['db' => 'reception_entry_date', 'dt' => 3],
            ['db' => 'reception_receiving_date', 'dt' => 4],
            ['db' => 'reception_delivery_note_number', 'dt' => 5],
            ['db' => 'reception_validated', 'dt' => 6],
        ];
        $computeDatatableResults = SSP::customSimple($_POST, $receptionsTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        foreach ($datatableResult["data"] as $row) {
            $receptionId = (int)$row[0];
            $receptionStore = $row[1];
            $receptionSupplier = $row[2];
            $entryDate = new DateTime($row[3]);
            $receivingDate = new DateTime($row[4]);
            $deliveryNoteNumber = $row[5];
            $receptionValidated = (int)$row[6];
            $actions = "
                <button title='{$this->translator->_("shared.consult")}' about='{$receptionId}' class='mini ui icon button app-primary-button app-consult-action'>
                    <i class='eye icon'></i>
                </button>
            ";
            if ($receptionValidated == 0) {
                $actions .= "
                    <button title='{$this->translator->_("shared.edit")}' about='{$receptionId}' class='mini ui icon button app-secondary-button app-edit-action'>
                        <i class='edit icon'></i>
                    </button>
                    <button title='{$this->translator->_("shared.delete")}' about='{$receptionId}' class='mini ui red icon button app-delete-action'>
                        <i class='trash icon'></i>
                    </button>
                ";
            } else {
                $actions .= "
                    <button title='{$this->translator->_("shared.print")}' about='{$receptionId}' class='mini ui purple icon button app-print-action'>
                        <i class='print icon'></i>
                    </button>
                ";
            }
            $tableRow = [
                $receptionId,
                $receptionStore,
                $receptionSupplier,
                $entryDate->format(DATE_FORMAT),
                $receivingDate->format(DATE_FORMAT),
                $deliveryNoteNumber,
                $actions,
            ];
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    public function receptionproductsdatatableAction()
    {
        $name = $_POST["name"] != "" ? $_POST["name"] : null;
        $category = (int)$_POST["category"] != 0 ? (int)$_POST["category"] : null;
        $whereName = "";
        $whereCategory = "";
        if ($name != null) {
            $whereName .= " AND p.designation LIKE '%{$name}%'";
        }
        if ($category != null) {
            $whereCategory .= " AND p.id_categorie_produit = {$category}";
        }
        $where = "p.statut = 1 AND p.stockable = 1{$whereName}{$whereCategory} order by p.designation ASC";
        $receptionProductsTable = "
                (
                    SELECT p.id_produit AS product_id,
                    p.designation AS product_designation, p.prix_vente AS product_sale_price, 
                    p.prix_vente2 AS product_sale_price_2, p.prix_vente3 AS product_sale_price_3, 
                    p.prix_achat AS product_buying_price, p.reference_produit AS product_reference,
                    p.image AS product_image, p.pourcentage_max_remise AS product_max_reduction_percent,
                    p.perissable AS product_perishable
                    FROM produit p
                    WHERE {$where}
                ) AS receptionProductsTable
        ";
        $primaryKey = 'product_id';
        $columns = [
            ['db' => 'product_designation', 'dt' => 0],
            ['db' => 'product_buying_price', 'dt' => 1],
            ['db' => 'product_image', 'dt' => 2],
            ['db' => 'product_sale_price', 'dt' => 3],
            ['db' => 'product_max_reduction_percent', 'dt' => 4],
            ['db' => 'product_id', 'dt' => 5],
            ['db' => 'product_perishable', 'dt' => 6],
        ];
        $computeDatatableResults = SSP::customSimple($_POST, $receptionProductsTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        foreach ($datatableResult["data"] as $row) {
            $productDesignation = $row[0];
            $productBuyingPrice = $row[1];
            $productImage = $row[2] != null ? $row[2] : "";
            $productSalePrice = (float)$row[3];
            $productMaxReductionPercent = (float)$row[4];
            $productId = $row[5];
            $productPerishable = $row[6];
            $tableRow = [
                [
                    $productDesignation,
                    $productBuyingPrice,
                    $productImage,
                    $productId,
                    $productSalePrice,
                    $productMaxReductionPercent,
                    $productPerishable,
                ],
            ];
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    /**
     * 100 success
     * 101 duplicate delivery note number
     * 102 unknown error
     */
    public function createAction()
    {
        $receptionDao = new ReceptionDAO();
        $databaseConnection = $receptionDao->entityManager->getConnection();
        $databaseConnection->beginTransaction();
        try {
            $orderId = isset($_POST["order"]) ? (int)$_POST["order"] : null;
            $supplierId = (int)$_POST["supplier"];
            $storeId = (int)$_POST["store"];
            $deliveryNoteNumber = $_POST["deliveryNoteNumber"];
            if ($receptionDao->receptionDeliveryNoteNumberAlreadyExists($deliveryNoteNumber)) {
                echo ApiResponse::respond(101);
                exit();
            }
            $receivingDate = new DateTime($_POST["receivingDate"]);
            $commentOnReception = $_POST["commentOnReception"];
            $content = json_decode($_POST["content"], true);
            $reception = $receptionDao->create();
            if ($orderId != null) {
                $orderDao = new OrderDAO();
                $order = $orderDao->getOrderById($orderId);
                if ($order->getReceptionner() == 1) {
                    echo ApiResponse::respond();
                    exit();
                }
                $reception->setIdCommande($order);
                $order->setReceptionner(1);
                $orderDao->update($order);
                LogDAO::registerOperationForCurrentUser(
                    "{$this->translator->_("order.index.orderReceivedSuccessfully")} : N°{$order->getIdCommande()}"
                );
            }
            $supplierDao = new SupplierDAO();
            $supplier = $supplierDao->getSupplierById($supplierId);
            $reception->setIdFournisseur($supplier);
            $storeDao = new StoreDAO();
            $store = $storeDao->getStoreById($storeId);
            $reception->setIdMagasin($store);
            $reception->setNumBl($deliveryNoteNumber);
            $reception->setDateReception($receivingDate);
            $reception->setDateSaisie(new DateTime());
            $reception->setObservation($commentOnReception);
            $reception->setIdUtilisateur(LogDAO::getCurrentUser());
            $reception->setStatut(1);
            $reception->setValider(0);
            $receptionDao->save($reception);
            $receptionLineDao = new ReceptionLineDAO();
            $productDao = new ProductDAO();
            $count = 1;
            foreach ($content as $receptionDetail) {
                $receptionLine = $receptionLineDao->create();
                $receptionLine->setQuantite((int)$receptionDetail["quantity"]);
                $receptionLine->setCodeBarre($receptionDetail["batchNumber"]);
                $receptionLine->setPrixAchat((float)$receptionDetail["buyingPrice"]);
                $receptionLine->setPrixUnitaire((float)$receptionDetail["sellingPrice"]);
                $receptionLine->setNumLot("{$count}");
                $receptionLine->setDesignation($receptionDetail["designation"]);
                $receptionLine->setPourcentageMaxRemise((float)$receptionDetail["maximumReductionPercent"]);
                $receptionLine->setNumLotFournisseur($receptionDetail["batchNumber"]);
                $receptionLine->setNumLigne($count);
                $product = $productDao->getProductById((int)$receptionDetail["id"]);
                $receptionLine->setIdProduit($product);
                if (GESTION_DATE_PEREMPTION_PRODUIT == 1 && $receptionLine->getIdProduit()->getPerissable() == 1 && $receptionDetail["expirationDate"] != "") {
                    $receptionLine->setDatePeremption(new DateTime($receptionDetail["expirationDate"]));
                }
                $receptionLine->setIdReception($reception);
                $receptionLineDao->save($receptionLine);
                $receptionLine->setNumLot("{$receptionLine->getIdLigneReception()}{$count}");
                $receptionLineDao->update($receptionLine);
                $count++;
            }
            $databaseConnection->commit();
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("reception.index.receptionRegisteredSuccessfully")} : N°{$reception->getIdReception()}"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            $databaseConnection->rollBack();
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function detailAction()
    {
        try {
            $id = (int)$_POST["id"];
            $receptionDao = new ReceptionDAO();
            $reception = $receptionDao->getReceptionById($id);
            $result = [
                "supplier" => $reception->getIdFournisseur()->getIdFournisseur(),
                "supplierName" => $reception->getIdFournisseur()->getNomFournisseur(),
                "store" => $reception->getIdMagasin()->getIdMagasin(),
                "storeName" => $reception->getIdMagasin()->getNomMagasin(),
                "deliveryNoteNumber" => $reception->getNumBl(),
                "receivingDate" => $reception->getDateReception()->format(DATE_FORMAT),
                "entryDate" => $reception->getDateSaisie()->format(DATE_FORMAT),
                "commentOnReception" => $reception->getObservation(),
                "content" => [],
            ];
            if ($reception->getIdCommande() != null) {
                $result["orderId"] = $reception->getIdCommande()->getIdCommande();
            }
            $receptionLineDao = new ReceptionLineDAO();
            $receptionLines = $receptionLineDao->getReceptionLinesOfReception($reception);
            foreach ($receptionLines as $receptionLine) {
                $result["content"][] = [
                    "id" => $receptionLine->getIdProduit()->getIdProduit(),
                    "quantity" => $receptionLine->getQuantite(),
                    "designation" => $receptionLine->getDesignation(),
                    "buyingPrice" => $receptionLine->getPrixAchat(),
                    "sellingPrice" => $receptionLine->getPrixUnitaire(),
                    "maximumReductionPercent" => $receptionLine->getPourcentageMaxRemise(),
                    "batchNumber" => $receptionLine->getNumLotFournisseur(),
                    "expirationDate" => $receptionLine->getDatePeremption() != null ? $receptionLine->getDatePeremption()->format(DATE_FORMAT) : "",
                    "isPerishable" => $receptionLine->getIdProduit()->getPerissable(),
                ];
            }
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 duplicate delivery note number
     * 102 unknown error
     */
    public function updateAction()
    {
        $receptionDao = new ReceptionDAO();
        $databaseConnection = $receptionDao->entityManager->getConnection();
        $databaseConnection->beginTransaction();
        try {
            $id = (int)$_POST["id"];
            $reception = $receptionDao->getReceptionById($id);
            if ($reception->getValider() == 1) {
                echo ApiResponse::respond();
                exit();
            }
            $supplierId = (int)$_POST["supplier"];
            $storeId = (int)$_POST["store"];
            $deliveryNoteNumber = $_POST["deliveryNoteNumber"];
            if ($receptionDao->receptionDeliveryNoteNumberAlreadyExists($deliveryNoteNumber, $reception)) {
                echo ApiResponse::respond(101);
                exit();
            }
            $receivingDate = new DateTime($_POST["receivingDate"]);
            $commentOnReception = $_POST["commentOnReception"];
            $content = json_decode($_POST["content"], true);
            $supplierDao = new SupplierDAO();
            $supplier = $supplierDao->getSupplierById($supplierId);
            $reception->setIdFournisseur($supplier);
            $storeDao = new StoreDAO();
            $store = $storeDao->getStoreById($storeId);
            $reception->setIdMagasin($store);
            $reception->setNumBl($deliveryNoteNumber);
            $reception->setDateReception($receivingDate);
            $reception->setObservation($commentOnReception);
            $receptionLineDao = new ReceptionLineDAO();
            $receptionLines = $receptionLineDao->getReceptionLinesOfReception($reception);
            foreach ($receptionLines as $receptionLine) {
                $receptionLineDao->delete($receptionLine);
            }
            $productDao = new ProductDAO();
            $count = 1;
            foreach ($content as $receptionDetail) {
                $receptionLine = $receptionLineDao->create();
                $receptionLine->setQuantite((int)$receptionDetail["quantity"]);
                $receptionLine->setCodeBarre($receptionDetail["batchNumber"]);
                $receptionLine->setPrixAchat((float)$receptionDetail["buyingPrice"]);
                $receptionLine->setPrixUnitaire((float)$receptionDetail["sellingPrice"]);
                $receptionLine->setDesignation($receptionDetail["designation"]);
                $receptionLine->setPourcentageMaxRemise((float)$receptionDetail["maximumReductionPercent"]);
                $receptionLine->setNumLotFournisseur($receptionDetail["batchNumber"]);
                $receptionLine->setNumLot("{$count}");
                $receptionLine->setNumLigne($count);
                $product = $productDao->getProductById((int)$receptionDetail["id"]);
                $receptionLine->setIdProduit($product);
                if (GESTION_DATE_PEREMPTION_PRODUIT == 1 && $receptionLine->getIdProduit()->getPerissable() == 1 && $receptionDetail["expirationDate"] != "") {
                    $receptionLine->setDatePeremption(new DateTime($receptionDetail["expirationDate"]));
                }
                $receptionLine->setIdReception($reception);
                $receptionLineDao->save($receptionLine);
                $receptionLine->setNumLot("{$receptionLine->getIdLigneReception()}{$count}");
                $receptionLineDao->update($receptionLine);
                $count++;
            }
            $databaseConnection->commit();
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("reception.index.receptionUpdatedSuccessfully")} : N°{$reception->getIdReception()}"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            $databaseConnection->rollBack();
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function deleteAction()
    {
        try {
            $id = (int)$_POST["id"];
            $receptionDao = new ReceptionDAO();
            $reception = $receptionDao->getReceptionById($id);
            if ($reception->getStatut() == 0) {
                echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE);
                exit();
            }
            if ($reception->getValider() == 1) {
                echo ApiResponse::respond(ALREADY_VALIDATED_ISSUE_CODE);
                exit();
            }
            $reception->setStatut(0);
            $receptionDao->update($reception);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("order.index.receptionDeletedSuccessfully")} : N°{$reception->getIdReception()}"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    public function validationAction()
    {
        AccessRightChecker::checkAccessRight($this, 14);
        $supplierDao = new SupplierDAO();
        $productCategoryDao = new ProductCategoryDAO();
        $storeDao = new StoreDAO();
        $currentUser = LogDAO::getCurrentUser();
        $stores = [];
        if (
            $currentUser->getIdPointVente() == null || $currentUser->getIdProfil()->getCode() == "ADMIN"
            || $currentUser->getIdProfil()->getCode() == "MANA"
            || $currentUser->getIdProfil()->getCode() == "COMP"
        ) {
            $stores = $storeDao->getAllStores();
        } else {
            $stores[] = $currentUser->getIdPointVente()->getIdMagasin();
        }
        $this->view->suppliers = $supplierDao->listSuppliers();
        $this->view->productCategories = $productCategoryDao->getAllProductCategories();
        $this->view->stores = $stores;
        $this->view->translator = $this->translator;
    }

    public function datatablevalidationAction()
    {
        $store = (int)$_POST["store"] != 0 ? (int)$_POST["store"] : null;
        $supplier = (int)$_POST["supplier"] != 0 ? (int)$_POST["supplier"] : null;
        $from = $_POST["from"] != "" ? $_POST["from"] : null;
        $to = $_POST["to"] != "" ? $_POST["to"] : null;
        $whereStore = "";
        $whereSupplier = "";
        $whereFrom = "";
        $whereTo = "";
        if ($store != null) {
            $whereStore .= " AND r.id_magasin = {$store}";
        }
        if ($supplier != null) {
            $whereSupplier .= " AND r.id_fournisseur = {$supplier}";
        }
        if ($from != null) {
            $whereFrom .= " AND r.date_reception >= '{$from} 00:00:00'";
        }
        if ($to != null) {
            $whereTo .= " AND r.date_reception <= '{$to} 23:59:59'";
        }
        $where = "r.statut = 1 AND r.valider = 0{$whereStore}{$whereSupplier}{$whereFrom}{$whereTo}";
        $receptionsValidationTable = "
                (
                    SELECT r.id_reception AS reception_id, m.nom_magasin AS reception_store, f.nom_fournisseur AS reception_supplier,
                    r.date_saisie AS reception_entry_date, r.date_reception AS reception_receiving_date, r.num_bl AS reception_delivery_note_number
                    FROM reception r JOIN magasin m ON r.id_magasin = m.id_magasin JOIN fournisseur f ON r.id_fournisseur = f.id_fournisseur
                    WHERE {$where}
                ) AS receptionsValidationTable
        ";
        $primaryKey = 'reception_id';
        $columns = [
            ['db' => 'reception_id', 'dt' => 0],
            ['db' => 'reception_store', 'dt' => 1],
            ['db' => 'reception_supplier', 'dt' => 2],
            ['db' => 'reception_entry_date', 'dt' => 3],
            ['db' => 'reception_receiving_date', 'dt' => 4],
            ['db' => 'reception_delivery_note_number', 'dt' => 5],
        ];
        $computeDatatableResults = SSP::customSimple($_POST, $receptionsValidationTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        foreach ($datatableResult["data"] as $row) {
            $receptionId = (int)$row[0];
            $receptionStore = $row[1];
            $receptionSupplier = $row[2];
            $entryDate = new DateTime($row[3]);
            $receivingDate = new DateTime($row[4]);
            $deliveryNoteNumber = $row[5];
            $actions = "
                <button title='{$this->translator->_("shared.consult")}' about='{$receptionId}' class='mini ui icon button app-primary-button app-consult-action'>
                    <i class='eye icon'></i>
                </button>
                <button title='{$this->translator->_("shared.validate")}' about='{$receptionId}' class='mini ui green icon button app-validate-action'>
                    <i class='check icon'></i>
                </button>
            ";
            $tableRow = [
                $receptionId,
                $receptionStore,
                $receptionSupplier,
                $entryDate->format(DATE_FORMAT),
                $receivingDate->format(DATE_FORMAT),
                $deliveryNoteNumber,
                $actions,
            ];
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function validateAction()
    {
        $receptionDao = new ReceptionDAO();
        $databaseConnection = $receptionDao->entityManager->getConnection();
        $databaseConnection->beginTransaction();
        try {
            $id = (int)$_POST["id"];
            $receptionDao = new ReceptionDAO();
            $reception = $receptionDao->getReceptionById($id);
            if ($reception->getStatut() == 0) {
                echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE);
                exit();
            }
            if ($reception->getValider() == 1) {
                echo ApiResponse::respond(ALREADY_VALIDATED_ISSUE_CODE);
                exit();
            }
            $receptionLineDao = new ReceptionLineDAO();
            $receptionLines = $receptionLineDao->getReceptionLinesOfReception($reception);
            $stockDao = new StockDAO();
            foreach ($receptionLines as $receptionLine) {
                //Always consider use of expiration date and product perishable status when dealing with stocks
                $product = $receptionLine->getIdProduit();
                $store = $reception->getIdMagasin();
                $mostRecentStockOnProduct = $stockDao->getMustRecentStockOfProducAtStore($product, $store);
                if ((GESTION_DATE_PEREMPTION_PRODUIT == 0 || (GESTION_DATE_PEREMPTION_PRODUIT == 1 && $product->getPerissable() == 0)) && $mostRecentStockOnProduct != null) {
                    //In this case just update current stock quantity and entry date
                    $mostRecentStockOnProduct->setQuantite($mostRecentStockOnProduct->getQuantite() + $receptionLine->getQuantite());
                    $mostRecentStockOnProduct->setQuantiteEntree($mostRecentStockOnProduct->getQuantiteEntree() + $receptionLine->getQuantite());
                    $mostRecentStockOnProduct->setDateEntree(new DateTime());
                    $mostRecentStockOnProduct->setPrixAchat($receptionLine->getPrixAchat());
                    $mostRecentStockOnProduct->setPrixVente($receptionLine->getPrixUnitaire());
                    $stockDao->update($mostRecentStockOnProduct);
                } else {
                    //Else create new stock at all
                    $stock = $stockDao->create();
                    $stock->setIdProduit($receptionLine->getIdProduit());
                    $stock->setIdMagasin($reception->getIdMagasin());
                    $stock->setDateEntree($reception->getDateReception());
                    $stock->setQuantite($receptionLine->getQuantite());
                    $stock->setQuantiteEntree($receptionLine->getQuantite());
                    $stock->setPrixAchat($receptionLine->getPrixAchat());
                    $stock->setPrixVente($receptionLine->getPrixUnitaire());
                    $stock->setMarge($receptionLine->getPrixUnitaire() - $receptionLine->getPrixAchat());
                    $desiredLength = (int)DEFAULT_BATCH_NUMBER_LENGTH;
                    $prefixedBatchNumber = strlen($receptionLine->getNumLot()) < $desiredLength ? str_pad($receptionLine->getNumLot(), $desiredLength, "0", STR_PAD_LEFT) : $receptionLine->getNumLot();
                    $stock->setNumLot($prefixedBatchNumber);
                    $stock->setIdReception($reception);
                    $stock->setNumLotFournisseur($receptionLine->getNumLotFournisseur());
                    $stock->setCodeBarre($receptionLine->getCodeBarre());
                    if (GESTION_DATE_PEREMPTION_PRODUIT == 1 && $product->getPerissable() == 1 && $receptionLine->getDatePeremption() != null) {
                        $stock->setDatePeremption($receptionLine->getDatePeremption());
                    }
                    $stock->setDesignation($receptionLine->getDesignation());
                    $stock->setPrixVente2($receptionLine->getIdProduit()->getPrixVente2());
                    $stock->setPrixVente3($receptionLine->getIdProduit()->getPrixVente3());
                    $stock->setStatut(1);
                    $stockDao->save($stock);
                }
            }
            $reception->setValider(1);
            $reception->setDateValidation(new DateTime());
            $receptionDao->update($reception);
            $databaseConnection->commit();
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("reception.index.receptionValidatedSuccessfully")} : N°{$reception->getIdReception()}"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            $databaseConnection->rollBack();
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }
}
