<?php

include APPLICATION_PATH . '/includes/daos.php';

class SalaryController extends Zend_Controller_Action
{
    private $translator;

    public function init()
    {
        session_start();
        SubscriptionChecker::checkSubscriptionValidity($this);
        if (isset($_SESSION[SESSION_USER_ID]) && $_SESSION[SESSION_USER_ID] != null) {
            $_SESSION[LAST_ACTION_DATE] = new DateTime();
            LogDAO::updateLastActionDateForCurrentUser();
            $this->translator = Translator::init();
        } else {
            if ($this->getRequest()->isPost()) {
                echo ApiResponse::respond(SESSION_EXPIRED_CODE);
                exit();
            } else {
                $this->_helper->redirector('index', 'Index');
            }
        }
    }

    public function payrollsectionsAction()
    {
        AccessRightChecker::checkAccessRight($this, 120);
        $this->view->translator = $this->translator;
    }

    public function datatablepayrollsectionsAction()
    {
        $sign = $_POST["sign"] != "" ? (int)$_POST["sign"] : null;
        $name = $_POST["name"] != "" ? $_POST["name"] : null;
        $whereSign = "";
        $whereName = "";
        if (!is_null($sign)) {
            $whereSign .= " AND r.type = {$sign}";
        }
        if ($name != null) {
            $whereName .= " AND r.nom_rubrique LIKE '%{$name}%'";
        }
        $where = "r.statut = 1{$whereSign}{$whereName}";
        $payrollSectionsTable = "
                (
                    SELECT r.id_rubrique_paie as payroll_section_id, r.nom_rubrique AS payroll_section_name, r.code_rubrique AS payroll_section_code,
                    r.montant_defaut AS payroll_section_default_amount, r.type AS payroll_section_sign
                    FROM rubrique_paie r
                    WHERE {$where}
                ) AS payrollSectionsTable
        ";
        $primaryKey = 'payroll_section_id';
        $columns = [
            ['db' => 'payroll_section_name', 'dt' => 0],
            ['db' => 'payroll_section_code', 'dt' => 1],
            ['db' => 'payroll_section_sign', 'dt' => 2],
            ['db' => 'payroll_section_default_amount', 'dt' => 3],
            ['db' => 'payroll_section_id', 'dt' => 4],
        ];
        $computeDatatableResults = SSP::customSimple($_POST, $payrollSectionsTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        foreach ($datatableResult["data"] as $row) {
            $payrollSectionName = $row[0];
            $payrollSectionCode = $row[1];
            $payrollSectionSign = (int)$row[2];
            $payrollSectionDefaultAmount = number_format((float)$row[3], 0, '', ' ');
            $payrollSectionId = $row[4];
            $actions = "
                <button title='{$this->translator->_("shared.edit")}' about='{$payrollSectionId}' class='mini ui icon button app-secondary-button app-edit-action'>
                    <i class='edit icon'></i>
                </button>
                <button title='{$this->translator->_("shared.delete")}' about='{$payrollSectionId}' class='mini ui red icon button app-delete-action'>
                    <i class='trash icon'></i>
                </button>
            ";
            $tableRow = [
                $payrollSectionName,
                $payrollSectionCode,
                $payrollSectionSign == POSITIVE_PAYROLL_SECTION_SIGN ?
                    "<a class='ui label app-primary-label'>{$this->translator->_("salary.payrollSections.positive")}</a>"
                    : "<a class='ui label app-secondary-label'>{$this->translator->_("salary.payrollSections.negative")}</a>",
                $payrollSectionDefaultAmount,
                $actions,
            ];
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    /**
     * 100 success
     * 101 duplicate on designation
     * 102 duplicate on code
     * 103 unknown error
     */
    public function createpayrollsectionAction()
    {
        try {
            $designation = $_POST["designation"];
            $code = $_POST["code"];
            $sign = (int)$_POST["sign"];
            $defaultAmount = (float)$_POST["defaultAmount"];
            $payrollSectionDao = new PayrollSectionDAO();
            if ($payrollSectionDao->payrollSectionNameAlreadyExists($designation)) {
                echo ApiResponse::respond(101);
                exit();
            }
            if ($payrollSectionDao->payrollSectionCodeAlreadyExists($code)) {
                echo ApiResponse::respond(102);
                exit();
            }
            $payrollSection = $payrollSectionDao->create();
            $payrollSection->setNomRubrique($designation);
            $payrollSection->setCodeRubrique($code);
            $payrollSection->setType($sign);
            $payrollSection->setMontantDefaut($defaultAmount);
            $payrollSection->setStatut(1);
            $payrollSectionDao->save($payrollSection);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("salary.payrollSections.payrollSectionSuccessfullyRegistered")} : N°{$payrollSection->getIdRubriquePaie()}({$payrollSection->getNomRubrique()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(103, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function detailpayrollsectionAction()
    {
        try {
            $id = (int)$_POST["id"];
            $payrollSectionDao = new PayrollSectionDAO();
            $payrollSection = $payrollSectionDao->getPayrollSectionById($id);
            $result = [
                "designation" => $payrollSection->getNomRubrique(),
                "code" => $payrollSection->getCodeRubrique(),
                "sign" => $payrollSection->getType(),
                "defaultAmount" => $payrollSection->getMontantDefaut(),
            ];
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 duplicate on designation
     * 102 duplicate on code
     * 103 unknown error
     */
    public function updatepayrollsectionAction()
    {
        try {
            $id = (int)$_POST["id"];
            $designation = $_POST["designation"];
            $code = $_POST["code"];
            $sign = (int)$_POST["sign"];
            $defaultAmount = (float)$_POST["defaultAmount"];
            $payrollSectionDao = new PayrollSectionDAO();
            $payrollSection = $payrollSectionDao->getPayrollSectionById($id);
            if ($payrollSectionDao->payrollSectionNameAlreadyExists($designation, $payrollSection)) {
                echo ApiResponse::respond(101);
                exit();
            }
            if ($payrollSectionDao->payrollSectionCodeAlreadyExists($code, $payrollSection)) {
                echo ApiResponse::respond(102);
                exit();
            }
            $payrollSection->setNomRubrique($designation);
            $payrollSection->setCodeRubrique($code);
            $payrollSection->setType($sign);
            $payrollSection->setMontantDefaut($defaultAmount);
            $payrollSectionDao->update($payrollSection);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("salary.payrollSections.payrollSectionUpdatedSuccessfully")} : N°{$payrollSection->getIdRubriquePaie()}({$payrollSection->getNomRubrique()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(103, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function deletepayrollsectionAction()
    {
        try {
            $id = (int)$_POST["id"];
            $payrollSectionDao = new PayrollSectionDAO();
            $payrollSection = $payrollSectionDao->getPayrollSectionById($id);
            if ($payrollSection->getStatut() == 0) {
                echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE);
                exit();
            }
            $payrollSection->setStatut(0);
            $payrollSectionDao->update($payrollSection);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("salary.payrollSections.payrollSectionDeletedSuccessfully")} : N°{$payrollSection->getIdRubriquePaie()}({$payrollSection->getNomRubrique()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    public function payrolltemplatesAction()
    {
        AccessRightChecker::checkAccessRight($this, 121);
        $this->view->translator = $this->translator;
    }

    public function datatablepayrolltemplatesAction()
    {
        $name = $_POST["name"] != "" ? $_POST["name"] : null;
        $whereName = "";
        if ($name != null) {
            $whereName .= " AND m.nom_modele LIKE '%{$name}%'";
        }
        $where = "m.statut = 1{$whereName}";
        $payrollTemplatesTable = "
                (
                    SELECT m.id_modele_paie as payroll_template_id, m.nom_modele AS payroll_template_name,
                    m.montant_salaire AS payroll_template_salary_amount
                    FROM modele_paie_mensuel m
                    WHERE {$where}
                ) AS payrollTemplatesTable
        ";
        $primaryKey = 'payroll_template_id';
        $columns = [
            ['db' => 'payroll_template_name', 'dt' => 0],
            ['db' => 'payroll_template_salary_amount', 'dt' => 1],
            ['db' => 'payroll_template_id', 'dt' => 2],
        ];
        $computeDatatableResults = SSP::customSimple($_POST, $payrollTemplatesTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        foreach ($datatableResult["data"] as $row) {
            $payrollTemplateName = $row[0];
            $payrollTemplateSalaryAmount = number_format((float)$row[1], 0, '', ' ');
            $payrollTemplateId = $row[2];
            $actions = "
                <button title='{$this->translator->_("shared.consult")}' about='{$payrollTemplateId}' class='mini ui icon button app-primary-button app-consult-action'>
                    <i class='eye icon'></i>
                </button>
                <button title='{$this->translator->_("shared.edit")}' about='{$payrollTemplateId}' class='mini ui icon button app-secondary-button app-edit-action'>
                    <i class='edit icon'></i>
                </button>
                <button title='{$this->translator->_("shared.delete")}' about='{$payrollTemplateId}' class='mini ui red icon button app-delete-action'>
                    <i class='trash icon'></i>
                </button>
            ";
            $tableRow = [
                $payrollTemplateName,
                $payrollTemplateSalaryAmount,
                $actions,
            ];
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    public function payrolltemplatessectionsdatatableAction()
    {
        $sign = $_POST["sign"] != "" ? (int)$_POST["sign"] : null;
        $name = $_POST["name"] != "" ? $_POST["name"] : null;
        $whereSign = "";
        $whereName = "";
        if (!is_null($sign)) {
            $whereSign .= " AND r.type = {$sign}";
        }
        if ($name != null) {
            $whereName .= " AND r.nom_rubrique LIKE '%{$name}%'";
        }
        $where = "r.statut = 1{$whereSign}{$whereName}";
        $payrollTemplatesSectionsTable = "
                (
                    SELECT r.id_rubrique_paie as payroll_section_id, r.nom_rubrique AS payroll_section_name, r.code_rubrique AS payroll_section_code,
                    r.montant_defaut AS payroll_section_default_amount, r.type AS payroll_section_sign
                    FROM rubrique_paie r
                    WHERE {$where}
                ) AS payrollTemplatesSectionsTable
        ";
        $primaryKey = 'payroll_section_id';
        $columns = [
            ['db' => 'payroll_section_name', 'dt' => 0],
            ['db' => 'payroll_section_sign', 'dt' => 1],
            ['db' => 'payroll_section_default_amount', 'dt' => 2],
            ['db' => 'payroll_section_id', 'dt' => 3],
        ];
        $computeDatatableResults = SSP::customSimple($_POST, $payrollTemplatesSectionsTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        foreach ($datatableResult["data"] as $row) {
            $payrollSectionName = $row[0];
            $payrollSectionSign = (int)$row[1];
            $payrollSectionDefaultAmount = (float)$row[2];
            $payrollSectionId = $row[3];
            $tableRow = [
                $payrollSectionName,
                $payrollSectionSign == POSITIVE_PAYROLL_SECTION_SIGN ?
                    "<a class='ui label app-primary-label'>{$this->translator->_("salary.payrollTemplates.positive")}</a>"
                    : "<a class='ui label app-secondary-label'>{$this->translator->_("salary.payrollTemplates.negative")}</a>",
                number_format($payrollSectionDefaultAmount, 0, '', ' '),
                [
                    $payrollSectionDefaultAmount,
                    $payrollSectionId,
                    $payrollSectionName,
                    $payrollSectionSign
                ],
            ];
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    /**
     * 100 success
     * 101 duplicate
     * 102 unknown error
     */
    public function createpayrolltemplateAction()
    {
        $monthlyPayModelDao = new MonthlyPayModelDAO();
        $databaseConnection = $monthlyPayModelDao->entityManager->getConnection();
        $databaseConnection->beginTransaction();
        try {
            $designation = $_POST["designation"];
            $content = json_decode($_POST["content"], true);
            if ($monthlyPayModelDao->monthlyPayModelNameAlreadyExists($designation)) {
                $databaseConnection->rollBack();
                echo ApiResponse::respond(101);
                exit();
            }
            $monthlyPayModel = $monthlyPayModelDao->create();
            $monthlyPayModel->setNomModele($designation);
            $monthlyPayModel->setMontantSalaire(0);
            $monthlyPayModel->setStatut(1);
            $monthlyPayModelDao->save($monthlyPayModel);
            $monthlyPayModelDetailDao = new MonthlyPayModelDetailDAO();
            $payrollSectionDao = new PayrollSectionDAO();
            $salaryAmount = 0;
            foreach ($content as $detail) {
                $monthlyPayModelDetail = $monthlyPayModelDetailDao->create();
                $monthlyPayModelDetail->setIdModelePaie($monthlyPayModel);
                $payrollSection = $payrollSectionDao->getPayrollSectionById((int)$detail["id"]);
                $monthlyPayModelDetail->setIdRubriquePaie($payrollSection);
                $monthlyPayModelDetail->setType($payrollSection->getType());
                $monthlyPayModelDetail->setMontant((float)$detail["amount"]);
                $monthlyPayModelDetail->setStatut(1);
                $monthlyPayModelDetailDao->save($monthlyPayModelDetail);
                if ($payrollSection->getType() == POSITIVE_PAYROLL_SECTION_SIGN) {
                    $salaryAmount += (float)$detail["amount"];
                } else {
                    $salaryAmount -= (float)$detail["amount"];
                }
            }
            $monthlyPayModel->setMontantSalaire($salaryAmount);
            $monthlyPayModelDao->update($monthlyPayModel);
            $databaseConnection->commit();
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("salary.payrollTemplates.payrollTemplateSuccessfullyRegistered")} : N°{$monthlyPayModel->getIdModelePaie()}({$monthlyPayModel->getNomModele()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            $databaseConnection->rollBack();
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function detailpayrolltemplateAction()
    {
        try {
            $id = (int)$_POST["id"];
            $monthlyPayModelDao = new MonthlyPayModelDAO();
            $monthlyPayModel = $monthlyPayModelDao->getMonthlyPayModelById($id);
            $result = [
                "designation" => $monthlyPayModel->getNomModele(),
                "content" => []
            ];
            $monthlyPayModelDetailDao = new MonthlyPayModelDetailDAO();
            $monthlyPayModelDetails = $monthlyPayModelDetailDao->getDetailsOfMonthlyPayModel($monthlyPayModel);
            foreach ($monthlyPayModelDetails as $monthlyPayModelDetail) {
                $result["content"][] = [
                    "id" => $monthlyPayModelDetail->getIdRubriquePaie()->getIdRubriquePaie(),
                    "designation" => $monthlyPayModelDetail->getIdRubriquePaie()->getNomRubrique(),
                    "sign" => $monthlyPayModelDetail->getType(),
                    "amount" => $monthlyPayModelDetail->getMontant(),
                ];
            }
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 duplicate
     * 102 unknown error
     */
    public function updatepayrolltemplateAction()
    {
        $monthlyPayModelDao = new MonthlyPayModelDAO();
        $databaseConnection = $monthlyPayModelDao->entityManager->getConnection();
        $databaseConnection->beginTransaction();
        try {
            $id = (int)$_POST["id"];
            $designation = $_POST["designation"];
            $content = json_decode($_POST["content"], true);
            $monthlyPayModel = $monthlyPayModelDao->getMonthlyPayModelById($id);
            if ($monthlyPayModelDao->monthlyPayModelNameAlreadyExists($designation, $monthlyPayModel)) {
                $databaseConnection->rollBack();
                echo ApiResponse::respond(101);
                exit();
            }
            $monthlyPayModel->setNomModele($designation);
            $monthlyPayModelDao->update($monthlyPayModel);
            $monthlyPayModelDetailDao = new MonthlyPayModelDetailDAO();
            $monthlyPayModelDetails = $monthlyPayModelDetailDao->getDetailsOfMonthlyPayModel($monthlyPayModel);
            foreach ($monthlyPayModelDetails as $monthlyPayModelDetail) {
                $monthlyPayModelDetailDao->delete($monthlyPayModelDetail);
            }
            $payrollSectionDao = new PayrollSectionDAO();
            $salaryAmount = 0;
            foreach ($content as $detail) {
                $monthlyPayModelDetail = $monthlyPayModelDetailDao->create();
                $monthlyPayModelDetail->setIdModelePaie($monthlyPayModel);
                $payrollSection = $payrollSectionDao->getPayrollSectionById((int)$detail["id"]);
                $monthlyPayModelDetail->setIdRubriquePaie($payrollSection);
                $monthlyPayModelDetail->setType($payrollSection->getType());
                $monthlyPayModelDetail->setMontant((float)$detail["amount"]);
                $monthlyPayModelDetail->setStatut(1);
                $monthlyPayModelDetailDao->save($monthlyPayModelDetail);
                if ($payrollSection->getType() == POSITIVE_PAYROLL_SECTION_SIGN) {
                    $salaryAmount += (float)$detail["amount"];
                } else {
                    $salaryAmount -= (float)$detail["amount"];
                }
            }
            $monthlyPayModel->setMontantSalaire($salaryAmount);
            $monthlyPayModelDao->update($monthlyPayModel);
            $databaseConnection->commit();
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("salary.payrollTemplates.payrollTemplateUpdatedSuccessfully")} : N°{$monthlyPayModel->getIdModelePaie()}({$monthlyPayModel->getNomModele()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            $databaseConnection->rollBack();
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function deletepayrolltemplateAction()
    {
        try {
            $id = (int)$_POST["id"];
            $monthlyPayModelDao = new MonthlyPayModelDAO();
            $monthlyPayModel = $monthlyPayModelDao->getMonthlyPayModelById($id);
            if ($monthlyPayModel->getStatut() == 0) {
                echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE);
                exit();
            }
            $monthlyPayModel->setStatut(0);
            $monthlyPayModelDao->update($monthlyPayModel);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("salary.payrollTemplates.payrollTemplateDeletedSuccessfully")} : N°{$monthlyPayModel->getIdModelePaie()}({$monthlyPayModel->getNomModele()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    public function salarygenerationAction()
    {
        AccessRightChecker::checkAccessRight($this, 123);
        $payrollSectionDao = new PayrollSectionDAO();
        $payrollSections = $payrollSectionDao->getAllPayrollSections();
        $this->view->payrollSections = $payrollSections;
        $this->view->translator = $this->translator;
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function loadsalaryinfosAction()
    {
        try {
            $yearNumber = (int)$_POST["year"];
            $monthNumber = (int)$_POST["month"];
            $monthDao = new MonthDAO();
            $payrollSectionDao = new PayrollSectionDAO();
            $monthlyPayDao = new MonthlyPayDAO();
            $monthlyPayDetailDao = new MonthlyPayDetailDAO();
            $month = $monthDao->getMonthByYearAndNumber($yearNumber, $monthNumber);
            $result = [
                "sections" => [],
                "salary" => [],
            ];
            if ($month != null) {
                //There are already monthly pay lines for this month
                $allPayrollSectionInvolvedIds = $monthlyPayDetailDao->getAllPayrollSectionsOfAMonth($month);
                $allPayrollSectionInvolved = [];
                foreach ($allPayrollSectionInvolvedIds as $involvePaySectionId) {
                    $payrollSection = $payrollSectionDao->getPayrollSectionById((int)$involvePaySectionId["payrollSectionId"]);
                    $allPayrollSectionInvolved[] = $payrollSection;
                    $result["sections"][] = [
                        "sectionId" => (int)$involvePaySectionId["payrollSectionId"],
                        "sectionName" => $payrollSection->getNomRubrique(),
                        "sectionSign" => $payrollSection->getType(),
                        "sectionCode" => $payrollSection->getCodeRubrique(),
                    ];
                }
                $allMonthlyPayOfMonth = $monthlyPayDao->getMontlyPayOfMonth($month);
                foreach ($allMonthlyPayOfMonth as $monthlyPay) {
                    $employeeLine = [
                        "employeeId" => $monthlyPay->getIdEmploye()->getIdUtilisateur(),
                        "employeeName" => "{$monthlyPay->getIdEmploye()->getIdPersonne()->getNom()} {$monthlyPay->getIdEmploye()->getIdPersonne()->getPrenom()}",
                        "details" => [],
                        "validated" => $monthlyPay->getValide() == 1 ? 1 : 0,
                    ];
                    foreach ($allPayrollSectionInvolved as $payrollSection) {
                        $monthlyPayDetail = $monthlyPayDetailDao->getMonthlyPayDetailOfMonthlyPayAndPayrollSection($monthlyPay, $payrollSection);
                        if ($monthlyPayDetail != null) {
                            $employeeLine["details"][] = [
                                "sectionId" => $payrollSection->getIdRubriquePaie(),
                                "sign" => $monthlyPayDetail->getType(),
                                "amount" => $monthlyPayDetail->getMontant(),
                            ];
                        } else {
                            $employeeLine["details"][] = [
                                "sectionId" => $payrollSection->getIdRubriquePaie(),
                                "sign" => $payrollSection->getType(),
                                "amount" => 0,
                            ];
                        }
                    }
                    $result["salary"][] = $employeeLine;
                }
            } else {
                //This is a new salary input, just load base info
                $userDao = new UserDAO();
                $monthlyPayModelDetailDao = new MonthlyPayModelDetailDAO();
                $allPayrollSections = $payrollSectionDao->getAllPayrollSections();
                foreach ($allPayrollSections as $payrollSection) {
                    $result["sections"][] = [
                        "sectionId" => $payrollSection->getIdRubriquePaie(),
                        "sectionName" => $payrollSection->getNomRubrique(),
                        "sectionSign" => $payrollSection->getType(),
                        "sectionCode" => $payrollSection->getCodeRubrique(),
                    ];
                }
                $salaryGenerationBase = $_POST["base"];
                $previousMonth = null;
                if ($salaryGenerationBase == "PREVIOUS") {
                    $monthNumberLetter = ((int)$monthNumber <= 9) ? "0{$monthNumber}" : "$monthNumber";
                    $selectedMonthFirstDay = new DateTime("{$yearNumber}-{$monthNumberLetter}-01");
                    $selectedMonthFirstDay->modify(" -1 day");
                    $previousMonthYearNumber = $selectedMonthFirstDay->format("Y");
                    $previousMonthMonthNumber = $selectedMonthFirstDay->format("m");
                    $previousMonth = $monthDao->getMonthByYearAndNumber($previousMonthYearNumber, $previousMonthMonthNumber);
                }
                $allEmployees = $userDao->listEmployees();
                foreach ($allEmployees as $employee) {
                    $employeeLine = [
                        "employeeId" => $employee->getIdUtilisateur(),
                        "employeeName" => "{$employee->getIdPersonne()->getNom()} {$employee->getIdPersonne()->getPrenom()}",
                        "details" => [],
                        "validated" => 0,
                    ];
                    foreach ($allPayrollSections as $payrollSection) {
                        if ($salaryGenerationBase == "FREE") {
                            $employeeLine["details"][] = [
                                "sectionId" => $payrollSection->getIdRubriquePaie(),
                                "sign" => $payrollSection->getType(),
                                "amount" => 0,
                            ];
                        } else if ($salaryGenerationBase == "TEMPLATE") {
                            $monthlyPayModelDetail = $employee->getIdModelePaie() != null ? $monthlyPayModelDetailDao->getDetailByMonthlyPayModelAndPayrollSection($employee->getIdModelePaie(), $payrollSection) : null;
                            if ($monthlyPayModelDetail != null) {
                                $employeeLine["details"][] = [
                                    "sectionId" => $payrollSection->getIdRubriquePaie(),
                                    "sign" => $monthlyPayModelDetail->getType(),
                                    "amount" => $monthlyPayModelDetail->getMontant(),
                                ];
                            } else {
                                $employeeLine["details"][] = [
                                    "sectionId" => $payrollSection->getIdRubriquePaie(),
                                    "sign" => $payrollSection->getType(),
                                    "amount" => 0,
                                ];
                            }
                        } else if ($salaryGenerationBase == "PREVIOUS") {
                            $monthlyPay = $previousMonth != null ? $monthlyPayDao->getMontlyPayByMonthEmployeeAndMonth($employee, $previousMonth) : null;
                            $monthlyPayDetail = $monthlyPay != null ? $monthlyPayDetailDao->getMonthlyPayDetailOfMonthlyPayAndPayrollSection($monthlyPay, $payrollSection) : null;
                            if ($monthlyPayDetail != null) {
                                $employeeLine["details"][] = [
                                    "sectionId" => $payrollSection->getIdRubriquePaie(),
                                    "sign" => $monthlyPayDetail->getType(),
                                    "amount" => $monthlyPayDetail->getMontant(),
                                ];
                            } else {
                                $employeeLine["details"][] = [
                                    "sectionId" => $payrollSection->getIdRubriquePaie(),
                                    "sign" => $payrollSection->getType(),
                                    "amount" => 0,
                                ];
                            }
                        }
                    }
                    $result["salary"][] = $employeeLine;
                }
            }
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function savesalaryAction()
    {
        $userDao = new UserDAO();
        $currentUser = LogDAO::getCurrentUser();
        $databaseConnection = $userDao->entityManager->getConnection();
        $databaseConnection->beginTransaction();
        try {
            $yearNumber = (int)$_POST["year"];
            $monthNumber = (int)$_POST["month"];
            $monthDao = new MonthDAO();
            $payrollSectionDao = new PayrollSectionDAO();
            $monthlyPayDao = new MonthlyPayDAO();
            $monthlyPayDetailDao = new MonthlyPayDetailDAO();
            $payrollSectionDao = new PayrollSectionDAO();
            $month = $monthDao->getMonthByYearAndNumber($yearNumber, $monthNumber);
            if ($month == null) {
                $month = $monthDao->create();
                $month->setAnnee((int)$yearNumber);
                $month->setNumeroMois((int)$monthNumber);
                $month->setStatut(1);
                $monthDao->save($month);
            }
            $content = json_decode($_POST["content"], true);
            foreach ($content as $employeeDetail) {
                $employee = $userDao->getUserById((int)$employeeDetail["id"]);
                $details = json_decode($employeeDetail["details"], true);
                $monthlyPay = $monthlyPayDao->getMontlyPayByMonthEmployeeAndMonth($employee, $month);
                if ($monthlyPay == null) {
                    $monthlyPay = $monthlyPayDao->create();
                    $monthlyPay->setIdEmploye($employee);
                    $monthlyPay->setIdMois($month);
                    $monthlyPay->setIdUtilisateur($currentUser);
                    $monthlyPay->setMontantSalaire(0);
                    $monthlyPay->setValide(0);
                    $monthlyPay->setStatut(1);
                    $monthlyPayDao->save($monthlyPay);
                }
                if ($monthlyPay->getValide() == 1) continue;
                $monthlyPayDetails = $monthlyPayDetailDao->getMonthlyPayDetailOfMonthlyPay($monthlyPay);
                foreach ($monthlyPayDetails as $monthlyPayDetail) {
                    $monthlyPayDetailDao->delete($monthlyPayDetail);
                }
                $totalSalary = 0;
                foreach ($details as $detail) {
                    if ((float)$detail["amount"] == 0) continue;
                    $payrollSection = $payrollSectionDao->getPayrollSectionById((int)$detail["id"]);
                    $newMonthlyPayDetail = $monthlyPayDetailDao->create();
                    $newMonthlyPayDetail->setIdPaieMensuel($monthlyPay);
                    $newMonthlyPayDetail->setIdRubriquePaie($payrollSection);
                    $newMonthlyPayDetail->setMontant((float)$detail["amount"]);
                    $newMonthlyPayDetail->setType($payrollSection->getType());
                    $newMonthlyPayDetail->setStatut(1);
                    $monthlyPayDetailDao->save($newMonthlyPayDetail);
                    if ($payrollSection->getType() == POSITIVE_PAYROLL_SECTION_SIGN) {
                        $totalSalary += (float)$detail["amount"];
                    } else {
                        $totalSalary -= (float)$detail["amount"];
                    }
                }
                if ($totalSalary <= 0) {
                    $databaseConnection->rollBack();
                    echo ApiResponse::respond(101);
                    exit();
                }
                $monthlyPay->setMontantSalaire($totalSalary);
                $monthlyPayDao->update($monthlyPay);
            }
            $monthNumberLetter = ((int)$monthNumber <= 9) ? "0{$monthNumber}" : "$monthNumber";
            $selectedMonthYearDate = new DateTime("{$yearNumber}-{$monthNumberLetter}-01");
            $momentOfSelectMonthYearDate = new \Moment\Moment($selectedMonthYearDate->format(DATE_FORMAT));
            $monthAndYear = "{$momentOfSelectMonthYearDate->format(DATE_FORMAT_MONTH)} {$momentOfSelectMonthYearDate->format(DATE_FORMAT_YEAR)}";
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("salary.salaryGeneration.salariesSavedSuccessfully")} ($monthAndYear)"
            );
            $databaseConnection->commit();
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            $databaseConnection->rollBack();
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function deletesalaryAction()
    {
        try {
            $yearNumber = (int)$_POST["year"];
            $monthNumber = (int)$_POST["month"];
            $employeeId = (int)$_POST["employeeId"];
            $monthDao = new MonthDAO();
            $monthlyPayDao = new MonthlyPayDAO();
            $userDao = new UserDAO();
            $month = $monthDao->getMonthByYearAndNumber($yearNumber, $monthNumber);
            $employee = $userDao->getUserById($employeeId);
            if ($month != null) {
                $monthlyPay = $monthlyPayDao->getMontlyPayByMonthEmployeeAndMonth($employee, $month);
                if ($monthlyPay != null) {
                    if ($monthlyPay->getStatut() == 0) {
                        echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE, null, ALREADY_DELETED_ISSUE_CODE);
                        exit();
                    }
                    if ($monthlyPay->getValide() == 1) {
                        echo ApiResponse::respond(ALREADY_VALIDATED_ISSUE_CODE, null, ALREADY_VALIDATED_ISSUE_CODE);
                        exit();
                    }
                    $monthlyPay->setStatut(0);
                    $monthlyPayDao->update($monthlyPay);
                    $monthNumberLetter = ((int)$monthNumber <= 9) ? "0{$monthNumber}" : "$monthNumber";
                    $selectedMonthYearDate = new DateTime("{$yearNumber}-{$monthNumberLetter}-01");
                    $momentOfSelectMonthYearDate = new \Moment\Moment($selectedMonthYearDate->format(DATE_FORMAT));
                    $monthAndYear = "{$momentOfSelectMonthYearDate->format(DATE_FORMAT_MONTH)} {$momentOfSelectMonthYearDate->format(DATE_FORMAT_YEAR)}";
                    LogDAO::registerOperationForCurrentUser(
                        "{$this->translator->_("salary.salaryGeneration.salaryDeletedSuccessfully")} {$employee->getIdPersonne()->getNom()} {$employee->getIdPersonne()->getPrenom()} ($monthAndYear, {$monthlyPay->getMontantSalaire()} {$this->translator->_("shared.xaf")})"
                    );
                }
            }
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    public function salaryvalidationAction()
    {
        AccessRightChecker::checkAccessRight($this, 125);
        $payrollSectionDao = new PayrollSectionDAO();
        $payrollSections = $payrollSectionDao->getAllPayrollSections();
        $this->view->payrollSections = $payrollSections;
        $this->view->translator = $this->translator;
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function loadsalaryinfosforvalidationAction()
    {
        try {
            $yearNumber = (int)$_POST["year"];
            $monthNumber = (int)$_POST["month"];
            $monthDao = new MonthDAO();
            $payrollSectionDao = new PayrollSectionDAO();
            $monthlyPayDao = new MonthlyPayDAO();
            $monthlyPayDetailDao = new MonthlyPayDetailDAO();
            $month = $monthDao->getMonthByYearAndNumber($yearNumber, $monthNumber);
            $result = [
                "sections" => [],
                "salary" => [],
            ];
            if ($month != null) {
                $allPayrollSectionInvolvedIds = $monthlyPayDetailDao->getAllPayrollSectionsOfAMonth($month);
                $allPayrollSectionInvolved = [];
                foreach ($allPayrollSectionInvolvedIds as $involvePaySectionId) {
                    $payrollSection = $payrollSectionDao->getPayrollSectionById((int)$involvePaySectionId["payrollSectionId"]);
                    $allPayrollSectionInvolved[] = $payrollSection;
                    $result["sections"][] = [
                        "sectionId" => (int)$involvePaySectionId["payrollSectionId"],
                        "sectionName" => $payrollSection->getNomRubrique(),
                        "sectionSign" => $payrollSection->getType(),
                        "sectionCode" => $payrollSection->getCodeRubrique(),
                    ];
                }
                $allMonthlyPayOfMonth = $monthlyPayDao->getMontlyPayOfMonth($month);
                foreach ($allMonthlyPayOfMonth as $monthlyPay) {
                    $employeeLine = [
                        "employeeId" => $monthlyPay->getIdEmploye()->getIdUtilisateur(),
                        "employeeName" => "{$monthlyPay->getIdEmploye()->getIdPersonne()->getNom()} {$monthlyPay->getIdEmploye()->getIdPersonne()->getPrenom()}",
                        "details" => [],
                        "validated" => $monthlyPay->getValide() == 1 ? 1 : 0,
                    ];
                    foreach ($allPayrollSectionInvolved as $payrollSection) {
                        $monthlyPayDetail = $monthlyPayDetailDao->getMonthlyPayDetailOfMonthlyPayAndPayrollSection($monthlyPay, $payrollSection);
                        if ($monthlyPayDetail != null) {
                            $employeeLine["details"][] = [
                                "sectionId" => $payrollSection->getIdRubriquePaie(),
                                "sign" => $monthlyPayDetail->getType(),
                                "amount" => $monthlyPayDetail->getMontant(),
                            ];
                        } else {
                            $employeeLine["details"][] = [
                                "sectionId" => $payrollSection->getIdRubriquePaie(),
                                "sign" => $payrollSection->getType(),
                                "amount" => 0,
                            ];
                        }
                    }
                    $result["salary"][] = $employeeLine;
                }
            } else {
                $payrollSections = $payrollSectionDao->getAllPayrollSections();
                foreach ($payrollSections as $payrollSection) {
                    $result["sections"][] = [
                        "sectionId" => $payrollSection->getIdRubriquePaie(),
                        "sectionName" => $payrollSection->getNomRubrique(),
                        "sectionSign" => $payrollSection->getType(),
                        "sectionCode" => $payrollSection->getCodeRubrique(),
                    ];
                }
            }
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function validatesalaryAction()
    {
        try {
            $yearNumber = (int)$_POST["year"];
            $monthNumber = (int)$_POST["month"];
            $employeeId = (int)$_POST["employeeId"];
            $monthDao = new MonthDAO();
            $monthlyPayDao = new MonthlyPayDAO();
            $userDao = new UserDAO();
            $month = $monthDao->getMonthByYearAndNumber($yearNumber, $monthNumber);
            $employee = $userDao->getUserById($employeeId);
            if ($month != null) {
                $monthlyPay = $monthlyPayDao->getMontlyPayByMonthEmployeeAndMonth($employee, $month);
                if ($monthlyPay != null) {
                    if ($monthlyPay->getStatut() == 0) {
                        echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE, null, ALREADY_DELETED_ISSUE_CODE);
                        exit();
                    }
                    if ($monthlyPay->getValide() == 1) {
                        echo ApiResponse::respond(ALREADY_VALIDATED_ISSUE_CODE, null, ALREADY_VALIDATED_ISSUE_CODE);
                        exit();
                    }
                    $monthlyPay->setValide(1);
                    $monthlyPayDao->update($monthlyPay);
                    $monthNumberLetter = ((int)$monthNumber <= 9) ? "0{$monthNumber}" : "$monthNumber";
                    $selectedMonthYearDate = new DateTime("{$yearNumber}-{$monthNumberLetter}-01");
                    $momentOfSelectMonthYearDate = new \Moment\Moment($selectedMonthYearDate->format(DATE_FORMAT));
                    $monthAndYear = "{$momentOfSelectMonthYearDate->format(DATE_FORMAT_MONTH)} {$momentOfSelectMonthYearDate->format(DATE_FORMAT_YEAR)}";
                    LogDAO::registerOperationForCurrentUser(
                        "{$this->translator->_("salary.salaryGeneration.salaryValidatedSuccessfully")} {$employee->getIdPersonne()->getNom()} {$employee->getIdPersonne()->getPrenom()} ($monthAndYear, {$monthlyPay->getMontantSalaire()} {$this->translator->_("shared.xaf")})"
                    );
                }
            }
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    public function payslipsAction()
    {
        AccessRightChecker::checkAccessRight($this, 124);
        $payrollSectionDao = new PayrollSectionDAO();
        $payrollSections = $payrollSectionDao->getAllPayrollSections();
        $this->view->payrollSections = $payrollSections;
        $this->view->translator = $this->translator;
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function loadsalaryinfosforpayslipsAction()
    {
        try {
            $yearNumber = (int)$_POST["year"];
            $monthNumber = (int)$_POST["month"];
            $monthDao = new MonthDAO();
            $payrollSectionDao = new PayrollSectionDAO();
            $monthlyPayDao = new MonthlyPayDAO();
            $monthlyPayDetailDao = new MonthlyPayDetailDAO();
            $month = $monthDao->getMonthByYearAndNumber($yearNumber, $monthNumber);
            $result = [
                "sections" => [],
                "salary" => [],
            ];
            if ($month != null) {
                $allPayrollSectionInvolvedIds = $monthlyPayDetailDao->getAllPayrollSectionsOfAMonth($month);
                $allPayrollSectionInvolved = [];
                foreach ($allPayrollSectionInvolvedIds as $involvePaySectionId) {
                    $payrollSection = $payrollSectionDao->getPayrollSectionById((int)$involvePaySectionId["payrollSectionId"]);
                    $allPayrollSectionInvolved[] = $payrollSection;
                    $result["sections"][] = [
                        "sectionId" => (int)$involvePaySectionId["payrollSectionId"],
                        "sectionName" => $payrollSection->getNomRubrique(),
                        "sectionSign" => $payrollSection->getType(),
                        "sectionCode" => $payrollSection->getCodeRubrique(),
                    ];
                }
                $allMonthlyPayOfMonth = $monthlyPayDao->getMontlyPayValidatedOfMonth($month);
                foreach ($allMonthlyPayOfMonth as $monthlyPay) {
                    $employeeLine = [
                        "employeeId" => $monthlyPay->getIdEmploye()->getIdUtilisateur(),
                        "employeeName" => "{$monthlyPay->getIdEmploye()->getIdPersonne()->getNom()} {$monthlyPay->getIdEmploye()->getIdPersonne()->getPrenom()}",
                        "details" => [],
                    ];
                    foreach ($allPayrollSectionInvolved as $payrollSection) {
                        $monthlyPayDetail = $monthlyPayDetailDao->getMonthlyPayDetailOfMonthlyPayAndPayrollSection($monthlyPay, $payrollSection);
                        if ($monthlyPayDetail != null) {
                            $employeeLine["details"][] = [
                                "sectionId" => $payrollSection->getIdRubriquePaie(),
                                "sign" => $monthlyPayDetail->getType(),
                                "amount" => $monthlyPayDetail->getMontant(),
                            ];
                        } else {
                            $employeeLine["details"][] = [
                                "sectionId" => $payrollSection->getIdRubriquePaie(),
                                "sign" => $payrollSection->getType(),
                                "amount" => 0,
                            ];
                        }
                    }
                    $result["salary"][] = $employeeLine;
                }
            } else {
                $payrollSections = $payrollSectionDao->getAllPayrollSections();
                foreach ($payrollSections as $payrollSection) {
                    $result["sections"][] = [
                        "sectionId" => $payrollSection->getIdRubriquePaie(),
                        "sectionName" => $payrollSection->getNomRubrique(),
                        "sectionSign" => $payrollSection->getType(),
                        "sectionCode" => $payrollSection->getCodeRubrique(),
                    ];
                }
            }
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }
}
