<?php

include APPLICATION_PATH . '/includes/daos.php';

class SaleController extends Zend_Controller_Action
{
    private $translator;

    public function init()
    {
        session_start();
        if (isset($_SESSION[SESSION_USER_ID]) && $_SESSION[SESSION_USER_ID] != null) {
            $_SESSION[LAST_ACTION_DATE] = new DateTime();
            $this->translator = Translator::init($this->getParam("language"));
        } else {
            if ($this->getRequest()->isPost()) {
                echo ApiResponse::respond(SESSION_EXPIRED_CODE);
                exit();
            } else {
                $this->_helper->redirector('index', 'Index');
            }
        }
    }

    public function sellAction()
    {
        AccessRightChecker::checkAccessRight($this, 15);

        $productCategoryDao = new ProductCategoryDAO();
        $paymentMethodDao = new PaymentMethodDAO();
        $userDAO = new UserDAO();
        $pointOfSaleDAO = new PointOfSaleDAO();
        $customerInvoiceDAO = new CustomerInvoiceDAO();

        $user = $userDAO->getUserById($_SESSION[SESSION_USER_ID]);

        //default point of sale
        $pointOfSale = null;

        //get last cash session's point of sale
        $lastSale = $customerInvoiceDAO->getLastInvoiceOfUser($user);

        if(is_object($lastSale))
        {
            $now = new \DateTime();
            
            $cashSessionExceptedClosingDate = $lastSale->getIdSessionCaissier()->getDatePrevuCloture();
            
            if($cashSessionExceptedClosingDate >= $now)
            {
                $pointOfSale = $lastSale->getIdPointVente();
            }
        }
        
        
        if(!$pointOfSale)
        {
            $pointOfSale = $user->getIdPointVente();
        }

        //user functions
        $isMainCashier = $user->getCaissierPrincipal() === 1;

        $isCashier = $user->getCaissier() === 1;

        $codeProfil = $user->getIdProfil()->getCode();

        $isSeller = false;

        if($codeProfil === CODE_PROFIL_VENDEUR)
        {
            $isSeller = true;
        }

        $pointOfSaleList = $pointOfSaleDAO->listPointOfSales();

        $paymentMethodList = $paymentMethodDao->getPaymentMethodList();

        $paymentMethods =  array();

        foreach ($paymentMethodList as $paymentMethod) {
            $id = $paymentMethod->getId();
            $code = $paymentMethod->getCode();
            $name = $this->translator->_($paymentMethod->getDesignation());

            $paymentMethods[$id] = array(
                "id" =>  $id,
                "code" => $code,
                "nom" => $name,
            );
        }

        $this->view->categories = $productCategoryDao->getAllProductCategories();
        $this->view->paymentMethods = $paymentMethods;
        $this->view->pointOfSaleName = (is_object($pointOfSale)) ? $pointOfSale->getNomPointVente() : "";
        $this->view->pointOfSaleId = (is_object($pointOfSale)) ? $pointOfSale->getIdPointVente(): "";
        $this->view->pointOfSaleList = $pointOfSaleList;
        $this->view->user = $user;
        $this->view->userFunctions = array("isMainCashier" => $isMainCashier, "isCashier" => $isCashier, "isSeller" => $isSeller );
        $this->view->translator = $this->translator;
    }

    public function getsellsettingsAction()
    {
        $userDAO = new UserDAO();
        $pointOfSaleDAO = new PointOfSaleDAO();
        $cashSessionDAO = new CashSessionDAO();
        $customerLocationDAO = new CustomerLocationDAO();
        $customerInvoiceDAO = new CustomerInvoiceDAO();

        $userId = (int) $_POST["userId"];
        $pointOfSaleId = (int) $_POST["pointOfSaleId"];

        //user point of sale
        $user = $userDAO->getUserById($userId);

        //point of sale
        $pointOfSale = $pointOfSaleDAO->getPointOfSaleById($pointOfSaleId);

        if(!$user  || !$pointOfSale)
        {
            echo ApiResponse::respond(101, "{$this->translator->_("sale.sell.unknowUserOrPointOfSale")}!");
            exit();
        }
        
        //Now
        $now = new \DateTime();

        //user functions
        $isMainCashier = $user->getCaissierPrincipal() === 1;

        $isCashier = $user->getCaissier() === 1;

        //list of opened sessions for cashier users
        $notClosedCashSessionList = array();

        $sellerSession = array();

        if($isCashier || $isMainCashier)
        {
            //NB: only one session can be opened at a time
            $notClosedCashSessionList = $cashSessionDAO->getNotClosedCashSessionsByUser($user, $pointOfSale );
        }
        else //is seller (or other user)
        {
            $notClosedCashSessionList = $cashSessionDAO->getNotClosedMainCashSessionsByPointOfSale($pointOfSale);

            $sellerSession = $this->getUserSellerSession( $user );
        }

        $cashSessions = array();

        foreach ($notClosedCashSessionList as $key => $cashSession) 
        {
            $sessionId = $cashSession->getIdSessionCaisse();
            $beginDate = $cashSession->getDateOuverture()->format("d-m-Y H:i:s");
            $cashierId = $cashSession->getIdUtilisateur()->getIdUtilisateur();
            $cashierName = $cashSession->getIdUtilisateur()->getIdPersonne()->getNom(). ' '.$cashSession->getIdUtilisateur()->getIdPersonne()->getPrenom();
            $closed = $cashSession->getFerme();
            $exceptedClosingDate = $cashSession->getDatePrevuCloture();

            if( $now <= $exceptedClosingDate )
            {
                $cashSessions[] = array(
                    "id" => $sessionId,
                    "beginDate" => $beginDate,
                    "cashierId" => $cashierId,
                    "cashierName" => $cashierName,
                    "closed" => $closed
                );
            }
            else
            {
                if($isCashier || $isMainCashier)
                {
                    //Forced closing
                    $cashSession->setFerme(1);
                    $cashSession->setCloturer(1);
                    $cashSession->setDateCloture( $now );
                    $cashSessionDAO->update($cashSession);
                    unset($notClosedCashSessionList[$key]); 
                }
            }
        }

        //Automatic opening session for cashier
        if(count($notClosedCashSessionList) === 0  && ($isCashier || $isMainCashier) )
        {
            $enDate = new DateTime("tomorrow - 1 min");
            $cashSession = $cashSessionDAO->create();
            $cashSession->setIdUtilisateur($user);
            $cashSession->setDateOuverture($now);
            $cashSession->setIdPointVente($pointOfSale);
            $cashSession->setFerme(0);
            $cashSession->setCloturer(0);
            $cashSession->setDatePrevuCloture($enDate);
            $cashSession->setPwdCaisse($user->getPassword());
            $cashSessionDAO->save($cashSession);

            $cashSessionId = $cashSession->getIdSessionCaisse();
            $cashierId = $cashSession->getIdUtilisateur()->getIdUtilisateur();
            $cashierName = $cashSession->getIdUtilisateur()->getIdPersonne()->getNom(). ' '.$cashSession->getIdUtilisateur()->getIdPersonne()->getPrenom();
            $closed = $cashSession->getFerme();
            $beginDate = $now->format("d-m-Y H:i:s");

            $cashSessions[] =  array(
                "id" => $cashSessionId,
                "beginDate" => $beginDate,
                "cashierId" => $cashierId,
                "cashierName" => $cashierName,
                "closed" => $closed
            );
        }

        //get last cash session of user
        $lastSale = $customerInvoiceDAO->getLastInvoiceOfUser($user);

        $lastValidSessionId = 0; 

        if(is_object($lastSale))
        {
            $now = new \DateTime();

            $lastCashSession = $cashSessionExceptedClosingDate = $lastSale->getIdSessionCaissier();
            
            $cashSessionExceptedClosingDate = $lastCashSession->getDatePrevuCloture();

            $sessionLock = $lastCashSession->getCloturer() || $lastCashSession->getFerme();
            
            if($cashSessionExceptedClosingDate >= $now && !$sessionLock)
            {
                $lastValidSessionId = $lastSale->getIdSessionCaissier()->getIdSessionCaisse();
            }
        }

        //Location list
        $locations = array();

        if((int) GESTION_EMPLACEMENT === 1)
        {
            //location list
            $locationList = $customerLocationDAO->getLocationByPointOfSale($pointOfSale);

            foreach($locationList as $location)
            {
                $locations [] = array(
                    "id"=> $location->getIdEmplacement(),
                    "name"=> $location->getDesignation(),
                );
            }
        }

        //pending invoice
        if(($isCashier || $isMainCashier) && isset($cashSessions[0]))
        {
            $activeCheckoutSessionId = $cashSessions[0]["id"];
            //Get pending invoices for user Session
            $pendingInvoices = $customerInvoiceDAO->getSessionPendingInvoices($activeCheckoutSessionId, true);

        }
        else{
            //Get pending invoices for user
            $pendingInvoices = $customerInvoiceDAO->getSessionPendingInvoices($sellerSession["id"], false);
        }
         
        $numberOfPendingInvoices = (int) $pendingInvoices["count"];

        //service provider list
        $employees = array();

        $employeesList = $userDAO->listEmployees($pointOfSale);

        foreach($employeesList as $employee)
        {
            $employees [] = array(
                "id"=> $employee->getIdUtilisateur(),
                "name"=> $employee->getIdPersonne()->getNom(). ' '.$employee->getIdPersonne()->getPrenom(),
            );
        }

        $result = array(
            "cashSessions" => $cashSessions,
            "sellerSession" => $sellerSession,
            "lastValidSessionId"   => $lastValidSessionId,
            "locations" => $locations,
            "employees" => $employees,
            "numberOfPendingInvoices" => $numberOfPendingInvoices
        );

        echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
        exit();
    }

    public function invoiceprocessingAction()
    {
        
        try {
            
            $purpose = isset($_POST["purpose"]) ? $_POST["purpose"]:null;

            $customerInvoiceDAO = new CustomerInvoiceDAO();
            $customerInvoiceLineDAO = new CustomerInvoiceLineDAO();
            $userDAO = new UserDAO();
            $pointOfSaleDAO  = new PointOfSaleDAO();

            $user = $userDAO->getUserById($_SESSION[SESSION_USER_ID]);

            if(!is_object($user))
            {
                echo ApiResponse::respond(101, "{$this->translator->_("sale.sell.unknowUser")}!");
                        exit();
            }

            $pointOfSaleId = isset($_POST["pointOfSaleId"])? $_POST["pointOfSaleId"]:null;

            //point of sale
            $pointOfSale = $pointOfSaleDAO->getPointOfSaleById($pointOfSaleId);

            if(!$pointOfSale)
            {
                echo ApiResponse::respond(101, "{$this->translator->_("sale.sell.unknowPointOfSale")}!");
                exit();
            }

            //get params
            $invoiceBasket = $_POST["invoice"];

            $invoiceBasket["user"] = array(
                "id" => $user->getIdUtilisateur(),
                "name" => $user->getIdPersonne()->getNom() . " " . $user->getIdPersonne()->getPrenom()
            );

            $invoiceBasket["pointOfSale"] = array(
                                "id" => $pointOfSale->getIdPointVente(),
                                "name" => $pointOfSale->getNomPointVente()
                            );

            //check action to do
            if($purpose == "save")
            {
                //check if it is sale with destocking in stock
                $saleWithDestocking = VENTE_MISE_EN_ATTENTE_AVEC_DESTOCKAGE;
                                     
                $invoiceBasket["saleWithDestocking"] = $saleWithDestocking;

                //pendind invoice
                $invoiceBasket["pending"] = 1;
                $invoiceBasket["validated"] = 0;

                echo $customerInvoiceDAO->registerCustomerInvoice($invoiceBasket, $this->translator);

                exit();
            }
            elseif($purpose == "pay")
            {
                $invoiceNumber = (int) $invoiceBasket["invoiceNumber"];
                
                
                if($invoiceNumber)
                {
                    $invoice = $customerInvoiceDAO->getCustomerInvoiceById($invoiceNumber);

                    if($invoice->getValider()  ===  1 )
                    {
                        echo ApiResponse::respond(101, "{$this->translator->_("sale.sell.currentInvoiceAlreadyValidatedAndCannotBeModified")}!");
                        exit();
                    }

                    //deleting old ligne
                    $oldCustumerInvoiceLine = $customerInvoiceLineDAO->getCustomerInvoiceLinesOfInvoice($invoice);

                    foreach( $oldCustumerInvoiceLine as $line){
                        $customerInvoiceLineDAO->delete($line);
                    }
                }

                $invoiceBasket["saleWithDestocking"] = 1;
                $invoiceBasket["pending"] = 0;
                $invoiceBasket["validated"] = 1;

                echo $customerInvoiceDAO->registerCustomerInvoice($invoiceBasket, $this->translator);

                exit();
            }

            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null);

        } catch (Exception $ex) {
            
            echo ApiResponse::respond(101, $ex->getMessage(), null);

        }
        exit();
    }


    public function findproductAction()
    {
        try {
            $productDAO = new ProductDAO();

            $promotionDAO = new PromotionDAO();

            $priceSettingDAO = new PriceSettingDAO();

            $personDAO = new PersonDAO();

            $userDAO = new UserDAO();

            $customerProductPriceDAO = new CustomerProductPriceDAO();

            $promotionLineDAO = new PromotionLineDAO();

            $cashSessionDAO = new CashSessionDAO();

            $pointOfSaleDAO = new PointOfSaleDAO();

            $name = $_POST["name"];

            $category = (isset($_POST["categoryId"])) ? $_POST["categoryId"] : null;

            $sessionId = (isset($_POST["sessionId"])) ? $_POST["sessionId"] : null;

            $currentIndex = (isset($_POST["currentIndex"])) ? (int) $_POST["currentIndex"] : null;

            $maxLength = (isset($_POST["maxLength"])) ? (int) $_POST["maxLength"] : null;

            $customer = isset($_POST["customer"]) ? $personDAO->getPersonById($_POST["customer"]) : null;

            $pointOfSaleId =(isset($_POST["pointOfSaleId"])) ? $_POST["pointOfSaleId"] : null;

            $now = new \DateTime();

            $resultat = array();

            $imageUrlBase = $this->view->baseUrl() . '/images/products';

            $defautImage = $this->view->baseUrl() . '/images/defaults/image_placeholder.png';

            $user = $userDAO->getUserById($_SESSION[SESSION_USER_ID]);

            //priority to cash session point of sale
            if( $sessionId ){

                $session =  $cashSessionDAO->getCashSessionById($sessionId);

                $pointOfSale = ($session) ? $session->getIdPointVente() : (($user->getIdPointVente()) ? $user->getIdPointVente() : null);
            }
            else
            {
                if($pointOfSaleId)
                {
                    $pointOfSale = $pointOfSaleDAO->getPointOfSaleById($pointOfSaleId);
                }
                else
                {
                    $pointOfSale = ($user->getIdPointVente()) ? $user->getIdPointVente() : null;
                }
            }

            if(!$pointOfSale)
            {
                echo ApiResponse::respond(101, "{$this->translator->_("sale.sell.unknowPointOfSale")}!");
                exit();
            }

            $store = $pointOfSale->getIdMagasin();

            $currentsPromotions = $promotionDAO->currentPromotion($now);

            $prixApplicablePointVente = (int) $priceSettingDAO->getConsideredPriceByPointOfSale($pointOfSale, $now);


            if (TYPE_VENTE_PRODUIT == TYPE_VENTE_PRODUIT_SUR_STOCK) {
                
                $typeService = 0;

                $stockSepare = GESTION_DATE_PEREMPTION_PRODUIT; //1 ou 0

                $availableProductList = $productDAO->getProductAvailableByCriteria($store, $name, $category, $stockSepare,  $currentIndex, $maxLength);

                //services
                $serviceList = $productDAO->getProductsFiltered($name, $category, $typeService);

                //juste pour eviter un bug dans le foreach qui va suivre
                $productAndServiceList = array();
            } else //vente sur catalogue
            {
                //recherche de produit et service
                $productAndServiceList = $productDAO->getProductsFiltered($name, $category, $currentIndex, $maxLength);

                //Juste pour eviter un bug dans le foreach qui va suivre
                $availableProductList  = array();

                //Juste pour eviter un bug dans le foreach qui va suivre
                $serviceList = array();
            }

            foreach (array_merge($availableProductList, $serviceList, $productAndServiceList) as $product) {
                if (is_object($product)) //catalogue
                {
                    $quantity = 0;
                    $datePeremption = "";
                    $productId = $product->getIdProduit();
                    $stockId = 0;
                    $batchNum = 0;
                    $name = $product->getDesignation();
                    $maxReduction = $product->getPourcentageMaxRemise() ? $product->getPourcentageMaxRemise() : 0;
                    $category = $product->getIdCategorieProduit() ? $product->getIdCategorieProduit()->getIdCategorie() : "";
                    $productType = ($product->getStockable() == 1) ? 1 : 0;
                    $applicablePrice = $product->getPrixVente();

                    //image
                    $imageUrl = $defautImage;

                    if ($product->getImage() != null) {
                        if (file_exists($imageUrlBase .  "/" . $product->getImage())) {
                            $imageUrl = $imageUrlBase .  "/" . $product->getImage();
                        }
                    }

                    if ($prixApplicablePointVente == 1) {
                        $applicablePrice = $product->getPrixVente2();
                    } elseif ($prixApplicablePointVente == 2) {
                        $applicablePrice = $product->getPrixVente3();
                    }
                } else //stock
                {
                    $quantity =  $product["quantity"];
                    $datePeremption = $product["datePeremption"];
                    $productId = $product["idProduit"];
                    $stockId = $product["idStock"];
                    $batchNum = $product["numLot"];
                    $name = $product["designation"];
                    $maxReduction = (float) $product["pourcentageMaxRemise"];
                    $category = (int) $product["idCategorie"];
                    $productType = 1; //stockable
                    
                    //image
                     $imageUrl = $defautImage;

                     if ($product["image"] != null) {
                         if (file_exists($imageUrlBase .  "/" . $product["image"])) {
                             $imageUrl = $imageUrlBase .  "/" . $product["image"];
                         }
                     }

                    $applicablePrice = $product["prixVente"];

                    if ($prixApplicablePointVente == 1) {
                        $applicablePrice = $product["prixVente2"];
                    } elseif ($prixApplicablePointVente == 2) {
                        $applicablePrice = $product["prixVente3"];
                    }
                }

                //pour la promotion eventuelle
                $promotionExist = 0;
                $oldPrice = $applicablePrice;


                if ((int) PRIX_PAR_CLIENT == 1 && $customer) {
                    $custumerProductPrice = $customerProductPriceDAO->getCustumerPriceByProductAndClient($productId, $customer);

                    if ($custumerProductPrice) {
                        $applicablePrice = $custumerProductPrice->getPrix();
                    }
                }

                if (count($currentsPromotions) > 0) {
                    foreach ($currentsPromotions as $promotion) {
                       
                        $promotionLine = $promotionLineDAO->getPromotionLineByProduct($promotion, $productId);
         
                        if ($promotionLine) {
                            $percentage = $promotionLine->getPourcentage();
                            $promotionExist = 1;
                            $oldPrice = $applicablePrice;
                            $applicablePrice = (int) ($applicablePrice - $applicablePrice * $percentage / 100);

                            //on a trouver le prix promotionnel
                            break;
                        }
                    }
                }

                $newLine = array(
                    "nom" => $name,
                    "productId" => $productId,
                    "stockId" => $stockId,
                    "batchNum" => $batchNum,
                    "prix" => $applicablePrice,
                    "prixFormat" => number_format((float) $applicablePrice, 0, '', ' '),
                    "categorie" => $category,
                    "maxReduction" => $maxReduction,
                    "quantite" => $quantity,
                    "promotion" => $promotionExist,
                    "type"  => $productType,
                    "image" => $imageUrl,
                    "ancienPrix" => $oldPrice,
                    "datePeremption"  => $datePeremption,
                );

                $resultat[] = $newLine;
            }

            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, array("products" => $resultat));
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }

        exit();
    }
    
    public function saleslogAction()
    {
        try 
        {
            $customerInvoiceDAO = new CustomerInvoiceDAO();

            $cashSessionDAO = new CashSessionDAO();

            $cashMovementDAO = new CashMovementDAO();

            $customerInvoiceLineDAO = new CustomerInvoiceLineDAO();
            
            $sessionId = (isset($_POST["sessionId"])) ? $_POST["sessionId"] : null;

            $isCashier = (isset($_POST["isCashier"])) ? $_POST["isCashier"] : null;
            
            $purpose = (isset($_POST["purpose"])) ? $_POST["purpose"] : null;

            $cashSession = $cashSessionDAO->getCashSessionById($sessionId);
                
            if(!$cashSession)
            {
                echo ApiResponse::respond(101, "{$this->translator->_("sale.sell.unknowCashoutSession")}!");
                exit();
            }

            $allSessionInvoices = [];

            $ortherCashMovement = [];

            $sellSessionTotal = [];

            $salesProducts = [];

            if($isCashier && $sessionId)
            {
                $allSessionInvoices = $customerInvoiceDAO->getInvoicesOfSession($cashSession);

                $ortherCashMovement = $cashMovementDAO->listEffectiveValidCashMovementOnCashSession($cashSession);

                $sellSessionTotal = $cashMovementDAO->getBalanceOnSession($cashSession);

                $salesProducts = $customerInvoiceLineDAO->getArticlesBySalesSession($cashSession, true);  
            }
            else{

                $allSessionInvoices = $customerInvoiceDAO->getInvoicesOfSellerSession($sessionId);

                $salesProducts = $customerInvoiceLineDAO->getArticlesBySalesSession($sessionId, false);  
            }

            $linkPdf = "";

            $invoicesArray = array();

            foreach($allSessionInvoices as $invoice)
            {
                $invoicesArray [] = $customerInvoiceDAO->getDetailedInvoiceArray($invoice);
            }

            $nbrOfSales = count($invoicesArray);

            $cashMovementsArray = array();
            foreach($ortherCashMovement as $cashMovement)
            {
                $cashMovementsArray [] = array(
                    "id" =>  $cashMovement->getIdMvtCaisse(),
                    "type" =>  $cashMovement->getType(),
                    "date" => $cashMovement->getDateMouvement()->format("d/m/Y H:i:s"),
                    "montant" => $cashMovement->getMontant(),
                    "motif" => $cashMovement->getMotif(),
                    "mode" =>  $cashMovement->getIdModePaiement()->getDesignation(),
                    "reference" => $cashMovement->getRefMouvement()
                );
            }

            if( $purpose === "pdf")
            {
                $linkPdf = $this->makeSalesLogPdf($cashSession, $invoicesArray, $cashMovementsArray, $salesProducts);
            }
           
            $result = array(
                "invoices" => $invoicesArray,
                "cashMovements" => $cashMovementsArray,
                "linkPdf" => $linkPdf,
                "nbrOfSales" => $nbrOfSales,
                "sellSessionTotal" => $sellSessionTotal,
                "salesProducts" => $salesProducts
            );
            
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
            exit();

        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }

        exit();
    }



    public function getdetailedinvoiceAction()
    {
        try 
        {
            $customerInvoiceDAO = new CustomerInvoiceDAO();
            
            $invoiceId = (isset($_POST["invoiceId"])) ? $_POST["invoiceId"] : null;

            $invoice = $customerInvoiceDAO->getCustomerInvoiceById($invoiceId);

            if(!$invoice)
            {
                echo ApiResponse::respond(101, "{$this->translator->_("sale.sell.unknowInvoice")}!");
                exit();
            }

            $result = $customerInvoiceDAO->getDetailedInvoiceArray($invoice);

            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
            exit();
        }
        catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }

        exit();
    }

    public function getpendinginvoicesAction()
    {
        try 
        {
            $cashSessionDAO = new CashSessionDAO();
            
            $customerInvoiceDAO = new CustomerInvoiceDAO();

            $pointOfSaleDAO = new PointOfSaleDAO();
            
            $userDAO = new UserDAO();
            
            $sessionId = (isset($_POST["sessionId"])) ? $_POST["sessionId"] : null;

            $pointOfSaleId = (isset($_POST["pointOfSaleId"])) ? $_POST["pointOfSaleId"] : null;

            $userId = (isset($_POST["userId"])) ? $_POST["userId"] : null;

            $isCashier = (isset($_POST["isCashier"])) ? (int) $_POST["isCashier"] : 0;

            $user = $userDAO->getUserById($userId);

            if(!$user)
            {
                echo ApiResponse::respond(101, "{$this->translator->_("sale.sell.unknowUser")}!");
                exit();
            }

            //point of sale
            $pointOfSale = $pointOfSaleDAO->getPointOfSaleById($pointOfSaleId);

            if(!$pointOfSale)
            {
                echo ApiResponse::respond(101, "{$this->translator->_("sale.sell.unknowPointOfSale")}!");
                exit();
            }
            
            //////
            $cashoutSession =  $cashSessionDAO->getCashSessionById($sessionId);

            if(!$cashoutSession) 
            {
                echo ApiResponse::respond(101, "{$this->translator->_("sale.sell.unknowCashoutSession")}!");
                exit();
            }

            if($isCashier)
            {
                //Get pending invoices for user Session
                $pendingInvoices = $customerInvoiceDAO->getSessionPendingInvoices($cashoutSession, true);

            }
            else{
                //Get pending invoices for user
                $pendingInvoices = $customerInvoiceDAO->getSessionPendingInvoices($cashoutSession, false);
            }

            $count = $pendingInvoices["count"];
            
            $invoices = $pendingInvoices["invoices"];

            $detailedPendingInvoices = []; 
            
           foreach( $invoices as $invoice)
           {
                $detailedPendingInvoices [] = $customerInvoiceDAO->getDetailedInvoiceArray($invoice);
           }

           $result = array(
                "count" => $count,
                "invoices" => $detailedPendingInvoices,
           );

           echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
           exit();

        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }

        exit();
    }

    public function getinvoicepdfAction()
    {
        try 
        {
            $customerInvoiceDAO = new CustomerInvoiceDAO();
            
            $invoiceId = (isset($_POST["invoiceId"])) ? $_POST["invoiceId"] : null;

            $invoice = $customerInvoiceDAO->getCustomerInvoiceById($invoiceId);

            if(!$invoice)
            {
                echo ApiResponse::respond(101, "{$this->translator->_("sale.sell.unknowInvoice")}!");
                exit();
            }
        
            $name = "bill_".$invoiceId;

            $fileLink = $this->generateinvoicemodel1($invoiceId, $name);

            $result = array(
                "link" => $fileLink
            );
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
            exit();
 
         } catch (Exception $ex) {
             echo ApiResponse::respond(101, $ex->getMessage());
         }
 
         exit();
     }

    public function getUserSellerSession($user)
    {
        $cashSessionDAO = new CashSessionDAO();

        $notClosedCashSessionList = $cashSessionDAO->getNotClosedUserSessionsByPointOfSale($user);
        
        $cashSession = array();

        $now = new \DateTime();

        foreach ($notClosedCashSessionList as $key => $cashSession) 
        {
            $sessionId = $cashSession->getIdSessionCaisse();
            $beginDate = $cashSession->getDateOuverture()->format("d-m-Y H:i:s");
            $cashierId = $cashSession->getIdUtilisateur()->getIdUtilisateur();
            $cashierName = $cashSession->getIdUtilisateur()->getIdPersonne()->getNom(). ' '.$cashSession->getIdUtilisateur()->getIdPersonne()->getPrenom();
            $closed = $cashSession->getFerme();
            $exceptedClosingDate = $cashSession->getDatePrevuCloture();

            if( $now <= $exceptedClosingDate )
            {
                $cashSession = array(
                    "id" => $sessionId,
                    "beginDate" => $beginDate,
                    "cashierId" => $cashierId,
                    "cashierName" => $cashierName,
                    "closed" => $closed
                );
                break;
            }
            else
            {
                //Forced closing
                $cashSession->setFerme(1);
                $cashSession->setCloturer(1);
                $cashSession->setDateCloture( $now );
                $cashSessionDAO->update($cashSession);
                unset($notClosedCashSessionList[$key]); 
                
            }
        }

        //Automatic opening session for cashier
        if(count($notClosedCashSessionList) === 0 )
        {
            $enDate = new DateTime("tomorrow - 1 min");
            $cashSession = $cashSessionDAO->create();
            $cashSession->setIdUtilisateur($user);
            $cashSession->setDateOuverture($now);
            $cashSession->setIdPointVente($user->getIdPointVente());
            $cashSession->setFerme(0);
            $cashSession->setCloturer(0);
            $cashSession->setDatePrevuCloture($enDate);
            $cashSession->setPwdCaisse($user->getPassword());
            $cashSessionDAO->save($cashSession);

            $cashSessionId = $cashSession->getIdSessionCaisse();
            $cashierId = $cashSession->getIdUtilisateur()->getIdUtilisateur();
            $cashierName = $cashSession->getIdUtilisateur()->getIdPersonne()->getNom(). ' '.$cashSession->getIdUtilisateur()->getIdPersonne()->getPrenom();
            $closed = $cashSession->getFerme();
            $beginDate = $now->format("d-m-Y H:i:s");

            $cashSession =  array(
                "id" => $cashSessionId,
                "beginDate" => $beginDate,
                "cashierId" => $cashierId,
                "cashierName" => $cashierName,
                "closed" => $closed
            );
        }

        return $cashSession;

    }

    public function generateinvoicemodel1 ($invoiceNumber, $fileName = null)
	{
		$structureName = NOM_STRUCTURE;
		$logo = LOGO_STRUCTURE;
		$structurePo = BP_STRUCTURE;
		$structurePhone = TEL_STRUCTURE;
		
		$customerInvoiceDAO = new CustomerInvoiceDAO();
		$customerInvoiceLineDAO = new CustomerInvoiceLineDAO();
        $invoicePaymentDAO = new InvoicePaymentDAO();

		$invoice = $customerInvoiceDAO->getCustomerInvoiceById($invoiceNumber);
		
        if(!$invoice)
        {
            return "";
        }

		
		$cashier = ($invoice->getIdSessionCaissier())? $invoice->getIdSessionCaissier()->getIdUtilisateur()->getLogin() : $invoice->getIdUtilisateur()->getLogin() ;
        $location = $invoice->getIdEmplacement() ?$invoice->getIdEmplacement()->getDesignation(): "";
		$dateTime = $invoice->getDateFacture()->format("d/m/Y H:i:s");
		$customer = ($invoice->getIdPersonne())? $invoice->getIdPersonne()->getNom(). " ". $invoice->getIdPersonne()->getPrenom():$invoice->getNomClient();
		$invoicePaymentsList = $invoicePaymentDAO->listPaymentsOfInvoice($invoice);
		$pointOfSale = $invoice->getIdPointVente();
        $pointOfSaleLocation = ($pointOfSale)? trim($pointOfSale->getLocalisation()) : trim(LOCALISATION);
		//$structurePhone = ($pointOfSale && $pointOfSale->getTelephone())? $pointOfSale->getTelephone() : trim($structurePhone);
        $bigWidthFormat = (TYPE_IMPRESSION == FORMAT_A4_FACTURE || TYPE_IMPRESSION == TYPE_IMPRESSION_NORMALE);
        $cssTable = "";
        $cssPageMargin = "";
		if( $bigWidthFormat )
		{	
            $cssTable = "
                        table { 
									width: 100%;  
									border-right: 0.1mm solid black;									
									border-left: 0.1mm solid black;									
									border-top: 0.1mm solid black;									
									border-bottom: 0.1mm solid black;									
									font-family: helvetica;
									border-collapse: collapse;
                            }
                            div {  
                                font-family: helvetica;
                             }";
			$cssPageMargin = "@page { 
									margin-top: 5px;
									margin-left : 5px;
									margin-right : 5px;
									margin-bottom : 5px;
							 }";
		}
		else
		{
			$cssTable = "table { 
									width: 100%;   
									font-family: Helvetica;
									border-collapse: collapse;
							}";

			$cssPageMargin = "@page { 
									margin-top: 5px;
									margin-left : 5px;
									margin-right : 5px;
									margin-bottom : 5px;
									line-height: 5mm;
                                    font-family: Helvetica;
							 }";
		}

        //QR code
        $qrCodeText = "http". '://'. $this->getRequest()->getHttpHost() . $this->view->baseUrl()."/documents/invoices/". $fileName;

        ob_start();
        
        $img = QRcode::png($qrCodeText, false, QR_ECLEVEL_L, 6, 2);

        $imageContent = ob_get_contents();
        ob_end_clean();

        header("Content-type: text/html");

        $qr_image = base64_encode($imageContent);
        $srcLink = str_replace(""," ",'data:image/jpeg;base64,'.$qr_image);
        //qr code end*/   

        $customerId = "000000";
        $customerAddress = "";
        $customerPhone = "";
        $customerCity = "";
        $customerEmail = "";

		if($invoice->getIdPersonne())
		{
            $customerId = $invoice->getIdPersonne()->getIdPersonne();
            $customerAddress = $invoice->getIdPersonne()->getAdresse();
            $customerPhone = $invoice->getIdPersonne()->getTelephone();
            $customerCity = $invoice->getIdPersonne()->getVille();
            $customerEmail =  $invoice->getIdPersonne()->getEmail();	
            $customerId = sprintf("%06d", $customerId);		
		}
		
		$invoiceLineList = $customerInvoiceLineDAO->getCustomerInvoiceLinesOfInvoice($invoice);
		
		$htmlInvoice = '<html>
                                <style>
                                    #footer { position: fixed; left: 0px; bottom: 0px; right: 0px; height: 45px; }
                                        
                                        ' . $cssPageMargin . '
                                        
                                        ' . $cssTable . '

                                        h2 { margin: 0; padding: 0; }

                                        .border th { 
                                                border-bottom: double;                                      
                                                font-weight: bold; 
                                        }

                                        .border td { 
                                                text-align: center; 
                                                color: #000000; 
                                        }
                                        
                                        .enteteFacture{
                                        text-justify: auto;
                                        text-align: center;
                                        }
                                </style>
                                <body>
                                <div>
                                <p class="enteteFacture">';

                                if(FACTURE_AFFICHER_LOGO_ET_NOM_STRUCTURE == "10" || FACTURE_AFFICHER_LOGO_ET_NOM_STRUCTURE == "11")
                                { 
                                    $logoStructure= LOGO_STRUCTURE;
            
                                    $cheminLogo = $_SERVER["DOCUMENT_ROOT"]. $this->view->baseUrl() . '/img/'.$logoStructure;
                                    if(file_exists($cheminLogo))
                                    {
                                        if( $bigWidthFormat )
                                        {
                                            $htmlInvoice .= "<div style='float:left; margin-left: 15px; margin-bottom: 5px;'><img src='". $cheminLogo . "' alt='Logo' width='150' height='100' /></div>";
                                        }
                                        else
                                        {
                                            $htmlInvoice .= '<div style="width: 100%;">
                                                                <div style="text-align: left; margin-left: 5px; width: 80%; float: left;"> 
                                                                    <img src="'. $cheminLogo . '" alt="Logo" width="100%" height="60" />
                                                                </div>
                                                            </div>';
                                        }
                                
                                    }
                                    else
                                    {
                                        if( $bigWidthFormat )
                                        {
                                            $htmlInvoice .= "<div style='float:left; margin-left: 15px; margin-bottom: 5px;'><span style='font-size: 16px; font-weight: bold;'>" . $structureName . "</span> </div>";
                                        }
                                        else
                                        {
                                            if(strlen($structureName)>21)
                                            {
                                                $fontSizeNomStructure = 18;
                                            }
                                            else{
                                                $fontSizeNomStructure = 20;
                                            }
                                            $htmlInvoice .= "<span style='font-size: ".$fontSizeNomStructure ."; font-weight: bold;'>" . $structureName . "</span> <br />";
                                        }
                                    }
                                }

                                if(FACTURE_AFFICHER_LOGO_ET_NOM_STRUCTURE == "01" || FACTURE_AFFICHER_LOGO_ET_NOM_STRUCTURE == "11")
                                {
                                    if( $bigWidthFormat )
                                    {
                                        $htmlInvoice .= "<div style='float:left; margin-left: 15px; margin-bottom: 5px;'><span style='font-size: 16px; font-weight: bold;'>" . $structureName . "</span> </div>";
                                    }
                                    else
                                    {
                                        if(strlen($structureName)>21)
                                        {
                                            $fontSizeNomStructure = 16;
                                        }
                                        else{
                                            $fontSizeNomStructure = 18;
                                        }
                                        $htmlInvoice .= "<div style='text-align: center; margin-left: 15px;  margin-top: 5px; margin-bottom: 5px;'><br /><span style='font-size: ".$fontSizeNomStructure."; font-weight: bold;'>" . $structureName . "</span> </div>";
                                    }
                                }

                                $bp_Tel = "";
                                $onlyBP =  "";
                                $onlyTel =  "";
                                if($structurePo != "")
                                {
                                    $bp_Tel .= "{$this->translator->_("sale.sell.poBox")}: " . $structurePo;
                                    $onlyBP .= "{$this->translator->_("sale.sell.poBox")}: " . $structurePo;
                                    
                                }

                                if($structurePhone != "")  //bon algo pas très bon, mais bon ça marche
                                {
                                    $bp_Tel .= " - {$this->translator->_("sale.sell.phone")}: " . $structurePhone;
                                    $onlyTel .= "{$this->translator->_("sale.sell.phone")}: " . $structurePhone;
                                }
                                if( $bigWidthFormat )
                                {
                                    $htmlInvoice .= " <div style='text-align: right;'>"; //ouverture div principal	
                                    $htmlInvoice .= " <div style='font-size: 20px ; margin-right: 20px; font-weight: bold;'>{$this->translator->_("sale.sell.invoice")} ".(($invoice->getEnAttente()==1)?" - {$this->translator->_("sale.sell.onHold")} ": "")." ".(($invoice->getTypeVente()==2 && $invoice->getValider() != 1) ?" - {$this->translator->_("sale.sell.inReservation")}": "")."<br />Réf./N°: " . $invoiceNumber . "  </div> ";
                                }
                                else
                                {
                                    $htmlInvoice .= " <div style='font-size: 12px; text-align: center; font-weight: bold;'>";
                                    if($pointOfSaleLocation != "")
                                    { 
                                        $htmlInvoice .= "<div style='margin-top: -5px; margin-bottom: 5px;'><span style='font-style: italic; font-size: 13px;'>".$pointOfSaleLocation."</span></div>";
                                    }
                                    if(trim(NUMERO_CONTRIBUABLE) != "")
                                    {
                                        $htmlInvoice .= "<span style='font-size: 12px;'>{$this->translator->_("sale.sell.uin")}: ".NUMERO_CONTRIBUABLE."</span><br/>";
                                    }
                                    
                                    if(trim(NUMERO_REGISTRE_COMMERCE) != "")
                                    {
                                        $htmlInvoice .= "<span style='font-size: 12px;'>{$this->translator->_("sale.sell.regCom")}: ".NUMERO_REGISTRE_COMMERCE."</span><br/>";
                                    }
                                    if($onlyBP != "")
                                    {
                                        $htmlInvoice .= " <span style='font-size: 12px;'>" . $onlyBP . "</span> <br />";
                                    }
                                    $htmlInvoice .= " <span style='font-size: 11px;'>" . $onlyTel . "</span> <br />";
                                    $htmlInvoice .= "</div> ";
                                }
									
							   
								if( $bigWidthFormat )
								{ 
                        
                                    $htmlInvoice .= "<div style='font-size: 10px; text-align: right; margin-right: 20px;'>{$this->translator->_("sale.sell.customerNum")}: " . $customerId . "</div>
                                                    <div style='font-size: 8px; text-align: right; margin-right: 20px; margin-bottom: 20px;' >Date: " . $dateTime . "</div>";
                                    $htmlInvoice .= " </div>";	//fermeture div principal
								}
								else
								{
								   $htmlInvoice .= '    <div style="width: 100%; text-align: center;">
                                                            <div style="text-align: left; float: left; width: 70%; margin-left: 15px;">
                                                                <span style="font-weight: bold;">' ."{$this->translator->_("sale.sell.invoice")}".' N° ' . $invoiceNumber . '</span>
                                                            </div>
                                                            <div style="text-align: right; margin-right: 3px;" >
                                                                <img width="30" height="30" style ="text-align: right;" src="'. $srcLink. '" alt="" />
                                                            </div>
                                                        </div>';
									
									$htmlInvoice .= "   <div style='text-align: left; font-size: 10px; margin-left: 15px; line-height: 15px;'>
										                    <span>{$this->translator->_("sale.sell.date")}: " . $dateTime . "</span> 
                                                            <span style='font-size: 11px;' align='right'> -- {$this->translator->_("sale.sell.cashier")}: " . $cashier . "</span> 
                                                        </div>
                                                        <div style='text-align: left; margin-left: 15px; line-height: 15px;'>
										                    <span style='text-align: left; font-size: 11px;'>{$this->translator->_("sale.sell.customer")}: " . $customer . " ".(($customerPhone)? "(".$customerPhone .")" :"")."</span> <span style='text-align: right; font-size: 10px;'>".($invoice->getServeur()? "  --- {$this->translator->_("sale.sell.serviceProvider")}: ".$invoice->getServeur()->getLogin():"")."</span>
                                                        </div>";
                                    if( $location )
                                    {
                                        $htmlInvoice .= "<div style='text-align: right; margin-right: 15px; line-height: 15px;'>
										                    <span style='text-align: right; font-size: 11px; font-weight: bold;'>{$this->translator->_("sale.sell.location")}: " . $location ."</span>
                                                         </div>";
                                    }
								}
								
                $htmlInvoice .= "   </p>  </div>";
                                    
                $ilYaRemise = false;
                foreach ($invoiceLineList as $line) 
                {
                    if($line->getForfaitRemise() > 0 ) 
                   { 
                        $ilYaRemise = true;
                        break;
                   }
                }

				//entete, uniquement pour le format A4
				if( $bigWidthFormat )
				{  
				
						$htmlInvoice .= "<table style='border: 0px; margin-top: 10px; margin-left: 10px; margin-right: 22px;'>";
							$htmlInvoice .= " <tr><td style='font-size: 11px; font-weight: bold; width: 55%;'>Emetteur:</td>
												   <td style='width: 3%;'></td>
												   <td style='font-size: 11px; font-weight: bold; width: 42%;'>Adressé à:</td> 
											  </tr>";
								$htmlInvoice .= " <tr><td style='font-size:11px;'>";
								$htmlInvoice .= "<div style='background-color: #b1b5b2;'>";
								$htmlInvoice .= "<div style='font-size: 11px; font-weight: bold;'> <span style='margin-left: 5px;'>" . $structureName . "</span></div>";
								$htmlInvoice .= "<div style='font-size: 10px;'> <span style='margin-left: 5px;'>".NUMERO_CONTRIBUABLE."</span></div>";
								$htmlInvoice .= "<div style='font-size: 8px;'> <span style='margin-left: 5px;'>{$this->translator->_("sale.sell.poBox")}: " . $structurePo. "</span></div> <br />";
								$htmlInvoice .= "<div style='font-size: 8px;'> <span style='margin-left: 5px;'>{$this->translator->_("sale.sell.phone")}: " . $structurePhone . "</span></div>";
								$htmlInvoice .= "<div style='font-size: 8px;'> <span style='margin-left: 5px;'>" . SITEWEB_STRUCTURE . "</span></div>";
								$htmlInvoice .= "<div style='font-size: 8px;'> <span style='margin-left: 5px;'>" . EMAIL_STRUCTURE . "</span></div>";
								$htmlInvoice .= "</div>";
							$htmlInvoice .= "	  </td>
												   <td></td>
												   <td style='font-size: 10px; border: 1px solid black;'>";
                            $htmlInvoice .= "      <span style='font-weight: bold;'>" . $customer . "</span>";	
                            if($customerCity != "")
                            {
                                $htmlInvoice .= "      <br /><span>" . $customerCity . "</span>";	
                            }
                            if($customerAddress != "")
                            {
                                $htmlInvoice .= "      <br /><span>" . $customerAddress . "</span>";	
                            }
                            if($customerPhone != "")
                            {
                                $htmlInvoice .= "      <br /><span>{$this->translator->_("sale.sell.phone")}: " . $customerPhone . "</span>";	
                            }
                           
                            if($customerEmail != "")
                            {
                                $htmlInvoice .= "      <br /><span>{$this->translator->_("sale.sell.email")}: " . $customerEmail . "</span>";	
                            }

							$htmlInvoice .= "	   </td> 
											  </tr>";
							$htmlInvoice .= " </table><br />";
								
				}

        $htmlInvoice .=    "<div style='margin-right: 15px; margin-left: 15px; text-align: center;'>";


            if($ilYaRemise)
             {
                $htmlInvoice .=    "<table class='table' style='font-size: 10px;'>
                                    <thead>
                                    <tr>
                                        <th class='border' style='width: 40%; text-align: left; border-bottom: 1px solid black;'>{$this->translator->_("sale.sell.product")}</th>
                                        <th class='border' style='width: 5%; text-align: center; border-bottom: 1px solid black;'>{$this->translator->_("sale.sell.qty")}</th>
                                        <th class='border' style='width: 17%; text-align: center; border-bottom: 1px solid black;'>{$this->translator->_("sale.sell.up")}</th>
                                        <th class='border' style='width: 15%; text-align: center; border-bottom: 1px solid black;'>{$this->translator->_("sale.sell.red")}</th>
                                        <th class='border' style='width: 23%; text-align: center; border-bottom: 1px solid black;'>{$this->translator->_("sale.sell.tp")}</th>
                                    </tr>
                                    </thead>";
             }
             else
             {
                 $htmlInvoice .=    "<table class='table' width='100%' style='font-size: 10px;'>
                                        <thead>
                                        <tr>
                                            <th class='border' style='width: 55%; text-align: left; border-bottom: 1px solid black;'>{$this->translator->_("sale.sell.product")}</th>
                                            <th class='border' style='width: 5%; text-align: center; border-bottom: 1px solid black;'>{$this->translator->_("sale.sell.qty")}</th>
                                            <th class='border' style='width: 18%; text-align: center; border-bottom: 1px solid black;'>{$this->translator->_("sale.sell.up")}</th>
                                            <th class='border' style='width: 22%; text-align: center; border-bottom: 1px solid black;'>{$this->translator->_("sale.sell.tp")}</th>
                                        </tr>
                                        </thead>";
             }
                 
             $htmlInvoice .= "<tbody>";
         
                    $total = 0; 
                    $totalNet = 0; 
                    $totalRemise = 0; 
                    $tauxTva = (float)$invoice->getTauxTva(); 
                    $tauxIr = (float)$invoice->getTauxIr(); 
					//hauteur d'un td ici
					$heightTd = "";
					if( $bigWidthFormat )
					{ 
						if(ORIENTATION_IMPRESSION == TYPE_ORIENTATION_PORTRAIT)
						{
							$dimensionReservee = 550;
							if(count($invoicePaymentsList)>3)
							{
								$dimensionReservee = 350;
								if(count($invoicePaymentsList)>5)
								{
									$dimensionReservee = 300;
								}
							}
						}
						else
						{
							$dimensionReservee = 200;
						}
                        $nombreLigne = count($invoiceLineList);  //+1

                        if($nombreLigne <= 0) 
                        {
                            $nombreLigne = 1;
                        }

                        $heightTd = "height: ".((int)$dimensionReservee - 30*$nombreLigne)."px;"; 
					}
                    $nbreProduit = 0;
                    foreach ($invoiceLineList as $uneLigne) 
                    {
                        $prixVente = $uneLigne->getQuantite() * $uneLigne->getPrixUnitaire();
                        $total += ceil ($prixVente);
                        $totalNet += $uneLigne->getQuantite() * ($uneLigne->getPrixUnitaire() + $uneLigne->getForfaitRemise());
                        $totalRemise += $uneLigne->getQuantite() * $uneLigne->getForfaitRemise();
                        $qte = (int)$uneLigne->getQuantite() ;
						$nom = $uneLigne->getDesignation();
						$fontSizeNom = ""; //pour les longues chaines

                        if($uneLigne->getIdProduit()->getStockable() == 1)
                        {
                            $nbreProduit += $qte;
                        }

						if( $bigWidthFormat )
						{
                            if(strlen($nom)>35)
                            {
                                $fontSizeNom = "font-size: 12px;";	
                                if(strlen($nom)>55)
                                {
                                    $fontSizeNom = "font-size: 11px;";	
                                    if(strlen($nom)>65)
                                    {
                                        $fontSizeNom = "font-size: 8px;";	
                                    }
                                }
                            }
						    //
							$description = trim($uneLigne->getIdProduit()->getDescription());
							if($description != "")
							{
								$tabDescription = explode("\n", $description);
								$nom .= "<div style='font-size: 9px; line-height: 9px; padding-left: 5px;'>";
								if(is_array($tabDescription) && count($tabDescription)>0)
								{
									$nbLine = 0;
									foreach($tabDescription as $uneDescription)
									{
										$nbLine++;
										
										if($nbLine > 1)
											$nom .= "<br />";
										
										if($nbLine > 3)
										{
											$nom .= "...";	
											break; //maximum 3 lignes, raison de lisibilité
										}
											
										$nom .= $uneDescription;	
									}
								}
								else
								{
									$nom .= $description;
								}
								
								$nom .= "</div";
							}
						}
                        else{
                            $bordureElement = " line-height: 15px;"; //10
                            $fontSizeNom = "font-size: 12px;"; //11
                            if(strlen($nom)>35)
                            {
                                $fontSizeNom = "font-size: 10px;";	 //9
                                if(strlen($nom)>55)
                                {
                                    $fontSizeNom = "font-size: 9px;";	//8
                                    if(strlen($nom)>65)
                                    {
                                        $fontSizeNom = "font-size: 8px;";	//7
                                    }
                                }
                            }
                            //

                        }
                        
                        $montantLigne = $uneLigne->getPrixUnitaire()+$uneLigne->getForfaitRemise(); 
						$htmlInvoice .= "<tr style='".$bordureElement."'>
                                            <td style='".$fontSizeNom." ".$bordureElement." font-weight: bold;'>".$nom."</td>
                                            <td style='text-align: right; font-weight: bold;".$bordureElement."'>".$qte."</td>
                                            <td style='text-align: right; ".$bordureElement." font-weight: bold;'>".number_format($montantLigne, 0 , '' , ' ')."</td>";
                    if($ilYaRemise){
                        $htmlInvoice .= "   <td style='text-align: right; font-size: 11px; ".$bordureElement."'>".number_format($uneLigne->getForfaitRemise() , 0 , '' , ' ')."</td>";
                    }
                         $htmlInvoice .= "<td style='text-align: right; font-weight: bold;".$bordureElement."'>".number_format($prixVente , 0 , '' , ' ')."</td>
                                       </tr>";
                    }
                    $montantVerse = $invoice->getMontantVerse();

                    
                    if( $bigWidthFormat )
					{
                        $htmlInvoice .= "<tr><td  style='". $heightTd ."px; ".$bordureElement."'></td><td style='".$bordureElement."'></td><td style='".$bordureElement."'></td>".(($ilYaRemise)?"<td style='".$bordureElement."'></td>":"")."<td style='".$bordureElement."'></td></tr>";
                    }

                    $htmlInvoice .= "</tbody>
                                    </table>";

                                    

                    if($bigWidthFormat)
                    {
                        $widthIntule = "230px";
                        $widthMontant = "60px";
                    }
                    else{
                        $widthIntule = "50%";
                        $widthMontant = "50%";
                    }
                      
                    if($bigWidthFormat)
                    {
                        $htmlInvoice .=    '<table border="0" style="border: 0; line-height: 15px;" align="right">'; 
                    }
                    else{
                        $htmlInvoice .=    '<table border="0" style="border: 0; line-height: 20px;" align="right">'; 
                    } 
                                  
                    $htmlInvoice .=  '<tbody>';  

                    if($totalRemise > 0) 
                    {
                         $htmlInvoice .=  "<tr>";   
                         $htmlInvoice .=  " <td colspan=3 style='width: ".$widthIntule."; font-size: 13px; font-weight: bold;' align='right'> {$this->translator->_("sale.sell.invoiceTotal")} </td> 
                                            <td style='width: ".$widthMontant."; font-size: 13px; font-weight: bold;' align='right'>".number_format($totalNet, 0 , '' , ' ')." {$this->translator->_("sale.sell.xaf")}</td>";
                         $htmlInvoice .=  "</tr>"; 
                         $htmlInvoice .=  "<tr>"; 
                         $htmlInvoice .=  "  <td colspan=3 style='font-size: 13px; font-weight: bold;' align='right'> {$this->translator->_("sale.sell.discountAmount")} </td> 
                                             <td style='font-size: 13px; font-weight: bold;' align='right'>".number_format($totalRemise, 0 , '' , ' ')." {$this->translator->_("sale.sell.xaf")}</td> ";
                        $htmlInvoice .=  "</tr>"; 
                    }
                    
                    if($tauxIr > 0 || $tauxTva > 0) {
                        $htmlInvoice .=  "<tr>"; 
                        $htmlInvoice .= "   <td colspan=3 style='width: ".$widthIntule."; font-size: 11px;' align='right'> {$this->translator->_("sale.sell.preTaxAmount")}</td> 
                                            <td style='width: ".$widthMontant."; font-size: 11px;' align='right'>".number_format($total, 0 , '' , ' ')." {$this->translator->_("sale.sell.xaf")}</td>";
                        $htmlInvoice .=  "</tr>"; 
                    }
                   

                    if($tauxIr > 0) 
                    {
                        $htmlInvoice .=  "<tr>";  
                        $htmlInvoice .= "   <td colspan=3 style='font-size: 11px;' align='right'> {$this->translator->_("sale.sell.itAmount")} (".$tauxIr."%):</td> 
                                            <td style='font-size: 11px;' align='right'>".number_format($total*$tauxIr/100, 0 , '' , ' ')." {$this->translator->_("sale.sell.xaf")}</td>";
                        $htmlInvoice .=  "</tr>"; 

                    }
					if($tauxTva > 0) 
                    {
                        
                        $htmlInvoice .=  "<tr>";  
                        $htmlInvoice .= "   <td colspan=3 style='font-size: 11px;' align='right'>{$this->translator->_("sale.sell.vatAmount")} (".$tauxTva."%):</td> 
                                            <td style='font-size: 11px' align='right'>".number_format($total*$tauxTva/100, 0 , '' , ' ')." {$this->translator->_("sale.sell.xaf")}</td>";
                        $htmlInvoice .=  "</tr>"; 
                        
                        if( $invoice->getTvaRetenu() != 1) //tva non retenu
						{
                            $total = $total + $total*$tauxTva/100;
                        }
                    }

                    
                    if($tauxIr > 0 || $tauxTva > 0) {
                        $htmlInvoice .=  "<tr>"; 
                        $htmlInvoice .= "   <td colspan=3  style='font-size: 11px;' align='right'>{$this->translator->_("sale.sell.includingTaxAmount")}</td> 
                                            <td style='font-size: 11px;' align='right'>".number_format($invoice->getMontantTtc(), 0 , '' , ' ')." {$this->translator->_("sale.sell.xaf")}</td>";
                        $htmlInvoice .=  "</tr>"; 
                    }
                    
                   
					if($nbreProduit > 0)
					{
						 $htmlInvoice .= "<tr> 
											 <td colspan='4' style='font-size: 12px;' align='right'>({$this->translator->_("sale.sell.numberOfproducts")}: <b>". $nbreProduit . "</b>)</td>
										 </tr>";
					 }

					if($invoice->getForfaitRemise() > 0){
						
						$totalRemise = $invoice->getForfaitRemise();
						$pctge = $invoice->getPctRemise();


						$total -= $totalRemise ;

						$htmlInvoice .=  "<tr>";  
						$htmlInvoice .= "   <td colspan=3 style='font-weight: bold; font-size: 11px; ' align='right'>{$this->translator->_("sale.sell.invoiceAmount")} </td> 
											<td style='font-weight: bold; font-size: 12px; ' align='right'>".number_format($invoice->getMontantHt(), 0 , '' , ' ')." {$this->translator->_("sale.sell.xaf")}</td>";
						$htmlInvoice .=  "</tr>";

						$htmlInvoice .=  "<tr>";  
						$htmlInvoice .= "   <td colspan=3 style='font-weight: bold; font-size: 11px; ' align='right'>{$this->translator->_("sale.sell.discount")} (".$pctge."%): </td> 
											<td style='font-weight: bold; font-size: 12px; ' align='right'>".number_format($totalRemise, 0 , '' , ' ')." {$this->translator->_("sale.sell.xaf")}</td>";
						$htmlInvoice .=  "</tr>"; 
					}

				$htmlInvoice .=  "<tr>";  
				$htmlInvoice .= "   <td colspan=3 style='font-weight: bold; font-size: 13px; ' align='right'>{$this->translator->_("sale.sell.totalToPay")} </td> 
									<td style='font-weight: bold; font-size: 14px; ' align='right'>".number_format($invoice->getMontantNet(), 0 , '' , ' ')." {$this->translator->_("sale.sell.xaf")}</td>";
				$htmlInvoice .=  "</tr>"; 

                $paye = $invoicePaymentDAO->getSumOfInvoicePayments($invoice);

            if( $paye > 0 or $paye == $total )
            {
                $reste = (int) ($total - $paye);

                if( $bigWidthFormat )
                { 
                    $fontSizeMt = "font-size: 14px;";
                    if($reste > 9999999 || $paye > 9999999)
                    {
                        $fontSizeMt = "font-size: 12px;"; 
                    }
                    $htmlInvoice .=    "<tr>
                                                <td colspan='2' align='center'><span style = 'font-size: 13px; font-weight: bold; text-align: center;'>". ((count($invoicePaymentsList) > 1) ? "{$this->translator->_("sale.sell.xaf")}": '') ."</span></td>
                                                <td align='right'>{$this->translator->_("sale.sell.xaf")}:</td>
                                                <td align='right' style='".$fontSizeMt."'>".number_format($paye , 0 , '' , ' ')." {$this->translator->_("sale.sell.xaf")}" . " </td>
                                            </tr>
                                            <tr> 
                                                <td width='20%' style='font-size: 11px; font-weight: bold;'>". ((count($invoicePaymentsList) > 1) ? "{$this->translator->_("sale.sell.date")}": '') ."</td>
                                                <td width='20%' style='font-size: 11px; font-weight: bold;' align='right'>". ((count($invoicePaymentsList) > 1) ? "{$this->translator->_("sale.sell.amount")}": '') ."</td>
                                                <td align='right'><span style = 'font-weight: bold;'>{$this->translator->_("sale.sell.toRepay")}</span></td>
                                                <th align='right' width='50%'  style='".$fontSizeMt." background-color: #b1b5b2;'>".number_format($reste, 0 , '' , ' ')." {$this->translator->_("sale.sell.xaf")}</th>
                                            </tr>";
                }
                else
                { 
                        $modePaiementText = "";

                        if( is_array($invoicePaymentsList) && count($invoicePaymentsList) == 1  && isset($invoicePaymentsList[0]) && is_object($invoicePaymentsList[0]) )
                        {						
                            $modePaiementText = "(" . $invoicePaymentsList[0]->getIdModePaiement()->getDesignation() .")";
                        }
                        
                        $aRembourser =( ($montantVerse - $total) >= 0 )? $montantVerse - $total : 0;
                        
                        $htmlInvoice .=    "<tr>
                                                <td colspan='3' style = 'font-size: 12px; font-weight: bold;' align='right'>{$this->translator->_("sale.sell.paid")} " . $modePaiementText ."</td>
                                                <td align='right'>".number_format($montantVerse , 0 , '' , ' ')." {$this->translator->_("sale.sell.xaf")} </td>
                                            </tr>
                                            <tr> 
                                                <td colspan='3' style = 'font-size: 12px; font-weight: bold;' align='right'>{$this->translator->_("sale.sell.repay")}  </td>
                                                <td align='right'>".number_format($aRembourser, 0 , '' , ' ')." {$this->translator->_("sale.sell.xaf")}</td>
                                            </tr>";

                        if(count($invoicePaymentsList) > 1)
                        {
                            $htmlInvoice .= "<tr><td colspan='4' style='font-size: 10px;' align='center'><u>{$this->translator->_("sale.sell.paymentSummary")} </u></td></tr>";
                            foreach ( $invoicePaymentsList as $paiementFacture )
                            {
                                $montantPaie = $paiementFacture->getMontant();
                                $modePaiementText = $paiementFacture->getIdModePaiement()->getDesignation();
                                $htmlInvoice .= "<tr><td colspan=3 style='font-size: 9px;' align='left'>".$modePaiementText ."</td><td style='font-size: 9px;' align='right' >". number_format($montantPaie, 0 , '' , ' '). " {$this->translator->_("sale.sell.xaf")}</td></tr>";
                            }
                        }
                    
                        
                }
            }
            else{
                /// QR code for mobile pay
                if(OM_PAYMENT_CODE_TEMPLATE && MOMO_PAYMENT_CODE_TEMPLATE)
                {
                    //Orange Money (OM)
                    $qrCodeOM = str_replace("{{AMOUNT}}", $invoice->getMontantNet(), OM_PAYMENT_CODE_TEMPLATE);

                    ob_start();
                    
                    QRcode::png($qrCodeOM, false, QR_ECLEVEL_L, 6, 2);

                    $imageContent = ob_get_contents();
                    ob_end_clean();

                    header("Content-type: text/html");

                    $qrImgOM = base64_encode($imageContent);
                    $srcOM = str_replace(""," ",'data:image/jpeg;base64,'.$qrImgOM);

                    //MTN Mobile money (MOMO)
                    $qrCodeMOMO = str_replace("{{AMOUNT}}", $invoice->getMontantNet(), MOMO_PAYMENT_CODE_TEMPLATE);
                    $qrCodeMOMO = str_replace("{{REFERENCE}}", $invoiceNumber, $qrCodeMOMO);
                    ob_start();
                    
                    QRcode::png($qrCodeMOMO, false, QR_ECLEVEL_L, 6, 2);

                    $imageContent = ob_get_contents();
                    ob_end_clean();

                    header("Content-type: text/html");

                    $qrImgMOMO = base64_encode($imageContent);
                    $srcMOMO = str_replace(""," ",'data:image/jpeg;base64,'.$qrImgMOMO);
                    
                    $htmlInvoice .= '<tr style="font-size: 10px;">
                                        <td colspan="2" align="center" width="45%">
                                            <img width="40" height="40" src="'. $srcOM. '" alt="" />
                                            <br />OM
                                        </td>
                                        <td colspan="2" align="center" width="45%">
                                            <img width="40" height="40" src="'. $srcMOMO. '" alt="" />
                                            <br />MOMO
                                        </td>
                                    </tr>';
                }
                //qr code end*/  
            }


            if( $bigWidthFormat )
            { 
           
                if(count($invoicePaymentsList) > 1)
                {
                    foreach ( $invoicePaymentsList as $paiementFacture ){
                    $datePaie = $paiementFacture->getDatePaiement()->format("d/m/Y");
                    $montantPaie = number_format($paiementFacture->getMontant(), 0 , '' , ' '). " {$this->translator->_("sale.sell.xaf")}";
                    $htmlInvoice .= "<tr><td style='font-size: 12px;' align='left'>".$datePaie."</td><td style='font-size: 12px;' align='right' >".$montantPaie."</td><td colspan='2'> </td></tr>";
                    }
                }

            }
            if( $bigWidthFormat )
            {
                ///en bas
            }
            else
            {
                $htmlInvoice .=" <tr><td colspan='4' style='font-size: 8px; line-height: 10px; font-style: italic; font-weight: bold;' align='center'>". PHRASE_ARTICLE_VENDU ."</td> </tr>";
                					//<tr><td colspan='4' style='font-size: ".$fontSize10."; font-weight: bold; line-height: 10px;' align='center'>". PHRASE_BAS_PAGE ."</td> </tr> ";
				$htmlInvoice .= "</tbody></table>";

            }

            $htmlInvoice .= "</tbody></table>"; 
					
			 if( $bigWidthFormat )
			 {
                 $htmlInvoice .=" 
                                <div id='footer'>   
                                    <div align='left'> <span style='font-size: 8px; text-align: left;'>". PHRASE_ARTICLE_VENDU ."</span><hr /> </div>
                                    <span style='font-size: 7px; text-align: left;'>". PHRASE_BAS_PAGE ."</span> 
                                </div> ";
                
			 } 

             if( $bigWidthFormat )
             {
                 
             }
             else
             {
                $htmlInvoice .=" 
                <div id='footer'>   
                </div> ";
 
             }

             $htmlInvoice .= "</body></html>";

            $options = new \Dompdf\Options();
            $options->set('isRemoteEnabled', TRUE);
            $dompdf = new \Dompdf\Dompdf($options);
            $dompdf->set_paper(TYPE_IMPRESSION, ORIENTATION_IMPRESSION);
            $dompdf->load_html($htmlInvoice);
            $dompdf->set_base_path($_SERVER['DOCUMENT_ROOT']);
            $dompdf->render();

            $output = $dompdf->output();

            $nomCompletFichierToSave = PUBLIC_PATH . "/documents/invoices/" . $fileName . ".pdf";
            
            file_put_contents($nomCompletFichierToSave, $output);

            $link = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS']== 'on' ? "https" : "http") .
			
                    '://'. $this->getRequest()->getHttpHost() . $this->view->baseUrl(). "/documents/invoices/" . $fileName . ".pdf";


		return $link;
	}

    public function makeSalesLogPdf($cashSession, $invoices, $cashMovements, $salesProducts){
     
            $customerInvoiceLineDAO = new CustomerInvoiceLineDAO();
            $cashMovementDAO = new CashMovementDAO();

            $structureName = NOM_STRUCTURE;
            $structurePoBox = BP_STRUCTURE;
            $dateCloture = $cashSession->getDateCloture()? $cashSession->getDateCloture()->format("d/m/Y H:i:s") : " ";
            $dateDebut = $cashSession->getDateOuverture()->format("d/m/Y H:i:s");
            $cashier =  $cashSession->getIdUtilisateur()->getIdPersonne()->getNom() . " " . $cashSession->getIdUtilisateur()->getIdPersonne()->getPrenom();
            $cashierId =  $cashSession->getIdUtilisateur()->getIdUtilisateur();
            $totalSomme = 0;
            $totalSommeDeposit = 0;
            $totalSommeReduction = 0;
            $totalEncaisse = 0;
            $entete = "<h2 style='text-align: center'> {$this->translator->_("sale.sell.salesSessionLog")} </h2> <br/> ";
            $entete .= " {$this->translator->_("sale.sell.cashier")} : " . $cashier . "<br/>"
                    . "  {$this->translator->_("sale.sell.cashSessionNumber")} : " .$cashSession->getIdSessionCaisse() . "<br/>" ;
            $entete .= " {$this->translator->_("sale.sell.beginDate")} : " . $dateDebut . "<br/>";
            $entete .= " {$this->translator->_("sale.sell.endDate")} : " . $dateCloture . "";
            $entete .= "";
            $chaineBody = "
                <html>
                    <body>
                        <style>
                            @page { 
                                margin-top: 0px;
                                margin-left : 4px;
                                margin-right : 4px;
                                margin-bottom : 0px;
                                font-family: helvetica; 
                            }
                            body { 
                                margin-top: 0px;
                                margin-left : 4px;
                                margin-right : 4px;
                                margin-bottom : 0px;
                            }
                            #footer { position: fixed; left: 0px; bottom: 0px; right: 0px; height: 0px; }

                            table { 
                                width: 100%; 
                                color: #717375; 
                                font-family: helvetica; 
                                line-height: 4mm;
                                font-size: 10px;
                                border-collapse: collapse;
                            }

                            h2 { margin: 0; padding: 0; }

                            p { margin: 0px; font-size: 10px;}

                            .boxtop {
                                margin-bottom: 30px;
                            }

                            .border th { 
                                border: 1px solid #000;  
                                color: #000000;                                        
                                font-weight: bold; 
                                font-size: 10px; 
                                text-align: center; 
                            }
                            
                            .f { 
                                
                                color: #000000;                                        
                                font-weight: normal; 
                                font-family: arial;
                                font-size: 14px; 
                                text-align: center; 
                            }
                            
                            .bordertop{
                                border-top : 1px solid #000;
                            }
                            
                            .borderbottom{
                                border-bottom : 1px solid #000;
                            }

                            .border td { 
                                text-align: center; 
                                border: 1px solid #000;  
                                color: #000000; 
                            }

                            .title {
                                font-size: 14px;
                                font-weight: bold;
                                text-transform: uppercase;
                            }

                            .p2p { width: 2%; } .p10p { width: 10%; } .p15p { width: 15%; } 
                            .p20p { width: 20%; } .p55p { width: 55%; } 
                            .p25p { width: 25%; } .p44p { width: 44%; } 
                            .p50p { width: 50%; } .p54p { width: 54%; } 
                            .p60p { width: 60%; } .p75p { width: 75%; }
                        </style>

                                <div>                                        
                                    <p>
                                            <h2>" . $structureName . "</h2>
                                            <h4>{$this->translator->_("sale.sell.poBox")}: " . $structurePoBox . "</h3>                                   
                                            <h6 style='text-align: left'>" . $entete . "</h6>
                                    </p> 
                                    <hr />
                                </div>
                                    <table id='contenttable' class='table table-bordered table-responsive border' width='100%'>
                                        <thead>
                                        <tr>
                                            <th>#</th>
                                            <th>{$this->translator->_("sale.sell.date")}</th>
                                            <th>{$this->translator->_("sale.sell.amount")}</th>
                                            <th>{$this->translator->_("sale.sell.discountAmount")}</th>
                                            <th>{$this->translator->_("sale.sell.amountPaid")}</th>
                                        </tr>
                                        </thead>
                                        <tbody id='contentbody'>";
                $dernierVendeur = null;
                $sommeVendeur = 0;
                $sommeRegle = 0;
                $sommeReduction = 0;
                foreach ($invoices as $invoiceArray) {
                    if ($invoiceArray["checkoutSession"]["cashierId"] != $dernierVendeur) {
                        if ($dernierVendeur) {
                            $chaineBody .= "<tr class='bordertop'><td style=' font-weight : bold; text-align:right;' colspan='2'>Total " .$dernierVendeur->getIdPersonne()->getPrenom() ."</td><td style='font-weight:bold; text-align:right;'>" . number_format($sommeVendeur, 0, '', ' ') . "</td><td style='text-align : right; font-weight : bold;'>" . number_format($sommeReduction, 0, '', ' ') . "</td><td style='text-align : right; font-weight : bold;'>" . number_format($sommeRegle, 0, '', ' ') . "</td><td style='text-align : right; font-weight : bold;'>" . number_format($sommeDeposit, 0, '', ' ') . "</td></tr>";
                        }
    //                            $chaineBody .= "<tr><td style=' text-align : left; font-weight : bold;' colspan='4'>Vendeur : " . $vente->getIdVendeur()->getIdPersonne()->getNom() . " " . $vente->getIdVendeur()->getIdPersonne()->getPrenom() . "</td></tr>";
                        $sommeVendeur = 0;
                        $sommeRegle = 0; 
                        $sommeReduction = 0;
                        $dernierVendeur =$invoiceArray["checkoutSession"]["cashierId"];
                    }

                    $paye = $invoiceArray["paid"]["totalPaid"] - $invoiceArray["refunded"]["totalRefunded"];
                    $invoiceNumber = $invoiceArray ["invoiceNumber"];
                    $billingDate = $invoiceArray ["billingDate"];
                    $netAmount =  $invoiceArray ["totals"]["netAmount"]["amount"];
                    $discountAmount = $invoiceArray ["totals"]["discountsAmount"]["amount"];
                   
                    $chaineBody .= "<tr><td>" . $invoiceNumber. "</td><td>" . $billingDate . "</td><td style='text-align : right;'>" . number_format($netAmount, 0, '', ' ') . "</td><td style='text-align : right;'>" . number_format($discountAmount, 0, '', ' ') . "</td><td style='text-align : right;'>" . number_format($paye, 0, '', ' ') . "</td></tr>";
                    $sommeVendeur += $netAmount;
                    $sommeRegle += $paye;
                    $sommeReduction  += $discountAmount;
                    ///
                    $totalSomme += $netAmount +  $discountAmount;
                    $totalEncaisse += $paye;
                    $totalSommeReduction += $netAmount; 
                }
                if ($dernierVendeur) {
                    $chaineBody .= "<tr class='bordertop'><td style=' font-weight : bold; text-align:right;' colspan='2'>Total " . $invoiceArray["checkoutSession"]["cashierName"] ."</td><td style='font-weight:bold; text-align:right;'>" . number_format($sommeVendeur, 0, '', ' ') . "</td><td style='text-align : right; font-weight : bold;'>" . number_format($sommeReduction, 0, '', ' ') . "</td><td style='text-align : right; font-weight : bold;'>" . number_format($sommeRegle, 0, '', ' ') . "</td></tr>";
                }
                    $chaineBody .= "<tr class='bordertop'><td style=' font-weight : bold; text-align:right;' colspan='2'>{$this->translator->_("sale.sell.summaryTotals")}</td><td style='font-weight:bold; text-align:right;'>" . number_format($totalSomme, 0, '', ' ') . "</td><td style='text-align : right; font-weight : bold;'>" . number_format($totalSommeReduction, 0, '', ' ') . "</td><td style='text-align : right; font-weight : bold;'>" . number_format($totalEncaisse, 0, '', ' ') . "</td></tr>";

            $chaineBody .= "
                                        </tbody>
                                    </table>
                        <div>
                            <h6><center><u>{$this->translator->_("sale.sell.salesSummary")}</u><center></h6>                             
                        </div>
                ";
                
                
            
            $chaineBody .= "<table class='table table-bordered table-responsive border' width='100%'>
                                            <tr>
                                                <th>{$this->translator->_("sale.sell.designation")}</th>
                                                <th>{$this->translator->_("sale.sell.qty")}</th>
                                                <th>{$this->translator->_("sale.sell.amount")}</th>
                                            </tr>
                                            <tbody id='contentbody'>";
            
            
            
            $dernierPointVente = null;
            $sommeTotal = 0;
            $qteMagasin = 0;
            $sommePointVente = 0;
            ///
            $derniereCategorie = null;
            $sommeCategorie = 0;
            $sommeQteCategorie = 0;
            ///
            foreach ($salesProducts as $ligneVente) {
                
                if ($ligneVente['idPointVente'] != $dernierPointVente) {
                    if ($derniereCategorie) {
                        $chaineBody .= "<tr class='bordertop'><td style='font-weight : bold; text-align:right;'>{$this->translator->_("sale.sell.totalCategory")}</td><td style='font-weight:bold; text-align:right;'>" . number_format($sommeQteCategorie, 0, '', ' ') . "</td><td style='font-weight:bold; text-align:right;'>" . number_format($sommeCategorie, 0, '', ' ') . "</td></tr>";
                    }
                    if ($dernierPointVente) {
                        $chaineBody .= "<tr class='bordertop'><td colspan ='2' style='font-weight : bold; text-align:right;'>{$this->translator->_("sale.sell.totalPointOfSale")}</td><td style='font-weight:bold; text-align:right;'>" . number_format($sommePointVente, 0, '', ' ') . "</td></tr>";
                    }

                    $chaineBody .= "<tr><td colspan='3' style=' text-align : left; font-weight : bold;'>{$this->translator->_("sale.sell.pointOfSale")} : " . $ligneVente['nomPointVente'] . "</td>";
                    $sommePointVente = 0;
                    $derniereCategorie = null;
                    $sommeCategorie = 0;
                    $sommeQteCategorie = 0;
                    $dernierPointVente = $ligneVente['idPointVente'];
                }
                
                if ($ligneVente['idCategorie'] != $derniereCategorie) {
                    if ($derniereCategorie) {
                        $chaineBody .= "<tr class='bordertop'><td style='font-weight : bold; text-align:right;'>{$this->translator->_("sale.sell.totalCategory")}</td><td style='font-weight:bold; text-align:right;'>" . number_format($sommeQteCategorie, 0, '', ' ') . "</td><td style='font-weight:bold; text-align:right;'>" . number_format($sommeCategorie, 0, '', ' ') . "</td></tr>";
                    }
                    $chaineBody .= "<tr><td colspan='3' style=' text-align : left; font-weight : bold;'>" .$ligneVente['nomCategorie']  . "</td>";
                    $sommeCategorie = 0;
                    $sommeQteCategorie = 0;
                    $derniereCategorie = $ligneVente['idCategorie'];
                }
                
                $chaineBody .= "<tr><td style='text-align : left;'>" . $ligneVente['designation']  . "</td><td style='text-align : right;'>" . number_format($ligneVente['quantite'] , 0, '', ' ') . "</td><td style='text-align : right;'>" . number_format($ligneVente['montant'] , 0, '', ' ') . "</td></tr>";
                $sommePointVente += $ligneVente['montant'] ;
                $sommeTotal += $ligneVente['montant'];
                $sommeCategorie += $ligneVente['montant'] ;
                $sommeQteCategorie += $ligneVente['quantite'];
            }
            if ($derniereCategorie) {
                $chaineBody .= "<tr class='bordertop'><td style='font-weight : bold; text-align:right;'>{$this->translator->_("sale.sell.totalCategory")}</td><td style='font-weight:bold; text-align:right;'>" . number_format($sommeQteCategorie, 0, '', ' ') . "</td><td style='font-weight:bold; text-align:right;'>" . number_format($sommeCategorie, 0, '', ' ') . "</td></tr>";
            }
            if ($dernierPointVente) {
                $chaineBody .= "<tr class='bordertop'><td colspan ='2' style='font-weight : bold; text-align:right;'>{$this->translator->_("sale.sell.totalPointOfSale")}</td><td style='font-weight:bold; text-align:right;'>" . number_format($sommePointVente, 0, '', ' ') . "</td></tr>";
            }
            
            
                $chaineBody .= "<tr class='bordertop'><td colspan ='2' style='font-weight : bold; text-align:right;'>{$this->translator->_("sale.sell.grandTotal")}</td><td style='font-weight:bold; text-align:right;'>" . number_format($sommeTotal, 0, '', ' ') . "</td></tr>";
            
                
                //////////
                $chaineBody .= "     </tbody></table>";


                $chaineBody .= "<div>
                                <h6><center><u>{$this->translator->_("sale.sell.exceptionalCheckInOut")}</u><center></h6>                             
                            </div>";
                $chaineBody .= "<table class='table table-bordered table-responsive border' width='100%'>
                                            <tr>
                                                <th>{$this->translator->_("sale.sell.date")}</th>
                                                <th>{$this->translator->_("sale.sell.enteredAmount")}</th>
                                                <th>{$this->translator->_("sale.sell.takenOutAmount")}</th>
                                                <th>{$this->translator->_("sale.sell.raison")}</th>
                                            </tr>
                                            <tbody>";
                                 
                $totalEntree = 0;
                $totalSortie = 0;
                foreach($cashMovements as $unMvt)
                {
                    $chaineBody .= "<tr>
                                        <td>".$unMvt["date"] ."</td>";

                    if($unMvt["type"] == "Entree")
                    {
                        $totalEntree += $unMvt["montant"];
                        $chaineBody .= "<td align='right'>" . number_format($unMvt["montant"], 0, '', ' ') . "</td>
                                        <td align='right'>0</td>";

                    }
                    else{
                        $totalSortie += $unMvt["montant"];
                        $chaineBody .= "<td align='right'>0</td>
                                        <td align='right'>" . number_format($unMvt["montant"], 0, '', ' ') . "</td>";
                    }
                    $chaineBody .= "    <td>". $unMvt["motif"] . "</td>
                                    </tr>";
                
                }

                $chaineBody .= "<tr>
                                <th>Total</th>
                                <th align='right'>" . number_format($totalEntree, 0, '', ' ') . "</th>
                                <th align='right'>" . number_format($totalSortie, 0, '', ' ') . "</th>
                                <th></th>
                                </tr>";
            
                $chaineBody .= "     </tbody></table> ";

            //affichage du montant globlale en caisse
            $cashMovements = $cashMovementDAO->getSessionCashMovementByPaymentMethod($cashSession );

            $chaineBody .= "<br /><table class='table table-bordered table-responsive border' width='100%'>
                            <caption><b>{$this->translator->_("sale.sell.sessionSummary")}</b> </caption>";

            foreach( $cashMovements as $cashMvt)
            {
                $paymentMethod = $cashMvt["modePaiement"];
                $amount = $cashMvt["montant"];
                $chaineBody .= "<tr><td style=\"text-align: left; width: 60%; \">" . $paymentMethod . "</td> <th style='font-size: 14px; text-align: right;'>".number_format($amount, 0, '', ' ') ." &nbsp;</th></tr>";
            }
            
            /*
            $chaineBody .= "<tr><th style=\"text-align: left; width: 60%; line-height: 17px; font-size: 16px;\">TOTAL</th><th style='font-size: 16px; text-align: right;'>".number_format($totalGeneral, 0, '', ' ')." &nbsp;</th></tr>
                            <tr><th style=\"text-align: left; width: 60%; line-height: 17px; font-size: 12px;\">TOTAL REDUCTION</th><th style='font-size: 14px; text-align: right;'>". number_format($totalSommeReduction, 0, '', ' ') ." &nbsp;</th></tr>
                            ";
            */
            $chaineBody .= "    </table>

                                <div id='footer'>
                                    <hr/>
                                    <p>Powered By ".NOM_APPLICATION."</p>                                      
                                </div>
                            </body>
                        </html>";
                
            // var_dump($fichierretourner);
            $options = new \Dompdf\Options();
            $options->set('isRemoteEnabled', TRUE);
            $dompdf = new \Dompdf\Dompdf($options);
            $dompdf->set_paper(TYPE_IMPRESSION, "portrait");
            $dompdf->load_html($chaineBody);
            $dompdf->set_base_path($_SERVER['DOCUMENT_ROOT']);
            $dompdf->render();

            $output = $dompdf->output();
            $fileName = "salesLog_".$cashierId;
           
            $nomCompletFichierToSave = PUBLIC_PATH . "/documents/reports/" . $fileName . ".pdf";
        
            file_put_contents($nomCompletFichierToSave, $output);

            $link = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS']== 'on' ? "https" : "http") .
            
                    '://'. $this->getRequest()->getHttpHost() . $this->view->baseUrl(). "/documents/reports/" . $fileName . ".pdf";
    
            return  $link;
    }
}
