<?php

include APPLICATION_PATH . '/includes/daos.php';

class SettingController extends Zend_Controller_Action
{
    private $translator;
    private $days;

    private function getPriceName($consideredPriceNumber)
    {
        switch ($consideredPriceNumber) {
            case 0:
                return $this->translator->_("setting.priceSetting.defaultSalePrice");
            case 1:
                return $this->translator->_("setting.priceSetting.salePrice2");
            case 2:
                return $this->translator->_("setting.priceSetting.salePrice3");
        }
    }

    public function init()
    {
        session_start();
        SubscriptionChecker::checkSubscriptionValidity($this);
        if (isset($_SESSION[SESSION_USER_ID]) && $_SESSION[SESSION_USER_ID] != null) {
            $_SESSION[LAST_ACTION_DATE] = new DateTime();
            LogDAO::updateLastActionDateForCurrentUser();
            $this->translator = Translator::init();
            $this->days = [
                "1" => $this->translator->_("shared.days.monday"),
                "2" => $this->translator->_("shared.days.tuesday"),
                "3" => $this->translator->_("shared.days.wednesday"),
                "4" => $this->translator->_("shared.days.thursday"),
                "5" => $this->translator->_("shared.days.friday"),
                "6" => $this->translator->_("shared.days.saturday"),
                "7" => $this->translator->_("shared.days.sunday"),
            ];
        } else {
            if ($this->getRequest()->isPost()) {
                echo ApiResponse::respond(SESSION_EXPIRED_CODE);
                exit();
            } else {
                $this->_helper->redirector('index', 'Index');
            }
        }
    }

    public function indexAction()
    {
        AccessRightChecker::checkAccessRight($this, 24);
        $this->view->translator = $this->translator;
    }

    public function pricesettingAction()
    {
        AccessRightChecker::checkAccessRight($this, 136);
        $pointOfSaleDao = new PointOfSaleDAO();
        $pointOfSales = $pointOfSaleDao->listPointOfSales();
        $this->view->pointOfSales = $pointOfSales;
        $this->view->days = $this->days;
        $this->view->translator = $this->translator;
    }

    public function datatablepricesettingAction()
    {
        try {
            $pointOfSale = (int)$_POST["pointOfSale"] != 0 ? (int)$_POST["pointOfSale"] : null;
            $consideredPrice = (int)$_POST["consideredPrice"] != 0 ? (int)$_POST["consideredPrice"] : null;
            $wherePointOfSale = "";
            $whereConsideredPrice = "";
            if ($pointOfSale != null) {
                $wherePointOfSale .= " AND p.id_point_vente = {$pointOfSale}";
            }
            if ($consideredPrice != null) {
                $whereConsideredPrice .= " AND p.prix_applicable = {$consideredPrice}";
            }
            $where = "p.statut = 1{$wherePointOfSale}{$whereConsideredPrice}";
            $priceSettingsTable = "
                (
                    SELECT p.id_parametre_prix AS price_setting_id , p.jour_debut AS begin_day, 
                    p.jour_fin AS end_day , p.prix_applicable as considered_price, 
                    pv.nom_point_vente as point_of_sale_name
                    FROM parametre_prix p JOIN point_vente pv ON p.id_point_vente = pv.id_point_vente 
                    WHERE {$where}
                ) AS priceSettingsTable
            ";
            $primaryKey = 'price_setting_id';
            $columns = [
                ['db' => 'point_of_sale_name', 'dt' => 0],
                ['db' => 'begin_day', 'dt' => 1],
                ['db' => 'end_day', 'dt' => 2],
                ['db' => 'considered_price', 'dt' => 3],
                ['db' => 'price_setting_id', 'dt' => 4],
            ];
            $computeDatatableResults = SSP::customSimple($_POST, $priceSettingsTable, $primaryKey, $columns);
            $datatableResult = $computeDatatableResults["datatable"];
            $finalResult = $computeDatatableResults["custom"];
            foreach ($datatableResult["data"] as $row) {
                $pointOfSaleName = $row[0];
                $beginDayExplode = explode(" ", $row[1]);
                $beginDay = "{$this->days[(int)$beginDayExplode[0]]} {$beginDayExplode[1]}";
                $endDayExplode = explode(" ", $row[2]);
                $endDay = "{$this->days[(int)$endDayExplode[0]]} {$endDayExplode[1]}";
                $consideredPrice = $this->getPriceName((int)$row[3]);
                $priceSettingId = (int)$row[4];
                $actions = "
                    <button title='{$this->translator->_("shared.edit")}' about='{$priceSettingId}' class='mini ui icon button app-secondary-button app-edit-action'>
                        <i class='edit icon'></i>
                    </button>
                    <button title='{$this->translator->_("shared.delete")}' about='{$priceSettingId}' class='mini ui red icon button app-delete-action'>
                        <i class='trash icon'></i>
                    </button>
                ";
                $tableRow = [
                    $pointOfSaleName,
                    $beginDay,
                    $endDay,
                    $consideredPrice,
                    $actions,
                ];
                $finalResult["data"][] = $tableRow;
            }
            echo json_encode($finalResult);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 overlap
     * 102 unknown error
     */
    public function createpricesettingAction()
    {
        try {
            $pointOfSaleId = (int)$_POST["pointOfSale"];
            $startDay = (int)$_POST["startDay"];
            $startDayHour = $_POST["startDayHour"];
            $endDay = (int)$_POST["endDay"];
            $endDayHour = $_POST["endDayHour"];
            $consideredPrice = (int)$_POST["consideredPrice"];
            $pointOfSaleDao = new PointOfSaleDAO();
            $priceSettingDao = new PriceSettingDAO();
            $pointOfSale = $pointOfSaleDao->getPointOfSaleById($pointOfSaleId);
            $priceSettingsOfPointOfSale = $priceSettingDao->getPriceSettingsByPointOfSale($pointOfSale);
            $lastStartDay = $priceSettingDao->getLastDayOfNumber($startDay);
            $lastEndDay = $priceSettingDao->getLastDayOfNumber($endDay);
            $lastStartDay = new DateTime("{$lastStartDay->format(DATE_FORMAT)} {$startDayHour}");
            $lastEndDay = new DateTime("{$lastEndDay->format(DATE_FORMAT)} {$endDayHour}");
            $toTimeStart = strtotime($lastStartDay->format(DATE_FORMAT_WITH_TIME));
            $toTimeEnd = strtotime($lastEndDay->format(DATE_FORMAT_WITH_TIME));
            foreach ($priceSettingsOfPointOfSale as $currentPriceSetting) {
                $explodeCurrentStart = explode(" ", $currentPriceSetting->getJourDebut());
                $explodeCurrentEnd = explode(" ", $currentPriceSetting->getJourFin());
                $currentLastStartDate = $priceSettingDao->getLastDayOfNumber((int)$explodeCurrentStart[0]);
                $currentLastEndDate = $priceSettingDao->getLastDayOfNumber((int)$explodeCurrentEnd[0]);
                $currentLastStartDate = new DateTime("{$currentLastStartDate->format(DATE_FORMAT)} {$explodeCurrentStart[1]}");
                $currentLastEndDate = new DateTime("{$currentLastEndDate->format(DATE_FORMAT)} {$explodeCurrentEnd[1]}");
                $timeCurrentStart = strtotime($currentLastStartDate->format(DATE_FORMAT_WITH_TIME));
                $timeCurrentEnd = strtotime($currentLastEndDate->format(DATE_FORMAT_WITH_TIME));
                if ($timeCurrentStart <= $toTimeStart && $timeCurrentEnd >= $toTimeStart) {
                    echo ApiResponse::respond(101, null, ["overlap" => "{$this->days[(int)$explodeCurrentStart[0]]} {$explodeCurrentStart[1]}"]);
                    exit();
                }
                if ($toTimeStart <= $timeCurrentEnd && $toTimeEnd >= $timeCurrentEnd) {
                    echo ApiResponse::respond(101, null, ["overlap" => "{$this->days[(int)$explodeCurrentStart[0]]} {$explodeCurrentStart[1]}"]);
                    exit();
                }
                if ($toTimeStart >= $timeCurrentStart && $toTimeEnd <= $timeCurrentEnd) {
                    echo ApiResponse::respond(101, null, ["overlap" => "{$this->days[(int)$explodeCurrentStart[0]]} {$explodeCurrentStart[1]}"]);
                    exit();
                }
                if ($timeCurrentStart >= $toTimeStart && $timeCurrentEnd <= $toTimeEnd) {
                    echo ApiResponse::respond(101, null, ["overlap" => "{$this->days[(int)$explodeCurrentStart[0]]} {$explodeCurrentStart[1]}"]);
                    exit();
                }
                if ($toTimeStart <= $timeCurrentStart && $toTimeEnd >= $timeCurrentStart) {
                    echo ApiResponse::respond(101, null, ["overlap" => "{$this->days[(int)$explodeCurrentStart[0]]} {$explodeCurrentStart[1]}"]);
                    exit();
                }
                if ($timeCurrentStart <= $toTimeEnd && $toTimeEnd <= $timeCurrentEnd) {
                    echo ApiResponse::respond(101, null, ["overlap" => "{$this->days[(int)$explodeCurrentStart[0]]} {$explodeCurrentStart[1]}"]);
                    exit();
                }
            }
            $priceSetting = $priceSettingDao->create();
            $priceSetting->setIdPointVente($pointOfSale);
            $priceSetting->setJourDebut("{$startDay} {$startDayHour}");
            $priceSetting->setJourFin("{$endDay} {$endDayHour}");
            $priceSetting->setPrixApplicable($consideredPrice);
            $priceSetting->setStatut(1);
            $priceSettingDao->save($priceSetting);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("setting.priceSetting.configurationSuccessfullyRegistered")} : N°{$priceSetting->getIdParametrePrix()} {$pointOfSale->getNomPointVente()}({$this->days[$startDay]} {$startDayHour} - {$this->days[$endDay]} {$endDayHour})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function detailpricesettingAction()
    {
        try {
            $id = (int)$_POST["id"];
            $priceSettingDao = new PriceSettingDAO();
            $priceSetting = $priceSettingDao->getPriceSettingById($id);
            $explodeStart = explode(" ", $priceSetting->getJourDebut());
            $explodeEnd = explode(" ", $priceSetting->getJourFin());
            $result = [
                "pointOfSale" => $priceSetting->getIdPointVente()->getIdPointVente(),
                "startDay" => (int)$explodeStart[0],
                "startDayHour" => $explodeStart[1],
                "endDay" => (int)$explodeEnd[0],
                "endDayHour" => $explodeEnd[1],
                "consideredPrice" => $priceSetting->getPrixApplicable(),
            ];
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 overlap
     * 102 unknown error
     */
    public function updatepricesettingAction()
    {
        try {
            $id = (int)$_POST["id"];
            $pointOfSaleId = (int)$_POST["pointOfSale"];
            $startDay = (int)$_POST["startDay"];
            $startDayHour = $_POST["startDayHour"];
            $endDay = (int)$_POST["endDay"];
            $endDayHour = $_POST["endDayHour"];
            $consideredPrice = (int)$_POST["consideredPrice"];
            $pointOfSaleDao = new PointOfSaleDAO();
            $priceSettingDao = new PriceSettingDAO();
            $pointOfSale = $pointOfSaleDao->getPointOfSaleById($pointOfSaleId);
            $priceSettingsOfPointOfSale = $priceSettingDao->getPriceSettingsByPointOfSale($pointOfSale);
            $lastStartDay = $priceSettingDao->getLastDayOfNumber($startDay);
            $lastEndDay = $priceSettingDao->getLastDayOfNumber($endDay);
            $lastStartDay = new DateTime("{$lastStartDay->format(DATE_FORMAT)} {$startDayHour}");
            $lastEndDay = new DateTime("{$lastEndDay->format(DATE_FORMAT)} {$endDayHour}");
            $toTimeStart = strtotime($lastStartDay->format(DATE_FORMAT_WITH_TIME));
            $toTimeEnd = strtotime($lastEndDay->format(DATE_FORMAT_WITH_TIME));
            foreach ($priceSettingsOfPointOfSale as $currentPriceSetting) {
                if ($currentPriceSetting->getIdParametrePrix() == $id) continue;
                $explodeCurrentStart = explode(" ", $currentPriceSetting->getJourDebut());
                $explodeCurrentEnd = explode(" ", $currentPriceSetting->getJourFin());
                $currentLastStartDate = $priceSettingDao->getLastDayOfNumber((int)$explodeCurrentStart[0]);
                $currentLastEndDate = $priceSettingDao->getLastDayOfNumber((int)$explodeCurrentEnd[0]);
                $currentLastStartDate = new DateTime("{$currentLastStartDate->format(DATE_FORMAT)} {$explodeCurrentStart[1]}");
                $currentLastEndDate = new DateTime("{$currentLastEndDate->format(DATE_FORMAT)} {$explodeCurrentEnd[1]}");
                $timeCurrentStart = strtotime($currentLastStartDate->format(DATE_FORMAT_WITH_TIME));
                $timeCurrentEnd = strtotime($currentLastEndDate->format(DATE_FORMAT_WITH_TIME));
                if ($timeCurrentStart <= $toTimeStart && $timeCurrentEnd >= $toTimeStart) {
                    echo ApiResponse::respond(101, null, ["overlap" => "{$this->days[(int)$explodeCurrentStart[0]]} {$explodeCurrentStart[1]}"]);
                    exit();
                }
                if ($toTimeStart <= $timeCurrentEnd && $toTimeEnd >= $timeCurrentEnd) {
                    echo ApiResponse::respond(101, null, ["overlap" => "{$this->days[(int)$explodeCurrentStart[0]]} {$explodeCurrentStart[1]}"]);
                    exit();
                }
                if ($toTimeStart >= $timeCurrentStart && $toTimeEnd <= $timeCurrentEnd) {
                    echo ApiResponse::respond(101, null, ["overlap" => "{$this->days[(int)$explodeCurrentStart[0]]} {$explodeCurrentStart[1]}"]);
                    exit();
                }
                if ($timeCurrentStart >= $toTimeStart && $timeCurrentEnd <= $toTimeEnd) {
                    echo ApiResponse::respond(101, null, ["overlap" => "{$this->days[(int)$explodeCurrentStart[0]]} {$explodeCurrentStart[1]}"]);
                    exit();
                }
                if ($toTimeStart <= $timeCurrentStart && $toTimeEnd >= $timeCurrentStart) {
                    echo ApiResponse::respond(101, null, ["overlap" => "{$this->days[(int)$explodeCurrentStart[0]]} {$explodeCurrentStart[1]}"]);
                    exit();
                }
                if ($timeCurrentStart <= $toTimeEnd && $toTimeEnd <= $timeCurrentEnd) {
                    echo ApiResponse::respond(101, null, ["overlap" => "{$this->days[(int)$explodeCurrentStart[0]]} {$explodeCurrentStart[1]}"]);
                    exit();
                }
            }
            $priceSetting = $priceSettingDao->getPriceSettingById($id);
            $priceSetting->setIdPointVente($pointOfSale);
            $priceSetting->setJourDebut("{$startDay} {$startDayHour}");
            $priceSetting->setJourFin("{$endDay} {$endDayHour}");
            $priceSetting->setPrixApplicable($consideredPrice);
            $priceSettingDao->update($priceSetting);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("setting.priceSetting.configurationUpdatedSuccessfully")} : N°{$priceSetting->getIdParametrePrix()} {$pointOfSale->getNomPointVente()}({$this->days[$startDay]} {$startDayHour} - {$this->days[$endDay]} {$endDayHour})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function deletepricesettingAction()
    {
        try {
            $id = (int)$_POST["id"];
            $priceSettingDao = new PriceSettingDAO();
            $priceSetting = $priceSettingDao->getPriceSettingById($id);
            if ($priceSetting->getStatut() == 0) {
                echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE);
                exit();
            }
            $pointOfSale = $priceSetting->getIdPointVente();
            $explodeStart = explode(" ", $priceSetting->getJourDebut());
            $explodeEnd = explode(" ", $priceSetting->getJourFin());
            $startDay = (int)$explodeStart[0];
            $startDayHour = $explodeStart[1];
            $endDay = (int)$explodeEnd[0];
            $endDayHour = $explodeEnd[1];
            $priceSetting->setStatut(0);
            $priceSettingDao->update($priceSetting);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("setting.priceSetting.configurationDeletedSuccessfully")} : N°{$priceSetting->getIdParametrePrix()} {$pointOfSale->getNomPointVente()}({$this->days[$startDay]} {$startDayHour} - {$this->days[$endDay]} {$endDayHour})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function updatesettingsAction()
    {
        try {
            $section = $_POST["section"];
            $parameterDao = new ParameterDAO();
            switch ($section) {
                case "APPLICATION_THEME_CONFIGURATION":
                    $primaryColor = $_POST["primary"];
                    $secondaryColor = $_POST["secondary"];
                    $tertiaryColor = $_POST["tertiary"];
                    $dbAppPrimaryBrand = $parameterDao->getParameterByName("DB_APP_PRIMARY_BRAND");
                    $dbAppSecondaryBrand = $parameterDao->getParameterByName("DB_APP_SECONDARY_BRAND");
                    $dbAppThirdBrand = $parameterDao->getParameterByName("DB_APP_THIRD_BRAND");
                    $dbAppPrimaryBrand->setValeur($primaryColor);
                    $dbAppSecondaryBrand->setValeur($secondaryColor);
                    $dbAppThirdBrand->setValeur($tertiaryColor);
                    $parameterDao->update($dbAppPrimaryBrand);
                    $parameterDao->update($dbAppSecondaryBrand);
                    $parameterDao->update($dbAppThirdBrand);
                    LogDAO::registerOperationForCurrentUser(
                        "{$this->translator->_("setting.index.applicationThemeUpdatedSuccessfully")}"
                    );
                    echo ApiResponse::respond();
                    exit();
                case "BUSINESS_INFORMATION_CONFIGURATION":
                    $businessName = $_POST["businessName"];
                    $businessEmail = $_POST["businessEmail"];
                    $businessWebsite = $_POST["businessWebsite"];
                    $businessPhoneNumber = $_POST["businessPhoneNumber"];
                    $businessPoBox = $_POST["businessPoBox"];
                    $businessActivitySector = $_POST["businessActivitySector"];
                    $businessNameParameter = $parameterDao->getParameterByName("NOM_STRUCTURE");
                    $businessEmailParameter = $parameterDao->getParameterByName("EMAIL_STRUCTURE");
                    $businessWebsiteParameter = $parameterDao->getParameterByName("SITEWEB_STRUCTURE");
                    $businessPhoneNumberParameter = $parameterDao->getParameterByName("TEL_STRUCTURE");
                    $businessPoBoxParameter = $parameterDao->getParameterByName("BP_STRUCTURE");
                    $businessActivitySectorParameter = $parameterDao->getParameterByName("TYPE_ACTIVITE_COMMERCE");
                    $businessNameParameter->setValeur($businessName);
                    $businessEmailParameter->setValeur($businessEmail);
                    $businessWebsiteParameter->setValeur($businessWebsite);
                    $businessPhoneNumberParameter->setValeur($businessPhoneNumber);
                    $businessPoBoxParameter->setValeur($businessPoBox);
                    $businessActivitySectorParameter->setValeur($businessActivitySector);
                    $parameterDao->update($businessNameParameter);
                    $parameterDao->update($businessEmailParameter);
                    $parameterDao->update($businessWebsiteParameter);
                    $parameterDao->update($businessPhoneNumberParameter);
                    $parameterDao->update($businessPoBoxParameter);
                    $parameterDao->update($businessActivitySectorParameter);
                    $result = ["shouldReload" => 0];
                    if (isset($_FILES["businessLogoInput"]) && $_FILES["businessLogoInput"]["name"] != "") {
                        $uploadedFilename = $_FILES["businessLogoInput"]["name"];
                        $uploadFilenameArray = explode(".", $uploadedFilename);
                        $extension = end($uploadFilenameArray);
                        $currentDate = new DateTime();
                        $randomName = "logo_{$currentDate->format("YmdHis")}";
                        $filename = PUBLIC_PATH . "/images/business/{$randomName}.{$extension}";
                        $businessLogoParameter = $parameterDao->getParameterByName("LOGO_STRUCTURE");
                        if ($businessLogoParameter->getValeur() != null) {
                            unlink(PUBLIC_PATH . "/images/business/{$businessLogoParameter->getValeur()}");
                        }
                        move_uploaded_file($_FILES["businessLogoInput"]["tmp_name"], $filename);
                        $businessLogoParameter->setValeur("{$randomName}.{$extension}");
                        $parameterDao->update($businessLogoParameter);
                        $result["shouldReload"] = 1;
                    }
                    LogDAO::registerOperationForCurrentUser(
                        "{$this->translator->_("setting.index.businessInformationUpdatedSuccessfully")}"
                    );
                    echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
                    exit();
                case "PRODUCTIVITY_ENHANCING_COLOR_CONFIGURATION":
                    $paidInvoicesColor = $_POST["paidInvoicesColor"];
                    $inPaymentInvoicesColor = $_POST["inPaymentInvoicesColor"];
                    $unpaidInvoicesColor = $_POST["unpaidInvoicesColor"];
                    $soonToExpireProductsColor = $_POST["soonToExpireProductsColor"];
                    $expiredProductsColor = $_POST["expiredProductsColor"];
                    $productsAtTheThresholdColor = $_POST["productsAtTheThresholdColor"];
                    $outOfStockProductsColor = $_POST["outOfStockProductsColor"];
                    $paidInvoicesColorParameter = $parameterDao->getParameterByName("COULEUR_FACTURE_REGLEE");
                    $inPaymentInvoicesColorParameter = $parameterDao->getParameterByName("COULEUR_FACTURE_EN_COURS_REGLEMENT");
                    $unpaidInvoicesColorParameter = $parameterDao->getParameterByName("COULEUR_FACTURE_SANS_PAIEMENT");
                    $soonToExpireProductsColorParameter = $parameterDao->getParameterByName("COULEUR_PRODUIT_PROCHE_DATE_PEREMPTION");
                    $expiredProductsColorParameter = $parameterDao->getParameterByName("COULEUR_PRODUIT_PERIME");
                    $productsAtTheThresholdColorParameter = $parameterDao->getParameterByName("COULEUR_PRODUIT_SEUIL_ATTEINT");
                    $outOfStockProductsColorParameter = $parameterDao->getParameterByName("COULEUR_PRODUIT_EN_RUPTURE");
                    $paidInvoicesColorParameter->setValeur($paidInvoicesColor);
                    $inPaymentInvoicesColorParameter->setValeur($inPaymentInvoicesColor);
                    $unpaidInvoicesColorParameter->setValeur($unpaidInvoicesColor);
                    $soonToExpireProductsColorParameter->setValeur($soonToExpireProductsColor);
                    $expiredProductsColorParameter->setValeur($expiredProductsColor);
                    $productsAtTheThresholdColorParameter->setValeur($productsAtTheThresholdColor);
                    $outOfStockProductsColorParameter->setValeur($outOfStockProductsColor);
                    $parameterDao->update($paidInvoicesColorParameter);
                    $parameterDao->update($inPaymentInvoicesColorParameter);
                    $parameterDao->update($unpaidInvoicesColorParameter);
                    $parameterDao->update($soonToExpireProductsColorParameter);
                    $parameterDao->update($expiredProductsColorParameter);
                    $parameterDao->update($productsAtTheThresholdColorParameter);
                    $parameterDao->update($outOfStockProductsColorParameter);
                    LogDAO::registerOperationForCurrentUser(
                        "{$this->translator->_("setting.index.productivityEnhancingColorsUpdatedSuccessfully")}"
                    );
                    echo ApiResponse::respond();
                    exit();
                case "BILLING_ITEMS_CONFIGURATION":
                    $phraseAtTheBottomOfSalesTickets = $_POST["phraseAtTheBottomOfSalesTickets"];
                    $showBusinessNameOrLogoOnInvoice = $_POST["showBusinessNameOrLogoOnInvoice"];
                    $printFormat = $_POST["printFormat"];
                    $printOrientation = $_POST["printOrientation"];
                    $typeOfBilling = $_POST["typeOfBilling"];
                    $considerVAT = $_POST["considerVAT"];
                    $VATPercent = $_POST["VATPercent"];
                    $considerIT = $_POST["considerIT"];
                    $ITPercent = $_POST["ITPercent"];
                    $phraseAtTheBottomOfSalesTicketsParameter = $parameterDao->getParameterByName("PHRASE_ARTICLE_VENDU");
                    $showBusinessNameOrLogoOnInvoiceParameter = $parameterDao->getParameterByName("FACTURE_AFFICHER_LOGO_ET_NOM_STRUCTURE");
                    $printFormatParameter = $parameterDao->getParameterByName("TYPE_IMPRESSION");
                    $printOrientationParameter = $parameterDao->getParameterByName("ORIENTATION_IMPRESSION");
                    $typeOfBillingParameter = $parameterDao->getParameterByName("TYPE_VENTE_PRODUIT");
                    $considerVATParameter = $parameterDao->getParameterByName("GESTION_TVA");
                    $VATPercentParameter = $parameterDao->getParameterByName("VALEUR_TVA");
                    $considerITParameter = $parameterDao->getParameterByName("GESTION_IR");
                    $ITPercentParameter = $parameterDao->getParameterByName("VALEUR_IR");
                    $phraseAtTheBottomOfSalesTicketsParameter->setValeur($phraseAtTheBottomOfSalesTickets);
                    $showBusinessNameOrLogoOnInvoiceParameter->setValeur($showBusinessNameOrLogoOnInvoice);
                    $printFormatParameter->setValeur($printFormat);
                    $printOrientationParameter->setValeur($printOrientation);
                    $typeOfBillingParameter->setValeur($typeOfBilling);
                    $considerVATParameter->setValeur($considerVAT);
                    if ((int)$considerVAT == 1) {
                        $VATPercentParameter->setValeur($VATPercent);
                    } else {
                        $VATPercentParameter->setValeur(null);
                    }
                    $considerITParameter->setValeur($considerIT);
                    if ((int)$considerIT == 1) {
                        $ITPercentParameter->setValeur($ITPercent);
                    } else {
                        $ITPercentParameter->setValeur(null);
                    }
                    $parameterDao->update($phraseAtTheBottomOfSalesTicketsParameter);
                    $parameterDao->update($showBusinessNameOrLogoOnInvoiceParameter);
                    $parameterDao->update($printFormatParameter);
                    $parameterDao->update($printOrientationParameter);
                    $parameterDao->update($typeOfBillingParameter);
                    $parameterDao->update($considerVATParameter);
                    $parameterDao->update($VATPercentParameter);
                    $parameterDao->update($considerITParameter);
                    $parameterDao->update($ITPercentParameter);
                    LogDAO::registerOperationForCurrentUser(
                        "{$this->translator->_("setting.index.billingItemsUpdatedSuccessfully")}"
                    );
                    echo ApiResponse::respond();
                    exit();
                case "LOYALTY_CONFIGURATION":
                    $loyaltyCardValidityPeriod = $_POST["loyaltyCardValidityPeriod"];
                    $loyaltyCardMinimumPurchaseAmount = $_POST["loyaltyCardMinimumPurchaseAmount"];
                    $loyaltyCardPointsOfMinimumPurchase = $_POST["loyaltyCardPointsOfMinimumPurchase"];
                    $voucherValidityPeriod = $_POST["voucherValidityPeriod"];
                    $voucherMinimumPointsToHave = $_POST["voucherMinimumPointsToHave"];
                    $voucherAmountForMinimumPoints = $_POST["voucherAmountForMinimumPoints"];
                    $loyaltyCardValidityPeriodParameter = $parameterDao->getParameterByName("DUREE_VALIDITE_CARTE_FIDELITE");
                    $loyaltyCardMinimumPurchaseAmountParameter = $parameterDao->getParameterByName("MONTANT_ACHAT_MIN_CARTE_FIDELITE");
                    $loyaltyCardPointsOfMinimumPurchaseParameter = $parameterDao->getParameterByName("NOMBRE_POINT_PAR_MONTANT_ACHAT_MIN_CARTE_FIDELITE");
                    $voucherValidityPeriodParameter = $parameterDao->getParameterByName("DUREE_DEFAULT_BON_ACHAT_CLIENT");
                    $voucherMinimumPointsToHaveParameter = $parameterDao->getParameterByName("NOMBRE_POINT_MIN_POUR_BON_ACHAT");
                    $voucherAmountForMinimumPointsParameter = $parameterDao->getParameterByName("MONTANT_BON_ACHAT_PAR_NOMBRE_POINT_MIN");
                    $loyaltyCardValidityPeriodParameter->setValeur($loyaltyCardValidityPeriod);
                    $loyaltyCardMinimumPurchaseAmountParameter->setValeur($loyaltyCardMinimumPurchaseAmount);
                    $loyaltyCardPointsOfMinimumPurchaseParameter->setValeur($loyaltyCardPointsOfMinimumPurchase);
                    $voucherValidityPeriodParameter->setValeur($voucherValidityPeriod);
                    $voucherMinimumPointsToHaveParameter->setValeur($voucherMinimumPointsToHave);
                    $voucherAmountForMinimumPointsParameter->setValeur($voucherAmountForMinimumPoints);
                    $parameterDao->update($loyaltyCardValidityPeriodParameter);
                    $parameterDao->update($loyaltyCardMinimumPurchaseAmountParameter);
                    $parameterDao->update($loyaltyCardPointsOfMinimumPurchaseParameter);
                    $parameterDao->update($voucherValidityPeriodParameter);
                    $parameterDao->update($voucherMinimumPointsToHaveParameter);
                    $parameterDao->update($voucherAmountForMinimumPointsParameter);
                    LogDAO::registerOperationForCurrentUser(
                        "{$this->translator->_("setting.index.loyaltySettingsUpdatedSuccessfully")}"
                    );
                    echo ApiResponse::respond();
                    exit();
                case "SMS_AND_EMAIL_CONFIGURATION":
                    $smsLogin = $_POST["smsLogin"];
                    $smsPassword = $_POST["smsPassword"];
                    $smsUserID = $_POST["smsUserID"];
                    $smsServer = $_POST["smsServer"];
                    $emailLogin = $_POST["emailLogin"];
                    $emailPassword = $_POST["emailPassword"];
                    $emailDomainName = $_POST["emailDomainName"];
                    $emailPort = $_POST["emailPort"];
                    $insertLogoInEmails = $_POST["insertLogoInEmails"];
                    $smsLoginParameter = $parameterDao->getParameterByName("PARAMETRE_SMS_LOGIN");
                    $smsPasswordParameter = $parameterDao->getParameterByName("PARAMETRE_SMS_PASS");
                    $smsUserIDParameter = $parameterDao->getParameterByName("PARAMETRE_SMS_USER_ID");
                    $smsServerParameter = $parameterDao->getParameterByName("PARAMETRE_SMS_SERVEUR");
                    $emailLoginParameter = $parameterDao->getParameterByName("PARAMETRE_MESSAGERIE_EMAIL");
                    $emailPasswordParameter = $parameterDao->getParameterByName("PARAMETRE_MESSAGERIE_PASS_EMAIL");
                    $emailDomainNameParameter = $parameterDao->getParameterByName("PARAMETRE_MESSAGERIE_NOM_DOMAINE");
                    $emailPortParameter = $parameterDao->getParameterByName("PARAMETRE_MESSAGERIE_PORT");
                    $insertLogoInEmailsParameter = $parameterDao->getParameterByName("LOGO_PRESENT_DANS_LES_MAILS");
                    $smsLoginParameter->setValeur($smsLogin);
                    $smsPasswordParameter->setValeur($smsPassword);
                    $smsUserIDParameter->setValeur($smsUserID);
                    $smsServerParameter->setValeur($smsServer);
                    $emailLoginParameter->setValeur($emailLogin);
                    $emailPasswordParameter->setValeur($emailPassword);
                    $emailDomainNameParameter->setValeur($emailDomainName);
                    $emailPortParameter->setValeur($emailPort);
                    $insertLogoInEmailsParameter->setValeur($insertLogoInEmails);
                    $parameterDao->update($smsLoginParameter);
                    $parameterDao->update($smsPasswordParameter);
                    $parameterDao->update($smsUserIDParameter);
                    $parameterDao->update($smsServerParameter);
                    $parameterDao->update($emailLoginParameter);
                    $parameterDao->update($emailPasswordParameter);
                    $parameterDao->update($emailDomainNameParameter);
                    $parameterDao->update($emailPortParameter);
                    $parameterDao->update($insertLogoInEmailsParameter);
                    LogDAO::registerOperationForCurrentUser(
                        "{$this->translator->_("setting.index.smsAndEmailSettingsUpdatedSuccessfully")}"
                    );
                    echo ApiResponse::respond();
                    exit();
                case "OTHER_CONFIGURATION":
                    $statisticsPeriodicity = $_POST["statisticsPeriodicity"];
                    $numberOfDaysForExpirationAlert = $_POST["numberOfDaysForExpirationAlert"];
                    $advanceRequiredForOrders = $_POST["advanceRequiredForOrders"];
                    $takeCareOfIndividualsPricesPerCustomer = $_POST["takeCareOfIndividualsPricesPerCustomer"];
                    $checkoutManagement = $_POST["checkoutManagement"];
                    $useProductReferences = $_POST["useProductReferences"];
                    $considerExpirationDate = $_POST["considerExpirationDate"];
                    $defaultThresholdQuantity = $_POST["defaultThresholdQuantity"];
                    $desiredLengthForBatchNumbers = $_POST["desiredLengthForBatchNumbers"];
                    $deadlineForDeclaringAnAppointmentAsHonored = $_POST["deadlineForDeclaringAnAppointmentAsHonored"];
                    $defaultLanguage = $_POST["defaultLanguage"];
                    $statisticsPeriodicityParameter = $parameterDao->getParameterByName("TYPE_AFFICHAGE_CHART");
                    $numberOfDaysForExpirationAlertParameter = $parameterDao->getParameterByName("NOMBRE_JOUR_POUR_ALERTE_PEREMPTION");
                    $advanceRequiredForOrdersParameter = $parameterDao->getParameterByName("AVANCE_SUR_COMMANDE");
                    $takeCareOfIndividualsPricesPerCustomerParameter = $parameterDao->getParameterByName("PRIX_PAR_CLIENT");
                    $checkoutManagementParameter = $parameterDao->getParameterByName("TYPE_GESTION_CAISSE");
                    $useProductReferencesParameter = $parameterDao->getParameterByName("GESTION_REFERENCE_PRODUIT");
                    $considerExpirationDateParameter = $parameterDao->getParameterByName("GESTION_DATE_PEREMPTION_PRODUIT");
                    $defaultThresholdQuantityParameter = $parameterDao->getParameterByName("QUANTITE_SEUIL_PRODUIT_PAR_DEFAUT");
                    $desiredLengthForBatchNumbersParameter = $parameterDao->getParameterByName("DEFAULT_BATCH_NUMBER_LENGTH");
                    $deadlineForDeclaringAnAppointmentAsHonoredParameter = $parameterDao->getParameterByName("DEADLINE_FOR_DECLARING_AN_APPOINTMENT_AS_HONORED");
                    $defaultLanguageParameter = $parameterDao->getParameterByName("DEFAULT_LANGUAGE");
                    $statisticsPeriodicityParameter->setValeur($statisticsPeriodicity);
                    $numberOfDaysForExpirationAlertParameter->setValeur($numberOfDaysForExpirationAlert);
                    $advanceRequiredForOrdersParameter->setValeur($advanceRequiredForOrders);
                    $takeCareOfIndividualsPricesPerCustomerParameter->setValeur($takeCareOfIndividualsPricesPerCustomer);
                    $checkoutManagementParameter->setValeur($checkoutManagement);
                    $useProductReferencesParameter->setValeur($useProductReferences);
                    $considerExpirationDateParameter->setValeur($considerExpirationDate);
                    $defaultThresholdQuantityParameter->setValeur($defaultThresholdQuantity);
                    $desiredLengthForBatchNumbersParameter->setValeur($desiredLengthForBatchNumbers);
                    $deadlineForDeclaringAnAppointmentAsHonoredParameter->setValeur($deadlineForDeclaringAnAppointmentAsHonored);
                    $defaultLanguageParameter->setValeur($defaultLanguage);
                    $parameterDao->update($statisticsPeriodicityParameter);
                    $parameterDao->update($numberOfDaysForExpirationAlertParameter);
                    $parameterDao->update($advanceRequiredForOrdersParameter);
                    $parameterDao->update($takeCareOfIndividualsPricesPerCustomerParameter);
                    $parameterDao->update($checkoutManagementParameter);
                    $parameterDao->update($useProductReferencesParameter);
                    $parameterDao->update($considerExpirationDateParameter);
                    $parameterDao->update($defaultThresholdQuantityParameter);
                    $parameterDao->update($desiredLengthForBatchNumbersParameter);
                    $parameterDao->update($deadlineForDeclaringAnAppointmentAsHonoredParameter);
                    $parameterDao->update($defaultLanguageParameter);
                    LogDAO::registerOperationForCurrentUser(
                        "{$this->translator->_("setting.index.restOfGeneralSettingsUpdatedSuccessfully")}"
                    );
                    echo ApiResponse::respond();
                    exit();
            }
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }
}
