<?php

include APPLICATION_PATH . '/includes/daos.php';

class StoreController extends Zend_Controller_Action
{
    private $translator;

    public function init()
    {
        session_start();
        SubscriptionChecker::checkSubscriptionValidity($this);
        if (isset($_SESSION[SESSION_USER_ID]) && $_SESSION[SESSION_USER_ID] != null) {
            $_SESSION[LAST_ACTION_DATE] = new DateTime();
            LogDAO::updateLastActionDateForCurrentUser();
            $this->translator = Translator::init();
        } else {
            if ($this->getRequest()->isPost()) {
                echo ApiResponse::respond(SESSION_EXPIRED_CODE);
                exit();
            } else {
                $this->_helper->redirector('index', 'Index');
            }
        }
    }

    public function indexAction()
    {
        AccessRightChecker::checkAccessRight($this, 74);
        $subscriptionPackageDao = new SubscriptionPackageDAO();
        $storeDao = new StoreDAO();
        $subscriptionPackage = $subscriptionPackageDao->getSubscriptionPackageById((int)SUBSCRIPTION_PACKAGE);
        $this->view->maxStores = $subscriptionPackage->getCountStores() + SUBSCRIPTION_COUNT_MORE_SALE_POINTS;
        $this->view->currentStores = $storeDao->getStoresCount();
        $this->view->translator = $this->translator;
    }

    public function datatableAction()
    {
        $name = $_POST["name"] != "" ? $_POST["name"] : null;
        $whereName = "";
        if ($name != null) {
            $whereName .= " AND m.nom_magasin LIKE '%{$name}%'";
        }
        $where = "m.statut = 1{$whereName}";
        $storesTable = "
                (
                    SELECT m.id_magasin as store_id, m.nom_magasin as store_name
                    FROM magasin m
                    WHERE {$where}
                ) AS storesTable
        ";
        $primaryKey = 'store_id';
        $columns = [
            ['db' => 'store_name', 'dt' => 0],
            ['db' => 'store_id', 'dt' => 1],
        ];
        $computeDatatableResults = SSP::customSimple($_POST, $storesTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        foreach ($datatableResult["data"] as $row) {
            $storeName = $row[0];
            $storeId = $row[1];
            $actions = "
                <button title='{$this->translator->_("shared.edit")}' about='{$storeId}' class='mini ui icon button app-secondary-button app-edit-action'>
                    <i class='edit icon'></i>
                </button>
                <button title='{$this->translator->_("shared.delete")}' about='{$storeId}' class='mini ui red icon button app-delete-action'>
                    <i class='trash icon'></i>
                </button>
            ";
            $tableRow = [
                $storeName,
                $actions,
            ];
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    /**
     * 100 success
     * 101 duplicate
     * 102 Subscription limits reached
     * 103 unknown error
     */
    public function createAction()
    {
        $subscriptionPackageDao = new SubscriptionPackageDAO();
        $storeDao = new StoreDAO();
        $subscriptionPackage = $subscriptionPackageDao->getSubscriptionPackageById((int)SUBSCRIPTION_PACKAGE);
        $maxStores = $subscriptionPackage->getCountStores() + SUBSCRIPTION_COUNT_MORE_SALE_POINTS;
        $currentStores = $storeDao->getStoresCount();
        if ($currentStores >= $maxStores) {
            echo ApiResponse::respond(102, null, ["maxStores" => $maxStores, "currentStores" => $currentStores]);
            exit();
        }
        try {
            $name = $_POST["name"];
            if ($storeDao->storeNameAlreadyExists($name)) {
                echo ApiResponse::respond(101);
                exit();
            }
            $store = $storeDao->create();
            $store->setNomMagasin($name);
            $store->setStatut(1);
            $storeDao->save($store);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("store.index.storeSuccessfullyRegistered")} : N°{$store->getIdMagasin()}({$store->getNomMagasin()})"
            );
            $currentStores = $storeDao->getStoresCount();
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["maxStores" => $maxStores, "currentStores" => $currentStores]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(103, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 duplicate
     * 102 unknown error
     */
    public function updateAction()
    {
        try {
            $id = (int)$_POST["id"];
            $name = $_POST["name"];
            $storeDao = new StoreDAO();
            $store = $storeDao->getStoreById($id);
            if ($storeDao->storeNameAlreadyExists($name, $store)) {
                echo ApiResponse::respond(101);
                exit();
            }
            $store->setNomMagasin($name);
            $storeDao->update($store);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("store.index.storeUpdatedSuccessfully")} : N°{$store->getIdMagasin()}({$store->getNomMagasin()})"
            );
            $subscriptionPackageDao = new SubscriptionPackageDAO();
            $subscriptionPackage = $subscriptionPackageDao->getSubscriptionPackageById((int)SUBSCRIPTION_PACKAGE);
            $maxStores = $subscriptionPackage->getCountStores() + SUBSCRIPTION_COUNT_MORE_SALE_POINTS;
            $currentStores = $storeDao->getStoresCount();
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["maxStores" => $maxStores, "currentStores" => $currentStores]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function deleteAction()
    {
        try {
            $id = (int)$_POST["id"];
            $storeDao = new StoreDAO();
            $store = $storeDao->getStoreById($id);
            if ($store->getStatut() == 0) {
                echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE);
                exit();
            }
            $store->setStatut(0);
            $storeDao->update($store);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("store.index.storeDeletedSuccessfully")} : N°{$store->getIdMagasin()}({$store->getNomMagasin()})"
            );
            $subscriptionPackageDao = new SubscriptionPackageDAO();
            $subscriptionPackage = $subscriptionPackageDao->getSubscriptionPackageById((int)SUBSCRIPTION_PACKAGE);
            $maxStores = $subscriptionPackage->getCountStores() + SUBSCRIPTION_COUNT_MORE_SALE_POINTS;
            $currentStores = $storeDao->getStoresCount();
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["maxStores" => $maxStores, "currentStores" => $currentStores]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }
}
