<?php

include APPLICATION_PATH . '/includes/daos.php';

class SubscriptionController extends Zend_Controller_Action
{
    private $translator;

    public function init()
    {
        session_start();
        $this->translator = Translator::init();
        if (isset($_SESSION[SESSION_USER_ID]) && $_SESSION[SESSION_USER_ID] != null) {
            $_SESSION[LAST_ACTION_DATE] = new DateTime();
            LogDAO::updateLastActionDateForCurrentUser();
        } else {
            if ($this->getRequest()->getActionName() == "index") {
                if ($this->getRequest()->isPost()) {
                    echo ApiResponse::respond(SESSION_EXPIRED_CODE);
                    exit();
                } else {
                    $this->_helper->redirector('index', 'Index');
                }
            }
        }
    }

    public function indexAction()
    {
        if(!in_array($_SESSION[SESSION_USER_PROFILE_CODE], IDENTIFIED_PROFILES_CODES)) {
            $this->_helper->redirector('index', 'Cheat');
        }
        SubscriptionChecker::checkSubscriptionValidity($this);
        $subscriptionPackageDao = new SubscriptionPackageDAO();
        $userDao = new UserDAO();
        $productDao = new ProductDAO();
        $pointOfSaleDao = new PointOfSaleDAO();
        $storeDao = new StoreDAO();
        $subscriptionPackage = $subscriptionPackageDao->getSubscriptionPackageById((int)SUBSCRIPTION_PACKAGE);
        $expirationDate = date(DATE_FORMAT, SUBSCRIPTION_EXPIRATION_DATE);
        $activationDate = date(DATE_FORMAT, SUBSCRIPTION_ACTIVATION_DATE);
        $momentOfExpirationDate = new \Moment\Moment($expirationDate);
        $momentOfActivationDate = new \Moment\Moment($activationDate);
        $this->view->expirationDate = ucfirst($momentOfExpirationDate->format(DATE_FORMAT_LONG));
        $this->view->activationDate = ucfirst($momentOfActivationDate->format(DATE_FORMAT_LONG));
        $this->view->subscriptionPackage = $subscriptionPackage;
        $this->view->maxUsers = $subscriptionPackage->getCountUsers() + SUBSCRIPTION_MORE_COUNT_USERS;
        $this->view->maxProducts = $subscriptionPackage->getCountProducts() + SUBSCRIPTION_MORE_COUNT_PRODUCTS;
        $this->view->maxPointsOfSale = $subscriptionPackage->getCountSalePoints() + SUBSCRIPTION_COUNT_MORE_SALE_POINTS;
        $this->view->maxStores = $subscriptionPackage->getCountStores() + SUBSCRIPTION_COUNT_MORE_SALE_POINTS;
        $this->view->currentUsers = $userDao->getUsersCount();
        $this->view->currentProducts = $productDao->getProductsCount();
        $this->view->currentPointsOfSale = $pointOfSaleDao->getPointsOfSaleCount();
        $this->view->currentStores = $storeDao->getStoresCount();
        $this->view->translator = $this->translator;
    }

    public function expiredAction()
    {
        if (is_null(SUBSCRIPTION_ACTIVATION_DATE) || is_null(SUBSCRIPTION_EXPIRATION_DATE) || is_null(SUBSCRIPTION_PACKAGE)) {
            $this->_helper->redirector('uncertified', 'Subscription');
        }
        $subscriptionPackageDao = new SubscriptionPackageDAO();
        $subscriptionPackage = $subscriptionPackageDao->getSubscriptionPackageById((int)SUBSCRIPTION_PACKAGE);
        $expirationDate = date(DATE_FORMAT, SUBSCRIPTION_EXPIRATION_DATE);
        $completeExpirationDate = strtotime(date(DATE_FORMAT_WITH_SECONDS, SUBSCRIPTION_EXPIRATION_DATE));
        $currentDateTimestamp = strtotime(date(DATE_FORMAT_WITH_SECONDS));
        if ($currentDateTimestamp < $completeExpirationDate) {
            $this->_helper->redirector('index', 'Index');
        }
        $momentOfExpirationDate = new \Moment\Moment($expirationDate);
        $this->view->expirationDate = ucfirst($momentOfExpirationDate->format(DATE_FORMAT_LONG));
        $this->view->subscriptionPackage = $subscriptionPackage;
        $this->view->translator = $this->translator;
    }

    public function uncertifiedAction()
    {
        if (!is_null(SUBSCRIPTION_ACTIVATION_DATE) && !is_null(SUBSCRIPTION_EXPIRATION_DATE) && !is_null(SUBSCRIPTION_PACKAGE)) {
            $this->_helper->redirector('index', 'Index');
        }

        $this->view->translator = $this->translator;
    }
}
