<?php

include APPLICATION_PATH . '/includes/daos.php';

class SupplierController extends Zend_Controller_Action
{
    private $translator;

    public function init()
    {
        SubscriptionChecker::checkSubscriptionValidity($this);
        session_start();
        if (isset($_SESSION[SESSION_USER_ID]) && $_SESSION[SESSION_USER_ID] != null) {
            $_SESSION[LAST_ACTION_DATE] = new DateTime();
            LogDAO::updateLastActionDateForCurrentUser();
            $this->translator = Translator::init();
        } else {
            if ($this->getRequest()->isPost()) {
                echo ApiResponse::respond(SESSION_EXPIRED_CODE);
                exit();
            } else {
                $this->_helper->redirector('index', 'Index');
            }
        }
    }

    public function indexAction()
    {
        AccessRightChecker::checkAccessRight($this, 104);
        $this->view->translator = $this->translator;
    }

    /**
     * @throws Exception
     */
    public function datatableAction()
    {
        $name = $_POST["name"] != "" ? $_POST["name"] : null;
        $societyType = $_POST["societyType"] != "" ? $_POST["societyType"] : null;
        $phone = $_POST["phone"] != "" ? $_POST["phone"] : null;
        $whereName = "";
        $whereSocietyType = "";
        $wherePhone = "";
        if ($name != null) {
            $whereName .= " AND f.nom_fournisseur LIKE '%{$name}%'";
        }
        if ($societyType != null) {
            $whereSocietyType .= " AND f.raison_sociale = '{$societyType}'";
        }
        if ($phone != null) {
            $wherePhone .= " AND f.telephone LIKE '%{$phone}%'";
        }
        $where = "f.statut = 1{$whereName}{$whereSocietyType}{$wherePhone}";
        $suppliersTable = "
                (
                    SELECT f.nom_fournisseur AS supplier_name, 
                    f.raison_sociale AS supplier_society_type, f.adresse AS supplier_address,
                    f.telephone AS supplier_phone, f.boite_postale AS supplier_po_box,
                    f.id_fournisseur AS supplier_id, f.compte_comptable AS supplier_accounting_account
                    FROM fournisseur f
                    WHERE {$where}
                ) AS suppliersTable
        ";
        $primaryKey = 'supplier_id';
        $columns = [
            ['db' => 'supplier_name', 'dt' => 0],
            ['db' => 'supplier_society_type', 'dt' => 1],
            ['db' => 'supplier_phone', 'dt' => 2],
            ['db' => 'supplier_address', 'dt' => 3],
            ['db' => 'supplier_po_box', 'dt' => 4],
            ['db' => 'supplier_accounting_account', 'dt' => 5],
            ['db' => 'supplier_id', 'dt' => 6],
        ];
        $computeDatatableResults = SSP::customSimple($_POST, $suppliersTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        foreach ($datatableResult["data"] as $row) {
            $supplierName = $row[0];
            $supplierSocietyType = $this->translator->_("supplier.index.{$row[1]}");
            $supplierPhone = $row[2];
            $supplierAddress = $row[3];
            $supplierPoBox = $row[4];
            $supplierAccountingAccount = $row[5];
            $supplierId = (int)$row[6];
            $actions = "
                <button title='{$this->translator->_("shared.edit")}' about='{$supplierId}' class='mini ui icon button app-secondary-button app-edit-action'>
                    <i class='edit icon'></i>
                </button>
                <button title='{$this->translator->_("shared.delete")}' about='{$supplierId}' class='mini ui red icon button app-delete-action'>
                    <i class='trash icon'></i>
                </button>
            ";
            $tableRow = [
                $supplierName,
                $supplierSocietyType,
                $supplierPhone,
                $supplierAddress,
                $supplierPoBox,
                $supplierAccountingAccount,
                $actions,
            ];
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    /**
     * 100 success
     * 101 duplicate on supplier name
     * 102 unknown error
     */
    public function createAction()
    {
        try {
            $supplierName = $_POST["supplierName"];
            $societyType = $_POST["societyType"];
            $phoneNumber = $_POST["phoneNumber"];
            $poBox = $_POST["poBox"] != "" ? $_POST["poBox"] : null;
            $address = $_POST["address"] != "" ? $_POST["address"] : null;
            $accountingAccount = $_POST["accountingAccount"] != "" ? $_POST["accountingAccount"] : null;
            $supplierDao = new SupplierDAO();
            if ($supplierDao->supplierNameAlreadyExists($supplierName)) {
                echo ApiResponse::respond(101);
                exit();
            }
            $supplier = $supplierDao->create();
            $supplier->setNomFournisseur($supplierName);
            $supplier->setRaisonSociale($societyType);
            $supplier->setTelephone($phoneNumber);
            if ($poBox != null) {
                $supplier->setBoitePostale($poBox);
            }
            if ($address != null) {
                $supplier->setAdresse($address);
            }
            if ($accountingAccount != null) {
                $supplier->setCompteComptable($accountingAccount);
            }
            $supplier->setStatut(1);
            $supplierDao->save($supplier);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("supplier.index.supplierSuccessfullyRegistered")} : N°{$supplier->getIdFournisseur()}({$supplier->getNomFournisseur()})"
            );
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["id" => $supplier->getIdFournisseur()]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function detailAction()
    {
        try {
            $id = (int)$_POST["id"];
            $supplierDao = new SupplierDAO();
            $supplier = $supplierDao->getSupplierById($id);
            $result = [
                "supplierName" => $supplier->getNomFournisseur(),
                "societyType" => $supplier->getRaisonSociale(),
                "phoneNumber" => $supplier->getTelephone(),
                "poBox" => $supplier->getBoitePostale(),
                "address" => $supplier->getAdresse(),
                "accountingAccount" => $supplier->getCompteComptable(),
            ];
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 duplicate on supplier name
     * 102 unknown error
     */
    public function updateAction()
    {
        try {
            $id = $_POST["id"];
            $supplierName = $_POST["supplierName"];
            $societyType = $_POST["societyType"];
            $phoneNumber = $_POST["phoneNumber"];
            $poBox = $_POST["poBox"] != "" ? $_POST["poBox"] : null;
            $address = $_POST["address"] != "" ? $_POST["address"] : null;
            $accountingAccount = $_POST["accountingAccount"] != "" ? $_POST["accountingAccount"] : null;
            $supplierDao = new SupplierDAO();
            $supplier = $supplierDao->getSupplierById($id);
            if ($supplierDao->supplierNameAlreadyExists($supplierName, $supplier)) {
                echo ApiResponse::respond(101);
                exit();
            }
            $supplier->setNomFournisseur($supplierName);
            $supplier->setRaisonSociale($societyType);
            $supplier->setTelephone($phoneNumber);
            if ($poBox != null) {
                $supplier->setBoitePostale($poBox);
            }
            if ($address != null) {
                $supplier->setAdresse($address);
            }
            if ($accountingAccount != null) {
                $supplier->setCompteComptable($accountingAccount);
            }
            $supplierDao->update($supplier);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("supplier.index.supplierUpdatedSuccessfully")} : N°{$supplier->getIdFournisseur()}({$supplier->getNomFournisseur()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function deleteAction()
    {
        try {
            $id = (int)$_POST["id"];
            $supplierDao = new SupplierDAO();
            $supplier = $supplierDao->getSupplierById($id);
            if ($supplier->getStatut() == 0) {
                echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE);
                exit();
            }
            $supplier->setStatut(0);
            $supplierDao->update($supplier);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("supplier.index.supplierDeletedSuccessfully")} : N°{$supplier->getIdFournisseur()}({$supplier->getNomFournisseur()})"
            );
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }
}
