<?php

include APPLICATION_PATH . '/includes/daos.php';

class UserController extends Zend_Controller_Action
{
    private $translator;

    public function init()
    {
        session_start();
        SubscriptionChecker::checkSubscriptionValidity($this);
        if (isset($_SESSION[SESSION_USER_ID]) && $_SESSION[SESSION_USER_ID] != null) {
            $_SESSION[LAST_ACTION_DATE] = new DateTime();
            LogDAO::updateLastActionDateForCurrentUser();
            $this->translator = Translator::init();
        } else {
            if ($this->getRequest()->isPost()) {
                echo ApiResponse::respond(SESSION_EXPIRED_CODE);
                exit();
            } else {
                $this->_helper->redirector('index', 'Index');
            }
        }
    }

    public function indexAction()
    {
        AccessRightChecker::checkAccessRight($this, 18);
        $pointOfSaleDao = new PointOfSaleDAO();
        $monthlyPayModelDao = new MonthlyPayModelDAO();
        $profileDao = new ProfileDAO();
        $userDao = new UserDAO();
        $subscriptionPackageDao = new SubscriptionPackageDAO();
        $subscriptionPackage = $subscriptionPackageDao->getSubscriptionPackageById((int)SUBSCRIPTION_PACKAGE);
        $this->view->maxUsers = $subscriptionPackage->getCountUsers() + SUBSCRIPTION_MORE_COUNT_USERS;
        $this->view->currentUsers = $userDao->getUsersCount();
        $this->view->pointOfSales = $pointOfSaleDao->listPointOfSales();
        $this->view->monthlyPayModels = $monthlyPayModelDao->listMonthlyPayModels();
        $this->view->profiles = $profileDao->listProfiles();
        $this->view->translator = $this->translator;
    }

    public function datatableAction()
    {
        $name = $_POST["name"] != "" ? $_POST["name"] : null;
        $login = $_POST["login"] != "" ? $_POST["login"] : null;
        $profile = (int)$_POST["profile"] != 0 ? (int)$_POST["profile"] : null;
        $pointOfSale = (int)$_POST["pointOfSale"] != 0 ? (int)$_POST["pointOfSale"] : null;
        $whereName = "";
        $whereLogin = "";
        $whereProfile = "";
        $wherePointOfSale = "";
        if ($name != null) {
            $whereName .= " AND CONCAT(pe.nom, ' ', COALESCE(pe.prenom, '')) LIKE '%{$name}%'";
        }
        if ($login != null) {
            $whereLogin .= " AND u.login LIKE '%{$login}%'";
        }
        if ($profile != null) {
            $whereProfile .= " AND u.id_profil = {$profile}";
        }
        if ($pointOfSale != null) {
            $wherePointOfSale .= " AND u.id_point_vente = {$pointOfSale}";
        }
        $protectedLogin = PROTECTED_APP_LOGIN;
        $where = "u.statut = 1 AND u.login <> '{$protectedLogin}'{$whereName}{$whereLogin}{$whereProfile}{$wherePointOfSale}";
        $usersTable = "
                (
                    SELECT u.login AS user_login,
                    pr.nom AS user_profile, CONCAT(pe.nom, ' ', COALESCE(pe.prenom, '')) AS user_name,
                    u.id_utilisateur AS user_id, ps.nom_point_vente AS user_point_of_sale, u.last_login_date AS user_last_login,
                    u.last_logout_date AS user_last_logout, u.last_action_date AS user_last_activity,
                    u.profile_image AS user_profile_image
                    FROM utilisateur u JOIN profil pr ON u.id_profil = pr.id_profil JOIN personne pe ON
                    u.id_personne = pe.id_personne 
                    LEFT OUTER JOIN point_vente ps ON (u.id_point_vente = ps.id_point_vente OR ps.id_point_vente IS NULL)
                    WHERE {$where}
                ) AS usersTable
        ";
        $primaryKey = 'user_id';
        $columns = [
            ['db' => 'user_name', 'dt' => 0],
            ['db' => 'user_login', 'dt' => 1],
            ['db' => 'user_profile', 'dt' => 2],
            ['db' => 'user_point_of_sale', 'dt' => 3],
            ['db' => 'user_last_login', 'dt' => 4],
            ['db' => 'user_last_logout', 'dt' => 5],
            ['db' => 'user_last_activity', 'dt' => 6],
            ['db' => 'user_id', 'dt' => 7],
            ['db' => 'user_profile_image', 'dt' => 8],
        ];
        $computeDatatableResults = SSP::customSimple($_POST, $usersTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        $currentDateTime = new DateTime();
        foreach ($datatableResult["data"] as $row) {
            $userName = $row[0];
            $userLogin = $row[1];
            $userProfile = $row[2];
            $pointOfSale = $row[3] != null ? $row[3] : "
                <div class='ui label app-secondary-label'>
                    {$this->translator->_("user.index.notAttached")}
                </div>
            ";
            $userLastLogin = $row[4] != null ? new DateTime($row[4]) : null;
            $userLastLogout = $row[5] != null ? new DateTime($row[5]) : null;
            $userLastActivity = $row[6] != null ? new DateTime($row[6]) : null;
            $status = "";
            if ($userLastActivity != null && $userLastLogout != null) {
                if ($userLastLogout->format(DATE_FORMAT_WITH_SECONDS) == $userLastActivity->format(DATE_FORMAT_WITH_SECONDS)) {
                    $status = "
                        <div class='ui black label'>
                            {$this->translator->_("user.index.offline")}
                        </div>
                    ";
                } else {
                    $differenceInterval = $currentDateTime->diff($userLastActivity);
                    $minutesCount = $differenceInterval->days * 24 * 60;
                    $minutesCount += $differenceInterval->h * 60;
                    $minutesCount += $differenceInterval->i;
                    if ($minutesCount < DEFAULT_USER_SESSION_DURATION) {
                        $status = "
                            <div class='ui green label'>
                                {$this->translator->_("user.index.online")}
                            </div>
                        ";
                    } else {
                        $status = "
                            <div class='ui grey label'>
                                {$this->translator->_("user.index.probablyOffline")}
                            </div>
                        ";
                    }
                }
            } else {
                $status = "
                    <div class='ui brown label'>
                        {$this->translator->_("user.index.unknown")}
                    </div>
                ";
            }
            $userId = $row[7];
            $userProfileImage = $row[8];
            $actions = "
                <button title='{$this->translator->_("shared.consult")}' about='{$userId}' class='mini ui icon button app-primary-button app-consult-action'>
                    <i class='eye icon'></i>
                </button>
                <button title='{$this->translator->_("shared.edit")}' about='{$userId}' class='mini ui icon button app-secondary-button app-edit-action'>
                    <i class='edit icon'></i>
                </button>
                <button title='{$this->translator->_("shared.delete")}' about='{$userId}' class='mini ui red icon button app-delete-action'>
                    <i class='trash icon'></i>
                </button>
            ";
            $tableRow = [
                [
                    $userProfileImage,
                    $userId,
                ],
                $userName,
                $userLogin,
                $userProfile,
                $pointOfSale,
                $userLastLogin != null ? $userLastLogin->format(DATE_FORMAT_WITH_TIME) : $this->translator->_("shared.undefined"),
                $userLastLogout != null ? $userLastLogout->format(DATE_FORMAT_WITH_TIME) : $this->translator->_("shared.undefined"),
                $userLastActivity != null ? $userLastActivity->format(DATE_FORMAT_WITH_TIME) : $this->translator->_("shared.undefined"),
                $status,
                $actions,
            ];
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    /**
     * 100 success
     * 101 login not authorized
     * 102 duplicate login
     * 103 subscription limits reached
     * 104 unknown error
     */
    public function createAction()
    {
        $userDao = new UserDAO();
        $subscriptionPackageDao = new SubscriptionPackageDAO();
        $subscriptionPackage = $subscriptionPackageDao->getSubscriptionPackageById((int)SUBSCRIPTION_PACKAGE);
        $maxUsers = $subscriptionPackage->getCountUsers() + SUBSCRIPTION_MORE_COUNT_USERS;
        $currentUsers = $userDao->getUsersCount();
        if ($currentUsers >= $maxUsers) {
            echo ApiResponse::respond(103, null, ["maxUsers" => $maxUsers, "currentUsers" => $currentUsers]);
            exit();
        }
        try {
            $lastName = $_POST["lastName"];
            $firstName = $_POST["firstName"];
            $phoneNumber = $_POST["phoneNumber"];
            $email = $_POST["email"];
            $gender = $_POST["gender"];
            $dateOfBirth = $_POST["dateOfBirth"] != "" ? $_POST["dateOfBirth"] : null;
            $userIsACashier = (int)$_POST["userIsACashier"];
            $userIsAMasterCashier = (int)$_POST["userIsAMasterCashier"];
            $userCanMakeDiscounts = (int)$_POST["userCanMakeDiscounts"];
            $pointOfSaleId = (int)$_POST["pointOfSale"] != 0 ? (int)$_POST["pointOfSale"] : null;
            $userIsASalariedEmployee = (int)$_POST["userIsASalariedEmployee"];
            $monthlyPayModelId = (int)$_POST["monthlyPayModel"] != 0 ? (int)$_POST["monthlyPayModel"] : null;
            $ribNumber = $_POST["ribNumber"] != "" ? $_POST["ribNumber"] : null;
            $bank = $_POST["bank"] != "" ? $_POST["bank"] : null;
            $bankAgency = $_POST["bankAgency"] != "" ? $_POST["bankAgency"] : null;
            $login = $_POST["login"];
            $profileId = (int)$_POST["profile"];
            $personDao = new PersonDAO();
            if ($login == PROTECTED_APP_LOGIN) {
                echo ApiResponse::respond(101);
                exit();
            }
            if ($userDao->getUserByLogin($login) != null) {
                echo ApiResponse::respond(102);
                exit();
            }
            $person = $personDao->create();
            $person->setNom($lastName);
            $person->setPrenom($firstName);
            $person->setType(TYPE_PERSONNE_UTILISATEUR);
            $person->setNature(NATURE_CLIENT_PERSONNE_PHYSIQUE);
            $person->setSexe($gender);
            $person->setTelephone($phoneNumber);
            $person->setEmail($email);
            if ($dateOfBirth != null) {
                $person->setDateNaissance(new DateTime($dateOfBirth));
            }
            $person->setDateCreation(new DateTime());
            $person->setStatut(1);
            $personDao->save($person);
            $user = $userDao->create();
            $profileDao = new ProfileDAO();
            $profile = $profileDao->getProfileById($profileId);
            $user->setIdProfil($profile);
            $user->setLogin($login);
            $user->setPassword(sha1(md5($login)));
            $user->setCaissier($userIsACashier);
            $user->setIdPersonne($person);
            $user->setDroitReduction($userCanMakeDiscounts);
            if ($pointOfSaleId != null) {
                $pointOfSaleDao = new PointOfSaleDAO();
                $pointOfSale = $pointOfSaleDao->getPointOfSaleById($pointOfSaleId);
                $user->setIdPointVente($pointOfSale);
            }
            $user->setCaissierPrincipal($userIsAMasterCashier);
            if ($monthlyPayModelId != null) {
                $monthlyPayModalDao = new MonthlyPayModelDAO();
                $monthlyPayModel = $monthlyPayModalDao->getMonthlyPayModelById($monthlyPayModelId);
                $user->setIdModelePaie($monthlyPayModel);
            }
            $user->setEmploye($userIsASalariedEmployee);
            $user->setRib($ribNumber);
            $user->setBank($bank);
            $user->setBankAgency($bankAgency);
            $user->setStatut(1);
            $userDao->save($user);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("user.index.userSuccessfullyRegistered")} : N°{$user->getIdUtilisateur()}({$user->getLogin()}:{$person->getNom()} {$person->getPrenom()})"
            );
            $currentUsers = $userDao->getUsersCount();
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["maxUsers" => $maxUsers, "currentUsers" => $currentUsers]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(104, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function detailAction()
    {
        try {
            $id = (int)$_POST["id"];
            $userDao = new UserDAO();
            $user = $userDao->getUserById($id);
            $person = $user->getIdPersonne();
            $result = [
                "lastName" => $person->getNom(),
                "firstName" => $person->getPrenom(),
                "phoneNumber" => $person->getTelephone(),
                "email" => $person->getEmail(),
                "gender" => $person->getSexe(),
                "dateOfBirth" => $person->getDateNaissance() != null ? $person->getDateNaissance()->format(DATE_FORMAT) : "",
                "userIsACashier" => $user->getCaissier() != null ? $user->getCaissier() : UTILISATEUR_NON_CAISSIER,
                "userIsAMasterCashier" => $user->getCaissierPrincipal() != null ? $user->getCaissierPrincipal() : 0,
                "userCanMakeDiscounts" => $user->getDroitReduction(),
                "pointOfSale" => $user->getIdPointVente() != null ? $user->getIdPointVente()->getIdPointVente() : 0,
                "userIsASalariedEmployee" => $user->getEmploye(),
                "monthlyPayModel" => $user->getIdModelePaie() != null ? $user->getIdModelePaie()->getIdModelePaie() : 0,
                "ribNumber" => $user->getRib(),
                "bank" => $user->getBank(),
                "bankAgency" => $user->getBankAgency(),
                "profile" => $user->getIdProfil()->getIdProfil(),
                "login" => $user->getLogin(),
                "profileImage" => $user->getProfileImage() != null ? $user->getProfileImage() : "",
            ];
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function updateAction()
    {
        try {
            $id = (int)$_POST["id"];
            $lastName = $_POST["lastName"];
            $firstName = $_POST["firstName"];
            $phoneNumber = $_POST["phoneNumber"];
            $email = $_POST["email"];
            $gender = $_POST["gender"];
            $dateOfBirth = $_POST["dateOfBirth"] != "" ? $_POST["dateOfBirth"] : null;
            $userIsACashier = (int)$_POST["userIsACashier"];
            $userIsAMasterCashier = (int)$_POST["userIsAMasterCashier"];
            $userCanMakeDiscounts = (int)$_POST["userCanMakeDiscounts"];
            $pointOfSaleId = (int)$_POST["pointOfSale"] != 0 ? (int)$_POST["pointOfSale"] : null;
            $userIsASalariedEmployee = (int)$_POST["userIsASalariedEmployee"];
            $monthlyPayModelId = (int)$_POST["monthlyPayModel"] != 0 ? (int)$_POST["monthlyPayModel"] : null;
            $ribNumber = $_POST["ribNumber"] != "" ? $_POST["ribNumber"] : null;
            $bank = $_POST["bank"] != "" ? $_POST["bank"] : null;
            $bankAgency = $_POST["bankAgency"] ? $_POST["bankAgency"] : null;
            $profileId = (int)$_POST["profile"];
            $resetPassword = (int)$_POST["resetPassword"] != 0 ? (int)$_POST["resetPassword"] : null;
            $userDao = new UserDAO();
            $personDao = new PersonDAO();
            $user = $userDao->getUserById($id);
            $person = $user->getIdPersonne();
            $person->setNom($lastName);
            $person->setPrenom($firstName);
            $person->setType(TYPE_PERSONNE_UTILISATEUR);
            $person->setNature(NATURE_CLIENT_PERSONNE_PHYSIQUE);
            $person->setSexe($gender);
            $person->setTelephone($phoneNumber);
            $person->setEmail($email);
            if ($dateOfBirth != null) {
                $person->setDateNaissance(new DateTime($dateOfBirth));
            } else {
                $person->setDateNaissance(null);
            }
            $person->setDateCreation(new DateTime());
            $personDao->update($person);
            $profileDao = new ProfileDAO();
            $profile = $profileDao->getProfileById($profileId);
            $user->setIdProfil($profile);
            if ($resetPassword != null) {
                $user->setPassword(sha1(md5($user->getLogin())));
            }
            $user->setCaissier($userIsACashier);
            $user->setIdPersonne($person);
            $user->setDroitReduction($userCanMakeDiscounts);
            if ($pointOfSaleId != null) {
                $pointOfSaleDao = new PointOfSaleDAO();
                $pointOfSale = $pointOfSaleDao->getPointOfSaleById($pointOfSaleId);
                $user->setIdPointVente($pointOfSale);
            } else {
                $user->setIdPointVente(null);
            }
            $user->setCaissierPrincipal($userIsAMasterCashier);
            if ($monthlyPayModelId != null) {
                $monthlyPayModalDao = new MonthlyPayModelDAO();
                $monthlyPayModel = $monthlyPayModalDao->getMonthlyPayModelById($monthlyPayModelId);
                $user->setIdModelePaie($monthlyPayModel);
            } else {
                $user->setIdModelePaie(null);
            }
            $user->setRib($ribNumber);
            $user->setBank($bank);
            $user->setBankAgency($bankAgency);
            $user->setEmploye($userIsASalariedEmployee);
            $userDao->update($user);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("user.index.userUpdatedSuccessfully")} : N°{$user->getIdUtilisateur()}({$user->getLogin()}:{$person->getNom()} {$person->getPrenom()})"
            );
            $subscriptionPackageDao = new SubscriptionPackageDAO();
            $subscriptionPackage = $subscriptionPackageDao->getSubscriptionPackageById((int)SUBSCRIPTION_PACKAGE);
            $maxUsers = $subscriptionPackage->getCountUsers() + SUBSCRIPTION_MORE_COUNT_USERS;
            $currentUsers = $userDao->getUsersCount();
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["maxUsers" => $maxUsers, "currentUsers" => $currentUsers]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function deleteAction()
    {
        try {
            $id = (int)$_POST["id"];
            $userDao = new UserDAO();
            $user = $userDao->getUserById($id);
            if ($user->getStatut() == 0) {
                echo ApiResponse::respond(ALREADY_DELETED_ISSUE_CODE);
                exit();
            }
            $person = $user->getIdPersonne();
            $user->setStatut(0);
            $userDao->update($user);
            LogDAO::registerOperationForCurrentUser(
                "{$this->translator->_("user.index.userDeletedSuccessfully")} : N°{$user->getIdUtilisateur()}({$user->getLogin()}:{$person->getNom()} {$person->getPrenom()})"
            );
            $subscriptionPackageDao = new SubscriptionPackageDAO();
            $subscriptionPackage = $subscriptionPackageDao->getSubscriptionPackageById((int)SUBSCRIPTION_PACKAGE);
            $maxUsers = $subscriptionPackage->getCountUsers() + SUBSCRIPTION_MORE_COUNT_USERS;
            $currentUsers = $userDao->getUsersCount();
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["maxUsers" => $maxUsers, "currentUsers" => $currentUsers]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unkwown error
     */
    public function updateprofileimageAction()
    {
        try {
            $userDao = new UserDAO();
            $currentUser = LogDAO::getCurrentUser();
            if (isset($_FILES["loggedUserProfileImageInput"]) && $_FILES["loggedUserProfileImageInput"]["name"] != "") {
                $uploadedFilename = $_FILES["loggedUserProfileImageInput"]["name"];
                $uploadFilenameArray = explode(".", $uploadedFilename);
                $extension = end($uploadFilenameArray);
                $currentDate = new DateTime();
                $randomName = $currentDate->format("YmdHis");
                $filename = PUBLIC_PATH . "/images/users/{$randomName}_{$currentUser->getIdUtilisateur()}.{$extension}";
                if ($currentUser->getProfileImage() != null) {
                    error_reporting(E_ERROR | E_PARSE);
                    unlink(PUBLIC_PATH . "/images/users/{$currentUser->getProfileImage()}");
                }
                $currentUser->setProfileImage("{$randomName}_{$currentUser->getIdUtilisateur()}.{$extension}");
                move_uploaded_file($_FILES["loggedUserProfileImageInput"]["tmp_name"], $filename);
                $userDao->update($currentUser);
                $_SESSION[SESSION_USER_PROFILE_IMAGE] = $currentUser->getProfileImage();
            }
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function loadcashiersAction()
    {
        try {
            $pointOfSaleDao = new PointOfSaleDAO();
            $userDao = new UserDAO();
            $pointOfSaleIds = json_decode($_POST["pointsOfSale"], true);
            $pointsOfSale = [];
            foreach ($pointOfSaleIds as $pointOfSaleId) {
                $pointsOfSale[] = $pointOfSaleDao->getPointOfSaleById((int)$pointOfSaleId);
            }
            $cashiers = [];
            if (count($pointsOfSale) == 1) {
                $cashiers = $userDao->listCashiers($pointsOfSale[0]);
            } else if (count($pointsOfSale) == 0) {
                $cashiers = $userDao->listCashiers();
            } else {
                $cashiers = $userDao->listCashiersOfPointsOfSale($pointsOfSale);
            }
            $result = [];
            foreach ($cashiers as $cashier) {
                $result[] = [
                    "id" => $cashier->getIdUtilisateur(),
                    "name" => "{$cashier->getIdPersonne()->getNom()} {$cashier->getIdPersonne()->getPrenom()}",
                ];
            }
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function twofactoroncodeAction()
    {
        try {
            $userDao = new UserDAO();
            $generatedOtp = Utility::generateOtp(6);
            $currentUser = LogDAO::getCurrentUser();
            $currentUser->setTwoFactorOtp($generatedOtp);
            $userDao->update($currentUser);
            $primaryBrand = APP_PRIMARY_BRAND;
            $secondaryBrand = APP_SECONDARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $currentDate = new DateTime();
            $currentYear = $currentDate->format("Y");
            $applicationName = NOM_APPLICATION;
            $mail = new PHPMailer\PHPMailer\PHPMailer();
            $mail->isSMTP();
            $mail->Host = CRESCITA_DEFAULT_MAIL_DOMAIN;
            $mail->SMTPAuth = true;
            $mail->Username = CRESCITA_DEFAULT_MAIL_EMAIL;
            $mail->Password = CRESCITA_DEFAULT_MAIL_APP_PASSWORD;
            $mail->SMTPSecure = PHPMailer\PHPMailer\PHPMailer::ENCRYPTION_SMTPS;
            $mail->Port = (int)CRESCITA_DEFAULT_MAIL_PORT;
            $mail->setFrom(CRESCITA_DEFAULT_MAIL_EMAIL, $applicationName);
            $mail->addReplyTo('no-reply@crescita.com');
            $mail->addAddress($currentUser->getIdPersonne()->getEmail(), "{$currentUser->getIdPersonne()->getNom()} {$currentUser->getIdPersonne()->getPrenom()}");
            $mail->isHTML(true);
            $mail->CharSet = "UTF-8";
            $mail->Subject = $this->translator->_("shared.doubleAuthentication");
            $emailHtml = "
                <html lang='en'>
                    <head>
                        <style>
                            .main-table {
                                border: none; 
                                margin: 50px; 
                                background-color: {$primaryBrand}55;
                                min-width: 90%;
                            }

                            @media screen and (max-width:600px) {
                                .main-table {
                                    margin: 0;
                                }
                            }
                        </style>
                    </head>
                    <body>
                        <div style='background-image: url(cid:app_background); display: flex; justify-content: center;'>
                            <table class='main-table'>
                                <tr>
                                    <div style='border-width: 2px; border-style: solid; border-color: {$secondaryBrand}; padding: 1em 1em; border-radius: 4px;'>
                                        <p style='text-align: center;'>
                                            <i style='color: {$thirdBrand}; background-color: {$primaryBrand}; font-style: italic; font-size: 16px; padding: 5px; border-radius: 6px;'>
                                                {$applicationName}
                                            </i>
                                        </p>
                                        <p style='text-align: center;'>
                                            <img style='width: 125px; height: 100px; border-radius: 6px;' src='/images/assets/app_logo.png' />
                                        </p>
                                        <p style='color: {$thirdBrand}; text-align: center; font-weight: bold; font-size: 16px;'>{$this->translator->_("user.twoFactorOnCode.youAreReceivingThisEmailBecauseYouHaveAskedForDoubleAuthenticationToBeActivatedOnYourAccount")}<p>
                                        <p style='color: {$thirdBrand}; text-align: center; font-weight: bold; font-size: 16px;'>{$this->translator->_("shared.theVerificationCodeIs")}:</p>
                                        <br/>
                                        <p style='color: {$secondaryBrand}; text-align: center; font-weight: bold; font-size: 18px;'>{$generatedOtp}</p>
                                    </div>
                                </tr>
                                <tr>
                                    <div style='border-width: 1.5px; border-style: solid; border-color: {$primaryBrand}; border-radius: 6px;'></div>
                                    <p style='text-align: center;'>
                                        <i style='font-weight: bold; font-size: 20px; color: {$thirdBrand};'>Crescita</i>
                                        <br/>
                                        <i style='color: {$thirdBrand}'>&copy; 2022-{$currentYear} Crescita</i>
                                    </p>
                                </tr>
                            </table>
                        </div>
                    </body>
                </html>
            ";
            $mail->msgHTML($emailHtml, PUBLIC_PATH);
            $mail->addEmbeddedImage(PUBLIC_PATH . '/images/assets/app_background.png', 'app_background');
            $mail->AltBody = "{$this->translator->_("user.twoFactorOnCode.youAreReceivingThisEmailBecauseYouHaveAskedForDoubleAuthenticationToBeActivatedOnYourAccount")} {$this->translator->_("shared.theVerificationCodeIs")}: {$generatedOtp}";
            $mail->send();
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function twofactoroffcodeAction()
    {
        try {
            $userDao = new UserDAO();
            $generatedOtp = Utility::generateOtp(6);
            $currentUser = LogDAO::getCurrentUser();
            $currentUser->setTwoFactorOtp($generatedOtp);
            $userDao->update($currentUser);
            $primaryBrand = APP_PRIMARY_BRAND;
            $secondaryBrand = APP_SECONDARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $currentDate = new DateTime();
            $currentYear = $currentDate->format("Y");
            $applicationName = NOM_APPLICATION;
            $mail = new PHPMailer\PHPMailer\PHPMailer();
            $mail->isSMTP();
            $mail->Host = CRESCITA_DEFAULT_MAIL_DOMAIN;
            $mail->SMTPAuth = true;
            $mail->Username = CRESCITA_DEFAULT_MAIL_EMAIL;
            $mail->Password = CRESCITA_DEFAULT_MAIL_APP_PASSWORD;
            $mail->SMTPSecure = PHPMailer\PHPMailer\PHPMailer::ENCRYPTION_SMTPS;
            $mail->Port = (int)CRESCITA_DEFAULT_MAIL_PORT;
            $mail->setFrom(CRESCITA_DEFAULT_MAIL_EMAIL, $applicationName);
            $mail->addReplyTo('no-reply@crescita.com');
            $mail->addAddress($currentUser->getIdPersonne()->getEmail(), "{$currentUser->getIdPersonne()->getNom()} {$currentUser->getIdPersonne()->getPrenom()}");
            $mail->isHTML(true);
            $mail->CharSet = "UTF-8";
            $mail->Subject = $this->translator->_("shared.doubleAuthentication");
            $emailHtml = "
                <html lang='en'>
                    <head>
                        <style>
                            .main-table {
                                border: none; 
                                margin: 50px; 
                                background-color: {$primaryBrand}55;
                                min-width: 90%;
                            }

                            @media screen and (max-width:600px) {
                                .main-table {
                                    margin: 0;
                                }
                            }
                        </style>
                    </head>
                    <body>
                        <div style='background-image: url(cid:app_background); display: flex; justify-content: center;'>
                            <table class='main-table'>
                                <tr>
                                    <div style='border-width: 2px; border-style: solid; border-color: {$secondaryBrand}; padding: 1em 1em; border-radius: 4px;'>
                                        <p style='text-align: center;'>
                                            <i style='color: {$thirdBrand}; background-color: {$primaryBrand}; font-style: italic; font-size: 16px; padding: 5px; border-radius: 6px;'>
                                                {$applicationName}
                                            </i>
                                        </p>
                                        <p style='text-align: center;'>
                                            <img style='width: 125px; height: 100px; border-radius: 6px;' src='/images/assets/app_logo.png' />
                                        </p>
                                        <p style='color: {$thirdBrand}; text-align: center; font-weight: bold; font-size: 16px;'>{$this->translator->_("user.twoFactorOffCode.youAreReceivingThisEmailBecauseYouHaveAskedForDoubleAuthenticationToBeDeActivatedOnYourAccount")}<p>
                                        <p style='color: {$thirdBrand}; text-align: center; font-weight: bold; font-size: 16px;'>{$this->translator->_("shared.theVerificationCodeIs")}:</p>
                                        <br/>
                                        <p style='color: {$secondaryBrand}; text-align: center; font-weight: bold; font-size: 18px;'>{$generatedOtp}</p>
                                    </div>
                                </tr>
                                <tr>
                                    <div style='border-width: 1.5px; border-style: solid; border-color: {$primaryBrand}; border-radius: 6px;'></div>
                                    <p style='text-align: center;'>
                                        <i style='font-weight: bold; font-size: 20px; color: {$thirdBrand};'>Crescita</i>
                                        <br/>
                                        <i style='color: {$thirdBrand}'>&copy; 2022-{$currentYear} Crescita</i>
                                    </p>
                                </tr>
                            </table>
                        </div>
                    </body>
                </html>
            ";
            $mail->msgHTML($emailHtml, PUBLIC_PATH);
            $mail->addEmbeddedImage(PUBLIC_PATH . '/images/assets/app_background.png', 'app_background');
            $mail->AltBody = "{$this->translator->_("user.twoFactorOffCode.youAreReceivingThisEmailBecauseYouHaveAskedForDoubleAuthenticationToBeDeActivatedOnYourAccount")} {$this->translator->_("shared.theVerificationCodeIs")}: {$generatedOtp}";
            $mail->send();
            echo ApiResponse::respond();
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 incorrect verification code
     * 102 unknown error
     */
    public function enabletwofactorAction()
    {
        try {
            $verificationCode = $_POST["verificationCode"];
            $currentUser = LogDAO::getCurrentUser();
            if ($currentUser->getTwoFactorOtp() == $verificationCode) {
                $currentUser->setTwoFactorEnabled(1);
                $userDao = new UserDAO();
                $userDao->update($currentUser);
                $_SESSION[SESSION_USER_TWO_FACTOR_ENABLED] = 1;
                LogDAO::registerOperationForCurrentUser(
                    $this->translator->_("shared.doubleAuthenticationActivated")
                );
                echo ApiResponse::respond();
            } else {
                echo ApiResponse::respond(101);
            }
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 incorrect verification code
     * 102 unknown error
     */
    public function disabletwofactorAction()
    {
        try {
            $verificationCode = $_POST["verificationCode"];
            $currentUser = LogDAO::getCurrentUser();
            if ($currentUser->getTwoFactorOtp() == $verificationCode) {
                $currentUser->setTwoFactorEnabled(0);
                $userDao = new UserDAO();
                $userDao->update($currentUser);
                $_SESSION[SESSION_USER_TWO_FACTOR_ENABLED] = 0;
                LogDAO::registerOperationForCurrentUser(
                    $this->translator->_("shared.doubleAuthenticationDeactivated")
                );
                echo ApiResponse::respond();
            } else {
                echo ApiResponse::respond(101);
            }
        } catch (Exception $ex) {
            echo ApiResponse::respond(102, $ex->getMessage());
        }
        exit();
    }
}
