<?php

//All app reports are located here

include APPLICATION_PATH . '/includes/daos.php';

class ViewController extends Zend_Controller_Action
{
    private $translator;

    public function init()
    {
        session_start();
        SubscriptionChecker::checkSubscriptionValidity($this);
        if (isset($_SESSION[SESSION_USER_ID]) && $_SESSION[SESSION_USER_ID] != null) {
            $_SESSION[LAST_ACTION_DATE] = new DateTime();
            LogDAO::updateLastActionDateForCurrentUser();
            $this->translator = Translator::init();
        } else {
            if ($this->getRequest()->isPost()) {
                echo ApiResponse::respond(SESSION_EXPIRED_CODE);
                exit();
            } else {
                $this->_helper->redirector('index', 'Index');
            }
        }
    }

    public function stockAction()
    {
        AccessRightChecker::checkAccessRight($this, 50);
        $storeDao = new StoreDAO();
        $currentUser = LogDAO::getCurrentUser();
        $stores = [];
        if ($currentUser->getIdPointVente() == null || in_array($currentUser->getIdProfil()->getCode(), IDENTIFIED_PROFILES_CODES)) {
            $stores = $storeDao->getAllStores();
        } else {
            $stores[] = $currentUser->getIdPointVente()->getIdMagasin();
        }
        $this->view->stores = $stores;
        $this->view->translator = $this->translator;
    }

    public function datatablestockAction()
    {
        $store = (int)$_POST["store"] != 0 ? (int)$_POST["store"] : null;
        $name = $_POST["name"] != "" ? $_POST["name"] : null;
        $batchNumber = $_POST["batchNumber"] != "" ? $_POST["batchNumber"] : null;
        $from = $_POST["from"] != "" ? $_POST["from"] : null;
        $to = $_POST["to"] != "" ? $_POST["to"] : null;
        $whereStore = "";
        $whereName = "";
        $whereBatchNumber = "";
        $whereFrom = "";
        $whereTo = "";
        if ($store != null) {
            $whereStore .= " AND s.id_magasin = {$store}";
        }
        if ($name != null) {
            $whereName .= " AND s.designation LIKE '%{$name}%'";
        }
        if ($batchNumber != null) {
            $whereBatchNumber .= " AND s.num_lot LIKE '%{$batchNumber}%'";
        }
        if ($from != null) {
            $whereFrom .= " AND s.date_peremption >= '{$from} 00:00:00'";
        }
        if ($to != null) {
            $whereTo .= " AND s.date_peremption <= '{$to} 23:59:59'";
        }
        $where = "s.statut = 1 AND s.quantite > 0 AND p.statut = 1{$whereStore}{$whereName}{$whereBatchNumber}{$whereFrom}{$whereTo}";
        $stocksTable = "
                (
                    SELECT s.id_stock AS stock_id, s.designation AS product_name, m.nom_magasin AS store_name, s.num_lot AS batch_number, s.quantite AS quantity,
                    s.date_peremption AS expiration_date, s.prix_achat AS buying_price, s.prix_vente AS sale_price,
                    s.quantite_entree AS initial_quantity, p.quantite_alerte AS alert_quantity
                    FROM stock s JOIN magasin m ON s.id_magasin = m.id_magasin JOIN produit p ON s.id_produit = p.id_produit
                    WHERE {$where}
                ) AS stocksTable
        ";
        $primaryKey = 'stock_id';
        $columns = [];
        if (GESTION_DATE_PEREMPTION_PRODUIT == 1) {
            $columns = [
                ['db' => 'product_name', 'dt' => 0],
                ['db' => 'store_name', 'dt' => 1],
                ['db' => 'batch_number', 'dt' => 2],
                ['db' => 'quantity', 'dt' => 3],
                ['db' => 'expiration_date', 'dt' => 4],
                ['db' => 'initial_quantity', 'dt' => 5],
                ['db' => 'buying_price', 'dt' => 6],
                ['db' => 'sale_price', 'dt' => 7],
                ['db' => 'alert_quantity', 'dt' => 8],
                ['db' => 'stock_id', 'dt' => 9],
            ];
        } else {
            $columns = [
                ['db' => 'product_name', 'dt' => 0],
                ['db' => 'store_name', 'dt' => 1],
                ['db' => 'batch_number', 'dt' => 2],
                ['db' => 'quantity', 'dt' => 3],
                ['db' => 'initial_quantity', 'dt' => 4],
                ['db' => 'buying_price', 'dt' => 5],
                ['db' => 'sale_price', 'dt' => 6],
                ['db' => 'alert_quantity', 'dt' => 7],
                ['db' => 'stock_id', 'dt' => 8],
            ];
        }

        $computeDatatableResults = SSP::customSimple($_POST, $stocksTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        foreach ($datatableResult["data"] as $row) {
            $productName = $row[0];
            $storeName = $row[1];
            $stockId = GESTION_DATE_PEREMPTION_PRODUIT == 1 ? $row[9] : $row[8];
            $batchNumber = $row[2];
            $quantity = (int)$row[3];
            $stockDao = new StockDAO();
            $stock = $stockDao->getStockById($stockId);
            $expirationDate = GESTION_DATE_PEREMPTION_PRODUIT == 1 && $stock->getIdProduit()->getPerissable() == 1 && $row[4] != null ? $row[4] : null;
            $initialQuantity = GESTION_DATE_PEREMPTION_PRODUIT == 1 ? (int)$row[5] : (int)$row[4];
            $buyingPrice = GESTION_DATE_PEREMPTION_PRODUIT == 1 ? (float)$row[6] : (float)$row[5];
            $salePrice = GESTION_DATE_PEREMPTION_PRODUIT == 1 ? (float)$row[7] : (float)$row[6];
            $totalBuyingPrice = $initialQuantity * $buyingPrice;
            $totalSalePrice = ($initialQuantity - $quantity) * $salePrice;
            $alertQuantity = GESTION_DATE_PEREMPTION_PRODUIT == 1 ? $row[8] : $row[7];
            $alertQuantity = $alertQuantity != null ? (int)$alertQuantity : (int)QUANTITE_SEUIL_PRODUIT_PAR_DEFAUT;
            $backgroundColor = "";
            if (GESTION_DATE_PEREMPTION_PRODUIT == 1) {
                if ($expirationDate != null) {
                    $effectiveExpirationDate = new DateTime($expirationDate);
                    $currentDate = new DateTime();
                    $differenceInterval = $effectiveExpirationDate->diff($currentDate);
                    $daysCount = -1 * $differenceInterval->format('%R%a');
                    if ($daysCount <= 0) {
                        $backgroundColor = COULEUR_PRODUIT_PERIME;
                        if ($quantity <= $alertQuantity) {
                            $backgroundColor = "linear-gradient(to bottom, " . COULEUR_PRODUIT_SEUIL_ATTEINT . " 10%, " . COULEUR_PRODUIT_PERIME . " 70%)";
                        }
                    } else {
                        if ($quantity <= $alertQuantity) {
                            $backgroundColor = COULEUR_PRODUIT_SEUIL_ATTEINT;
                        } else {
                            if ($daysCount <= NOMBRE_JOUR_POUR_ALERTE_PEREMPTION) {
                                $backgroundColor = COULEUR_PRODUIT_PROCHE_DATE_PEREMPTION;
                            }
                        }
                    }
                } else {
                    if ($quantity <= $alertQuantity) {
                        $backgroundColor = COULEUR_PRODUIT_SEUIL_ATTEINT;
                    }
                }
            } else {
                if ($quantity <= $alertQuantity) {
                    $backgroundColor = COULEUR_PRODUIT_SEUIL_ATTEINT;
                }
            }
            $tableRow = [];
            if (GESTION_DATE_PEREMPTION_PRODUIT == 1) {
                $effectiveExpirationDate = $expirationDate != null ? new DateTime($expirationDate) : null;
                $tableRow = [
                    $productName,
                    $storeName,
                    $batchNumber,
                    $quantity,
                    $effectiveExpirationDate != null ? $effectiveExpirationDate->format(DATE_FORMAT) : null,
                    $initialQuantity,
                    number_format($buyingPrice, 0, '', ' '),
                    number_format($salePrice, 0, '', ' '),
                    number_format($totalBuyingPrice, 0, '', ' '),
                    number_format($totalSalePrice, 0, '', ' '),
                    $backgroundColor,
                ];
            } else {
                $tableRow = [
                    $productName,
                    $storeName,
                    $batchNumber,
                    $quantity,
                    $initialQuantity,
                    number_format($buyingPrice, 0, '', ' '),
                    number_format($salePrice, 0, '', ' '),
                    number_format($totalBuyingPrice, 0, '', ' '),
                    number_format($totalSalePrice, 0, '', ' '),
                    $backgroundColor,
                ];
            }
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    public function cumulativestockAction()
    {
        AccessRightChecker::checkAccessRight($this, 51);
        $storeDao = new StoreDAO();
        $currentUser = LogDAO::getCurrentUser();
        $stores = [];
        if ($currentUser->getIdPointVente() == null || in_array($currentUser->getIdProfil()->getCode(), IDENTIFIED_PROFILES_CODES)) {
            $stores = $storeDao->getAllStores();
        } else {
            $stores[] = $currentUser->getIdPointVente()->getIdMagasin();
        }
        $this->view->stores = $stores;
        $this->view->translator = $this->translator;
    }

    public function datatablecumulativestockAction()
    {
        $store = (int)$_POST["store"] != 0 ? (int)$_POST["store"] : null;
        $stockAtTheDateOf = $_POST["stockAtTheDateOf"];
        $name = $_POST["name"] != "" ? $_POST["name"] : null;
        $whereStore = "";
        $whereName = "";
        if ($store != null) {
            $whereStore .= " AND m.id_magasin = {$store}";
        }
        if ($name != null) {
            $whereName .= " AND p.designation LIKE '%{$name}%'";
        }
        $where = "summary.quantity > 0 AND p.statut = 1 AND m.statut = 1{$whereStore}{$whereName}";
        $cumulativeStocksTable = "
                (
                    SELECT summary.cumulative_stock_id AS cumulative_stock_id, summary.product_id AS product_id, summary.store_id AS store_id,
                    summary.quantity AS quantity, p.designation AS product_name, m.nom_magasin AS store_name, p.prix_achat AS buying_price, 
                    p.prix_vente AS sale_price, p.quantite_alerte AS alert_quantity
                    FROM
                    (
                        SELECT entries.cumulative_stock_id AS cumulative_stock_id, entries.product_id AS product_id, entries.store_id AS store_id,
                        (entries.entry_quantity - ifnull(exits.exit_quantity, 0)) AS quantity
                        FROM
                        (
                            SELECT CONCAT(p.id_produit, '_', m.id_magasin) AS cumulative_stock_id, p.id_produit AS product_id, SUM(lr.quantite) AS entry_quantity, m.id_magasin AS store_id
                            FROM ligne_reception lr JOIN produit p ON lr.id_produit = p.id_produit JOIN reception r ON r.id_reception = lr.id_reception
                            JOIN magasin m ON r.id_magasin = m.id_magasin
                            WHERE p.statut = 1 AND r.statut = 1 AND r.valider = 1 AND r.date_saisie <= '{$stockAtTheDateOf} 23:59:59'
                            GROUP BY p.id_produit, m.id_magasin
                        ) AS entries
                        LEFT OUTER JOIN
                        (
                            SELECT CONCAT(p.id_produit, '_', m.id_magasin) AS cumulative_stock_id, p.id_produit AS product_id, SUM(ls.quantite) AS exit_quantity, m.id_magasin AS store_id
                            FROM ligne_sortie ls JOIN stock st ON ls.id_stock = st.id_stock JOIN produit p ON st.id_produit = p.id_produit 
                            JOIN sortie so ON ls.id_sortie = so.id_sortie JOIN magasin m ON so.id_magasin = m.id_magasin
                            WHERE p.statut = 1 AND so.statut = 1 AND so.valider = 1 AND so.date_saisie <= '{$stockAtTheDateOf} 23:59:59'
                            GROUP BY p.id_produit, m.id_magasin
                        ) AS exits
                        ON (entries.cumulative_stock_id = exits.cumulative_stock_id OR exits.cumulative_stock_id IS NULL) 
                        AND (entries.product_id = exits.product_id OR exits.product_id IS NULL)
                        AND (entries.store_id = exits.store_id OR exits.store_id IS NULL)
                    ) AS summary
                    JOIN produit p ON summary.product_id = p.id_produit JOIN magasin m ON summary.store_id = m.id_magasin
                    WHERE {$where}
                ) AS cumulativeStocksTable
        ";
        $primaryKey = 'cumulative_stock_id';
        $columns = [
            ['db' => 'product_name', 'dt' => 0],
            ['db' => 'store_name', 'dt' => 1],
            ['db' => 'quantity', 'dt' => 2],
            ['db' => 'buying_price', 'dt' => 3],
            ['db' => 'sale_price', 'dt' => 4],
            ['db' => 'alert_quantity', 'dt' => 5],
        ];

        $computeDatatableResults = SSP::customSimple($_POST, $cumulativeStocksTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        foreach ($datatableResult["data"] as $row) {
            $productName = $row[0];
            $storeName = $row[1];
            $quantity = (int)$row[2];
            $buyingPrice = (float)$row[3];
            $salePrice = (float)$row[4];
            $alertQuantity = (int)$row[5];
            $tableRow = [
                $productName,
                $storeName,
                $quantity,
                number_format($buyingPrice, 0, '', ' '),
                number_format($salePrice, 0, '', ' '),
                number_format($quantity * $buyingPrice, 0, '', ' '),
                $quantity <= $alertQuantity ? COULEUR_PRODUIT_SEUIL_ATTEINT : "",
            ];
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    public function productsandservicesAction()
    {
        AccessRightChecker::checkAccessRight($this, 145);
        $productCategoryDao = new ProductCategoryDAO();
        $this->view->categories = $productCategoryDao->getAllProductCategories();
        $this->view->translator = $this->translator;
    }

    public function datatableproductsandservicesAction()
    {
        $name = $_POST["name"] != "" ? $_POST["name"] : null;
        $category = (int)$_POST["category"] != 0 ? (int)$_POST["category"] : null;
        $reference = $_POST["reference"] != "" ? $_POST["reference"] : null;
        $type = $_POST["type"] != "" ? (int)$_POST["type"] : null;
        $whereName = " AND p.designation LIKE '%{$name}%'";
        $whereCategory = "";
        $whereReference = "";
        $whereType = "";
        if (!is_null($name)) {
            $whereName .= " AND p.designation LIKE '%{$name}%'";
        }
        if (!is_null($category)) {
            $whereCategory .= " AND p.id_categorie_produit = {$category}";
        }
        if (!is_null($reference)) {
            $whereReference .= " AND p.reference_produit LIKE '%{$reference}%'";
        }
        if (!is_null($type)) {
            $whereType .= " AND p.stockable = {$type}";
        }
        $where = "p.statut = 1{$whereName}{$whereCategory}{$whereReference}{$whereType}";
        $productsTable = "
                (
                    SELECT p.id_produit AS product_id,
                    p.designation AS product_designation, p.prix_vente AS product_sale_price, 
                    p.prix_vente2 AS product_sale_price_2, p.prix_vente3 AS product_sale_price_3, 
                    p.prix_achat AS product_buying_price, p.reference_produit AS product_reference, 
                    c.nom AS category_name, p.stockable AS product_type
                    FROM produit p JOIN categorie_produit c ON p.id_categorie_produit = c.id_categorie 
                    WHERE {$where}
                    ORDER BY c.id_categorie ASC, p.designation ASC
                ) AS productsTable
        ";
        $primaryKey = 'product_id';
        $columns = [
            ['db' => 'product_designation', 'dt' => 0],
            ['db' => 'category_name', 'dt' => 1],
            ['db' => 'product_sale_price', 'dt' => 2],
            ['db' => 'product_sale_price_2', 'dt' => 3],
            ['db' => 'product_sale_price_3', 'dt' => 4],
            ['db' => 'product_buying_price', 'dt' => 5],
            ['db' => 'product_id', 'dt' => 6],
            ['db' => 'product_reference', 'dt' => 7],
            ['db' => 'product_type', 'dt' => 8],
        ];
        $computeDatatableResults = SSP::customSimple($_POST, $productsTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        foreach ($datatableResult["data"] as $row) {
            $productDesignation = $row[0];
            $categoryName = $row[1];
            $productSalePrice = number_format((float)$row[2], 0, '', ' ');
            $productSalePrice2 = number_format((float)$row[3], 0, '', ' ');
            $productSalePrice3 = number_format((float)$row[4], 0, '', ' ');
            $productBuyingPrice = number_format((float)$row[5], 0, '', ' ');
            $productId = $row[6];
            $productReference = $row[7];
            $productType = $this->translator->_((int)$row[8] == 1 ? "product.index.product" : "product.index.service");
            $actions = "
                <button title='{$this->translator->_("shared.consult")}' about='{$productId}' class='mini ui icon button app-primary-button app-consult-action'>
                    <i class='eye icon'></i>
                </button>
            ";
            $tableRow = [
                $productDesignation,
                $categoryName,
                $productType,
            ];
            if (GESTION_REFERENCE_PRODUIT == 1) {
                $tableRow[] = $productReference;
            }
            $tableRow[] = $productSalePrice;
            $tableRow[] = $productSalePrice2;
            $tableRow[] = $productSalePrice3;
            $tableRow[] = $productBuyingPrice;
            $tableRow[] = $actions;
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    public function orderAction()
    {
        AccessRightChecker::checkAccessRight($this, 35);
        $supplierDao = new SupplierDAO();
        $this->view->suppliers = $supplierDao->listSuppliers();
        $this->view->translator = $this->translator;
    }

    public function datatableorderAction()
    {
        $supplier = (int)$_POST["supplier"] != 0 ? (int)$_POST["supplier"] : null;
        $from = $_POST["from"] != "" ? $_POST["from"] : null;
        $to = $_POST["to"] != "" ? $_POST["to"] : null;
        $whereSupplier = "";
        $whereFrom = "";
        $whereTo = "";
        if ($supplier != null) {
            $whereSupplier .= " AND c.id_fournisseur = {$supplier}";
        }
        if ($from != null) {
            $whereFrom .= " AND c.date_commande >= '{$from} 00:00:00'";
        }
        if ($to != null) {
            $whereTo .= " AND c.date_commande <= '{$to} 23:59:59'";
        }
        $where = "c.statut = 1{$whereSupplier}{$whereFrom}{$whereTo}";
        $ordersTable = "
                (
                    SELECT c.id_commande AS order_id, f.nom_fournisseur AS order_supplier,
                    c.date_commande AS order_date, c.date_livraison AS order_delivery_date,
                    c.date_saisie AS order_entry_date, c.montant_ht AS order_total,
                    c.valider AS order_validated, c.facturer AS order_invoiced,
                    c.receptionner AS order_received
                    FROM commande c JOIN fournisseur f ON c.id_fournisseur = f.id_fournisseur
                    WHERE {$where}
                ) AS ordersTable
        ";
        $primaryKey = 'order_id';
        $columns = [
            ['db' => 'order_id', 'dt' => 0],
            ['db' => 'order_supplier', 'dt' => 1],
            ['db' => 'order_date', 'dt' => 2],
            ['db' => 'order_entry_date', 'dt' => 3],
            ['db' => 'order_total', 'dt' => 4],
            ['db' => 'order_validated', 'dt' => 5],
            ['db' => 'order_received', 'dt' => 6],
            ['db' => 'order_delivery_date', 'dt' => 7],
        ];
        $computeDatatableResults = SSP::customSimple($_POST, $ordersTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        foreach ($datatableResult["data"] as $row) {
            $orderId = (int)$row[0];
            $orderSupplier = $row[1];
            $orderDate = new DateTime($row[2]);
            $orderEntryDate = new DateTime($row[3]);
            $orderTotal = number_format((float)$row[4], 0, '', ' ');
            $orderDeliveryDate = $row[7] != null ? new DateTime($row[7]) : null;
            $actions = "
                <button title='{$this->translator->_("shared.consult")}' about='{$orderId}' class='mini ui icon button app-primary-button app-consult-action'>
                    <i class='eye icon'></i>
                </button>
            ";
            $tableRow = [
                $orderId,
                $orderSupplier,
                $orderDate->format(DATE_FORMAT),
                $orderDeliveryDate != null ? $orderDeliveryDate->format(DATE_FORMAT) : "",
                $orderEntryDate->format(DATE_FORMAT_WITH_SECONDS),
                $orderTotal,
                $actions,
            ];
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    public function receptionAction()
    {
        AccessRightChecker::checkAccessRight($this, 57);
        $supplierDao = new SupplierDAO();
        $currentUser = LogDAO::getCurrentUser();
        $storeDao = new StoreDAO();
        $stores = [];
        if ($currentUser->getIdPointVente() == null || in_array($currentUser->getIdProfil()->getCode(), IDENTIFIED_PROFILES_CODES)) {
            $stores = $storeDao->getAllStores();
        } else {
            $stores[] = $currentUser->getIdPointVente()->getIdMagasin();
        }
        $this->view->suppliers = $supplierDao->listSuppliers();
        $this->view->stores = $stores;
        $this->view->translator = $this->translator;
    }

    public function datatablereceptionAction()
    {
        $store = (int)$_POST["store"] != 0 ? (int)$_POST["store"] : null;
        $supplier = (int)$_POST["supplier"] != 0 ? (int)$_POST["supplier"] : null;
        $from = $_POST["from"] != "" ? $_POST["from"] : null;
        $to = $_POST["to"] != "" ? $_POST["to"] : null;
        $whereStore = "";
        $whereSupplier = "";
        $whereFrom = "";
        $whereTo = "";
        if ($store != null) {
            $whereStore .= " AND r.id_magasin = {$store}";
        }
        if ($supplier != null) {
            $whereSupplier .= " AND r.id_fournisseur = {$supplier}";
        }
        if ($from != null) {
            $whereFrom .= " AND r.date_reception >= '{$from} 00:00:00'";
        }
        if ($to != null) {
            $whereTo .= " AND r.date_reception <= '{$to} 23:59:59'";
        }
        $where = "r.statut = 1{$whereStore}{$whereSupplier}{$whereFrom}{$whereTo}";
        $receptionsTable = "
                (
                    SELECT r.id_reception AS reception_id, m.nom_magasin AS reception_store, f.nom_fournisseur AS reception_supplier,
                    r.date_saisie AS reception_entry_date, r.date_reception AS reception_receiving_date, r.num_bl AS reception_delivery_note_number, 
                    r.valider AS reception_validated
                    FROM reception r JOIN magasin m ON r.id_magasin = m.id_magasin JOIN fournisseur f ON r.id_fournisseur = f.id_fournisseur
                    WHERE {$where}
                ) AS receptionsTable
        ";
        $primaryKey = 'reception_id';
        $columns = [
            ['db' => 'reception_id', 'dt' => 0],
            ['db' => 'reception_store', 'dt' => 1],
            ['db' => 'reception_supplier', 'dt' => 2],
            ['db' => 'reception_entry_date', 'dt' => 3],
            ['db' => 'reception_receiving_date', 'dt' => 4],
            ['db' => 'reception_delivery_note_number', 'dt' => 5],
            ['db' => 'reception_validated', 'dt' => 6],
        ];
        $computeDatatableResults = SSP::customSimple($_POST, $receptionsTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        foreach ($datatableResult["data"] as $row) {
            $receptionId = (int)$row[0];
            $receptionStore = $row[1];
            $receptionSupplier = $row[2];
            $entryDate = new DateTime($row[3]);
            $receivingDate = new DateTime($row[4]);
            $deliveryNoteNumber = $row[5];
            $actions = "
                <button title='{$this->translator->_("shared.consult")}' about='{$receptionId}' class='mini ui icon button app-primary-button app-consult-action'>
                    <i class='eye icon'></i>
                </button>
            ";
            $tableRow = [
                $receptionId,
                $receptionStore,
                $receptionSupplier,
                $entryDate->format(DATE_FORMAT),
                $receivingDate->format(DATE_FORMAT),
                $deliveryNoteNumber,
                $actions,
            ];
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    public function itemstoorderAction()
    {
        AccessRightChecker::checkAccessRight($this, 67);
        $storeDao = new StoreDAO();
        $currentUser = LogDAO::getCurrentUser();
        $stores = [];
        if ($currentUser->getIdPointVente() == null || in_array($currentUser->getIdProfil()->getCode(), IDENTIFIED_PROFILES_CODES)) {
            $stores = $storeDao->getAllStores();
        } else {
            $stores[] = $currentUser->getIdPointVente()->getIdMagasin();
        }
        $this->view->stores = $stores;
        $this->view->translator = $this->translator;
    }

    public function soontoexpireitemsAction()
    {
        AccessRightChecker::checkAccessRight($this, 146);
        $storeDao = new StoreDAO();
        $currentUser = LogDAO::getCurrentUser();
        $stores = [];
        if ($currentUser->getIdPointVente() == null || in_array($currentUser->getIdProfil()->getCode(), IDENTIFIED_PROFILES_CODES)) {
            $stores = $storeDao->getAllStores();
        } else {
            $stores[] = $currentUser->getIdPointVente()->getIdMagasin();
        }
        $this->view->stores = $stores;
        $this->view->translator = $this->translator;
    }

    public function expireditemsAction()
    {
        AccessRightChecker::checkAccessRight($this, 68);
        $storeDao = new StoreDAO();
        $currentUser = LogDAO::getCurrentUser();
        $stores = [];
        if ($currentUser->getIdPointVente() == null || in_array($currentUser->getIdProfil()->getCode(), IDENTIFIED_PROFILES_CODES)) {
            $stores = $storeDao->getAllStores();
        } else {
            $stores[] = $currentUser->getIdPointVente()->getIdMagasin();
        }
        $this->view->stores = $stores;
        $this->view->translator = $this->translator;
    }

    public function exceptionaloutingsAction()
    {
        AccessRightChecker::checkAccessRight($this, 58);
        $currentUser = LogDAO::getCurrentUser();
        $storeDao = new StoreDAO();
        $stores = [];
        if ($currentUser->getIdPointVente() == null || in_array($currentUser->getIdProfil()->getCode(), IDENTIFIED_PROFILES_CODES)) {
            $stores = $storeDao->getAllStores();
        } else {
            $stores[] = $currentUser->getIdPointVente()->getIdMagasin();
        }
        $exceptionalOutingTypeDao = new ExceptionalOutingTypeDAO();
        $this->view->stores = $stores;
        $this->view->exceptionalOutingTypes = $exceptionalOutingTypeDao->getAllExceptionalOutingTypes();
        $this->view->translator = $this->translator;
    }

    public function datatableexceptionaloutingsAction()
    {
        $store = (int)$_POST["store"] != 0 ? (int)$_POST["store"] : null;
        $exitType = $_POST["exitType"] != "" ? (int)$_POST["exitType"] : null;
        $from = $_POST["from"] != "" ? $_POST["from"] : null;
        $to = $_POST["to"] != "" ? $_POST["to"] : null;
        $whereStore = "";
        $whereExitType = "";
        $whereFrom = "";
        $whereTo = "";
        if ($store != null) {
            $whereStore .= " AND s.id_magasin = {$store}";
        }
        if ($exitType != null) {
            $exceptionalOutingTypeDao = new ExceptionalOutingTypeDAO();
            $exceptionOutingType = $exceptionalOutingTypeDao->getExceptionalOutingTypeById($exitType);
            $whereExitType .= " AND s.type_sortie = '{$exceptionOutingType->getCode()}'";
        }
        if ($from != null) {
            $whereFrom .= " AND s.date_sortie >= '{$from} 00:00:00'";
        }
        if ($to != null) {
            $whereTo .= " AND s.date_sortie <= '{$to} 23:59:59'";
        }
        $where = "s.statut = 1 AND s.type_sortie <> '" . TYPE_SORTIE_VENTE . "'{$whereStore}{$whereExitType}{$whereFrom}{$whereTo}";
        $exceptionalOutingsTable = "
                (
                    SELECT s.id_sortie AS exit_id, m.nom_magasin AS exit_store, tse.nom AS exit_type, s.date_saisie AS exit_entry_date,
                    s.date_sortie AS exit_outing_date, s.motif AS exit_comment, CONCAT(pe.nom, ' ', COALESCE(pe.prenom, '')) AS exit_user,
                    s.valider AS exit_validated
                    FROM sortie s JOIN magasin m ON s.id_magasin = m.id_magasin JOIN type_sortie_exceptionnelle tse ON s.type_sortie = tse.code
                    JOIN utilisateur u ON s.id_utilisateur = u.id_utilisateur JOIN personne pe ON u.id_personne = pe.id_personne 
                    WHERE {$where}
                ) AS exceptionalOutingsTable
        ";
        $primaryKey = 'exit_id';
        $columns = [
            ['db' => 'exit_id', 'dt' => 0],
            ['db' => 'exit_store', 'dt' => 1],
            ['db' => 'exit_type', 'dt' => 2],
            ['db' => 'exit_entry_date', 'dt' => 3],
            ['db' => 'exit_outing_date', 'dt' => 4],
            ['db' => 'exit_comment', 'dt' => 5],
            ['db' => 'exit_user', 'dt' => 6],
            ['db' => 'exit_validated', 'dt' => 7],
        ];
        $computeDatatableResults = SSP::customSimple($_POST, $exceptionalOutingsTable, $primaryKey, $columns);
        $datatableResult = $computeDatatableResults["datatable"];
        $finalResult = $computeDatatableResults["custom"];
        foreach ($datatableResult["data"] as $row) {
            $exitId = (int)$row[0];
            $exitStore = $row[1];
            $exitType = $row[2];
            $exitEntryDate = new DateTime($row[3]);
            $exitOutingDate = new DateTime($row[4]);
            $exitComment = $row[5];
            $exitUser = $row[6];
            $exitValidated = (int)$row[7];
            $actions = "
                <button title='{$this->translator->_("shared.consult")}' about='{$exitId}' class='mini ui icon button app-primary-button app-consult-action'>
                    <i class='eye icon'></i>
                </button>
            ";
            $tableRow = [
                $exitId,
                $exitStore,
                $exitType,
                $exitEntryDate->format(DATE_FORMAT),
                $exitOutingDate->format(DATE_FORMAT),
                $exitComment,
                $exitUser,
                $actions,
            ];
            $finalResult["data"][] = $tableRow;
        }
        echo json_encode($finalResult);
        exit();
    }

    public function productpathAction()
    {
        AccessRightChecker::checkAccessRight($this, 53);
        $storeDao = new StoreDAO();
        $currentUser = LogDAO::getCurrentUser();
        $stores = [];
        if ($currentUser->getIdPointVente() == null || in_array($currentUser->getIdProfil()->getCode(), IDENTIFIED_PROFILES_CODES)) {
            $stores = $storeDao->getAllStores();
        } else {
            $stores[] = $currentUser->getIdPointVente()->getIdMagasin();
        }
        $this->view->stores = $stores;
        $this->view->translator = $this->translator;
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function displayproductpathAction()
    {
        try {
            $storeDao = new StoreDAO();
            $productDao = new ProductDAO();
            $stockDao = new StockDAO();
            $storeId = (int)$_POST["store"];
            $productId = (int)$_POST["product"];
            $startDate = new DateTime($_POST["startDate"]);
            $endDate = new DateTime("{$_POST["endDate"]} 23:59:59");
            $store = $storeDao->getStoreById($storeId);
            $product = $productDao->getProductById($productId);
            $dateBeforeStart = clone $startDate;
            $dateBeforeStart->modify(" -1 day");
            $quantityAtPeriodStart = $stockDao->getProductQuantityAtDate($product, $dateBeforeStart, $store);
            $quantityAtPeriodEnd = $stockDao->getProductQuantityAtDate($product, $endDate, $store);
            $result = [
                "quantityAtPeriodStart" => $quantityAtPeriodStart,
                "movements" => $stockDao->getProductMovementResumeOnPeriod($product, $startDate, $endDate, $this->translator, $store),
                "quantityAtPeriodEnd" => $quantityAtPeriodEnd,
            ];
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    public function stocksituationAction()
    {
        AccessRightChecker::checkAccessRight($this, 66);
        $storeDao = new StoreDAO();
        $productCategoryDao = new ProductCategoryDAO();
        $currentUser = LogDAO::getCurrentUser();
        $stores = [];
        if ($currentUser->getIdPointVente() == null || in_array($currentUser->getIdProfil()->getCode(), IDENTIFIED_PROFILES_CODES)) {
            $stores = $storeDao->getAllStores();
        } else {
            $stores[] = $currentUser->getIdPointVente()->getIdMagasin();
        }
        $this->view->stores = $stores;
        $this->view->categories = $productCategoryDao->getAllProductCategories();
        $this->view->translator = $this->translator;
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function loadstocksituationAction()
    {
        try {
            $stockDao = new StockDAO();
            $productCategoryDao = new ProductCategoryDAO();
            $storeDao = new StoreDAO();
            $productDao = new ProductDAO();
            $storeId = (int)$_POST["store"];
            $productCategoryId = (int)$_POST["category"] != 0 ? (int)$_POST["category"] : null;
            $productId = (int)$_POST["product"] != 0 ? (int)$_POST["product"] : null;
            $startDate = new DateTime($_POST["startDate"]);
            $endDate = new DateTime($_POST["endDate"]);
            $dateBeforeStart = clone $startDate;
            $dateBeforeStart->modify("-1 day");
            $store = $storeDao->getStoreById($storeId);
            $productCategory = $productCategoryId != null ? $productCategoryDao->getProductCategoryById($productCategoryId) : null;
            $product = $productId != null ? $productDao->getProductById($productId) : null;
            $allProductsConcernedIds = $stockDao->getAllProductsInStock($store, null, $productCategory, $product);
            $result = [];
            foreach ($allProductsConcernedIds as $productConcernedId) {
                $product = $productDao->getProductById($productConcernedId);
                $result[] = [
                    "designation" => $product->getDesignation(),
                    "quantityAtBegin" => $stockDao->getProductQuantityAtDate($product, $dateBeforeStart, $store),
                    "quantityAtEnd" => $stockDao->getProductQuantityAtDate($product, $endDate, $store),
                    "quantityEnteredDuringPeriod" => $stockDao->getProductQuantityEnteredOverPeriod($product, $startDate, $endDate, $store),
                    "quantityIssuedDuringPeriod" => $stockDao->getProductQuantityIssuedOverPeriod($product, $startDate, $endDate, $store),
                ];
            }
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    public function followupAction()
    {
        AccessRightChecker::checkAccessRight($this, 150);
        $pointOfSaleDao = new PointOfSaleDAO();
        $currentUser = LogDAO::getCurrentUser();
        $pointsOfSale = [];
        if ($currentUser->getIdPointVente() == null || in_array($currentUser->getIdProfil()->getCode(), IDENTIFIED_PROFILES_CODES)) {
            $pointsOfSale = $pointOfSaleDao->listPointOfSales();
        } else {
            $pointsOfSale[] = $currentUser->getIdPointVente();
        }
        $this->view->pointsOfSale = $pointsOfSale;
        $this->view->translator = $this->translator;
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function loadfollowupAction()
    {
        try {
            $pointOfSaleId = (int)$_POST["pointOfSale"] != 0 ? (int)$_POST["pointOfSale"] : null;
            $daysCount = (int)$_POST["daysCount"];
            $pointOfSaleDao = new PointOfSaleDAO();
            $pointOfSale = $pointOfSaleId != null ? $pointOfSaleDao->getPointOfSaleById($pointOfSaleId) : null;
            $customerInvoiceDao = new CustomerInvoiceDAO();
            $currentDate = new DateTime();
            $currentDate->modify("- {$daysCount} days");
            $customersToFollowUp = $customerInvoiceDao->getCustomerToFollowUp($currentDate, $pointOfSale);
            $result = [];
            foreach ($customersToFollowUp as $customerToFollowUp) {
                $result[] = [
                    "name" => "{$customerToFollowUp["customer_last_name"]} {$customerToFollowUp["customer_first_name"]}",
                    "last_visit_date" => $customerToFollowUp["last_invoice_date"],
                    "phone" => $customerToFollowUp["customer_phone"],
                    "email" => $customerToFollowUp["customer_email"],
                ];
            }
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    public function bestcustomersAction()
    {
        AccessRightChecker::checkAccessRight($this, 151);
        $pointOfSaleDao = new PointOfSaleDAO();
        $currentUser = LogDAO::getCurrentUser();
        $pointsOfSale = [];
        if ($currentUser->getIdPointVente() == null || in_array($currentUser->getIdProfil()->getCode(), IDENTIFIED_PROFILES_CODES)) {
            $pointsOfSale = $pointOfSaleDao->listPointOfSales();
        } else {
            $pointsOfSale[] = $currentUser->getIdPointVente();
        }
        $this->view->pointsOfSale = $pointsOfSale;
        $this->view->translator = $this->translator;
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function loadbestcustomersAction()
    {
        try {
            $pointOfSaleId = (int)$_POST["pointOfSale"] != 0 ? (int)$_POST["pointOfSale"] : null;
            $startDate = new DateTime("{$_POST["startDate"]} 00:00:00");
            $endDate = new DateTime("{$_POST["endDate"]} 23:59:59");
            $minPurchaseSum = (float)$_POST["minPurchaseSum"];
            $pointOfSaleDao = new PointOfSaleDAO();
            $pointOfSale = $pointOfSaleId != null ? $pointOfSaleDao->getPointOfSaleById($pointOfSaleId) : null;
            $customerInvoiceDao = new CustomerInvoiceDAO();
            $bestCustomers = $customerInvoiceDao->getBestCustomers($startDate, $endDate, $minPurchaseSum, $pointOfSale);
            $result = [];
            foreach ($bestCustomers as $bestCustomer) {
                $result[] = [
                    "name" => "{$bestCustomer["customer_last_name"]} {$bestCustomer["customer_first_name"]}",
                    "purchaseSum" => $bestCustomer["customer_turnover"],
                    "phone" => $bestCustomer["customer_phone"],
                    "email" => $bestCustomer["customer_email"],
                ];
            }
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    public function regularcustomersAction()
    {
        AccessRightChecker::checkAccessRight($this, 152);
        $pointOfSaleDao = new PointOfSaleDAO();
        $currentUser = LogDAO::getCurrentUser();
        $pointsOfSale = [];
        if ($currentUser->getIdPointVente() == null || in_array($currentUser->getIdProfil()->getCode(), IDENTIFIED_PROFILES_CODES)) {
            $pointsOfSale = $pointOfSaleDao->listPointOfSales();
        } else {
            $pointsOfSale[] = $currentUser->getIdPointVente();
        }
        $this->view->pointsOfSale = $pointsOfSale;
        $this->view->translator = $this->translator;
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function loadregularcustomersAction()
    {
        try {
            $pointOfSaleId = (int)$_POST["pointOfSale"] != 0 ? (int)$_POST["pointOfSale"] : null;
            $startDate = new DateTime("{$_POST["startDate"]} 00:00:00");
            $endDate = new DateTime("{$_POST["endDate"]} 23:59:59");
            $minNumberOfVisits = (float)$_POST["minNumberOfVisits"];
            $pointOfSaleDao = new PointOfSaleDAO();
            $pointOfSale = $pointOfSaleId != null ? $pointOfSaleDao->getPointOfSaleById($pointOfSaleId) : null;
            $customerInvoiceDao = new CustomerInvoiceDAO();
            $regularCustomers = $customerInvoiceDao->getSatisfactionSurveyCustomers($startDate, $endDate, $minNumberOfVisits, $pointOfSale);
            $result = [];
            foreach ($regularCustomers as $regularCustomer) {
                $result[] = [
                    "name" => "{$regularCustomer["customer_last_name"]} {$regularCustomer["customer_first_name"]}",
                    "visits" => $regularCustomer["customer_visits"],
                    "phone" => $regularCustomer["customer_phone"],
                    "email" => $regularCustomer["customer_email"],
                ];
            }
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    public function outofstockAction()
    {
        AccessRightChecker::checkAccessRight($this, 153);
        $storeDao = new StoreDAO();
        $productCategoryDao = new ProductCategoryDAO();
        $currentUser = LogDAO::getCurrentUser();
        $stores = [];
        if ($currentUser->getIdPointVente() == null || in_array($currentUser->getIdProfil()->getCode(), IDENTIFIED_PROFILES_CODES)) {
            $stores = $storeDao->getAllStores();
        } else {
            $stores[] = $currentUser->getIdPointVente()->getIdMagasin();
        }
        $this->view->stores = $stores;
        $this->view->categories = $productCategoryDao->getAllProductCategories();
        $this->view->translator = $this->translator;
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function loadoutofstockAction()
    {
        try {
            $storeId = (int)$_POST["store"];
            $categoryId = (int)$_POST["category"] != 0 ? (int)$_POST["category"] : null;
            $storeDao = new StoreDAO();
            $productCategoryDao = new ProductCategoryDAO();
            $stockDao = new StockDAO();
            $store = $storeDao->getStoreById($storeId);
            $productCategory = $categoryId != null ? $productCategoryDao->getProductCategoryById($categoryId) : null;
            $outOfStockProductsIds = $stockDao->getProductsOutOfStock($store, $productCategory);
            $result = [];
            $productDao = new ProductDAO();
            $exitLineDao = new ExitLineDAO();
            foreach ($outOfStockProductsIds as $outOfStockProductId) {
                $product = $productDao->getProductById($outOfStockProductId["product_id"]);
                $lastExitLine = $exitLineDao->getLastExitLineOfProduct($product);
                $result[] = [
                    "name" => $product->getDesignation(),
                    "last_exit_date" => $lastExitLine != null ? $lastExitLine->getIdSortie()->getDateValidation()->format(DATE_FORMAT) : "",
                ];
            }
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, $result);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    public function cashiersalesAction()
    {
        AccessRightChecker::checkAccessRight($this, 70);
        $pointOfSaleDao = new PointOfSaleDAO();
        $currentUser = LogDAO::getCurrentUser();
        $pointsOfSale = [];
        if ($currentUser->getIdPointVente() == null || in_array($currentUser->getIdProfil()->getCode(), IDENTIFIED_PROFILES_CODES)) {
            $pointsOfSale = $pointOfSaleDao->listPointOfSales();
        } else {
            $pointsOfSale[] = $currentUser->getIdPointVente();
        }
        $this->view->pointsOfSale = $pointsOfSale;
        $this->view->translator = $this->translator;
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function loadcashiersalesAction()
    {
        try {
            $pointOfSaleDao = new PointOfSaleDAO();
            $userDao = new UserDAO();
            $customerInvoiceDao = new CustomerInvoiceDAO();
            $customerInvoiceLineDao = new CustomerInvoiceLineDAO();
            $invoicePaymentDao = new InvoicePaymentDAO();
            $pointsOfSaleIds = json_decode($_POST["pointsOfSale"], true);
            $cashiersIds = json_decode($_POST["cashiers"], true);
            $startDate = new DateTime("{$_POST["startDate"]}:00");
            $endDate = new DateTime("{$_POST["endDate"]}:59");
            $pointsOfSale = [];
            $cashiers = [];
            foreach ($pointsOfSaleIds as $pointOfSaleId) {
                $pointsOfSale[] = $pointOfSaleDao->getPointOfSaleById((int)$pointOfSaleId);
            }
            foreach ($cashiersIds as $cashierId) {
                $cashiers[] = $userDao->getUserById((int)$cashierId);
            }
            $salesResult = [];
            $productsCountResult = [];
            $salesPerCashiers = $customerInvoiceDao->getSalesByPointsOfSalesAndCashiers($pointsOfSale, $cashiers, $startDate, $endDate);
            $productsCountPerCashiers = $customerInvoiceLineDao->getSoldArticleCountByPointsOfSaleAndCashiers($pointsOfSale, $cashiers, $startDate, $endDate);
            foreach ($salesPerCashiers as $salePerCashier) {
                $salesResult[] = [
                    "id" => $salePerCashier->getIdFacture(),
                    "dateAndHour" => $salePerCashier->getDateFacture()->format(DATE_FORMAT_WITH_SECONDS),
                    "amount" => $salePerCashier->getMontantNet(),
                    "reductionAmount" => $salePerCashier->getForfaitRemise(),
                    "paidAmount" => $invoicePaymentDao->getSumOfInvoicePayments($salePerCashier),
                    "pointOfSaleId" => $salePerCashier->getIdPointVente()->getIdPointVente(),
                    "pointOfSaleName" => $salePerCashier->getIdPointVente()->getNomPointVente(),
                    "cashierId" => $salePerCashier->getIdUtilisateur()->getIdUtilisateur(),
                    "cashierName" => "{$salePerCashier->getIdUtilisateur()->getIdPersonne()->getNom()} {$salePerCashier->getIdUtilisateur()->getIdPersonne()->getPrenom()}",
                ];
            }
            foreach ($productsCountPerCashiers as $productCountPerCashier) {
                $cashier = $userDao->getUserById((int)$productCountPerCashier["cashier_id"]);
                $productsCountResult[] = [
                    "designation" => $productCountPerCashier["product_name"],
                    "productId" => $productCountPerCashier["product_id"],
                    "category" => $productCountPerCashier["category_name"],
                    "categoryId" => $productCountPerCashier["category_id"],
                    "quantity" => (int)$productCountPerCashier["sold_quantity"],
                    "buyingPrice" => (float)$productCountPerCashier["buying_price"],
                    "unitPrice" => (float)$productCountPerCashier["unit_price"],
                    "salesSum" => (float)$productCountPerCashier["sold_amount"],
                    "profit" => (float)$productCountPerCashier["profit"],
                    "pointOfSaleId" => (int)$productCountPerCashier["point_of_sale_id"],
                    "pointOfSaleName" => $productCountPerCashier["point_of_sale_name"],
                    "cashierId" => $productCountPerCashier["cashier_id"],
                    "cashierName" => "{$cashier->getIdPersonne()->getNom()} {$cashier->getIdPersonne()->getPrenom()}",
                ];
            }
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["sales" => $salesResult, "productsCount" => $productsCountResult]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    public function salesatpointofsaleAction()
    {
        AccessRightChecker::checkAccessRight($this, 70);
        $pointOfSaleDao = new PointOfSaleDAO();
        $currentUser = LogDAO::getCurrentUser();
        $pointsOfSale = [];
        if ($currentUser->getIdPointVente() == null || in_array($currentUser->getIdProfil()->getCode(), IDENTIFIED_PROFILES_CODES)) {
            $pointsOfSale = $pointOfSaleDao->listPointOfSales();
        } else {
            $pointsOfSale[] = $currentUser->getIdPointVente();
        }
        $this->view->pointsOfSale = $pointsOfSale;
        $this->view->translator = $this->translator;
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function loadsalesatpointofsaleAction()
    {
        try {
            $pointOfSaleDao = new PointOfSaleDAO();
            $customerInvoiceDao = new CustomerInvoiceDAO();
            $customerInvoiceLineDao = new CustomerInvoiceLineDAO();
            $invoicePaymentDao = new InvoicePaymentDAO();
            $pointsOfSaleIds = json_decode($_POST["pointsOfSale"], true);
            $startDate = new DateTime("{$_POST["startDate"]}:00");
            $endDate = new DateTime("{$_POST["endDate"]}:59");
            $pointsOfSale = [];
            foreach ($pointsOfSaleIds as $pointOfSaleId) {
                $pointsOfSale[] = $pointOfSaleDao->getPointOfSaleById((int)$pointOfSaleId);
            }
            $salesResult = [];
            $productsCountResult = [];
            $sales = $customerInvoiceDao->getSalesByPointsOfSales($pointsOfSale, $startDate, $endDate);
            $productsCount = $customerInvoiceLineDao->getSoldArticleCountByPointsOfSale($pointsOfSale, $startDate, $endDate);
            foreach ($sales as $sale) {
                $salesResult[] = [
                    "id" => $sale->getIdFacture(),
                    "dateAndHour" => $sale->getDateFacture()->format(DATE_FORMAT_WITH_SECONDS),
                    "amount" => $sale->getMontantNet(),
                    "reductionAmount" => $sale->getForfaitRemise(),
                    "paidAmount" => $invoicePaymentDao->getSumOfInvoicePayments($sale),
                    "pointOfSaleId" => $sale->getIdPointVente()->getIdPointVente(),
                    "pointOfSaleName" => $sale->getIdPointVente()->getNomPointVente(),
                ];
            }
            foreach ($productsCount as $productCount) {
                $productsCountResult[] = [
                    "designation" => $productCount["product_name"],
                    "productId" => $productCount["product_id"],
                    "category" => $productCount["category_name"],
                    "categoryId" => $productCount["category_id"],
                    "quantity" => (int)$productCount["sold_quantity"],
                    "buyingPrice" => (float)$productCount["buying_price"],
                    "unitPrice" => (float)$productCount["unit_price"],
                    "salesSum" => (float)$productCount["sold_amount"],
                    "profit" => (float)$productCount["profit"],
                    "pointOfSaleId" => (int)$productCount["point_of_sale_id"],
                    "pointOfSaleName" => $productCount["point_of_sale_name"],
                ];
            }
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["sales" => $salesResult, "productsCount" => $productsCountResult]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    public function productsalesAction()
    {
        AccessRightChecker::checkAccessRight($this, 72);
        $pointOfSaleDao = new PointOfSaleDAO();
        $productCategoryDao = new ProductCategoryDAO();
        $currentUser = LogDAO::getCurrentUser();
        $pointsOfSale = [];
        if ($currentUser->getIdPointVente() == null || in_array($currentUser->getIdProfil()->getCode(), IDENTIFIED_PROFILES_CODES)) {
            $pointsOfSale = $pointOfSaleDao->listPointOfSales();
        } else {
            $pointsOfSale[] = $currentUser->getIdPointVente();
        }
        $this->view->pointsOfSale = $pointsOfSale;
        $this->view->productCategories = $productCategoryDao->getAllProductCategories();
        $this->view->translator = $this->translator;
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function loadproductsalesAction()
    {
        try {
            $pointOfSaleDao = new PointOfSaleDAO();
            $productCategoryDao = new ProductCategoryDAO();
            $productDao = new ProductDAO();
            $customerInvoiceLineDao = new CustomerInvoiceLineDAO();
            $pointsOfSaleIds = json_decode($_POST["pointsOfSale"], true);
            $productCategoriesIds = json_decode($_POST["categories"], true);
            $productsIds = json_decode($_POST["products"], true);
            $startDate = new DateTime("{$_POST["startDate"]}:00");
            $endDate = new DateTime("{$_POST["endDate"]}:59");
            $pointsOfSale = [];
            foreach ($pointsOfSaleIds as $pointOfSaleId) {
                $pointsOfSale[] = $pointOfSaleDao->getPointOfSaleById((int)$pointOfSaleId);
            }
            $productCategories = [];
            foreach ($productCategoriesIds as $productCategoryId) {
                $productCategories[] = $productCategoryDao->getProductCategoryById($productCategoryId);
            }
            $products = [];
            foreach ($productsIds as $productId) {
                $products[] = $productDao->getProductById($productId);
            }
            $productsCountResult = [];
            $productsCount = $customerInvoiceLineDao->getSoldArticleCount($pointsOfSale, $productCategories, $products, $startDate, $endDate);
            foreach ($productsCount as $productCount) {
                $productsCountResult[] = [
                    "designation" => $productCount["product_name"],
                    "productId" => $productCount["product_id"],
                    "category" => $productCount["category_name"],
                    "categoryId" => $productCount["category_id"],
                    "quantity" => (int)$productCount["sold_quantity"],
                    "buyingPrice" => (float)$productCount["buying_price"],
                    "unitPrice" => (float)$productCount["unit_price"],
                    "salesSum" => (float)$productCount["sold_amount"],
                    "profit" => (float)$productCount["profit"],
                    "pointOfSaleId" => (int)$productCount["point_of_sale_id"],
                    "pointOfSaleName" => $productCount["point_of_sale_name"],
                ];
            }
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["productsCount" => $productsCountResult]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    public function customersalesAction()
    {
        AccessRightChecker::checkAccessRight($this, 109);
        $pointOfSaleDao = new PointOfSaleDAO();
        $currentUser = LogDAO::getCurrentUser();
        $pointsOfSale = [];
        if ($currentUser->getIdPointVente() == null || in_array($currentUser->getIdProfil()->getCode(), IDENTIFIED_PROFILES_CODES)) {
            $pointsOfSale = $pointOfSaleDao->listPointOfSales();
        } else {
            $pointsOfSale[] = $currentUser->getIdPointVente();
        }
        $this->view->pointsOfSale = $pointsOfSale;
        $this->view->translator = $this->translator;
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function loadcustomersalesAction()
    {
        try {
            $personDao = new PersonDAO();
            $pointOfSaleDao = new PointOfSaleDAO();
            $customerInvoiceDao = new CustomerInvoiceDAO();
            $customerInvoiceLineDao = new CustomerInvoiceLineDAO();
            $invoicePaymentDao = new InvoicePaymentDAO();
            $customerId = (int)$_POST["customer"];
            $customer = $personDao->getPersonById($customerId);
            $pointsOfSaleIds = json_decode($_POST["pointsOfSale"], true);
            $startDate = new DateTime("{$_POST["startDate"]}:00");
            $endDate = new DateTime("{$_POST["endDate"]}:59");
            $pointsOfSale = [];
            foreach ($pointsOfSaleIds as $pointOfSaleId) {
                $pointsOfSale[] = $pointOfSaleDao->getPointOfSaleById((int)$pointOfSaleId);
            }
            $salesResult = [];
            $productsCountResult = [];
            $sales = $customerInvoiceDao->getSalesByCustomer($customer, $pointsOfSale, $startDate, $endDate);
            $productsCount = $customerInvoiceLineDao->getSoldArticleCountByCustomer($customer, $pointsOfSale, $startDate, $endDate);
            foreach ($sales as $sale) {
                $salesResult[] = [
                    "id" => $sale->getIdFacture(),
                    "dateAndHour" => $sale->getDateFacture()->format(DATE_FORMAT_WITH_SECONDS),
                    "amount" => $sale->getMontantNet(),
                    "reductionAmount" => $sale->getForfaitRemise(),
                    "paidAmount" => $invoicePaymentDao->getSumOfInvoicePayments($sale),
                    "pointOfSaleId" => $sale->getIdPointVente()->getIdPointVente(),
                    "pointOfSaleName" => $sale->getIdPointVente()->getNomPointVente(),
                ];
            }
            foreach ($productsCount as $productCount) {
                $productsCountResult[] = [
                    "designation" => $productCount["product_name"],
                    "productId" => $productCount["product_id"],
                    "category" => $productCount["category_name"],
                    "categoryId" => $productCount["category_id"],
                    "quantity" => (int)$productCount["sold_quantity"],
                    "buyingPrice" => (float)$productCount["buying_price"],
                    "unitPrice" => (float)$productCount["unit_price"],
                    "salesSum" => (float)$productCount["sold_amount"],
                    "profit" => (float)$productCount["profit"],
                    "pointOfSaleId" => (int)$productCount["point_of_sale_id"],
                    "pointOfSaleName" => $productCount["point_of_sale_name"],
                ];
            }
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["sales" => $salesResult, "productsCount" => $productsCountResult]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    public function statsperformanceAction()
    {
        AccessRightChecker::checkAccessRight($this, 40);
        $pointOfSaleDao = new PointOfSaleDAO();
        $currentUser = LogDAO::getCurrentUser();
        $pointsOfSale = [];
        if ($currentUser->getIdPointVente() == null || in_array($currentUser->getIdProfil()->getCode(), IDENTIFIED_PROFILES_CODES)) {
            $pointsOfSale = $pointOfSaleDao->listPointOfSales();
        } else {
            $pointsOfSale[] = $currentUser->getIdPointVente();
        }
        $this->view->pointsOfSale = $pointsOfSale;
        $this->view->translator = $this->translator;
    }

    protected static function sortComparativeSalesArray($pointOfSale1, $pointOfSale2)
    {
        if ($pointOfSale1["sales"] == $pointOfSale2["sales"])
            return 0;
        else if ($pointOfSale1["sales"] > $pointOfSale2["sales"])
            return -1;
        else
            return 1;
    }

    protected static function sortComparativeFundMovementsArray($pointOfSale1, $pointOfSale2)
    {
        if ($pointOfSale1["entries"] == $pointOfSale2["entries"])
            return 0;
        else if ($pointOfSale1["entries"] > $pointOfSale2["entries"])
            return -1;
        else
            return 1;
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function loadstatsperformanceAction()
    {
        try {
            $chartType = $_POST["chartType"];
            switch ($chartType) {
                case "GLOBAL_SALES":
                    $pointOfSaleDao = new PointOfSaleDAO();
                    $customerInvoiceDao = new CustomerInvoiceDAO();
                    $pointOfSaleId = (int)$_POST["pointOfSale"] != 0 ? (int)$_POST["pointOfSale"] : null;
                    $pointOfSale = $pointOfSaleId != null ? $pointOfSaleDao->getPointOfSaleById($pointOfSaleId) : null;
                    $displayType = $_POST["displayType"];
                    $salesOccurrences = [];
                    $paymentsOccurrences = [];
                    $countsOccurrences = [];
                    if ($displayType == "Daily") {
                        $startDate = new DateTime($_POST["startDate"]);
                        $endDate = new DateTime($_POST["endDate"]);
                        while (strtotime($startDate->format(DATE_FORMAT)) <= strtotime($endDate->format(DATE_FORMAT))) {
                            $saleOccurrence = ["x" => $startDate->format(DATE_FORMAT)];
                            $paymentOccurrence = ["x" => $startDate->format(DATE_FORMAT)];
                            $countOccurrence = ["x" => $startDate->format(DATE_FORMAT)];
                            $sumOfAmountOfInvoicesOfDay = $customerInvoiceDao->getSumOfAmountOfInvoicesOfDay($startDate->format(DATE_FORMAT), $pointOfSale);
                            $saleOccurrence["y"] = $sumOfAmountOfInvoicesOfDay ? $sumOfAmountOfInvoicesOfDay : 0;
                            $sumOfPaymentOfInvoicesOnDay = $customerInvoiceDao->getSumOfPaymentsOfInvoicesOfDay($startDate->format(DATE_FORMAT), $pointOfSale);
                            $paymentOccurrence["y"] = $sumOfPaymentOfInvoicesOnDay ? $sumOfPaymentOfInvoicesOnDay : 0;
                            $invoiceCountOfDay = $customerInvoiceDao->getInvoicesCountOfDay($startDate->format(DATE_FORMAT), $pointOfSale);
                            $countOccurrence["y"] = $invoiceCountOfDay ? $invoiceCountOfDay : 0;
                            $salesOccurrences[] = $saleOccurrence;
                            $paymentsOccurrences[] = $paymentOccurrence;
                            $countsOccurrences[] = $countOccurrence;
                            $startDate->modify("+1 day");
                        }
                    } else {
                        $years = json_decode($_POST["years"], true);
                        foreach ($years as $key => $year) {
                            $years[$key] = (int)$year;
                        }
                        asort($years);
                        $currentDate = new DateTime();
                        $currentTime = strtotime($currentDate->format(DATE_FORMAT));
                        $months = ["01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12"];
                        foreach ($years as $year) {
                            foreach ($months as $month) {
                                $monthDate = new DateTime("{$year}-{$month}-01");
                                if (strtotime($monthDate->format(DATE_FORMAT)) > $currentTime) continue;
                                $monthMoment = new \Moment\Moment($monthDate->format(DATE_FORMAT));
                                $saleOccurrence["x"] = "{$monthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                                $paymentOccurrence["x"] = "{$monthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                                $countOccurrence["x"] = "{$monthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                                $sumOfAmountOfInvoicesOfMonth = $customerInvoiceDao->getSumOfAmountOfInvoicesOnMonth($monthDate, $pointOfSale);
                                $saleOccurrence["y"] = $sumOfAmountOfInvoicesOfMonth ? $sumOfAmountOfInvoicesOfMonth : 0;
                                $sumOfPaymentOfInvoicesOfMonth = $customerInvoiceDao->getSumOfPaymentsOfInvoicesOnMonth($monthDate, $pointOfSale);
                                $paymentOccurrence["y"] = $sumOfPaymentOfInvoicesOfMonth ? $sumOfPaymentOfInvoicesOfMonth : 0;
                                $invoicesCountOfMonth = $customerInvoiceDao->getInvoicesCountOnMonth($monthDate, $pointOfSale);
                                $countOccurrence["y"] = $invoicesCountOfMonth? $invoicesCountOfMonth : 0;
                                $salesOccurrences[] = $saleOccurrence;
                                $paymentsOccurrences[] = $paymentOccurrence;
                                $countsOccurrences[] = $countOccurrence;
                            }
                        }
                    }
                    echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["sales" => $salesOccurrences, "payments" => $paymentsOccurrences, "counts" => $countsOccurrences]);
                    exit();
                case "GLOBAL_FUNDS":
                    $pointOfSaleDao = new PointOfSaleDAO();
                    $cashMovementDao = new CashMovementDAO();
                    $pointOfSaleId = (int)$_POST["pointOfSale"] != 0 ? (int)$_POST["pointOfSale"] : null;
                    $pointOfSale = $pointOfSaleId != null ? $pointOfSaleDao->getPointOfSaleById($pointOfSaleId) : null;
                    $displayType = $_POST["displayType"];
                    $entriesOccurrences = [];
                    $exitsOccurrences = [];
                    if ($displayType == "Daily") {
                        $startDate = new DateTime($_POST["startDate"]);
                        $endDate = new DateTime($_POST["endDate"]);
                        while (strtotime($startDate->format(DATE_FORMAT)) <= strtotime($endDate->format(DATE_FORMAT))) {
                            $entryOccurrence = ["x" => $startDate->format(DATE_FORMAT)];
                            $exitOccurrence = ["x" => $startDate->format(DATE_FORMAT)];
                            $cashMovement = $cashMovementDao->getSumOfGlobalPaymentsOnDay($startDate->format(DATE_FORMAT), $pointOfSale);
                            $entryOccurrence["y"] = $cashMovement["sumOfMoneyIn"] ? $cashMovement["sumOfMoneyIn"] : 0;
                            $exitOccurrence["y"] = $cashMovement["sumOfMoneyOut"] ? $cashMovement["sumOfMoneyOut"] : 0;
                            $entriesOccurrences[] = $entryOccurrence;
                            $exitsOccurrences[] = $exitOccurrence;
                            $startDate->modify("+1 day");
                        }
                    } else {
                        $years = json_decode($_POST["years"], true);
                        foreach ($years as $key => $year) {
                            $years[$key] = (int)$year;
                        }
                        asort($years);
                        $currentDate = new DateTime();
                        $currentTime = strtotime($currentDate->format(DATE_FORMAT));
                        $months = ["01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12"];
                        foreach ($years as $year) {
                            foreach ($months as $month) {
                                $monthDate = new DateTime("{$year}-{$month}-01");
                                if (strtotime($monthDate->format(DATE_FORMAT)) > $currentTime) continue;
                                $monthMoment = new \Moment\Moment($monthDate->format(DATE_FORMAT));
                                $entryOccurrence["x"] = "{$monthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                                $exitOccurrence["x"] = "{$monthMoment->format(DATE_FORMAT_MONTH)} {$year}";
                                $cashMovementOfMonth = $cashMovementDao->getSumOfGlobalPaymentsOnMonth($monthDate, $pointOfSale);
                                $entryOccurrence["y"] = $cashMovementOfMonth["sumOfMoneyIn"] ? $cashMovementOfMonth["sumOfMoneyIn"] : 0;
                                $exitOccurrence["y"] = $cashMovementOfMonth["sumOfMoneyOut"] ? $cashMovementOfMonth["sumOfMoneyOut"] : 0;
                                $entriesOccurrences[] = $entryOccurrence;
                                $exitsOccurrences[] = $exitOccurrence;
                            }
                        }
                    }
                    echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["entries" => $entriesOccurrences, "exits" => $exitsOccurrences]);
                    exit();
                case "GLOBAL_SOLD_ARTICLES":
                    $pointOfSaleDao = new PointOfSaleDAO();
                    $customerInvoiceLineDao = new CustomerInvoiceLineDAO();
                    $pointOfSaleId = (int)$_POST["pointOfSale"] != 0 ? (int)$_POST["pointOfSale"] : null;
                    $pointOfSale = $pointOfSaleId != null ? $pointOfSaleDao->getPointOfSaleById($pointOfSaleId) : null;
                    $startDate = new DateTime("{$_POST["startDate"]} 00:00:00");
                    $endDate = new DateTime("{$_POST["endDate"]} 23:59:59");
                    $count = (int)$_POST["count"];
                    $bestSoldArticles = $customerInvoiceLineDao->getBestSoldArticles($startDate, $endDate, $count, $pointOfSale);
                    $quantitiesOccurrences = [];
                    $amountsOccurrences = [];
                    foreach ($bestSoldArticles as $bestSoldArticle) {
                        $quantitiesOccurrences[] = [
                            "x" => $bestSoldArticle["product_name"],
                            "y" => $bestSoldArticle["sold_quantity"],
                        ];
                        $amountsOccurrences[] = [
                            "x" => $bestSoldArticle["product_name"],
                            "y" => $bestSoldArticle["sold_amount"],
                        ];
                    }
                    echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["quantities" => $quantitiesOccurrences, "amounts" => $amountsOccurrences]);
                    exit();
                case "GLOBAL_BEST_CUSTOMERS":
                    $pointOfSaleDao = new PointOfSaleDAO();
                    $customerInvoiceDao = new CustomerInvoiceDAO();
                    $pointOfSaleId = (int)$_POST["pointOfSale"] != 0 ? (int)$_POST["pointOfSale"] : null;
                    $pointOfSale = $pointOfSaleId != null ? $pointOfSaleDao->getPointOfSaleById($pointOfSaleId) : null;
                    $startDate = new DateTime("{$_POST["startDate"]} 00:00:00");
                    $endDate = new DateTime("{$_POST["endDate"]} 23:59:59");
                    $count = (int)$_POST["count"];
                    $bestCustomers = $customerInvoiceDao->getBestCustomersLimited($startDate, $endDate, $count, $pointOfSale);
                    $amountsOccurrences = [];
                    $countsOccurrences = [];
                    foreach ($bestCustomers as $bestCustomer) {
                        $amountsOccurrences[] = [
                            "x" => "{$bestCustomer["customer_last_name"]} {$bestCustomer["customer_first_name"]}",
                            "y" => $bestCustomer["customer_turnover"],
                        ];
                        $countsOccurrences[] = [
                            "x" => "{$bestCustomer["customer_last_name"]} {$bestCustomer["customer_first_name"]}",
                            "y" => $bestCustomer["sales_count"],
                        ];
                    }
                    echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["amounts" => $amountsOccurrences, "counts" => $countsOccurrences]);
                    exit();
                case "COMPARE_SALES":
                    $pointOfSaleDao = new PointOfSaleDAO();
                    $customerInvoiceDao = new CustomerInvoiceDAO();
                    $pointsOfSaleIds = json_decode($_POST["pointsOfSale"], true);
                    $pointsOfSale = [];
                    foreach ($pointsOfSaleIds as $pointOfSaleId) {
                        $pointsOfSale[] = $pointOfSaleDao->getPointOfSaleById((int)$pointOfSaleId);
                    }
                    if (count($pointsOfSale) == 0) {
                        $pointsOfSale = $pointOfSaleDao->listPointOfSales();
                    }
                    $startDate = new DateTime("{$_POST["startDate"]} 00:00:00");
                    $endDate = new DateTime("{$_POST["endDate"]} 23:59:59");
                    $pointsOfSalesOccurrences = [];
                    foreach ($pointsOfSale as $pointOfSale) {
                        $pointsOfSalesOccurrences[] = [
                            "x" => $pointOfSale->getNomPointVente(),
                            "sales" => $customerInvoiceDao->getSumOfAmountOfInvoicesOverPeriod($startDate, $endDate, $pointOfSale),
                            "payments" => $customerInvoiceDao->getSumOfPaymentsOfInvoicesOverPeriod($startDate, $endDate, $pointOfSale),
                            "count" => $customerInvoiceDao->getInvoicesCountOverPeriod($startDate, $endDate, $pointOfSale),
                        ];
                    }
                    usort($pointsOfSalesOccurrences, "ViewController::sortComparativeSalesArray");
                    echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["pointsOfSale" => $pointsOfSalesOccurrences]);
                    exit();
                case "COMPARE_FUND_MOVEMENTS":
                    $pointOfSaleDao = new PointOfSaleDAO();
                    $cashMovementDao = new CashMovementDAO();
                    $pointsOfSaleIds = json_decode($_POST["pointsOfSale"], true);
                    $pointsOfSale = [];
                    foreach ($pointsOfSaleIds as $pointOfSaleId) {
                        $pointsOfSale[] = $pointOfSaleDao->getPointOfSaleById((int)$pointOfSaleId);
                    }
                    if (count($pointsOfSale) == 0) {
                        $pointsOfSale = $pointOfSaleDao->listPointOfSales();
                    }
                    $startDate = new DateTime("{$_POST["startDate"]} 00:00:00");
                    $endDate = new DateTime("{$_POST["endDate"]} 23:59:59");
                    $pointsOfSalesOccurrences = [];
                    foreach ($pointsOfSale as $pointOfSale) {
                        $cashMovement = $cashMovementDao->getSumOfGlobalPaymentsOnInterval($startDate, $endDate, $pointOfSale);
                        $pointsOfSalesOccurrences[] = [
                            "x" => $pointOfSale->getNomPointVente(),
                            "entries" => $cashMovement["sumOfMoneyIn"] ? $cashMovement["sumOfMoneyIn"] : 0,
                            "exits" => $cashMovement["sumOfMoneyOut"] ? $cashMovement["sumOfMoneyOut"] : 0,
                        ];
                    }
                    usort($pointsOfSalesOccurrences, "ViewController::sortComparativeFundMovementsArray");
                    echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["pointsOfSale" => $pointsOfSalesOccurrences]);
                    exit();
            }
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }
}
