<?php

//This file is intent to check subscription validity on any request

class SubscriptionChecker
{
    static function checkSubscriptionValidity($controllerInstance)
    {
        if (is_null(SUBSCRIPTION_ACTIVATION_DATE) || is_null(SUBSCRIPTION_EXPIRATION_DATE) || is_null(SUBSCRIPTION_PACKAGE)) {
            if ($controllerInstance->getRequest()->isPost()) {
                echo ApiResponse::respond(UNCERTIFIED_SITE_CODE);
            } else {
                $base = rtrim($controllerInstance->getRequest()->getBaseUrl(), '/');
                $correctUrl = $base . '/Subscription/uncertified';
                header("Location: {$correctUrl}");
            }
            exit();
        }

        $subscriptionPackageDao = new SubscriptionPackageDAO();
        $subscriptionPackage = $subscriptionPackageDao->getSubscriptionPackageById((int)SUBSCRIPTION_PACKAGE);
        if (is_null($subscriptionPackage)) {
            if ($controllerInstance->getRequest()->isPost()) {
                echo ApiResponse::respond(UNCERTIFIED_SITE_CODE);
            } else {
                $base = rtrim($controllerInstance->getRequest()->getBaseUrl(), '/');
                $correctUrl = $base . '/Subscription/uncertified';
                header("Location: {$correctUrl}");
            }
            exit();
        }

        $activationDateTimestamp = (int)SUBSCRIPTION_ACTIVATION_DATE;
        $expirationDateTimestamp = (int)SUBSCRIPTION_EXPIRATION_DATE;
        $lastVerificationDateTimestamp = (int)LCE_LAST_VERIFICATION_DATE;
        $currentDateTimestamp = strtotime(date(DATE_FORMAT_WITH_SECONDS));
        if ($currentDateTimestamp < $lastVerificationDateTimestamp || $currentDateTimestamp < $activationDateTimestamp || $currentDateTimestamp > $expirationDateTimestamp) {
            if ($controllerInstance->getRequest()->isPost()) {
                echo ApiResponse::respond(EXPIRED_SUBSCRIPTION_CODE);
            } else {
                $base = rtrim($controllerInstance->getRequest()->getBaseUrl(), '/');
                $correctUrl = $base . '/Subscription/expired';
                header("Location: {$correctUrl}");
            }
            exit();
        }

        $parameterDao = new ParameterDAO();
        $lceLastVerificationDateParameter = $parameterDao->getParameterByName("LCE_LAST_VERIFICATION_DATE");
        $lceLastVerificationDateParameter->setValeur($currentDateTimestamp);
        $parameterDao->update($lceLastVerificationDateParameter);
    }
}
