<?php

include_once APPLICATION_PATH . '/includes/daos.php';

$menuDao = new MenuDAO();
$profileLineDao = new ProfileLineDAO();
$userDao = new UserDAO();
$currentUser = $userDao->getUserById($_SESSION[SESSION_USER_ID]);
$menus = [];
$parents = [];
$parentsMenu = $menuDao->getTopMenus();

foreach ($parentsMenu as $parent) {
    $parents[$parent->getIdMenu()] = [
        "icon" => $parent->getIcon(),
        "name" => $this->translator->_($parent->getNom()),
    ];
    $menus[$parent->getIdMenu()] = [];
    $childrenMenu = $menuDao->getChildMenusOfMenu($parent);
    if (is_array($childrenMenu) && count($childrenMenu) > 0) {
        foreach ($childrenMenu as $childMenu) {
            if ($profileLineDao->hasMenuRight($currentUser->getIdProfil()->getIdProfil(), $childMenu->getIdMenu())) {
                $menus[$parent->getIdMenu()][] = [
                    "id" => $childMenu->getIdMenu(),
                    "name" => $this->translator->_($childMenu->getNom()),
                    "link" => $this->url(array('controller' => $childMenu->getControlleur(), 'action' => $childMenu->getAction()), null, true),
                ];
            }
        }
    }
}

?>
<script src="<?= $this->baseUrl('javascript/utility/menu_activator.js'); ?>"></script>
<script type="text/javascript">
    let parents = <?= json_encode($parents) ?>;
    let menus = <?= json_encode($menus) ?>;
    $(document).ready(function() {
        let documentWidth = $(document).width();

        //Activate particles background of app-main-bloc
        $(".app-main-bloc")
            .particles()
            .init({
                    background: {
                        color: {
                            value: "<?= APP_PRIMARY_BRAND ?>",
                        },
                    },
                    fpsLimit: 15,
                    interactivity: {
                        events: {
                            onClick: {
                                enable: false,
                                mode: "push",
                            },
                            onHover: {
                                enable: false,
                                mode: "repulse",
                            },
                            resize: false,
                        },
                        modes: {
                            push: {
                                quantity: 4,
                            },
                            repulse: {
                                distance: 200,
                                duration: 1,
                            },
                        },
                    },
                    particles: {
                        color: {
                            value: "<?= APP_THIRD_BRAND ?>",
                        },
                        links: {
                            color: "transparent",
                            distance: 200,
                            enable: false,
                            opacity: 0.5,
                            width: 1,
                        },
                        collisions: {
                            enable: true,
                        },
                        move: {
                            direction: "none",
                            enable: true,
                            outModes: {
                                default: "bounce",
                            },
                            random: false,
                            speed: 0.35,
                            straight: false,
                        },
                        number: {
                            density: {
                                enable: true,
                                area: 800,
                            },
                            value: 150,
                        },
                        opacity: {
                            value: 0.5,
                        },
                        shape: {
                            type: "circle",
                        },
                        size: {
                            value: {
                                min: 1,
                                max: 5
                            },
                        },
                    },
                    detectRetina: true,
                },
                function(container) {
                    $(container.canvas.element).css({
                        "z-index": "-1",
                    });
                    $(".app-content-bloc").css({
                        "background-color": "rgba(13, 12, 12, 0.3)",
                    });
                }
            );

        //Click on parent menu
        $(".parent-menu").on("click", function(event) {
            event.preventDefault();
            let parentId = $(this).attr("about");
            let childrenMenus = menus[parentId];
            $(".app-menu-detail-bloc-module-section").empty();
            $(".app-menu-detail-bloc-module-section").append(
                `<i class='${parents[parentId]["icon"]} app-menu-detail-bloc-module-section-icon'></i>`
            );
            $(".app-menu-detail-bloc-module-section").append(
                `<strong class='app-menu-detail-bloc-module-section-name'>${parents[parentId]["name"]}</strong>`
            );
            $(".app-menu-detail-bloc-children-section").empty();
            childrenMenus.forEach(function(child) {
                $(".app-menu-detail-bloc-children-section").append(
                    +child["id"] === MenuActivator.activeChild ?
                    `<a href='${child["link"]}' class='app-menu-detail-bloc-child-item child-menu active' about='${child["id"]}'>
                        <strong class='app-menu-detail-bloc-child-item-text'>${child["name"]}</strong>
                        <i class='fa fa-caret-right fa-fw app-menu-detail-bloc-child-item-suffix-icon'></i>
                     </a>` :
                    `<a href='${child["link"]}' class='app-menu-detail-bloc-child-item child-menu' about='${child["id"]}'>
                        <strong class='app-menu-detail-bloc-child-item-text'>${child["name"]}</strong>
                        <i class='fa fa-caret-right fa-fw app-menu-detail-bloc-child-item-suffix-icon'></i>
                     </a>`
                );
            });
            $(".app-menu-detail-bloc").css({
                "display": "flex",
                "min-height": `${$(document).height()}px`,
            });
            $(".app-menu-detail-dimmer").css({
                "display": "flex",
                "min-height": `${$(document).height()}px`,
            });
        });

        //Initialize sticky for app-menu-bloc-sticky
        if (documentWidth >= SCREEN_WIDTH_MIN_DIMENSION_FOR_STICKY) {
            $(".app-menu-bloc-sticky").sticky({
                context: ".app-main-bloc",
                observeChanges: true,
            });
        }

        //Try sticky on app-content-bloc-top-section
        $(".app-content-bloc-top-section").addClass("ui sticky");
        $(".app-content-bloc-top-section").sticky({
            context: ".app-main-bloc",
            observeChanges: true,
        });

        //Click on dimmer or back icon
        $(".app-menu-detail-dimmer,.app-menu-detail-bloc-back-icon").on("click", function(event) {
            event.preventDefault();
            $(".app-menu-detail-dimmer").css({
                "display": "none",
            });
            $(".app-menu-detail-bloc").css({
                "display": "none",
            });
        });

        //Click on collapse menu
        $(".app-menu-bloc-minify-content").on("click", function(event) {
            event.preventDefault();
            $(".app-menu-bloc").css({
                "display": "none",
            });
            $(".app-menu-minified-bloc").css({
                "display": "flex",
            });
            $(".app-content-bloc").css({
                "width": "calc(100vw - 75px)",
            });
            if (documentWidth >= SCREEN_WIDTH_MIN_DIMENSION_FOR_STICKY) {
                $(".app-menu-minified-bloc-sticky").sticky({
                    context: ".app-main-bloc",
                    observeChanges: true,
                });
            }
        });

        //Click on expand menu
        $(".app-menu-minified-bloc-expand-content").on("click", function(event) {
            event.preventDefault();
            $(".app-menu-bloc").css({
                "display": "flex",
            });
            $(".app-menu-minified-bloc").css({
                "display": "none",
            });
            $(".app-content-bloc").css({
                "width": "calc(100vw - 250px)",
            });
            if (documentWidth >= SCREEN_WIDTH_MIN_DIMENSION_FOR_STICKY) {
                $(".app-menu-bloc-sticky").sticky("refresh");
            }
        });

        //User logged user info popup
        Utility.activatePopup("loggedUserTopAvatarImage", "loggedUserInfoPopup", {
            position: "bottom center",
        });

        //Click on loggedUserProfileImageEditIcon
        $("#loggedUserProfileImageEditIcon").on("click", function(event) {
            event.preventDefault();
            $("#loggedUserProfileImageInput").trigger("click");
        });

        //Profile image selection handler
        $("#loggedUserProfileImageInput").on("change", function() {
            if (this.files && this.files[0]) {
                if (this.files[0].size > IMAGE_UPLOAD_LIMIT_SIZE) {
                    Toaster.warning("", IMAGE_UPLOAD_SIZE_TOO_LARGE_LABEL);
                    return;
                }
                let reader = new FileReader();
                reader.onload = async function(event) {
                    $("#loggedUserTopAvatarImage").attr("src", event.target.result);
                    $("#loggerUserProfileImageZoom").attr("src", event.target.result);
                    let parameters = new FormData($("#loggedUserProfileImageForm")[0]);
                    let response = ApiResponse.create();
                    try {
                        response = await Requester.postFormData(
                            "<?= $this->url(array("controller" => "User", "action" => "updateprofileimage")); ?>",
                            parameters
                        );
                    } catch (_) {}
                };
                reader.readAsDataURL(this.files[0]);
            }
        });

        //Initialize two factor checkbox
        <?php
        if ($_SESSION[SESSION_USER_TWO_FACTOR_ENABLED]) {
        ?>
            $("#twoFactorCheckbox").checkbox("check");
        <?php
        } else {
        ?>
            $("#twoFactorCheckbox").checkbox("uncheck");
        <?php
        }
        ?>
        $("#twoFactorInput").removeClass("hidden");

        //Click on twoFactorInput
        $("#twoFactorInput, .app-content-bloc-top-section-user-part-two-factor-label").on("click", async function(event) {
            event.preventDefault();
            Utility.hidePopup("loggedUserTopAvatarImage");
            let currentState = +"<?= $_SESSION[SESSION_USER_TWO_FACTOR_ENABLED] ?>";
            if (currentState === 1) {
                Utility.showModal("disableTwoFactorModal");
                $("#disableTwoFactorDimmer").show();
                let response = ApiResponse.create();
                try {
                    response = await Requester.get(
                        "<?= $this->url(array("controller" => "User", "action" => "twofactoroffcode")); ?>"
                    );
                } catch (_) {}
                $("#disableTwoFactorDimmer").hide();
            } else {
                Utility.showModal("enableTwoFactorModal");
                $("#enableTwoFactorDimmer").show();
                let response = ApiResponse.create();
                try {
                    response = await Requester.get(
                        "<?= $this->url(array("controller" => "User", "action" => "twofactoroncode")); ?>"
                    );
                } catch (_) {}
                $("#enableTwoFactorDimmer").hide();
            }
        });

        //Click on verifyEnableTwoFactorOtpCodeButton
        $("#verifyEnableTwoFactorOtpCodeButton").on("click", async function(event) {
            event.preventDefault();
            let verificationCode = $("#enableTwoFactorOtpCodeInput").val().trim();
            if (verificationCode === "") {
                Toaster.warning("", "<?= $this->translator->_("shared.pleaseEnterTheVerificationCode"); ?>");
                return;
            }
            let parameters = {
                verificationCode: verificationCode,
            };
            $(this).addClass("loading");
            let response = ApiResponse.create();
            try {
                response = await Requester.post(
                    "<?= $this->url(array("controller" => "User", "action" => "enabletwofactor")); ?>",
                    parameters
                );
            } catch (_) {}
            if (response.isSuccess) {
                if (response.shouldNotify) {
                    if (response.code === REQUEST_SUCCESS_CODE) {
                        Toaster.success("", "<?= $this->translator->_("shared.doubleAuthenticationActivated"); ?>");
                        await Utility.delay(1000);
                        location.reload();
                    } else if (response.code === 101) {
                        Toaster.error("", "<?= $this->translator->_("shared.incorrectVerificationCode"); ?>");
                    } else {
                        Toaster.error("", "<?= $this->translator->_("shared.anErrorOccurredPleaseTryAgainLater"); ?>");
                    }
                }
            } else {
                if (response.shouldNotify) {
                    Toaster.error("", "<?= $this->translator->_("shared.anErrorOccurredPleaseTryAgainLater"); ?>");
                }
            }
            $(this).removeClass("loading");
        });

        //Click on verifyDisableTwoFactorOtpCodeButton
        $("#verifyDisableTwoFactorOtpCodeButton").on("click", async function(evnet) {
            event.preventDefault();
            let verificationCode = $("#disableTwoFactorOtpCodeInput").val().trim();
            if (verificationCode === "") {
                Toaster.warning("", "<?= $this->translator->_("shared.pleaseEnterTheVerificationCode"); ?>");
                return;
            }
            let parameters = {
                verificationCode: verificationCode,
            };
            $(this).addClass("loading");
            let response = ApiResponse.create();
            try {
                response = await Requester.post(
                    "<?= $this->url(array("controller" => "User", "action" => "disabletwofactor")); ?>",
                    parameters
                );
            } catch (_) {}
            if (response.isSuccess) {
                if (response.shouldNotify) {
                    if (response.code === REQUEST_SUCCESS_CODE) {
                        Toaster.success("", "<?= $this->translator->_("shared.doubleAuthenticationDeactivated"); ?>");
                        await Utility.delay(1000);
                        location.reload();
                        Utility.hideModal("disableTwoFactorModal");
                    } else if (response.code === 101) {
                        Toaster.error("", "<?= $this->translator->_("shared.incorrectVerificationCode"); ?>");
                    } else {
                        Toaster.error("", "<?= $this->translator->_("shared.anErrorOccurredPleaseTryAgainLater"); ?>");
                    }
                }
            } else {
                if (response.shouldNotify) {
                    Toaster.error("", "<?= $this->translator->_("shared.anErrorOccurredPleaseTryAgainLater"); ?>");
                }
            }
            $(this).removeClass("loading");
        });

        //Date and time on page management
        function updateTime() {
            let currentTime = Time.create();
            $(".app-menu-bloc-current-date-container-text").html(currentTime.format(TimeFormat.middle));
            $(".app-menu-bloc-current-time-container-text").html(currentTime.format(TimeFormat.seconds));
            $(".app-menu-minified-bloc-current-date-container-text").html(currentTime.format(TimeFormat.middleCompact));
            $(".app-menu-minified-bloc-current-time-container-text").html(currentTime.format(TimeFormat.seconds));
            setTimeout(function() {
                updateTime();
            }, 1000);
        }
        updateTime();

        //Click on change password icon
        $("#changePassword").on("click", function(event) {
            event.preventDefault();
            Utility.showModal("changePasswordModal");
        });

        //Click on show current password icon
        $("#showCurrentPasswordButton").on("click", function() {
            if ($(this).hasClass("slash")) {
                $(this).removeClass("slash");
                $("#currentPasswordInput").attr("type", "text");
            } else {
                $(this).addClass("slash");
                $("#currentPasswordInput").attr("type", "password");
            }
        });

        //Click on show new password icon
        $("#showNewPasswordButton").on("click", function() {
            if ($(this).hasClass("slash")) {
                $(this).removeClass("slash");
                $("#newPasswordInput").attr("type", "text");
            } else {
                $(this).addClass("slash");
                $("#newPasswordInput").attr("type", "password");
            }
        });

        //Click on show confirm new password icon
        $("#showConfirmNewPasswordButton").on("click", function() {
            if ($(this).hasClass("slash")) {
                $(this).removeClass("slash");
                $("#confirmNewPasswordInput").attr("type", "text");
            } else {
                $(this).addClass("slash");
                $("#confirmNewPasswordInput").attr("type", "password");
            }
        });

        //Click on show current password due to login icon
        $("#showCurrentPasswordDueToLoginButton").on("click", function() {
            if ($(this).hasClass("slash")) {
                $(this).removeClass("slash");
                $("#currentPasswordDueToLoginInput").attr("type", "text");
            } else {
                $(this).addClass("slash");
                $("#currentPasswordDueToLoginInput").attr("type", "password");
            }
        });

        //Click on show new password icon
        $("#showNewPasswordDueToLoginButton").on("click", function() {
            if ($(this).hasClass("slash")) {
                $(this).removeClass("slash");
                $("#newPasswordDueToLoginInput").attr("type", "text");
            } else {
                $(this).addClass("slash");
                $("#newPasswordDueToLoginInput").attr("type", "password");
            }
        });

        //Click on show confirm new password icon
        $("#showConfirmNewPasswordDueToLoginButton").on("click", function() {
            if ($(this).hasClass("slash")) {
                $(this).removeClass("slash");
                $("#confirmNewPasswordDueToLoginInput").attr("type", "text");
            } else {
                $(this).addClass("slash");
                $("#confirmNewPasswordDueToLoginInput").attr("type", "password");
            }
        });

        //Click on change password button
        $("#changePasswordButton").on("click", async function(event) {
            event.preventDefault();
            let currentPassword = $("#currentPasswordInput").val().trim();
            let newPassword = $("#newPasswordInput").val().trim();
            let confirmNewPassword = $("#confirmNewPasswordInput").val().trim();
            if (currentPassword === "") {
                Toaster.warning("", "<?= $this->translator->_("shared.pleaseEnterYourCurrentPassword"); ?>");
                return;
            }
            if (newPassword === "") {
                Toaster.warning("", "<?= $this->translator->_("shared.pleaseEnterYourNewPassword"); ?>");
                return;
            }
            if (confirmNewPassword === "") {
                Toaster.warning("", "<?= $this->translator->_("shared.pleaseConfirmTheNewPassword"); ?>");
                return;
            }
            if (newPassword !== confirmNewPassword) {
                Toaster.warning("", "<?= $this->translator->_("shared.passwordMismatch"); ?>");
                return;
            }
            if (newPassword === currentPassword) {
                Toaster.warning("", "<?= $this->translator->_("shared.theNewPasswordMustBeDifferentFromTheCurrentOne"); ?>");
                return;
            }
            let parameters = {
                currentPassword: currentPassword,
                newPassword: newPassword,
            };
            $(this).addClass("loading");
            let response = ApiResponse.create();
            try {
                response = await Requester.post(
                    "<?= $this->url(array("controller" => "Password", "action" => "change")); ?>",
                    parameters
                );
            } catch (_) {}
            if (response.isSuccess) {
                if (response.shouldNotify) {
                    if (response.code === REQUEST_SUCCESS_CODE) {
                        Toaster.success("", "<?= $this->translator->_("shared.youChangedYourPassword"); ?>");
                        location.replace("<?= $this->url(array('controller' => 'Index', 'action' => 'signout')); ?>");
                    } else if (response.code === 101) {
                        Toaster.error("", "<?= $this->translator->_("shared.incorrectCurrentPassword"); ?>")
                    } else {
                        Toaster.error("", "<?= $this->translator->_("shared.anErrorOccurredPleaseTryAgainLater"); ?>");
                    }
                }
            } else {
                if (response.shouldNotify) {
                    Toaster.error("", "<?= $this->translator->_("shared.anErrorOccurredPleaseTryAgainLater"); ?>");
                }
            }
            $(this).removeClass("loading");
        });

        //Click on change password due to login button
        $("#changePasswordDueToLoginButton").on("click", async function(event) {
            event.preventDefault();
            let currentPassword = $("#currentPasswordDueToLoginInput").val().trim();
            let newPassword = $("#newPasswordDueToLoginInput").val().trim();
            let confirmNewPassword = $("#confirmNewPasswordDueToLoginInput").val().trim();
            if (currentPassword === "") {
                Toaster.warning("", "<?= $this->translator->_("shared.pleaseEnterYourCurrentPassword"); ?>");
                return;
            }
            if (newPassword === "") {
                Toaster.warning("", "<?= $this->translator->_("shared.pleaseEnterYourNewPassword"); ?>");
                return;
            }
            if (confirmNewPassword === "") {
                Toaster.warning("", "<?= $this->translator->_("shared.pleaseConfirmTheNewPassword"); ?>");
                return;
            }
            if (newPassword !== confirmNewPassword) {
                Toaster.warning("", "<?= $this->translator->_("shared.passwordMismatch"); ?>");
                return;
            }
            if (newPassword === currentPassword) {
                Toaster.warning("", "<?= $this->translator->_("shared.theNewPasswordMustBeDifferentFromTheCurrentOne"); ?>");
                return;
            }
            let parameters = {
                currentPassword: currentPassword,
                newPassword: newPassword,
            };
            $(this).addClass("loading");
            let response = ApiResponse.create();
            try {
                response = await Requester.post(
                    "<?= $this->url(array("controller" => "Password", "action" => "change")); ?>",
                    parameters
                );
            } catch (_) {}
            if (response.isSuccess) {
                if (response.shouldNotify) {
                    if (response.code === REQUEST_SUCCESS_CODE) {
                        Toaster.success("", "<?= $this->translator->_("shared.youChangedYourPassword"); ?>");
                        location.replace("<?= $this->url(array('controller' => 'Index', 'action' => 'signout')); ?>");
                    } else if (response.code === 101) {
                        Toaster.error("", "<?= $this->translator->_("shared.incorrectCurrentPassword"); ?>")
                    } else {
                        Toaster.error("", "<?= $this->translator->_("shared.anErrorOccurredPleaseTryAgainLater"); ?>");
                    }
                }
            } else {
                if (response.shouldNotify) {
                    Toaster.error("", "<?= $this->translator->_("shared.anErrorOccurredPleaseTryAgainLater"); ?>");
                }
            }
            $(this).removeClass("loading");
        });

        //Click on show current password icon
        $("#showSignInAgainPasswordButton").on("click", function() {
            if ($(this).hasClass("slash")) {
                $(this).removeClass("slash");
                $("#signInAgainPasswordInput").attr("type", "text");
            } else {
                $(this).addClass("slash");
                $("#signInAgainPasswordInput").attr("type", "password");
            }
        });

        //Click on sign again button
        $("#signInAgainButton").on("click", async function(event) {
            event.preventDefault();
            let login = $("#signInAgainLoginInput").val().trim();
            if (login === "") {
                Toaster.warning("", "<?= $this->translator->_("shared.pleaseEnterYourLogin"); ?>");
                return;
            }
            let password = $("#signInAgainPasswordInput").val().trim();
            if (password === "") {
                Toaster.warning("", "<?= $this->translator->_("shared.pleaseEnterYourPassword"); ?>");
                return;
            }
            let parameters = {
                login: login,
                password: password,
            };
            $(this).addClass("loading");
            let response = ApiResponse.create();
            try {
                response = await Requester.post(
                    "<?= $this->url(array("controller" => "Index", "action" => "signin")); ?>",
                    parameters
                );
            } catch (_) {}
            if (response.isSuccess) {
                if (response.code === REQUEST_SUCCESS_CODE) {
                    Toaster.success("<?= $this->translator->_("shared.signInSucceed"); ?>", "<?= $this->translator->_("shared.youAreLoggedInAgain"); ?>");
                    Utility.hideModal("signInAgainModal");
                } else if (response.code === 101) {
                    Toaster.error("<?= $this->translator->_("shared.signInFailed") ?>", "<?= $this->translator->_("shared.loginOrPasswordIncorrect"); ?>")
                } else {
                    Toaster.error("", "<?= $this->translator->_("shared.anErrorOccurredPleaseTryAgainLater"); ?>");
                }
            } else {
                Toaster.error("", "<?= $this->translator->_("shared.anErrorOccurredPleaseTryAgainLater"); ?>");
            }
            $(this).removeClass("loading");
        });

        //Click on goToFullScreen button
        $("#goToFullscreen").on("click", function(event) {
            event.preventDefault();
            Utility.togglePageFullScreen();
        });

        //Click on refreshPage button
        $("#refreshPage").on("click", function(event) {
            event.preventDefault();
            location.reload();
        });

        //Check if the password is the same as login
        async function checkPasswordIsSameAsLogin() {
            let response = ApiResponse.create();
            try {
                response = await Requester.post(
                    "<?= $this->url(array("controller" => "Password", "action" => "passwordislogin")); ?>", {}
                );
            } catch (_) {}
            if (response.isSuccess) {
                if (response.shouldNotify) {
                    if (response.code === REQUEST_SUCCESS_CODE) {
                        if (response.data["same"] === 1) {
                            Utility.showModal("changePasswordDueToLoginModal");
                        }
                    }
                }
            }
        }
        checkPasswordIsSameAsLogin();

        //If we are on mobile or screen is small enough always collapse the menu
        if (Utility.weAreOnMobileOrTablet() || documentWidth < SCREEN_WIDTH_MIN_DIMENSION_FOR_STICKY) {
            MenuActivator.collapseMenu();
        }

        //Language selection dropdown initialization
        Utility.initDropdown("languageSelect", {
            clearable: false,
            onChange: async function(language, _, __) {
                if (language !== "<?= $_SESSION[SESSION_LANGUAGE] ?>") {
                    let response = ApiResponse.create();
                    try {
                        response = await Requester.post(
                            "<?= $this->url(array("controller" => "Language", "action" => "changelocale")); ?>", {
                                "locale": language
                            },
                        );
                    } catch (_) {}
                    if (response.isSuccess) {
                        if (response.shouldNotify) {
                            if (response.code === REQUEST_SUCCESS_CODE) {
                                location.reload();
                            }
                        }
                    }
                }
            }
        });
        Utility.selectValueOnDropdown("languageSelect", "<?= $_SESSION[SESSION_LANGUAGE] ?>");

        //Prevent any form from submitting
        $("form").on("submit", function(event) {
            event.preventDefault();
        });

        //Add subscription ending warning when needed
        let checkWarningCurrentDate = Time.create();
        let checkWarningExpirationDate = Time.createFromTimestamp(+"<?= SUBSCRIPTION_EXPIRATION_DATE ?>");
        if (checkWarningExpirationDate.difference(checkWarningCurrentDate) <= +"<?= SUBSCRIPTION_ENDING_WARNING_DAYS_COUNT ?>") {
            $(".app-content-bloc-main-section").prepend(`
                <div class='app-content-bloc-main-bloc-subscription-warning-bloc'>
                    <strong><i class="info circle icon"></i> <?= $this->translator->_("shared.yourSubscriptionIsExpiringSoon"); ?></strong>
                    <div id='subscriptionWarningCountdown' class='flipdown mini'></div>
                </div>
            `);
            $(".app-content-bloc-main-section").css("padding-top", "0px");
            let flipdown = new FlipDown(+"<?= SUBSCRIPTION_EXPIRATION_DATE ?>", "subscriptionWarningCountdown", {
                headings: [
                    "<?= $this->translator->_("subscription.days"); ?>",
                    "<?= $this->translator->_("subscription.hours"); ?>",
                    "<?= $this->translator->_("subscription.minutes"); ?>",
                    "<?= $this->translator->_("subscription.seconds"); ?>",
                ],
                theme: "secondary",
            });
            flipdown.start();
            flipdown.ifEnded(() => {
                location.reload();
            });
        }
    });
</script>