//This utility act as a chart.js interface
//So be sure that chart.js is loaded

const ChartType = {
  area: "area",
  bar: "bar",
  bubble: "bubble",
  doughnut: "doughnut",
  pie: "pie",
  line: "line",
  polarArea: "polarArea",
  radar: "radar",
  scatter: "scatter",
};

class Dataset {
  type = null;
  label = null;
  data = null; // {x, y} or just y
  backgroundColor = null;
  borderColor = null;
  order = null;

  constructor(
    type = ChartType.bar,
    label = "",
    data = [],
    backgroundColor = null,
    borderColor = null,
    order = null
  ) {
    this.type = type;
    this.label = label;
    this.data = data;
    this.backgroundColor = backgroundColor;
    this.borderColor = borderColor;
    this.order = order;
  }

  static create(
    type = ChartType.bar,
    label = "",
    data = [],
    backgroundColor = null,
    borderColor = null,
    order = null
  ) {
    return new Dataset(type, label, data, backgroundColor, borderColor, order);
  }
}

class ChartRender {
  static render({
    canvasId,
    type = ChartType.bar,
    labels = [],
    datasets = [],
    options = {},
  }) {
    let canvasContext = document.getElementById(canvasId).getContext("2d");
    return new Chart(canvasContext, {
      plugins: [ChartDataLabels],
      type: type,
      data: {
        labels: labels,
        datasets: datasets.map((dataset) => {
          let chartDataset = {
            label: dataset.label,
            data: dataset.data,
            type: dataset.type,
          };
          if (dataset.backgroundColor !== null)
            chartDataset["backgroundColor"] = dataset.backgroundColor;
          if (dataset.borderColor !== null)
            chartDataset["borderColor"] = dataset.borderColor;
          if (dataset.order !== null) chartDataset["order"] = dataset.order;

          return chartDataset;
        }),
      },
      options: {
        maintainAspectRatio: false,
        plugins: {
          legend: {
            display: true,
            position: "bottom",
          },
          datalabels: {
            anchor: "end",
            labels: {
              title: {
                font: {
                  weight: "bold",
                  size: 8,
                },
              },
            },
            display: "auto",
            align: "top",
            formatter: function (value, _) {
              return Utility.formatAmount(value);
            },
          },
          tooltip: {
            callbacks: {
              label: function (context) {
                let label = context.dataset.label || "";

                if (label) {
                  label += ": ";
                }
                if (context.parsed.y !== null) {
                  label += Utility.formatAmount(context.parsed.y);
                }
                return label;
              },
            },
          },
          customCanvasBackgroundColor: {
            color: APP_THIRD_BRAND,
          },
        },
        scales: {
          y: {
            ticks: {
              //We will format amount on y axes
              callback: function (value, _, __) {
                return Utility.formatAmount(value);
              },
            },
          },
        },
        ...options,
      },
    });
  }
}
