const REQUEST_SUCCESS_CODE = 100;
const SESSION_EXPIRED_CODE = 210;
const NO_ACCESS_RIGHT_DETECTED_CODE = 320;
const ALREADY_DELETED_ISSUE_CODE = 430;
const ALREADY_VALIDATED_ISSUE_CODE = 540;
const UNCERTIFIED_SITE_CODE = 650;
const EXPIRED_SUBSCRIPTION_CODE = 760;

class ApiResponse {
  code = null;
  message = null;
  data = null;
  isSuccess = null;
  shouldNotify = null;

  constructor(
    isSuccess = false,
    shouldNotify = true,
    code = null,
    message = null,
    data = null
  ) {
    this.code = code;
    this.message = message;
    this.data = data;
    this.isSuccess = isSuccess;
    this.shouldNotify = shouldNotify;
  }

  static create(
    isSuccess = false,
    shouldNotify = true,
    code = null,
    message = null,
    data = null
  ) {
    return new ApiResponse(isSuccess, shouldNotify, code, message, data);
  }
}

class Requester {
  static async post(url, data) {
    let response = ApiResponse.create();
    await $.ajax({
      url: url,
      data: data,
      type: "post",
      dataType: "json",
      success: function (data) {
        if (
          data.code !== SESSION_EXPIRED_CODE &&
          data.code !== NO_ACCESS_RIGHT_DETECTED_CODE &&
          data.code !== ALREADY_DELETED_ISSUE_CODE &&
          data.code !== ALREADY_VALIDATED_ISSUE_CODE &&
          data.code !== UNCERTIFIED_SITE_CODE &&
          data.code !== EXPIRED_SUBSCRIPTION_CODE
        ) {
          response = ApiResponse.create(
            true,
            true,
            data.code,
            data.message,
            data.data
          );
        } else if (data.code === SESSION_EXPIRED_CODE) {
          response = ApiResponse.create(false, false, SESSION_EXPIRED_CODE);
          ion.sound.play("warning");
          Utility.showModal("signInAgainModal");
        } else if (data.code === ALREADY_DELETED_ISSUE_CODE) {
          Toaster.info("", ALREADY_DELETED_ISSUE_LABEL);
          response = ApiResponse.create(
            true,
            false,
            REQUEST_SUCCESS_CODE,
            data.message,
            data.data
          );
        } else if (data.code === ALREADY_VALIDATED_ISSUE_CODE) {
          Toaster.info("", ALREADY_VALIDATED_ISSUE_LABEL);
          response = ApiResponse.create(
            true,
            false,
            REQUEST_SUCCESS_CODE,
            data.message,
            data.data
          );
        } else if (data.code === UNCERTIFIED_SITE_CODE) {
          location.replace(`${SITE_PUBLIC_URL}/Subscription/uncertified`);
        } else if (data.code === EXPIRED_SUBSCRIPTION_CODE) {
          location.replace(`${SITE_PUBLIC_URL}/Subscription/expired`);
        } else {
          response = ApiResponse.create(
            false,
            false,
            NO_ACCESS_RIGHT_DETECTED_CODE
          );
          ion.sound.play("warning");
          Utility.showModal("illegalAccessModal");
        }
      },
    });
    return response;
  }

  static async postFormData(url, data) {
    let response = ApiResponse.create();
    await $.ajax({
      url: url,
      data: data,
      type: "post",
      dataType: "json",
      contentType: false,
      processData: false,
      success: function (data) {
        if (
          data.code !== SESSION_EXPIRED_CODE &&
          data.code !== NO_ACCESS_RIGHT_DETECTED_CODE &&
          data.code !== ALREADY_DELETED_ISSUE_CODE &&
          data.code !== ALREADY_VALIDATED_ISSUE_CODE &&
          data.code !== UNCERTIFIED_SITE_CODE &&
          data.code !== EXPIRED_SUBSCRIPTION_CODE
        ) {
          response = ApiResponse.create(
            true,
            true,
            data.code,
            data.message,
            data.data
          );
        } else if (data.code === SESSION_EXPIRED_CODE) {
          response = ApiResponse.create(false, false, SESSION_EXPIRED_CODE);
          ion.sound.play("warning");
          Utility.showModal("signInAgainModal");
        } else if (data.code === ALREADY_DELETED_ISSUE_CODE) {
          Toaster.info("", ALREADY_DELETED_ISSUE_LABEL);
          response = ApiResponse.create(
            true,
            false,
            REQUEST_SUCCESS_CODE,
            data.message,
            data.data
          );
        } else if (data.code === ALREADY_VALIDATED_ISSUE_CODE) {
          Toaster.info("", ALREADY_VALIDATED_ISSUE_LABEL);
          response = ApiResponse.create(
            true,
            false,
            REQUEST_SUCCESS_CODE,
            data.message,
            data.data
          );
        } else if (data.code === UNCERTIFIED_SITE_CODE) {
          location.replace(`${SITE_PUBLIC_URL}/Subscription/uncertified`);
        } else if (data.code === EXPIRED_SUBSCRIPTION_CODE) {
          location.replace(`${SITE_PUBLIC_URL}/Subscription/expired`);
        } else {
          response = ApiResponse.create(
            false,
            false,
            NO_ACCESS_RIGHT_DETECTED_CODE
          );
          ion.sound.play("warning");
          Utility.showModal("illegalAccessModal");
        }
      },
    });
    return response;
  }

  static async get(url, data = null) {
    let response = ApiResponse.create();
    await $.ajax({
      url: url,
      data: data,
      type: "get",
      dataType: "json",
      contentType: false,
      processData: false,
      success: function (data) {
        if (
          data.code !== SESSION_EXPIRED_CODE &&
          data.code !== NO_ACCESS_RIGHT_DETECTED_CODE &&
          data.code !== ALREADY_DELETED_ISSUE_CODE &&
          data.code !== ALREADY_VALIDATED_ISSUE_CODE &&
          data.code !== UNCERTIFIED_SITE_CODE &&
          data.code !== EXPIRED_SUBSCRIPTION_CODE
        ) {
          response = ApiResponse.create(
            true,
            true,
            data.code,
            data.message,
            data.data
          );
        } else if (data.code === SESSION_EXPIRED_CODE) {
          response = ApiResponse.create(false, false, SESSION_EXPIRED_CODE);
          ion.sound.play("warning");
          Utility.showModal("signInAgainModal");
        } else if (data.code === ALREADY_DELETED_ISSUE_CODE) {
          Toaster.info("", ALREADY_DELETED_ISSUE_LABEL);
          response = ApiResponse.create(
            true,
            false,
            REQUEST_SUCCESS_CODE,
            data.message,
            data.data
          );
        } else if (data.code === ALREADY_VALIDATED_ISSUE_CODE) {
          Toaster.info("", ALREADY_VALIDATED_ISSUE_LABEL);
          response = ApiResponse.create(
            true,
            false,
            REQUEST_SUCCESS_CODE,
            data.message,
            data.data
          );
        } else if (data.code === UNCERTIFIED_SITE_CODE) {
          location.replace(`${SITE_PUBLIC_URL}/Subscription/uncertified`);
        } else if (data.code === EXPIRED_SUBSCRIPTION_CODE) {
          location.replace(`${SITE_PUBLIC_URL}/Subscription/expired`);
        } else {
          response = ApiResponse.create(
            false,
            false,
            NO_ACCESS_RIGHT_DETECTED_CODE
          );
          ion.sound.play("warning");
          Utility.showModal("illegalAccessModal");
        }
      },
    });
    return response;
  }
}
