//This utility act as moment js interface
//So be sure that moment js is loaded on the page

const TimeFormat = {
  compact: "YYYY-MM-DD",
  compactWithTime: "YYYY-MM-DD HH:mm",
  compactWithSeconds: "YYYY-MM-DD HH:mm:ss",
  middle: "Do MMMM YYYY",
  middleCompact: "Do MMM YY",
  middleWithTime: "Do MMMM YYYY HH:mm",
  middleWithSeconds: "Do MMMM YYYY HH:mm:ss",
  long: "dddd Do MMMM YYYY",
  longWithTime: "dddd Do MMMM YYYY HH:mm",
  longWithSeconds: "dddd Do MMMM YYYY HH:mm:ss",
  time: "HH:mm",
  seconds: "HH:mm:ss",
  secondsOnly: "ss",
  year: "YYYY",
  month: "MM",
  day: "DD",
  monthLetter: "MMMM",
};
const DatepickerFormat = {
  date: "Y-m-d",
  datetime: "Y-m-d H:i",
  time: "H:i",
};
const TimeUnits = {
  years: "years",
  months: "months",
  weeks: "weeks",
  days: "days",
  hours: "hours",
  minutes: "minutes",
  seconds: "seconds",
};
const TimeUnit = {
  year: "year",
  month: "month",
  week: "week",
  day: "day",
};
const FullCalendarViews = {
  dayGridMonth: "dayGridMonth",
  timeGridWeek: "timeGridWeek",
  timeGridDay: "timeGridDay",
};

class Time {
  time = null;

  constructor(time = null, format = TimeFormat.compact) {
    this.time = time ? moment(time, format) : moment();
  }

  static setLocale(locale) {
    moment.locale(locale);
  }

  static create(time = null, format = TimeFormat.compact) {
    return new Time(time, format);
  }

  static createFromTimestamp(timestamp) {
    let momentTimestamp = moment.unix(timestamp);
    return new Time(
      momentTimestamp.format(TimeFormat.compactWithSeconds),
      TimeFormat.compactWithSeconds
    );
  }

  static isValid(time, format = TimeFormat.compact) {
    return moment(time, format).isValid();
  }

  static getWeekDayDate(day = 0) {
    return Time.create(moment().weekday(day).format(TimeFormat.compact));
  }

  format(format = TimeFormat.long) {
    return Utility.capitalizeAll(this.time.format(format));
  }

  difference(time, unit = TimeUnits.days) {
    return this.time.diff(time.time, unit);
  }

  add(count, unit = TimeUnits.days) {
    this.time.add(count, unit);
  }

  subtract(count, unit = TimeUnits.days) {
    this.time.subtract(count, unit);
  }

  startOf(unit = TimeUnit.year) {
    return this.time.startOf(unit);
  }

  endOf(unit = TimeUnit.year) {
    return this.time.endOf(unit);
  }

  isAfter(time) {
    return this.time.isAfter(time.time);
  }

  isBefore(time) {
    return this.time.isBefore(time.time);
  }

  relative() {
    return this.time.fromNow();
  }

  isSame(time) {
    return this.time.isSame(time.time);
  }
}
