//Be sure that Jquery is loaded on the page
const ConfirmationTheme = {
  white: "white",
  dark: "dark",
  light: "light",
};

class Utility {
  //Semantic UI
  static showModal(
    modalId,
    settings = {},
    closable = false,
    autofocus = false
  ) {
    $(`#${modalId}`)
      .modal(settings)
      .modal("setting", "closable", closable)
      .modal("setting", "autofocus", autofocus)
      .modal("setting", "observeChanges", true)
      .modal("setting", "allowMultiple", true)
      .modal("show");
  }

  //Semantic UI
  static hideModal(modalId) {
    $(`#${modalId}`).modal("hide");
  }

  //Semantic UI
  static refreshModal(modalId) {
    $(`#${modalId}`).modal("refresh");
  }

  //Semantic UI
  static disableModal(modalId, inverted = false) {
    let dimmerClass = inverted ? "inverted" : "";
    $(`#${modalId}`).append(
      `<div class='ui active ${dimmerClass} dimmer to-disable-modal'></div>`
    );
  }

  //Semantic UI
  static enableModal(modalId) {
    $(`#${modalId}`).children(".to-disable-modal").remove();
  }

  static validEmail(mail) {
    let mailFormat = /^\w+([\.-]?\w+)*@\w+([\.-]?\w+)*(\.\w{2,3})+$/;
    return mail.match(mailFormat);
  }

  static capitalize(string) {
    return `${string[0].toString().toUpperCase()}${string
      .toString()
      .substring(1)}`;
  }

  static capitalizeAll(string) {
    let splitString = string.toString().toLowerCase().split(" ");
    for (let i = 0; i < splitString.length; i++) {
      splitString[i] =
        splitString[i].charAt(0).toUpperCase() + splitString[i].substring(1);
    }
    return splitString.join(" ");
  }

  //Jquery datatable, infos here https://datatables.net/
  static datatable(tableId, options = {}) {
    $(`#${tableId}`).css("width", "100%");
    let datatableInstance = $(`#${tableId}`).DataTable({
      pageLength: 8,
      ...options,
    });
    datatableInstance
      .buttons()
      .container()
      .appendTo(
        $("div.eight.column:eq(0)", datatableInstance.table().container())
      );
    return datatableInstance;
  }

  //The table instance stay on the current page, infos here https://datatables.net/
  static refreshDatatable(datatableInstance) {
    datatableInstance.ajax.reload(null, false);
  }

  //Semantic UI
  static initDropdown(selectId, options = {}) {
    if (options["clearable"] === undefined || options["clearable"]) {
      options["fullTextSearch"] = true;
    }
    $(`#${selectId}`).dropdown({
      clearable: true,
      forceSelection: false,
      message: {
        noResults: NO_RESULTS_FOUND_LABEL,
      },
      ...options,
    });
  }

  //Semantic UI
  static selectValueOnDropdown(dropdownId, value) {
    $(`#${dropdownId}`).dropdown("set selected", value);
  }

  //Semantic UI
  static selectValuesOnDropdown(dropdownId, values) {
    $(`#${dropdownId}`).dropdown("set selected", values);
  }

  //Semantic UI
  static getDropdownText(dropdownId) {
    return $(`#${dropdownId}`).dropdown("get text");
  }

  //Semantic UI
  static getDropdownValue(dropdownId) {
    return $(`#${dropdownId}`).dropdown("get value");
  }

  //Semantic UI
  static addOptionToDropdown(dropdownId, value, label) {
    $(`#${dropdownId}`).append(`
            <option value='${value}'>
                ${label}
            <option/>
        `);
    $(`#${dropdownId}`).children().last().remove();
    $(`#${dropdownId}`).dropdown("refresh");
  }

  //Semantic UI
  static clearDropdown(dropdownId) {
    $(`#${dropdownId}`).dropdown("clear");
    $(`#${dropdownId}`)
      .parent()
      .children("i")
      .first()
      .attr("class", "dropdown icon");
  }

  //Semantic UI
  static refreshDropdown(dropdownId) {
    $(`#${dropdownId}`).dropdown("refresh");
  }

  //Semantic UI
  static emptyDropdown(dropdownId) {
    $(`#${dropdownId} > option`).not("[value='']").remove();
    $(`#${dropdownId}`).dropdown("refresh");
  }

  //Semantic UI
  static resetForm(formId) {
    $(`#${formId}`)[0].reset();
    $(`#${formId} > .field > div.ui.dropdown > select`).dropdown("clear");
    $(`#${formId} > .field > div.ui.dropdown > i`).attr(
      "class",
      "dropdown icon"
    );
    $(`#${formId} > .fields > .field > div.ui.dropdown > select`).dropdown(
      "clear"
    );
    $(`#${formId} > .fields > .field > div.ui.dropdown > i`).attr(
      "class",
      "dropdown icon"
    );
  }

  //Semantic UI
  static addLoadingStateToDropdown(dropdownId) {
    $(`#${dropdownId}`).parent().addClass("loading");
  }

  //Semantic UI
  static removeLoadingStateFromDropdown(dropdownId) {
    $(`#${dropdownId}`).parent().removeClass("loading");
  }

  static delay = (ms) => new Promise((res) => setTimeout(res, ms));

  //Use Jquery Confirm, take infos here https://craftpip.github.io/jquery-confirm/
  static async askConfirmation({
    title = "",
    message,
    theme = ConfirmationTheme.white,
    confirmLabel,
    cancelLabel,
  }) {
    ion.sound.play("warning");
    let confirmed = false;
    let alreadyClosed = false;
    $.confirm({
      title: title,
      content: message,
      theme: theme,
      animationBounce: 2.5,
      confirmButton: confirmLabel,
      cancelButton: cancelLabel,
      boxWidth: "500px",
      useBootstrap: false,
      buttons: {
        cancel: {
          text: cancelLabel,
          btnClass: "ui button app-secondary-button",
          action: function () {
            alreadyClosed = true;
          },
        },
        confirm: {
          text: confirmLabel,
          btnClass: "ui button app-primary-button",
          action: function () {
            confirmed = true;
            alreadyClosed = true;
          },
        },
      },
    });
    while (!alreadyClosed) {
      await Utility.delay(30);
    }
    return confirmed;
  }

  static formatAmount(
    amount,
    decimals = 0,
    decimalSeparator = "",
    thousandsSeparator = " "
  ) {
    amount = amount * 1;
    let str = amount
      .toFixed(decimals ? decimals : 0)
      .toString()
      .split(".");
    let parts = [];
    for (let i = str[0].length; i > 0; i -= 3) {
      parts.unshift(str[0].substring(Math.max(0, i - 3), i));
    }
    str[0] = parts.join(thousandsSeparator ? thousandsSeparator : ",");
    return str.join(decimalSeparator ? decimalSeparator : ".");
  }

  /**
   * For PrintJS take information here https://printjs.crabbly.com/
   * For PdfViewer take information here https://pdfjs.express/blog/how-to-use-pdf-js
   */
  static makePrint(httpLink) {
    //Safari and Firefox browser will be handle as mobile device
    const appleExpression = /Apple/i;
    const safariExpression = /Safari/i;
    let isSafari =
      appleExpression.test(navigator.vendor) &&
      safariExpression.test(navigator.userAgent);
    let isFirefox = navigator.userAgent.toLowerCase().indexOf("firefox") > -1;
    if (!Utility.weAreOnMobileOrTablet() && !isSafari && !isFirefox) {
      printJS(httpLink);
    } else {
      let printModalId = `autoPrintModal${+new Date()}`;
      let closePrintModalId = `closeAutoPrintModal${+new Date()}`;
      $("body").prepend(`
        <div id="${printModalId}" class="ui fullscreen long modal">
          <i class="close icon"></i>
          <div class="header">
              ${PRINTING_LABEL}
          </div>
          <div class="content">
            <iframe src="${SITE_PUBLIC_URL}/pdfviewer/web/viewer.html?file=${httpLink}" style="width: 100%; min-height: 85vh;" frameborder="0"></iframe>
          </div>
          <div class="actions">
              <button id="${closePrintModalId}" class="ui button app-primary-button">
                  ${CLOSE_LABEL}
              </button>
          </div>
        </div>
      `);
      Utility.showModal(printModalId);
      $("body").on("click", `#${closePrintModalId}`, function (event) {
        event.preventDefault();
        Utility.hideModal(printModalId);
      });
    }
  }

  //Take information here https://xdsoft.net/jqplugins/datetimepicker/#datepicker
  static initCalendarOnInput(inputId, options = {}) {
    let currentValue = "";
    if (options["timepicker"]) options["step"] = 1;
    $(`#${inputId}`).datetimepicker({
      format: DatepickerFormat.date,
      formatDate: DatepickerFormat.date,
      timepicker: false,
      ...options,
    });

    //This class is used for change trigger on val function usage
    $(`#${inputId}`).addClass("crescita-datepicker-input");

    $(`#${inputId}`).on("change", function () {
      if ($(this).val() !== "") {
        currentValue = $(this).val();
      } else if (currentValue !== "") {
        $(this).val(currentValue);
      }
    });
  }

  static togglePageFullScreen() {
    if (
      (document.fullScreenElement && document.fullScreenElement !== null) ||
      (!document.mozFullScreen && !document.webkitIsFullScreen)
    ) {
      if (document.documentElement.requestFullScreen) {
        document.documentElement.requestFullScreen();
      } else if (document.documentElement.mozRequestFullScreen) {
        document.documentElement.mozRequestFullScreen();
      } else if (document.documentElement.webkitRequestFullScreen) {
        document.documentElement.webkitRequestFullScreen(
          Element.ALLOW_KEYBOARD_INPUT
        );
      }
    } else {
      if (document.cancelFullScreen) {
        document.cancelFullScreen();
      } else if (document.mozCancelFullScreen) {
        document.mozCancelFullScreen();
      } else if (document.webkitCancelFullScreen) {
        document.webkitCancelFullScreen();
      }
    }
  }

  //Take information here https://seballot.github.io/spectrum
  static initColorPickerOnButton(buttonId, color, options = {}) {
    $(`#${buttonId}`)
      .parent()
      .children(".color-picker-label")
      .first()
      .children(".color-picker-representation")
      .first()
      .css({ "background-color": `${color}` });
    $(`#${buttonId}`).spectrum({
      color: color,
      showPalette: true,
      showInput: true,
      showButtons: false,
      showAlpha: false,
      allowEmpty: false,
      change: function (color) {
        $(`#${buttonId}`)
          .parent()
          .children(".color-picker-label")
          .first()
          .children(".color-picker-representation")
          .first()
          .css({ "background-color": `${color.toHexString()}` });
      },
      move: function (color) {
        $(`#${buttonId}`)
          .parent()
          .children(".color-picker-label")
          .first()
          .children(".color-picker-representation")
          .first()
          .css({ "background-color": `${color.toHexString()}` });
      },
      hide: function (color) {
        $(`#${buttonId}`)
          .parent()
          .children(".color-picker-label")
          .first()
          .children(".color-picker-representation")
          .first()
          .css({ "background-color": `${color.toHexString()}` });
      },
      dragStart: function (_, color) {
        $(`#${buttonId}`)
          .parent()
          .children(".color-picker-label")
          .first()
          .children(".color-picker-representation")
          .first()
          .css({ "background-color": `${color.toHexString()}` });
      },
      dragStop: function (_, color) {
        $(`#${buttonId}`)
          .parent()
          .children(".color-picker-label")
          .first()
          .children(".color-picker-representation")
          .first()
          .css({ "background-color": `${color.toHexString()}` });
        onChange(color.toHexString());
      },
      ...options,
    });
    $(`#${buttonId}`).on("click", function (event) {
      event.preventDefault();
      Utility.showColorPickerOnButton(buttonId);
      return false;
    });
  }

  //Take information here https://seballot.github.io/spectrum
  static showColorPickerOnButton(buttonId) {
    $(`#${buttonId}`).spectrum("show");
  }

  //Take information here https://seballot.github.io/spectrum
  static getColorPickerValueOnButton(buttonId) {
    return $(`#${buttonId}`).spectrum("get").toHexString();
  }

  //Take information here https://seballot.github.io/spectrum
  static setColorPickerValueOnButton(buttonId, color) {
    $(`#${buttonId}`).spectrum("set", color);
    $(`#${buttonId}`)
      .parent()
      .children(".color-picker-label")
      .first()
      .children(".color-picker-representation")
      .first()
      .css({ "background-color": `${color}` });
    return false;
  }

  //Semantic UI
  static activatePopup(targetId, popupId, options = {}) {
    $(`#${targetId}`).popup({
      popup: $(`#${popupId}`),
      on: "click",
      ...options,
    });
  }

  static hidePopup(popupId) {
    $(`#${popupId}`).popup("hide");
  }

  static initImagePickerOnButton({ buttonId, inputId, legendId, zoomId }) {
    $(`#${buttonId}`).on("click", function (event) {
      event.preventDefault();
      $(`#${inputId}`).trigger("click");
    });

    $(`#${inputId}`).on("change", function () {
      if (this.files && this.files[0]) {
        if (this.files[0].size > IMAGE_UPLOAD_LIMIT_SIZE) {
          Toaster.warning("", IMAGE_UPLOAD_SIZE_TOO_LARGE_LABEL);
          return;
        }
        let reader = new FileReader();
        reader.onload = function (event) {
          $(`#${legendId}`).attr("src", event.target.result);
          $(`#${zoomId}`).attr("src", event.target.result);
        };
        reader.readAsDataURL(this.files[0]);
      }
    });
  }

  static adaptTextColorToRowBackgroundColor(row) {
    let currentBackgroundColor = $(row).css("background-color");
    if (
      currentBackgroundColor !== "white" &&
      currentBackgroundColor !== "#ffffff" &&
      currentBackgroundColor !== "rgb(255, 255, 255)" &&
      currentBackgroundColor !== ""
    ) {
      $(row).css("color", "#ffffff");
      $(row).children("td").css("border-top-color", "#ffffff");
      $(row).children("td").css("border-bottom-color", "#ffffff");
    }
  }

  //Take information here https://github.com/kaimallea/isMobile
  static weAreOnMobileOrTablet() {
    return isMobile.any;
  }

  //Take information here https://github.com/Kibo/numbers2words
  static convertToWords(numberToConvert, capitalize = true) {
    let toWordsConverter = new T2W(NUMBERS_TO_WORDS_CURRENT_LOCALE);
    let numberWords = toWordsConverter.toWords(numberToConvert);
    return capitalize ? Utility.capitalize(numberWords) : numberWords;
  }
}
