<?php

//All app print actions are located here

include APPLICATION_PATH . '/includes/daos.php';

class ReportController extends Zend_Controller_Action
{
    private $translator;

    public function init()
    {
        session_start();
        SubscriptionChecker::checkSubscriptionValidity($this);
        if (isset($_SESSION[SESSION_USER_ID]) && $_SESSION[SESSION_USER_ID] != null) {
            $_SESSION[LAST_ACTION_DATE] = new DateTime();
            LogDAO::updateLastActionDateForCurrentUser();
            $this->translator = Translator::init();
        } else {
            if ($this->getRequest()->isPost()) {
                echo ApiResponse::respond(SESSION_EXPIRED_CODE);
                exit();
            } else {
                $this->_helper->redirector('index', 'Index');
            }
        }
    }

    public function printproductsAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/products") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $target = (int)$_POST["target"]; //0 all products/services, 1 products, 2 services, 3 products in stock
            $productDao = new ProductDAO();
            $stockDao = new StockDAO();
            $productCategoryDao = new ProductCategoryDAO();
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = "";
            $printFilters = $this->translator->_("report.noFilter");
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: center;  
                                color: #000000; 
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            bold {
                                color: black;
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
            ";
            if ($target == 0) { //all products/services
                $documentHtml .= "
                                    <tr>
                                        <td class='title'>{$this->translator->_("report.printProducts.productsAndServicesList")}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border'>
                                <thead>
                                    <tr>													   												   
                                        <th>{$this->translator->_("report.printProducts.productOrService")}</th>
                ";
                $printTitle = $this->translator->_("report.printProducts.productsAndServicesList");
            }
            if ($target == 1) { //all products
                $documentHtml .= "
                                    <tr>
                                        <td class='title'>{$this->translator->_("report.printProducts.productsList")}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border'>
                                <thead>
                                    <tr>													   												   
                                        <th>{$this->translator->_("report.printProducts.product")}</th>
                ";
                $printTitle = $this->translator->_("report.printProducts.productsList");
            }
            if ($target == 2) { //all services
                $documentHtml .= "
                                    <tr>
                                        <td class='title'>{$this->translator->_("report.printProducts.servicesList")}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border'>
                                <thead>
                                    <tr>													   												   
                                        <th>{$this->translator->_("report.printProducts.service")}</th>
                ";
                $printTitle = $this->translator->_("report.printProducts.servicesList");
            }
            if ($target == 3) { //products in stock
                $documentHtml .= "
                                    <tr>
                                        <td class='title'>{$this->translator->_("report.printProducts.productsInStockList")}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border'>
                                <thead>
                                    <tr>													   												   
                                        <th>{$this->translator->_("report.printProducts.product")}</th>
                ";
                $printTitle = $this->translator->_("report.printProducts.productsInStockList");
            }
            $documentHtml .= "
                                        <th>{$this->translator->_("report.printProducts.buyingPrice")}</th>
                                        <th>{$this->translator->_("report.printProducts.salePrice")}</th>
                                        <th>{$this->translator->_("report.printProducts.margin")}</th>                                               
                                    </tr>
                                </thead>
                                <tbody>
            ";
            $atLeastOneProduct = false;
            if ($target == 0 || $target == 1 || $target == 2) {
                $productCategoriesList = $productCategoryDao->getAllProductCategories();
                foreach ($productCategoriesList as $productCategory) {
                    $productList = $target == 0 ?
                        $productDao->getProductsFiltered(null, $productCategory) : ($target == 1 ?
                            $productDao->getProductsFiltered(null, $productCategory, 1) : $productDao->getProductsFiltered(null, $productCategory, 0));
                    if (count($productList) > 0) {
                        $documentHtml .= "
                                    <tr>
                                        <td colspan='4' style='font-weight: bold;'>
                                            {$productCategory->getNom()}
                                        </td>
                                    </tr>
                        ";
                        foreach ($productList as $product) {
                            $atLeastOneProduct = true;
                            $buyingPrice = number_format($product->getPrixAchat(), 0, '', ' ');
                            $salePrice = number_format($product->getPrixVente(), 0, '', ' ');
                            $margin = number_format($product->getMarge(), 0, '', ' ');
                            $documentHtml .= "
                                    <tr>
                                        <td style='text-align:left;'>{$product->getDesignation()}</td>
                                        <td style='text-align:right;'>{$buyingPrice}</td>
                                        <td style='text-align:right;'>{$salePrice}</td>
                                        <td style='text-align:right;'>{$margin}</td>
                                    </tr>
                            ";
                        }
                    }
                }
            } else {
                $productCategoriesInStockIdsList = $stockDao->getAllCategoriesAvailableInStockByStore();
                foreach ($productCategoriesInStockIdsList as $productCategoryId) {
                    $productCategory = $productCategoryDao->getProductCategoryById($productCategoryId["idCategorie"]);
                    $productIdsList = $stockDao->getDistinctProductInStockByCategory($productCategory);
                    $documentHtml .= "
                                    <tr>
                                        <td colspan='4' style='font-weight: bold;'>
                                            {$productCategory->getNom()}
                                        </td>
                                    </tr>
                    ";
                    foreach ($productIdsList as $productId) {
                        $atLeastOneProduct = true;
                        $product = $productDao->getProductById($productId["idProduit"]);
                        $buyingPrice = number_format($product->getPrixAchat(), 0, '', ' ');
                        $salePrice = number_format($product->getPrixVente(), 0, '', ' ');
                        $margin = number_format($product->getMarge(), 0, '', ' ');
                        $documentHtml .= "
                                    <tr>
                                        <td style='text-align:left;'>{$product->getDesignation()}</td>
                                        <td style='text-align:right;'>{$buyingPrice}</td>
                                        <td style='text-align:right;'>{$salePrice}</td>
                                        <td style='text-align:right;'>{$margin}</td>
                                    </tr>
                        ";
                    }
                }
            }
            if (!$atLeastOneProduct) {
                $documentHtml .= "
                                    <tr>
                                        <td colspan='4'>
                                            {$this->translator->_("shared.noDataAvailable")}
                                        </td>
                                    </tr>
                ";
            }
            $documentHtml .= "
                                </tbody>
                            </table>
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "product_list_{$_SESSION[SESSION_USER_ID]}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/products/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/products/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage() . $ex->getLine() . $ex->getFile());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printstocksAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/stocks") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $storeId = $_POST["store"] != "" ? (int)$_POST["store"] : null;
            $from = $_POST["from"] != "" ? new DateTime("{$_POST["from"]} 00:00:00") : null;
            $to = $_POST["to"] != "" ? new DateTime("{$_POST["to"]} 23:59:59") : null;
            $storeDao = new StoreDAO();
            $stockDao = new StockDAO();
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $secondaryBrand = APP_SECONDARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = $this->translator->_("report.printStocks.stocksList");
            $printFilters = "";
            $considerExpirationDate = GESTION_DATE_PEREMPTION_PRODUIT == 1;
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: left;  
                                color: #000000; 
                              
                            }

                            td.center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            td.not-perishable {
                                color: {$secondaryBrand} !important;
                                font-weight: bold;
                                font-size: 10px;
                            }

                            bold {
                                color: black;
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }

                            i.violet-label {
                                background-color: #6435c9;
                                color: white;
                                padding: 2px;
                                border-radius: 6px;
                                margin: 3px;
                            }

                            i.secondary-label {
                                background-color: {$secondaryBrand};
                                color: white;
                                padding: 2px;
                                border-radius: 6px;
                                margin: 3px;
                            }

                            td.left.aligned {
                                text-align: left;
                            }

                            td.right.aligned {
                                text-align: right;
                            }

                            tr.app-total-tr {
                                background-color: {$primaryBrand}80;
                            }

                            tr.app-total-tr > td {
                                color: {$thirdBrand};
                            }

                            tr.app-highlight-tr {
                                background-color: {$secondaryBrand}80;
                            }

                            tr.app-highlight-tr > td {
                                color: {$thirdBrand};
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$this->translator->_("report.printStocks.stocksList")}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
            ";
            $store = $storeId != null ?  $storeDao->getStoreById($storeId) : null;
            $storeName = $store != null ? $store->getNomMagasin() : $this->translator->_("report.printStocks.allStores");
            $documentHtml .= "
                            <table>
                                <thead>
                                    <tr>
                                        <th class='center-text'>{$this->translator->_("report.printStocks.store")} : <bold class='black-bold'>{$storeName}</bold></th>
                            
            ";
            $printFilters .= "{$this->translator->_("report.printStocks.store")} : {$storeName}";
            if ($considerExpirationDate) {
                if ($from != null) {
                    $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printStocks.expirationDateAfter")} : <bold class='black-bold'>{$from->format(DATE_FORMAT)}</bold></th>
                    ";
                    $printFilters .= ", {$this->translator->_("report.printStocks.expirationDateAfter")} : {$from->format(DATE_FORMAT)}";
                }
                if ($to != null) {
                    $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printStocks.expirationDateBefore")} : <bold class='black-bold'>{$from->format(DATE_FORMAT)}</bold></th>
                    ";
                    $printFilters .= ", {$this->translator->_("report.printStocks.expirationDateBefore")} : {$from->format(DATE_FORMAT)}";
                }
            }
            $documentHtml .= "
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border'>
                                <thead>
                                    <tr>
                                        <th style='width: 30%;'>{$this->translator->_("report.printStocks.designation")}</th>
            ";
            $documentHtml .= "
                                        <th>{$this->translator->_("report.printStocks.batchNumber")}</th>
                                        <th>{$this->translator->_("report.printStocks.qty")}</th>
            ";
            if ($considerExpirationDate) {
                $documentHtml .= "
                                        <th>{$this->translator->_("report.printStocks.expirationDate")}</th>
                ";
            }
            $documentHtml .= "
                                        <th>{$this->translator->_("report.printStocks.initialQty")}</th>
                                        <th>{$this->translator->_("report.printStocks.buyingPrice")}</th>
                                        <th>{$this->translator->_("report.printStocks.salePrice")}</th>
                                        <th style='width: 10%;'>{$this->translator->_("report.printStocks.buyingTotal")}</th>
                                        <th style='width: 10%;'>{$this->translator->_("report.printStocks.saleTotal")}</th>
                                        <th style='width: 10%;'>{$this->translator->_("report.printStocks.profit")}</th>
                                    </tr>
                                </thead>
                                <tbody>
            ";
            $stocksList = $stockDao->getAvailableStockByStoreAndExpirationDateInterval($store, $from, $to);
            $effectiveColspan = 9;
            if ($considerExpirationDate) $effectiveColspan++;
            $totalsColspan = 2;
            $currentStoreId = null;
            $currentStoreName = null;
            $currentCategoryId = null;
            $currentCategoryName = null;
            $currentStoreTotalQuantity = 0;
            $currentStoreTotalInitialQuantity = 0;
            $currentStoreTotalBuying = 0;
            $currentStoreTotalSale = 0;
            $currentStoreTotalProfit = 0;
            $currentCategoryTotalQuantity = 0;
            $currentCategoryTotalInitialQuantity = 0;
            $currentCategoryTotalBuying = 0;
            $currentCategoryTotalSale = 0;
            $currentCategoryTotalProfit = 0;
            $totalQuantity = 0;
            $totalInitialQuantity = 0;
            $totalBuying = 0;
            $totalSale = 0;
            $totalProfit = 0;
            foreach ($stocksList as $stock) {
                if ($stock->getIdMagasin()->getIdMagasin() != $currentStoreId) {
                    if ($currentStoreId != null) {
                        if ($currentCategoryId != null) {
                            $categoryTotalBuyingSum = number_format($currentCategoryTotalBuying, 0, '', ' ');
                            $categoryTotalSaleSum = number_format($currentCategoryTotalSale, 0, '', ' ');
                            $categoryTotalProfitSum = number_format($currentCategoryTotalProfit, 0, '', ' ');
                            $documentHtml .= "
                                    <tr class='app-total-tr'>
                                        <td colspan='{$totalsColspan}' class='right aligned'>
                                            {$this->translator->_("report.printStocks.totals")}
                                            {$currentCategoryName}
                                        </td>
                                        <td>{$currentCategoryTotalQuantity}</td>
                            ";
                            if ($considerExpirationDate) {
                                $documentHtml .= "
                                        <td></td>
                                ";
                            }
                            $documentHtml .= "
                                        <td>{$currentCategoryTotalInitialQuantity}</td>
                                        <td colspan='2'></td>
                                        <td class='right aligned'>{$categoryTotalBuyingSum}</td>
                                        <td class='right aligned'>{$categoryTotalSaleSum}</td>
                                        <td class='right aligned'>{$categoryTotalProfitSum}</td>
                                    </tr>
                            ";
                        }
                        $storeTotalBuyingSum = number_format($currentStoreTotalBuying, 0, '', ' ');
                        $storeTotalSaleSum = number_format($currentStoreTotalSale, 0, '', ' ');
                        $storeTotalProfitSum = number_format($currentStoreTotalProfit, 0, '', ' ');
                        $documentHtml .= "
                                    <tr class='app-total-tr'>
                                        <td colspan='{$totalsColspan}' class='right aligned'>
                                            <i class='violet-label'>
                                                {$this->translator->_("report.printStocks.totals")}
                                                {$currentStoreName}
                                            </i>
                                        </td>
                                        <td>{$currentStoreTotalQuantity}</td>
                        ";
                        if ($considerExpirationDate) {
                            $documentHtml .= "
                                        <td></td>
                            ";
                        }
                        $documentHtml .= "
                                        <td>{$currentStoreTotalInitialQuantity}</td>
                                        <td colspan='2'></td>
                                        <td class='right aligned'>{$storeTotalBuyingSum}</td>
                                        <td class='right aligned'>{$storeTotalSaleSum}</td>
                                        <td class='right aligned'>{$storeTotalProfitSum}</td>
                                    </tr>
                        ";
                    }
                    if ($store == null) {
                        $documentHtml .= "
                                    <tr>
                                        <td colspan='{$effectiveColspan}' class='center-text'>
                                            <i class='violet-label'>
                                                {$stock->getIdMagasin()->getNomMagasin()}
                                            </i>
                                        </td>
                                    </tr>
                        ";
                    }
                    $currentStoreId = $stock->getIdMagasin()->getIdMagasin();
                    $currentStoreName = $stock->getIdMagasin()->getNomMagasin();
                    $currentStoreTotalQuantity = $stock->getQuantite();
                    $currentStoreTotalInitialQuantity = $stock->getQuantiteEntree();
                    $currentStoreTotalBuying = $stock->getQuantite() * $stock->getPrixAchat();
                    $currentStoreTotalSale = ($stock->getQuantiteEntree() - $stock->getQuantite()) * $stock->getPrixVente();
                    $currentStoreTotalProfit =  ($stock->getQuantiteEntree() - $stock->getQuantite()) * ($stock->getPrixVente() - $stock->getPrixAchat());
                    $currentCategoryId = null;
                    $currentCategoryName = null;
                    $currentCategoryTotalQuantity = 0;
                    $currentCategoryTotalInitialQuantity = 0;
                    $currentCategoryTotalBuying = 0;
                    $currentCategoryTotalSale = 0;
                    $currentCategoryTotalProfit = 0;
                } else {
                    $currentStoreTotalQuantity += $stock->getQuantite();
                    $currentStoreTotalInitialQuantity += $stock->getQuantiteEntree();
                    $currentStoreTotalBuying += $stock->getQuantite() * $stock->getPrixAchat();
                    $currentStoreTotalSale += ($stock->getQuantiteEntree() - $stock->getQuantite()) * $stock->getPrixVente();
                    $currentStoreTotalProfit +=  ($stock->getQuantiteEntree() - $stock->getQuantite()) * ($stock->getPrixVente() - $stock->getPrixAchat());
                }
                if ($stock->getIdProduit()->getIdCategorieProduit()->getIdCategorie() != $currentCategoryId) {
                    if ($currentCategoryId != null) {
                        $categoryTotalBuyingSum = number_format($currentCategoryTotalBuying, 0, '', ' ');
                        $categoryTotalSaleSum = number_format($currentCategoryTotalSale, 0, '', ' ');
                        $categoryTotalProfitSum = number_format($currentCategoryTotalProfit, 0, '', ' ');
                        $documentHtml .= "
                                    <tr class='app-total-tr'>
                                        <td colspan='{$totalsColspan}' class='right aligned'>
                                            {$this->translator->_("report.printStocks.totals")}
                                            {$currentCategoryName}
                                        </td>
                                        <td>{$currentCategoryTotalQuantity}</td>
                        ";
                        if ($considerExpirationDate) {
                            $documentHtml .= "
                                        <td></td>
                            ";
                        }
                        $documentHtml .= "
                                        <td>{$currentCategoryTotalInitialQuantity}</td>
                                        <td colspan='2'></td>
                                        <td class='right aligned'>{$categoryTotalBuyingSum}</td>
                                        <td class='right aligned'>{$categoryTotalSaleSum}</td>
                                        <td class='right aligned'>{$categoryTotalProfitSum}</td>
                                    </tr>
                        ";
                    }
                    $documentHtml .= "
                                    <tr class='app-highlight-tr'>
                                        <td colspan='{$effectiveColspan}' class='left aligned'>
                                            {$stock->getIdProduit()->getIdCategorieProduit()->getNom()}
                                        </td>
                                    </tr>
                    ";
                    $currentCategoryId = $stock->getIdProduit()->getIdCategorieProduit()->getIdCategorie();
                    $currentCategoryName = $stock->getIdProduit()->getIdCategorieProduit()->getNom();
                    $currentCategoryTotalQuantity = $stock->getQuantite();
                    $currentCategoryTotalInitialQuantity = $stock->getQuantiteEntree();
                    $currentCategoryTotalBuying = $stock->getQuantite() * $stock->getPrixAchat();
                    $currentCategoryTotalSale = ($stock->getQuantiteEntree() - $stock->getQuantite()) * $stock->getPrixVente();
                    $currentCategoryTotalProfit = ($stock->getQuantiteEntree() - $stock->getQuantite()) * ($stock->getPrixVente() - $stock->getPrixAchat());
                } else {
                    $currentCategoryTotalQuantity += $stock->getQuantite();
                    $currentCategoryTotalInitialQuantity += $stock->getQuantiteEntree();
                    $currentCategoryTotalBuying += $stock->getQuantite() * $stock->getPrixAchat();
                    $currentCategoryTotalSale += ($stock->getQuantiteEntree() - $stock->getQuantite()) * $stock->getPrixVente();
                    $currentCategoryTotalProfit += ($stock->getQuantiteEntree() - $stock->getQuantite()) * ($stock->getPrixVente() - $stock->getPrixAchat());
                }
                $totalQuantity += $stock->getQuantite();
                $totalInitialQuantity += $stock->getQuantiteEntree();
                $totalBuying += $stock->getQuantite() * $stock->getPrixAchat();
                $totalSale += ($stock->getQuantiteEntree() - $stock->getQuantite()) * $stock->getPrixVente();
                $totalProfit += ($stock->getQuantiteEntree() - $stock->getQuantite()) * ($stock->getPrixVente() - $stock->getPrixAchat());
                $documentHtml .= "
                                    <tr>
                                        <td>{$stock->getDesignation()}</td>
                                        <td>{$stock->getNumLot()}</td>
                                        <td class=''>{$stock->getQuantite()}</td>
                ";
                if ($considerExpirationDate) {
                    $stockExpirationDate = $stock->getDatePeremption() != null ? $stock->getDatePeremption()->format(DATE_FORMAT) : "";
                    if ($stock->getIdProduit()->getPerissable() == 1 && $stock->getDatePeremption() != null) {
                        $documentHtml .= "
                                        <td>{$stockExpirationDate}</td>
                        ";
                    } else {
                        $documentHtml .= "
                                        <td class='not-perishable'>{$this->translator->_("shared.notPerishable")}</td>
                        ";
                    }
                }
                $stockBuyingPrice = number_format($stock->getPrixAchat(), 0, '', ' ');
                $stockSalePrice = number_format($stock->getPrixVente(), 0, '', ' ');
                $stockTotalBuyingPrice = number_format($stock->getQuantite() * $stock->getPrixAchat(), 0, '', ' ');
                $stockTotalSalePrice = number_format(($stock->getQuantiteEntree() - $stock->getQuantite()) * $stock->getPrixVente(), 0, '', ' ');
                $stockTotalProfitPrice = number_format(($stock->getQuantiteEntree() - $stock->getQuantite()) * ($stock->getPrixVente() - $stock->getPrixAchat()), 0, '', ' ');
                $documentHtml .= "
                                        <td>{$stock->getQuantiteEntree()}</td>
                                        <td class='right aligned'>{$stockBuyingPrice}</td>
                                        <td class='right aligned'>{$stockSalePrice}</td>
                                        <td class='right aligned'>{$stockTotalBuyingPrice}</td>
                                        <td class='right aligned'>{$stockTotalSalePrice}</td>
                                        <td class='right aligned'>{$stockTotalProfitPrice}</td>
                                    </tr>
                ";
            }
            if ($currentStoreId != null) {
                if ($currentCategoryId != null) {
                    $categoryTotalBuyingSum = number_format($currentCategoryTotalBuying, 0, '', ' ');
                    $categoryTotalSaleSum = number_format($currentCategoryTotalSale, 0, '', ' ');
                    $categoryTotalProfitSum = number_format($currentCategoryTotalProfit, 0, '', ' ');
                    $documentHtml .= "
                                    <tr class='app-total-tr'>
                                        <td colspan='{$totalsColspan}' class='right aligned'>
                                            {$this->translator->_("report.printStocks.totals")}
                                            {$currentCategoryName}
                                        </td>
                                        <td>{$currentCategoryTotalQuantity}</td>
                    ";
                    if ($considerExpirationDate) {
                        $documentHtml .= "
                                        <td></td>
                        ";
                    }
                    $documentHtml .= "
                                        <td>{$currentCategoryTotalInitialQuantity}</td>
                                        <td colspan='2'></td>
                                        <td class='right aligned'>{$categoryTotalBuyingSum}</td>
                                        <td class='right aligned'>{$categoryTotalSaleSum}</td>
                                        <td class='right aligned'>{$categoryTotalProfitSum}</td>
                                    </tr>
                    ";
                }
                $storeTotalBuyingSum = number_format($currentStoreTotalBuying, 0, '', ' ');
                $storeTotalSaleSum = number_format($currentStoreTotalSale, 0, '', ' ');
                $storeTotalProfitSum = number_format($currentStoreTotalProfit, 0, '', ' ');
                $documentHtml .= "
                                    <tr class='app-total-tr'>
                                        <td colspan='{$totalsColspan}' class='right aligned'>
                                            <i class='violet-label'>
                                                {$this->translator->_("report.printStocks.totals")}
                                                {$currentStoreName}
                                            </i>
                                        </td>
                                        <td>{$currentStoreTotalQuantity}</td>
                ";
                if ($considerExpirationDate) {
                    $documentHtml .= "
                                        <td></td>
                    ";
                }
                $documentHtml .= "
                                        <td>{$currentStoreTotalInitialQuantity}</td>
                                        <td colspan='2'></td>
                                        <td class='right aligned'>{$storeTotalBuyingSum}</td>
                                        <td class='right aligned'>{$storeTotalSaleSum}</td>
                                        <td class='right aligned'>{$storeTotalProfitSum}</td>
                                    </tr>
                ";
            }
            if ($store == null) {
                $totalBuyingSum = number_format($totalBuying, 0, '', ' ');
                $totalSaleSum = number_format($totalSale, 0, '', ' ');
                $totalProfitSum = number_format($totalProfit, 0, '', ' ');
                $documentHtml .= "
                                    <tr class='app-total-tr'>
                                        <td colspan='$totalsColspan' class='right aligned'>
                                            <i class='secondary-label'>
                                                {$this->translator->_("report.printStocks.totals")}
                                            </i>
                                        </td>
                                        <td>{$totalQuantity}</td>
                ";
                if ($considerExpirationDate) {
                    $documentHtml .= "
                                        <td></td>
                    ";
                }
                $documentHtml .= "
                                        <td>{$totalInitialQuantity}</td>
                                        <td colspan='2'></td>
                                        <td class='right aligned'>{$totalBuyingSum}</td>
                                        <td class='right aligned'>{$totalSaleSum}</td>
                                        <td class='right aligned'>{$totalProfitSum}</td>
                                    </tr>
                ";
            }
            if (count($stocksList) == 0) {
                $documentHtml .= "
                                    <tr>
                                        <td colspan='{$effectiveColspan}'>
                                            {$this->translator->_("shared.noDataAvailable")}
                                        </td>
                                    </tr>
                ";
            }
            $documentHtml .= "
                                </tbody>
                            </table>
                        </div>
                    </body>
                </html> 
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "stock_list_{$_SESSION[SESSION_USER_ID]}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/stocks/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/stocks/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printmovementsAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/interstores") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $originStoreId = $_POST["originStore"] != "" ? (int)$_POST["originStore"] : null;
            $receivingStoreId = $_POST["receivingStore"] != "" ? (int)$_POST["receivingStore"] : null;
            $from = $_POST["from"] != "" ? new DateTime("{$_POST["from"]} 00:00:00") : null;
            $to = $_POST["to"] != "" ? new DateTime("{$_POST["to"]} 23:59:59") : null;
            $storeMovementDao = new StoreMovementDAO();
            $storeDao = new StoreDAO();
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $secondaryBrand = APP_SECONDARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = $this->translator->_("report.printMovements.interStoreMovementsList");
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: center;  
                                color: #000000; 
                              
                            }

                            .center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            td.validated {
                                color: {$primaryBrand} !important;
                                font-weight: bold;
                            }

                            td.not-validated {
                                color: {$secondaryBrand} !important;
                                font-weight: bold;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            bold {
                                color: black;
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$this->translator->_("report.printMovements.interStoreMovementsList")}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table>
                                <thead>
                                    <tr>
            ";
            $originStore = $originStoreId != null ? $storeDao->getStoreById($originStoreId) : null;
            $receivingStore = $receivingStoreId != null ? $storeDao->getStoreById($receivingStoreId) : null;
            $originStoreName = $originStore != null ? $originStore->getNomMagasin() : $this->translator->_("report.printMovements.allStores");
            $receivingStoreName = $receivingStore != null ? $receivingStore->getNomMagasin() : $this->translator->_("report.printMovements.allStores");
            $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printMovements.originStore")} : <bold class='black-bold'>{$originStoreName}</bold></th>
                                        <th class='center-text'>{$this->translator->_("report.printMovements.receivingStore")} : <bold class='black-bold'>{$receivingStoreName}</bold></th>
            ";
            $printFilters .= "{$this->translator->_("report.printMovements.originStore")} : {$originStoreName}";
            $printFilters .= ", {$this->translator->_("report.printMovements.receivingStore")} : {$receivingStoreName}";
            if ($from != null) {
                $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printMovements.dateAfter")} : <bold class='black-bold'>{$from->format(DATE_FORMAT)}</bold></th>
                ";
                $printFilters .= ", {$this->translator->_("report.printMovements.dateAfter")} : {$from->format(DATE_FORMAT)}";
            }
            if ($to != null) {
                $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printMovements.dateBefore")} : <bold class='black-bold'>{$to->format(DATE_FORMAT)}</bold></th>
                ";
                $printFilters .= ", {$this->translator->_("report.printMovements.dateBefore")} : {$to->format(DATE_FORMAT)}";
            }
            $documentHtml .= "
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border'>
                                <thead>
                                    <tr>
                                        <th>{$this->translator->_("report.printMovements.number")}</th>
                                        <th>{$this->translator->_("report.printMovements.entryDate")}</th>
                                        <th>{$this->translator->_("report.printMovements.movementDate")}</th>
                                        <th>{$this->translator->_("report.printMovements.status")}</th>
                                    </tr>
                                </thead>
                                <tbody>
            ";
            $storeMovementList = $storeMovementDao->getStoreMovementList($originStore, $receivingStore, $from, $to);
            $currentOriginStoreId = null;
            $currentReceivingStoreId = null;
            foreach ($storeMovementList as $storeMovement) {
                if (($originStore == null || $receivingStore == null) && ($storeMovement->getMagasinEmetteur()->getIdMagasin() != $currentOriginStoreId || $storeMovement->getMagasinRecepteur()->getIdMagasin() != $currentReceivingStoreId)) {
                    $effectiveColspan = 4;
                    if ($originStore == null && $receivingStore == null) $effectiveColspan /= 2;
                    $documentHtml .= "
                                    <tr>
                    ";
                    if ($originStore == null) {
                        $documentHtml .= "
                                        <td colspan='{$effectiveColspan}' class='black-bold'>
                                            {$this->translator->_("report.printMovements.originStore")} : {$storeMovement->getMagasinEmetteur()->getNomMagasin()}
                                        </td>
                        ";
                    }
                    if ($receivingStore == null) {
                        $documentHtml .= "
                                        <td colspan='{$effectiveColspan}' class='black-bold'>
                                            {$this->translator->_("report.printMovements.receivingStore")} : {$storeMovement->getMagasinRecepteur()->getNomMagasin()}
                                        </td>
                        ";
                    }
                    $documentHtml .= "
                                    </tr>
                    ";
                    $currentOriginStoreId = $storeMovement->getMagasinEmetteur()->getIdMagasin();
                    $currentReceivingStoreId = $storeMovement->getMagasinRecepteur()->getIdMagasin();
                }
                $storeMovementStatus = $this->translator->_($storeMovement->getValide() == 1 ? "report.printMovements.validated" : "report.printMovements.notValidated");
                $storeMovementStatusClassName = $storeMovement->getValide() == 1 ? 'validated' : 'not-validated';
                $documentHtml .= "
                                    <tr>
                                        <td>{$storeMovement->getIdMvtMagasin()}</td>
                                        <td>{$storeMovement->getDateSaisie()->format(DATE_FORMAT_WITH_TIME)}</td>
                                        <td>{$storeMovement->getDateMvt()->format(DATE_FORMAT)}</td>
                                        <td class='{$storeMovementStatusClassName}'>{$storeMovementStatus}</td>
                                    </tr>
                ";
            }
            if (count($storeMovementList) == 0) {
                $effectiveColspan = 4;
                $documentHtml .= "
                                    <tr>
                                        <td colspan='{$effectiveColspan}'>
                                            {$this->translator->_("shared.noDataAvailable")}
                                        </td>
                                    </tr>
                ";
            }
            $documentHtml .= "
                                </tbody>
                            </table>
                        </div>
                    </body>
                </html> 
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "inter_store_list_{$_SESSION[SESSION_USER_ID]}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/interstores/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/interstores/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printmovementAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/interstores") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $id = (int)$_POST["id"];
            $storeMovementDao = new StoreMovementDAO();
            $storeMovementDetailDao = new StoreMovementDetailDAO();
            $storeMovement = $storeMovementDao->getStoreMovementById($id);
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = "{$this->translator->_("report.printMovement.interStoreSheetNumber")} : {$id}";
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: center;  
                                color: #000000; 
                            }

                            .center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            .criteria {
                                font-size: 14px;
                                font-weight: bold;
                                color: black;
                                text-align: center;
                            }

                            bold {
                                color: black;
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$this->translator->_("report.printMovement.interStoreSheetNumber")} : {$id}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table>
                                <thead>
                                    <tr>
                                        <td class='criteria'>
                                            {$this->translator->_("report.printMovement.originStore")} : {$storeMovement->getMagasinEmetteur()->getNomMagasin()}
                                        </td>
                                        <td class='criteria'>
                                            {$this->translator->_("report.printMovement.receivingStore")} : {$storeMovement->getMagasinRecepteur()->getNomMagasin()}
                                        </td>
                                        <td class='criteria'>
                                            {$this->translator->_("report.printMovement.date")} : {$storeMovement->getDateMvt()->format(DATE_FORMAT)}
                                        </td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border'>
                                <thead>
                                    <tr>
                                        <th style='width: 40%;'>{$this->translator->_("report.printMovement.designation")}</th>
                                        <th>{$this->translator->_("report.printMovement.batchNumber")}</th>
                                        <th>{$this->translator->_("report.printMovement.sellingPrice")}</th>
                                        <th>{$this->translator->_("report.printMovement.quantity")}</th>
                                        <th style='width: 15%;'>{$this->translator->_("report.printMovement.total")}</th>
                                    </tr>
                                </thead>
                                <tbody>
            ";
            $printFilters .= "{$this->translator->_("report.printMovement.originStore")} : {$storeMovement->getMagasinEmetteur()->getNomMagasin()}";
            $printFilters .= ", {$this->translator->_("report.printMovement.receivingStore")} : {$storeMovement->getMagasinRecepteur()->getNomMagasin()}";
            $printFilters .= ", {$this->translator->_("report.printMovement.date")} : {$storeMovement->getDateMvt()->format(DATE_FORMAT)}";
            $storeMovementDetails = $storeMovementDetailDao->getDetailOfStoreMovement($storeMovement);
            $storeMovementTotal = 0;
            foreach ($storeMovementDetails as $storeMovementDetail) {
                $sellingPrice = number_format($storeMovementDetail->getIdStock()->getPrixVente(), 0, '', ' ');
                $batchNumber = $storeMovementDetail->getIdStock()->getNumLot();
                $designation = $storeMovementDetail->getIdStock()->getDesignation();
                $quantity = $storeMovementDetail->getQte();
                $total = number_format($quantity * $storeMovementDetail->getIdStock()->getPrixVente(), 0, '', ' ');
                $storeMovementTotal += $quantity * $storeMovementDetail->getIdStock()->getPrixVente();
                $documentHtml .= "
                                    <tr>
                                        <td style='text-align: left;'>{$designation}</td>
                                        <td>{$batchNumber}</td>
                                        <td style='text-align: right;'>{$sellingPrice}</td>
                                        <td>{$quantity}</td>
                                        <td style='text-align: right;'>{$total}</td>
                                    </tr>
                ";
            }
            $storeMovementTotalFormat = number_format($storeMovementTotal, 0, '', ' ');
            $documentHtml .= "      <tr>
                                        <td colspan='4' style='text-align:right; font-weight: bold'>{$this->translator->_("report.printMovement.total")}</td>
                                        <td style='text-align:right;'>{$storeMovementTotalFormat}</td>
                                    </tr>
                                </tbody>
                            </table>
                            <div style='width: 100%; height: 50px;'></div>
                            <table class='equalized'>
                                <tr>
                                    <th style='text-decoration: underline; text-align: left;' class='black-bold'>
                                        {$this->translator->_("report.printMovement.storekeeperSignature")}
                                    </th>
                                    <th style='text-decoration: underline; text-align: right;' class='black-bold'>
                                        {$this->translator->_("report.printMovement.carrierSignature")}
                                    </th>
                                </tr>
                            </table>
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "inter_store_sheet_{$_SESSION[SESSION_USER_ID]}_{$id}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/interstores/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/interstores/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printcumulativestocksAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/stocks") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $storeId = $_POST["store"] != "" ? (int)$_POST["store"] : null;
            $stockAtTheDateOf = $_POST["stockAtTheDateOf"];
            $name = $_POST["name"] != "" ? $_POST["name"] : null;
            $storeDao = new StoreDAO();
            $stockDao = new StockDAO();
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $secondaryBrand = APP_SECONDARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = $this->translator->_("report.printCumulativeStocks.cumulativeStocksList");
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }
                            
                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: left;  
                                color: #000000; 
                              
                            }

                            td.center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            bold {
                                color: black;
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }

                            i.violet-label {
                                background-color: #6435c9;
                                color: white;
                                padding: 2px;
                                border-radius: 6px;
                                margin: 3px;
                            }

                            i.secondary-label {
                                background-color: {$secondaryBrand};
                                color: white;
                                padding: 2px;
                                border-radius: 6px;
                                margin: 3px;
                            }

                            td.left.aligned {
                                text-align: left;
                            }

                            td.right.aligned {
                                text-align: right;
                            }

                            tr.app-total-tr {
                                background-color: {$primaryBrand}80;
                            }

                            tr.app-total-tr > td {
                                color: {$thirdBrand};
                            }

                            tr.app-highlight-tr {
                                background-color: {$secondaryBrand}80;
                            }

                            tr.app-highlight-tr > td {
                                color: {$thirdBrand};
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$this->translator->_("report.printCumulativeStocks.cumulativeStocksList")}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table>
                                <thead>
                                    <tr>
            ";
            $store = $storeId != null ? $storeDao->getStoreById($storeId) : null;
            $storeName = $store != null ? $store->getNomMagasin() : $this->translator->_("report.printCumulativeStocks.allStores");
            $stockAtTheDateOfDateTime = new DateTime($stockAtTheDateOf);
            $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printCumulativeStocks.store")} : <bold class='black-bold'>{$storeName}</bold></th>
                                        <th class='center-text'>{$this->translator->_("report.printCumulativeStocks.date")} : <bold class='black-bold'>{$stockAtTheDateOfDateTime->format(DATE_FORMAT)}</bold></th>
            ";
            $printFilters .= "{$this->translator->_("report.printCumulativeStocks.store")} : {$storeName}";
            $printFilters .= ", {$this->translator->_("report.printCumulativeStocks.date")} : {$stockAtTheDateOfDateTime->format(DATE_FORMAT)}";
            if ($name != null) {
                $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printCumulativeStocks.productNameContaining")} : <bold class='black-bold'>{$name}</bold></th>
                ";
                $printFilters .= ", {$this->translator->_("report.printCumulativeStocks.productNameContaining")} : {$name}";
            }
            $documentHtml .= "
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border'>
                                <thead>
                                    <tr>
                                        <th>{$this->translator->_("report.printCumulativeStocks.designation")}</th>
                                        <th>{$this->translator->_("report.printCumulativeStocks.quantity")}</th>
                                        <th>{$this->translator->_("report.printCumulativeStocks.buyingPrice")}</th>
                                        <th>{$this->translator->_("report.printCumulativeStocks.salePrice")}</th>
                                        <th>{$this->translator->_("report.printCumulativeStocks.buyingTotal")}</th>
                                    </tr>
                                </thead>
                                <tbody>
            ";
            $whereStore = "";
            $whereName = "";
            if ($storeId != null) {
                $whereStore .= " AND m.id_magasin = {$storeId}";
            }
            if ($name != null) {
                $whereName .= " AND p.designation LIKE '%{$name}%'";
            }
            $where = "summary.quantity > 0 AND p.statut = 1 AND m.statut = 1{$whereStore}{$whereName}";
            $cumulativeStocksTable = "
                SELECT summary.cumulative_stock_id AS cumulative_stock_id, summary.product_id AS product_id, summary.store_id AS store_id,
                summary.quantity AS quantity, p.designation AS product_name, m.nom_magasin AS store_name, p.prix_achat AS buying_price, 
                p.prix_vente AS sale_price, p.quantite_alerte AS alert_quantity, c.id_categorie AS category_id, c.nom AS category_name
                FROM
                (
                    SELECT entries.cumulative_stock_id AS cumulative_stock_id, entries.product_id AS product_id, entries.store_id AS store_id,
                    (entries.entry_quantity - ifnull(exits.exit_quantity, 0)) AS quantity
                    FROM
                    (
                        SELECT CONCAT(p.id_produit, '_', m.id_magasin) AS cumulative_stock_id, p.id_produit AS product_id, SUM(lr.quantite) AS entry_quantity, m.id_magasin AS store_id
                        FROM ligne_reception lr JOIN produit p ON lr.id_produit = p.id_produit JOIN reception r ON r.id_reception = lr.id_reception
                        JOIN magasin m ON r.id_magasin = m.id_magasin
                        WHERE p.statut = 1 AND r.statut = 1 AND r.valider = 1 AND r.date_saisie <= '{$stockAtTheDateOf} 23:59:59'
                        GROUP BY p.id_produit, m.id_magasin
                    ) AS entries
                    LEFT OUTER JOIN
                    (
                        SELECT CONCAT(p.id_produit, '_', m.id_magasin) AS cumulative_stock_id, p.id_produit AS product_id, SUM(ls.quantite) AS exit_quantity, m.id_magasin AS store_id
                        FROM ligne_sortie ls JOIN stock st ON ls.id_stock = st.id_stock JOIN produit p ON st.id_produit = p.id_produit 
                        JOIN sortie so ON ls.id_sortie = so.id_sortie JOIN magasin m ON so.id_magasin = m.id_magasin
                        WHERE p.statut = 1 AND so.statut = 1 AND so.valider = 1 AND so.date_saisie <= '{$stockAtTheDateOf} 23:59:59'
                        GROUP BY p.id_produit, m.id_magasin
                    ) AS exits
                    ON (entries.cumulative_stock_id = exits.cumulative_stock_id OR exits.cumulative_stock_id IS NULL) 
                    AND (entries.product_id = exits.product_id OR exits.product_id IS NULL)
                    AND (entries.store_id = exits.store_id OR exits.store_id IS NULL)
                ) AS summary
                JOIN produit p ON summary.product_id = p.id_produit JOIN categorie_produit c ON p.id_categorie_produit = c.id_categorie
                JOIN magasin m ON summary.store_id = m.id_magasin
                WHERE {$where}
                ORDER BY store_id ASC, category_id ASC, product_name ASC
            ";
            $cumulativeStocksListStatement = $stockDao->entityManager->getConnection()->prepare($cumulativeStocksTable);
            $cumulativeStocksListStatement->execute();
            $cumulativeStocksList = $cumulativeStocksListStatement->fetchAll();
            $effectiveColspan = 5;
            $currentStoreId = null;
            $currentStoreName = null;
            $currentStoreTotalQuantity = 0;
            $currentStoreTotalBuying = 0;
            $currentCategoryId = null;
            $currentCategoryName = null;
            $currentCategoryTotalQuantity = 0;
            $currentCategoryTotalBuying = 0;
            $totalQuantity = 0;
            $totalBuying = 0;
            foreach ($cumulativeStocksList as $cumulativeStock) {
                if ((int)$cumulativeStock["store_id"] != $currentStoreId) {
                    if ($currentStoreId != null) {
                        if ($currentCategoryId != null) {
                            $categoryBuyingSum = number_format($currentCategoryTotalBuying, 0, '', ' ');
                            $documentHtml .= "
                                    <tr class='app-total-tr'>
                                        <td class='right aligned'>
                                            {$this->translator->_("report.printCumulativeStocks.totals")}
                                            {$currentCategoryName}
                                        </td>
                                        <td>{$currentCategoryTotalQuantity}</td>
                                        <td colspan='2'></td>
                                        <td class='right aligned'>{$categoryBuyingSum}</td>
                                    </tr>
                            ";
                        }
                        $storeBuyingSum = number_format($currentStoreTotalBuying, 0, '', ' ');
                        $documentHtml .= "
                                    <tr>
                                        <td class='right aligned'>
                                            <i class='violet-label'>
                                                {$this->translator->_("report.printCumulativeStocks.totals")}
                                                {$currentStoreName}
                                            </i>
                                        </td>
                                        <td>{$currentStoreTotalQuantity}</td>
                                        <td colspan='2'></td>
                                        <td class='right aligned'>{$storeBuyingSum}</td>
                                    </tr>
                        ";
                    }
                    if ($store == null) {
                        $documentHtml .= "
                                    <tr>
                                        <td colspan='{$effectiveColspan}' class='center-text'>
                                            <i class='violet-label'>
                                                {$cumulativeStock["store_name"]}
                                            </i>
                                        </td>
                                    </tr>
                        ";
                    }
                    $currentStoreId = (int)$cumulativeStock["store_id"];
                    $currentStoreName = $cumulativeStock["store_name"];
                    $currentStoreTotalQuantity = (int)$cumulativeStock["quantity"];
                    $currentStoreTotalBuying = (int)$cumulativeStock["quantity"] * (float)$cumulativeStock["buying_price"];
                    $currentCategoryId = null;
                    $currentCategoryName = null;
                    $currentCategoryTotalQuantity = 0;
                    $currentCategoryTotalBuying = 0;
                } else {
                    $currentStoreTotalQuantity += (int)$cumulativeStock["quantity"];
                    $currentStoreTotalBuying += (int)$cumulativeStock["quantity"] * (float)$cumulativeStock["buying_price"];
                }
                if ((int)$cumulativeStock["category_id"] != $currentCategoryId) {
                    if ($currentCategoryId != null) {
                        $categoryBuyingSum = number_format($currentCategoryTotalBuying, 0, '', ' ');
                        $documentHtml .= "
                                    <tr class='app-total-tr'>
                                        <td class='right aligned'>
                                            {$this->translator->_("report.printCumulativeStocks.totals")}
                                            {$currentCategoryName}
                                        </td>
                                        <td>{$currentCategoryTotalQuantity}</td>
                                        <td colspan='2'></td>
                                        <td class='right aligned'>{$categoryBuyingSum}</td>
                                    </tr>
                        ";
                    }
                    $documentHtml .= "
                                    <tr class='app-highlight-tr'>
                                        <td colspan='{$effectiveColspan}'>{$cumulativeStock["category_name"]}</td>
                                    </tr>
                    ";
                    $currentCategoryId = (int)$cumulativeStock["category_id"];
                    $currentCategoryName = $cumulativeStock["category_name"];
                    $currentCategoryTotalQuantity = (int)$cumulativeStock["quantity"];
                    $currentCategoryTotalBuying = (int)$cumulativeStock["quantity"] * (float)$cumulativeStock["buying_price"];
                } else {
                    $currentCategoryTotalQuantity += (int)$cumulativeStock["quantity"];
                    $currentCategoryTotalBuying += (int)$cumulativeStock["quantity"] * (float)$cumulativeStock["buying_price"];
                }
                $totalQuantity += (int)$cumulativeStock["quantity"];
                $totalBuying += (int)$cumulativeStock["quantity"] * (float)$cumulativeStock["buying_price"];
                $quantity = (int)$cumulativeStock["quantity"];
                $buyingPrice = number_format((float)$cumulativeStock["buying_price"], 0, '', ' ');
                $salePrice = number_format((float)$cumulativeStock["sale_price"], 0, '', ' ');
                $totalBuyingPrice = number_format((int)$cumulativeStock["quantity"] * (float)$cumulativeStock["buying_price"], 0, '', ' ');
                $documentHtml .= "
                                    <tr>
                                        <td style='text-align: left;'>{$cumulativeStock["product_name"]}</td>
                                        <td>{$quantity}</td>
                                        <td class='right aligned'>{$buyingPrice}</td>
                                        <td class='right aligned'>{$salePrice}</td>
                                        <td class='right aligned'>{$totalBuyingPrice}</td>
                                    </tr>
                ";
            }
            if ($currentStoreId != null) {
                if ($currentCategoryId != null) {
                    $categoryBuyingSum = number_format($currentCategoryTotalBuying, 0, '', ' ');
                    $documentHtml .= "
                                    <tr class='app-total-tr'>
                                        <td class='right aligned'>
                                            {$this->translator->_("report.printCumulativeStocks.totals")}
                                            {$currentCategoryName}
                                        </td>
                                        <td>{$currentCategoryTotalQuantity}</td>
                                        <td colspan='2'></td>
                                        <td class='right aligned'>{$categoryBuyingSum}</td>
                                    </tr>
                    ";
                }
                $storeBuyingSum = number_format($currentStoreTotalBuying, 0, '', ' ');
                $documentHtml .= "
                                    <tr>
                                        <td class='right aligned'>
                                            <i class='violet-label'>
                                                {$this->translator->_("report.printCumulativeStocks.totals")}
                                                {$currentStoreName}
                                            </i>
                                        </td>
                                        <td>{$currentStoreTotalQuantity}</td>
                                        <td colspan='2'></td>
                                        <td class='right aligned'>{$storeBuyingSum}</td>
                                    </tr>
                ";
            }
            if ($store == null) {
                $totalBuyingSum = number_format($totalBuying, 0, '', ' ');
                $documentHtml .= "
                                    <tr>
                                        <td class='right aligned'>
                                            <i class='secondary-label'>
                                                {$this->translator->_("report.printCumulativeStocks.totals")}
                                            </i>
                                        </td>
                                        <td>{$totalQuantity}</td>
                                        <td colspan='2'></td>
                                        <td class='right aligned'>{$totalBuyingSum}</td>
                                    </tr>
                ";
            }
            if (count($cumulativeStocksList) == 0) {
                $documentHtml .= "
                                    <tr>
                                        <td colspan='{$effectiveColspan}'>
                                            {$this->translator->_("shared.noDataAvailable")}
                                        </td>
                                    </tr>
                ";
            }
            $documentHtml .= " 
                                </tbody>
                            </table>
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "cumulative_stocks_list_{$_SESSION[SESSION_USER_ID]}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/stocks/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/stocks/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printtransformationsAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/stocks") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $storeId = $_POST["store"] != "" ? (int)$_POST["store"] : null;
            $from = $_POST["from"] != "" ? new DateTime("{$_POST["from"]} 00:00:00") : null;
            $to = $_POST["to"] != "" ? new DateTime("{$_POST["to"]} 23:59:59") : null;
            $transformationDao = new TransformationDAO();
            $storeDao = new StoreDAO();
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $secondaryBrand = APP_SECONDARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = $this->translator->_("report.printTransformations.transformationsList");
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: center;  
                                color: #000000; 
                              
                            }

                            .center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            td.validated {
                                color: {$primaryBrand} !important;
                                font-weight: bold;
                            }

                            td.not-validated {
                                color: {$secondaryBrand} !important;
                                font-weight: bold;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            bold {
                                color: black;
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$this->translator->_("report.printTransformations.transformationsList")}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table>
                                <thead>
                                    <tr>
            ";
            $store = $storeId != null ? $storeDao->getStoreById($storeId) : null;
            $storeName = $store != null ? $store->getNomMagasin() : $this->translator->_("report.printTransformations.allStores");
            $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printTransformations.store")} : <bold class='black-bold'>{$storeName}</bold></th>
            ";
            $printFilters .= "{$this->translator->_("report.printTransformations.store")} : {$storeName}";
            if ($from != null) {
                $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printTransformations.dateAfter")} : <bold class='black-bold'>{$from->format(DATE_FORMAT)}</bold></th>
                ";
                $printFilters .= ", {$this->translator->_("report.printTransformations.dateAfter")} : {$from->format(DATE_FORMAT)}";
            }
            if ($to != null) {
                $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printTransformations.dateBefore")} : <bold class='black-bold'>{$to->format(DATE_FORMAT)}</bold></th>
                ";
                $printFilters .= ", {$this->translator->_("report.printTransformations.dateBefore")} : {$to->format(DATE_FORMAT)}";
            }
            $documentHtml .= "
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border'>
                                <thead>
                                    <tr>
                                        <th>{$this->translator->_("report.printTransformations.number")}</th>
                                        <th>{$this->translator->_("report.printTransformations.productObtained")}</th>
                                        <th>{$this->translator->_("report.printTransformations.qty")}</th>
                                        <th>{$this->translator->_("report.printTransformations.transformationDate")}</th>
                                        <th>{$this->translator->_("report.printTransformations.buyingPrice")}</th>
                                        <th>{$this->translator->_("report.printTransformations.sellingPrice")}</th>
                                        <th>{$this->translator->_("report.printTransformations.status")}</th>
                                    </tr>
                                </thead>
                                <tbody>
            ";
            $transformationList = $transformationDao->getTransformationList($store, $from, $to);
            $currentStoreId = null;
            foreach ($transformationList as $transformation) {
                if ($store == null && $transformation->getIdMagasin()->getIdMagasin() != $currentStoreId) {
                    $effectiveColspan = 7;
                    $documentHtml .= "
                                    <tr>
                                        <td colspan='{$effectiveColspan}' class='black-bold'>
                                            {$transformation->getIdMagasin()->getNomMagasin()}
                                        </td>
                                    </tr>
                    ";
                    $currentStoreId = $transformation->getIdMagasin()->getIdMagasin();
                }
                $buyingPrice = number_format($transformation->getPrixAchat(), 0, '', ' ');
                $sellingPrice = number_format($transformation->getPrixVente(), 0, '', ' ');
                $transformationStatus = $this->translator->_($transformation->getValider() == 1 ? "report.printTransformations.validated" : "report.printTransformations.notValidated");
                $transformationStatusClassName = $transformation->getValider() == 1 ? 'validated' : 'not-validated';
                $documentHtml .= "
                                    <tr>
                                        <td>{$transformation->getIdTransformationProduit()}</td>
                                        <td>{$transformation->getIdProduit()->getDesignation()}</td>
                                        <td>{$transformation->getQuantite()}</td>
                                        <td>{$transformation->getDateTransformation()->format(DATE_FORMAT)}</td>
                                        <td style='text-align: right;'>{$buyingPrice}</td>
                                        <td style='text-align: right;'>{$sellingPrice}</td>
                                        <td class='{$transformationStatusClassName}'>{$transformationStatus}</td>
                                    </tr>
                ";
            }
            if (count($transformationList) == 0) {
                $effectiveColspan = 7;
                $documentHtml .= "
                                    <tr>
                                        <td colspan='{$effectiveColspan}'>
                                            {$this->translator->_("shared.noDataAvailable")}
                                        </td>
                                    </tr>
                ";
            }
            $documentHtml .= "
                                </tbody>
                            </table>
                        </div>
                    </body>
                </html> 
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "transformation_list_{$_SESSION[SESSION_USER_ID]}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/stocks/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/stocks/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printtransformationAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/stocks") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $id = (int)$_POST["id"];
            $transformationDao = new TransformationDAO();
            $transformationDetailDao = new TransformationDetailDAO();
            $transformation = $transformationDao->getTransformationById($id);
            $sellingPrice = number_format($transformation->getPrixVente(), 0, '', ' ');
            $buyingPrice = number_format($transformation->getPrixAchat(), 0, '', ' ');
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = "{$this->translator->_("report.printTransformation.transformationSheetNumber")} : {$id}";
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }
                            

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: center;  
                                color: #000000; 
                            }

                            .center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            .criteria {
                                font-size: 14px;
                                font-weight: bold;
                                color: black;
                                text-align: center;
                            }

                            bold {
                                color: black;
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$this->translator->_("report.printTransformation.transformationSheetNumber")} : {$id}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table>
                                <thead>
                                    <tr>
                                        <td class='criteria'>
                                            {$this->translator->_("report.printTransformation.store")} : {$transformation->getIdMagasin()->getNomMagasin()}
                                        </td>
                                        <td class='criteria'>
                                            {$this->translator->_("report.printTransformation.date")} : {$transformation->getDateTransformation()->format(DATE_FORMAT)}
                                        </td>
                                        <td class='criteria'>
                                            {$this->translator->_("report.printTransformation.productObtained")} : {$transformation->getIdProduit()->getDesignation()}
                                        </td>
                                    </tr>
                                    <tr>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table>
                                <thead>
                                    <tr>
                                        <td class='criteria'>
                                            {$this->translator->_("report.printTransformation.quantity")} : {$transformation->getQuantite()}
                                        </td>
                                        <td class='criteria'>
                                            {$this->translator->_("report.printTransformation.sellingPrice")} : {$sellingPrice}
                                        </td>
                                        <td class='criteria'>
                                            {$this->translator->_("report.printTransformation.buyingPrice")} : {$buyingPrice}
                                        </td>
            ";
            $printFilters .= "{$this->translator->_("report.printTransformation.store")} : {$transformation->getIdMagasin()->getNomMagasin()}";
            $printFilters .= ", {$this->translator->_("report.printTransformation.date")} : {$transformation->getDateTransformation()->format(DATE_FORMAT)}";
            $printFilters .= ", {$this->translator->_("report.printTransformation.productObtained")} : {$transformation->getIdProduit()->getDesignation()}";
            $printFilters .= ", {$this->translator->_("report.printTransformation.quantity")} : {$transformation->getQuantite()}";
            $printFilters .= ", {$this->translator->_("report.printTransformation.sellingPrice")} : {$sellingPrice}";
            $printFilters .= ", {$this->translator->_("report.printTransformation.buyingPrice")} : {$buyingPrice}";
            if (GESTION_DATE_PEREMPTION_PRODUIT == 1 && $transformation->getIdProduit()->getPerissable() == 1 && $transformation->getDatePeremption() != null) {
                $documentHtml .= "
                                        <td class='criteria'>
                                            {$this->translator->_("report.printTransformation.expirationDate")} : {$transformation->getDatePeremption()->format(DATE_FORMAT)}
                                        </td>
                ";
                $printFilters .= ", {$this->translator->_("report.printTransformation.expirationDate")} : {$transformation->getDatePeremption()->format(DATE_FORMAT)}";
            }
            $documentHtml .= "
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border'>
                                <thead>
                                    <tr>
                                        <th>{$this->translator->_("report.printTransformation.product")}</th>
                                        <th>{$this->translator->_("report.printTransformation.batchNumber")}</th>
                                        <th>{$this->translator->_("report.printTransformation.quantity")}</th>
                                    </tr>
                                </thead>
                                <tbody>
            ";
            $transformationDetails = $transformationDetailDao->getDetailsOfTransformation($transformation);
            foreach ($transformationDetails as $transformationDetail) {
                $documentHtml .= "
                                    <tr>
                                        <td style='text-align: left;'>{$transformationDetail->getIdStockSource()->getDesignation()}</td>
                                        <td>{$transformationDetail->getIdStockSource()->getNumLot()}</td>
                                        <td>{$transformationDetail->getQuantiteSource()}</td>
                                    </tr>
                ";
            }
            $documentHtml .= "      
                                </tbody>
                            </table>
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "transformation_sheet_{$_SESSION[SESSION_USER_ID]}_{$id}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/stocks/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/stocks/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printatthresholditemsAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/stocks") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $storeId = $_POST["store"] != "" ? (int)$_POST["store"] : null;
            $name = $_POST["name"] != "" ? $_POST["name"] : null;
            $batchNumber = $_POST["batchNumber"] != "" ? $_POST["batchNumber"] : null;
            $storeDao = new StoreDAO();
            $stockDao = new StockDAO();
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = $this->translator->_("report.printAtThresholdItems.itemsAtThresholdList");
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: center;  
                                color: #000000; 
                              
                            }

                            .center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            bold {
                                color: black;
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$this->translator->_("report.printAtThresholdItems.itemsAtThresholdList")}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table>
                                <thead>
                                    <tr>
            ";
            $store = $storeId != null ? $storeDao->getStoreById($storeId) : null;
            $storeName = $store != null ? $store->getNomMagasin() : $this->translator->_("report.printAtThresholdItems.allStores");
            $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printAtThresholdItems.store")} : <bold class='black-bold'>{$storeName}</bold></th>
            ";
            $printFilters .= "{$this->translator->_("report.printAtThresholdItems.store")} : {$storeName}";
            if ($name != null) {
                $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printAtThresholdItems.productNameContaining")} : <bold class='black-bold'>{$name}</bold></th>
                ";
                $printFilters .= ", {$this->translator->_("report.printAtThresholdItems.productNameContaining")} : {$name}";
            }
            if ($batchNumber != null) {
                $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printAtThresholdItems.batchNumberContaining")} : <bold class='black-bold'>{$batchNumber}</bold></th>
                ";
                $printFilters .= ", {$this->translator->_("report.printAtThresholdItems.batchNumberContaining")} : {$batchNumber}";
            }
            $documentHtml .= "
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border'>
                                <thead>
                                    <tr>
                                        <th>{$this->translator->_("report.printAtThresholdItems.product")}</th>
                                        <th>{$this->translator->_("report.printAtThresholdItems.batchNumber")}</th>
                                        <th>{$this->translator->_("report.printAtThresholdItems.quantity")}</th>
                                    </tr>
                                </thead>
                                <tbody>
            ";
            $whereStore = "";
            $whereName = "";
            $whereBatchNumber = "";
            if ($storeId != null) {
                $whereStore .= " AND s.id_magasin = {$storeId}";
            }
            if ($name != null) {
                $whereName .= " AND s.designation LIKE  '%{$name}%'";
            }
            if ($batchNumber != null) {
                $whereBatchNumber .= " AND s.num_lot LIKE '%{$batchNumber}%'";
            }
            $where = "s.statut = 1 AND p.statut = 1 AND s.quantite > 0 AND s.quantite <= ifnull(p.quantite_alerte, 0){$whereStore}{$whereName}{$whereBatchNumber}";
            $atThresholdStocksTable = "
                    SELECT s.id_produit AS stock_id, s.designation AS product_name, s.num_lot AS batch_number, s.quantite AS quantity,
                    m.nom_magasin AS store_name, s.id_magasin AS store_id
                    FROM stock s JOIN produit p ON s.id_produit = p.id_produit JOIN magasin m ON s.id_magasin = m.id_magasin
                    WHERE {$where}
                    ORDER BY store_id ASC, product_name ASC
            ";
            $atThresholdStocksListStatement = $stockDao->entityManager->getConnection()->prepare($atThresholdStocksTable);
            $atThresholdStocksListStatement->execute();
            $atThresholdList = $atThresholdStocksListStatement->fetchAll();
            $currentStoreId = null;
            foreach ($atThresholdList as $thresholdStock) {
                if ($store == null && (int)$thresholdStock["store_id"] != $currentStoreId) {
                    $effectiveColspan = 3;
                    $documentHtml .= "
                                    <tr>
                                        <td colspan='{$effectiveColspan}' class='black-bold'>
                                            {$thresholdStock["store_name"]}
                                        </td>
                                    </tr>
                    ";
                    $currentStoreId = (int)$thresholdStock["store_id"];
                }
                $batchNumber = $thresholdStock["batch_number"];
                $quantity = (int)$thresholdStock["quantity"];
                $documentHtml .= "
                                    <tr>
                                        <td style='text-align: left;'>{$thresholdStock["product_name"]}</td>
                                        <td>{$batchNumber}</td>
                                        <td>{$quantity}</td>
                                    </tr>
                ";
            }
            if (count($atThresholdList) == 0) {
                $effectiveColspan = 3;
                $documentHtml .= "
                                    <tr>
                                        <td colspan='{$effectiveColspan}'>
                                            {$this->translator->_("shared.noDataAvailable")}
                                        </td>
                                    </tr>
                ";
            }
            $documentHtml .= " 
                                </tbody>
                            </table>
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "at_threshold_items_list_{$_SESSION[SESSION_USER_ID]}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/stocks/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/stocks/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printexpireditemsAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/stocks") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $storeId = $_POST["store"] != "" ? (int)$_POST["store"] : null;
            $name = $_POST["name"] != "" ? $_POST["name"] : null;
            $batchNumber = $_POST["batchNumber"] != "" ? $_POST["batchNumber"] : null;
            $storeDao = new StoreDAO();
            $stockDao = new StockDAO();
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = $this->translator->_("report.printExpiredItems.expiredItemsList");
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }
                            

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: center;  
                                color: #000000; 
                              
                            }

                            .center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            bold {
                                color: black;
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$this->translator->_("report.printExpiredItems.expiredItemsList")}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table>
                                <thead>
                                    <tr>
            ";
            $store = $storeId != null ? $storeDao->getStoreById($storeId) : null;
            $storeName = $store != null ? $store->getNomMagasin() : $this->translator->_("report.printExpiredItems.allStores");
            $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printExpiredItems.store")} : <bold class='black-bold'>{$storeName}</bold></th>
            ";
            $printFilters .= "{$this->translator->_("report.printExpiredItems.store")} : {$storeName}";
            if ($name != null) {
                $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printExpiredItems.productNameContaining")} : <bold class='black-bold'>{$name}</bold></th>
                ";
                $printFilters .= ", {$this->translator->_("report.printExpiredItems.productNameContaining")} : {$name}";
            }
            if ($batchNumber != null) {
                $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printExpiredItems.batchNumberContaining")} : <bold class='black-bold'>{$batchNumber}</bold></th>
                ";
                $printFilters .= ", {$this->translator->_("report.printExpiredItems.batchNumberContaining")} : {$batchNumber}";
            }
            $documentHtml .= "
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border'>
                                <thead>
                                    <tr>
                                        <th>{$this->translator->_("report.printExpiredItems.product")}</th>
                                        <th>{$this->translator->_("report.printExpiredItems.batchNumber")}</th>
                                        <th>{$this->translator->_("report.printExpiredItems.expirationDate")}</th>
                                        <th>{$this->translator->_("report.printExpiredItems.quantity")}</th>
                                    </tr>
                                </thead>
                                <tbody>
            ";
            $currentDate = new DateTime();
            $whereStore = "";
            $whereName = "";
            $whereBatchNumber = "";
            if ($store != null) {
                $whereStore .= " AND s.id_magasin = {$storeId}";
            }
            if ($name != null) {
                $whereName .= " AND s.designation LIKE  '%{$name}%'";
            }
            if ($batchNumber != null) {
                $whereBatchNumber .= " AND s.num_lot LIKE '%{$batchNumber}%'";
            }
            $whereExpirationDate = " AND s.date_peremption <= '{$currentDate->format(DATE_FORMAT_WITH_TIME)}'";
            $where = "s.statut = 1 AND p.statut = 1 AND s.quantite > 0{$whereStore}{$whereName}{$whereBatchNumber}{$whereExpirationDate}";
            $expiredStocksTable = "
                    SELECT s.id_produit AS stock_id, s.designation AS product_name, s.num_lot AS batch_number, s.quantite AS quantity,
                    m.nom_magasin AS store_name, s.date_peremption AS expiration_date, s.id_magasin AS store_id
                    FROM stock s JOIN produit p ON s.id_produit = p.id_produit JOIN magasin m ON s.id_magasin = m.id_magasin
                    WHERE {$where}
                    ORDER BY store_id ASC, product_name ASC
            ";
            $expiredStocksListStatement = $stockDao->entityManager->getConnection()->prepare($expiredStocksTable);
            $expiredStocksListStatement->execute();
            $expiredStockList = $expiredStocksListStatement->fetchAll();
            $currentStoreId = null;
            foreach ($expiredStockList as $expiredStock) {
                if ($store == null && (int)$expiredStock["store_id"] != $currentStoreId) {
                    $effectiveColspan = 4;
                    $documentHtml .= "
                                    <tr>
                                        <td colspan='{$effectiveColspan}' class='black-bold'>
                                            {$expiredStock["store_name"]}
                                        </td>
                                    </tr>
                    ";
                    $currentStoreId = (int)$expiredStock["store_id"];
                }
                $batchNumber = $expiredStock["batch_number"];
                $expirationDate = new DateTime($expiredStock["expiration_date"]);
                $quantity = (int)$expiredStock["quantity"];
                $documentHtml .= "
                                    <tr>
                                        <td style='text-align: left;'>{$expiredStock["product_name"]}</td>
                                        <td>{$batchNumber}</td>
                                        <td>{$expirationDate->format(DATE_FORMAT)}</td>
                                        <td>{$quantity}</td>
                                    </tr>
                ";
            }
            if (count($expiredStockList) == 0) {
                $effectiveColspan = 4;
                $documentHtml .= "
                                    <tr>
                                        <td colspan='{$effectiveColspan}'>
                                            {$this->translator->_("shared.noDataAvailable")}
                                        </td>
                                    </tr>
                ";
            }
            $documentHtml .= " 
                                </tbody>
                            </table>
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "expired_items_list_{$_SESSION[SESSION_USER_ID]}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/stocks/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/stocks/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printsoontoexpireitemsAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/stocks") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $storeId = $_POST["store"] != "" ? (int)$_POST["store"] : null;
            $name = $_POST["name"] != "" ? $_POST["name"] : null;
            $batchNumber = $_POST["batchNumber"] != "" ? $_POST["batchNumber"] : null;
            $storeDao = new StoreDAO();
            $stockDao = new StockDAO();
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = $this->translator->_("report.printSoonToExpireItems.soonToExpiredItemsList");
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }
                            

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: center;  
                                color: #000000; 
                              
                            }

                            .center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            bold {
                                color: black;
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$this->translator->_("report.printSoonToExpireItems.soonToExpiredItemsList")}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table>
                                <thead>
                                    <tr>
            ";
            $store = $storeId != null ? $storeDao->getStoreById($storeId) : null;
            $storeName = $store != null ? $store->getNomMagasin() : $this->translator->_("report.printSoonToExpireItems.allStores");
            $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printSoonToExpireItems.store")} : <bold class='black-bold'>{$storeName}</bold></th>
            ";
            $printFilters .= "{$this->translator->_("report.printSoonToExpireItems.store")} : {$storeName}";
            if ($name != null) {
                $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printSoonToExpireItems.productNameContaining")} : <bold class='black-bold'>{$name}</bold></th>
                ";
                $printFilters .= ", {$this->translator->_("report.printSoonToExpireItems.productNameContaining")} : {$name}";
            }
            if ($batchNumber != null) {
                $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printSoonToExpireItems.batchNumberContaining")} : <bold class='black-bold'>{$batchNumber}</bold></th>
                ";
                $printFilters .= ", {$this->translator->_("report.printSoonToExpireItems.batchNumberContaining")} : $batchNumber}";
            }
            $documentHtml .= "
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border'>
                                <thead>
                                    <tr>
                                        <th>{$this->translator->_("report.printSoonToExpireItems.product")}</th>
                                        <th>{$this->translator->_("report.printSoonToExpireItems.batchNumber")}</th>
                                        <th>{$this->translator->_("report.printSoonToExpireItems.expirationDate")}</th>
                                        <th>{$this->translator->_("report.printSoonToExpireItems.quantity")}</th>
                                    </tr>
                                </thead>
                                <tbody>
            ";
            $currentDate = new DateTime();
            $limitExpirationDate = new DateTime();
            $daysForExpirationAlert = (int)NOMBRE_JOUR_POUR_ALERTE_PEREMPTION;
            $limitExpirationDate->modify("+ {$daysForExpirationAlert} days");
            $store = (int)$_POST["store"] != 0 ? (int)$_POST["store"] : null;
            $name = $_POST["name"] != "" ? $_POST["name"] : null;
            $batchNumber = $_POST["batchNumber"] != "" ? $_POST["batchNumber"] : null;
            $whereStore = "";
            $whereName = "";
            $whereBatchNumber = "";
            if ($store != null) {
                $whereStore .= " AND s.id_magasin = {$store}";
            }
            if ($name != null) {
                $whereName .= " AND s.designation LIKE  '%{$name}%'";
            }
            if ($batchNumber != null) {
                $whereBatchNumber .= " AND s.num_lot LIKE '%{$batchNumber}%'";
            }
            $whereExpirationDate = " AND s.date_peremption > '{$currentDate->format(DATE_FORMAT_WITH_TIME)}' AND s.date_peremption <= '{$limitExpirationDate->format(DATE_FORMAT)} 23:59:59'";
            $where = "s.statut = 1 AND p.statut = 1 AND s.quantite > 0{$whereStore}{$whereName}{$whereBatchNumber}{$whereExpirationDate}";
            $soonToExpireStocksTable = "
                    SELECT s.id_produit AS stock_id, s.designation AS product_name, s.num_lot AS batch_number, s.quantite AS quantity,
                    m.nom_magasin AS store_name, s.date_peremption AS expiration_date, s.id_magasin AS store_id
                    FROM stock s JOIN produit p ON s.id_produit = p.id_produit JOIN magasin m ON s.id_magasin = m.id_magasin
                    WHERE {$where}
                    ORDER BY store_id ASC, product_name ASC
            ";
            $soonToExpireStocksListStatement = $stockDao->entityManager->getConnection()->prepare($soonToExpireStocksTable);
            $soonToExpireStocksListStatement->execute();
            $soonToExpireStockList = $soonToExpireStocksListStatement->fetchAll();
            $currentStoreId = null;
            foreach ($soonToExpireStockList as $soonToExpireStock) {
                if ($store == null && (int)$soonToExpireStock["store_id"] != $currentStoreId) {
                    $effectiveColspan = 4;
                    $documentHtml .= "
                                    <tr>
                                        <td colspan='{$effectiveColspan}' class='black-bold'>
                                            {$soonToExpireStock["store_name"]}
                                        </td>
                                    </tr>
                    ";
                    $currentStoreId = (int)$soonToExpireStock["store_id"];
                }
                $batchNumber = $soonToExpireStock["batch_number"];
                $expirationDate = new DateTime($soonToExpireStock["expiration_date"]);
                $quantity = (int)$soonToExpireStock["quantity"];
                $documentHtml .= "
                                    <tr>
                                        <td style='text-align: left;'>{$soonToExpireStock["product_name"]}</td>
                                        <td>{$batchNumber}</td>
                                        <td>{$expirationDate->format(DATE_FORMAT)}</td>
                                        <td>{$quantity}</td>
                                    </tr>
                ";
            }
            if (count($soonToExpireStockList) == 0) {
                $effectiveColspan = 4;
                $documentHtml .= "
                                    <tr>
                                        <td colspan='{$effectiveColspan}'>
                                            {$this->translator->_("shared.noDataAvailable")}
                                        </td>
                                    </tr>
                ";
            }
            $documentHtml .= " 
                                </tbody>
                            </table>
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "soon_to_expire_items_list_{$_SESSION[SESSION_USER_ID]}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/stocks/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/stocks/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printproductbarcodeAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/products") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            foreach (new DirectoryIterator(PUBLIC_PATH . "/temporary") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $id = (int)$_POST["id"];
            $productDao = new ProductDAO();
            $product = $productDao->getProductById($id);
            $barcode = new \Com\Tecnick\Barcode\Barcode();
            $productBarcodeImageName = "product_bar_code_{$id}.png";
            $targetPath = PUBLIC_PATH . "/temporary/{$productBarcodeImageName}";
            $barcodeObject = $barcode->getBarcodeObj('C128C', $product->getCodeBarre(), 450, 70, 'black', [0, 0, 0, 0]);
            $barcodeImageData = $barcodeObject->getPngData();
            file_put_contents($targetPath, $barcodeImageData);
            $barcodeImageLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/temporary/{$productBarcodeImageName}";
            $documentHtml = "
                <html>
                    <body>
                        <style>
                            @page {
                                margin: 5px 0;
                            }

                            .main {
                                padding-left: 10px;
                                padding-right: 10px;
                            }

                            .barcode-image {
                                width: 100%;
                                height: 50px;
                            }

                            .top-barcode {
                                text-align: center;
                                font-weight: bold;
                                font-size: 13px;
                            }

                            .bottom-barcode {
                                text-align: center;
                                letter-spacing: 4px;
                                font-weight: bold;
                                font-size: 13px;
                            }
                        </style>
                        <div class='main'>
                            <div class='top-barcode'>CRESCITA</div>
                            <img class='barcode-image' src='{$barcodeImageLink}' alt='barcode'/>
                            <div class='bottom-barcode'>{$product->getCodeBarre()}</div>
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper(BARCODE_DIMENSIONS, "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "barcode_product_{$_SESSION[SESSION_USER_ID]}_{$id}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/products/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/products/{$filename}";
            $currentDate = new DateTime();
            $printCode = Utility::generatePrintCode($currentDate);
            $printTitle = "{$this->translator->_("report.printBarCode.barCodeOf")} {$product->getDesignation()}({$id})";
            $printFilters = $this->translator->_("report.noFilter");
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printordersAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/orders") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $supplierId = $_POST["supplier"] != "" ? (int)$_POST["supplier"] : null;
            $from = $_POST["from"] != "" ? new DateTime($_POST["from"]) : null;
            $to = $_POST["to"] != "" ? new DateTime($_POST["to"]) : null;
            $supplierDao = new SupplierDAO();
            $orderDao = new OrderDAO();
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $secondaryBrand = APP_SECONDARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = $this->translator->_("report.printOrders.ordersList");
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }
                            

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: center;  
                                color: #000000; 
                              
                            }

                            .center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            td.validated {
                                color: {$primaryBrand} !important;
                                font-weight: bold;
                            }

                            td.not-validated {
                                color: {$secondaryBrand} !important;
                                font-weight: bold;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            bold {
                                color: black;
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$this->translator->_("report.printOrders.ordersList")}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table>
                                <thead>
                                    <tr>
            ";
            $supplier = $supplierId != null ? $supplierDao->getSupplierById($supplierId) : null;
            $supplierName = $supplier != null ? $supplier->getNomFournisseur() : $this->translator->_("report.printOrders.allSuppliers");
            $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printOrders.supplier")} : <bold class='black-bold'>{$supplierName}</bold></th>
            ";
            $printFilters .= "{$this->translator->_("report.printOrders.supplier")} : {$supplierName}";
            if ($from != null) {
                $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printOrders.dateAfter")} : <bold class='black-bold'>{$from->format(DATE_FORMAT)}</bold></th>
                ";
                $printFilters .= ", {$this->translator->_("report.printOrders.dateAfter")} : {$from->format(DATE_FORMAT)}";
            }
            if ($to != null) {
                $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printOrders.dateBefore")} : <bold class='black-bold'>{$to->format(DATE_FORMAT)}</bold></th>
                ";
                $printFilters .= ", {$this->translator->_("report.printOrders.dateBefore")} : {$to->format(DATE_FORMAT)}";
            }
            $documentHtml .= "
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border'>
                                <thead>
                                    <tr>
                                        <th>{$this->translator->_("report.printOrders.number")}</th>
                                        <th>{$this->translator->_("report.printOrders.orderDate")}</th>
                                        <th>{$this->translator->_("report.printOrders.wishedDeliveryDate")}</th>
                                        <th>{$this->translator->_("report.printOrders.entryDate")}</th>
                                        <th>{$this->translator->_("report.printOrders.total")}</th>
                                        <th>{$this->translator->_("report.printOrders.status")}</th>
                                    </tr>
                                </thead>
                                <tbody>
            ";
            $ordersList = $orderDao->getOrdersFiltered($supplier, $from, $to);
            $currentSupplierId = null;
            foreach ($ordersList as $order) {
                if ($supplier == null && $order->getIdFournisseur()->getIdFournisseur() !== $currentSupplierId) {
                    $effectiveColspan = 6;
                    $documentHtml .= "
                                    <tr>
                                        <td colspan='{$effectiveColspan}' class='black-bold'>
                                            {$order->getIdFournisseur()->getNomFournisseur()}
                                        </td>
                                    </tr>
                    ";
                    $currentSupplierId = $order->getIdFournisseur()->getIdFournisseur();
                }
                $total = number_format($order->getMontantTtc(), 0, '', ' ');
                $orderStatus = $this->translator->_($order->getValider() == 1 ? "report.printOrders.validated" : "report.printOrders.notValidated");
                $orderStatusClassName = $order->getValider() == 1 ? 'validated' : 'not-validated';
                $deliveryDate = $order->getDateLivraison() != null ? $order->getDateLivraison()->format(DATE_FORMAT) : $this->translator->_("shared.undefined");
                $documentHtml .= "
                                    <tr>
                                        <td>{$order->getIdCommande()}</td>
                                        <td>{$order->getDateCommande()->format(DATE_FORMAT)}</td>
                                        <td>{$deliveryDate}</td>
                                        <td>{$order->getDateSaisie()->format(DATE_FORMAT_WITH_SECONDS)}</td>
                                        <td style='text-align: right;'>{$total}</td>
                                        <td class='{$orderStatusClassName}'>{$orderStatus}</td>
                                    </tr>
                ";
            }
            if (count($ordersList) == 0) {
                $effectiveColspan = 6;
                $documentHtml .= "
                                    <tr>
                                        <td colspan='{$effectiveColspan}'>
                                            {$this->translator->_("shared.noDataAvailable")}
                                        </td>
                                    </tr>
                ";
            }
            $documentHtml .= " 
                                </tbody>
                            </table>
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "orders_list_{$_SESSION[SESSION_USER_ID]}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/orders/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/orders/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printorderAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/orders") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $id = (int)$_POST["id"];
            $orderDao = new OrderDAO();
            $orderLineDao = new OrderLineDAO();
            $order = $orderDao->getOrderById($id);
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = "{$this->translator->_("report.printOrder.orderSheetNumber")} : {$id}";
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }
                            

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: center;  
                                color: #000000; 
                            }

                            .center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            .criteria {
                                font-size: 14px;
                                font-weight: bold;
                                color: black;
                                text-align: center;
                            }

                            .equalized {
                                table-layout: fixed;
                            }

                            bold {
                                color: black;
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                        <td>
                                            <bold>
                                                <h3 style='text-align: right;'>{$this->translator->_("report.printOrder.addressedTo")}:</h3>
                                                <h1 style='text-align: right;'>{$order->getIdFournisseur()->getNomFournisseur()}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td colspan='2' class='title'>{$this->translator->_("report.printOrder.orderSheetNumber")} : {$id}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='equalized'>
                                <thead>
                                    <tr>
                                        <td class='criteria'>
                                            {$this->translator->_("report.printOrder.date")} : {$order->getDateCommande()->format(DATE_FORMAT)}
                                        </td>
            ";
            $printFilters = "{$this->translator->_("report.printOrder.date")} : {$order->getDateCommande()->format(DATE_FORMAT)}";
            if ($order->getDateLivraison() != null) {
                $documentHtml .= "
                                        <td class='criteria'>
                                            {$this->translator->_("report.printOrder.wishedDeliveryDate")} : {$order->getDateLivraison()->format(DATE_FORMAT)}
                                        </td>
                ";
                $printFilters .= ", {$this->translator->_("report.printOrder.wishedDeliveryDate")} : {$order->getDateLivraison()->format(DATE_FORMAT)}";
            }
            $documentHtml .= "
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border'>
                                <thead>
                                    <tr>
                                        <th>{$this->translator->_("report.printOrder.designation")}</th>
                                        <th>{$this->translator->_("report.printOrder.quantity")}</th>
                                        <th>{$this->translator->_("report.printOrder.unitP")}</th>
                                        <th>{$this->translator->_("report.printOrder.total")}</th>
                                    </tr>
                                </thead>
                                <tbody>
            ";
            $orderLines = $orderLineDao->getOrderLinesByOrder($order);
            foreach ($orderLines as $orderLine) {
                $unitPrice = number_format($orderLine->getPrixUnitaire(), 0, '', ' ');
                $total = number_format($orderLine->getQuantite() * $orderLine->getPrixUnitaire(), 0, '', ' ');
                $documentHtml .= "
                                    <tr>
                                        <td style='text-align: left;'>{$orderLine->getIdProduit()->getDesignation()}</td>
                                        <td>{$orderLine->getQuantite()}</td>
                                        <td style='text-align: right;'>{$unitPrice}</td>
                                        <td style='text-align: right;'>{$total}</td>
                                    </tr>
                ";
            }
            $orderTotal = number_format($order->getMontantTtc(), 0, '', ' ');
            $documentHtml .= "      <tr>
                                        <td colspan='3' style='text-align:right; font-weight: bold'>{$this->translator->_("report.printOrder.total")}</td>
                                        <td style='text-align:right;'>{$orderTotal}</td>
                                    </tr>
                                </tbody>
                            </table>
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "order_sheet_{$_SESSION[SESSION_USER_ID]}_{$id}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/orders/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/orders/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printreceptionsAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/receptions") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $storeId = $_POST["store"] != "" ? (int)$_POST["store"] : null;
            $supplierId = $_POST["supplier"] != "" ? (int)$_POST["supplier"] : null;
            $from = $_POST["from"] != "" ? new DateTime($_POST["from"]) : null;
            $to = $_POST["to"] != "" ? new DateTime($_POST["to"]) : null;
            $storeDao = new StoreDAO();
            $supplierDao = new SupplierDAO();
            $receptionDao = new ReceptionDAO();
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $secondaryBrand = APP_SECONDARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = $this->translator->_("report.printReceptions.receptionsList");
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: center;  
                                color: #000000; 
                              
                            }

                            .center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            td.validated {
                                color: {$primaryBrand} !important;
                                font-weight: bold;
                            }

                            td.not-validated {
                                color: {$secondaryBrand} !important;
                                font-weight: bold;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            bold {
                                color: black;
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$this->translator->_("report.printReceptions.receptionsList")}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table>
                                <thead>
                                    <tr>
            ";
            $store = $storeId != null ? $storeDao->getStoreById($storeId) : null;
            $storeName = $store != null ? $store->getNomMagasin() : $this->translator->_("report.printReceptions.allStores");
            $supplier = $supplierId != null ? $supplierDao->getSupplierById($supplierId) : null;
            $supplierName = $supplier != null ? $supplier->getNomFournisseur() : $this->translator->_("report.printReceptions.allSuppliers");
            $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printReceptions.store")} : <bold class='black-bold'>{$storeName}</bold></th>
                                        <th class='center-text'>{$this->translator->_("report.printReceptions.supplier")} : <bold class='black-bold'>{$supplierName}</bold></th>
            ";
            $printFilters .= "{$this->translator->_("report.printReceptions.store")} : {$storeName}";
            $printFilters .= ", {$this->translator->_("report.printReceptions.supplier")} : {$supplierName}";
            if ($from != null) {
                $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printReceptions.dateAfter")} : <bold class='black-bold'>{$from->format(DATE_FORMAT)}</bold></th>
                ";
                $printFilters .= ", {$this->translator->_("report.printReceptions.dateAfter")} :{$from->format(DATE_FORMAT)}";
            }
            if ($to != null) {
                $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printReceptions.dateBefore")} : <bold class='black-bold'>{$to->format(DATE_FORMAT)}</bold></th>
                ";
                $printFilters .= ", {$this->translator->_("report.printReceptions.dateBefore")} : {$to->format(DATE_FORMAT)}";
            }
            $documentHtml .= "
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border'>
                                <thead>
                                    <tr>
                                        <th colspan='2'>{$this->translator->_("report.printReceptions.number")}</th>
                                        <th colspan='2'>{$this->translator->_("report.printReceptions.entryDate")}</th>
                                        <th colspan='2'>{$this->translator->_("report.printReceptions.receivingDate")}</th>
                                        <th colspan='2'>{$this->translator->_("report.printReceptions.deliveryNoteNumber")}</th>
                                        <th colspan='2'>{$this->translator->_("report.printReceptions.status")}</th>
                                    </tr>
                                </thead>
                                <tbody>
            ";
            $receptionsList = $receptionDao->getReceptionsFiltered($store, $supplier, $from, $to);
            $currentStoreId = null;
            $currentSupplierId = null;
            foreach ($receptionsList as $reception) {
                if (($store == null || $supplier == null) && ($reception->getIdMagasin()->getIdMagasin() != $currentStoreId || $reception->getIdFournisseur()->getIdFournisseur() != $currentSupplierId)) {
                    $effectiveColspan = 10;
                    $documentHtml .= "
                                    <tr>
                    ";
                    if ($store == null && $supplier == null) $effectiveColspan /= 2;
                    if ($store == null) {
                        $documentHtml .= "
                                        <td colspan='{$effectiveColspan}' class='black-bold'>
                                            {$this->translator->_("report.printReceptions.store")} : {$reception->getIdMagasin()->getNomMagasin()}
                                        </td>
                        ";
                    }
                    if ($supplier == null) {
                        $documentHtml .= "
                                        <td colspan='{$effectiveColspan}' class='black-bold'>
                                            {$this->translator->_("report.printReceptions.supplier")} : {$reception->getIdFournisseur()->getNomFournisseur()}
                                        </td>
                        ";
                    }
                    $documentHtml .= "
                                    </tr>
                    ";
                    $currentStoreId = $reception->getIdMagasin()->getIdMagasin();
                    $currentSupplierId = $reception->getIdFournisseur()->getIdFournisseur();
                }
                $receptionStatus = $this->translator->_($reception->getValider() == 1 ? "report.printReceptions.validated" : "report.printReceptions.notValidated");
                $receptionStatusClassName = $reception->getValider() == 1 ? 'validated' : 'not-validated';
                $documentHtml .= "
                                    <tr>
                                        <td colspan='2'>{$reception->getIdReception()}</td>
                                        <td colspan='2'>{$reception->getDateSaisie()->format(DATE_FORMAT)}</td>
                                        <td colspan='2'>{$reception->getDateReception()->format(DATE_FORMAT)}</td>
                                        <td colspan='2'>{$reception->getNumBl()}</td>
                                        <td colspan='2' class='{$receptionStatusClassName}'>{$receptionStatus}</td>
                                    </tr>
                ";
            }
            if (count($receptionsList) == 0) {
                $effectiveColspan = 10;
                $documentHtml .= "
                                    <tr>
                                        <td colspan='{$effectiveColspan}'>
                                            {$this->translator->_("shared.noDataAvailable")}
                                        </td>
                                    </tr>
                ";
            }
            $documentHtml .= " 
                                </tbody>
                            </table>
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "receptions_list_{$_SESSION[SESSION_USER_ID]}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/receptions/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/receptions/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printreceptionAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/receptions") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $id = (int)$_POST["id"];
            $receptionDao = new ReceptionDAO();
            $receptionLineDao = new ReceptionLineDAO();
            $reception = $receptionDao->getReceptionById($id);
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $secondaryBrand = APP_SECONDARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = "{$this->translator->_("report.printReception.receiptSheetNumber")} : {$id}";
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: center;  
                                color: #000000; 
                            }

                            .center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            td.not-perishable {
                                color: {$secondaryBrand} !important;
                                font-weight: bold;
                                font-size: 10px;
                            }

                            bold {
                                color: black;
                            }

                            .criteria {
                                font-size: 14px;
                                font-weight: bold;
                                color: black;
                                text-align: center;
                            }

                            .equalized {
                                table-layout: fixed;
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                        <td>
                                            <bold>
                                                <h3 style='text-align: right;'>{$this->translator->_("report.printReception.receivedFrom")}:</h3>
                                                <h1 style='text-align: right;'>{$reception->getIdFournisseur()->getNomFournisseur()}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td colspan='2' class='title'>{$this->translator->_("report.printReception.receiptSheetNumber")} : {$id}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='equalized'>
                                <thead>
                                    <tr>
            ";
            if ($reception->getIdCommande() != null) {
                $documentHtml .= "
                                        <td class='criteria'>{$this->translator->_("report.printReception.originalOrderNumber")} : {$reception->getIdCommande()->getIdCommande()}</td>
                ";
                $printFilters .= "{$this->translator->_("report.printReception.originalOrderNumber")} : {$reception->getIdCommande()->getIdCommande()}";
                $printFilters .= ", ";
            }
            $documentHtml .= "
                                        <td class='criteria'>{$this->translator->_("report.printReception.date")} : {$reception->getDateReception()->format(DATE_FORMAT)}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border'>
                                <thead>
                                    <tr>
                                        <th>{$this->translator->_("report.printReception.designation")}</th>
                                        <th>{$this->translator->_("report.printReception.quantity")}</th>
                                        <th>{$this->translator->_("report.printReception.buyingPrice")}</th>
                                        <th>{$this->translator->_("report.printReception.sellingPrice")}</th>
                                        <th>{$this->translator->_("report.printReception.maxReductionPercent")}</th>
                                        <th>{$this->translator->_("report.printReception.batchNumber")}</th>
            ";
            $printFilters .= "{$this->translator->_("report.printReception.date")} : {$reception->getDateReception()->format(DATE_FORMAT)}";
            if (GESTION_DATE_PEREMPTION_PRODUIT == 1) {
                $documentHtml .= "
                                        <th>{$this->translator->_("report.printReception.expirationDate")}</th>
                ";
            }
            $documentHtml .= "
                                    </tr>
                                </thead>
                                <tbody>
            ";
            $receptionLines = $receptionLineDao->getReceptionLinesOfReception($reception);
            $totalBuyingPrice = 0;
            $totalSellingPrice = 0;
            foreach ($receptionLines as $receptionLine) {
                $totalBuyingPrice += $receptionLine->getPrixAchat();
                $totalSellingPrice += $receptionLine->getPrixUnitaire();
                $buyingPrice = number_format($receptionLine->getPrixAchat(), 0, '', ' ');
                $sellingPrice = number_format($receptionLine->getPrixUnitaire(), 0, '', ' ');
                $documentHtml .= "
                                    <tr>
                                        <td style='text-align: left;'>{$receptionLine->getDesignation()}</td>
                                        <td>{$receptionLine->getQuantite()}</td>
                                        <td style='text-align: right;'>{$buyingPrice}</td>
                                        <td style='text-align: right;'>{$sellingPrice}</td>
                                        <td>{$receptionLine->getPourcentageMaxRemise()}</td>
                                        <td>{$receptionLine->getNumLotFournisseur()}</td>
                ";
                if (GESTION_DATE_PEREMPTION_PRODUIT == 1) {
                    if ($receptionLine->getIdProduit()->getPerissable() == 1 && $receptionLine->getDatePeremption() != null) {
                        $documentHtml .= "
                                        <td>{$receptionLine->getDatePeremption()->format(DATE_FORMAT)}</td>
                        ";
                    } else {
                        $documentHtml .= "
                                        <td class='not-perishable'>{$this->translator->_("shared.notPerishable")}</td>
                        ";
                    }
                }
                $documentHtml .= "
                                    </tr>
                ";
            }
            $formatTotalBuyingPrice = number_format($totalBuyingPrice, 0, '', ' ');
            $formatTotalSellingPrice = number_format($totalSellingPrice, 0, '', ' ');
            $effectiveColspan = 2;
            if (GESTION_DATE_PEREMPTION_PRODUIT == 1) $effectiveColspan++;
            $documentHtml .= "  
                                    <tr>
                                        <td colspan='2' style='text-align: right; font-weight: bold;'>{$this->translator->_("report.printReception.total")}</td>
                                        <td style='text-align: right;'>{$formatTotalBuyingPrice}</td>
                                        <td style='text-align: right;'>{$formatTotalSellingPrice}</td>
                                        <td colspan='{$effectiveColspan}'></td>
                                    </tr>
                                </tbody>
                            </table>
                            <div style='width: 100%; height: 50px;'></div>
                            <table class='equalized'>
                                <tr>
                                    <th style='text-decoration: underline; text-align: left;' class='black-bold'>
                                        {$this->translator->_("report.printReception.storekeeperSignature")}
                                    </th>
                                    <th style='text-decoration: underline; text-align: right;' class='black-bold'>
                                        {$this->translator->_("report.printReception.carrierSignature")}
                                    </th>
                                </tr>
                            </table>
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "reception_sheet_{$_SESSION[SESSION_USER_ID]}_{$id}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/receptions/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/receptions/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printstockbatchnumberAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/stocks") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            foreach (new DirectoryIterator(PUBLIC_PATH . "/temporary") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $id = (int)$_POST["id"];
            $stockDao = new StockDAO();
            $stock = $stockDao->getStockById($id);
            $barcode = new \Com\Tecnick\Barcode\Barcode();
            $stockBatchNumberImageName = "stock_batch_number_{$id}.png";
            $targetPath = PUBLIC_PATH . "/temporary/{$stockBatchNumberImageName}";
            $barcodeObject = $barcode->getBarcodeObj('C128C', $stock->getNumLot(), 450, 70, 'black', [0, 0, 0, 0]);
            $barcodeImageData = $barcodeObject->getPngData();
            file_put_contents($targetPath, $barcodeImageData);
            $barcodeImageLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/temporary/{$stockBatchNumberImageName}";
            $documentHtml = "
                <html>
                    <body>
                        <style>
                            @page {
                                margin: 5px 0;
                            }

                            .main {
                                padding-left: 10px;
                                padding-right: 10px;
                            }

                            .barcode-image {
                                width: 100%;
                                height: 50px;
                            }

                            .top-barcode {
                                text-align: center;
                                font-weight: bold;
                                font-size: 13px;
                            }

                            .bottom-barcode {
                                text-align: center;
                                letter-spacing: 4px;
                                font-weight: bold;
                                font-size: 13px;
                            }
                        </style>
                        <div class='main'>
                            <div class='top-barcode'>CRESCITA</div>
                            <img class='barcode-image' src='{$barcodeImageLink}' alt='barcode'/>
                            <div class='bottom-barcode'>{$stock->getNumLot()}</div>
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper(BARCODE_DIMENSIONS, "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "stock_batch_number_{$_SESSION[SESSION_USER_ID]}_{$id}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/stocks/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/stocks/{$filename}";
            $currentDate = new DateTime();
            $printCode = Utility::generatePrintCode($currentDate);
            $printTitle = "{$this->translator->_("report.printBatchNumber.batchNumber")} {$stock->getNumLot()}({$id})";
            $printFilters = $this->translator->_("report.noFilter");
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printviewstocksAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/stocks") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $storeId = $_POST["store"] != "" ? (int)$_POST["store"] : null;
            $name = $_POST["name"] != "" ? $_POST["name"] : null;
            $batchNumber = $_POST["batchNumber"] != "" ? $_POST["batchNumber"] : null;
            $from = $_POST["from"] != "" ? new DateTime("{$_POST["from"]} 00:00:00") : null;
            $to = $_POST["to"] != "" ? new DateTime("{$_POST["to"]} 23:59:59") : null;
            $storeDao = new StoreDAO();
            $stockDao = new StockDAO();
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $secondaryBrand = APP_SECONDARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = $this->translator->_("report.printViewStocks.stocksReport");
            $printFilters = "";
            $considerExpirationDate = GESTION_DATE_PEREMPTION_PRODUIT == 1;
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: left;  
                                color: #000000; 
                            }

                            td.center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            td.not-perishable {
                                color: {$secondaryBrand} !important;
                                font-weight: bold;
                                font-size: 10px;
                            }

                            bold {
                                color: black;
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }

                            i.violet-label {
                                background-color: #6435c9;
                                color: white;
                                padding: 2px;
                                border-radius: 6px;
                                margin: 3px;
                            }

                            i.secondary-label {
                                background-color: {$secondaryBrand};
                                color: white;
                                padding: 2px;
                                border-radius: 6px;
                                margin: 3px;
                            }

                            td.left.aligned {
                                text-align: left;
                            }

                            td.right.aligned {
                                text-align: right;
                            }

                            tr.app-total-tr {
                                background-color: {$primaryBrand}80;
                            }

                            tr.app-total-tr > td {
                                color: {$thirdBrand};
                            }

                            tr.app-highlight-tr {
                                background-color: {$secondaryBrand}80;
                            }

                            tr.app-highlight-tr > td {
                                color: {$thirdBrand};
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$this->translator->_("report.printViewStocks.stocksReport")}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
            ";
            $store = $storeId != null ?  $storeDao->getStoreById($storeId) : null;
            $storeName = $store != null ? $store->getNomMagasin() : $this->translator->_("report.printViewStocks.allStores");
            $documentHtml .= "
                            <table>
                                <thead>
                                    <tr>
                                        <th class='center-text'>{$this->translator->_("report.printViewStocks.store")} : <bold class='black-bold'>{$storeName}</bold></th>
                            
            ";
            $printFilters .= "{$this->translator->_("report.printViewStocks.store")} : {$storeName}";
            if ($name != null) {
                $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printViewStocks.productNameContaining")} : <bold class='black-bold'>{$name}</bold></th>
                ";
                $printFilters .= ", {$this->translator->_("report.printViewStocks.productNameContaining")} : {$name}";
            }
            if ($batchNumber != null) {
                $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printViewStocks.batchNumberContaining")} : <bold class='black-bold'>{$batchNumber}</bold></th>
                ";
                $printFilters .= ", {$this->translator->_("report.printViewStocks.batchNumberContaining")} : {$batchNumber}";
            }
            if ($considerExpirationDate) {
                if ($from != null) {
                    $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printViewStocks.expirationDateAfter")} : <bold class='black-bold'>{$from->format(DATE_FORMAT)}</bold></th>
                    ";
                    $printFilters .= ", {$this->translator->_("report.printViewStocks.expirationDateAfter")} : {$from->format(DATE_FORMAT)}";
                }
                if ($to != null) {
                    $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printViewStocks.expirationDateBefore")} : <bold class='black-bold'>{$from->format(DATE_FORMAT)}</bold></th>
                    ";
                    $printFilters .= ", {$this->translator->_("report.printViewStocks.expirationDateBefore")} : {$from->format(DATE_FORMAT)}";
                }
            }
            $documentHtml .= "
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border'>
                                <thead>
                                    <tr>
                                        <th style='width: 30%;'>{$this->translator->_("report.printViewStocks.designation")}</th>
                                        <th>{$this->translator->_("report.printStocks.batchNumber")}</th>
                                        <th>{$this->translator->_("report.printStocks.qty")}</th>
            ";
            if ($considerExpirationDate) {
                $documentHtml .= "
                                        <th>{$this->translator->_("report.printViewStocks.expirationDate")}</th>
                ";
            }
            $documentHtml .= "
                                        <th>{$this->translator->_("report.printViewStocks.initialQty")}</th>
                                        <th>{$this->translator->_("report.printViewStocks.buyingPrice")}</th>
                                        <th>{$this->translator->_("report.printViewStocks.salePrice")}</th>
                                        <th style='width: 10%;'>{$this->translator->_("report.printViewStocks.buyingTotal")}</th>
                                        <th style='width: 10%;'>{$this->translator->_("report.printViewStocks.saleTotal")}</th>
                                        <th style='width: 10%;'>{$this->translator->_("report.printViewStocks.profit")}</th>
                                    </tr>
                                </thead>
                                <tbody>
            ";
            $whereStore = "";
            $whereName = "";
            $whereBatchNumber = "";
            $whereFrom = "";
            $whereTo = "";
            if ($storeId != null) {
                $whereStore .= " AND s.id_magasin = {$storeId}";
            }
            if ($name != null) {
                $whereName .= " AND s.designation LIKE '%{$name}%'";
            }
            if ($batchNumber != null) {
                $whereBatchNumber .= " AND s.num_lot LIKE '%{$batchNumber}%'";
            }
            if ($from != null) {
                $whereFrom .= " AND s.date_peremption >= '{$from->format(DATE_FORMAT)} 00:00:00'";
            }
            if ($to != null) {
                $whereTo .= " AND s.date_peremption <= '{$to->format(DATE_FORMAT)} 23:59:59'";
            }
            $where = "s.statut = 1 AND s.quantite > 0 AND p.statut = 1{$whereStore}{$whereName}{$whereBatchNumber}{$whereFrom}{$whereTo}";
            $stocksTable = "
                SELECT s.id_stock AS stock_id
                FROM stock s JOIN magasin m ON s.id_magasin = m.id_magasin JOIN produit p ON s.id_produit = p.id_produit
                JOIN categorie_produit c ON p.id_categorie_produit = c.id_categorie
                WHERE {$where}
                ORDER BY s.id_magasin ASC, c.id_categorie ASC, p.designation ASC
            ";
            $stocksListStatement = $stockDao->entityManager->getConnection()->prepare($stocksTable);
            $stocksListStatement->execute();
            $stocksList = $stocksListStatement->fetchAll();
            $effectiveColspan = 9;
            if ($considerExpirationDate) $effectiveColspan++;
            $totalsColspan = 2;
            $currentStoreId = null;
            $currentStoreName = null;
            $currentCategoryId = null;
            $currentCategoryName = null;
            $currentStoreTotalQuantity = 0;
            $currentStoreTotalInitialQuantity = 0;
            $currentStoreTotalBuying = 0;
            $currentStoreTotalSale = 0;
            $currentStoreTotalProfit = 0;
            $currentCategoryTotalQuantity = 0;
            $currentCategoryTotalInitialQuantity = 0;
            $currentCategoryTotalBuying = 0;
            $currentCategoryTotalSale = 0;
            $currentCategoryTotalProfit = 0;
            $totalQuantity = 0;
            $totalInitialQuantity = 0;
            $totalBuying = 0;
            $totalSale = 0;
            $totalProfit = 0;
            foreach ($stocksList as $stockArray) {
                $stock = $stockDao->getStockById($stockArray["stock_id"]);
                if ($stock->getIdMagasin()->getIdMagasin() != $currentStoreId) {
                    if ($currentStoreId != null) {
                        if ($currentCategoryId != null) {
                            $categoryTotalBuyingSum = number_format($currentCategoryTotalBuying, 0, '', ' ');
                            $categoryTotalSaleSum = number_format($currentCategoryTotalSale, 0, '', ' ');
                            $categoryTotalProfitSum = number_format($currentCategoryTotalProfit, 0, '', ' ');
                            $documentHtml .= "
                                    <tr class='app-total-tr'>
                                        <td colspan='{$totalsColspan}' class='right aligned'>
                                            {$this->translator->_("report.printViewStocks.totals")}
                                            {$currentCategoryName}
                                        </td>
                                        <td>{$currentCategoryTotalQuantity}</td>
                            ";
                            if ($considerExpirationDate) {
                                $documentHtml .= "
                                        <td></td>
                                ";
                            }
                            $documentHtml .= "
                                        <td>{$currentCategoryTotalInitialQuantity}</td>
                                        <td colspan='2'></td>
                                        <td class='right aligned'>{$categoryTotalBuyingSum}</td>
                                        <td class='right aligned'>{$categoryTotalSaleSum}</td>
                                        <td class='right aligned'>{$categoryTotalProfitSum}</td>
                                    </tr>
                            ";
                        }
                        $storeTotalBuyingSum = number_format($currentStoreTotalBuying, 0, '', ' ');
                        $storeTotalSaleSum = number_format($currentStoreTotalSale, 0, '', ' ');
                        $storeTotalProfitSum = number_format($currentStoreTotalProfit, 0, '', ' ');
                        $documentHtml .= "
                                    <tr class='app-total-tr'>
                                        <td colspan='{$totalsColspan}' class='right aligned'>
                                            <i class='violet-label'>
                                                {$this->translator->_("report.printViewStocks.totals")}
                                                {$currentStoreName}
                                            </i>
                                        </td>
                                        <td>{$currentStoreTotalQuantity}</td>
                        ";
                        if ($considerExpirationDate) {
                            $documentHtml .= "
                                        <td></td>
                            ";
                        }
                        $documentHtml .= "
                                        <td>{$currentStoreTotalInitialQuantity}</td>
                                        <td colspan='2'></td>
                                        <td class='right aligned'>{$storeTotalBuyingSum}</td>
                                        <td class='right aligned'>{$storeTotalSaleSum}</td>
                                        <td class='right aligned'>{$storeTotalProfitSum}</td>
                                    </tr>
                        ";
                    }
                    if ($store == null) {
                        $documentHtml .= "
                                    <tr>
                                        <td colspan='{$effectiveColspan}' class='center-text'>
                                            <i class='violet-label'>
                                                {$stock->getIdMagasin()->getNomMagasin()}
                                            </i>
                                        </td>
                                    </tr>
                        ";
                    }
                    $currentStoreId = $stock->getIdMagasin()->getIdMagasin();
                    $currentStoreName = $stock->getIdMagasin()->getNomMagasin();
                    $currentStoreTotalQuantity = $stock->getQuantite();
                    $currentStoreTotalInitialQuantity = $stock->getQuantiteEntree();
                    $currentStoreTotalBuying = $stock->getQuantite() * $stock->getPrixAchat();
                    $currentStoreTotalSale = ($stock->getQuantiteEntree() - $stock->getQuantite()) * $stock->getPrixVente();
                    $currentStoreTotalProfit =  ($stock->getQuantiteEntree() - $stock->getQuantite()) * ($stock->getPrixVente() - $stock->getPrixAchat());
                    $currentCategoryId = null;
                    $currentCategoryName = null;
                    $currentCategoryTotalQuantity = 0;
                    $currentCategoryTotalInitialQuantity = 0;
                    $currentCategoryTotalBuying = 0;
                    $currentCategoryTotalSale = 0;
                    $currentCategoryTotalProfit = 0;
                } else {
                    $currentStoreTotalQuantity += $stock->getQuantite();
                    $currentStoreTotalInitialQuantity += $stock->getQuantiteEntree();
                    $currentStoreTotalBuying += $stock->getQuantite() * $stock->getPrixAchat();
                    $currentStoreTotalSale += ($stock->getQuantiteEntree() - $stock->getQuantite()) * $stock->getPrixVente();
                    $currentStoreTotalProfit +=  ($stock->getQuantiteEntree() - $stock->getQuantite()) * ($stock->getPrixVente() - $stock->getPrixAchat());
                }
                if ($stock->getIdProduit()->getIdCategorieProduit()->getIdCategorie() != $currentCategoryId) {
                    if ($currentCategoryId != null) {
                        $categoryTotalBuyingSum = number_format($currentCategoryTotalBuying, 0, '', ' ');
                        $categoryTotalSaleSum = number_format($currentCategoryTotalSale, 0, '', ' ');
                        $categoryTotalProfitSum = number_format($currentCategoryTotalProfit, 0, '', ' ');
                        $documentHtml .= "
                                    <tr class='app-total-tr'>
                                        <td colspan='{$totalsColspan}' class='right aligned'>
                                            {$this->translator->_("report.printViewStocks.totals")}
                                            {$currentCategoryName}
                                        </td>
                                        <td>{$currentCategoryTotalQuantity}</td>
                        ";
                        if ($considerExpirationDate) {
                            $documentHtml .= "
                                        <td></td>
                            ";
                        }
                        $documentHtml .= "
                                        <td>{$currentCategoryTotalInitialQuantity}</td>
                                        <td colspan='2'></td>
                                        <td class='right aligned'>{$categoryTotalBuyingSum}</td>
                                        <td class='right aligned'>{$categoryTotalSaleSum}</td>
                                        <td class='right aligned'>{$categoryTotalProfitSum}</td>
                                    </tr>
                        ";
                    }
                    $documentHtml .= "
                                    <tr class='app-highlight-tr'>
                                        <td colspan='{$effectiveColspan}' class='left aligned'>
                                            {$stock->getIdProduit()->getIdCategorieProduit()->getNom()}
                                        </td>
                                    </tr>
                    ";
                    $currentCategoryId = $stock->getIdProduit()->getIdCategorieProduit()->getIdCategorie();
                    $currentCategoryName = $stock->getIdProduit()->getIdCategorieProduit()->getNom();
                    $currentCategoryTotalQuantity = $stock->getQuantite();
                    $currentCategoryTotalInitialQuantity = $stock->getQuantiteEntree();
                    $currentCategoryTotalBuying = $stock->getQuantite() * $stock->getPrixAchat();
                    $currentCategoryTotalSale = ($stock->getQuantiteEntree() - $stock->getQuantite()) * $stock->getPrixVente();
                    $currentCategoryTotalProfit = ($stock->getQuantiteEntree() - $stock->getQuantite()) * ($stock->getPrixVente() - $stock->getPrixAchat());
                } else {
                    $currentCategoryTotalQuantity += $stock->getQuantite();
                    $currentCategoryTotalInitialQuantity += $stock->getQuantiteEntree();
                    $currentCategoryTotalBuying += $stock->getQuantite() * $stock->getPrixAchat();
                    $currentCategoryTotalSale += ($stock->getQuantiteEntree() - $stock->getQuantite()) * $stock->getPrixVente();
                    $currentCategoryTotalProfit += ($stock->getQuantiteEntree() - $stock->getQuantite()) * ($stock->getPrixVente() - $stock->getPrixAchat());
                }
                $totalQuantity += $stock->getQuantite();
                $totalInitialQuantity += $stock->getQuantiteEntree();
                $totalBuying += $stock->getQuantite() * $stock->getPrixAchat();
                $totalSale += ($stock->getQuantiteEntree() - $stock->getQuantite()) * $stock->getPrixVente();
                $totalProfit += ($stock->getQuantiteEntree() - $stock->getQuantite()) * ($stock->getPrixVente() - $stock->getPrixAchat());
                $documentHtml .= "
                                    <tr>
                                        <td>{$stock->getDesignation()}</td>
                                        <td>{$stock->getNumLot()}</td>
                                        <td class=''>{$stock->getQuantite()}</td>
                ";
                if ($considerExpirationDate) {
                    $stockExpirationDate = $stock->getDatePeremption() != null ? $stock->getDatePeremption()->format(DATE_FORMAT) : "";
                    if ($stock->getIdProduit()->getPerissable() == 1 && $stock->getDatePeremption() != null) {
                        $documentHtml .= "
                                        <td>{$stockExpirationDate}</td>
                        ";
                    } else {
                        $documentHtml .= "
                                        <td class='not-perishable'>{$this->translator->_("shared.notPerishable")}</td>
                        ";
                    }
                }
                $stockBuyingPrice = number_format($stock->getPrixAchat(), 0, '', ' ');
                $stockSalePrice = number_format($stock->getPrixVente(), 0, '', ' ');
                $stockTotalBuyingPrice = number_format($stock->getQuantite() * $stock->getPrixAchat(), 0, '', ' ');
                $stockTotalSalePrice = number_format(($stock->getQuantiteEntree() - $stock->getQuantite()) * $stock->getPrixVente(), 0, '', ' ');
                $stockTotalProfitPrice = number_format(($stock->getQuantiteEntree() - $stock->getQuantite()) * ($stock->getPrixVente() - $stock->getPrixAchat()), 0, '', ' ');
                $documentHtml .= "
                                        <td>{$stock->getQuantiteEntree()}</td>
                                        <td class='right aligned'>{$stockBuyingPrice}</td>
                                        <td class='right aligned'>{$stockSalePrice}</td>
                                        <td class='right aligned'>{$stockTotalBuyingPrice}</td>
                                        <td class='right aligned'>{$stockTotalSalePrice}</td>
                                        <td class='right aligned'>{$stockTotalProfitPrice}</td>
                                    </tr>
                ";
            }
            if ($currentStoreId != null) {
                if ($currentCategoryId != null) {
                    $categoryTotalBuyingSum = number_format($currentCategoryTotalBuying, 0, '', ' ');
                    $categoryTotalSaleSum = number_format($currentCategoryTotalSale, 0, '', ' ');
                    $categoryTotalProfitSum = number_format($currentCategoryTotalProfit, 0, '', ' ');
                    $documentHtml .= "
                                    <tr class='app-total-tr'>
                                        <td colspan='{$totalsColspan}' class='right aligned'>
                                            {$this->translator->_("report.printViewStocks.totals")}
                                            {$currentCategoryName}
                                        </td>
                                        <td>{$currentCategoryTotalQuantity}</td>
                    ";
                    if ($considerExpirationDate) {
                        $documentHtml .= "
                                        <td></td>
                        ";
                    }
                    $documentHtml .= "
                                        <td>{$currentCategoryTotalInitialQuantity}</td>
                                        <td colspan='2'></td>
                                        <td class='right aligned'>{$categoryTotalBuyingSum}</td>
                                        <td class='right aligned'>{$categoryTotalSaleSum}</td>
                                        <td class='right aligned'>{$categoryTotalProfitSum}</td>
                                    </tr>
                    ";
                }
                $storeTotalBuyingSum = number_format($currentStoreTotalBuying, 0, '', ' ');
                $storeTotalSaleSum = number_format($currentStoreTotalSale, 0, '', ' ');
                $storeTotalProfitSum = number_format($currentStoreTotalProfit, 0, '', ' ');
                $documentHtml .= "
                                    <tr class='app-total-tr'>
                                        <td colspan='{$totalsColspan}' class='right aligned'>
                                            <i class='violet-label'>
                                                {$this->translator->_("report.printViewStocks.totals")}
                                                {$currentStoreName}
                                            </i>
                                        </td>
                                        <td>{$currentStoreTotalQuantity}</td>
                ";
                if ($considerExpirationDate) {
                    $documentHtml .= "
                                        <td></td>
                    ";
                }
                $documentHtml .= "
                                        <td>{$currentStoreTotalInitialQuantity}</td>
                                        <td colspan='2'></td>
                                        <td class='right aligned'>{$storeTotalBuyingSum}</td>
                                        <td class='right aligned'>{$storeTotalSaleSum}</td>
                                        <td class='right aligned'>{$storeTotalProfitSum}</td>
                                    </tr>
                ";
            }
            if ($store == null) {
                $totalBuyingSum = number_format($totalBuying, 0, '', ' ');
                $totalSaleSum = number_format($totalSale, 0, '', ' ');
                $totalProfitSum = number_format($totalProfit, 0, '', ' ');
                $documentHtml .= "
                                    <tr class='app-total-tr'>
                                        <td colspan='$totalsColspan' class='right aligned'>
                                            <i class='secondary-label'>
                                                {$this->translator->_("report.printViewStocks.totals")}
                                            </i>
                                        </td>
                                        <td>{$totalQuantity}</td>
                ";
                if ($considerExpirationDate) {
                    $documentHtml .= "
                                        <td></td>
                    ";
                }
                $documentHtml .= "
                                        <td>{$totalInitialQuantity}</td>
                                        <td colspan='2'></td>
                                        <td class='right aligned'>{$totalBuyingSum}</td>
                                        <td class='right aligned'>{$totalSaleSum}</td>
                                        <td class='right aligned'>{$totalProfitSum}</td>
                                    </tr>
                ";
            }
            if (count($stocksList) == 0) {
                $documentHtml .= "
                                    <tr>
                                        <td colspan='{$effectiveColspan}'>
                                            {$this->translator->_("shared.noDataAvailable")}
                                        </td>
                                    </tr>
                ";
            }
            $documentHtml .= "
                                </tbody>
                            </table>
                        </div>
                    </body>
                </html> 
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "stocks_report_{$_SESSION[SESSION_USER_ID]}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/stocks/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/stocks/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printviewcumulativestocksAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/stocks") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $storeId = $_POST["store"] != "" ? (int)$_POST["store"] : null;
            $stockAtTheDateOf = $_POST["stockAtTheDateOf"];
            $name = $_POST["name"] != "" ? $_POST["name"] : null;
            $storeDao = new StoreDAO();
            $stockDao = new StockDAO();
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $secondaryBrand = APP_SECONDARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = $this->translator->_("report.printViewCumulativeStocks.cumulativeStocksReport");
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }
                            

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: left;  
                                color: #000000; 
                              
                            }

                            td.center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            bold {
                                color: black;
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }

                            i.violet-label {
                                background-color: #6435c9;
                                color: white;
                                padding: 2px;
                                border-radius: 6px;
                                margin: 3px;
                            }

                            i.secondary-label {
                                background-color: {$secondaryBrand};
                                color: white;
                                padding: 2px;
                                border-radius: 6px;
                                margin: 3px;
                            }

                            td.left.aligned {
                                text-align: left;
                            }

                            td.right.aligned {
                                text-align: right;
                            }

                            tr.app-total-tr {
                                background-color: {$primaryBrand}80;
                            }

                            tr.app-total-tr > td {
                                color: {$thirdBrand};
                            }

                            tr.app-highlight-tr {
                                background-color: {$secondaryBrand}80;
                            }

                            tr.app-highlight-tr > td {
                                color: {$thirdBrand};
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$this->translator->_("report.printViewCumulativeStocks.cumulativeStocksReport")}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table>
                                <thead>
                                    <tr>
            ";
            $store = $storeId != null ? $storeDao->getStoreById($storeId) : null;
            $storeName = $store != null ? $store->getNomMagasin() : $this->translator->_("report.printViewCumulativeStocks.allStores");
            $stockAtTheDateOfDateTime = new DateTime($stockAtTheDateOf);
            $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printViewCumulativeStocks.store")} : <bold class='black-bold'>{$storeName}</bold></th>
                                        <th class='center-text'>{$this->translator->_("report.printViewCumulativeStocks.date")} : <bold class='black-bold'>{$stockAtTheDateOfDateTime->format(DATE_FORMAT)}</bold></th>
            ";
            $printFilters .= "{$this->translator->_("report.printViewCumulativeStocks.store")} : {$storeName}";
            $printFilters .= ", {$this->translator->_("report.printViewCumulativeStocks.date")} : {$stockAtTheDateOfDateTime->format(DATE_FORMAT)}";
            if ($name != null) {
                $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printViewCumulativeStocks.productNameContaining")} : <bold class='black-bold'>{$name}</bold></th>
                ";
                $printFilters .= ", {$this->translator->_("report.printViewCumulativeStocks.productNameContaining")} : {$name}";
            }
            $documentHtml .= "
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border'>
                                <thead>
                                    <tr>
                                        <th>{$this->translator->_("report.printViewCumulativeStocks.designation")}</th>
                                        <th>{$this->translator->_("report.printViewCumulativeStocks.quantity")}</th>
                                        <th>{$this->translator->_("report.printViewCumulativeStocks.buyingPrice")}</th>
                                        <th>{$this->translator->_("report.printViewCumulativeStocks.salePrice")}</th>
                                        <th>{$this->translator->_("report.printViewCumulativeStocks.buyingTotal")}</th>
                                    </tr>
                                </thead>
                                <tbody>
            ";
            $whereStore = "";
            $whereName = "";
            if ($storeId != null) {
                $whereStore .= " AND m.id_magasin = {$storeId}";
            }
            if ($name != null) {
                $whereName .= " AND p.designation LIKE '%{$name}%'";
            }
            $where = "summary.quantity > 0 AND p.statut = 1 AND m.statut = 1{$whereStore}{$whereName}";
            $cumulativeStocksTable = "
                SELECT summary.cumulative_stock_id AS cumulative_stock_id, summary.product_id AS product_id, summary.store_id AS store_id,
                summary.quantity AS quantity, p.designation AS product_name, m.nom_magasin AS store_name, p.prix_achat AS buying_price, 
                p.prix_vente AS sale_price, p.quantite_alerte AS alert_quantity, c.id_categorie AS category_id, c.nom AS category_name
                FROM
                (
                    SELECT entries.cumulative_stock_id AS cumulative_stock_id, entries.product_id AS product_id, entries.store_id AS store_id,
                    (entries.entry_quantity - ifnull(exits.exit_quantity, 0)) AS quantity
                    FROM
                    (
                        SELECT CONCAT(p.id_produit, '_', m.id_magasin) AS cumulative_stock_id, p.id_produit AS product_id, SUM(lr.quantite) AS entry_quantity, m.id_magasin AS store_id
                        FROM ligne_reception lr JOIN produit p ON lr.id_produit = p.id_produit JOIN reception r ON r.id_reception = lr.id_reception
                        JOIN magasin m ON r.id_magasin = m.id_magasin
                        WHERE p.statut = 1 AND r.statut = 1 AND r.valider = 1 AND r.date_saisie <= '{$stockAtTheDateOf} 23:59:59'
                        GROUP BY p.id_produit, m.id_magasin
                    ) AS entries
                    LEFT OUTER JOIN
                    (
                        SELECT CONCAT(p.id_produit, '_', m.id_magasin) AS cumulative_stock_id, p.id_produit AS product_id, SUM(ls.quantite) AS exit_quantity, m.id_magasin AS store_id
                        FROM ligne_sortie ls JOIN stock st ON ls.id_stock = st.id_stock JOIN produit p ON st.id_produit = p.id_produit 
                        JOIN sortie so ON ls.id_sortie = so.id_sortie JOIN magasin m ON so.id_magasin = m.id_magasin
                        WHERE p.statut = 1 AND so.statut = 1 AND so.valider = 1 AND so.date_saisie <= '{$stockAtTheDateOf} 23:59:59'
                        GROUP BY p.id_produit, m.id_magasin
                    ) AS exits
                    ON (entries.cumulative_stock_id = exits.cumulative_stock_id OR exits.cumulative_stock_id IS NULL) 
                    AND (entries.product_id = exits.product_id OR exits.product_id IS NULL)
                    AND (entries.store_id = exits.store_id OR exits.store_id IS NULL)
                ) AS summary
                JOIN produit p ON summary.product_id = p.id_produit JOIN categorie_produit c ON p.id_categorie_produit = c.id_categorie
                JOIN magasin m ON summary.store_id = m.id_magasin
                WHERE {$where}
                ORDER BY store_id ASC, category_id ASC, product_name ASC
            ";
            $cumulativeStocksListStatement = $stockDao->entityManager->getConnection()->prepare($cumulativeStocksTable);
            $cumulativeStocksListStatement->execute();
            $cumulativeStocksList = $cumulativeStocksListStatement->fetchAll();
            $effectiveColspan = 5;
            $currentStoreId = null;
            $currentStoreName = null;
            $currentStoreTotalQuantity = 0;
            $currentStoreTotalBuying = 0;
            $currentCategoryId = null;
            $currentCategoryName = null;
            $currentCategoryTotalQuantity = 0;
            $currentCategoryTotalBuying = 0;
            $totalQuantity = 0;
            $totalBuying = 0;
            foreach ($cumulativeStocksList as $cumulativeStock) {
                if ((int)$cumulativeStock["store_id"] != $currentStoreId) {
                    if ($currentStoreId != null) {
                        if ($currentCategoryId != null) {
                            $categoryBuyingSum = number_format($currentCategoryTotalBuying, 0, '', ' ');
                            $documentHtml .= "
                                    <tr class='app-total-tr'>
                                        <td class='right aligned'>
                                            {$this->translator->_("report.printViewCumulativeStocks.totals")}
                                            {$currentCategoryName}
                                        </td>
                                        <td>{$currentCategoryTotalQuantity}</td>
                                        <td colspan='2'></td>
                                        <td class='right aligned'>{$categoryBuyingSum}</td>
                                    </tr>
                            ";
                        }
                        $storeBuyingSum = number_format($currentStoreTotalBuying, 0, '', ' ');
                        $documentHtml .= "
                                    <tr>
                                        <td class='right aligned'>
                                            <i class='violet-label'>
                                                {$this->translator->_("report.printViewCumulativeStocks.totals")}
                                                {$currentStoreName}
                                            </i>
                                        </td>
                                        <td>{$currentStoreTotalQuantity}</td>
                                        <td colspan='2'></td>
                                        <td class='right aligned'>{$storeBuyingSum}</td>
                                    </tr>
                        ";
                    }
                    if ($store == null) {
                        $documentHtml .= "
                                    <tr>
                                        <td colspan='{$effectiveColspan}' class='center-text'>
                                            <i class='violet-label'>
                                                {$cumulativeStock["store_name"]}
                                            </i>
                                        </td>
                                    </tr>
                        ";
                    }
                    $currentStoreId = (int)$cumulativeStock["store_id"];
                    $currentStoreName = $cumulativeStock["store_name"];
                    $currentStoreTotalQuantity = (int)$cumulativeStock["quantity"];
                    $currentStoreTotalBuying = (int)$cumulativeStock["quantity"] * (float)$cumulativeStock["buying_price"];
                    $currentCategoryId = null;
                    $currentCategoryName = null;
                    $currentCategoryTotalQuantity = 0;
                    $currentCategoryTotalBuying = 0;
                } else {
                    $currentStoreTotalQuantity += (int)$cumulativeStock["quantity"];
                    $currentStoreTotalBuying += (int)$cumulativeStock["quantity"] * (float)$cumulativeStock["buying_price"];
                }
                if ((int)$cumulativeStock["category_id"] != $currentCategoryId) {
                    if ($currentCategoryId != null) {
                        $categoryBuyingSum = number_format($currentCategoryTotalBuying, 0, '', ' ');
                        $documentHtml .= "
                                    <tr class='app-total-tr'>
                                        <td class='right aligned'>
                                            {$this->translator->_("report.printViewCumulativeStocks.totals")}
                                            {$currentCategoryName}
                                        </td>
                                        <td>{$currentCategoryTotalQuantity}</td>
                                        <td colspan='2'></td>
                                        <td class='right aligned'>{$categoryBuyingSum}</td>
                                    </tr>
                        ";
                    }
                    $documentHtml .= "
                                    <tr class='app-highlight-tr'>
                                        <td colspan='{$effectiveColspan}'>{$cumulativeStock["category_name"]}</td>
                                    </tr>
                    ";
                    $currentCategoryId = (int)$cumulativeStock["category_id"];
                    $currentCategoryName = $cumulativeStock["category_name"];
                    $currentCategoryTotalQuantity = (int)$cumulativeStock["quantity"];
                    $currentCategoryTotalBuying = (int)$cumulativeStock["quantity"] * (float)$cumulativeStock["buying_price"];
                } else {
                    $currentCategoryTotalQuantity += (int)$cumulativeStock["quantity"];
                    $currentCategoryTotalBuying += (int)$cumulativeStock["quantity"] * (float)$cumulativeStock["buying_price"];
                }
                $totalQuantity += (int)$cumulativeStock["quantity"];
                $totalBuying += (int)$cumulativeStock["quantity"] * (float)$cumulativeStock["buying_price"];
                $quantity = (int)$cumulativeStock["quantity"];
                $buyingPrice = number_format((float)$cumulativeStock["buying_price"], 0, '', ' ');
                $salePrice = number_format((float)$cumulativeStock["sale_price"], 0, '', ' ');
                $totalBuyingPrice = number_format((int)$cumulativeStock["quantity"] * (float)$cumulativeStock["buying_price"], 0, '', ' ');
                $documentHtml .= "
                                    <tr>
                                        <td style='text-align: left;'>{$cumulativeStock["product_name"]}</td>
                                        <td>{$quantity}</td>
                                        <td class='right aligned'>{$buyingPrice}</td>
                                        <td class='right aligned'>{$salePrice}</td>
                                        <td class='right aligned'>{$totalBuyingPrice}</td>
                                    </tr>
                ";
            }
            if ($currentStoreId != null) {
                if ($currentCategoryId != null) {
                    $categoryBuyingSum = number_format($currentCategoryTotalBuying, 0, '', ' ');
                    $documentHtml .= "
                                    <tr class='app-total-tr'>
                                        <td class='right aligned'>
                                            {$this->translator->_("report.printViewCumulativeStocks.totals")}
                                            {$currentCategoryName}
                                        </td>
                                        <td>{$currentCategoryTotalQuantity}</td>
                                        <td colspan='2'></td>
                                        <td class='right aligned'>{$categoryBuyingSum}</td>
                                    </tr>
                    ";
                }
                $storeBuyingSum = number_format($currentStoreTotalBuying, 0, '', ' ');
                $documentHtml .= "
                                    <tr>
                                        <td class='right aligned'>
                                            <i class='violet-label'>
                                                {$this->translator->_("report.printViewCumulativeStocks.totals")}
                                                {$currentStoreName}
                                            </i>
                                        </td>
                                        <td>{$currentStoreTotalQuantity}</td>
                                        <td colspan='2'></td>
                                        <td class='right aligned'>{$storeBuyingSum}</td>
                                    </tr>
                ";
            }
            if ($store == null) {
                $totalBuyingSum = number_format($totalBuying, 0, '', ' ');
                $documentHtml .= "
                                    <tr>
                                        <td class='right aligned'>
                                            <i class='secondary-label'>
                                                {$this->translator->_("report.printViewCumulativeStocks.totals")}
                                            </i>
                                        </td>
                                        <td>{$totalQuantity}</td>
                                        <td colspan='2'></td>
                                        <td class='right aligned'>{$totalBuyingSum}</td>
                                    </tr>
                ";
            }
            if (count($cumulativeStocksList) == 0) {
                $documentHtml .= "
                                    <tr>
                                        <td colspan='{$effectiveColspan}'>
                                            {$this->translator->_("shared.noDataAvailable")}
                                        </td>
                                    </tr>
                ";
            }
            $documentHtml .= " 
                                </tbody>
                            </table>
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "cumulative_stocks_report_{$_SESSION[SESSION_USER_ID]}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/stocks/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/stocks/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printviewproductsAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/products") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $name = $_POST["name"] != "" ? $_POST["name"] : null;
            $category = (int)$_POST["category"] != 0 ? (int)$_POST["category"] : null;
            $reference = $_POST["reference"] != "" ? $_POST["reference"] : null;
            $type = $_POST["type"] != "" ? (int)$_POST["type"] : null; //1 products, 0 services else products and services
            $productDao = new ProductDAO();
            $productCategoryDao = new ProductCategoryDAO();
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = "";
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: center;  
                                color: #000000; 
                            }

                            .center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            bold {
                                color: black;
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
            ";
            if (is_null($type)) { //all products and services
                $documentHtml .= "
                                    <tr>
                                        <td class='title'>{$this->translator->_("report.printViewProducts.productsAndServicesReport")}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                ";
                $printTitle = $this->translator->_("report.printViewProducts.productsAndServicesReport");
            }
            if (!is_null($type) && $type == 1) { //all products
                $documentHtml .= "
                                    <tr>
                                        <td class='title'>{$this->translator->_("report.printViewProducts.productsReport")}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                ";
                $printTitle = $this->translator->_("report.printViewProducts.productsReport");
            }
            if (!is_null($type) && $type == 0) { //all services
                $documentHtml .= "
                                    <tr>
                                        <td class='title'>{$this->translator->_("report.printViewProducts.servicesReport")}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                ";
                $printTitle = $this->translator->_("report.printViewProducts.servicesReport");
            }
            if (!is_null($category) || !is_null($name) || !is_null($reference)) {
                $documentHtml .= "
                            <table>
                                <thead>
                                    <tr>
                ";
                if (!is_null($category)) {
                    $selectedCategory = $productCategoryDao->getProductCategoryById($category);
                    $documentHtml .= "
                                    <th class='center-text'>{$this->translator->_("report.printViewProducts.category")} : <bold class='black-bold'>{$selectedCategory->getNom()}</bold></th>
                    ";
                    $printFilters .= "{$this->translator->_("report.printViewProducts.category")} : {$selectedCategory->getNom()}";
                }
                if (!is_null($name)) {
                    $documentHtml .= "
                                    <th class='center-text'>{$this->translator->_("report.printViewProducts.designationContaining")} : <bold class='black-bold'>{$name}</bold></th>
                    ";
                    if (!is_null($category)) $printFilters .= ", ";
                    $printFilters .= "{$this->translator->_("report.printViewProducts.designationContaining")} : {$name}";
                }
                if (!is_null($reference)) {
                    $documentHtml .= "
                                    <th class='center-text'>{$this->translator->_("report.printViewProducts.referenceContaining")} : <bold class='black-bold'>{$reference}</bold></th>
                    ";
                    if (!is_null($category) || !is_null($name)) $printFilters .= ", ";
                    $printFilters .= "{$this->translator->_("report.printViewProducts.referenceContaining")} : {$reference}";
                }
                $documentHtml .= "
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                ";
            } else {
                $printFilters = $this->translator->_("report.noFilter");
            }
            $documentHtml .= "
                            <table class='border'>
                                <thead>
                                    <tr>													   												   
                                        <th>{$this->translator->_("report.printViewProducts.designation")}</th>
                                        <th>{$this->translator->_("report.printViewProducts.buyingPrice")}</th>
                                        <th>{$this->translator->_("report.printViewProducts.salePrice")}</th>
                                        <th>{$this->translator->_("report.printViewProducts.margin")}</th>                                               
                                    </tr>
                                </thead>
                                <tbody>
            ";
            $whereName = " AND p.designation LIKE '%{$name}%'";
            $whereCategory = "";
            $whereReference = "";
            $whereType = "";
            if (!is_null($name)) {
                $whereName .= " AND p.designation LIKE '%{$name}%'";
            }
            if (!is_null($category)) {
                $whereCategory .= " AND p.id_categorie_produit = {$category}";
            }
            if (!is_null($reference)) {
                $whereReference .= " AND p.reference_produit LIKE '%{$reference}%'";
            }
            if (!is_null($type)) {
                $whereType .= " AND p.stockable = {$type}";
            }
            $where = "p.statut = 1{$whereName}{$whereCategory}{$whereReference}{$whereType}";
            $productsTable = "
                SELECT p.id_produit AS product_id, c.id_categorie AS category_id
                FROM produit p JOIN categorie_produit c ON p.id_categorie_produit = c.id_categorie 
                WHERE {$where}
                ORDER BY c.id_categorie ASC, p.designation ASC
            ";
            $productsStatement = $productDao->entityManager->getConnection()->prepare($productsTable);
            $productsStatement->execute();
            $productsList = $productsStatement->fetchAll();
            $currentCategoryId = null;
            foreach ($productsList as $productArray) {
                if ((int)$productArray["category_id"] != $currentCategoryId && is_null($category)) {
                    $productCategory = $productCategoryDao->getProductCategoryById((int)$productArray["category_id"]);
                    $documentHtml .= "
                                    <tr>
                                        <td colspan='4' style='font-weight: bold;'>
                                            {$productCategory->getNom()}
                                        </td>
                                    </tr>
                    ";
                    $currentCategoryId = (int)$productArray["category_id"];
                }
                $product = $productDao->getProductById((int)$productArray["product_id"]);
                $buyingPrice = number_format($product->getPrixAchat(), 0, '', ' ');
                $salePrice = number_format($product->getPrixVente(), 0, '', ' ');
                $margin = number_format($product->getMarge(), 0, '', ' ');
                $documentHtml .= "
                                    <tr>
                                        <td style='text-align:left;'>{$product->getDesignation()}</td>
                                        <td style='text-align:right;'>{$buyingPrice}</td>
                                        <td style='text-align:right;'>{$salePrice}</td>
                                        <td style='text-align:right;'>{$margin}</td>
                                    </tr>
                    ";
            }
            if (count($productsList) == 0) {
                $documentHtml .= "
                                    <tr>
                                        <td colspan='4'>
                                            {$this->translator->_("shared.noDataAvailable")}
                                        </td>
                                    </tr>
                ";
            }
            $documentHtml .= "
                                </tbody>
                            </table>
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "product_services_report_{$_SESSION[SESSION_USER_ID]}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/products/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/products/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage() . $ex->getLine() . $ex->getFile());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printviewordersAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/orders") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $supplierId = $_POST["supplier"] != "" ? (int)$_POST["supplier"] : null;
            $from = $_POST["from"] != "" ? new DateTime($_POST["from"]) : null;
            $to = $_POST["to"] != "" ? new DateTime($_POST["to"]) : null;
            $supplierDao = new SupplierDAO();
            $orderDao = new OrderDAO();
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $secondaryBrand = APP_SECONDARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = $this->translator->_("report.printViewOrders.ordersReport");
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }
                            

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: center;  
                                color: #000000; 
                              
                            }

                            .center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            td.validated {
                                color: {$primaryBrand} !important;
                                font-weight: bold;
                            }

                            td.not-validated {
                                color: {$secondaryBrand} !important;
                                font-weight: bold;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            bold {
                                color: black;
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$this->translator->_("report.printViewOrders.ordersReport")}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table>
                                <thead>
                                    <tr>
            ";
            $supplier = $supplierId != null ? $supplierDao->getSupplierById($supplierId) : null;
            $supplierName = $supplier != null ? $supplier->getNomFournisseur() : $this->translator->_("report.printViewOrders.allSuppliers");
            $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printViewOrders.supplier")} : <bold class='black-bold'>{$supplierName}</bold></th>
            ";
            $printFilters .= "{$this->translator->_("report.printViewOrders.supplier")} : {$supplierName}";
            if ($from != null) {
                $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printViewOrders.dateAfter")} : <bold class='black-bold'>{$from->format(DATE_FORMAT)}</bold></th>
                ";
                $printFilters .= ", {$this->translator->_("report.printViewOrders.dateAfter")} : {$from->format(DATE_FORMAT)}";
            }
            if ($to != null) {
                $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printViewOrders.dateBefore")} : <bold class='black-bold'>{$to->format(DATE_FORMAT)}</bold></th>
                ";
                $printFilters .= ", {$this->translator->_("report.printViewOrders.dateBefore")} : {$to->format(DATE_FORMAT)}";
            }
            $documentHtml .= "
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border'>
                                <thead>
                                    <tr>
                                        <th>{$this->translator->_("report.printViewOrders.number")}</th>
                                        <th>{$this->translator->_("report.printViewOrders.orderDate")}</th>
                                        <th>{$this->translator->_("report.printViewOrders.wishedDeliveryDate")}</th>
                                        <th>{$this->translator->_("report.printViewOrders.entryDate")}</th>
                                        <th>{$this->translator->_("report.printViewOrders.total")}</th>
                                        <th>{$this->translator->_("report.printViewOrders.status")}</th>
                                    </tr>
                                </thead>
                                <tbody>
            ";
            $whereSupplier = "";
            $whereFrom = "";
            $whereTo = "";
            if ($supplierId != null) {
                $whereSupplier .= " AND c.id_fournisseur = {$supplierId}";
            }
            if ($from != null) {
                $whereFrom .= " AND c.date_commande >= '{$from->format(DATE_FORMAT)} 00:00:00'";
            }
            if ($to != null) {
                $whereTo .= " AND c.date_commande <= '{$to->format(DATE_FORMAT)} 23:59:59'";
            }
            $where = "c.statut = 1{$whereSupplier}{$whereFrom}{$whereTo}";
            $ordersTable = "
                    SELECT c.id_commande AS order_id
                    FROM commande c JOIN fournisseur f ON c.id_fournisseur = f.id_fournisseur
                    WHERE {$where}
                    ORDER BY c.id_fournisseur ASC, c.id_commande ASC
            ";
            $ordersStatement = $orderDao->entityManager->getConnection()->prepare($ordersTable);
            $ordersStatement->execute();
            $ordersList = $ordersStatement->fetchAll();
            $currentSupplierId = null;
            foreach ($ordersList as $orderTable) {
                $order = $orderDao->getOrderById((int)$orderTable["order_id"]);
                if ($supplier == null && $order->getIdFournisseur()->getIdFournisseur() !== $currentSupplierId) {
                    $effectiveColspan = 6;
                    $documentHtml .= "
                                    <tr>
                                        <td colspan='{$effectiveColspan}' class='black-bold'>
                                            {$order->getIdFournisseur()->getNomFournisseur()}
                                        </td>
                                    </tr>
                    ";
                    $currentSupplierId = $order->getIdFournisseur()->getIdFournisseur();
                }
                $total = number_format($order->getMontantTtc(), 0, '', ' ');
                $orderStatus = $this->translator->_($order->getValider() == 1 ? "report.printViewOrders.validated" : "report.printViewOrders.notValidated");
                $orderStatusClassName = $order->getValider() == 1 ? 'validated' : 'not-validated';
                $deliveryDate = $order->getDateLivraison() != null ? $order->getDateLivraison()->format(DATE_FORMAT) : $this->translator->_("shared.undefined");
                $documentHtml .= "
                                    <tr>
                                        <td>{$order->getIdCommande()}</td>
                                        <td>{$order->getDateCommande()->format(DATE_FORMAT)}</td>
                                        <td>{$deliveryDate}</td>
                                        <td>{$order->getDateSaisie()->format(DATE_FORMAT_WITH_SECONDS)}</td>
                                        <td style='text-align: right;'>{$total}</td>
                                        <td class='{$orderStatusClassName}'>{$orderStatus}</td>
                                    </tr>
                ";
            }
            if (count($ordersList) == 0) {
                $effectiveColspan = 6;
                $documentHtml .= "
                                    <tr>
                                        <td colspan='{$effectiveColspan}'>
                                            {$this->translator->_("shared.noDataAvailable")}
                                        </td>
                                    </tr>
                ";
            }
            $documentHtml .= " 
                                </tbody>
                            </table>
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "orders_report_{$_SESSION[SESSION_USER_ID]}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/orders/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/orders/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printviewreceptionsAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/receptions") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $storeId = $_POST["store"] != "" ? (int)$_POST["store"] : null;
            $supplierId = $_POST["supplier"] != "" ? (int)$_POST["supplier"] : null;
            $from = $_POST["from"] != "" ? new DateTime($_POST["from"]) : null;
            $to = $_POST["to"] != "" ? new DateTime($_POST["to"]) : null;
            $storeDao = new StoreDAO();
            $supplierDao = new SupplierDAO();
            $receptionDao = new ReceptionDAO();
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $secondaryBrand = APP_SECONDARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = $this->translator->_("report.printViewReceptions.receptionsReport");
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: center;  
                                color: #000000; 
                              
                            }

                            .center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            td.validated {
                                color: {$primaryBrand} !important;
                                font-weight: bold;
                            }

                            td.not-validated {
                                color: {$secondaryBrand} !important;
                                font-weight: bold;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            bold {
                                color: black;
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$this->translator->_("report.printViewReceptions.receptionsReport")}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table>
                                <thead>
                                    <tr>
            ";
            $store = $storeId != null ? $storeDao->getStoreById($storeId) : null;
            $storeName = $store != null ? $store->getNomMagasin() : $this->translator->_("report.printViewReceptions.allStores");
            $supplier = $supplierId != null ? $supplierDao->getSupplierById($supplierId) : null;
            $supplierName = $supplier != null ? $supplier->getNomFournisseur() : $this->translator->_("report.printViewReceptions.allSuppliers");
            $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printViewReceptions.store")} : <bold class='black-bold'>{$storeName}</bold></th>
                                        <th class='center-text'>{$this->translator->_("report.printViewReceptions.supplier")} : <bold class='black-bold'>{$supplierName}</bold></th>
            ";
            $printFilters .= "{$this->translator->_("report.printViewReceptions.store")} : {$storeName}";
            $printFilters .= ", {$this->translator->_("report.printViewReceptions.supplier")} : {$supplierName}";
            if ($from != null) {
                $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printViewReceptions.dateAfter")} : <bold class='black-bold'>{$from->format(DATE_FORMAT)}</bold></th>
                ";
                $printFilters .= ", {$this->translator->_("report.printViewReceptions.dateAfter")} : {$from->format(DATE_FORMAT)}";
            }
            if ($to != null) {
                $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printViewReceptions.dateBefore")} : <bold class='black-bold'>{$to->format(DATE_FORMAT)}</bold></th>
                ";
                $printFilters .= ", {$this->translator->_("report.printViewReceptions.dateBefore")} : {$to->format(DATE_FORMAT)}";
            }
            $documentHtml .= "
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border'>
                                <thead>
                                    <tr>
                                        <th colspan='2'>{$this->translator->_("report.printViewReceptions.number")}</th>
                                        <th colspan='2'>{$this->translator->_("report.printViewReceptions.entryDate")}</th>
                                        <th colspan='2'>{$this->translator->_("report.printViewReceptions.receivingDate")}</th>
                                        <th colspan='2'>{$this->translator->_("report.printViewReceptions.deliveryNoteNumber")}</th>
                                        <th colspan='2'>{$this->translator->_("report.printViewReceptions.status")}</th>
                                    </tr>
                                </thead>
                                <tbody>
            ";
            $whereStore = "";
            $whereSupplier = "";
            $whereFrom = "";
            $whereTo = "";
            if ($storeId != null) {
                $whereStore .= " AND r.id_magasin = {$storeId}";
            }
            if ($supplierId != null) {
                $whereSupplier .= " AND r.id_fournisseur = {$supplierId}";
            }
            if ($from != null) {
                $whereFrom .= " AND r.date_reception >= '{$from->format(DATE_FORMAT)} 00:00:00'";
            }
            if ($to != null) {
                $whereTo .= " AND r.date_reception <= '{$to->format(DATE_FORMAT)} 23:59:59'";
            }
            $where = "r.statut = 1{$whereStore}{$whereSupplier}{$whereFrom}{$whereTo}";
            $receptionsTable = "
                SELECT r.id_reception AS reception_id
                FROM reception r JOIN magasin m ON r.id_magasin = m.id_magasin JOIN fournisseur f ON r.id_fournisseur = f.id_fournisseur
                WHERE {$where}
                ORDER BY r.id_magasin ASC, r.id_fournisseur ASC
            ";
            $receptionsStatement = $receptionDao->entityManager->getConnection()->prepare($receptionsTable);
            $receptionsStatement->execute();
            $receptionsList = $receptionsStatement->fetchAll();
            $currentStoreId = null;
            $currentSupplierId = null;
            foreach ($receptionsList as $receptionArray) {
                $reception = $receptionDao->getReceptionById($receptionArray["reception_id"]);
                if (($store == null || $supplier == null) && ($reception->getIdMagasin()->getIdMagasin() != $currentStoreId || $reception->getIdFournisseur()->getIdFournisseur() != $currentSupplierId)) {
                    $effectiveColspan = 10;
                    $documentHtml .= "
                                    <tr>
                    ";
                    if ($store == null && $supplier == null) $effectiveColspan /= 2;
                    if ($store == null) {
                        $documentHtml .= "
                                        <td colspan='{$effectiveColspan}' class='black-bold'>
                                            {$this->translator->_("report.printViewReceptions.store")} : {$reception->getIdMagasin()->getNomMagasin()}
                                        </td>
                        ";
                    }
                    if ($supplier == null) {
                        $documentHtml .= "
                                        <td colspan='{$effectiveColspan}' class='black-bold'>
                                            {$this->translator->_("report.printViewReceptions.supplier")} : {$reception->getIdFournisseur()->getNomFournisseur()}
                                        </td>
                        ";
                    }
                    $documentHtml .= "
                                    </tr>
                    ";
                    $currentStoreId = $reception->getIdMagasin()->getIdMagasin();
                    $currentSupplierId = $reception->getIdFournisseur()->getIdFournisseur();
                }
                $receptionStatus = $this->translator->_($reception->getValider() == 1 ? "report.printViewReceptions.validated" : "report.printViewReceptions.notValidated");
                $receptionStatusClassName = $reception->getValider() == 1 ? 'validated' : 'not-validated';
                $documentHtml .= "
                                    <tr>
                                        <td colspan='2'>{$reception->getIdReception()}</td>
                                        <td colspan='2'>{$reception->getDateSaisie()->format(DATE_FORMAT)}</td>
                                        <td colspan='2'>{$reception->getDateReception()->format(DATE_FORMAT)}</td>
                                        <td colspan='2'>{$reception->getNumBl()}</td>
                                        <td colspan='2' class='{$receptionStatusClassName}'>{$receptionStatus}</td>
                                    </tr>
                ";
            }
            if (count($receptionsList) == 0) {
                $effectiveColspan = 10;
                $documentHtml .= "
                                    <tr>
                                        <td colspan='{$effectiveColspan}'>
                                            {$this->translator->_("shared.noDataAvailable")}
                                        </td>
                                    </tr>
                ";
            }
            $documentHtml .= " 
                                </tbody>
                            </table>
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "receptions_report_{$_SESSION[SESSION_USER_ID]}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/receptions/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/receptions/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printviewitemstoorderAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/stocks") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $storeId = $_POST["store"] != "" ? (int)$_POST["store"] : null;
            $name = $_POST["name"] != "" ? $_POST["name"] : null;
            $batchNumber = $_POST["batchNumber"] != "" ? $_POST["batchNumber"] : null;
            $storeDao = new StoreDAO();
            $stockDao = new StockDAO();
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = $this->translator->_("report.printViewItemsToOrder.itemsToOrderReport");
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: center;  
                                color: #000000; 
                              
                            }

                            .center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            bold {
                                color: black;
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$this->translator->_("report.printViewItemsToOrder.itemsToOrderReport")}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table>
                                <thead>
                                    <tr>
            ";
            $store = $storeId != null ? $storeDao->getStoreById($storeId) : null;
            $storeName = $store != null ? $store->getNomMagasin() : $this->translator->_("report.printViewItemsToOrder.allStores");
            $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printViewItemsToOrder.store")} : <bold class='black-bold'>{$storeName}</bold></th>
            ";
            $printFilters .= "{$this->translator->_("report.printViewItemsToOrder.store")} : {$storeName}";
            if ($name != null) {
                $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printViewItemsToOrder.productNameContaining")} : <bold class='black-bold'>{$name}</bold></th>
                ";
                $printFilters .= ", {$this->translator->_("report.printViewItemsToOrder.productNameContaining")} : {$name}";
            }
            if ($batchNumber != null) {
                $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printViewItemsToOrder.batchNumberContaining")} : <bold class='black-bold'>{$batchNumber}</bold></th>
                ";
                $printFilters .= ", {$this->translator->_("report.printViewItemsToOrder.batchNumberContaining")} : {$batchNumber}";
            }
            $documentHtml .= "
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border'>
                                <thead>
                                    <tr>
                                        <th>{$this->translator->_("report.printViewItemsToOrder.product")}</th>
                                        <th>{$this->translator->_("report.printViewItemsToOrder.batchNumber")}</th>
                                        <th>{$this->translator->_("report.printViewItemsToOrder.quantity")}</th>
                                    </tr>
                                </thead>
                                <tbody>
            ";
            $whereStore = "";
            $whereName = "";
            $whereBatchNumber = "";
            if ($storeId != null) {
                $whereStore .= " AND s.id_magasin = {$storeId}";
            }
            if ($name != null) {
                $whereName .= " AND s.designation LIKE  '%{$name}%'";
            }
            if ($batchNumber != null) {
                $whereBatchNumber .= " AND s.num_lot LIKE '%{$batchNumber}%'";
            }
            $where = "s.statut = 1 AND p.statut = 1 AND s.quantite > 0 AND s.quantite <= ifnull(p.quantite_alerte, 0){$whereStore}{$whereName}{$whereBatchNumber}";
            $atThresholdStocksTable = "
                    SELECT s.id_produit AS stock_id, s.designation AS product_name, s.num_lot AS batch_number, s.quantite AS quantity,
                    m.nom_magasin AS store_name, s.id_magasin AS store_id
                    FROM stock s JOIN produit p ON s.id_produit = p.id_produit JOIN magasin m ON s.id_magasin = m.id_magasin
                    WHERE {$where}
                    ORDER BY store_id ASC, product_name ASC
            ";
            $atThresholdStocksListStatement = $stockDao->entityManager->getConnection()->prepare($atThresholdStocksTable);
            $atThresholdStocksListStatement->execute();
            $atThresholdList = $atThresholdStocksListStatement->fetchAll();
            $currentStoreId = null;
            foreach ($atThresholdList as $thresholdStock) {
                if ($store == null && (int)$thresholdStock["store_id"] != $currentStoreId) {
                    $effectiveColspan = 3;
                    $documentHtml .= "
                                    <tr>
                                        <td colspan='{$effectiveColspan}' class='black-bold'>
                                            {$thresholdStock["store_name"]}
                                        </td>
                                    </tr>
                    ";
                    $currentStoreId = (int)$thresholdStock["store_id"];
                }
                $batchNumber = $thresholdStock["batch_number"];
                $quantity = (int)$thresholdStock["quantity"];
                $documentHtml .= "
                                    <tr>
                                        <td style='text-align: left;'>{$thresholdStock["product_name"]}</td>
                                        <td>{$batchNumber}</td>
                                        <td>{$quantity}</td>
                                    </tr>
                ";
            }
            if (count($atThresholdList) == 0) {
                $effectiveColspan = 3;
                $documentHtml .= "
                                    <tr>
                                        <td colspan='{$effectiveColspan}'>
                                            {$this->translator->_("shared.noDataAvailable")}
                                        </td>
                                    </tr>
                ";
            }
            $documentHtml .= " 
                                </tbody>
                            </table>
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "items_to_order_report_{$_SESSION[SESSION_USER_ID]}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/stocks/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/stocks/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printviewsoontoexpireitemsAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/stocks") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $storeId = $_POST["store"] != "" ? (int)$_POST["store"] : null;
            $name = $_POST["name"] != "" ? $_POST["name"] : null;
            $batchNumber = $_POST["batchNumber"] != "" ? $_POST["batchNumber"] : null;
            $storeDao = new StoreDAO();
            $stockDao = new StockDAO();
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = $this->translator->_("report.printViewSoonToExpireItems.soonToExpiredItemsReport");
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }
                            

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: center;  
                                color: #000000; 
                              
                            }

                            .center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            bold {
                                color: black;
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$this->translator->_("report.printViewSoonToExpireItems.soonToExpiredItemsReport")}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table>
                                <thead>
                                    <tr>
            ";
            $store = $storeId != null ? $storeDao->getStoreById($storeId) : null;
            $storeName = $store != null ? $store->getNomMagasin() : $this->translator->_("report.printViewSoonToExpireItems.allStores");
            $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printViewSoonToExpireItems.store")} : <bold class='black-bold'>{$storeName}</bold></th>
            ";
            $printFilters .= "{$this->translator->_("report.printViewSoonToExpireItems.store")} : {$storeName}";
            if ($name != null) {
                $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printViewSoonToExpireItems.productNameContaining")} : <bold class='black-bold'>{$name}</bold></th>
                ";
                $printFilters .= ", {$this->translator->_("report.printViewSoonToExpireItems.productNameContaining")} : {$name}";
            }
            if ($batchNumber != null) {
                $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printViewSoonToExpireItems.batchNumberContaining")} : <bold class='black-bold'>{$batchNumber}</bold></th>
                ";
                $printFilters .= ", {$this->translator->_("report.printViewSoonToExpireItems.batchNumberContaining")} : {$batchNumber}";
            }
            $documentHtml .= "
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border'>
                                <thead>
                                    <tr>
                                        <th>{$this->translator->_("report.printViewSoonToExpireItems.product")}</th>
                                        <th>{$this->translator->_("report.printViewSoonToExpireItems.batchNumber")}</th>
                                        <th>{$this->translator->_("report.printViewSoonToExpireItems.expirationDate")}</th>
                                        <th>{$this->translator->_("report.printViewSoonToExpireItems.quantity")}</th>
                                    </tr>
                                </thead>
                                <tbody>
            ";
            $currentDate = new DateTime();
            $limitExpirationDate = new DateTime();
            $daysForExpirationAlert = (int)NOMBRE_JOUR_POUR_ALERTE_PEREMPTION;
            $limitExpirationDate->modify("+ {$daysForExpirationAlert} days");
            $store = (int)$_POST["store"] != 0 ? (int)$_POST["store"] : null;
            $name = $_POST["name"] != "" ? $_POST["name"] : null;
            $batchNumber = $_POST["batchNumber"] != "" ? $_POST["batchNumber"] : null;
            $whereStore = "";
            $whereName = "";
            $whereBatchNumber = "";
            if ($store != null) {
                $whereStore .= " AND s.id_magasin = {$store}";
            }
            if ($name != null) {
                $whereName .= " AND s.designation LIKE  '%{$name}%'";
            }
            if ($batchNumber != null) {
                $whereBatchNumber .= " AND s.num_lot LIKE '%{$batchNumber}%'";
            }
            $whereExpirationDate = " AND s.date_peremption > '{$currentDate->format(DATE_FORMAT_WITH_TIME)}' AND s.date_peremption <= '{$limitExpirationDate->format(DATE_FORMAT)} 23:59:59'";
            $where = "s.statut = 1 AND p.statut = 1 AND s.quantite > 0{$whereStore}{$whereName}{$whereBatchNumber}{$whereExpirationDate}";
            $soonToExpireStocksTable = "
                    SELECT s.id_produit AS stock_id, s.designation AS product_name, s.num_lot AS batch_number, s.quantite AS quantity,
                    m.nom_magasin AS store_name, s.date_peremption AS expiration_date, s.id_magasin AS store_id
                    FROM stock s JOIN produit p ON s.id_produit = p.id_produit JOIN magasin m ON s.id_magasin = m.id_magasin
                    WHERE {$where}
                    ORDER BY store_id ASC, product_name ASC
            ";
            $soonToExpireStocksListStatement = $stockDao->entityManager->getConnection()->prepare($soonToExpireStocksTable);
            $soonToExpireStocksListStatement->execute();
            $soonToExpireStockList = $soonToExpireStocksListStatement->fetchAll();
            $currentStoreId = null;
            foreach ($soonToExpireStockList as $soonToExpireStock) {
                if ($store == null && (int)$soonToExpireStock["store_id"] != $currentStoreId) {
                    $effectiveColspan = 4;
                    $documentHtml .= "
                                    <tr>
                                        <td colspan='{$effectiveColspan}' class='black-bold'>
                                            {$soonToExpireStock["store_name"]}
                                        </td>
                                    </tr>
                    ";
                    $currentStoreId = (int)$soonToExpireStock["store_id"];
                }
                $batchNumber = $soonToExpireStock["batch_number"];
                $expirationDate = new DateTime($soonToExpireStock["expiration_date"]);
                $quantity = (int)$soonToExpireStock["quantity"];
                $documentHtml .= "
                                    <tr>
                                        <td style='text-align: left;'>{$soonToExpireStock["product_name"]}</td>
                                        <td>{$batchNumber}</td>
                                        <td>{$expirationDate->format(DATE_FORMAT)}</td>
                                        <td>{$quantity}</td>
                                    </tr>
                ";
            }
            if (count($soonToExpireStockList) == 0) {
                $effectiveColspan = 4;
                $documentHtml .= "
                                    <tr>
                                        <td colspan='{$effectiveColspan}'>
                                            {$this->translator->_("shared.noDataAvailable")}
                                        </td>
                                    </tr>
                ";
            }
            $documentHtml .= " 
                                </tbody>
                            </table>
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "soon_to_expire_items_report_{$_SESSION[SESSION_USER_ID]}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/stocks/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/stocks/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printviewexpireditemsAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/stocks") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $storeId = $_POST["store"] != "" ? (int)$_POST["store"] : null;
            $name = $_POST["name"] != "" ? $_POST["name"] : null;
            $batchNumber = $_POST["batchNumber"] != "" ? $_POST["batchNumber"] : null;
            $storeDao = new StoreDAO();
            $stockDao = new StockDAO();
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = $this->translator->_("report.printViewExpiredItems.expiredItemsReport");
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }
                            

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: center;  
                                color: #000000; 
                              
                            }

                            .center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            bold {
                                color: black;
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$this->translator->_("report.printViewExpiredItems.expiredItemsReport")}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table>
                                <thead>
                                    <tr>
            ";
            $store = $storeId != null ? $storeDao->getStoreById($storeId) : null;
            $storeName = $store != null ? $store->getNomMagasin() : $this->translator->_("report.printViewExpiredItems.allStores");
            $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printViewExpiredItems.store")} : <bold class='black-bold'>{$storeName}</bold></th>
            ";
            $printFilters .= "{$this->translator->_("report.printViewExpiredItems.store")} : {$storeName}";
            if ($name != null) {
                $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printViewExpiredItems.productNameContaining")} : <bold class='black-bold'>{$name}</bold></th>
                ";
                $printFilters .= ", {$this->translator->_("report.printViewExpiredItems.productNameContaining")} : {$name}";
            }
            if ($batchNumber != null) {
                $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printViewExpiredItems.batchNumberContaining")} : <bold class='black-bold'>{$batchNumber}</bold></th>
                ";
                $printFilters .= ", {$this->translator->_("report.printViewExpiredItems.batchNumberContaining")} : $batchNumber}";
            }
            $documentHtml .= "
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border'>
                                <thead>
                                    <tr>
                                        <th>{$this->translator->_("report.printViewExpiredItems.product")}</th>
                                        <th>{$this->translator->_("report.printViewExpiredItems.batchNumber")}</th>
                                        <th>{$this->translator->_("report.printViewExpiredItems.expirationDate")}</th>
                                        <th>{$this->translator->_("report.printViewExpiredItems.quantity")}</th>
                                    </tr>
                                </thead>
                                <tbody>
            ";
            $currentDate = new DateTime();
            $whereStore = "";
            $whereName = "";
            $whereBatchNumber = "";
            if ($store != null) {
                $whereStore .= " AND s.id_magasin = {$storeId}";
            }
            if ($name != null) {
                $whereName .= " AND s.designation LIKE  '%{$name}%'";
            }
            if ($batchNumber != null) {
                $whereBatchNumber .= " AND s.num_lot LIKE '%{$batchNumber}%'";
            }
            $whereExpirationDate = " AND s.date_peremption <= '{$currentDate->format(DATE_FORMAT_WITH_TIME)}'";
            $where = "s.statut = 1 AND p.statut = 1 AND s.quantite > 0{$whereStore}{$whereName}{$whereBatchNumber}{$whereExpirationDate}";
            $expiredStocksTable = "
                    SELECT s.id_produit AS stock_id, s.designation AS product_name, s.num_lot AS batch_number, s.quantite AS quantity,
                    m.nom_magasin AS store_name, s.date_peremption AS expiration_date, s.id_magasin AS store_id
                    FROM stock s JOIN produit p ON s.id_produit = p.id_produit JOIN magasin m ON s.id_magasin = m.id_magasin
                    WHERE {$where}
                    ORDER BY store_id ASC, product_name ASC
            ";
            $expiredStocksListStatement = $stockDao->entityManager->getConnection()->prepare($expiredStocksTable);
            $expiredStocksListStatement->execute();
            $expiredStockList = $expiredStocksListStatement->fetchAll();
            $currentStoreId = null;
            foreach ($expiredStockList as $expiredStock) {
                if ($store == null && (int)$expiredStock["store_id"] != $currentStoreId) {
                    $effectiveColspan = 4;
                    $documentHtml .= "
                                    <tr>
                                        <td colspan='{$effectiveColspan}' class='black-bold'>
                                            {$expiredStock["store_name"]}
                                        </td>
                                    </tr>
                    ";
                    $currentStoreId = (int)$expiredStock["store_id"];
                }
                $batchNumber = $expiredStock["batch_number"];
                $expirationDate = new DateTime($expiredStock["expiration_date"]);
                $quantity = (int)$expiredStock["quantity"];
                $documentHtml .= "
                                    <tr>
                                        <td style='text-align: left;'>{$expiredStock["product_name"]}</td>
                                        <td>{$batchNumber}</td>
                                        <td>{$expirationDate->format(DATE_FORMAT)}</td>
                                        <td>{$quantity}</td>
                                    </tr>
                ";
            }
            if (count($expiredStockList) == 0) {
                $effectiveColspan = 4;
                $documentHtml .= "
                                    <tr>
                                        <td colspan='{$effectiveColspan}'>
                                            {$this->translator->_("shared.noDataAvailable")}
                                        </td>
                                    </tr>
                ";
            }
            $documentHtml .= " 
                                </tbody>
                            </table>
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "expired_items_report_{$_SESSION[SESSION_USER_ID]}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/stocks/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/stocks/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printexceptionaloutingsAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/stocks") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $storeId = (int)$_POST["store"] != "" ? (int)$_POST["store"] : null;
            $exitTypeId = $_POST["exitType"] != "" ? (int)$_POST["exitType"] : null;
            $from = $_POST["from"] != "" ? new DateTime($_POST["from"]) : null;
            $to = $_POST["to"] != "" ? new DateTime($_POST["to"]) : null;
            $storeDao = new StoreDAO();
            $exceptionalOutingTypeDao = new ExceptionalOutingTypeDAO();
            $exitDao = new ExitDAO();
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $secondaryBrand = APP_SECONDARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = $this->translator->_("report.printExceptionalOutings.exceptionalOutingsList");
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: center;  
                                color: #000000; 
                              
                            }

                            .center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            td.validated {
                                color: {$primaryBrand} !important;
                                font-weight: bold;
                            }

                            td.not-validated {
                                color: {$secondaryBrand} !important;
                                font-weight: bold;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            .equalized {
                                table-layout: fixed;
                            }

                            bold {
                                color: black;
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$this->translator->_("report.printExceptionalOutings.exceptionalOutingsList")}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table>
                                <thead>
                                    <tr>
            ";
            $store = $storeId != null ? $storeDao->getStoreById($storeId) : null;
            $storeName = $store != null ? $store->getNomMagasin() : $this->translator->_("report.printExceptionalOutings.allStores");
            $exitType = $exitTypeId != null ? $exceptionalOutingTypeDao->getExceptionalOutingTypeById($exitTypeId) : null;
            $exitTypeName = $exitType != null ? $exitType->getNom() : $this->translator->_("report.printExceptionalOutings.allTypes");
            $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printExceptionalOutings.store")} : <bold class='black-bold'>{$storeName}</bold></th>
                                        <th class='center-text'>{$this->translator->_("report.printExceptionalOutings.type")} : <bold class='black-bold'>{$exitTypeName}</bold></th>
            ";
            $printFilters .= "{$this->translator->_("report.printExceptionalOutings.store")} : {$storeName}";
            $printFilters .= ", {$this->translator->_("report.printExceptionalOutings.type")} : {$exitTypeName}";
            if ($from != null) {
                $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printExceptionalOutings.dateAfter")} : <bold class='black-bold'>{$from->format(DATE_FORMAT)}</bold></th>
                ";
                $printFilters .= ", {$this->translator->_("report.printExceptionalOutings.dateAfter")} : {$from->format(DATE_FORMAT)}";
            }
            if ($to != null) {
                $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printExceptionalOutings.dateBefore")} : <bold class='black-bold'>{$to->format(DATE_FORMAT)}</bold></th>
                ";
                $printFilters .= ", {$this->translator->_("report.printExceptionalOutings.dateBefore")} : {$to->format(DATE_FORMAT)}";
            }
            $documentHtml .= "
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border equalized'>
                                <thead>
                                    <tr>
                                        <th colspan='2'>{$this->translator->_("report.printExceptionalOutings.number")}</th>
                                        <th colspan='2'>{$this->translator->_("report.printExceptionalOutings.entryDate")}</th>
                                        <th colspan='2'>{$this->translator->_("report.printExceptionalOutings.outingDate")}</th>
                                        <th colspan='2'>{$this->translator->_("report.printExceptionalOutings.user")}</th>
                                        <th colspan='2'>{$this->translator->_("report.printExceptionalOutings.status")}</th>
                                    </tr>
                                </thead>
                                <tbody>
            ";
            $whereStore = "";
            $whereExitType = "";
            $whereFrom = "";
            $whereTo = "";
            if ($storeId != null) {
                $whereStore .= " AND s.id_magasin = {$storeId}";
            }
            if ($exitType != null) {
                $whereExitType .= " AND s.type_sortie = '{$exitType->getCode()}'";
            }
            if ($from != null) {
                $whereFrom .= " AND s.date_sortie >= '{$from->format(DATE_FORMAT)} 00:00:00'";
            }
            if ($to != null) {
                $whereTo .= " AND s.date_sortie <= '{$to->format(DATE_FORMAT)} 23:59:59'";
            }
            $where = "s.statut = 1 AND s.type_sortie <> '" . TYPE_SORTIE_VENTE . "'{$whereStore}{$whereExitType}{$whereFrom}{$whereTo}";
            $exceptionalOutingsTable = "
                SELECT s.id_sortie AS exit_id
                FROM sortie s JOIN magasin m ON s.id_magasin = m.id_magasin JOIN type_sortie_exceptionnelle tse ON s.type_sortie = tse.code
                JOIN utilisateur u ON s.id_utilisateur = u.id_utilisateur JOIN personne pe ON u.id_personne = pe.id_personne 
                WHERE {$where}
                ORDER BY s.id_magasin ASC, s.type_sortie ASC
            ";
            $exceptionalOutingsListStatement = $exitDao->entityManager->getConnection()->prepare($exceptionalOutingsTable);
            $exceptionalOutingsListStatement->execute();
            $exceptionalOutingList = $exceptionalOutingsListStatement->fetchAll();
            $currentStoreId = null;
            $currentTypeCode = null;
            foreach ($exceptionalOutingList as $exitArray) {
                $exit = $exitDao->getExitById((int)$exitArray["exit_id"]);
                if (($store == null || $exitType == null) && ($exit->getIdMagasin()->getIdMagasin() != $currentStoreId || $exit->getTypeSortie() != $currentTypeCode)) {
                    $effectiveColspan = 10;
                    $documentHtml .= "
                                    <tr>
                    ";
                    if ($store == null && $exitType == null) $effectiveColspan /= 2;
                    if ($store == null) {
                        $documentHtml .= "
                                        <td colspan='{$effectiveColspan}' class='black-bold'>
                                            {$this->translator->_("report.printExceptionalOutings.store")} : {$exit->getIdMagasin()->getNomMagasin()}
                                        </td>
                        ";
                    }
                    if ($exitType == null) {
                        $effectiveExitType = $exceptionalOutingTypeDao->getExceptionalOutingTypeByCode($exit->getTypeSortie());
                        $documentHtml .= "
                                        <td colspan='{$effectiveColspan}' class='black-bold'>
                                            {$this->translator->_("report.printExceptionalOutings.type")} : {$effectiveExitType->getNom()}
                                        </td>
                        ";
                    }
                    $documentHtml .= "
                                    </tr>
                    ";
                    $currentStoreId = $exit->getIdMagasin()->getIdMagasin();
                    $currentTypeCode = $exit->getTypeSortie();
                }
                $exitStatus = $this->translator->_($exit->getValider() == 1 ? "report.printExceptionalOutings.validated" : "report.printExceptionalOutings.notValidated");
                $exitStatusClassName = $exit->getValider() == 1 ? 'validated' : 'not-validated';
                $documentHtml .= "
                                    <tr>
                                        <td colspan='2'>{$exit->getIdSortie()}</td>
                                        <td colspan='2'>{$exit->getDateSaisie()->format(DATE_FORMAT)}</td>
                                        <td colspan='2'>{$exit->getDateSortie()->format(DATE_FORMAT)}</td>
                                        <td colspan='2'>{$exit->getIdUtilisateur()->getIdPersonne()->getNom()} {$exit->getIdUtilisateur()->getIdPersonne()->getPrenom()}</td>
                                        <td colspan='2' class='{$exitStatusClassName}'>{$exitStatus}</td>
                                    </tr>
                ";
            }
            if (count($exceptionalOutingList) == 0) {
                $effectiveColspan = 10;
                $documentHtml .= "
                                    <tr>
                                        <td colspan='{$effectiveColspan}'>
                                            {$this->translator->_("shared.noDataAvailable")}
                                        </td>
                                    </tr>
                ";
            }
            $documentHtml .= " 
                                </tbody>
                            </table>
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "exceptional_outings_list_{$_SESSION[SESSION_USER_ID]}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/stocks/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/stocks/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printexceptionaloutingAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/stocks") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $id = (int)$_POST["id"];
            $exitDao = new ExitDAO();
            $exceptionalOutingTypeDao = new ExceptionalOutingTypeDAO();
            $exitLineDao = new ExitLineDAO();
            $exit = $exitDao->getExitById($id);
            $exitType = $exceptionalOutingTypeDao->getExceptionalOutingTypeByCode($exit->getTypeSortie());
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = "{$this->translator->_("report.printExceptionalOuting.exceptionalOutingSheetNumber")} : {$id}";
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }
                            

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: center;  
                                color: #000000; 
                            }

                            .center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            bold {
                                color: black;
                            }

                            .criteria {
                                font-size: 14px;
                                font-weight: bold;
                                color: black;
                                text-align: center;
                            }

                            .equalized {
                                table-layout: fixed;
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$this->translator->_("report.printExceptionalOuting.exceptionalOutingSheetNumber")} : {$id}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table>
                                <thead>
                                    <tr>
                                        <td class='criteria'>{$this->translator->_("report.printExceptionalOuting.store")} : {$exit->getIdMagasin()->getNomMagasin()}</td>
                                        <td class='criteria'>{$this->translator->_("report.printExceptionalOuting.type")} : {$exitType->getNom()}</td>
                                        <td class='criteria'>{$this->translator->_("report.printExceptionalOuting.date")} : {$exit->getDateSortie()->format(DATE_FORMAT)}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border'>
                                <thead>
                                    <tr>
                                        <th>{$this->translator->_("report.printExceptionalOuting.product")}</th>
                                        <th>{$this->translator->_("report.printExceptionalOuting.batchNumber")}</th>
                                        <th>{$this->translator->_("report.printExceptionalOuting.sellingPrice")}</th>
                                        <th>{$this->translator->_("report.printExceptionalOuting.quantity")}</th>
                                    </tr>
                                </thead>
                                <tbody>
            ";
            $printFilters .= "{$this->translator->_("report.printExceptionalOuting.store")} : {$exit->getIdMagasin()->getNomMagasin()}";
            $printFilters .= ", {$this->translator->_("report.printExceptionalOuting.type")} : {$exitType->getNom()}";
            $printFilters .= ", {$this->translator->_("report.printExceptionalOuting.date")} : {$exit->getDateSortie()->format(DATE_FORMAT)}";
            $exitLines = $exitLineDao->getExitLinesOfExit($exit);
            $totalSellingPrice = 0;
            foreach ($exitLines as $exitLine) {
                $totalSellingPrice += $exitLine->getPrixUnitaire();
                $sellingPrice = number_format($exitLine->getPrixUnitaire(), 0, '', ' ');
                $documentHtml .= "
                                    <tr>
                                        <td style='text-align: left;'>{$exitLine->getIdStock()->getDesignation()}</td>
                                        <td>{$exitLine->getIdStock()->getNumLot()}</td>
                                        <td style='text-align: right;'>{$sellingPrice}</td>
                                        <td>{$exitLine->getQuantite()}</td>
                                    </tr>
                ";
            }
            $formatTotalSellingPrice = number_format($totalSellingPrice, 0, '', ' ');
            $documentHtml .= "  
                                    <tr>
                                        <td colspan='2' style='text-align: right; font-weight: bold;'>{$this->translator->_("report.printExceptionalOuting.total")}</td>
                                        <td style='text-align: right;'>{$formatTotalSellingPrice}</td>
                                        <td></td>
                                    </tr>
                                </tbody>
                            </table>
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "exceptional_outing_sheet_{$_SESSION[SESSION_USER_ID]}_{$id}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/stocks/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/stocks/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printviewexceptionaloutingsAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/stocks") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $storeId = (int)$_POST["store"] != "" ? (int)$_POST["store"] : null;
            $exitTypeId = $_POST["exitType"] != "" ? (int)$_POST["exitType"] : null;
            $from = $_POST["from"] != "" ? new DateTime($_POST["from"]) : null;
            $to = $_POST["to"] != "" ? new DateTime($_POST["to"]) : null;
            $storeDao = new StoreDAO();
            $exceptionalOutingTypeDao = new ExceptionalOutingTypeDAO();
            $exitDao = new ExitDAO();
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $secondaryBrand = APP_SECONDARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = $this->translator->_("report.printViewExceptionalOutings.exceptionalOutingsReport");
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: center;  
                                color: #000000; 
                              
                            }

                            .center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            td.validated {
                                color: {$primaryBrand} !important;
                                font-weight: bold;
                            }

                            td.not-validated {
                                color: {$secondaryBrand} !important;
                                font-weight: bold;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            .equalized {
                                table-layout: fixed;
                            }

                            bold {
                                color: black;
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$this->translator->_("report.printViewExceptionalOutings.exceptionalOutingsReport")}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table>
                                <thead>
                                    <tr>
            ";
            $store = $storeId != null ? $storeDao->getStoreById($storeId) : null;
            $storeName = $store != null ? $store->getNomMagasin() : $this->translator->_("report.printViewExceptionalOutings.allStores");
            $exitType = $exitTypeId != null ? $exceptionalOutingTypeDao->getExceptionalOutingTypeById($exitTypeId) : null;
            $exitTypeName = $exitType != null ? $exitType->getNom() : $this->translator->_("report.printViewExceptionalOutings.allTypes");
            $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printViewExceptionalOutings.store")} : <bold class='black-bold'>{$storeName}</bold></th>
                                        <th class='center-text'>{$this->translator->_("report.printViewExceptionalOutings.type")} : <bold class='black-bold'>{$exitTypeName}</bold></th>
            ";
            $printFilters .= "{$this->translator->_("report.printViewExceptionalOutings.store")} : $storeName}";
            $printFilters .= ", {$this->translator->_("report.printViewExceptionalOutings.type")} : {$exitTypeName}";
            if ($from != null) {
                $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printViewExceptionalOutings.dateAfter")} : <bold class='black-bold'>{$from->format(DATE_FORMAT)}</bold></th>
                ";
                $printFilters .= ", {$this->translator->_("report.printViewExceptionalOutings.dateAfter")} : {$from->format(DATE_FORMAT)}";
            }
            if ($to != null) {
                $documentHtml .= "
                                        <th class='center-text'>{$this->translator->_("report.printViewExceptionalOutings.dateBefore")} : <bold class='black-bold'>{$to->format(DATE_FORMAT)}</bold></th>
                ";
                $printFilters .= ", {$this->translator->_("report.printViewExceptionalOutings.dateBefore")} : {$to->format(DATE_FORMAT)}";
            }
            $documentHtml .= "
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border equalized'>
                                <thead>
                                    <tr>
                                        <th colspan='2'>{$this->translator->_("report.printViewExceptionalOutings.number")}</th>
                                        <th colspan='2'>{$this->translator->_("report.printViewExceptionalOutings.entryDate")}</th>
                                        <th colspan='2'>{$this->translator->_("report.printViewExceptionalOutings.outingDate")}</th>
                                        <th colspan='2'>{$this->translator->_("report.printViewExceptionalOutings.user")}</th>
                                        <th colspan='2'>{$this->translator->_("report.printViewExceptionalOutings.status")}</th>
                                    </tr>
                                </thead>
                                <tbody>
            ";
            $whereStore = "";
            $whereExitType = "";
            $whereFrom = "";
            $whereTo = "";
            if ($storeId != null) {
                $whereStore .= " AND s.id_magasin = {$storeId}";
            }
            if ($exitType != null) {
                $whereExitType .= " AND s.type_sortie = '{$exitType->getCode()}'";
            }
            if ($from != null) {
                $whereFrom .= " AND s.date_sortie >= '{$from->format(DATE_FORMAT)} 00:00:00'";
            }
            if ($to != null) {
                $whereTo .= " AND s.date_sortie <= '{$to->format(DATE_FORMAT)} 23:59:59'";
            }
            $where = "s.statut = 1 AND s.type_sortie <> '" . TYPE_SORTIE_VENTE . "'{$whereStore}{$whereExitType}{$whereFrom}{$whereTo}";
            $exceptionalOutingsTable = "
                SELECT s.id_sortie AS exit_id
                FROM sortie s JOIN magasin m ON s.id_magasin = m.id_magasin JOIN type_sortie_exceptionnelle tse ON s.type_sortie = tse.code
                JOIN utilisateur u ON s.id_utilisateur = u.id_utilisateur JOIN personne pe ON u.id_personne = pe.id_personne 
                WHERE {$where}
                ORDER BY s.id_magasin ASC, s.type_sortie ASC
            ";
            $exceptionalOutingsListStatement = $exitDao->entityManager->getConnection()->prepare($exceptionalOutingsTable);
            $exceptionalOutingsListStatement->execute();
            $exceptionalOutingList = $exceptionalOutingsListStatement->fetchAll();
            $currentStoreId = null;
            $currentTypeCode = null;
            foreach ($exceptionalOutingList as $exitArray) {
                $exit = $exitDao->getExitById((int)$exitArray["exit_id"]);
                if (($store == null || $exitType == null) && ($exit->getIdMagasin()->getIdMagasin() != $currentStoreId || $exit->getTypeSortie() != $currentTypeCode)) {
                    $effectiveColspan = 10;
                    $documentHtml .= "
                                    <tr>
                    ";
                    if ($store == null && $exitType == null) $effectiveColspan /= 2;
                    if ($store == null) {
                        $documentHtml .= "
                                        <td colspan='{$effectiveColspan}' class='black-bold'>
                                            {$this->translator->_("report.printViewExceptionalOutings.store")} : {$exit->getIdMagasin()->getNomMagasin()}
                                        </td>
                        ";
                    }
                    if ($exitType == null) {
                        $effectiveExitType = $exceptionalOutingTypeDao->getExceptionalOutingTypeByCode($exit->getTypeSortie());
                        $documentHtml .= "
                                        <td colspan='{$effectiveColspan}' class='black-bold'>
                                            {$this->translator->_("report.printViewExceptionalOutings.type")} : {$effectiveExitType->getNom()}
                                        </td>
                        ";
                    }
                    $documentHtml .= "
                                    </tr>
                    ";
                    $currentStoreId = $exit->getIdMagasin()->getIdMagasin();
                    $currentTypeCode = $exit->getTypeSortie();
                }
                $exitStatus = $this->translator->_($exit->getValider() == 1 ? "report.printViewExceptionalOutings.validated" : "report.printViewExceptionalOutings.notValidated");
                $exitStatusClassName = $exit->getValider() == 1 ? 'validated' : 'not-validated';
                $documentHtml .= "
                                    <tr>
                                        <td colspan='2'>{$exit->getIdSortie()}</td>
                                        <td colspan='2'>{$exit->getDateSaisie()->format(DATE_FORMAT)}</td>
                                        <td colspan='2'>{$exit->getDateSortie()->format(DATE_FORMAT)}</td>
                                        <td colspan='2'>{$exit->getIdUtilisateur()->getIdPersonne()->getNom()} {$exit->getIdUtilisateur()->getIdPersonne()->getPrenom()}</td>
                                        <td colspan='2' class='{$exitStatusClassName}'>{$exitStatus}</td>
                                    </tr>
                ";
            }
            if (count($exceptionalOutingList) == 0) {
                $effectiveColspan = 10;
                $documentHtml .= "
                                    <tr>
                                        <td colspan='{$effectiveColspan}'>
                                            {$this->translator->_("shared.noDataAvailable")}
                                        </td>
                                    </tr>
                ";
            }
            $documentHtml .= " 
                                </tbody>
                            </table>
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "exceptional_outings_report_{$_SESSION[SESSION_USER_ID]}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/stocks/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/stocks/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printviewproductpathAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/reports") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $storeDao = new StoreDAO();
            $productDao = new ProductDAO();
            $stockDao = new StockDAO();
            $storeId = (int)$_POST["store"];
            $productId = (int)$_POST["product"];
            $startDate = new DateTime($_POST["startDate"]);
            $endDate = new DateTime("{$_POST["endDate"]} 23:59:59");
            $store = $storeDao->getStoreById($storeId);
            $product = $productDao->getProductById($productId);
            $dateBeforeStart = clone $startDate;
            $dateBeforeStart->modify(" -1 day");
            $momentDateBeforeStart = new \Moment\Moment($dateBeforeStart->format(DATE_FORMAT));
            $momentEndDate = new \Moment\Moment($endDate->format(DATE_FORMAT));
            $quantityAtPeriodStart = $stockDao->getProductQuantityAtDate($product, $dateBeforeStart, $store);
            $quantityAtPeriodEnd = $stockDao->getProductQuantityAtDate($product, $endDate, $store);
            $movements = $stockDao->getProductMovementResumeOnPeriod($product, $startDate, $endDate, $this->translator, $store);
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $secondaryBrand = APP_SECONDARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = "{$this->translator->_("report.printViewProductPath.movementHistoryOfArticle")}({$product->getDesignation()})";
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: center;  
                                color: #000000; 
                              
                            }

                            .center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            .start-quantity {
                                color: {$thirdBrand} !important;
                                font-weight: bold;
                                font-size: 16px !important;
                                padding: 5px;
                                background-color: {$secondaryBrand} !important;
                                width: 90px !important;
                                border-radius: 25px !important;
                                height: 24px !important;
                                text-align: center;
                                margin-left: 37% !important;
                                vertical-align: center;
                            }

                            .end-quantity {
                                color: {$thirdBrand} !important;
                                font-weight: bold;
                                font-size: 16px !important;
                                padding: 5px;
                                background-color: {$primaryBrand} !important;
                                width: 90px !important;
                                border-radius: 25px !important;
                                height: 24px !important;
                                text-align: center;
                                margin-left: 37% !important;
                                vertical-align: center;
                            }

                            .entry-line {
                                text-align: left !important;
                                color: {$primaryBrand} !important;
                                padding-left: 5px !important;
                                font-weight: bold !important;
                                font-size: 14px !important;
                            }

                            .exit-line {
                                text-align: right !important;
                                color: {$secondaryBrand} !important;
                                padding-right: 5px !important;
                                font-weight: bold !important;
                                font-size: 14px !important;
                            }

                            .entry-label {
                                color: {$thirdBrand} !important;
                                background-color: {$primaryBrand} !important;
                            }

                            .exit-label {
                                color: {$thirdBrand} !important;
                                background-color: {$secondaryBrand} !important;
                            }

                            .no-movement-found {
                                color: grey !important;
                                font-weight: bold !important;
                                font-size: 16px !important;
                            }

                            td.validated {
                                color: {$primaryBrand} !important;
                                font-weight: bold;
                            }

                            td.not-validated {
                                color: {$secondaryBrand} !important;
                                font-weight: bold;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            .equalized {
                                table-layout: fixed;
                            }

                            bold {
                                color: black;
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$this->translator->_("report.printViewProductPath.movementHistoryOfArticle")}({$product->getDesignation()})</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table>
                                <thead>
                                    <tr>
                                        <th class='center-text'>{$this->translator->_("report.printViewProductPath.store")} : <bold class='black-bold'>{$store->getNomMagasin()}</bold></th>
                                        <th class='center-text'>{$this->translator->_("report.printViewProductPath.dateAfter")} : <bold class='black-bold'>{$startDate->format(DATE_FORMAT)}</bold></th>
                                        <th class='center-text'>{$this->translator->_("report.printViewProductPath.dateBefore")} : <bold class='black-bold'>{$endDate->format(DATE_FORMAT)}</bold></th>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border equalized'>
                                <thead>
                                    <tr>
                                        <th>{$this->translator->_("report.printViewProductPath.quantityAtTheEveningOf")} {$momentDateBeforeStart->format(DATE_FORMAT_LONG)}</th>
                                        <th>{$this->translator->_("report.printViewProductPath.quantityAtTheEveningOf")} {$momentEndDate->format(DATE_FORMAT_LONG)}</th>
                                    </tr>
                                </thead>
                                <tbody>
                                    <tr>
                                        <td class='center-content'>
                                            <p class='start-quantity'>{$quantityAtPeriodStart}</p>
                                        </td>
                                        <td class='center-content'>
                                            <p class='end-quantity'>{$quantityAtPeriodEnd}</p>
                                        </td>
                                    </tr>
                                </tbody>
                            </table>
                            <hr/>
                            <table class='border equalized'>
                                <thead>
                                    <tr>
                                        <th>{$this->translator->_("report.printViewProductPath.detailsOfProductMovements")}</th>
                                    </tr>
                                </thead>
                                <tbody>
            ";
            $printFilters .= "{$this->translator->_("report.printViewProductPath.store")} : {$store->getNomMagasin()}";
            $printFilters .= ", {$this->translator->_("report.printViewProductPath.dateAfter")} : {$startDate->format(DATE_FORMAT)}";
            $printFilters .= ", {$this->translator->_("report.printViewProductPath.dateBefore")} : {$endDate->format(DATE_FORMAT)}";
            foreach ($movements as $movement) {
                $momentOfMovement = new \Moment\Moment($movement["date"]);
                $formatedDate = ucwords($momentOfMovement->format(DATE_FORMAT_LONG));
                $action = $movement["sign"] == "+" ? $this->translator->_("report.printViewProductPath.entryOf") : $this->translator->_("report.printViewProductPath.exitOf");
                $lineClass = $movement["sign"] == "+" ? "entry-line" : "exit-line";
                $labelClass = $movement["sign"] == "+" ? "entry-label" : "exit-label";
                $documentHtml .= "
                                    <tr>
                                        <td class='{$lineClass}'>
                                            <p>
                                                <bold class='{$labelClass}'>{$formatedDate}</bold> :
                                                {$action} <bold class='{$labelClass}'>{$movement["quantity"]}</bold> {$this->translator->_("report.printViewProductPath.pieces")}
                                                {$this->translator->_("report.printViewProductPath.withAMotive")}, {$movement["reason"]}
                                            </p>
                                        </td>
                                    </tr>
                ";
            }
            if (count($movements) == 0) {
                $documentHtml .= "
                                    <tr>
                                        <td class='no-movement-found'>{$this->translator->_("report.printViewProductPath.noMovementFound")}</td>
                                    </tr>
                ";
            }
            $documentHtml .= " 
                                </tbody>
                            </table>
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "movement_history_report_{$_SESSION[SESSION_USER_ID]}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/reports/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/reports/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printviewstocksituationAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/reports") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $stockDao = new StockDAO();
            $productCategoryDao = new ProductCategoryDAO();
            $storeDao = new StoreDAO();
            $productDao = new ProductDAO();
            $storeId = (int)$_POST["store"];
            $productCategoryId = (int)$_POST["category"] != 0 ? (int)$_POST["category"] : null;
            $productId = (int)$_POST["product"] != 0 ? (int)$_POST["product"] : null;
            $startDate = new DateTime($_POST["startDate"]);
            $endDate = new DateTime($_POST["endDate"]);
            $dateBeforeStart = clone $startDate;
            $dateBeforeStart->modify("-1 day");
            $momentStartDate = new \Moment\Moment($startDate->format(DATE_FORMAT));
            $momentEndDate = new \Moment\Moment($endDate->format(DATE_FORMAT));
            $startDateFormated = ucwords($momentStartDate->format(DATE_FORMAT_LONG));
            $endDateFormated = ucwords($momentEndDate->format(DATE_FORMAT_LONG));
            $store = $storeDao->getStoreById($storeId);
            $productCategory = $productCategoryId != null ? $productCategoryDao->getProductCategoryById($productCategoryId) : null;
            $product = $productId != null ? $productDao->getProductById($productId) : null;
            $allProductsConcernedIds = $stockDao->getAllProductsInStock($store, null, $productCategory, $product);
            $productCategoryName = $productCategory != null ? $productCategory->getNom() : $this->translator->_("report.printViewStockSituation.allCategories");
            $productName = $product != null ? $product->getDesignation() : $this->translator->_("report.printViewStockSituation.allProducts");
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $secondaryBrand = APP_SECONDARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = $this->translator->_("report.printViewStockSituation.stockSituationReport");
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: center;  
                                color: #000000; 
                              
                            }

                            .center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            .no-line-found {
                                color: grey !important;
                                font-weight: bold !important;
                                font-size: 16px !important;
                            }

                            td.validated {
                                color: {$primaryBrand} !important;
                                font-weight: bold;
                            }

                            td.not-validated {
                                color: {$secondaryBrand} !important;
                                font-weight: bold;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            .equalized {
                                table-layout: fixed;
                            }

                            bold {
                                color: black;
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$this->translator->_("report.printViewStockSituation.stockSituationReport")}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table>
                                <thead>
                                    <tr>
                                        <th class='center-text'>{$this->translator->_("report.printViewStockSituation.store")} : <bold class='black-bold'>{$store->getNomMagasin()}</bold></th>
                                        <th class='center-text'>{$this->translator->_("report.printViewStockSituation.category")} : <bold class='black-bold'>{$productCategoryName}</bold></th>
                                        <th class='center-text'>{$this->translator->_("report.printViewStockSituation.product")} : <bold class='black-bold'>{$productName}</bold></th>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table>
                                <thead>
                                    <tr>
                                        <th class='center-text'>{$this->translator->_("report.printViewStockSituation.dateAfter")} : <bold class='black-bold'>{$startDate->format(DATE_FORMAT)}</bold></th>
                                        <th class='center-text'>{$this->translator->_("report.printViewStockSituation.dateBefore")} : <bold class='black-bold'>{$endDate->format(DATE_FORMAT)}</bold></th>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border equalized'>
                                <thead>
                                    <tr>
                                        <th>{$this->translator->_("report.printViewStockSituation.product")}</th>
                                        <th>{$this->translator->_("report.printViewStockSituation.quantityOnTheEveOf")} {$startDateFormated}</th>
                                        <th>{$this->translator->_("report.printViewStockSituation.quantityEnteredOverThePeriod")}</th>
                                        <th>{$this->translator->_("report.printViewStockSituation.quantityIssuedOverThePeriod")}</th>
                                        <th>{$this->translator->_("report.printViewStockSituation.quantityAtTheEveningOf")} {$endDateFormated}</th>
                                    </tr>
                                </thead>
                                <tbody>
            ";
            $printFilters .= "{$this->translator->_("report.printViewStockSituation.store")} : {$store->getNomMagasin()}";
            $printFilters .= ", {$this->translator->_("report.printViewStockSituation.category")} : {$productCategoryName}";
            $printFilters .= ", {$this->translator->_("report.printViewStockSituation.product")} : {$productName}";
            $printFilters .= ", {$this->translator->_("report.printViewStockSituation.dateAfter")} : {$startDate->format(DATE_FORMAT)}";
            $printFilters .= ", {$this->translator->_("report.printViewStockSituation.dateBefore")} : {$endDate->format(DATE_FORMAT)}";
            foreach ($allProductsConcernedIds as $productConcernedId) {
                $product = $productDao->getProductById($productConcernedId);
                $quantityAtBegin = $stockDao->getProductQuantityAtDate($product, $dateBeforeStart, $store);
                $quantityEnteredDuringPeriod = $stockDao->getProductQuantityEnteredOverPeriod($product, $startDate, $endDate, $store);
                $quantityIssuedDuringPeriod = $stockDao->getProductQuantityIssuedOverPeriod($product, $startDate, $endDate, $store);
                $quantityAtEnd = $stockDao->getProductQuantityAtDate($product, $endDate, $store);
                $documentHtml .= "
                                    <tr>
                                        <td>{$product->getDesignation()}</td>
                                        <td>{$quantityAtBegin}</td>
                                        <td>{$quantityEnteredDuringPeriod}</td>
                                        <td>{$quantityIssuedDuringPeriod}</td>
                                        <td>{$quantityAtEnd}</td>
                                    </tr>
                ";
            }
            if (count($allProductsConcernedIds) == 0) {
                $documentHtml .= "
                                    <tr colspan='5'>
                                        <td class='no-line-found'>{$this->translator->_("report.printViewStockSituation.noStockSituationFound")}</td>
                                    </tr>
                ";
            }
            $documentHtml .= " 
                                </tbody>
                            </table>
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "stock_situation_report_{$_SESSION[SESSION_USER_ID]}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/reports/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/reports/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printpayslipAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/payslips") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $yearNumber = (int)$_POST["year"];
            $monthNumber = (int)$_POST["month"];
            $employeeId = (int)$_POST["employeeId"];
            $userDao = new UserDAO();
            $monthDao = new MonthDAO();
            $monthlyPayDao = new MonthlyPayDAO();
            $monthlyPayDetailDao = new MonthlyPayDetailDAO();
            $employee = $userDao->getUserById($employeeId);
            $month = $monthDao->getMonthByYearAndNumber($yearNumber, $monthNumber);
            $monthlyPay = $monthlyPayDao->getMontlyPayByMonthEmployeeAndMonth($employee, $month);
            $monthlyPayDetails = $monthlyPayDetailDao->getMonthlyPayDetailOfMonthlyPay($monthlyPay);
            $monthNumberLetter = ((int)$monthNumber <= 9) ? "0{$monthNumber}" : "$monthNumber";
            $selectedMonthYearDate = new DateTime("{$yearNumber}-{$monthNumberLetter}-01");
            $momentOfSelectMonthYearDate = new \Moment\Moment($selectedMonthYearDate->format(DATE_FORMAT));
            $monthAndYear = "{$momentOfSelectMonthYearDate->format(DATE_FORMAT_MONTH)} {$momentOfSelectMonthYearDate->format(DATE_FORMAT_YEAR)}";
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $secondaryBrand = APP_SECONDARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = $this->translator->_("report.printPayslip.payslip");
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }
                            

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: center;  
                                color: #000000; 
                            }

                            .center-text {
                                text-align: center;
                            }

                            td.right-text, th.right-text {
                                text-align: right;
                            }

                            td.left-text, th.left-text {
                                text-align: left;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            .equalized {
                                table-layout: fixed;
                            }

                            bold {
                                color: black;
                            }

                            td.bold {
                                font-weight: bold;
                            }

                            td.enterprise {
                                font-weight: bold;
                                font-size: 22px;
                            }

                            td.enterprise-detail {
                                font-size: 16px;
                            }

                            table.top-table {
                                margin-top: 20px;
                            }

                            tr.positive {
                                background-color: {$primaryBrand};
                            }

                            tr.negative {
                                background-color: {$secondaryBrand};
                            }

                            td.positive {
                                color: {$thirdBrand};
                                border: 1px solid {$thirdBrand}; 
                            }

                            td.negative {
                                color: {$thirdBrand};
                                border: 1px solid {$thirdBrand}; 
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table class='top-table'>
                                <thead>
                                    <tr>
                                        <td class='enterprise'>
                                            {$enterpriseName}
                                        </td>
                                        <td class='enterprise' style='text-align: right;'>
                                            {$this->translator->_("report.printPayslip.payslip")}
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='enterprise-detail'>
                                            {$this->translator->_("shared.pob")}: {$poBox}
                                        </td>
                                        <td>
                                        </td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border equalized'>
                                <thead>
                                    <tr>
                                        <td class='center-text'>
                                            {$this->translator->_("report.printPayslip.month")}
                                        </td>
                                        <td class='center-text'>
                                            {$this->translator->_("report.printPayslip.employee")}
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title center-text'>{$monthAndYear}</td>
                                        <td class='title center-text'>
                                            {$employee->getIdPersonne()->getNom()} {$employee->getIdPersonne()->getPrenom()}
                                        </td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border'>
                                <thead>
                                    <tr>
                                        <th class='left-text'>{$this->translator->_("report.printPayslip.payrollSection")}</th>
                                        <th class='right-text'>{$this->translator->_("report.printPayslip.positive")}</th>
                                        <th class='right-text'>{$this->translator->_("report.printPayslip.negative")}</th>
                                        <th class='right-text'>{$this->translator->_("report.printPayslip.total")}</th>
                                    </tr>
                                </thead>
                                <tbody>
            ";
            $printFilters .= "{$this->translator->_("report.printPayslip.month")} : {$monthAndYear}";
            $printFilters .= ", {$this->translator->_("report.printPayslip.employee")} : {$employee->getIdPersonne()->getNom()} {$employee->getIdPersonne()->getPrenom()}";
            $positiveTotal = 0;
            $negativeTotal = 0;
            $total = 0;
            foreach ($monthlyPayDetails as $monthlyPayDetail) {
                $positiveAmount = $monthlyPayDetail->getType() == POSITIVE_PAYROLL_SECTION_SIGN ? number_format($monthlyPayDetail->getMontant(), 0, '', ' ') : '----';
                $negativeAmount = $monthlyPayDetail->getType() == NEGATIVE_PAYROLL_SECTION_SIGN ? number_format($monthlyPayDetail->getMontant(), 0, '', ' ') : '----';
                $totalAmount = number_format($monthlyPayDetail->getMontant(), 0, '', ' ');
                if ($monthlyPayDetail->getType() == POSITIVE_PAYROLL_SECTION_SIGN) {
                    $total += $monthlyPayDetail->getMontant();
                } else {
                    $total -= $monthlyPayDetail->getMontant();
                }
                $positiveTotal += $monthlyPayDetail->getType() == POSITIVE_PAYROLL_SECTION_SIGN ? $monthlyPayDetail->getMontant() : 0;
                $negativeTotal += $monthlyPayDetail->getType() == NEGATIVE_PAYROLL_SECTION_SIGN ? $monthlyPayDetail->getMontant() : 0;
                $lineClass = $monthlyPayDetail->getType() == POSITIVE_PAYROLL_SECTION_SIGN ? "positive" : "negative";
                $documentHtml .= "
                                    <tr class='{$lineClass}'>
                                        <td class='left-text bold {$lineClass}'>{$monthlyPayDetail->getIdRubriquePaie()->getNomRubrique()}</td>
                                        <td class='right-text bold {$lineClass}'>{$positiveAmount}</td>
                                        <td class='right-text bold {$lineClass}'>{$negativeAmount}</td>
                                        <td class='right-text bold {$lineClass}'>{$totalAmount}</td>
                                    </tr>
                ";
            }
            $formatPositiveTotal = number_format($positiveTotal, 0, '', ' ');
            $formatNegativeTotal = number_format($negativeTotal, 0, '', ' ');
            $formatTotal = number_format($total, 0, '', ' ');
            $totalWords = ucfirst(Translator::convertToWords($total));
            $documentHtml .= "
                                    <tr>
                                        <td class='title right-text'>{$this->translator->_("report.printPayslip.total")}</td>
                                        <td class='title right-text'>{$formatPositiveTotal}</td>
                                        <td class='title right-text'>{$formatNegativeTotal}</td>
                                        <td class='title right-text'>{$formatTotal}</td>
                                    </tr>
                                </tbody>
                            </table>
                            <hr/>
                            <table>
                                <thead>
                                    <tr>
                                        <th class='right-text'>
                                            {$this->translator->_("report.printPayslip.salaryOf")} {$monthAndYear} : <i style='color: {$secondaryBrand}'>{$formatTotal} {$this->translator->_("shared.xaf")}</i>
                                        </th>
                                    </tr>
                                    <tr>
                                        <th class='right-text'>
                                            <i style='color: {$secondaryBrand}'>{$totalWords} {$this->translator->_("shared.xaf")}</i>
                                        </th>
                                    </tr>
                                </thead>
                            </table>
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "payslip_{$_SESSION[SESSION_USER_ID]}_{$employeeId}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/payslips/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/payslips/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printviewfollowupAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/reports") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $pointOfSaleDao = new PointOfSaleDAO();
            $customerInvoiceDao = new CustomerInvoiceDAO();
            $pointOfSaleId = (int)$_POST["pointOfSale"] != 0  ? (int)$_POST["pointOfSale"] : null;
            $pointOfSale = $pointOfSaleId != null ? $pointOfSaleDao->getPointOfSaleById($pointOfSaleId) : null;
            $daysCount = (int)$_POST["daysCount"];
            $followUpDate = new DateTime();
            $followUpDate->modify("- {$daysCount} days");
            $customersToFollowUp = $customerInvoiceDao->getCustomerToFollowUp($followUpDate, $pointOfSale);
            $pointOfSaleName = $pointOfSale != null ? $pointOfSale->getNomPointVente() : $this->translator->_("report.printViewFollowUp.allPointsOfSale");
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $secondaryBrand = APP_SECONDARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = $this->translator->_("report.printViewFollowUp.customersToFollowUp");
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: center;  
                                color: #000000; 
                              
                            }

                            .center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            .no-line-found {
                                color: grey !important;
                                font-weight: bold !important;
                                font-size: 16px !important;
                            }

                            td.validated {
                                color: {$primaryBrand} !important;
                                font-weight: bold;
                            }

                            td.not-validated {
                                color: {$secondaryBrand} !important;
                                font-weight: bold;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            .equalized {
                                table-layout: fixed;
                            }

                            bold {
                                color: black;
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$printTitle}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table>
                                <thead>
                                    <tr>
                                        <th class='center-text'>{$this->translator->_("report.printViewFollowUp.pointOfSale")} : <bold class='black-bold'>{$pointOfSaleName}</bold></th>
                                        <th class='center-text'>{$this->translator->_("report.printViewFollowUp.deadlineForLastVisit")} : <bold class='black-bold'>{$daysCount} {$this->translator->_("report.printViewFollowUp.days")}({$followUpDate->format(DATE_FORMAT)})</bold></th>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border equalized'>
                                <thead>
                                    <tr>
                                        <th>{$this->translator->_("report.printViewFollowUp.customer")}</th>
                                        <th>{$this->translator->_("report.printViewFollowUp.lastVisitDate")}</th>
                                        <th>{$this->translator->_("report.printViewFollowUp.phoneNumber")}</th>
                                        <th>{$this->translator->_("report.printViewFollowUp.email")}</th>
                                    </tr>
                                </thead>
                                <tbody>
            ";
            $printFilters .= "{$this->translator->_("report.printViewFollowUp.pointOfSale")} : {$pointOfSaleName}";
            $printFilters .= ", {$this->translator->_("report.printViewFollowUp.deadlineForLastVisit")} : {$daysCount} {$this->translator->_("report.printViewFollowUp.days")}({$followUpDate->format(DATE_FORMAT)})";
            foreach ($customersToFollowUp as $customerToFollowUp) {
                $documentHtml .= "
                                    <tr>
                                        <td>{$customerToFollowUp["customer_last_name"]} {$customerToFollowUp["customer_first_name"]}</td>
                                        <td>{$customerToFollowUp["last_invoice_date"]}</td>
                                        <td>{$customerToFollowUp["customer_phone"]}</td>
                                        <td>{$customerToFollowUp["customer_email"]}</td>
                                    </tr>
                ";
            }
            if (count($customersToFollowUp) == 0) {
                $documentHtml .= "
                                    <tr colspan='4'>
                                        <td class='no-line-found'>{$this->translator->_("report.printViewFollowUp.noCustomerToFollowUpFound")}</td>
                                    </tr>
                ";
            }
            $documentHtml .= " 
                                </tbody>
                            </table>
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "customers_to_follow_up_{$_SESSION[SESSION_USER_ID]}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/reports/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/reports/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printviewbestcustomersAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/reports") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $pointOfSaleDao = new PointOfSaleDAO();
            $customerInvoiceDao = new CustomerInvoiceDAO();
            $pointOfSaleId = (int)$_POST["pointOfSale"] != 0 ? (int)$_POST["pointOfSale"] : null;
            $startDate = new DateTime("{$_POST["startDate"]} 00:00:00");
            $endDate = new DateTime("{$_POST["endDate"]} 23:59:59");
            $minPurchaseSum = (float)$_POST["minPurchaseSum"];
            $minPurchaseSumFormatted = number_format($minPurchaseSum, 0, '', ' ');
            $pointOfSale = $pointOfSaleId != null ? $pointOfSaleDao->getPointOfSaleById($pointOfSaleId) : null;
            $bestCustomers = $customerInvoiceDao->getBestCustomers($startDate, $endDate, $minPurchaseSum, $pointOfSale);
            $pointOfSaleName = $pointOfSale != null ? $pointOfSale->getNomPointVente() : $this->translator->_("report.printViewBestCustomers.allPointsOfSale");
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $secondaryBrand = APP_SECONDARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = $this->translator->_("report.printViewBestCustomers.bestCustomers");
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: center;  
                                color: #000000; 
                              
                            }

                            .center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            .no-line-found {
                                color: grey !important;
                                font-weight: bold !important;
                                font-size: 16px !important;
                            }

                            td.validated {
                                color: {$primaryBrand} !important;
                                font-weight: bold;
                            }

                            td.not-validated {
                                color: {$secondaryBrand} !important;
                                font-weight: bold;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            .equalized {
                                table-layout: fixed;
                            }

                            bold {
                                color: black;
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$printTitle}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table>
                                <thead>
                                    <tr>
                                        <th class='center-text'>{$this->translator->_("report.printViewBestCustomers.pointOfSale")} : <bold class='black-bold'>{$pointOfSaleName}</bold></th>
                                        <th class='center-text'>{$this->translator->_("report.printViewBestCustomers.minPurchaseSum")} : <bold class='black-bold'>{$minPurchaseSumFormatted}</bold></th>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table>
                                <thead>
                                    <tr>
                                        <th class='center-text'>{$this->translator->_("report.printViewBestCustomers.startDate")} : <bold class='black-bold'>{$startDate->format(DATE_FORMAT)}</bold></th>
                                        <th class='center-text'>{$this->translator->_("report.printViewBestCustomers.endDate")} : <bold class='black-bold'>{$endDate->format(DATE_FORMAT)}</bold></th>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border equalized'>
                                <thead>
                                    <tr>
                                        <th>{$this->translator->_("report.printViewBestCustomers.customer")}</th>
                                        <th>{$this->translator->_("report.printViewBestCustomers.purchaseSum")}</th>
                                        <th>{$this->translator->_("report.printViewBestCustomers.phoneNumber")}</th>
                                        <th>{$this->translator->_("report.printViewBestCustomers.email")}</th>
                                    </tr>
                                </thead>
                                <tbody>
            ";
            $printFilters .= "{$this->translator->_("report.printViewBestCustomers.pointOfSale")} : {$pointOfSaleName}";
            $printFilters .= ", {$this->translator->_("report.printViewBestCustomers.minPurchaseSum")} : {$minPurchaseSumFormatted}";
            $printFilters .= ", {$this->translator->_("report.printViewBestCustomers.startDate")} : {$startDate->format(DATE_FORMAT)}";
            $printFilters .= ", {$this->translator->_("report.printViewBestCustomers.endDate")} : {$endDate->format(DATE_FORMAT)}";
            foreach ($bestCustomers as $bestCustomer) {
                $purchaseSum = number_format($bestCustomer["customer_turnover"], 0, '', ' ');
                $documentHtml .= "
                                    <tr>
                                        <td>{$bestCustomer["customer_last_name"]} {$bestCustomer["customer_first_name"]}</td>
                                        <td style='text-align: right;'>{$purchaseSum}</td>
                                        <td>{$bestCustomer["customer_phone"]}</td>
                                        <td>{$bestCustomer["customer_email"]}</td>
                                    </tr>
                ";
            }
            if (count($bestCustomers) == 0) {
                $documentHtml .= "
                                    <tr colspan='4'>
                                        <td class='no-line-found'>{$this->translator->_("report.printViewBestCustomers.noBestCustomerFound")}</td>
                                    </tr>
                ";
            }
            $documentHtml .= " 
                                </tbody>
                            </table>
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "best_customers_{$_SESSION[SESSION_USER_ID]}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/reports/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/reports/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printviewregularcustomersAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/reports") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $pointOfSaleDao = new PointOfSaleDAO();
            $customerInvoiceDao = new CustomerInvoiceDAO();
            $pointOfSaleId = (int)$_POST["pointOfSale"] != 0 ? (int)$_POST["pointOfSale"] : null;
            $startDate = new DateTime("{$_POST["startDate"]} 00:00:00");
            $endDate = new DateTime("{$_POST["endDate"]} 23:59:59");
            $minNumberOfVisits = (float)$_POST["minNumberOfVisits"];
            $pointOfSale = $pointOfSaleId != null ? $pointOfSaleDao->getPointOfSaleById($pointOfSaleId) : null;
            $regularCustomers = $customerInvoiceDao->getSatisfactionSurveyCustomers($startDate, $endDate, $minNumberOfVisits, $pointOfSale);
            $pointOfSaleName = $pointOfSale != null ? $pointOfSale->getNomPointVente() : $this->translator->_("report.printViewRegularCustomers.allPointsOfSale");
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $secondaryBrand = APP_SECONDARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = $this->translator->_("report.printViewRegularCustomers.regularCustomers");
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: center;  
                                color: #000000; 
                              
                            }

                            .center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            .no-line-found {
                                color: grey !important;
                                font-weight: bold !important;
                                font-size: 16px !important;
                            }

                            td.validated {
                                color: {$primaryBrand} !important;
                                font-weight: bold;
                            }

                            td.not-validated {
                                color: {$secondaryBrand} !important;
                                font-weight: bold;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            .equalized {
                                table-layout: fixed;
                            }

                            bold {
                                color: black;
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$printTitle}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table>
                                <thead>
                                    <tr>
                                        <th class='center-text'>{$this->translator->_("report.printViewRegularCustomers.pointOfSale")} : <bold class='black-bold'>{$pointOfSaleName}</bold></th>
                                        <th class='center-text'>{$this->translator->_("report.printViewRegularCustomers.minNumberOfVisits")} : <bold class='black-bold'>{$minNumberOfVisits}</bold></th>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table>
                                <thead>
                                    <tr>
                                        <th class='center-text'>{$this->translator->_("report.printViewRegularCustomers.startDate")} : <bold class='black-bold'>{$startDate->format(DATE_FORMAT)}</bold></th>
                                        <th class='center-text'>{$this->translator->_("report.printViewRegularCustomers.endDate")} : <bold class='black-bold'>{$endDate->format(DATE_FORMAT)}</bold></th>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border equalized'>
                                <thead>
                                    <tr>
                                        <th>{$this->translator->_("report.printViewRegularCustomers.customer")}</th>
                                        <th>{$this->translator->_("report.printViewRegularCustomers.numberOfVisits")}</th>
                                        <th>{$this->translator->_("report.printViewRegularCustomers.phoneNumber")}</th>
                                        <th>{$this->translator->_("report.printViewRegularCustomers.email")}</th>
                                    </tr>
                                </thead>
                                <tbody>
            ";
            $printFilters .= "{$this->translator->_("report.printViewRegularCustomers.pointOfSale")} : {$pointOfSaleName}";
            $printFilters .= ", {$this->translator->_("report.printViewRegularCustomers.numberOfVisits")} : {$minNumberOfVisits}";
            $printFilters .= ", {$this->translator->_("report.printViewRegularCustomers.startDate")} : {$startDate->format(DATE_FORMAT)}";
            $printFilters .= ", {$this->translator->_("report.printViewRegularCustomers.endDate")} : {$endDate->format(DATE_FORMAT)}";
            foreach ($regularCustomers as $regularCustomer) {
                $documentHtml .= "
                                    <tr>
                                        <td>{$regularCustomer["customer_last_name"]} {$regularCustomer["customer_first_name"]}</td>
                                        <td>{$regularCustomer["customer_visits"]}</td>
                                        <td>{$regularCustomer["customer_phone"]}</td>
                                        <td>{$regularCustomer["customer_email"]}</td>
                                    </tr>
                ";
            }
            if (count($regularCustomer) == 0) {
                $documentHtml .= "
                                    <tr colspan='4'>
                                        <td class='no-line-found'>{$this->translator->_("report.printViewRegularCustomers.noRegularCustomerFound")}</td>
                                    </tr>
                ";
            }
            $documentHtml .= " 
                                </tbody>
                            </table>
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "regular_customers_{$_SESSION[SESSION_USER_ID]}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/reports/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/reports/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printviewoutofstockAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/reports") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $stockDao = new StockDAO();
            $storeDao = new StoreDAO();
            $productCategoryDao = new ProductCategoryDAO();
            $storeId = (int)$_POST["store"];
            $categoryId = (int)$_POST["category"] != 0 ? (int)$_POST["categoey"] : 0;
            $store = $storeDao->getStoreById($storeId);
            $productCategory = $categoryId != null ? $productCategoryDao->getProductCategoryById($categoryId) : null;
            $outOfStockProductsIds = $stockDao->getProductsOutOfStock($store, $productCategory);
            $storeName = $store->getNomMagasin();
            $productCategoryName = $productCategory != null ? $productCategory->getNom() : $this->translator->_("report.printViewOutOfStock.allCategories");
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $secondaryBrand = APP_SECONDARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = $this->translator->_("report.printViewOutOfStock.outOfStockOfProducts");
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: center;  
                                color: #000000; 
                              
                            }

                            .center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            .no-line-found {
                                color: grey !important;
                                font-weight: bold !important;
                                font-size: 16px !important;
                            }

                            td.validated {
                                color: {$primaryBrand} !important;
                                font-weight: bold;
                            }

                            td.not-validated {
                                color: {$secondaryBrand} !important;
                                font-weight: bold;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            .equalized {
                                table-layout: fixed;
                            }

                            bold {
                                color: black;
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$printTitle}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table>
                                <thead>
                                    <tr>
                                        <th class='center-text'>{$this->translator->_("report.printViewOutOfStock.store")} : <bold class='black-bold'>{$storeName}</bold></th>
                                        <th class='center-text'>{$this->translator->_("report.printViewOutOfStock.category")} : <bold class='black-bold'>{$productCategoryName}</bold></th>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border equalized'>
                                <thead>
                                    <tr>
                                        <th>{$this->translator->_("report.printViewOutOfStock.product")}</th>
                                        <th>{$this->translator->_("report.printViewOutOfStock.lastExitDate")}</th>
                                    </tr>
                                </thead>
                                <tbody>
            ";
            $printFilters .= "{$this->translator->_("report.printViewOutOfStock.store")} : {$storeName}";
            $printFilters .= ", {$this->translator->_("report.printViewOutOfStock.category")} : {$productCategoryName}";
            $productDao = new ProductDAO();
            $exitLineDao = new ExitLineDAO();
            foreach ($outOfStockProductsIds as $outOfStockProductId) {
                $product = $productDao->getProductById($outOfStockProductId["product_id"]);
                $lastExitLine = $exitLineDao->getLastExitLineOfProduct($product);
                $momentOfLastExit = $lastExitLine != null ? new \Moment\Moment($lastExitLine->getIdSortie()->getDateValidation()->format(DATE_FORMAT)) : null;
                $formatMoment = $momentOfLastExit != null ? $momentOfLastExit->format(DATE_FORMAT_LONG) : "";
                $documentHtml .= "
                                    <tr>
                                        <td>{$product->getDesignation()}</td>
                                        <td>{$formatMoment}</td>
                                    </tr>
                ";
            }
            if (count($outOfStockProductsIds) == 0) {
                $documentHtml .= "
                                    <tr colspan='2'>
                                        <td class='no-line-found'>{$this->translator->_("report.printViewOutOfStock.noOutOfStockProductFound")}</td>
                                    </tr>
                ";
            }
            $documentHtml .= " 
                                </tbody>
                            </table>
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "out_of_stock_products_{$_SESSION[SESSION_USER_ID]}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/reports/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/reports/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printviewsalesbycashierAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/reports") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $pointOfSaleDao = new PointOfSaleDAO();
            $userDao = new UserDAO();
            $customerInvoiceDao = new CustomerInvoiceDAO();
            $invoicePaymentDao = new InvoicePaymentDAO();
            $pointsOfSaleIds = json_decode($_POST["pointsOfSale"], true);
            $cashiersIds = json_decode($_POST["cashiers"], true);
            $startDate = new DateTime("{$_POST["startDate"]}:00");
            $endDate = new DateTime("{$_POST["endDate"]}:59");
            $pointsOfSale = [];
            $cashiers = [];
            $pointOfSaleNames = "";
            $countPointOfSale = 0;
            foreach ($pointsOfSaleIds as $pointOfSaleId) {
                $pointOfSale = $pointOfSaleDao->getPointOfSaleById((int)$pointOfSaleId);
                $pointsOfSale[] = $pointOfSale;
                if ($countPointOfSale > 0) {
                    $pointOfSaleNames .= ", {$pointOfSale->getNomPointVente()}";
                } else {
                    $pointOfSaleNames .= $pointOfSale->getNomPointVente();
                }
                $countPointOfSale++;
            }
            if (count($pointsOfSale) == 0) $pointOfSaleNames = $this->translator->_("report.printViewSalesByCashier.allPointsOfSale");
            $cashierNames = "";
            $countCashier = 0;
            foreach ($cashiersIds as $cashierId) {
                $cashier = $userDao->getUserById((int)$cashierId);
                $cashiers[] = $cashier;
                if ($countCashier > 0) {
                    $cashierNames .= ", {$cashier->getIdPersonne()->getNom()} {$cashier->getIdPersonne()->getPrenom()}";
                } else {
                    $cashierNames .= "{$cashier->getIdPersonne()->getNom()} {$cashier->getIdPersonne()->getPrenom()}";
                }
            }
            if (count($cashiers) == 0) $cashierNames = $this->translator->_("report.printViewSalesByCashier.allCashiers");
            $salesPerCashiers = $customerInvoiceDao->getSalesByPointsOfSalesAndCashiers($pointsOfSale, $cashiers, $startDate, $endDate);
            $salesResult = [];
            foreach ($salesPerCashiers as $salePerCashier) {
                $salesResult[] = [
                    "id" => $salePerCashier->getIdFacture(),
                    "dateAndHour" => $salePerCashier->getDateFacture()->format(DATE_FORMAT_WITH_SECONDS),
                    "amount" => $salePerCashier->getMontantNet(),
                    "reductionAmount" => $salePerCashier->getForfaitRemise(),
                    "paidAmount" => $invoicePaymentDao->getSumOfInvoicePayments($salePerCashier),
                    "pointOfSaleId" => $salePerCashier->getIdPointVente()->getIdPointVente(),
                    "pointOfSaleName" => $salePerCashier->getIdPointVente()->getNomPointVente(),
                    "cashierId" => $salePerCashier->getIdUtilisateur()->getIdUtilisateur(),
                    "cashierName" => "{$salePerCashier->getIdUtilisateur()->getIdPersonne()->getNom()} {$salePerCashier->getIdUtilisateur()->getIdPersonne()->getPrenom()}",
                ];
            }
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $secondaryBrand = APP_SECONDARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = $this->translator->_("report.printViewSalesByCashier.salesPerCashier");
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: left;  
                                color: #000000; 
                            }

                            td.center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            .no-line-found {
                                color: grey !important;
                                font-weight: bold !important;
                                font-size: 16px !important;
                            }

                            td.validated {
                                color: {$primaryBrand} !important;
                                font-weight: bold;
                            }

                            td.not-validated {
                                color: {$secondaryBrand} !important;
                                font-weight: bold;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            .equalized {
                                table-layout: fixed;
                            }

                            bold {
                                color: black;
                            }

                            i.violet-label {
                                background-color: #6435c9;
                                color: white;
                                padding: 2px;
                                border-radius: 6px;
                                margin: 3px;
                            }

                            i.blue-label {
                                background-color: #2185d0;
                                color: white;
                                padding: 2px;
                                border-radius: 6px;
                                margin: 3px;
                            }

                            i.secondary-label {
                                background-color: {$secondaryBrand};
                                color: white;
                                padding: 2px;
                                border-radius: 6px;
                                margin: 3px;
                            }

                            th.left.aligned {
                                text-align: left;
                            }

                            td.right.aligned {
                                text-align: right;
                            }

                            tr.app-total-tr {
                                background-color: {$primaryBrand}80;
                            }

                            tr.app-total-tr > td {
                                color: {$thirdBrand};
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$printTitle}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border equalized'>
                                <thead>
                                    <tr>
                                        <th class='center-text'>{$this->translator->_("report.printViewSalesByCashier.pointsOfSale")}</th>
                                        <th class='center-text'>{$this->translator->_("report.printViewSalesByCashier.cashiers")}</th>
                                    </tr>
                                    <tr>
                                        <th class='center-text'>{$pointOfSaleNames}</th>
                                        <th class='center-text'>{$cashierNames}</th>
                                    </tr>
                                    <tr>
                                        <th class='center-text'>{$this->translator->_("report.printViewSalesByCashier.startDate")} : <bold class='black-bold'>{$startDate->format(DATE_FORMAT_WITH_TIME)}</bold></th>
                                        <th class='center-text'>{$this->translator->_("report.printViewSalesByCashier.endDate")} : <bold class='black-bold'>{$endDate->format(DATE_FORMAT_WITH_TIME)}</bold></th>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
            ";
            $printFilters .= "{$this->translator->_("report.printViewSalesByCashier.pointsOfSale")} : {$pointOfSaleNames}";
            $printFilters .= ", {$this->translator->_("report.printViewSalesByCashier.cashiers")} : {$cashierNames}";
            $printFilters .= ", {$this->translator->_("report.printViewSalesByCashier.startDate")} : {$startDate->format(DATE_FORMAT_WITH_TIME)}";
            $printFilters .= ", {$this->translator->_("report.printViewSalesByCashier.endDate")} : {$endDate->format(DATE_FORMAT_WITH_TIME)}";
            $currentSalesPointOfSaleId = null;
            $currentSalesCashierId = null;
            $salesSumForPointOfSale = [];
            $salesSumForCashierGlobal = [];
            $salesSumForCashierByPointOfSale = [];
            foreach ($salesResult as $sale) {
                if ($sale["pointOfSaleId"] != $currentSalesPointOfSaleId) {
                    $salesSumForPointOfSale[$sale["pointOfSaleId"]] = [
                        "amount" => $sale["amount"],
                        "reductionAmount" => $sale["reductionAmount"],
                        "paidAmount" => $sale["paidAmount"],
                        "name" => $sale["pointOfSaleName"],
                    ];
                    $salesSumForCashierByPointOfSale[$sale["pointOfSaleId"]] = [];
                    if ($currentSalesPointOfSaleId != null) {
                        if ($currentSalesCashierId != null) {
                            $cashierSalesSumAmount = number_format($salesSumForCashierByPointOfSale[$currentSalesPointOfSaleId][$currentSalesCashierId]["amount"], 0, '', ' ');
                            $cashierSalesSumReductionAmount = number_format($salesSumForCashierByPointOfSale[$currentSalesPointOfSaleId][$currentSalesCashierId]["reductionAmount"], 0, '', ' ');
                            $cashierSalesSumPaidAmount = number_format($salesSumForCashierByPointOfSale[$currentSalesPointOfSaleId][$currentSalesCashierId]["paidAmount"], 0, '', ' ');
                            $documentHtml .= "
                                <tr class='app-total-tr'>
                                    <td colspan='2' class='right aligned'>
                                        <i class='blue-label'>
                                            {$this->translator->_("report.printViewSalesByCashier.totals")}
                                            {$salesSumForCashierByPointOfSale[$currentSalesPointOfSaleId][$currentSalesCashierId]["name"]}
                                        </i>
                                    </td>
                                    <td class='right aligned'>{$cashierSalesSumAmount}</td>
                                    <td class='right aligned'>{$cashierSalesSumReductionAmount}</td>
                                    <td class='right aligned'>{$cashierSalesSumPaidAmount}</td>
                                </tr>
                            ";
                        }
                        $pointOfSaleSalesSumAmount = number_format($salesSumForPointOfSale[$currentSalesPointOfSaleId]["amount"], 0, '', ' ');
                        $pointOfSaleSalesSumReductionAmount = number_format($salesSumForPointOfSale[$currentSalesPointOfSaleId]["reductionAmount"], 0, '', ' ');
                        $pointOfSaleSalesSumPaidAmount = number_format($salesSumForPointOfSale[$currentSalesPointOfSaleId]["paidAmount"], 0, '', ' ');
                        $documentHtml .= "
                                    <tr class='app-total-tr'>
                                        <td colspan='2' class='right aligned'>
                                            <i class='violet-label'>
                                                {$this->translator->_("report.printViewSalesByCashier.totals")}
                                                {$salesSumForPointOfSale[$currentSalesPointOfSaleId]["name"]}
                                            </i>
                                        </td>
                                        <td class='right aligned'>{$pointOfSaleSalesSumAmount}</td>
                                        <td class='right aligned'>{$pointOfSaleSalesSumReductionAmount}</td>
                                        <td class='right aligned'>{$pointOfSaleSalesSumPaidAmount}</td>
                                    </tr>
                                </tbody>
                                <tfoot>
                                    <tr>
                                        <th>{$this->translator->_("report.printViewSalesByCashier.saleNo")}</th>
                                        <th>{$this->translator->_("report.printViewSalesByCashier.dateAndHour")}</th>
                                        <th>{$this->translator->_("report.printViewSalesByCashier.amount")}</th>
                                        <th>{$this->translator->_("report.printViewSalesByCashier.reduction")}</th>
                                        <th>{$this->translator->_("report.printViewSalesByCashier.paidAmount")}</th>
                                    </tr>
                                </tfoot>
                            </table>
                            <hr/>
                        ";
                    }
                    $documentHtml .= "
                        <table class='border equalized'>
                            <thead>
                    ";
                    if (count($pointsOfSale) !== 1) {
                        $documentHtml .= "
                                <tr>
                                    <th colspan='5' class='left aligned'>
                                        <i class='violet-label'>{$sale["pointOfSaleName"]}</i>
                                    </th>
                                </tr>
                        ";
                    }
                    $documentHtml .= "
                                <tr>
                                    <th>{$this->translator->_("report.printViewSalesByCashier.saleNo")}</th>
                                    <th>{$this->translator->_("report.printViewSalesByCashier.dateAndHour")}</th>
                                    <th>{$this->translator->_("report.printViewSalesByCashier.amount")}</th>
                                    <th>{$this->translator->_("report.printViewSalesByCashier.reduction")}</th>
                                    <th>{$this->translator->_("report.printViewSalesByCashier.paidAmount")}</th>
                                </tr>
                            </thead>
                            <tbody>
                    ";
                    $currentSalesPointOfSaleId = $sale["pointOfSaleId"];
                    $currentSalesCashierId = null;
                } else {
                    $salesSumForPointOfSale[$sale["pointOfSaleId"]]["amount"] += $sale["amount"];
                    $salesSumForPointOfSale[$sale["pointOfSaleId"]]["reductionAmount"] += $sale["reductionAmount"];
                    $salesSumForPointOfSale[$sale["pointOfSaleId"]]["paidAmount"] += $sale["paidAmount"];
                }
                if ($sale["cashierId"] != $currentSalesCashierId) {
                    $salesSumForCashierByPointOfSale[$sale["pointOfSaleId"]][$sale["cashierId"]] = [
                        "amount" => $sale["amount"],
                        "reductionAmount" => $sale["reductionAmount"],
                        "paidAmount" => $sale["paidAmount"],
                        "salesCount" => 1,
                        "salesUnpaidCount" => $sale["paidAmount"] == 0 ? 1 : 0,
                        "salesInPaymentCount" => $sale["paidAmount"] > 0 && $sale["paidAmount"] < $sale["amount"] ? 1 : 0,
                        "salesPaidCount" => $sale["paidAmount"] == $sale["amount"] ? 1 : 0,
                        "salesWithReductionCount" => $sale["reductionAmount"] > 0 ? 1 : 0,
                        "name" => $sale["cashierName"],
                    ];
                    if (!isset($salesSumForCashierGlobal[$sale["cashierId"]])) {
                        $salesSumForCashierGlobal[$sale["cashierId"]] = [
                            "amount" => $sale["amount"],
                            "reductionAmount" => $sale["reductionAmount"],
                            "paidAmount" => $sale["paidAmount"],
                            "salesCount" => 1,
                            "salesUnpaidCount" => $sale["paidAmount"] == 0 ? 1 : 0,
                            "salesInPaymentCount" => $sale["paidAmount"] > 0 && $sale["paidAmount"] < $sale["amount"] ? 1 : 0,
                            "salesPaidCount" => $sale["paidAmount"] == $sale["amount"] ? 1 : 0,
                            "salesWithReductionCount" => $sale["reductionAmount"] > 0 ? 1 : 0,
                            "name" => $sale["cashierName"],
                        ];
                    } else {
                        $salesSumForCashierGlobal[$sale["cashierId"]]["amount"] += $sale["amount"];
                        $salesSumForCashierGlobal[$sale["cashierId"]]["reductionAmount"] += $sale["reductionAmount"];
                        $salesSumForCashierGlobal[$sale["cashierId"]]["paidAmount"] += $sale["paidAmount"];
                        $salesSumForCashierGlobal[$sale["cashierId"]]["salesCount"] += 1;
                        $salesSumForCashierGlobal[$sale["cashierId"]]["salesUnpaidCount"] += $sale["paidAmount"] == 0 ? 1 : 0;
                        $salesSumForCashierGlobal[$sale["cashierId"]]["salesInPaymentCount"] += $sale["paidAmount"] > 0 && $sale["paidAmount"] < $sale["amount"] ? 1 : 0;
                        $salesSumForCashierGlobal[$sale["cashierId"]]["salesPaidCount"] += $sale["paidAmount"] == $sale["amount"] ? 1 : 0;
                        $salesSumForCashierGlobal[$sale["cashierId"]]["salesWithReductionCount"] += $sale["reductionAmount"] > 0 ? 1 : 0;
                    }
                    if ($currentSalesCashierId != null) {
                        $cashierSalesSumAmount = number_format($salesSumForCashierByPointOfSale[$currentSalesPointOfSaleId][$currentSalesCashierId]["amount"], 0, '', ' ');
                        $cashierSalesSumReductionAmount = number_format($salesSumForCashierByPointOfSale[$currentSalesPointOfSaleId][$currentSalesCashierId]["reductionAmount"], 0, '', ' ');
                        $cashierSalesSumPaidAmount = number_format($salesSumForCashierByPointOfSale[$currentSalesPointOfSaleId][$currentSalesCashierId]["paidAmount"], 0, '', ' ');
                        $documentHtml .= "
                            <tr class='app-total-tr'>
                                <td colspan='2' class='right aligned'>
                                    <i class='blue-label'>
                                        {$this->translator->_("view.cashierSales.totals")}
                                        {$salesSumForCashierByPointOfSale[$currentSalesPointOfSaleId][$currentSalesCashierId]["name"]}
                                    </i>
                                </td>
                                <td class='right aligned'>{$cashierSalesSumAmount}</td>
                                <td class='right aligned'>{$cashierSalesSumReductionAmount}</td>
                                <td class='right aligned'>{$cashierSalesSumPaidAmount}</td>
                            </tr>
                        ";
                    }
                    if (count($cashiers) != 1) {
                        $documentHtml .= "
                            <tr>
                                <td colspan='5' class='center-text'>
                                    <i class='blue-label'>{$sale["cashierName"]}</i>
                                </td>
                            </tr>
                        ";
                    }
                    $currentSalesCashierId = $sale["cashierId"];
                } else {
                    $salesSumForCashierByPointOfSale[$sale["pointOfSaleId"]][$sale["cashierId"]]["amount"] += $sale["amount"];
                    $salesSumForCashierByPointOfSale[$sale["pointOfSaleId"]][$sale["cashierId"]]["reductionAmount"] += $sale["reductionAmount"];
                    $salesSumForCashierByPointOfSale[$sale["pointOfSaleId"]][$sale["cashierId"]]["paidAmount"] += $sale["paidAmount"];
                    $salesSumForCashierByPointOfSale[$sale["pointOfSaleId"]][$sale["cashierId"]]["salesCount"] += 1;
                    $salesSumForCashierByPointOfSale[$sale["pointOfSaleId"]][$sale["cashierId"]]["salesUnpaidCount"] += $sale["paidAmount"] == 0 ? 1 : 0;
                    $salesSumForCashierByPointOfSale[$sale["pointOfSaleId"]][$sale["cashierId"]]["salesInPaymentCount"] += $sale["paidAmount"] > 0 && $sale["paidAmount"] < $sale["amount"] ? 1 : 0;
                    $salesSumForCashierByPointOfSale[$sale["pointOfSaleId"]][$sale["cashierId"]]["salesPaidCount"] += $sale["paidAmount"] == $sale["amount"] ? 1 : 0;
                    $salesSumForCashierByPointOfSale[$sale["pointOfSaleId"]][$sale["cashierId"]]["salesWithReductionCount"] += $sale["reductionAmount"] > 0 ? 1 : 0;
                    $salesSumForCashierGlobal[$sale["cashierId"]]["amount"] += $sale["amount"];
                    $salesSumForCashierGlobal[$sale["cashierId"]]["reductionAmount"] += $sale["reductionAmount"];
                    $salesSumForCashierGlobal[$sale["cashierId"]]["paidAmount"] += $sale["paidAmount"];
                    $salesSumForCashierGlobal[$sale["cashierId"]]["salesCount"] += 1;
                    $salesSumForCashierGlobal[$sale["cashierId"]]["salesUnpaidCount"] += $sale["paidAmount"] == 0 ? 1 : 0;
                    $salesSumForCashierGlobal[$sale["cashierId"]]["salesInPaymentCount"] += $sale["paidAmount"] > 0 && $sale["paidAmount"] < $sale["amount"] ? 1 : 0;
                    $salesSumForCashierGlobal[$sale["cashierId"]]["salesPaidCount"] += $sale["paidAmount"] == $sale["amount"] ? 1 : 0;
                    $salesSumForCashierGlobal[$sale["cashierId"]]["salesWithReductionCount"] += $sale["reductionAmount"] > 0 ? 1 : 0;
                }
                $saleMoment = new \Moment\Moment($sale["dateAndHour"]);
                $saleAmount = number_format($sale["amount"], 0, '', ' ');
                $saleReductionAmount = number_format($sale["reductionAmount"], 0, '', ' ');
                $salePaidAmount = number_format($sale["paidAmount"], 0, '', ' ');
                $documentHtml .= "
                    <tr>
                        <td>{$sale["id"]}</td>
                        <td>{$saleMoment->format(DATE_FORMAT_MIDDLE_WITH_SECONDS)}</td>
                        <td class='right aligned'>{$saleAmount}</td>
                        <td class='right aligned'>{$saleReductionAmount}</td>
                        <td class='right aligned'>{$salePaidAmount}</td>
                    </tr>
                ";
            }
            if ($currentSalesPointOfSaleId != null) {
                if ($currentSalesCashierId != null) {
                    $cashierSalesSumAmount = number_format($salesSumForCashierByPointOfSale[$currentSalesPointOfSaleId][$currentSalesCashierId]["amount"], 0, '', ' ');
                    $cashierSalesSumReductionAmount = number_format($salesSumForCashierByPointOfSale[$currentSalesPointOfSaleId][$currentSalesCashierId]["reductionAmount"], 0, '', ' ');
                    $cashierSalesSumPaidAmount = number_format($salesSumForCashierByPointOfSale[$currentSalesPointOfSaleId][$currentSalesCashierId]["paidAmount"], 0, '', ' ');
                    $documentHtml .= "
                        <tr class='app-total-tr'>
                            <td colspan='2' class='right aligned'>
                                <i class='blue-label'>
                                    {$this->translator->_("report.printViewSalesByCashier.totals")}
                                    {$salesSumForCashierByPointOfSale[$currentSalesPointOfSaleId][$currentSalesCashierId]["name"]}
                                </i>
                            </td>
                            <td class='right aligned'>{$cashierSalesSumAmount}</td>
                            <td class='right aligned'>{$cashierSalesSumReductionAmount}</td>
                            <td class='right aligned'>{$cashierSalesSumPaidAmount}</td>
                        </tr>
                    ";
                }
                $pointOfSaleSalesSumAmount = number_format($salesSumForPointOfSale[$currentSalesPointOfSaleId]["amount"], 0, '', ' ');
                $pointOfSaleSalesSumReductionAmount = number_format($salesSumForPointOfSale[$currentSalesPointOfSaleId]["reductionAmount"], 0, '', ' ');
                $pointOfSaleSalesSumPaidAmount = number_format($salesSumForPointOfSale[$currentSalesPointOfSaleId]["paidAmount"], 0, '', ' ');
                $documentHtml .= "
                            <tr class='app-total-tr'>
                                <td colspan='2' class='right aligned'>
                                    <i class='violet-label'>
                                        {$this->translator->_("report.printViewSalesByCashier.totals")}
                                        {$salesSumForPointOfSale[$currentSalesPointOfSaleId]["name"]}
                                    </i>
                                </td>
                                <td class='right aligned'>{$pointOfSaleSalesSumAmount}</td>
                                <td class='right aligned'>{$pointOfSaleSalesSumReductionAmount}</td>
                                <td class='right aligned'>{$pointOfSaleSalesSumPaidAmount}</td>
                            </tr>
                        </tbody>
                        <tfoot>
                            <tr>
                                <th>{$this->translator->_("report.printViewSalesByCashier.saleNo")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCashier.dateAndHour")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCashier.amount")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCashier.reduction")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCashier.paidAmount")}</th>
                            </tr>
                        </tfoot>
                    </table>
                    <hr/>
                ";
            }
            if (count($salesResult) > 0) {
                $salesTotal = 0;
                $paymentTotal = 0;
                $reductionTotal = 0;
                $numberOfSales = 0;
                $paid = 0;
                $paymentInProgess = 0;
                $unpaid = 0;
                $withReduction = 0;
                $documentHtml .= "
                    <table class='border'>
                        <thead>
                            <tr>
                                <th colspan='9' class='left aligned'>
                                    <i class='secondary-label'>{$this->translator->_("report.printViewSalesByCashier.summaryOfSalesPerCashier")}</i>
                                </th>
                            </tr>
                            <tr>
                                <th style='width: 20%;'>{$this->translator->_("report.printViewSalesByCashier.cashier")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewSalesByCashier.salesTotal")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewSalesByCashier.paymentTotal")}</th>
                                <th style='width: 10%'>{$this->translator->_("report.printViewSalesByCashier.reductionTotal")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCashier.numberOfSales")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCashier.paid")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCashier.paymentInProgess")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCashier.unpaid")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCashier.withReduction")}</th>
                            </tr>
                        </thead>
                        <tbody>
                ";
                foreach ($salesSumForCashierGlobal as $cashierId => $cashierValues) {
                    $salesTotal += $cashierValues["amount"];
                    $paymentTotal += $cashierValues["paidAmount"];
                    $reductionTotal += $cashierValues["reductionAmount"];
                    $numberOfSales += $cashierValues["salesCount"];
                    $paid += $cashierValues["salesPaidCount"];
                    $paymentInProgess += $cashierValues["salesInPaymentCount"];
                    $unpaid += $cashierValues["salesUnpaidCount"];
                    $withReduction += $cashierValues["salesWithReductionCount"];
                    $cashierAmount = number_format($cashierValues["amount"], 0, '', ' ');
                    $cashierPaiAmount = number_format($cashierValues["paidAmount"], 0, '', ' ');
                    $cashierReductionAmount = number_format($cashierValues["reductionAmount"], 0, '', ' ');
                    $documentHtml .= "
                        <tr>
                            <td>{$cashierValues["name"]}</td>
                            <td class='right aligned'>{$cashierAmount}</td>
                            <td class='right aligned'>{$cashierPaiAmount}</td>
                            <td class='right aligned'>{$cashierReductionAmount}</td>
                            <td>{$cashierValues["salesCount"]}</td>
                            <td>{$cashierValues["salesPaidCount"]}</td>
                            <td>{$cashierValues["salesInPaymentCount"]}</td>
                            <td>{$cashierValues["salesUnpaidCount"]}</td>
                            <td>{$cashierValues["salesWithReductionCount"]}</td>
                        </tr>
                    ";
                };
                $salesTotalFormat = number_format($salesTotal, 0, '', ' ');
                $paymentTotalFormat = number_format($paymentTotal, 0, '', ' ');
                $reductionTotalFormat = number_format($reductionTotal, 0, '', ' ');
                $documentHtml .= "
                            <tr class='app-total-tr'>
                                <td class='right aligned'>
                                    <i class='secondary-label'>{$this->translator->_("report.printViewSalesByCashier.totals")}</i>
                                </td>
                                <td class='right aligned'>{$salesTotalFormat}</td>
                                <td class='right aligned'>{$paymentTotalFormat}</td>
                                <td class='right aligned'>{$reductionTotalFormat}</td>
                                <td>{$numberOfSales}</td>
                                <td>{$paid}</td>
                                <td>{$paymentInProgess}</td>
                                <td>{$unpaid}</td>
                                <td>{$withReduction}</td>
                            </tr>
                        </tbody>
                        <tfoot>
                            <tr>
                                <th>{$this->translator->_("report.printViewSalesByCashier.cashier")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCashier.salesTotal")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCashier.paymentTotal")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCashier.reductionTotal")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCashier.numberOfSales")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCashier.paid")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCashier.paymentInProgess")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCashier.unpaid")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCashier.withReduction")}</th>
                            </tr>
                        </tfoot>
                    </table>
                ";
            }
            $documentHtml .= " 
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "sales_per_cashier_{$_SESSION[SESSION_USER_ID]}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/reports/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/reports/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printviewproductssoldbycashierAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/reports") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $pointOfSaleDao = new PointOfSaleDAO();
            $userDao = new UserDAO();
            $customerInvoiceLineDao = new CustomerInvoiceLineDAO();
            $pointsOfSaleIds = json_decode($_POST["pointsOfSale"], true);
            $cashiersIds = json_decode($_POST["cashiers"], true);
            $startDate = new DateTime("{$_POST["startDate"]}:00");
            $endDate = new DateTime("{$_POST["endDate"]}:59");
            $pointsOfSale = [];
            $cashiers = [];
            $pointOfSaleNames = "";
            $countPointOfSale = 0;
            foreach ($pointsOfSaleIds as $pointOfSaleId) {
                $pointOfSale = $pointOfSaleDao->getPointOfSaleById((int)$pointOfSaleId);
                $pointsOfSale[] = $pointOfSale;
                if ($countPointOfSale > 0) {
                    $pointOfSaleNames .= ", {$pointOfSale->getNomPointVente()}";
                } else {
                    $pointOfSaleNames .= $pointOfSale->getNomPointVente();
                }
                $countPointOfSale++;
            }
            if (count($pointsOfSale) == 0) $pointOfSaleNames = $this->translator->_("report.printViewProductsSoldByCashier.allPointsOfSale");
            $cashierNames = "";
            $countCashier = 0;
            foreach ($cashiersIds as $cashierId) {
                $cashier = $userDao->getUserById((int)$cashierId);
                $cashiers[] = $cashier;
                if ($countCashier > 0) {
                    $cashierNames .= ", {$cashier->getIdPersonne()->getNom()} {$cashier->getIdPersonne()->getPrenom()}";
                } else {
                    $cashierNames .= "{$cashier->getIdPersonne()->getNom()} {$cashier->getIdPersonne()->getPrenom()}";
                }
            }
            if (count($cashiers) == 0) $cashierNames = $this->translator->_("report.printViewProductsSoldByCashier.allCashiers");
            $productsCountPerCashiers = $customerInvoiceLineDao->getSoldArticleCountByPointsOfSaleAndCashiers($pointsOfSale, $cashiers, $startDate, $endDate);
            $productsCount = [];
            foreach ($productsCountPerCashiers as $productCountPerCashier) {
                $cashier = $userDao->getUserById((int)$productCountPerCashier["cashier_id"]);
                $productsCount[] = [
                    "designation" => $productCountPerCashier["product_name"],
                    "productId" => $productCountPerCashier["product_id"],
                    "category" => $productCountPerCashier["category_name"],
                    "categoryId" => $productCountPerCashier["category_id"],
                    "quantity" => (int)$productCountPerCashier["sold_quantity"],
                    "buyingPrice" => (float)$productCountPerCashier["buying_price"],
                    "unitPrice" => (float)$productCountPerCashier["unit_price"],
                    "salesSum" => (float)$productCountPerCashier["sold_amount"],
                    "profit" => (float)$productCountPerCashier["profit"],
                    "pointOfSaleId" => (int)$productCountPerCashier["point_of_sale_id"],
                    "pointOfSaleName" => $productCountPerCashier["point_of_sale_name"],
                    "cashierId" => $productCountPerCashier["cashier_id"],
                    "cashierName" => "{$cashier->getIdPersonne()->getNom()} {$cashier->getIdPersonne()->getPrenom()}",
                ];
            }
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $secondaryBrand = APP_SECONDARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = $this->translator->_("report.printViewProductsSoldByCashier.productsSoldPerCashier");
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: left;  
                                color: #000000; 
                            }

                            td.center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            .no-line-found {
                                color: grey !important;
                                font-weight: bold !important;
                                font-size: 16px !important;
                            }

                            td.validated {
                                color: {$primaryBrand} !important;
                                font-weight: bold;
                            }

                            td.not-validated {
                                color: {$secondaryBrand} !important;
                                font-weight: bold;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            .equalized {
                                table-layout: fixed;
                            }

                            bold {
                                color: black;
                            }

                            i.violet-label {
                                background-color: #6435c9;
                                color: white;
                                padding: 2px;
                                border-radius: 6px;
                                margin: 3px;
                            }

                            i.blue-label {
                                background-color: #2185d0;
                                color: white;
                                padding: 2px;
                                border-radius: 6px;
                                margin: 3px;
                            }

                            i.secondary-label {
                                background-color: {$secondaryBrand};
                                color: white;
                                padding: 2px;
                                border-radius: 6px;
                                margin: 3px;
                            }

                            th.left.aligned {
                                text-align: left;
                            }

                            td.right.aligned {
                                text-align: right;
                            }

                            tr.app-total-tr {
                                background-color: {$primaryBrand}80;
                            }

                            tr.app-total-tr > td {
                                color: {$thirdBrand};
                            }

                            tr.app-highlight-tr {
                                background-color: {$secondaryBrand}80;
                            }

                            tr.app-highlight-tr > td {
                                color: {$thirdBrand};
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$printTitle}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border equalized'>
                                <thead>
                                    <tr>
                                        <th class='center-text'>{$this->translator->_("report.printViewProductsSoldByCashier.pointsOfSale")}</th>
                                        <th class='center-text'>{$this->translator->_("report.printViewProductsSoldByCashier.cashiers")}</th>
                                    </tr>
                                    <tr>
                                        <th class='center-text'>{$pointOfSaleNames}</th>
                                        <th class='center-text'>{$cashierNames}</th>
                                    </tr>
                                    <tr>
                                        <th class='center-text'>{$this->translator->_("report.printViewProductsSoldByCashier.startDate")} : <bold class='black-bold'>{$startDate->format(DATE_FORMAT_WITH_TIME)}</bold></th>
                                        <th class='center-text'>{$this->translator->_("report.printViewProductsSoldByCashier.endDate")} : <bold class='black-bold'>{$endDate->format(DATE_FORMAT_WITH_TIME)}</bold></th>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
            ";
            $printFilters .= "{$this->translator->_("report.printViewProductsSoldByCashier.pointsOfSale")} : {$pointOfSaleNames}";
            $printFilters .= ", {$this->translator->_("report.printViewProductsSoldByCashier.cashiers")} : {$cashierNames}";
            $printFilters .= ", {$this->translator->_("report.printViewProductsSoldByCashier.startDate")} : {$startDate->format(DATE_FORMAT_WITH_TIME)}";
            $printFilters .= ", {$this->translator->_("report.printViewProductsSoldByCashier.endDate")} : {$endDate->format(DATE_FORMAT_WITH_TIME)}";
            $currentProductsPointOfSaleId = null;
            $currentProductsCashierId = null;
            $currentProductsCategoryId = null;
            $currentProductsCategoryName = null;
            $currentCategoryTotalQuantity = 0;
            $currentCategoryTotalSalesSum = 0;
            $currentCategoryTotalProfit = 0;
            $productsSumForPointOfSale = [];
            $productsSumForCashierGlobal = [];
            $productsSumForCashierByPointOfSale = [];
            foreach ($productsCount as $product) {
                if ($product["pointOfSaleId"] != $currentProductsPointOfSaleId) {
                    $productsSumForPointOfSale[$product["pointOfSaleId"]] = [
                        "quantity" => $product["quantity"],
                        "salesSum" => $product["salesSum"],
                        "profit" => $product["profit"],
                        "name" => $product["pointOfSaleName"],
                    ];
                    $productsSumForCashierByPointOfSale[$product["pointOfSaleId"]] = [];
                    if ($currentProductsPointOfSaleId != null) {
                        if ($currentProductsCashierId != null) {
                            if ($currentProductsCategoryId != null) {
                                $categoryTotalSalesSum = number_format($currentCategoryTotalSalesSum, 0, '', ' ');
                                $categoryTotalProfit = number_format($currentCategoryTotalProfit, 0, '', ' ');
                                $documentHtml .= "
                                    <tr class='app-total-tr'>
                                        <td class='right aligned'>
                                            {$this->translator->_("report.printViewProductsSoldByCashier.totals")}
                                            {$currentProductsCategoryName}
                                        </td>
                                        <td>{$currentCategoryTotalQuantity}</td>
                                        <td colspan='2'></td>
                                        <td class='right aligned'>{$categoryTotalSalesSum}</td>
                                        <td class='right aligned'>{$categoryTotalProfit}</td>
                                    </tr>
                                ";
                            }
                            $cashierProductsSoldSum = number_format($productsSumForCashierByPointOfSale[$currentProductsPointOfSaleId][$currentProductsCashierId]["salesSum"], 0, '', ' ');
                            $cashierProductsProfit = number_format($productsSumForCashierByPointOfSale[$currentProductsPointOfSaleId][$currentProductsCashierId]["profit"], 0, '', ' ');
                            $documentHtml .= "
                                    <tr class='app-total-tr'>
                                        <td class='right aligned'>
                                            <i class='blue-label'>
                                                {$this->translator->_("report.printViewProductsSoldByCashier.totals")}
                                                {$productsSumForCashierByPointOfSale[$currentProductsPointOfSaleId][$currentProductsCashierId]["name"]}
                                            </i>
                                        </td>
                                        <td>{$productsSumForCashierByPointOfSale[$currentProductsPointOfSaleId][$currentProductsCashierId]["quantity"]}</td>
                                        <td colspan='2'></td>
                                        <td class='right aligned'>{$cashierProductsSoldSum}</td>
                                        <td class='right aligned'>{$cashierProductsProfit}</td>
                                    </tr>
                            ";
                        }
                        $pointOfSaleProductsSoldSum = number_format($productsSumForPointOfSale[$currentProductsPointOfSaleId]["salesSum"], 0, '', ' ');
                        $pointOfSaleProductsProfit = number_format($productsSumForPointOfSale[$currentProductsPointOfSaleId]["profit"], 0, '', ' ');
                        $documentHtml .= "
                                    <tr class='app-total-tr'>
                                        <td class='right aligned'>
                                            <i class='violet-label'>
                                                {$this->translator->_("report.printViewProductsSoldByCashier.totals")}
                                                {$productsSumForPointOfSale[$currentProductsPointOfSaleId]["name"]}
                                            </i>
                                        </td>
                                        <td>{$productsSumForPointOfSale[$currentProductsPointOfSaleId]["quantity"]}</td>
                                        <td colspan='2'></td>
                                        <td class='right aligned'>{$pointOfSaleProductsSoldSum}</td>
                                        <td class='right aligned'>{$pointOfSaleProductsProfit}</td>
                                    </tr>
                                <tbody>
                                <tfoot>
                                    <tr>
                                        <th style='width: 40%;'>{$this->translator->_("report.printViewProductsSoldByCashier.designation")}</th>
                                        <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByCashier.quantity")}</th>
                                        <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByCashier.buyingPrice")}</th>
                                        <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByCashier.unitPrice")}</th>
                                        <th style='width: 15%;'>{$this->translator->_("report.printViewProductsSoldByCashier.sumOfSales")}</th>
                                        <th style='width: 15%;'>{$this->translator->_("report.printViewProductsSoldByCashier.profit")}</th>
                                    </tr>
                                </tfoot>
                            </table>
                            <hr/>
                        ";
                    }
                    $documentHtml .= "
                        <table class='border'>
                            <thead>
                    ";
                    if (count($pointsOfSale) != 1) {
                        $documentHtml .= "
                                <tr>
                                    <th colspan='6' class='left aligned'>
                                        <i class='violet-label'>{$product["pointOfSaleName"]}</i>
                                    </th>
                                </tr>
                    ";
                    }
                    $documentHtml .= "
                                <tr>
                                    <th style='width: 40%;'>{$this->translator->_("report.printViewProductsSoldByCashier.designation")}</th>
                                    <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByCashier.quantity")}</th>
                                    <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByCashier.buyingPrice")}</th>
                                    <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByCashier.unitPrice")}</th>
                                    <th style='width: 15%;'>{$this->translator->_("report.printViewProductsSoldByCashier.sumOfSales")}</th>
                                    <th style='width: 15%;'>{$this->translator->_("report.printViewProductsSoldByCashier.profit")}</th>
                                </tr>
                            </thead>
                            <tbody>
                    ";
                    $currentProductsPointOfSaleId = $product["pointOfSaleId"];
                    $currentProductsCashierId = null;
                } else {
                    $productsSumForPointOfSale[$product["pointOfSaleId"]]["quantity"] += $product["quantity"];
                    $productsSumForPointOfSale[$product["pointOfSaleId"]]["salesSum"] += $product["salesSum"];
                    $productsSumForPointOfSale[$product["pointOfSaleId"]]["profit"] += $product["profit"];
                }
                if ($product["cashierId"] != $currentProductsCashierId) {
                    $productsSumForCashierByPointOfSale[$product["pointOfSaleId"]][$product["cashierId"]] = [
                        "quantity" => $product["quantity"],
                        "salesSum" => $product["salesSum"],
                        "profit" => $product["profit"],
                        "name" => $product["cashierName"],
                    ];
                    if (!isset($productsSumForCashierGlobal[$product["cashierId"]])) {
                        $productsSumForCashierGlobal[$product["cashierId"]] = [
                            "quantity" => $product["quantity"],
                            "salesSum" => $product["salesSum"],
                            "profit" => $product["profit"],
                            "products" => [],
                            "name" => $product["cashierName"],
                        ];
                    } else {
                        $productsSumForCashierGlobal[$product["cashierId"]]["quantity"] += $product["quantity"];
                        $productsSumForCashierGlobal[$product["cashierId"]]["salesSum"] += $product["salesSum"];
                        $productsSumForCashierGlobal[$product["cashierId"]]["profit"] += $product["profit"];
                    }
                    if (!isset($productsSumForCashierGlobal[$product["cashierId"]]["products"][$product["productId"]])) {
                        $productsSumForCashierGlobal[$product["cashierId"]]["products"][$product["productId"]] = [
                            "quantity" => $product["quantity"],
                            "buyingPrice" => $product["buyingPrice"],
                            "unitPrice" => $product["unitPrice"],
                            "salesSum" => $product["salesSum"],
                            "profit" => $product["profit"],
                            "name" => $product["designation"],
                            "categoryId" => $product["categoryId"],
                            "category" => $product["category"],
                        ];
                    } else {
                        $productsSumForCashierGlobal[$product["cashierId"]]["products"][$product["productId"]]["quantity"] += $product["quantity"];
                        $productsSumForCashierGlobal[$product["cashierId"]]["products"][$product["productId"]]["salesSum"] += $product["salesSum"];
                        $productsSumForCashierGlobal[$product["cashierId"]]["products"][$product["productId"]]["profit"] += $product["profit"];
                    }
                    if ($currentProductsCashierId != null) {
                        if ($currentProductsCategoryId != null) {
                            $categoryTotalSalesSum = number_format($currentCategoryTotalSalesSum, 0, '', ' ');
                            $categoryTotalProfit = number_format($currentCategoryTotalProfit, 0, '', ' ');
                            $documentHtml .= "
                                <tr class='app-total-tr'>
                                    <td class='right aligned'>
                                        {$this->translator->_("report.printViewProductsSoldByCashier.totals")}
                                        {$currentProductsCategoryName}
                                    </td>
                                    <td>{$currentCategoryTotalQuantity}</td>
                                    <td colspan='2'></td>
                                    <td class='right aligned'>{$categoryTotalSalesSum}</td>
                                    <td class='right aligned'>{$categoryTotalProfit}</td>
                                </tr>
                            ";
                        }
                        $cashierProductsSoldSum = number_format($productsSumForCashierByPointOfSale[$currentProductsPointOfSaleId][$currentProductsCashierId]["salesSum"], 0, '', ' ');
                        $cashierProductsProfit = number_format($productsSumForCashierByPointOfSale[$currentProductsPointOfSaleId][$currentProductsCashierId]["profit"], 0, '', ' ');
                        $documentHtml .= "
                                <tr class='app-total-tr'>
                                    <td class='right aligned'>
                                        <i class='blue-label'>
                                            {$this->translator->_("report.printViewProductsSoldByCashier.totals")}
                                            {$productsSumForCashierByPointOfSale[$currentProductsPointOfSaleId][$currentProductsCashierId]["name"]}
                                        </i>
                                    </td>
                                    <td>{$productsSumForCashierByPointOfSale[$currentProductsPointOfSaleId][$currentProductsCashierId]["quantity"]}</td>
                                    <td colspan='2'></td>
                                    <td class='right aligned'>{$cashierProductsSoldSum}</td>
                                    <td class='right aligned'>{$cashierProductsProfit}</td>
                                </tr>
                        ";
                    }
                    if (count($cashiers) !== 1) {
                        $documentHtml .= "
                            <tr>
                                <td colspan='6' class='center-text'>
                                    <i class='blue-label'>{$product["cashierName"]}</i>
                                </td>
                            </tr>
                        ";
                    }
                    $currentProductsCashierId = $product["cashierId"];
                    $currentProductsCategoryId = null;
                    $currentProductsCategoryName = null;
                    $currentCategoryTotalQuantity = 0;
                    $currentCategoryTotalSalesSum = 0;
                    $currentCategoryTotalProfit = 0;
                } else {
                    $productsSumForCashierByPointOfSale[$product["pointOfSaleId"]][$product["cashierId"]]["quantity"] += $product["quantity"];
                    $productsSumForCashierByPointOfSale[$product["pointOfSaleId"]][$product["cashierId"]]["salesSum"] += $product["salesSum"];
                    $productsSumForCashierByPointOfSale[$product["pointOfSaleId"]][$product["cashierId"]]["profit"] += $product["profit"];
                    $productsSumForCashierGlobal[$product["cashierId"]]["quantity"] += $product["quantity"];
                    $productsSumForCashierGlobal[$product["cashierId"]]["salesSum"] += $product["salesSum"];
                    $productsSumForCashierGlobal[$product["cashierId"]]["profit"] += $product["profit"];
                    if (isset($productsSumForCashierGlobal[$product["cashierId"]]["products"][$product["productId"]])) {
                        $productsSumForCashierGlobal[$product["cashierId"]]["products"][$product["productId"]]["quantity"] += $product["quantity"];
                        $productsSumForCashierGlobal[$product["cashierId"]]["products"][$product["productId"]]["salesSum"] += $product["salesSum"];
                        $productsSumForCashierGlobal[$product["cashierId"]]["products"][$product["productId"]]["profit"] += $product["profit"];
                    } else {
                        $productsSumForCashierGlobal[$product["cashierId"]]["products"][$product["productId"]] = [
                            "quantity" => $product["quantity"],
                            "buyingPrice" => $product["buyingPrice"],
                            "unitPrice" => $product["unitPrice"],
                            "salesSum" => $product["salesSum"],
                            "profit" => $product["profit"],
                            "name" => $product["designation"],
                            "categoryId" => $product["categoryId"],
                            "category" => $product["category"],
                        ];
                    }
                }
                if ($product["categoryId"] != $currentProductsCategoryId) {
                    if ($currentProductsCategoryId != null) {
                        $categoryTotalSalesSum = number_format($currentCategoryTotalSalesSum, 0, '', ' ');
                        $categoryTotalProfit = number_format($currentCategoryTotalProfit, 0, '', ' ');
                        $documentHtml .= "
                            <tr class='app-total-tr'>
                                <td class='right aligned'>
                                    {$this->translator->_("report.printViewProductsSoldByCashier.totals")}
                                    {$currentProductsCategoryName}
                                </td>
                                <td>{$currentCategoryTotalQuantity}</td>
                                <td colspan='2'></td>
                                <td class='right aligned'>{$categoryTotalSalesSum}</td>
                                <td class='right aligned'>{$categoryTotalProfit}</td>
                            </tr>
                        ";
                    }
                    $documentHtml .= "
                        <tr class='app-highlight-tr'>
                            <td colspan='6'>{$product["category"]}</td>
                        </tr>
                    ";
                    $currentProductsCategoryId = $product["categoryId"];
                    $currentProductsCategoryName = $product["category"];
                    $currentCategoryTotalQuantity = $product["quantity"];
                    $currentCategoryTotalSalesSum = $product["salesSum"];
                    $currentCategoryTotalProfit = $product["profit"];
                } else {
                    $currentCategoryTotalQuantity += $product["quantity"];
                    $currentCategoryTotalSalesSum += $product["salesSum"];
                    $currentCategoryTotalProfit += $product["profit"];
                }
                $buyingPrice = number_format($product["buyingPrice"], 0, '', ' ');
                $unitPrice = number_format($product["unitPrice"], 0, '', ' ');
                $salesSum = number_format($product["salesSum"], 0, '', ' ');
                $profit = number_format($product["profit"], 0, '', ' ');
                $documentHtml .= "
                    <tr>
                        <td>{$product["designation"]}</td>
                        <td>{$product["quantity"]}</td>
                        <td class='right aligned'>{$buyingPrice}</td>
                        <td class='right aligned'>{$unitPrice}</td>
                        <td class='right aligned'>{$salesSum}</td>
                        <td class='right aligned'>{$profit}</td>
                    </tr>
                ";
            }
            if ($currentProductsPointOfSaleId != null) {
                if ($currentProductsCashierId != null) {
                    if ($currentProductsCategoryId != null) {
                        $categoryTotalSalesSum = number_format($currentCategoryTotalSalesSum, 0, '', ' ');
                        $categoryTotalProfit = number_format($currentCategoryTotalProfit, 0, '', ' ');
                        $documentHtml .= "
                            <tr class='app-total-tr'>
                                <td class='right aligned'>
                                    {$this->translator->_("report.printViewProductsSoldByCashier.totals")}
                                    {$currentProductsCategoryName}
                                </td>
                                <td>{$currentCategoryTotalQuantity}</td>
                                <td colspan='2'></td>
                                <td class='right aligned'>{$categoryTotalSalesSum}</td>
                                <td class='right aligned'>{$categoryTotalProfit}</td>
                            </tr>
                        ";
                    }
                    $cashierProductsSoldSum = number_format($productsSumForCashierByPointOfSale[$currentProductsPointOfSaleId][$currentProductsCashierId]["salesSum"], 0, '', ' ');
                    $cashierProductsProfit = number_format($productsSumForCashierByPointOfSale[$currentProductsPointOfSaleId][$currentProductsCashierId]["profit"], 0, '', ' ');
                    $documentHtml .= "
                            <tr class='app-total-tr'>
                                <td class='right aligned'>
                                    <i class='blue-label'>
                                        {$this->translator->_("report.printViewProductsSoldByCashier.totals")}
                                        {$productsSumForCashierByPointOfSale[$currentProductsPointOfSaleId][$currentProductsCashierId]["name"]}
                                    </i>
                                </td>
                                <td>{$productsSumForCashierByPointOfSale[$currentProductsPointOfSaleId][$currentProductsCashierId]["quantity"]}</td>
                                <td colspan='2'></td>
                                <td class='right aligned'>{$cashierProductsSoldSum}</td>
                                <td class='right aligned'>{$cashierProductsProfit}</td>
                            </tr>
                    ";
                }
                $pointOfSaleProductsSoldSum = number_format($productsSumForPointOfSale[$currentProductsPointOfSaleId]["salesSum"], 0, '', ' ');
                $pointOfSaleProductsProfit = number_format($productsSumForPointOfSale[$currentProductsPointOfSaleId]["profit"], 0, '', ' ');
                $documentHtml .= "
                            <tr class='app-total-tr'>
                                <td class='right aligned'>
                                    <i class='violet-label'>
                                        {$this->translator->_("report.printViewProductsSoldByCashier.totals")}
                                        {$productsSumForPointOfSale[$currentProductsPointOfSaleId]["name"]}
                                    </i>
                                </td>
                                <td>{$productsSumForPointOfSale[$currentProductsPointOfSaleId]["quantity"]}</td>
                                <td colspan='2'></td>
                                <td class='right aligned'>{$pointOfSaleProductsSoldSum}</td>
                                <td class='right aligned'>{$pointOfSaleProductsProfit}</td>
                            </tr>
                        <tbody>
                        <tfoot>
                            <tr>
                                <th style='width: 40%;'>{$this->translator->_("report.printViewProductsSoldByCashier.designation")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByCashier.quantity")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByCashier.buyingPrice")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByCashier.unitPrice")}</th>
                                <th style='width: 15%;'>{$this->translator->_("report.printViewProductsSoldByCashier.sumOfSales")}</th>
                                <th style='width: 15%;'>{$this->translator->_("report.printViewProductsSoldByCashier.profit")}</th>
                            </tr>
                        </tfoot>
                    </table>
                    <hr/>
                ";
            }
            if (count($pointsOfSale) !== 1 && count($productsCount) > 0) {
                $quantityTotal = 0;
                $salesSumTotal = 0;
                $profitTotal = 0;
                $summaryProductsCategoryId = null;
                $summaryProductsCategoryName = null;
                $summaryCategoryTotalQuantity = 0;
                $summaryCategoryTotalSalesSum = 0;
                $summaryCategoryTotalProfit = 0;
                $documentHtml .= "
                    <table class='border'>
                        <thead>
                            <tr>
                                <th colspan='6' class='left aligned'>
                                    <i class='secondary-label'>{$this->translator->_("report.printViewProductsSoldByCashier.summaryOfProductsSoldPerCashier")}</i>
                                </th>
                            </tr>
                            <tr>
                                <th style='width: 40%;'>{$this->translator->_("report.printViewProductsSoldByCashier.designation")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByCashier.quantity")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByCashier.buyingPrice")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByCashier.unitPrice")}</th>
                                <th style='width: 15%;'>{$this->translator->_("report.printViewProductsSoldByCashier.sumOfSales")}</th>
                                <th style='width: 15%;'>{$this->translator->_("report.printViewProductsSoldByCashier.profit")}</th>
                            </tr>
                        </thead>
                        <tbody>
                ";
                foreach ($productsSumForCashierGlobal as $cashierId => $cashierValues) {
                    $quantityTotal += $cashierValues["quantity"];
                    $salesSumTotal += $cashierValues["salesSum"];
                    $profitTotal += $cashierValues["profit"];
                    $cashierSalesSum = number_format($cashierValues["salesSum"], 0, '', ' ');
                    $cashierProfit = number_format($cashierValues["profit"], 0, '', ' ');
                    $documentHtml .= "
                        <tr>
                            <td colspan='6' class='center-text'>
                                <i class='blue-label'>{$cashierValues["name"]}</i>
                            </td>
                        </tr>
                    ";
                    foreach ($cashierValues["products"] as $productId => $productValues) {
                        if ($productValues["categoryId"] != $summaryProductsCategoryId) {
                            if ($summaryProductsCategoryId != null) {
                                $categoryTotalSalesSum = number_format($summaryCategoryTotalSalesSum, 0, '', ' ');
                                $categoryTotalProfit = number_format($summaryCategoryTotalProfit, 0, '', ' ');
                                $documentHtml .= "
                                    <tr class='app-total-tr'>
                                        <td class='right aligned'>
                                            {$this->translator->_("report.printViewProductsSoldByCashier.totals")}
                                            {$summaryProductsCategoryName}
                                        </td>
                                        <td>{$summaryCategoryTotalQuantity}</td>
                                        <td colspan='2'></td>
                                        <td class='right aligned'>{$categoryTotalSalesSum}</td>
                                        <td class='right aligned'>{$categoryTotalProfit}</td>
                                    </tr>
                                ";
                            }
                            $documentHtml .= "
                                <tr class='app-highlight-tr'>
                                    <td colspan='6'>{$productValues["category"]}</td>
                                </tr>
                            ";
                            $summaryProductsCategoryId = $productValues["categoryId"];
                            $summaryProductsCategoryName = $productValues["category"];
                            $summaryCategoryTotalQuantity = $productValues["quantity"];
                            $summaryCategoryTotalSalesSum = $productValues["salesSum"];
                            $summaryCategoryTotalProfit = $productValues["profit"];
                        } else {
                            $summaryCategoryTotalQuantity += $productValues["quantity"];
                            $summaryCategoryTotalSalesSum += $productValues["salesSum"];
                            $summaryCategoryTotalProfit += $productValues["profit"];
                        }
                        $productBuyingPrice = number_format($productValues["buyingPrice"], 0, '', ' ');
                        $productUnitPrice = number_format($productValues["unitPrice"], 0, '', ' ');
                        $productSalesSum = number_format($productValues["salesSum"], 0, '', ' ');
                        $productProfit = number_format($productValues["profit"], 0, '', ' ');
                        $documentHtml .= "
                            <tr>
                                <td>{$productValues["name"]}</td>
                                <td>{$productValues["quantity"]}</td>
                                <td class='right aligned'>{$productBuyingPrice}</td>
                                <td class='right aligned'>{$productUnitPrice}</td>
                                <td class='right aligned'>{$productSalesSum}</td>
                                <td class='right aligned'>{$productProfit}</td>
                            </tr>
                        ";
                    };
                    if ($summaryProductsCategoryId != null) {
                        $categoryTotalSalesSum = number_format($summaryCategoryTotalSalesSum, 0, '', ' ');
                        $categoryTotalProfit = number_format($summaryCategoryTotalProfit, 0, '', ' ');
                        $documentHtml .= "
                            <tr class='app-total-tr'>
                                <td class='right aligned'>
                                    {$this->translator->_("report.printViewProductsSoldByCashier.totals")}
                                    {$summaryProductsCategoryName}
                                </td>
                                <td>{$summaryCategoryTotalQuantity}</td>
                                <td colspan='2'></td>
                                <td class='right aligned'>{$categoryTotalSalesSum}</td>
                                <td class='right aligned'>{$categoryTotalProfit}</td>
                            </tr>
                        ";
                    }
                    $documentHtml .= "
                        <tr class='app-total-tr'>
                            <td class='right aligned'>
                                <i class='blue-label'>
                                    {$this->translator->_("report.printViewProductsSoldByCashier.totals")}
                                    {$cashierValues["name"]}
                                </i>
                            </td>
                            <td>{$cashierValues["quantity"]}</td>
                            <td colspan='2'></td>
                            <td class='right aligned'>{$cashierSalesSum}</td>
                            <td class='right aligned'>{$cashierProfit}</td>
                        </tr>
                    ";
                    $summaryProductsCategoryId = null;
                    $summaryProductsCategoryName = null;
                    $summaryCategoryTotalQuantity = 0;
                    $summaryCategoryTotalSalesSum = 0;
                    $summaryCategoryTotalProfit = 0;
                };
                $salesSumTotalFormat = number_format($salesSumTotal, 0, '', ' ');
                $profitTotalFormat = number_format($profitTotal, 0, '', ' ');
                $documentHtml .= "
                            <tr class='app-total-tr'>
                                <td class='right aligned'>
                                    <i class='secondary-label'>{$this->translator->_("report.printViewProductsSoldByCashier.totals")}</a>
                                </td>
                                <td>{$quantityTotal}</td>
                                <td colspan='2'></span>
                                <td class='right aligned'>{$salesSumTotalFormat}</td>
                                <td class='right aligned'>{$profitTotalFormat}</td>
                            </tr>
                        </tbody>
                        <tfoot>
                            <tr>
                                <th style='width: 40%;'>{$this->translator->_("report.printViewProductsSoldByCashier.designation")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByCashier.quantity")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByCashier.buyingPrice")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByCashier.unitPrice")}</th>
                                <th style='width: 15%;'>{$this->translator->_("report.printViewProductsSoldByCashier.sumOfSales")}</th>
                                <th style='width: 15%;'>{$this->translator->_("report.printViewProductsSoldByCashier.profit")}</th>
                            </tr>
                        </tfoot>
                    </table>
                ";
            }
            $documentHtml .= " 
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "products_sold_per_cashier_{$_SESSION[SESSION_USER_ID]}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/reports/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/reports/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printviewsalesbypointofsaleAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/reports") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $pointOfSaleDao = new PointOfSaleDAO();
            $customerInvoiceDao = new CustomerInvoiceDAO();
            $invoicePaymentDao = new InvoicePaymentDAO();
            $pointsOfSaleIds = json_decode($_POST["pointsOfSale"], true);
            $startDate = new DateTime("{$_POST["startDate"]}:00");
            $endDate = new DateTime("{$_POST["endDate"]}:59");
            $pointsOfSale = [];
            $pointOfSaleNames = "";
            $countPointOfSale = 0;
            foreach ($pointsOfSaleIds as $pointOfSaleId) {
                $pointOfSale = $pointOfSaleDao->getPointOfSaleById((int)$pointOfSaleId);
                $pointsOfSale[] = $pointOfSale;
                if ($countPointOfSale > 0) {
                    $pointOfSaleNames .= ", {$pointOfSale->getNomPointVente()}";
                } else {
                    $pointOfSaleNames .= $pointOfSale->getNomPointVente();
                }
                $countPointOfSale++;
            }
            if (count($pointsOfSale) == 0) $pointOfSaleNames = $this->translator->_("report.printViewSalesByPointOfSale.allPointsOfSale");
            $sales = $customerInvoiceDao->getSalesByPointsOfSales($pointsOfSale, $startDate, $endDate);
            $salesResult = [];
            foreach ($sales as $sale) {
                $salesResult[] = [
                    "id" => $sale->getIdFacture(),
                    "dateAndHour" => $sale->getDateFacture()->format(DATE_FORMAT_WITH_SECONDS),
                    "amount" => $sale->getMontantNet(),
                    "reductionAmount" => $sale->getForfaitRemise(),
                    "paidAmount" => $invoicePaymentDao->getSumOfInvoicePayments($sale),
                    "pointOfSaleId" => $sale->getIdPointVente()->getIdPointVente(),
                    "pointOfSaleName" => $sale->getIdPointVente()->getNomPointVente(),
                ];
            }
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $secondaryBrand = APP_SECONDARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = $this->translator->_("report.printViewSalesByPointOfSale.salesPerPointOfSale");
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: left;  
                                color: #000000; 
                            }

                            td.center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            .no-line-found {
                                color: grey !important;
                                font-weight: bold !important;
                                font-size: 16px !important;
                            }

                            td.validated {
                                color: {$primaryBrand} !important;
                                font-weight: bold;
                            }

                            td.not-validated {
                                color: {$secondaryBrand} !important;
                                font-weight: bold;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            .equalized {
                                table-layout: fixed;
                            }

                            bold {
                                color: black;
                            }

                            i.violet-label {
                                background-color: #6435c9;
                                color: white;
                                padding: 2px;
                                border-radius: 6px;
                                margin: 3px;
                            }

                            i.blue-label {
                                background-color: #2185d0;
                                color: white;
                                padding: 2px;
                                border-radius: 6px;
                                margin: 3px;
                            }

                            i.secondary-label {
                                background-color: {$secondaryBrand};
                                color: white;
                                padding: 2px;
                                border-radius: 6px;
                                margin: 3px;
                            }

                            th.left.aligned {
                                text-align: left;
                            }

                            td.right.aligned {
                                text-align: right;
                            }

                            tr.app-total-tr {
                                background-color: {$primaryBrand}80;
                            }

                            tr.app-total-tr > td {
                                color: {$thirdBrand};
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$printTitle}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border equalized'>
                                <thead>
                                    <tr>
                                        <th colspan='2' class='center-text'>{$this->translator->_("report.printViewSalesByPointOfSale.pointsOfSale")} : <bold class='black-bold'>{$pointOfSaleNames}</bold></th>
                                    </tr>
                                    <tr>
                                        <th class='center-text'>{$this->translator->_("report.printViewSalesByPointOfSale.startDate")} : <bold class='black-bold'>{$startDate->format(DATE_FORMAT_WITH_TIME)}</bold></th>
                                        <th class='center-text'>{$this->translator->_("report.printViewSalesByPointOfSale.endDate")} : <bold class='black-bold'>{$endDate->format(DATE_FORMAT_WITH_TIME)}</bold></th>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
            ";
            $printFilters .= "{$this->translator->_("report.printViewSalesByPointOfSale.pointsOfSale")} : {$pointOfSaleNames}";
            $printFilters .= ", {$this->translator->_("report.printViewSalesByPointOfSale.startDate")} : {$startDate->format(DATE_FORMAT_WITH_TIME)}";
            $printFilters .= ", {$this->translator->_("report.printViewSalesByPointOfSale.endDate")} : {$endDate->format(DATE_FORMAT_WITH_TIME)}";
            $currentSalesPointOfSaleId = null;
            $salesSumForPointOfSale = [];
            foreach ($salesResult as $sale) {
                if ($sale["pointOfSaleId"] != $currentSalesPointOfSaleId) {
                    $salesSumForPointOfSale[$sale["pointOfSaleId"]] = [
                        "amount" => $sale["amount"],
                        "reductionAmount" => $sale["reductionAmount"],
                        "paidAmount" => $sale["paidAmount"],
                        "salesCount" => 1,
                        "salesUnpaidCount" => $sale["paidAmount"] == 0 ? 1 : 0,
                        "salesInPaymentCount" => $sale["paidAmount"] > 0 && $sale["paidAmount"] < $sale["amount"] ? 1 : 0,
                        "salesPaidCount" => $sale["paidAmount"] == $sale["amount"] ? 1 : 0,
                        "salesWithReductionCount" => $sale["reductionAmount"] > 0 ? 1 : 0,
                        "name" => $sale["pointOfSaleName"],
                    ];
                    if ($currentSalesPointOfSaleId != null) {
                        $pointOfSaleSalesSumAmount = number_format($salesSumForPointOfSale[$currentSalesPointOfSaleId]["amount"], 0, '', ' ');
                        $pointOfSaleSalesSumReductionAmount = number_format($salesSumForPointOfSale[$currentSalesPointOfSaleId]["reductionAmount"], 0, '', ' ');
                        $pointOfSaleSalesSumPaidAmount = number_format($salesSumForPointOfSale[$currentSalesPointOfSaleId]["paidAmount"], 0, '', ' ');
                        $documentHtml .= "
                                    <tr class='app-total-tr'>
                                        <td colspan='2' class='right aligned'>
                                            <i class='violet-label'>
                                                {$this->translator->_("report.printViewSalesByPointOfSale.totals")}
                                                {$salesSumForPointOfSale[$currentSalesPointOfSaleId]["name"]}
                                            </i>
                                        </td>
                                        <td class='right aligned'>{$pointOfSaleSalesSumAmount}</td>
                                        <td class='right aligned'>{$pointOfSaleSalesSumReductionAmount}</td>
                                        <td class='right aligned'>{$pointOfSaleSalesSumPaidAmount}</td>
                                    </tr>
                                </tbody>
                                <tfoot>
                                    <tr>
                                        <th>{$this->translator->_("report.printViewSalesByPointOfSale.saleNo")}</th>
                                        <th>{$this->translator->_("report.printViewSalesByPointOfSale.dateAndHour")}</th>
                                        <th>{$this->translator->_("report.printViewSalesByPointOfSale.amount")}</th>
                                        <th>{$this->translator->_("report.printViewSalesByPointOfSale.reduction")}</th>
                                        <th>{$this->translator->_("report.printViewSalesByPointOfSale.paidAmount")}</th>
                                    </tr>
                                </tfoot>
                            </table>
                            <hr/>
                        ";
                    }
                    $documentHtml .= "
                        <table class='border equalized'>
                            <thead>
                    ";
                    if (count($pointsOfSale) !== 1) {
                        $documentHtml .= "
                                <tr>
                                    <th colspan='5' class='left aligned'>
                                        <i class='violet-label'>{$sale["pointOfSaleName"]}</i>
                                    </th>
                                </tr>
                        ";
                    }
                    $documentHtml .= "
                                <tr>
                                    <th>{$this->translator->_("report.printViewSalesByPointOfSale.saleNo")}</th>
                                    <th>{$this->translator->_("report.printViewSalesByPointOfSale.dateAndHour")}</th>
                                    <th>{$this->translator->_("report.printViewSalesByPointOfSale.amount")}</th>
                                    <th>{$this->translator->_("report.printViewSalesByPointOfSale.reduction")}</th>
                                    <th>{$this->translator->_("report.printViewSalesByPointOfSale.paidAmount")}</th>
                                </tr>
                            </thead>
                            <tbody>
                    ";
                    $currentSalesPointOfSaleId = $sale["pointOfSaleId"];
                } else {
                    $salesSumForPointOfSale[$sale["pointOfSaleId"]]["amount"] += $sale["amount"];
                    $salesSumForPointOfSale[$sale["pointOfSaleId"]]["reductionAmount"] += $sale["reductionAmount"];
                    $salesSumForPointOfSale[$sale["pointOfSaleId"]]["paidAmount"] += $sale["paidAmount"];
                    $salesSumForPointOfSale[$sale["pointOfSaleId"]]["salesCount"] += 1;
                    $salesSumForPointOfSale[$sale["pointOfSaleId"]]["salesUnpaidCount"] += $sale["paidAmount"] == 0 ? 1 : 0;
                    $salesSumForPointOfSale[$sale["pointOfSaleId"]]["salesInPaymentCount"] += $sale["paidAmount"] > 0 && $sale["paidAmount"] < $sale["amount"] ? 1 : 0;
                    $salesSumForPointOfSale[$sale["pointOfSaleId"]]["salesPaidCount"] += $sale["paidAmount"] == $sale["amount"] ? 1 : 0;
                    $salesSumForPointOfSale[$sale["pointOfSaleId"]]["salesWithReductionCount"] += $sale["reductionAmount"] > 0 ? 1 : 0;
                }
                $saleMoment = new \Moment\Moment($sale["dateAndHour"]);
                $saleAmount = number_format($sale["amount"], 0, '', ' ');
                $saleReductionAmount = number_format($sale["reductionAmount"], 0, '', ' ');
                $salePaidAmount = number_format($sale["paidAmount"], 0, '', ' ');
                $documentHtml .= "
                    <tr>
                        <td>{$sale["id"]}</td>
                        <td>{$saleMoment->format(DATE_FORMAT_MIDDLE_WITH_SECONDS)}</td>
                        <td class='right aligned'>{$saleAmount}</td>
                        <td class='right aligned'>{$saleReductionAmount}</td>
                        <td class='right aligned'>{$salePaidAmount}</td>
                    </tr>
                ";
            }
            if ($currentSalesPointOfSaleId != null) {
                $pointOfSaleSalesSumAmount = number_format($salesSumForPointOfSale[$currentSalesPointOfSaleId]["amount"], 0, '', ' ');
                $pointOfSaleSalesSumReductionAmount = number_format($salesSumForPointOfSale[$currentSalesPointOfSaleId]["reductionAmount"], 0, '', ' ');
                $pointOfSaleSalesSumPaidAmount = number_format($salesSumForPointOfSale[$currentSalesPointOfSaleId]["paidAmount"], 0, '', ' ');
                $documentHtml .= "
                            <tr class='app-total-tr'>
                                <td colspan='2' class='right aligned'>
                                    <i class='violet-label'>
                                        {$this->translator->_("report.printViewSalesByPointOfSale.totals")}
                                        {$salesSumForPointOfSale[$currentSalesPointOfSaleId]["name"]}
                                    </i>
                                </td>
                                <td class='right aligned'>{$pointOfSaleSalesSumAmount}</td>
                                <td class='right aligned'>{$pointOfSaleSalesSumReductionAmount}</td>
                                <td class='right aligned'>{$pointOfSaleSalesSumPaidAmount}</td>
                            </tr>
                        </tbody>
                        <tfoot>
                            <tr>
                                <th>{$this->translator->_("report.printViewSalesByPointOfSale.saleNo")}</th>
                                <th>{$this->translator->_("report.printViewSalesByPointOfSale.dateAndHour")}</th>
                                <th>{$this->translator->_("report.printViewSalesByPointOfSale.amount")}</th>
                                <th>{$this->translator->_("report.printViewSalesByPointOfSale.reduction")}</th>
                                <th>{$this->translator->_("report.printViewSalesByPointOfSale.paidAmount")}</th>
                            </tr>
                        </tfoot>
                    </table>
                    <hr/>
                ";
            }
            if (count($salesResult) > 0) {
                $salesTotal = 0;
                $paymentTotal = 0;
                $reductionTotal = 0;
                $numberOfSales = 0;
                $paid = 0;
                $paymentInProgess = 0;
                $unpaid = 0;
                $withReduction = 0;
                $documentHtml .= "
                    <table class='border'>
                        <thead>
                            <tr>
                                <th colspan='9' class='left aligned'>
                                    <i class='secondary-label'>{$this->translator->_("report.printViewSalesByPointOfSale.summaryOfSalesPerPointOfSale")}</i>
                                </th>
                            </tr>
                            <tr>
                                <th style='width: 20%;'>{$this->translator->_("report.printViewSalesByPointOfSale.pointOfSale")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewSalesByPointOfSale.salesTotal")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewSalesByPointOfSale.paymentTotal")}</th>
                                <th style='width: 10%'>{$this->translator->_("report.printViewSalesByPointOfSale.reductionTotal")}</th>
                                <th>{$this->translator->_("report.printViewSalesByPointOfSale.numberOfSales")}</th>
                                <th>{$this->translator->_("report.printViewSalesByPointOfSale.paid")}</th>
                                <th>{$this->translator->_("report.printViewSalesByPointOfSale.paymentInProgess")}</th>
                                <th>{$this->translator->_("report.printViewSalesByPointOfSale.unpaid")}</th>
                                <th>{$this->translator->_("report.printViewSalesByPointOfSale.withReduction")}</th>
                            </tr>
                        </thead>
                        <tbody>
                ";
                foreach ($salesSumForPointOfSale as $pointOfSaleId => $pointOfSaleValues) {
                    $salesTotal += $pointOfSaleValues["amount"];
                    $paymentTotal += $pointOfSaleValues["paidAmount"];
                    $reductionTotal += $pointOfSaleValues["reductionAmount"];
                    $numberOfSales += $pointOfSaleValues["salesCount"];
                    $paid += $pointOfSaleValues["salesPaidCount"];
                    $paymentInProgess += $pointOfSaleValues["salesInPaymentCount"];
                    $unpaid += $pointOfSaleValues["salesUnpaidCount"];
                    $withReduction += $pointOfSaleValues["salesWithReductionCount"];
                    $pointOfSaleAmount = number_format($pointOfSaleValues["amount"], 0, '', ' ');
                    $pointOfSalePaidAmount = number_format($pointOfSaleValues["paidAmount"], 0, '', ' ');
                    $pointOfSaleReductionAmount = number_format($pointOfSaleValues["reductionAmount"], 0, '', ' ');
                    $documentHtml .= "
                        <tr>
                            <td>{$pointOfSaleValues["name"]}</td>
                            <td class='right aligned'>{$pointOfSaleAmount}</td>
                            <td class='right aligned'>{$pointOfSalePaidAmount}</td>
                            <td class='right aligned'>{$pointOfSaleReductionAmount}</td>
                            <td>{$pointOfSaleValues["salesCount"]}</td>
                            <td>{$pointOfSaleValues["salesPaidCount"]}</td>
                            <td>{$pointOfSaleValues["salesInPaymentCount"]}</td>
                            <td>{$pointOfSaleValues["salesUnpaidCount"]}</td>
                            <td>{$pointOfSaleValues["salesWithReductionCount"]}</td>
                        </tr>
                    ";
                };
                $salesTotalFormat = number_format($salesTotal, 0, '', ' ');
                $paymentTotalFormat = number_format($paymentTotal, 0, '', ' ');
                $reductionTotalFormat = number_format($reductionTotal, 0, '', ' ');
                $documentHtml .= "
                            <tr class='app-total-tr'>
                                <td class='right aligned'>
                                    <i class='secondary-label'>{$this->translator->_("report.printViewSalesByPointOfSale.totals")}</i>
                                </td>
                                <td class='right aligned'>{$salesTotalFormat}</td>
                                <td class='right aligned'>{$paymentTotalFormat}</td>
                                <td class='right aligned'>{$reductionTotalFormat}</td>
                                <td>{$numberOfSales}</td>
                                <td>{$paid}</td>
                                <td>{$paymentInProgess}</td>
                                <td>{$unpaid}</td>
                                <td>{$withReduction}</td>
                            </tr>
                        </tbody>
                        <tfoot>
                            <tr>
                                <th>{$this->translator->_("report.printViewSalesByPointOfSale.pointOfSale")}</th>
                                <th>{$this->translator->_("report.printViewSalesByPointOfSale.salesTotal")}</th>
                                <th>{$this->translator->_("report.printViewSalesByPointOfSale.paymentTotal")}</th>
                                <th>{$this->translator->_("report.printViewSalesByPointOfSale.reductionTotal")}</th>
                                <th>{$this->translator->_("report.printViewSalesByPointOfSale.numberOfSales")}</th>
                                <th>{$this->translator->_("report.printViewSalesByPointOfSale.paid")}</th>
                                <th>{$this->translator->_("report.printViewSalesByPointOfSale.paymentInProgess")}</th>
                                <th>{$this->translator->_("report.printViewSalesByPointOfSale.unpaid")}</th>
                                <th>{$this->translator->_("report.printViewSalesByPointOfSale.withReduction")}</th>
                            </tr>
                        </tfoot>
                    </table>
                ";
            }
            $documentHtml .= " 
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "sales_per_point_sale_{$_SESSION[SESSION_USER_ID]}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/reports/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/reports/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printviewproductssoldbypointofsaleAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/reports") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $pointOfSaleDao = new PointOfSaleDAO();
            $customerInvoiceLineDao = new CustomerInvoiceLineDAO();
            $pointsOfSaleIds = json_decode($_POST["pointsOfSale"], true);
            $startDate = new DateTime("{$_POST["startDate"]}:00");
            $endDate = new DateTime("{$_POST["endDate"]}:59");
            $pointsOfSale = [];
            $pointOfSaleNames = "";
            $countPointOfSale = 0;
            foreach ($pointsOfSaleIds as $pointOfSaleId) {
                $pointOfSale = $pointOfSaleDao->getPointOfSaleById((int)$pointOfSaleId);
                $pointsOfSale[] = $pointOfSale;
                if ($countPointOfSale > 0) {
                    $pointOfSaleNames .= ", {$pointOfSale->getNomPointVente()}";
                } else {
                    $pointOfSaleNames .= $pointOfSale->getNomPointVente();
                }
                $countPointOfSale++;
            }
            if (count($pointsOfSale) == 0) $pointOfSaleNames = $this->translator->_("report.printViewProductsSoldByPointOfSale.allPointsOfSale");
            $productsCount = $customerInvoiceLineDao->getSoldArticleCountByPointsOfSale($pointsOfSale, $startDate, $endDate);
            $productsCountResult = [];
            foreach ($productsCount as $productCount) {
                $productsCountResult[] = [
                    "designation" => $productCount["product_name"],
                    "productId" => $productCount["product_id"],
                    "category" => $productCount["category_name"],
                    "categoryId" => $productCount["category_id"],
                    "quantity" => (int)$productCount["sold_quantity"],
                    "buyingPrice" => (float)$productCount["buying_price"],
                    "unitPrice" => (float)$productCount["unit_price"],
                    "salesSum" => (float)$productCount["sold_amount"],
                    "profit" => (float)$productCount["profit"],
                    "pointOfSaleId" => (int)$productCount["point_of_sale_id"],
                    "pointOfSaleName" => $productCount["point_of_sale_name"],
                ];
            }
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $secondaryBrand = APP_SECONDARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = $this->translator->_("report.printViewProductsSoldByPointOfSale.productsSoldPerPointOfSale");
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: left;  
                                color: #000000; 
                            }

                            td.center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            .no-line-found {
                                color: grey !important;
                                font-weight: bold !important;
                                font-size: 16px !important;
                            }

                            td.validated {
                                color: {$primaryBrand} !important;
                                font-weight: bold;
                            }

                            td.not-validated {
                                color: {$secondaryBrand} !important;
                                font-weight: bold;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            .equalized {
                                table-layout: fixed;
                            }

                            bold {
                                color: black;
                            }

                            i.violet-label {
                                background-color: #6435c9;
                                color: white;
                                padding: 2px;
                                border-radius: 6px;
                                margin: 3px;
                            }

                            i.blue-label {
                                background-color: #2185d0;
                                color: white;
                                padding: 2px;
                                border-radius: 6px;
                                margin: 3px;
                            }

                            i.secondary-label {
                                background-color: {$secondaryBrand};
                                color: white;
                                padding: 2px;
                                border-radius: 6px;
                                margin: 3px;
                            }

                            th.left.aligned {
                                text-align: left;
                            }

                            td.right.aligned {
                                text-align: right;
                            }

                            tr.app-total-tr {
                                background-color: {$primaryBrand}80;
                            }

                            tr.app-total-tr > td {
                                color: {$thirdBrand};
                            }

                            tr.app-highlight-tr {
                                background-color: {$secondaryBrand}80;
                            }

                            tr.app-highlight-tr > td {
                                color: {$thirdBrand};
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$printTitle}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border equalized'>
                                <thead>
                                    <tr>
                                        <th colspan='2' class='center-text'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.pointsOfSale")} : <bold class='black-bold'>{$pointOfSaleNames}</bold></th>
                                    </tr>
                                    <tr>
                                        <th class='center-text'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.startDate")} : <bold class='black-bold'>{$startDate->format(DATE_FORMAT_WITH_TIME)}</bold></th>
                                        <th class='center-text'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.endDate")} : <bold class='black-bold'>{$endDate->format(DATE_FORMAT_WITH_TIME)}</bold></th>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
            ";
            $printFilters .= "{$this->translator->_("report.printViewProductsSoldByPointOfSale.pointsOfSale")} : {$pointOfSaleNames}";
            $printFilters .= ", {$this->translator->_("report.printViewProductsSoldByPointOfSale.startDate")} : {$startDate->format(DATE_FORMAT_WITH_TIME)}";
            $printFilters .= ", {$this->translator->_("report.printViewProductsSoldByPointOfSale.endDate")} : {$endDate->format(DATE_FORMAT_WITH_TIME)}";
            $currentProductsPointOfSaleId = null;
            $currentProductsCategoryId = null;
            $currentProductsCategoryName = null;
            $currentCategoryTotalQuantity = 0;
            $currentCategoryTotalSalesSum = 0;
            $currentCategoryTotalProfit = 0;
            $productsSumForPointOfSale = [];
            $productsSumGlobal = [];
            foreach ($productsCountResult as $product) {
                if ($product["pointOfSaleId"] != $currentProductsPointOfSaleId) {
                    $productsSumForPointOfSale[$product["pointOfSaleId"]] = [
                        "quantity" => $product["quantity"],
                        "salesSum" => $product["salesSum"],
                        "profit" => $product["profit"],
                        "name" => $product["pointOfSaleName"],
                    ];
                    $productsSumForCashierByPointOfSale[$product["pointOfSaleId"]] = [];
                    if ($currentProductsPointOfSaleId != null) {
                        if ($currentProductsCategoryId != null) {
                            $categoryTotalSalesSum = number_format($currentCategoryTotalSalesSum, 0, '', ' ');
                            $categoryTotalProfit = number_format($currentCategoryTotalProfit, 0, '', ' ');
                            $documentHtml .= "
                                <tr class='app-total-tr'>
                                    <td class='right aligned'>
                                        {$this->translator->_("report.printViewProductsSoldByPointOfSale.totals")}
                                        {$currentProductsCategoryName}
                                    </td>
                                    <td>{$currentCategoryTotalQuantity}</td>
                                    <td colspan='2'></td>
                                    <td class='right aligned'>{$categoryTotalSalesSum}</td>
                                    <td class='right aligned'>{$categoryTotalProfit}</td>
                                </tr>
                            ";
                        }
                        $pointOfSaleProductsSoldSum = number_format($productsSumForPointOfSale[$currentProductsPointOfSaleId]["salesSum"], 0, '', ' ');
                        $pointOfSaleProductsProfit = number_format($productsSumForPointOfSale[$currentProductsPointOfSaleId]["profit"], 0, '', ' ');
                        $documentHtml .= "
                                    <tr class='app-total-tr'>
                                        <td class='right aligned'>
                                            <i class='violet-label'>
                                                {$this->translator->_("report.printViewProductsSoldByPointOfSale.totals")}
                                                {$productsSumForPointOfSale[$currentProductsPointOfSaleId]["name"]}
                                            </i>
                                        </td>
                                        <td>{$productsSumForPointOfSale[$currentProductsPointOfSaleId]["quantity"]}</td>
                                        <td colspan='2'></td>
                                        <td class='right aligned'>{$pointOfSaleProductsSoldSum}</td>
                                        <td class='right aligned'>{$pointOfSaleProductsProfit}</td>
                                    </tr>
                                <tbody>
                                <tfoot>
                                    <tr>
                                        <th style='width: 40%;'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.designation")}</th>
                                        <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.quantity")}</th>
                                        <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.buyingPrice")}</th>
                                        <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.unitPrice")}</th>
                                        <th style='width: 15%;'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.sumOfSales")}</th>
                                        <th style='width: 15%;'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.profit")}</th>
                                    </tr>
                                </tfoot>
                            </table>
                            <hr/>
                        ";
                    }
                    $documentHtml .= "
                        <table class='border'>
                            <thead>
                    ";
                    if (count($pointsOfSale) != 1) {
                        $documentHtml .= "
                                <tr>
                                    <th colspan='6' class='left aligned'>
                                        <i class='violet-label'>{$product["pointOfSaleName"]}</i>
                                    </th>
                                </tr>
                    ";
                    }
                    $documentHtml .= "
                                <tr>
                                    <th style='width: 40%;'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.designation")}</th>
                                    <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.quantity")}</th>
                                    <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.buyingPrice")}</th>
                                    <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.unitPrice")}</th>
                                    <th style='width: 15%;'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.sumOfSales")}</th>
                                    <th style='width: 15%;'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.profit")}</th>
                                </tr>
                            </thead>
                            <tbody>
                    ";
                    $currentProductsPointOfSaleId = $product["pointOfSaleId"];
                    $currentProductsCategoryId = null;
                    $currentProductsCategoryName = null;
                    $currentCategoryTotalQuantity = 0;
                    $currentCategoryTotalSalesSum = 0;
                    $currentCategoryTotalProfit = 0;
                } else {
                    $productsSumForPointOfSale[$product["pointOfSaleId"]]["quantity"] += $product["quantity"];
                    $productsSumForPointOfSale[$product["pointOfSaleId"]]["salesSum"] += $product["salesSum"];
                    $productsSumForPointOfSale[$product["pointOfSaleId"]]["profit"] += $product["profit"];
                }
                if (!isset($productsSumGlobal[$product["productId"]])) {
                    $productsSumGlobal[$product["productId"]] = [
                        "quantity" => $product["quantity"],
                        "buyingPrice" => $product["buyingPrice"],
                        "unitPrice" => $product["unitPrice"],
                        "salesSum" => $product["salesSum"],
                        "profit" => $product["profit"],
                        "name" => $product["designation"],
                        "categoryId" => $product["categoryId"],
                        "category" => $product["category"],
                    ];
                } else {
                    $productsSumGlobal[$product["productId"]]["quantity"] += $product["quantity"];
                    $productsSumGlobal[$product["productId"]]["salesSum"] += $product["salesSum"];
                    $productsSumGlobal[$product["productId"]]["profit"] += $product["profit"];
                }
                if ($product["categoryId"] != $currentProductsCategoryId) {
                    if ($currentProductsCategoryId != null) {
                        $categoryTotalSalesSum = number_format($currentCategoryTotalSalesSum, 0, '', ' ');
                        $categoryTotalProfit = number_format($currentCategoryTotalProfit, 0, '', ' ');
                        $documentHtml .= "
                            <tr class='app-total-tr'>
                                <td class='right aligned'>
                                    {$this->translator->_("report.printViewProductsSoldByPointOfSale.totals")}
                                    {$currentProductsCategoryName}
                                </td>
                                <td>{$currentCategoryTotalQuantity}</td>
                                <td colspan='2'></td>
                                <td class='right aligned'>{$categoryTotalSalesSum}</td>
                                <td class='right aligned'>{$categoryTotalProfit}</td>
                            </tr>
                        ";
                    }
                    $documentHtml .= "
                        <tr class='app-highlight-tr'>
                            <td colspan='6'>{$product["category"]}</td>
                        </tr>
                    ";
                    $currentProductsCategoryId = $product["categoryId"];
                    $currentProductsCategoryName = $product["category"];
                    $currentCategoryTotalQuantity = $product["quantity"];
                    $currentCategoryTotalSalesSum = $product["salesSum"];
                    $currentCategoryTotalProfit = $product["profit"];
                } else {
                    $currentCategoryTotalQuantity += $product["quantity"];
                    $currentCategoryTotalSalesSum += $product["salesSum"];
                    $currentCategoryTotalProfit += $product["profit"];
                }
                $buyingPrice = number_format($product["buyingPrice"], 0, '', ' ');
                $unitPrice = number_format($product["unitPrice"], 0, '', ' ');
                $salesSum = number_format($product["salesSum"], 0, '', ' ');
                $profit = number_format($product["profit"], 0, '', ' ');
                $documentHtml .= "
                    <tr>
                        <td>{$product["designation"]}</td>
                        <td>{$product["quantity"]}</td>
                        <td class='right aligned'>{$buyingPrice}</td>
                        <td class='right aligned'>{$unitPrice}</td>
                        <td class='right aligned'>{$salesSum}</td>
                        <td class='right aligned'>{$profit}</td>
                    </tr>
                ";
            }
            if ($currentProductsPointOfSaleId != null) {
                if ($currentProductsCategoryId != null) {
                    $categoryTotalSalesSum = number_format($currentCategoryTotalSalesSum, 0, '', ' ');
                    $categoryTotalProfit = number_format($currentCategoryTotalProfit, 0, '', ' ');
                    $documentHtml .= "
                        <tr class='app-total-tr'>
                            <td class='right aligned'>
                                {$this->translator->_("report.printViewProductsSoldByPointOfSale.totals")}
                                {$currentProductsCategoryName}
                            </td>
                            <td>{$currentCategoryTotalQuantity}</td>
                            <td colspan='2'></td>
                            <td class='right aligned'>{$categoryTotalSalesSum}</td>
                            <td class='right aligned'>{$categoryTotalProfit}</td>
                        </tr>
                    ";
                }
                $pointOfSaleProductsSoldSum = number_format($productsSumForPointOfSale[$currentProductsPointOfSaleId]["salesSum"], 0, '', ' ');
                $pointOfSaleProductsProfit = number_format($productsSumForPointOfSale[$currentProductsPointOfSaleId]["profit"], 0, '', ' ');
                $documentHtml .= "
                            <tr class='app-total-tr'>
                                <td class='right aligned'>
                                    <i class='violet-label'>
                                        {$this->translator->_("report.printViewProductsSoldByPointOfSale.totals")}
                                        {$productsSumForPointOfSale[$currentProductsPointOfSaleId]["name"]}
                                    </i>
                                </td>
                                <td>{$productsSumForPointOfSale[$currentProductsPointOfSaleId]["quantity"]}</td>
                                <td colspan='2'></td>
                                <td class='right aligned'>{$pointOfSaleProductsSoldSum}</td>
                                <td class='right aligned'>{$pointOfSaleProductsProfit}</td>
                            </tr>
                        <tbody>
                        <tfoot>
                            <tr>
                                <th style='width: 40%;'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.designation")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.quantity")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.buyingPrice")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.unitPrice")}</th>
                                <th style='width: 15%;'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.sumOfSales")}</th>
                                <th style='width: 15%;'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.profit")}</th>
                            </tr>
                        </tfoot>
                    </table>
                    <hr/>
                ";
            }
            if (count($pointsOfSale) !== 1 && count($productsCount) > 0) {
                $quantityTotal = 0;
                $salesSumTotal = 0;
                $profitTotal = 0;
                $summaryProductsCategoryId = null;
                $summaryProductsCategoryName = null;
                $summaryCategoryTotalQuantity = 0;
                $summaryCategoryTotalSalesSum = 0;
                $summaryCategoryTotalProfit = 0;
                $documentHtml .= "
                    <table class='border'>
                        <thead>
                            <tr>
                                <th colspan='6' class='left aligned'>
                                    <i class='secondary-label'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.summaryOfProductsSold")}</i>
                                </th>
                            </tr>
                            <tr>
                                <th style='width: 40%;'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.designation")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.quantity")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.buyingPrice")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.unitPrice")}</th>
                                <th style='width: 15%;'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.sumOfSales")}</th>
                                <th style='width: 15%;'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.profit")}</th>
                            </tr>
                        </thead>
                        <tbody>
                ";
                foreach ($productsSumGlobal as $productId => $productValues) {
                    $quantityTotal += $productValues["quantity"];
                    $salesSumTotal += $productValues["salesSum"];
                    $profitTotal += $productValues["profit"];
                    if ($productValues["categoryId"] != $summaryProductsCategoryId) {
                        if ($summaryProductsCategoryId != null) {
                            $categoryTotalSalesSum = number_format($summaryCategoryTotalSalesSum, 0, '', ' ');
                            $categoryTotalProfit = number_format($summaryCategoryTotalProfit, 0, '', ' ');
                            $documentHtml .= "
                                <tr class='app-total-tr'>
                                    <td class='right aligned'>
                                        {$this->translator->_("report.printViewProductsSoldByPointOfSale.totals")}
                                        {$summaryProductsCategoryName}
                                    </td>
                                    <td>{$summaryCategoryTotalQuantity}</td>
                                    <td colspan='2'></td>
                                    <td class='right aligned'>{$categoryTotalSalesSum}</td>
                                    <td class='right aligned'>{$categoryTotalProfit}</td>
                                </tr>
                            ";
                        }
                        $documentHtml .= "
                            <tr class='app-highlight-tr'>
                                <td colspan='6'>{$productValues["category"]}</td>
                            </tr>
                        ";
                        $summaryProductsCategoryId = $productValues["categoryId"];
                        $summaryProductsCategoryName = $productValues["category"];
                        $summaryCategoryTotalQuantity = $productValues["quantity"];
                        $summaryCategoryTotalSalesSum = $productValues["salesSum"];
                        $summaryCategoryTotalProfit = $productValues["profit"];
                    } else {
                        $summaryCategoryTotalQuantity += $productValues["quantity"];
                        $summaryCategoryTotalSalesSum += $productValues["salesSum"];
                        $summaryCategoryTotalProfit += $productValues["profit"];
                    }
                    $productBuyingPrice = number_format($productValues["buyingPrice"], 0, '', ' ');
                    $productUnitPrice = number_format($productValues["unitPrice"], 0, '', ' ');
                    $productSalesSum = number_format($productValues["salesSum"], 0, '', ' ');
                    $productProfit = number_format($productValues["profit"], 0, '', ' ');
                    $documentHtml .= "
                        <tr>
                            <td>{$productValues["name"]}</td>
                            <td>{$productValues["quantity"]}</td>
                            <td class='right aligned'>{$productBuyingPrice}</td>
                            <td class='right aligned'>{$productUnitPrice}</td>
                            <td class='right aligned'>{$productSalesSum}</td>
                            <td class='right aligned'>{$productProfit}</td>
                        </tr>
                    ";
                };
                if ($summaryProductsCategoryId != null) {
                    $categoryTotalSalesSum = number_format($summaryCategoryTotalSalesSum, 0, '', ' ');
                    $categoryTotalProfit = number_format($summaryCategoryTotalProfit, 0, '', ' ');
                    $documentHtml .= "
                        <tr class='app-total-tr'>
                            <td class='right aligned'>
                                {$this->translator->_("report.printViewProductsSoldByPointOfSale.totals")}
                                {$summaryProductsCategoryName}
                            </td>
                            <td>{$summaryCategoryTotalQuantity}</td>
                            <td colspan='2'></td>
                            <td class='right aligned'>{$categoryTotalSalesSum}</td>
                            <td class='right aligned'>{$categoryTotalProfit}</td>
                        </tr>
                    ";
                }
                $salesSumTotalFormat = number_format($salesSumTotal, 0, '', ' ');
                $profitTotalFormat = number_format($profitTotal, 0, '', ' ');
                $documentHtml .= "
                            <tr class='app-total-tr'>
                                <td class='right aligned'>
                                    <i class='secondary-label'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.totals")}</a>
                                </td>
                                <td>{$quantityTotal}</td>
                                <td colspan='2'></span>
                                <td class='right aligned'>{$salesSumTotalFormat}</td>
                                <td class='right aligned'>{$profitTotalFormat}</td>
                            </tr>
                        </tbody>
                        <tfoot>
                            <tr>
                                <th style='width: 40%;'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.designation")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.quantity")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.buyingPrice")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.unitPrice")}</th>
                                <th style='width: 15%;'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.sumOfSales")}</th>
                                <th style='width: 15%;'>{$this->translator->_("report.printViewProductsSoldByPointOfSale.profit")}</th>
                            </tr>
                        </tfoot>
                    </table>
                ";
            }
            $documentHtml .= " 
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "products_sold_per_point_of_sale_{$_SESSION[SESSION_USER_ID]}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/reports/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/reports/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printviewproductsalesAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/reports") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $pointOfSaleDao = new PointOfSaleDAO();
            $productCategoryDao = new ProductCategoryDAO();
            $productDao = new ProductDAO();
            $customerInvoiceLineDao = new CustomerInvoiceLineDAO();
            $pointsOfSaleIds = json_decode($_POST["pointsOfSale"], true);
            $productCategoriesIds = json_decode($_POST["categories"], true);
            $productsIds = json_decode($_POST["products"], true);
            $startDate = new DateTime("{$_POST["startDate"]}:00");
            $endDate = new DateTime("{$_POST["endDate"]}:59");
            $pointsOfSale = [];
            $pointOfSaleNames = "";
            $countPointOfSale = 0;
            foreach ($pointsOfSaleIds as $pointOfSaleId) {
                $pointOfSale = $pointOfSaleDao->getPointOfSaleById((int)$pointOfSaleId);
                $pointsOfSale[] = $pointOfSale;
                if ($countPointOfSale > 0) {
                    $pointOfSaleNames .= ", {$pointOfSale->getNomPointVente()}";
                } else {
                    $pointOfSaleNames .= $pointOfSale->getNomPointVente();
                }
                $countPointOfSale++;
            }
            $productCategories = [];
            $productCategoriesNames = "";
            $countProductCategory = 0;
            foreach ($productCategoriesIds as $productCategoryId) {
                $productCategory = $productCategoryDao->getProductCategoryById((int)$productCategoryId);
                $productCategories[] = $productCategory;
                if ($countProductCategory > 0) {
                    $productCategoriesNames .= ", {$productCategory->getNom()}";
                } else {
                    $productCategoriesNames .= $productCategory->getNom();
                }
                $countProductCategory++;
            }
            $products = [];
            $productsNames = "";
            $countProduct = 0;
            foreach ($productsIds as $productId) {
                $product = $productDao->getProductById((int)$productId);
                $products[] = $product;
                if ($countProduct > 0) {
                    $productsNames .= ", {$product->getDesignation()}";
                } else {
                    $productsNames .= $product->getDesignation();
                }
                $countProduct++;
            }
            if (count($pointsOfSale) == 0) $pointOfSaleNames = $this->translator->_("report.printViewProductSales.allPointsOfSale");
            if (count($productCategories) == 0) $productCategoriesNames = $this->translator->_("report.printViewProductSales.allCategories");
            if (count($products) == 0) $productsNames = $this->translator->_("report.printViewProductSales.allProducts");
            $productsCount = $customerInvoiceLineDao->getSoldArticleCount($pointsOfSale, $productCategories, $products, $startDate, $endDate);
            $productsCountResult = [];
            foreach ($productsCount as $productCount) {
                $productsCountResult[] = [
                    "designation" => $productCount["product_name"],
                    "productId" => $productCount["product_id"],
                    "category" => $productCount["category_name"],
                    "categoryId" => $productCount["category_id"],
                    "quantity" => (int)$productCount["sold_quantity"],
                    "buyingPrice" => (float)$productCount["buying_price"],
                    "unitPrice" => (float)$productCount["unit_price"],
                    "salesSum" => (float)$productCount["sold_amount"],
                    "profit" => (float)$productCount["profit"],
                    "pointOfSaleId" => (int)$productCount["point_of_sale_id"],
                    "pointOfSaleName" => $productCount["point_of_sale_name"],
                ];
            }
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $secondaryBrand = APP_SECONDARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = $this->translator->_("report.printViewProductSales.productsSold");
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: left;  
                                color: #000000; 
                            }

                            td.center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            .no-line-found {
                                color: grey !important;
                                font-weight: bold !important;
                                font-size: 16px !important;
                            }

                            td.validated {
                                color: {$primaryBrand} !important;
                                font-weight: bold;
                            }

                            td.not-validated {
                                color: {$secondaryBrand} !important;
                                font-weight: bold;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            .equalized {
                                table-layout: fixed;
                            }

                            bold {
                                color: black;
                            }

                            i.violet-label {
                                background-color: #6435c9;
                                color: white;
                                padding: 2px;
                                border-radius: 6px;
                                margin: 3px;
                            }

                            i.blue-label {
                                background-color: #2185d0;
                                color: white;
                                padding: 2px;
                                border-radius: 6px;
                                margin: 3px;
                            }

                            i.secondary-label {
                                background-color: {$secondaryBrand};
                                color: white;
                                padding: 2px;
                                border-radius: 6px;
                                margin: 3px;
                            }

                            th.left.aligned {
                                text-align: left;
                            }

                            td.right.aligned {
                                text-align: right;
                            }

                            tr.app-total-tr {
                                background-color: {$primaryBrand}80;
                            }

                            tr.app-total-tr > td {
                                color: {$thirdBrand};
                            }

                            tr.app-highlight-tr {
                                background-color: {$secondaryBrand}80;
                            }

                            tr.app-highlight-tr > td {
                                color: {$thirdBrand};
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$printTitle}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border equalized'>
                                <thead>
                                    <tr>
                                        <th colspan='2' class='center-text'>{$this->translator->_("report.printViewProductSales.pointsOfSale")} : <bold class='black-bold'>{$pointOfSaleNames}</bold></th>
                                    </tr>
                                    <tr>
                                        <th class='center-text'>{$this->translator->_("report.printViewProductSales.categories")} : <bold class='black-bold'>{$productCategoriesNames}</bold></th>
                                        <th class='center-text'>{$this->translator->_("report.printViewProductSales.products")} : <bold class='black-bold'>{$productsNames}</bold></th>
                                    </tr>
                                    <tr>
                                        <th class='center-text'>{$this->translator->_("report.printViewProductSales.startDate")} : <bold class='black-bold'>{$startDate->format(DATE_FORMAT_WITH_TIME)}</bold></th>
                                        <th class='center-text'>{$this->translator->_("report.printViewProductSales.endDate")} : <bold class='black-bold'>{$endDate->format(DATE_FORMAT_WITH_TIME)}</bold></th>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
            ";
            $printFilters .= "{$this->translator->_("report.printViewProductSales.pointsOfSale")} : {$pointOfSaleNames}";
            $printFilters .= ", {$this->translator->_("report.printViewProductSales.categories")} : {$productCategoriesNames}";
            $printFilters .= ", {$this->translator->_("report.printViewProductSales.products")} : {$productsNames}";
            $printFilters .= ", {$this->translator->_("report.printViewProductSales.startDate")} : {$startDate->format(DATE_FORMAT_WITH_TIME)}";
            $printFilters .= ", {$this->translator->_("report.printViewProductSales.endDate")} : {$endDate->format(DATE_FORMAT_WITH_TIME)}";
            $currentProductsPointOfSaleId = null;
            $currentProductsCategoryId = null;
            $currentProductsCategoryName = null;
            $currentCategoryTotalQuantity = 0;
            $currentCategoryTotalSalesSum = 0;
            $currentCategoryTotalProfit = 0;
            $productsSumForPointOfSale = [];
            $productsSumGlobal = [];
            foreach ($productsCountResult as $product) {
                if ($product["pointOfSaleId"] != $currentProductsPointOfSaleId) {
                    $productsSumForPointOfSale[$product["pointOfSaleId"]] = [
                        "quantity" => $product["quantity"],
                        "salesSum" => $product["salesSum"],
                        "profit" => $product["profit"],
                        "name" => $product["pointOfSaleName"],
                    ];
                    $productsSumForCashierByPointOfSale[$product["pointOfSaleId"]] = [];
                    if ($currentProductsPointOfSaleId != null) {
                        if ($currentProductsCategoryId != null) {
                            $categoryTotalSalesSum = number_format($currentCategoryTotalSalesSum, 0, '', ' ');
                            $categoryTotalProfit = number_format($currentCategoryTotalProfit, 0, '', ' ');
                            $documentHtml .= "
                                <tr class='app-total-tr'>
                                    <td class='right aligned'>
                                        {$this->translator->_("report.printViewProductSales.totals")}
                                        {$currentProductsCategoryName}
                                    </td>
                                    <td>{$currentCategoryTotalQuantity}</td>
                                    <td colspan='2'></td>
                                    <td class='right aligned'>{$categoryTotalSalesSum}</td>
                                    <td class='right aligned'>{$categoryTotalProfit}</td>
                                </tr>
                            ";
                        }
                        $pointOfSaleProductsSoldSum = number_format($productsSumForPointOfSale[$currentProductsPointOfSaleId]["salesSum"], 0, '', ' ');
                        $pointOfSaleProductsProfit = number_format($productsSumForPointOfSale[$currentProductsPointOfSaleId]["profit"], 0, '', ' ');
                        $documentHtml .= "
                                    <tr class='app-total-tr'>
                                        <td class='right aligned'>
                                            <i class='violet-label'>
                                                {$this->translator->_("report.printViewProductSales.totals")}
                                                {$productsSumForPointOfSale[$currentProductsPointOfSaleId]["name"]}
                                            </i>
                                        </td>
                                        <td>{$productsSumForPointOfSale[$currentProductsPointOfSaleId]["quantity"]}</td>
                                        <td colspan='2'></td>
                                        <td class='right aligned'>{$pointOfSaleProductsSoldSum}</td>
                                        <td class='right aligned'>{$pointOfSaleProductsProfit}</td>
                                    </tr>
                                <tbody>
                                <tfoot>
                                    <tr>
                                        <th style='width: 40%;'>{$this->translator->_("report.printViewProductSales.designation")}</th>
                                        <th style='width: 10%;'>{$this->translator->_("report.printViewProductSales.quantity")}</th>
                                        <th style='width: 10%;'>{$this->translator->_("report.printViewProductSales.buyingPrice")}</th>
                                        <th style='width: 10%;'>{$this->translator->_("report.printViewProductSales.unitPrice")}</th>
                                        <th style='width: 15%;'>{$this->translator->_("report.printViewProductSales.sumOfSales")}</th>
                                        <th style='width: 15%;'>{$this->translator->_("report.printViewProductSales.profit")}</th>
                                    </tr>
                                </tfoot>
                            </table>
                            <hr/>
                        ";
                    }
                    $documentHtml .= "
                        <table class='border'>
                            <thead>
                    ";
                    if (count($pointsOfSale) != 1) {
                        $documentHtml .= "
                                <tr>
                                    <th colspan='6' class='left aligned'>
                                        <i class='violet-label'>{$product["pointOfSaleName"]}</i>
                                    </th>
                                </tr>
                    ";
                    }
                    $documentHtml .= "
                                <tr>
                                    <th style='width: 40%;'>{$this->translator->_("report.printViewProductSales.designation")}</th>
                                    <th style='width: 10%;'>{$this->translator->_("report.printViewProductSales.quantity")}</th>
                                    <th style='width: 10%;'>{$this->translator->_("report.printViewProductSales.buyingPrice")}</th>
                                    <th style='width: 10%;'>{$this->translator->_("report.printViewProductSales.unitPrice")}</th>
                                    <th style='width: 15%;'>{$this->translator->_("report.printViewProductSales.sumOfSales")}</th>
                                    <th style='width: 15%;'>{$this->translator->_("report.printViewProductSales.profit")}</th>
                                </tr>
                            </thead>
                            <tbody>
                    ";
                    $currentProductsPointOfSaleId = $product["pointOfSaleId"];
                    $currentProductsCategoryId = null;
                    $currentProductsCategoryName = null;
                    $currentCategoryTotalQuantity = 0;
                    $currentCategoryTotalSalesSum = 0;
                    $currentCategoryTotalProfit = 0;
                } else {
                    $productsSumForPointOfSale[$product["pointOfSaleId"]]["quantity"] += $product["quantity"];
                    $productsSumForPointOfSale[$product["pointOfSaleId"]]["salesSum"] += $product["salesSum"];
                    $productsSumForPointOfSale[$product["pointOfSaleId"]]["profit"] += $product["profit"];
                }
                if (!isset($productsSumGlobal[$product["productId"]])) {
                    $productsSumGlobal[$product["productId"]] = [
                        "quantity" => $product["quantity"],
                        "buyingPrice" => $product["buyingPrice"],
                        "unitPrice" => $product["unitPrice"],
                        "salesSum" => $product["salesSum"],
                        "profit" => $product["profit"],
                        "name" => $product["designation"],
                        "categoryId" => $product["categoryId"],
                        "category" => $product["category"],
                    ];
                } else {
                    $productsSumGlobal[$product["productId"]]["quantity"] += $product["quantity"];
                    $productsSumGlobal[$product["productId"]]["salesSum"] += $product["salesSum"];
                    $productsSumGlobal[$product["productId"]]["profit"] += $product["profit"];
                }
                if ($product["categoryId"] != $currentProductsCategoryId) {
                    if ($currentProductsCategoryId != null) {
                        $categoryTotalSalesSum = number_format($currentCategoryTotalSalesSum, 0, '', ' ');
                        $categoryTotalProfit = number_format($currentCategoryTotalProfit, 0, '', ' ');
                        $documentHtml .= "
                            <tr class='app-total-tr'>
                                <td class='right aligned'>
                                    {$this->translator->_("report.printViewProductSales.totals")}
                                    {$currentProductsCategoryName}
                                </td>
                                <td>{$currentCategoryTotalQuantity}</td>
                                <td colspan='2'></td>
                                <td class='right aligned'>{$categoryTotalSalesSum}</td>
                                <td class='right aligned'>{$categoryTotalProfit}</td>
                            </tr>
                        ";
                    }
                    $documentHtml .= "
                        <tr class='app-highlight-tr'>
                            <td colspan='6'>{$product["category"]}</td>
                        </tr>
                    ";
                    $currentProductsCategoryId = $product["categoryId"];
                    $currentProductsCategoryName = $product["category"];
                    $currentCategoryTotalQuantity = $product["quantity"];
                    $currentCategoryTotalSalesSum = $product["salesSum"];
                    $currentCategoryTotalProfit = $product["profit"];
                } else {
                    $currentCategoryTotalQuantity += $product["quantity"];
                    $currentCategoryTotalSalesSum += $product["salesSum"];
                    $currentCategoryTotalProfit += $product["profit"];
                }
                $buyingPrice = number_format($product["buyingPrice"], 0, '', ' ');
                $unitPrice = number_format($product["unitPrice"], 0, '', ' ');
                $salesSum = number_format($product["salesSum"], 0, '', ' ');
                $profit = number_format($product["profit"], 0, '', ' ');
                $documentHtml .= "
                    <tr>
                        <td>{$product["designation"]}</td>
                        <td>{$product["quantity"]}</td>
                        <td class='right aligned'>{$buyingPrice}</td>
                        <td class='right aligned'>{$unitPrice}</td>
                        <td class='right aligned'>{$salesSum}</td>
                        <td class='right aligned'>{$profit}</td>
                    </tr>
                ";
            }
            if ($currentProductsPointOfSaleId != null) {
                if ($currentProductsCategoryId != null) {
                    $categoryTotalSalesSum = number_format($currentCategoryTotalSalesSum, 0, '', ' ');
                    $categoryTotalProfit = number_format($currentCategoryTotalProfit, 0, '', ' ');
                    $documentHtml .= "
                        <tr class='app-total-tr'>
                            <td class='right aligned'>
                                {$this->translator->_("report.printViewProductSales.totals")}
                                {$currentProductsCategoryName}
                            </td>
                            <td>{$currentCategoryTotalQuantity}</td>
                            <td colspan='2'></td>
                            <td class='right aligned'>{$categoryTotalSalesSum}</td>
                            <td class='right aligned'>{$categoryTotalProfit}</td>
                        </tr>
                    ";
                }
                $pointOfSaleProductsSoldSum = number_format($productsSumForPointOfSale[$currentProductsPointOfSaleId]["salesSum"], 0, '', ' ');
                $pointOfSaleProductsProfit = number_format($productsSumForPointOfSale[$currentProductsPointOfSaleId]["profit"], 0, '', ' ');
                $documentHtml .= "
                            <tr class='app-total-tr'>
                                <td class='right aligned'>
                                    <i class='violet-label'>
                                        {$this->translator->_("report.printViewProductSales.totals")}
                                        {$productsSumForPointOfSale[$currentProductsPointOfSaleId]["name"]}
                                    </i>
                                </td>
                                <td>{$productsSumForPointOfSale[$currentProductsPointOfSaleId]["quantity"]}</td>
                                <td colspan='2'></td>
                                <td class='right aligned'>{$pointOfSaleProductsSoldSum}</td>
                                <td class='right aligned'>{$pointOfSaleProductsProfit}</td>
                            </tr>
                        <tbody>
                        <tfoot>
                            <tr>
                                <th style='width: 40%;'>{$this->translator->_("report.printViewProductSales.designation")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductSales.quantity")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductSales.buyingPrice")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductSales.unitPrice")}</th>
                                <th style='width: 15%;'>{$this->translator->_("report.printViewProductSales.sumOfSales")}</th>
                                <th style='width: 15%;'>{$this->translator->_("report.printViewProductSales.profit")}</th>
                            </tr>
                        </tfoot>
                    </table>
                    <hr/>
                ";
            }
            if (count($pointsOfSale) !== 1 && count($productsCount) > 0) {
                $quantityTotal = 0;
                $salesSumTotal = 0;
                $profitTotal = 0;
                $summaryProductsCategoryId = null;
                $summaryProductsCategoryName = null;
                $summaryCategoryTotalQuantity = 0;
                $summaryCategoryTotalSalesSum = 0;
                $summaryCategoryTotalProfit = 0;
                $documentHtml .= "
                    <table class='border'>
                        <thead>
                            <tr>
                                <th colspan='6' class='left aligned'>
                                    <i class='secondary-label'>{$this->translator->_("report.printViewProductSales.summaryOfProductsSold")}</i>
                                </th>
                            </tr>
                            <tr>
                                <th style='width: 40%;'>{$this->translator->_("report.printViewProductSales.designation")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductSales.quantity")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductSales.buyingPrice")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductSales.unitPrice")}</th>
                                <th style='width: 15%;'>{$this->translator->_("report.printViewProductSales.sumOfSales")}</th>
                                <th style='width: 15%;'>{$this->translator->_("report.printViewProductSales.profit")}</th>
                            </tr>
                        </thead>
                        <tbody>
                ";
                foreach ($productsSumGlobal as $productId => $productValues) {
                    $quantityTotal += $productValues["quantity"];
                    $salesSumTotal += $productValues["salesSum"];
                    $profitTotal += $productValues["profit"];
                    if ($productValues["categoryId"] != $summaryProductsCategoryId) {
                        if ($summaryProductsCategoryId != null) {
                            $categoryTotalSalesSum = number_format($summaryCategoryTotalSalesSum, 0, '', ' ');
                            $categoryTotalProfit = number_format($summaryCategoryTotalProfit, 0, '', ' ');
                            $documentHtml .= "
                                <tr class='app-total-tr'>
                                    <td class='right aligned'>
                                        {$this->translator->_("report.printViewProductSales.totals")}
                                        {$summaryProductsCategoryName}
                                    </td>
                                    <td>{$summaryCategoryTotalQuantity}</td>
                                    <td colspan='2'></td>
                                    <td class='right aligned'>{$categoryTotalSalesSum}</td>
                                    <td class='right aligned'>{$categoryTotalProfit}</td>
                                </tr>
                            ";
                        }
                        $documentHtml .= "
                            <tr class='app-highlight-tr'>
                                <td colspan='6'>{$productValues["category"]}</td>
                            </tr>
                        ";
                        $summaryProductsCategoryId = $productValues["categoryId"];
                        $summaryProductsCategoryName = $productValues["category"];
                        $summaryCategoryTotalQuantity = $productValues["quantity"];
                        $summaryCategoryTotalSalesSum = $productValues["salesSum"];
                        $summaryCategoryTotalProfit = $productValues["profit"];
                    } else {
                        $summaryCategoryTotalQuantity += $productValues["quantity"];
                        $summaryCategoryTotalSalesSum += $productValues["salesSum"];
                        $summaryCategoryTotalProfit += $productValues["profit"];
                    }
                    $productBuyingPrice = number_format($productValues["buyingPrice"], 0, '', ' ');
                    $productUnitPrice = number_format($productValues["unitPrice"], 0, '', ' ');
                    $productSalesSum = number_format($productValues["salesSum"], 0, '', ' ');
                    $productProfit = number_format($productValues["profit"], 0, '', ' ');
                    $documentHtml .= "
                        <tr>
                            <td>{$productValues["name"]}</td>
                            <td>{$productValues["quantity"]}</td>
                            <td class='right aligned'>{$productBuyingPrice}</td>
                            <td class='right aligned'>{$productUnitPrice}</td>
                            <td class='right aligned'>{$productSalesSum}</td>
                            <td class='right aligned'>{$productProfit}</td>
                        </tr>
                    ";
                };
                if ($summaryProductsCategoryId != null) {
                    $categoryTotalSalesSum = number_format($summaryCategoryTotalSalesSum, 0, '', ' ');
                    $categoryTotalProfit = number_format($summaryCategoryTotalProfit, 0, '', ' ');
                    $documentHtml .= "
                        <tr class='app-total-tr'>
                            <td class='right aligned'>
                                {$this->translator->_("report.printViewProductSales.totals")}
                                {$summaryProductsCategoryName}
                            </td>
                            <td>{$summaryCategoryTotalQuantity}</td>
                            <td colspan='2'></td>
                            <td class='right aligned'>{$categoryTotalSalesSum}</td>
                            <td class='right aligned'>{$categoryTotalProfit}</td>
                        </tr>
                    ";
                }
                $salesSumTotalFormat = number_format($salesSumTotal, 0, '', ' ');
                $profitTotalFormat = number_format($profitTotal, 0, '', ' ');
                $documentHtml .= "
                            <tr class='app-total-tr'>
                                <td class='right aligned'>
                                    <i class='secondary-label'>{$this->translator->_("report.printViewProductSales.totals")}</a>
                                </td>
                                <td>{$quantityTotal}</td>
                                <td colspan='2'></span>
                                <td class='right aligned'>{$salesSumTotalFormat}</td>
                                <td class='right aligned'>{$profitTotalFormat}</td>
                            </tr>
                        </tbody>
                        <tfoot>
                            <tr>
                                <th style='width: 40%;'>{$this->translator->_("report.printViewProductSales.designation")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductSales.quantity")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductSales.buyingPrice")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductSales.unitPrice")}</th>
                                <th style='width: 15%;'>{$this->translator->_("report.printViewProductSales.sumOfSales")}</th>
                                <th style='width: 15%;'>{$this->translator->_("report.printViewProductSales.profit")}</th>
                            </tr>
                        </tfoot>
                    </table>
                ";
            }
            $documentHtml .= " 
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "product_sales_{$_SESSION[SESSION_USER_ID]}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/reports/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/reports/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printviewsalesbycustomerAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/reports") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $personDao = new PersonDAO();
            $pointOfSaleDao = new PointOfSaleDAO();
            $customerInvoiceDao = new CustomerInvoiceDAO();
            $invoicePaymentDao = new InvoicePaymentDAO();
            $customerId = (int)$_POST["customer"];
            $customer = $personDao->getPersonById($customerId);
            $pointsOfSaleIds = json_decode($_POST["pointsOfSale"], true);
            $startDate = new DateTime("{$_POST["startDate"]}:00");
            $endDate = new DateTime("{$_POST["endDate"]}:59");
            $pointsOfSale = [];
            $pointOfSaleNames = "";
            $countPointOfSale = 0;
            foreach ($pointsOfSaleIds as $pointOfSaleId) {
                $pointOfSale = $pointOfSaleDao->getPointOfSaleById((int)$pointOfSaleId);
                $pointsOfSale[] = $pointOfSale;
                if ($countPointOfSale > 0) {
                    $pointOfSaleNames .= ", {$pointOfSale->getNomPointVente()}";
                } else {
                    $pointOfSaleNames .= $pointOfSale->getNomPointVente();
                }
                $countPointOfSale++;
            }
            if (count($pointsOfSale) == 0) $pointOfSaleNames = $this->translator->_("report.printViewSalesByCustomer.allPointsOfSale");
            $sales = $customerInvoiceDao->getSalesByCustomer($customer, $pointsOfSale, $startDate, $endDate);
            $salesResult = [];
            foreach ($sales as $sale) {
                $salesResult[] = [
                    "id" => $sale->getIdFacture(),
                    "dateAndHour" => $sale->getDateFacture()->format(DATE_FORMAT_WITH_SECONDS),
                    "amount" => $sale->getMontantNet(),
                    "reductionAmount" => $sale->getForfaitRemise(),
                    "paidAmount" => $invoicePaymentDao->getSumOfInvoicePayments($sale),
                    "pointOfSaleId" => $sale->getIdPointVente()->getIdPointVente(),
                    "pointOfSaleName" => $sale->getIdPointVente()->getNomPointVente(),
                ];
            }
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $secondaryBrand = APP_SECONDARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = $this->translator->_("report.printViewSalesByCustomer.salesPerCustomer");
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: left;  
                                color: #000000; 
                            }

                            td.center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            .no-line-found {
                                color: grey !important;
                                font-weight: bold !important;
                                font-size: 16px !important;
                            }

                            td.validated {
                                color: {$primaryBrand} !important;
                                font-weight: bold;
                            }

                            td.not-validated {
                                color: {$secondaryBrand} !important;
                                font-weight: bold;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            .equalized {
                                table-layout: fixed;
                            }

                            bold {
                                color: black;
                            }

                            i.violet-label {
                                background-color: #6435c9;
                                color: white;
                                padding: 2px;
                                border-radius: 6px;
                                margin: 3px;
                            }

                            i.blue-label {
                                background-color: #2185d0;
                                color: white;
                                padding: 2px;
                                border-radius: 6px;
                                margin: 3px;
                            }

                            i.secondary-label {
                                background-color: {$secondaryBrand};
                                color: white;
                                padding: 2px;
                                border-radius: 6px;
                                margin: 3px;
                            }

                            th.left.aligned {
                                text-align: left;
                            }

                            td.right.aligned {
                                text-align: right;
                            }

                            tr.app-total-tr {
                                background-color: {$primaryBrand}80;
                            }

                            tr.app-total-tr > td {
                                color: {$thirdBrand};
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$printTitle}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border equalized'>
                                <thead>
                                    <tr>
                                        <th class='center-text'>{$this->translator->_("report.printViewSalesByCustomer.customer")} : <bold class='black-bold'>{$customer->getNom()} {$customer->getPrenom()}</bold></th>
                                        <th class='center-text'>{$this->translator->_("report.printViewSalesByCustomer.pointsOfSale")} : <bold class='black-bold'>{$pointOfSaleNames}</bold></th>
                                    </tr>
                                    <tr>
                                        <th class='center-text'>{$this->translator->_("report.printViewSalesByCustomer.startDate")} : <bold class='black-bold'>{$startDate->format(DATE_FORMAT_WITH_TIME)}</bold></th>
                                        <th class='center-text'>{$this->translator->_("report.printViewSalesByCustomer.endDate")} : <bold class='black-bold'>{$endDate->format(DATE_FORMAT_WITH_TIME)}</bold></th>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
            ";
            $printFilters .= "{$this->translator->_("report.printViewSalesByCustomer.customer")} : {$customer->getNom()} {$customer->getPrenom()}";
            $printFilters .= ", {$this->translator->_("report.printViewSalesByCustomer.pointsOfSale")} : {$pointOfSaleNames}";
            $printFilters .= ", {$this->translator->_("report.printViewSalesByCustomer.startDate")} : {$startDate->format(DATE_FORMAT_WITH_TIME)}";
            $printFilters .= ", {$this->translator->_("report.printViewSalesByCustomer.endDate")} : {$endDate->format(DATE_FORMAT_WITH_TIME)}";
            $currentSalesPointOfSaleId = null;
            $salesSumForPointOfSale = [];
            foreach ($salesResult as $sale) {
                if ($sale["pointOfSaleId"] != $currentSalesPointOfSaleId) {
                    $salesSumForPointOfSale[$sale["pointOfSaleId"]] = [
                        "amount" => $sale["amount"],
                        "reductionAmount" => $sale["reductionAmount"],
                        "paidAmount" => $sale["paidAmount"],
                        "salesCount" => 1,
                        "salesUnpaidCount" => $sale["paidAmount"] == 0 ? 1 : 0,
                        "salesInPaymentCount" => $sale["paidAmount"] > 0 && $sale["paidAmount"] < $sale["amount"] ? 1 : 0,
                        "salesPaidCount" => $sale["paidAmount"] == $sale["amount"] ? 1 : 0,
                        "salesWithReductionCount" => $sale["reductionAmount"] > 0 ? 1 : 0,
                        "name" => $sale["pointOfSaleName"],
                    ];
                    if ($currentSalesPointOfSaleId != null) {
                        $pointOfSaleSalesSumAmount = number_format($salesSumForPointOfSale[$currentSalesPointOfSaleId]["amount"], 0, '', ' ');
                        $pointOfSaleSalesSumReductionAmount = number_format($salesSumForPointOfSale[$currentSalesPointOfSaleId]["reductionAmount"], 0, '', ' ');
                        $pointOfSaleSalesSumPaidAmount = number_format($salesSumForPointOfSale[$currentSalesPointOfSaleId]["paidAmount"], 0, '', ' ');
                        $documentHtml .= "
                                    <tr class='app-total-tr'>
                                        <td colspan='2' class='right aligned'>
                                            <i class='violet-label'>
                                                {$this->translator->_("report.printViewSalesByCustomer.totals")}
                                                {$salesSumForPointOfSale[$currentSalesPointOfSaleId]["name"]}
                                            </i>
                                        </td>
                                        <td class='right aligned'>{$pointOfSaleSalesSumAmount}</td>
                                        <td class='right aligned'>{$pointOfSaleSalesSumReductionAmount}</td>
                                        <td class='right aligned'>{$pointOfSaleSalesSumPaidAmount}</td>
                                    </tr>
                                </tbody>
                                <tfoot>
                                    <tr>
                                        <th>{$this->translator->_("report.printViewSalesByCustomer.saleNo")}</th>
                                        <th>{$this->translator->_("report.printViewSalesByCustomer.dateAndHour")}</th>
                                        <th>{$this->translator->_("report.printViewSalesByCustomer.amount")}</th>
                                        <th>{$this->translator->_("report.printViewSalesByCustomer.reduction")}</th>
                                        <th>{$this->translator->_("report.printViewSalesByCustomer.paidAmount")}</th>
                                    </tr>
                                </tfoot>
                            </table>
                            <hr/>
                        ";
                    }
                    $documentHtml .= "
                        <table class='border equalized'>
                            <thead>
                    ";
                    if (count($pointsOfSale) !== 1) {
                        $documentHtml .= "
                                <tr>
                                    <th colspan='5' class='left aligned'>
                                        <i class='violet-label'>{$sale["pointOfSaleName"]}</i>
                                    </th>
                                </tr>
                        ";
                    }
                    $documentHtml .= "
                                <tr>
                                    <th>{$this->translator->_("report.printViewSalesByCustomer.saleNo")}</th>
                                    <th>{$this->translator->_("report.printViewSalesByCustomer.dateAndHour")}</th>
                                    <th>{$this->translator->_("report.printViewSalesByCustomer.amount")}</th>
                                    <th>{$this->translator->_("report.printViewSalesByCustomer.reduction")}</th>
                                    <th>{$this->translator->_("report.printViewSalesByCustomer.paidAmount")}</th>
                                </tr>
                            </thead>
                            <tbody>
                    ";
                    $currentSalesPointOfSaleId = $sale["pointOfSaleId"];
                } else {
                    $salesSumForPointOfSale[$sale["pointOfSaleId"]]["amount"] += $sale["amount"];
                    $salesSumForPointOfSale[$sale["pointOfSaleId"]]["reductionAmount"] += $sale["reductionAmount"];
                    $salesSumForPointOfSale[$sale["pointOfSaleId"]]["paidAmount"] += $sale["paidAmount"];
                    $salesSumForPointOfSale[$sale["pointOfSaleId"]]["salesCount"] += 1;
                    $salesSumForPointOfSale[$sale["pointOfSaleId"]]["salesUnpaidCount"] += $sale["paidAmount"] == 0 ? 1 : 0;
                    $salesSumForPointOfSale[$sale["pointOfSaleId"]]["salesInPaymentCount"] += $sale["paidAmount"] > 0 && $sale["paidAmount"] < $sale["amount"] ? 1 : 0;
                    $salesSumForPointOfSale[$sale["pointOfSaleId"]]["salesPaidCount"] += $sale["paidAmount"] == $sale["amount"] ? 1 : 0;
                    $salesSumForPointOfSale[$sale["pointOfSaleId"]]["salesWithReductionCount"] += $sale["reductionAmount"] > 0 ? 1 : 0;
                }
                $saleMoment = new \Moment\Moment($sale["dateAndHour"]);
                $saleAmount = number_format($sale["amount"], 0, '', ' ');
                $saleReductionAmount = number_format($sale["reductionAmount"], 0, '', ' ');
                $salePaidAmount = number_format($sale["paidAmount"], 0, '', ' ');
                $documentHtml .= "
                    <tr>
                        <td>{$sale["id"]}</td>
                        <td>{$saleMoment->format(DATE_FORMAT_MIDDLE_WITH_SECONDS)}</td>
                        <td class='right aligned'>{$saleAmount}</td>
                        <td class='right aligned'>{$saleReductionAmount}</td>
                        <td class='right aligned'>{$salePaidAmount}</td>
                    </tr>
                ";
            }
            if ($currentSalesPointOfSaleId != null) {
                $pointOfSaleSalesSumAmount = number_format($salesSumForPointOfSale[$currentSalesPointOfSaleId]["amount"], 0, '', ' ');
                $pointOfSaleSalesSumReductionAmount = number_format($salesSumForPointOfSale[$currentSalesPointOfSaleId]["reductionAmount"], 0, '', ' ');
                $pointOfSaleSalesSumPaidAmount = number_format($salesSumForPointOfSale[$currentSalesPointOfSaleId]["paidAmount"], 0, '', ' ');
                $documentHtml .= "
                            <tr class='app-total-tr'>
                                <td colspan='2' class='right aligned'>
                                    <i class='violet-label'>
                                        {$this->translator->_("report.printViewSalesByCustomer.totals")}
                                        {$salesSumForPointOfSale[$currentSalesPointOfSaleId]["name"]}
                                    </i>
                                </td>
                                <td class='right aligned'>{$pointOfSaleSalesSumAmount}</td>
                                <td class='right aligned'>{$pointOfSaleSalesSumReductionAmount}</td>
                                <td class='right aligned'>{$pointOfSaleSalesSumPaidAmount}</td>
                            </tr>
                        </tbody>
                        <tfoot>
                            <tr>
                                <th>{$this->translator->_("report.printViewSalesByCustomer.saleNo")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCustomer.dateAndHour")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCustomer.amount")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCustomer.reduction")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCustomer.paidAmount")}</th>
                            </tr>
                        </tfoot>
                    </table>
                    <hr/>
                ";
            }
            if (count($salesResult) > 0) {
                $salesTotal = 0;
                $paymentTotal = 0;
                $reductionTotal = 0;
                $numberOfSales = 0;
                $paid = 0;
                $paymentInProgess = 0;
                $unpaid = 0;
                $withReduction = 0;
                $documentHtml .= "
                    <table class='border'>
                        <thead>
                            <tr>
                                <th colspan='9' class='left aligned'>
                                    <i class='secondary-label'>{$this->translator->_("report.printViewSalesByCustomer.summaryOfSalesPerPointOfSale")}</i>
                                </th>
                            </tr>
                            <tr>
                                <th style='width: 20%;'>{$this->translator->_("report.printViewSalesByCustomer.pointOfSale")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewSalesByCustomer.salesTotal")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewSalesByCustomer.paymentTotal")}</th>
                                <th style='width: 10%'>{$this->translator->_("report.printViewSalesByCustomer.reductionTotal")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCustomer.numberOfSales")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCustomer.paid")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCustomer.paymentInProgess")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCustomer.unpaid")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCustomer.withReduction")}</th>
                            </tr>
                        </thead>
                        <tbody>
                ";
                foreach ($salesSumForPointOfSale as $pointOfSaleId => $pointOfSaleValues) {
                    $salesTotal += $pointOfSaleValues["amount"];
                    $paymentTotal += $pointOfSaleValues["paidAmount"];
                    $reductionTotal += $pointOfSaleValues["reductionAmount"];
                    $numberOfSales += $pointOfSaleValues["salesCount"];
                    $paid += $pointOfSaleValues["salesPaidCount"];
                    $paymentInProgess += $pointOfSaleValues["salesInPaymentCount"];
                    $unpaid += $pointOfSaleValues["salesUnpaidCount"];
                    $withReduction += $pointOfSaleValues["salesWithReductionCount"];
                    $pointOfSaleAmount = number_format($pointOfSaleValues["amount"], 0, '', ' ');
                    $pointOfSalePaidAmount = number_format($pointOfSaleValues["paidAmount"], 0, '', ' ');
                    $pointOfSaleReductionAmount = number_format($pointOfSaleValues["reductionAmount"], 0, '', ' ');
                    $documentHtml .= "
                        <tr>
                            <td>{$pointOfSaleValues["name"]}</td>
                            <td class='right aligned'>{$pointOfSaleAmount}</td>
                            <td class='right aligned'>{$pointOfSalePaidAmount}</td>
                            <td class='right aligned'>{$pointOfSaleReductionAmount}</td>
                            <td>{$pointOfSaleValues["salesCount"]}</td>
                            <td>{$pointOfSaleValues["salesPaidCount"]}</td>
                            <td>{$pointOfSaleValues["salesInPaymentCount"]}</td>
                            <td>{$pointOfSaleValues["salesUnpaidCount"]}</td>
                            <td>{$pointOfSaleValues["salesWithReductionCount"]}</td>
                        </tr>
                    ";
                };
                $salesTotalFormat = number_format($salesTotal, 0, '', ' ');
                $paymentTotalFormat = number_format($paymentTotal, 0, '', ' ');
                $reductionTotalFormat = number_format($reductionTotal, 0, '', ' ');
                $documentHtml .= "
                            <tr class='app-total-tr'>
                                <td class='right aligned'>
                                    <i class='secondary-label'>{$this->translator->_("report.printViewSalesByCustomer.totals")}</i>
                                </td>
                                <td class='right aligned'>{$salesTotalFormat}</td>
                                <td class='right aligned'>{$paymentTotalFormat}</td>
                                <td class='right aligned'>{$reductionTotalFormat}</td>
                                <td>{$numberOfSales}</td>
                                <td>{$paid}</td>
                                <td>{$paymentInProgess}</td>
                                <td>{$unpaid}</td>
                                <td>{$withReduction}</td>
                            </tr>
                        </tbody>
                        <tfoot>
                            <tr>
                                <th>{$this->translator->_("report.printViewSalesByCustomer.pointOfSale")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCustomer.salesTotal")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCustomer.paymentTotal")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCustomer.reductionTotal")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCustomer.numberOfSales")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCustomer.paid")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCustomer.paymentInProgess")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCustomer.unpaid")}</th>
                                <th>{$this->translator->_("report.printViewSalesByCustomer.withReduction")}</th>
                            </tr>
                        </tfoot>
                    </table>
                ";
            }
            $documentHtml .= " 
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "sales_per_customer_{$_SESSION[SESSION_USER_ID]}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/reports/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/reports/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printviewproductssoldbycustomerAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/reports") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $personDao = new PersonDAO();
            $pointOfSaleDao = new PointOfSaleDAO();
            $customerInvoiceLineDao = new CustomerInvoiceLineDAO();
            $customerId = (int)$_POST["customer"];
            $customer = $personDao->getPersonById($customerId);
            $pointsOfSaleIds = json_decode($_POST["pointsOfSale"], true);
            $startDate = new DateTime("{$_POST["startDate"]}:00");
            $endDate = new DateTime("{$_POST["endDate"]}:59");
            $pointsOfSale = [];
            $pointOfSaleNames = "";
            $countPointOfSale = 0;
            foreach ($pointsOfSaleIds as $pointOfSaleId) {
                $pointOfSale = $pointOfSaleDao->getPointOfSaleById((int)$pointOfSaleId);
                $pointsOfSale[] = $pointOfSale;
                if ($countPointOfSale > 0) {
                    $pointOfSaleNames .= ", {$pointOfSale->getNomPointVente()}";
                } else {
                    $pointOfSaleNames .= $pointOfSale->getNomPointVente();
                }
                $countPointOfSale++;
            }
            if (count($pointsOfSale) == 0) $pointOfSaleNames = $this->translator->_("report.printViewProductsSoldByCustomer.allPointsOfSale");
            $productsCount = $customerInvoiceLineDao->getSoldArticleCountByCustomer($customer, $pointsOfSale, $startDate, $endDate);
            $productsCountResult = [];
            foreach ($productsCount as $productCount) {
                $productsCountResult[] = [
                    "designation" => $productCount["product_name"],
                    "productId" => $productCount["product_id"],
                    "category" => $productCount["category_name"],
                    "categoryId" => $productCount["category_id"],
                    "quantity" => (int)$productCount["sold_quantity"],
                    "buyingPrice" => (float)$productCount["buying_price"],
                    "unitPrice" => (float)$productCount["unit_price"],
                    "salesSum" => (float)$productCount["sold_amount"],
                    "profit" => (float)$productCount["profit"],
                    "pointOfSaleId" => (int)$productCount["point_of_sale_id"],
                    "pointOfSaleName" => $productCount["point_of_sale_name"],
                ];
            }
            $enterpriseName = NOM_STRUCTURE;
            $poBox = BP_STRUCTURE;
            $primaryBrand = APP_PRIMARY_BRAND;
            $secondaryBrand = APP_SECONDARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $appLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/assets/app_logo.png";
            $currentDate = new DateTime();
            $printTime = $currentDate->format(DATE_FORMAT_WITH_TIME);
            $printCode = Utility::generatePrintCode($currentDate);
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($printCode);
            $printTitle = $this->translator->_("report.printViewProductsSoldByCustomer.productsSoldPerCustomer");
            $printFilters = "";
            $documentHtml = "
                <html>
                    <head>
                    </head>
                    <body>
                        <style>
                            @page {
                                margin: 45px 0;
                            }
                        
                            #header {
                                position:fixed;
                                left: 0;  
                                top: -45px;
                                right: 0;
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                        
                            #footer { 
                                position: fixed; 
                                left: 0; 
                                bottom: -45px; 
                                right: 0; 
                                height: 31px;
                                width: 100%;
                                background-color: {$primaryBrand};
                                padding: 5px;
                            }
                            
                            .header-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }

                            .footer-image {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                left: 2px;
                            }
                            
                            .header-crescita {
                                font-weight: bold; 
                                font-size: 25px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -55px; 
                                left: 40px;
                            }
                            
                            .footer-crescita {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -55px;
                                left: 40px;
                            }
                            
                            .header-time {
                                font-weight: bold; 
                                font-size: 20px; 
                                color: {$thirdBrand}; 
                                position: relative; 
                                top: -105px; 
                                right: 50px;
                                text-align: right;
                            }
                            
                            .footer-time {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                top: -105px;
                                right: 50px;
                                text-align: right;
                            }

                            .header-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .footer-time:after {
                                content: 'N° ' + counter(page);
                                margin-left: 5px;
                                padding-left: 5px;
                                border-left: solid 2px {$thirdBrand};
                            }

                            .header-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }

                            .footer-print-code {
                                width: 30px; 
                                height: 30px; 
                                border-radius: 10px;
                                position: relative;
                                top: -160px;
                                left: 750px;
                            }
                            
                            .main {
                                padding-left: 25px;
                                padding-right: 25px;
                            }

                            table { 
                                width: 100%; 
                                color: #000000; 
                                font-family: helvetica; 
                                line-height: 5mm;
                                font-size: 13px;
                                border-collapse: collapse;
                            }

                            .border th { 
                                border: 1px solid #000000;  
                                color: #000000;                                       
                                font-weight: bold; 
                                font-size: 15px; 
                                text-align: center; 
                            }

                            .border td { 
                                border: 1px solid #000000; 
                                text-align: left;  
                                color: #000000; 
                            }

                            td.center-text {
                                text-align: center;
                            }

                            .black-bold {
                                font-weight: bold;
                                font-size: 15px;
                                color: black;
                            }

                            .no-line-found {
                                color: grey !important;
                                font-weight: bold !important;
                                font-size: 16px !important;
                            }

                            td.validated {
                                color: {$primaryBrand} !important;
                                font-weight: bold;
                            }

                            td.not-validated {
                                color: {$secondaryBrand} !important;
                                font-weight: bold;
                            }

                            .title {
                                font-size: 16px;
                                font-weight: bold;
                                text-align: center;
                            }

                            .equalized {
                                table-layout: fixed;
                            }

                            bold {
                                color: black;
                            }

                            i.violet-label {
                                background-color: #6435c9;
                                color: white;
                                padding: 2px;
                                border-radius: 6px;
                                margin: 3px;
                            }

                            i.blue-label {
                                background-color: #2185d0;
                                color: white;
                                padding: 2px;
                                border-radius: 6px;
                                margin: 3px;
                            }

                            i.secondary-label {
                                background-color: {$secondaryBrand};
                                color: white;
                                padding: 2px;
                                border-radius: 6px;
                                margin: 3px;
                            }

                            th.left.aligned {
                                text-align: left;
                            }

                            td.right.aligned {
                                text-align: right;
                            }

                            tr.app-total-tr {
                                background-color: {$primaryBrand}80;
                            }

                            tr.app-total-tr > td {
                                color: {$thirdBrand};
                            }

                            tr.app-highlight-tr {
                                background-color: {$secondaryBrand}80;
                            }

                            tr.app-highlight-tr > td {
                                color: {$thirdBrand};
                            }

                            td {
                                padding-right: 3px;
                                padding-left: 3px;
                            }
                        </style>
                        <div id='header'>
                            <img class='header-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='header-crescita'>Crescita</h3>
                            <h4 class='header-time'>{$printTime}</h4>
                            <img class='header-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div id='footer'>
                            <img class='footer-image' src='{$appLogoLink}' alt='logo'/>
                            <h3 class='footer-crescita'>Crescita</h3>
                            <h4 class='footer-time'>{$printTime}</h4>
                            <img class='footer-print-code' src='{$qrCodeImage}' alt='print-code'>
                        </div>
                        <div class='main'>
                            <table>
                                <thead>
                                    <tr>
                                        <td>
                                            <bold>
                                                <h1>{$enterpriseName}</h1>
                                                <h3>{$this->translator->_("shared.pob")}: {$poBox}</h3>
                                            </bold>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td class='title'>{$printTitle}</td>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
                            <table class='border equalized'>
                                <thead>
                                    <tr>
                                        <th class='center-text'>{$this->translator->_("report.printViewProductsSoldByCustomer.customer")} : <bold class='black-bold'>{$customer->getNom()} {$customer->getPrenom()}</bold></th>
                                        <th class='center-text'>{$this->translator->_("report.printViewProductsSoldByCustomer.pointsOfSale")} : <bold class='black-bold'>{$pointOfSaleNames}</bold></th>
                                    </tr>
                                    <tr>
                                        <th class='center-text'>{$this->translator->_("report.printViewProductsSoldByCustomer.startDate")} : <bold class='black-bold'>{$startDate->format(DATE_FORMAT_WITH_TIME)}</bold></th>
                                        <th class='center-text'>{$this->translator->_("report.printViewProductsSoldByCustomer.endDate")} : <bold class='black-bold'>{$endDate->format(DATE_FORMAT_WITH_TIME)}</bold></th>
                                    </tr>
                                </thead>
                            </table>
                            <hr/>
            ";
            $printFilters .= "{$this->translator->_("report.printViewProductsSoldByCustomer.customer")} : {$customer->getNom()} {$customer->getPrenom()}";
            $printFilters .= ", {$this->translator->_("report.printViewProductsSoldByCustomer.pointsOfSale")} : {$pointOfSaleNames}";
            $printFilters .= ", {$this->translator->_("report.printViewProductsSoldByCustomer.startDate")} : {$startDate->format(DATE_FORMAT_WITH_TIME)}";
            $printFilters .= ", {$this->translator->_("report.printViewProductsSoldByCustomer.endDate")} : {$endDate->format(DATE_FORMAT_WITH_TIME)}";
            $currentProductsPointOfSaleId = null;
            $currentProductsCategoryId = null;
            $currentProductsCategoryName = null;
            $currentCategoryTotalQuantity = 0;
            $currentCategoryTotalSalesSum = 0;
            $currentCategoryTotalProfit = 0;
            $productsSumForPointOfSale = [];
            $productsSumGlobal = [];
            foreach ($productsCountResult as $product) {
                if ($product["pointOfSaleId"] != $currentProductsPointOfSaleId) {
                    $productsSumForPointOfSale[$product["pointOfSaleId"]] = [
                        "quantity" => $product["quantity"],
                        "salesSum" => $product["salesSum"],
                        "profit" => $product["profit"],
                        "name" => $product["pointOfSaleName"],
                    ];
                    $productsSumForCashierByPointOfSale[$product["pointOfSaleId"]] = [];
                    if ($currentProductsPointOfSaleId != null) {
                        if ($currentProductsCategoryId != null) {
                            $categoryTotalSalesSum = number_format($currentCategoryTotalSalesSum, 0, '', ' ');
                            $categoryTotalProfit = number_format($currentCategoryTotalProfit, 0, '', ' ');
                            $documentHtml .= "
                                <tr class='app-total-tr'>
                                    <td class='right aligned'>
                                        {$this->translator->_("report.printViewProductsSoldByCustomer.totals")}
                                        {$currentProductsCategoryName}
                                    </td>
                                    <td>{$currentCategoryTotalQuantity}</td>
                                    <td colspan='2'></td>
                                    <td class='right aligned'>{$categoryTotalSalesSum}</td>
                                    <td class='right aligned'>{$categoryTotalProfit}</td>
                                </tr>
                            ";
                        }
                        $pointOfSaleProductsSoldSum = number_format($productsSumForPointOfSale[$currentProductsPointOfSaleId]["salesSum"], 0, '', ' ');
                        $pointOfSaleProductsProfit = number_format($productsSumForPointOfSale[$currentProductsPointOfSaleId]["profit"], 0, '', ' ');
                        $documentHtml .= "
                                    <tr class='app-total-tr'>
                                        <td class='right aligned'>
                                            <i class='violet-label'>
                                                {$this->translator->_("report.printViewProductsSoldByCustomer.totals")}
                                                {$productsSumForPointOfSale[$currentProductsPointOfSaleId]["name"]}
                                            </i>
                                        </td>
                                        <td>{$productsSumForPointOfSale[$currentProductsPointOfSaleId]["quantity"]}</td>
                                        <td colspan='2'></td>
                                        <td class='right aligned'>{$pointOfSaleProductsSoldSum}</td>
                                        <td class='right aligned'>{$pointOfSaleProductsProfit}</td>
                                    </tr>
                                <tbody>
                                <tfoot>
                                    <tr>
                                        <th style='width: 40%;'>{$this->translator->_("report.printViewProductsSoldByCustomer.designation")}</th>
                                        <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByCustomer.quantity")}</th>
                                        <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByCustomer.buyingPrice")}</th>
                                        <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByCustomer.unitPrice")}</th>
                                        <th style='width: 15%;'>{$this->translator->_("report.printViewProductsSoldByCustomer.sumOfSales")}</th>
                                        <th style='width: 15%;'>{$this->translator->_("report.printViewProductsSoldByCustomer.profit")}</th>
                                    </tr>
                                </tfoot>
                            </table>
                            <hr/>
                        ";
                    }
                    $documentHtml .= "
                        <table class='border'>
                            <thead>
                    ";
                    if (count($pointsOfSale) != 1) {
                        $documentHtml .= "
                                <tr>
                                    <th colspan='6' class='left aligned'>
                                        <i class='violet-label'>{$product["pointOfSaleName"]}</i>
                                    </th>
                                </tr>
                    ";
                    }
                    $documentHtml .= "
                                <tr>
                                    <th style='width: 40%;'>{$this->translator->_("report.printViewProductsSoldByCustomer.designation")}</th>
                                    <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByCustomer.quantity")}</th>
                                    <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByCustomer.buyingPrice")}</th>
                                    <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByCustomer.unitPrice")}</th>
                                    <th style='width: 15%;'>{$this->translator->_("report.printViewProductsSoldByCustomer.sumOfSales")}</th>
                                    <th style='width: 15%;'>{$this->translator->_("report.printViewProductsSoldByCustomer.profit")}</th>
                                </tr>
                            </thead>
                            <tbody>
                    ";
                    $currentProductsPointOfSaleId = $product["pointOfSaleId"];
                    $currentProductsCategoryId = null;
                    $currentProductsCategoryName = null;
                    $currentCategoryTotalQuantity = 0;
                    $currentCategoryTotalSalesSum = 0;
                    $currentCategoryTotalProfit = 0;
                } else {
                    $productsSumForPointOfSale[$product["pointOfSaleId"]]["quantity"] += $product["quantity"];
                    $productsSumForPointOfSale[$product["pointOfSaleId"]]["salesSum"] += $product["salesSum"];
                    $productsSumForPointOfSale[$product["pointOfSaleId"]]["profit"] += $product["profit"];
                }
                if (!isset($productsSumGlobal[$product["productId"]])) {
                    $productsSumGlobal[$product["productId"]] = [
                        "quantity" => $product["quantity"],
                        "buyingPrice" => $product["buyingPrice"],
                        "unitPrice" => $product["unitPrice"],
                        "salesSum" => $product["salesSum"],
                        "profit" => $product["profit"],
                        "name" => $product["designation"],
                        "categoryId" => $product["categoryId"],
                        "category" => $product["category"],
                    ];
                } else {
                    $productsSumGlobal[$product["productId"]]["quantity"] += $product["quantity"];
                    $productsSumGlobal[$product["productId"]]["salesSum"] += $product["salesSum"];
                    $productsSumGlobal[$product["productId"]]["profit"] += $product["profit"];
                }
                if ($product["categoryId"] != $currentProductsCategoryId) {
                    if ($currentProductsCategoryId != null) {
                        $categoryTotalSalesSum = number_format($currentCategoryTotalSalesSum, 0, '', ' ');
                        $categoryTotalProfit = number_format($currentCategoryTotalProfit, 0, '', ' ');
                        $documentHtml .= "
                            <tr class='app-total-tr'>
                                <td class='right aligned'>
                                    {$this->translator->_("report.printViewProductsSoldByCustomer.totals")}
                                    {$currentProductsCategoryName}
                                </td>
                                <td>{$currentCategoryTotalQuantity}</td>
                                <td colspan='2'></td>
                                <td class='right aligned'>{$categoryTotalSalesSum}</td>
                                <td class='right aligned'>{$categoryTotalProfit}</td>
                            </tr>
                        ";
                    }
                    $documentHtml .= "
                        <tr class='app-highlight-tr'>
                            <td colspan='6'>{$product["category"]}</td>
                        </tr>
                    ";
                    $currentProductsCategoryId = $product["categoryId"];
                    $currentProductsCategoryName = $product["category"];
                    $currentCategoryTotalQuantity = $product["quantity"];
                    $currentCategoryTotalSalesSum = $product["salesSum"];
                    $currentCategoryTotalProfit = $product["profit"];
                } else {
                    $currentCategoryTotalQuantity += $product["quantity"];
                    $currentCategoryTotalSalesSum += $product["salesSum"];
                    $currentCategoryTotalProfit += $product["profit"];
                }
                $buyingPrice = number_format($product["buyingPrice"], 0, '', ' ');
                $unitPrice = number_format($product["unitPrice"], 0, '', ' ');
                $salesSum = number_format($product["salesSum"], 0, '', ' ');
                $profit = number_format($product["profit"], 0, '', ' ');
                $documentHtml .= "
                    <tr>
                        <td>{$product["designation"]}</td>
                        <td>{$product["quantity"]}</td>
                        <td class='right aligned'>{$buyingPrice}</td>
                        <td class='right aligned'>{$unitPrice}</td>
                        <td class='right aligned'>{$salesSum}</td>
                        <td class='right aligned'>{$profit}</td>
                    </tr>
                ";
            }
            if ($currentProductsPointOfSaleId != null) {
                if ($currentProductsCategoryId != null) {
                    $categoryTotalSalesSum = number_format($currentCategoryTotalSalesSum, 0, '', ' ');
                    $categoryTotalProfit = number_format($currentCategoryTotalProfit, 0, '', ' ');
                    $documentHtml .= "
                        <tr class='app-total-tr'>
                            <td class='right aligned'>
                                {$this->translator->_("report.printViewProductsSoldByCustomer.totals")}
                                {$currentProductsCategoryName}
                            </td>
                            <td>{$currentCategoryTotalQuantity}</td>
                            <td colspan='2'></td>
                            <td class='right aligned'>{$categoryTotalSalesSum}</td>
                            <td class='right aligned'>{$categoryTotalProfit}</td>
                        </tr>
                    ";
                }
                $pointOfSaleProductsSoldSum = number_format($productsSumForPointOfSale[$currentProductsPointOfSaleId]["salesSum"], 0, '', ' ');
                $pointOfSaleProductsProfit = number_format($productsSumForPointOfSale[$currentProductsPointOfSaleId]["profit"], 0, '', ' ');
                $documentHtml .= "
                            <tr class='app-total-tr'>
                                <td class='right aligned'>
                                    <i class='violet-label'>
                                        {$this->translator->_("report.printViewProductsSoldByCustomer.totals")}
                                        {$productsSumForPointOfSale[$currentProductsPointOfSaleId]["name"]}
                                    </i>
                                </td>
                                <td>{$productsSumForPointOfSale[$currentProductsPointOfSaleId]["quantity"]}</td>
                                <td colspan='2'></td>
                                <td class='right aligned'>{$pointOfSaleProductsSoldSum}</td>
                                <td class='right aligned'>{$pointOfSaleProductsProfit}</td>
                            </tr>
                        <tbody>
                        <tfoot>
                            <tr>
                                <th style='width: 40%;'>{$this->translator->_("report.printViewProductsSoldByCustomer.designation")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByCustomer.quantity")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByCustomer.buyingPrice")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByCustomer.unitPrice")}</th>
                                <th style='width: 15%;'>{$this->translator->_("report.printViewProductsSoldByCustomer.sumOfSales")}</th>
                                <th style='width: 15%;'>{$this->translator->_("report.printViewProductsSoldByCustomer.profit")}</th>
                            </tr>
                        </tfoot>
                    </table>
                    <hr/>
                ";
            }
            if (count($pointsOfSale) !== 1 && count($productsCount) > 0) {
                $quantityTotal = 0;
                $salesSumTotal = 0;
                $profitTotal = 0;
                $summaryProductsCategoryId = null;
                $summaryProductsCategoryName = null;
                $summaryCategoryTotalQuantity = 0;
                $summaryCategoryTotalSalesSum = 0;
                $summaryCategoryTotalProfit = 0;
                $documentHtml .= "
                    <table class='border'>
                        <thead>
                            <tr>
                                <th colspan='6' class='left aligned'>
                                    <i class='secondary-label'>{$this->translator->_("report.printViewProductsSoldByCustomer.summaryOfProductsSold")}</i>
                                </th>
                            </tr>
                            <tr>
                                <th style='width: 40%;'>{$this->translator->_("report.printViewProductsSoldByCustomer.designation")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByCustomer.quantity")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByCustomer.buyingPrice")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByCustomer.unitPrice")}</th>
                                <th style='width: 15%;'>{$this->translator->_("report.printViewProductsSoldByCustomer.sumOfSales")}</th>
                                <th style='width: 15%;'>{$this->translator->_("report.printViewProductsSoldByCustomer.profit")}</th>
                            </tr>
                        </thead>
                        <tbody>
                ";
                foreach ($productsSumGlobal as $productId => $productValues) {
                    $quantityTotal += $productValues["quantity"];
                    $salesSumTotal += $productValues["salesSum"];
                    $profitTotal += $productValues["profit"];
                    if ($productValues["categoryId"] != $summaryProductsCategoryId) {
                        if ($summaryProductsCategoryId != null) {
                            $categoryTotalSalesSum = number_format($summaryCategoryTotalSalesSum, 0, '', ' ');
                            $categoryTotalProfit = number_format($summaryCategoryTotalProfit, 0, '', ' ');
                            $documentHtml .= "
                                <tr class='app-total-tr'>
                                    <td class='right aligned'>
                                        {$this->translator->_("report.printViewProductsSoldByCustomer.totals")}
                                        {$summaryProductsCategoryName}
                                    </td>
                                    <td>{$summaryCategoryTotalQuantity}</td>
                                    <td colspan='2'></td>
                                    <td class='right aligned'>{$categoryTotalSalesSum}</td>
                                    <td class='right aligned'>{$categoryTotalProfit}</td>
                                </tr>
                            ";
                        }
                        $documentHtml .= "
                            <tr class='app-highlight-tr'>
                                <td colspan='6'>{$productValues["category"]}</td>
                            </tr>
                        ";
                        $summaryProductsCategoryId = $productValues["categoryId"];
                        $summaryProductsCategoryName = $productValues["category"];
                        $summaryCategoryTotalQuantity = $productValues["quantity"];
                        $summaryCategoryTotalSalesSum = $productValues["salesSum"];
                        $summaryCategoryTotalProfit = $productValues["profit"];
                    } else {
                        $summaryCategoryTotalQuantity += $productValues["quantity"];
                        $summaryCategoryTotalSalesSum += $productValues["salesSum"];
                        $summaryCategoryTotalProfit += $productValues["profit"];
                    }
                    $productBuyingPrice = number_format($productValues["buyingPrice"], 0, '', ' ');
                    $productUnitPrice = number_format($productValues["unitPrice"], 0, '', ' ');
                    $productSalesSum = number_format($productValues["salesSum"], 0, '', ' ');
                    $productProfit = number_format($productValues["profit"], 0, '', ' ');
                    $documentHtml .= "
                        <tr>
                            <td>{$productValues["name"]}</td>
                            <td>{$productValues["quantity"]}</td>
                            <td class='right aligned'>{$productBuyingPrice}</td>
                            <td class='right aligned'>{$productUnitPrice}</td>
                            <td class='right aligned'>{$productSalesSum}</td>
                            <td class='right aligned'>{$productProfit}</td>
                        </tr>
                    ";
                };
                if ($summaryProductsCategoryId != null) {
                    $categoryTotalSalesSum = number_format($summaryCategoryTotalSalesSum, 0, '', ' ');
                    $categoryTotalProfit = number_format($summaryCategoryTotalProfit, 0, '', ' ');
                    $documentHtml .= "
                        <tr class='app-total-tr'>
                            <td class='right aligned'>
                                {$this->translator->_("report.printViewProductsSoldByCustomer.totals")}
                                {$summaryProductsCategoryName}
                            </td>
                            <td>{$summaryCategoryTotalQuantity}</td>
                            <td colspan='2'></td>
                            <td class='right aligned'>{$categoryTotalSalesSum}</td>
                            <td class='right aligned'>{$categoryTotalProfit}</td>
                        </tr>
                    ";
                }
                $salesSumTotalFormat = number_format($salesSumTotal, 0, '', ' ');
                $profitTotalFormat = number_format($profitTotal, 0, '', ' ');
                $documentHtml .= "
                            <tr class='app-total-tr'>
                                <td class='right aligned'>
                                    <i class='secondary-label'>{$this->translator->_("report.printViewProductsSoldByCustomer.totals")}</a>
                                </td>
                                <td>{$quantityTotal}</td>
                                <td colspan='2'></span>
                                <td class='right aligned'>{$salesSumTotalFormat}</td>
                                <td class='right aligned'>{$profitTotalFormat}</td>
                            </tr>
                        </tbody>
                        <tfoot>
                            <tr>
                                <th style='width: 40%;'>{$this->translator->_("report.printViewProductsSoldByCustomer.designation")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByCustomer.quantity")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByCustomer.buyingPrice")}</th>
                                <th style='width: 10%;'>{$this->translator->_("report.printViewProductsSoldByCustomer.unitPrice")}</th>
                                <th style='width: 15%;'>{$this->translator->_("report.printViewProductsSoldByCustomer.sumOfSales")}</th>
                                <th style='width: 15%;'>{$this->translator->_("report.printViewProductsSoldByCustomer.profit")}</th>
                            </tr>
                        </tfoot>
                    </table>
                ";
            }
            $documentHtml .= " 
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper("a4", "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "products_sold_per_customer_{$_SESSION[SESSION_USER_ID]}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/reports/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/reports/{$filename}";
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 100 success
     * 101 unknown error
     */
    public function printloyaltycardAction()
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/cards") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $loyaltyCardDao = new LoyaltyCardDAO();
            $loyaltyCardId = (int)$_POST["card"];
            $loyaltyCard = $loyaltyCardDao->getLoyaltyCardById($loyaltyCardId);
            $barcode = new \Com\Tecnick\Barcode\Barcode();
            $loyaltyCardBarCodeImageName = "loyalty_card_bar_code_{$loyaltyCardId}.png";
            $targetPath = PUBLIC_PATH . "/temporary/{$loyaltyCardBarCodeImageName}";
            $barcodeObject = $barcode->getBarcodeObj('C128C', $loyaltyCard->getCode(), 450, 70, 'white', [0, 0, 0, 0]);
            $barcodeImageData = $barcodeObject->getPngData();
            file_put_contents($targetPath, $barcodeImageData);
            $companyLogo = LOGO_STRUCTURE;
            $barcodeImageLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/temporary/{$loyaltyCardBarCodeImageName}";
            $companyLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/business/{$companyLogo}";
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($loyaltyCard->getCode());
            $primaryBrand = APP_PRIMARY_BRAND;
            $secondaryBrand = APP_SECONDARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $documentHtml = "
                <html>
                    <body>
                        <style>
                            @page {
                                margin: 0 0;
                            }

                            .main {
                                padding-left: 0;
                                padding-right: 0;
                            }

                            .first-page {
                                height: 186px;
                                width: 100%;
                            }

                            .second-page {
                                height: 100%;
                                width: 100%;
                                background-color: {$primaryBrand};
                            }

                            .left-side {
                                width: 70%;
                                height: 100%;
                                background-color: {$primaryBrand};
                                border-right: solid 3px {$thirdBrand};
                            }

                            .left-side-top {
                                width: 100%;
                                height: 75%;
                                border-bottom: solid 3px {$thirdBrand};
                                padding-left: 10px;
                                padding-right: 10px;
                            }

                            .left-side-bottom {
                                width: 100%;
                                height: calc(25% - 1px);
                                padding-left: 10px;
                                padding-right: 10px;
                            }

                            .right-side {
                                width: 30%;
                                height: 100%;
                                background-color: {$primaryBrand};
                                position: absolute;
                                left: 70%;
                                top: 0;
                                z-index: 10;
                                border-left: solid 3px {$thirdBrand};
                            }

                            .company-logo-front {
                                width: 60px;
                                height: 60px;
                                border-radius: 6px;
                                margin-top: 8px;
                                margin-bottom: 5px;
                                border: solid 3px {$thirdBrand};
                            }

                            .point-of-sale-front {
                                font-weight: bold;
                                font-size: 18px;
                                color: {$thirdBrand};
                                width: 100%;
                                text-align: center;
                                font-family: 'monospace';
                                text-transform: uppercase;
                            }

                            .loyalty-card {
                                font-weight: bold;
                                font-size: 25px;
                                color: {$secondaryBrand};
                                width: 100%;
                                text-align: center;
                                font-family: 'wallpoet';
                                font-style: italic;
                            }

                            .customer-name {
                                font-weight: bold;
                                font-size: 15px;
                                width: 100%;
                                color: {$thirdBrand};
                                font-family: 'give you glory';
                                text-transform: uppercase;
                            }

                            .scan-info {
                                margin-top: 8px;
                                font-size: 10px;
                                font-weight: bold;
                                color: {$thirdBrand};
                                width: 100%;
                                text-align: center;
                                font-family: 'fixed';
                                font-style: italic;
                            }

                            .card-bar-code {
                                width: 80px;
                                height: 35px;
                            }

                            .code-info {
                                width: 100%;
                                margin-top: 5px;
                                text-align: center;
                                font-size: 10px;
                                font-weight: bold;
                                font-family: 'fixed';
                                color: {$thirdBrand};
                            }

                            .scan-info-bis {
                                margin-top: 5px;
                                color: {$thirdBrand};
                                width: 100%;
                                text-align: center;
                            }

                            .card-qr-code {
                                width: 80px;
                                height: 75px;
                            }

                            .company-logo-back {
                                width: 60px;
                                height: 60px;
                                border-radius: 6px;
                                margin-top: 8px;
                                margin-bottom: 5px;
                                border: solid 3px {$thirdBrand};
                            }

                            .point-of-sale-back {
                                font-weight: bold;
                                font-size: 18px;
                                color: {$thirdBrand};
                                width: 100%;
                                text-align: center;
                                font-family: 'monospace';
                                text-transform: uppercase;
                            }

                            .term-conditions {
                                font-weight: bold;
                                font-size: 16px;
                                color: {$secondaryBrand};
                            }

                            .term-condition-line {
                                font-weight: bold;
                                font-size: 13px;
                                color: {$thirdBrand};
                            }

                            .term-dot {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                bottom: 3px;
                            }
                        </style>
                        <div class='main'>
                            <div class='first-page'>
                                <div class='left-side'>
                                    <div class='left-side-top'>
                                        <div style='width: 100%;'>
                                            <img class='company-logo-front' src='{$companyLogoLink}' />
                                        </div>
                                        <div style='width: 100%;'>
                                            <span class='point-of-sale-front'>{$loyaltyCard->getIdPointVente()->getNomPointVente()}</span>
                                        </div>
                                        <div style='width: 100%;'>
                                            <span class='loyalty-card'>{$this->translator->_("report.printLoyaltyCard.loyaltyCard")}</span>
                                        </div>
                                    </div>
                                    <div class='left-side-bottom'>
                                        <div style='margin-top: 10px;'>
                                            <span class='customer-name'>{$loyaltyCard->getIdPersonne()->getNom()} {$loyaltyCard->getIdPersonne()->getPrenom()}</span>
                                        </div>
                                    </div>
                                </div>
                                <div class='right-side'>
                                    <div class='scan-info'>
                                        {$this->translator->_("report.printLoyaltyCard.scanToKeepTrackOfPoints")}
                                    </div>
                                    <div style='width: 100%; margin-top: 5px; text-align: center;'>
                                        <img class='card-bar-code' src='{$barcodeImageLink}' />
                                    </div>
                                    <div class='code-info'>
                                        {$loyaltyCard->getCode()}
                                    </div>
                                    <div class='scan-info-bis'>
                                        <img class='card-qr-code' src='{$qrCodeImage}' />
                                    </div>
                                    <div style='width: 100%; text-align: center; font-size: 10px; font-weight: bold; color: {$thirdBrand}; padding-top: 3px;'>
                                        Exp: {$loyaltyCard->getDateExpiration()->format(DATE_FORMAT)}
                                    </div>
                                </div>
                            </div>
                            <div class='second-page'>
                                <div style='width: 100%; text-align: center;'>
                                    <img class='company-logo-back' src='{$companyLogoLink}' />
                                </div>
                                <div style='width: 100%; text-align: center; border-bottom: solid 3px {$thirdBrand};'>
                                    <span class='point-of-sale-back'>{$loyaltyCard->getIdPointVente()->getNomPointVente()}</span>
                                </div>
                                <div style='width: 100%; text-align: center; margin-top: 5px;'>
                                    <span class='term-conditions'>{$this->translator->_("report.printLoyaltyCard.termsAndConditions")}</span>
                                </div>
                                <div style='width: 100%; text-align: center;'>
                                    <div>
                                        <span class='term-condition-line'><i class='term-dot'>.</i> {$this->translator->_("report.printLoyaltyCard.pointsGatheredOnThisCardAreNonTransferrable")}</span>
                                    </div>
                                    <div>
                                        <span class='term-condition-line'><i class='term-dot'>.</i> {$this->translator->_("report.printLoyaltyCard.pointsAreValidOnlyUntilExpirationDate")}</span>
                                    </div>
                                </div>
                            </div>
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper(LOYALTY_CARD_DIMENSIONS, "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "loyalty_card_{$_SESSION[SESSION_USER_ID]}_{$loyaltyCardId}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/cards/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/cards/{$filename}";
            $currentDate = new DateTime();
            $printCode = Utility::generatePrintCode($currentDate);
            $printTitle = "{$this->translator->_("report.printLoyaltyCard.loyaltyCardNumber")} {$loyaltyCard->getCode()}({$loyaltyCardId})";
            $printFilters = $this->translator->_("report.noFilter");
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    public function printvoucherAction()
    {
        $customerVoucherDao = new CustomerVoucherDAO();
        $customerVoucherId = (int)$_POST["id"];
        $customerVoucher = $customerVoucherDao->getCustomerVoucherById($customerVoucherId);
        if ($customerVoucher->getIdCarteFidelite() != null) {
            $this->printgiftvoucherAction($customerVoucher);
        } else {
            $this->printpropervoucherAction($customerVoucher);
        }
    }

    /**
     * 
     * @param \Application\Models\BonAchatClient $customerVoucher
     * 100 success
     * 101 unknown error
     */
    private function printpropervoucherAction($customerVoucher)
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/vouchers") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $barcode = new \Com\Tecnick\Barcode\Barcode();
            $voucherBarCodeImageName = "voucher_bar_code_{$customerVoucher->getIdBonAchatClient()}.png";
            $targetPath = PUBLIC_PATH . "/temporary/{$voucherBarCodeImageName}";
            $barcodeObject = $barcode->getBarcodeObj('C128C', $customerVoucher->getCodeBonAchatClient(), 450, 70, 'white', [0, 0, 0, 0]);
            $barcodeImageData = $barcodeObject->getPngData();
            file_put_contents($targetPath, $barcodeImageData);
            $companyLogo = LOGO_STRUCTURE;
            $barcodeImageLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/temporary/{$voucherBarCodeImageName}";
            $companyLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/business/{$companyLogo}";
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($customerVoucher->getCodeBonAchatClient());
            $primaryBrand = APP_PRIMARY_BRAND;
            $secondaryBrand = APP_SECONDARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $voucherAmount = number_format($customerVoucher->getMontant(), 0, '', ' ');
            $documentHtml = "
                <html>
                    <body>
                        <style>
                            @page {
                                margin: 0 0;
                            }

                            .main {
                                padding-left: 0;
                                padding-right: 0;
                            }

                            .first-page {
                                height: 186px;
                                width: 100%;
                            }

                            .second-page {
                                height: 100%;
                                width: 100%;
                                background-color: {$primaryBrand};
                            }

                            .left-side {
                                width: 70%;
                                height: 100%;
                                background-color: {$primaryBrand};
                                border-right: solid 3px {$thirdBrand};
                            }

                            .left-side-top {
                                width: 100%;
                                height: 75%;
                                border-bottom: solid 3px {$thirdBrand};
                                padding-left: 10px;
                                padding-right: 10px;
                            }

                            .left-side-bottom {
                                width: 100%;
                                height: calc(25% - 1px);
                                padding-left: 10px;
                                padding-right: 10px;
                            }

                            .right-side {
                                width: 30%;
                                height: 100%;
                                background-color: {$primaryBrand};
                                position: absolute;
                                left: 70%;
                                top: 0;
                                z-index: 10;
                                border-left: solid 3px {$thirdBrand};
                            }

                            .company-logo-front {
                                width: 60px;
                                height: 60px;
                                border-radius: 6px;
                                margin-top: 8px;
                                margin-bottom: 5px;
                                border: solid 3px {$thirdBrand};
                            }

                            .point-of-sale-front {
                                font-weight: bold;
                                font-size: 18px;
                                color: {$thirdBrand};
                                width: 100%;
                                text-align: center;
                                font-family: 'monospace';
                                text-transform: uppercase;
                            }

                            .voucher {
                                font-weight: bold;
                                font-size: 22px;
                                color: {$secondaryBrand};
                                width: 100%;
                                text-align: center;
                                font-family: 'wallpoet';
                                font-style: italic;
                            }

                            .customer-name {
                                font-weight: bold;
                                font-size: 15px;
                                width: 100%;
                                color: {$thirdBrand};
                                font-family: 'give you glory';
                                text-transform: uppercase;
                            }

                            .scan-info {
                                margin-top: 8px;
                                font-size: 10px;
                                font-weight: bold;
                                color: {$thirdBrand};
                                width: 100%;
                                text-align: center;
                                font-family: 'fixed';
                                font-style: italic;
                            }

                            .voucher-bar-code {
                                width: 80px;
                                height: 35px;
                            }

                            .code-info {
                                width: 100%;
                                margin-top: 5px;
                                text-align: center;
                                font-size: 10px;
                                font-weight: bold;
                                font-family: 'fixed';
                                color: {$thirdBrand};
                            }

                            .scan-info-bis {
                                margin-top: 5px;
                                color: {$thirdBrand};
                                width: 100%;
                                text-align: center;
                            }

                            .voucher-qr-code {
                                width: 80px;
                                height: 75px;
                            }

                            .company-logo-back {
                                width: 60px;
                                height: 60px;
                                border-radius: 6px;
                                margin-top: 8px;
                                margin-bottom: 5px;
                                border: solid 3px {$thirdBrand};
                            }

                            .point-of-sale-back {
                                font-weight: bold;
                                font-size: 18px;
                                color: {$thirdBrand};
                                width: 100%;
                                text-align: center;
                                font-family: 'monospace';
                                text-transform: uppercase;
                            }

                            .term-conditions {
                                font-weight: bold;
                                font-size: 16px;
                                color: {$secondaryBrand};
                            }

                            .term-condition-line {
                                font-weight: bold;
                                font-size: 13px;
                                color: {$thirdBrand};
                            }

                            .term-dot {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                bottom: 3px;
                            }
                        </style>
                        <div class='main'>
                            <div class='first-page'>
                                <div class='left-side'>
                                    <div class='left-side-top'>
                                        <div style='width: 100%;'>
                                            <img class='company-logo-front' src='{$companyLogoLink}' />
                                        </div>
                                        <div style='width: 100%;'>
                                            <span class='point-of-sale-front'>{$customerVoucher->getIdPointVente()->getNomPointVente()}</span>
                                        </div>
                                        <div style='width: 100%;'>
                                            <span class='voucher'>{$this->translator->_("report.printVoucher.voucher")} <i style='color: {$thirdBrand}'>({$voucherAmount} {$this->translator->_("shared.xaf")})<i></span>
                                        </div>
                                    </div>
                                    <div class='left-side-bottom'>
                                        <div style='margin-top: 10px;'>
                                            <span class='customer-name'>{$customerVoucher->getIdPersonne()->getNom()} {$customerVoucher->getIdPersonne()->getPrenom()}</span>
                                        </div>
                                    </div>
                                </div>
                                <div class='right-side'>
                                    <div class='scan-info'>
                                        {$this->translator->_("report.printVoucher.scanHere")}
                                    </div>
                                    <div style='width: 100%; margin-top: 5px; text-align: center;'>
                                        <img class='voucher-bar-code' src='{$barcodeImageLink}' />
                                    </div>
                                    <div class='code-info'>
                                        {$customerVoucher->getCodeBonAchatClient()}
                                    </div>
                                    <div class='scan-info-bis'>
                                        <img class='voucher-qr-code' src='{$qrCodeImage}' />
                                    </div>
                                    <div style='width: 100%; text-align: center; font-size: 10px; font-weight: bold; color: {$thirdBrand}; padding-top: 3px;'>
                                        Exp: {$customerVoucher->getDateExpiration()->format(DATE_FORMAT)}
                                    </div>
                                </div>
                            </div>
                            <div class='second-page'>
                                <div style='width: 100%; text-align: center;'>
                                    <img class='company-logo-back' src='{$companyLogoLink}' />
                                </div>
                                <div style='width: 100%; text-align: center; border-bottom: solid 3px {$thirdBrand};'>
                                    <span class='point-of-sale-back'>{$customerVoucher->getIdPointVente()->getNomPointVente()}</span>
                                </div>
                                <div style='width: 100%; text-align: center; margin-top: 5px;'>
                                    <span class='term-conditions'>{$this->translator->_("report.printVoucher.termsAndConditions")}</span>
                                </div>
                                <div style='width: 100%; text-align: center;'>
                                    <div>
                                        <span class='term-condition-line'><i class='term-dot'>.</i> {$this->translator->_("report.printVoucher.thisVoucherCanOnlyBeUsedUntilItsExpiryDate")}</span>
                                    </div>
                                    <div>
                                        <span class='term-condition-line'><i class='term-dot'>.</i> {$this->translator->_("report.printVoucher.thisVoucherCanOnlyBeUsedOnce")}</span>
                                    </div>
                                </div>
                            </div>
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper(VOUCHER_DIMENSIONS, "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "voucher_{$_SESSION[SESSION_USER_ID]}_{$customerVoucher->getIdBonAchatClient()}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/vouchers/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/vouchers/{$filename}";
            $currentDate = new DateTime();
            $printCode = Utility::generatePrintCode($currentDate);
            $printTitle = "{$this->translator->_("report.printVoucher.voucherCode")} {$customerVoucher->getCodeBonAchatClient()}({$customerVoucher->getIdBonAchatClient()})";
            $printFilters = $this->translator->_("report.noFilter");
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }

    /**
     * 
     * @param \Application\Models\BonAchatClient $customerVoucher
     * 100 success
     * 101 unknown error
     */
    private function printgiftvoucherAction($customerVoucher)
    {
        try {
            //Before all delete files older than 1 hour
            foreach (new DirectoryIterator(PUBLIC_PATH . "/documents/vouchers") as $fileInfo) {
                if ($fileInfo->isDot()) continue;
                if ($fileInfo->isFile() && time() - $fileInfo->getCTime() >= 60 * 60) {
                    unlink($fileInfo->getRealPath());
                }
            }
            $barcode = new \Com\Tecnick\Barcode\Barcode();
            $voucherBarCodeImageName = "voucher_bar_code_{$customerVoucher->getIdBonAchatClient()}.png";
            $targetPath = PUBLIC_PATH . "/temporary/{$voucherBarCodeImageName}";
            $barcodeObject = $barcode->getBarcodeObj('C128C', $customerVoucher->getCodeBonAchatClient(), 450, 70, 'white', [0, 0, 0, 0]);
            $barcodeImageData = $barcodeObject->getPngData();
            file_put_contents($targetPath, $barcodeImageData);
            $companyLogo = LOGO_STRUCTURE;
            $barcodeImageLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/temporary/{$voucherBarCodeImageName}";
            $companyLogoLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/business/{$companyLogo}";
            $giftVoucherIndicatorLink = "http://{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/images/defaults/gift_voucher_indicator.png";
            $qrCodeOptions = new chillerlan\QRCode\QROptions(
                [
                    'eccLevel' => chillerlan\QRCode\QRCode::ECC_L,
                    'outputType' => chillerlan\QRCode\QRCode::OUTPUT_MARKUP_SVG,
                    'version' => 5,
                ]
            );
            $qrCodeImage = (new chillerlan\QRCode\QRCode($qrCodeOptions))->render($customerVoucher->getCodeBonAchatClient());
            $primaryBrand = APP_PRIMARY_BRAND;
            $secondaryBrand = APP_SECONDARY_BRAND;
            $thirdBrand = APP_THIRD_BRAND;
            $voucherAmount = number_format($customerVoucher->getMontant(), 0, '', ' ');
            $documentHtml = "
                <html>
                    <body>
                        <style>
                            @page {
                                margin: 0 0;
                            }

                            .main {
                                padding-left: 0;
                                padding-right: 0;
                            }

                            .first-page {
                                height: 186px;
                                width: 100%;
                            }

                            .second-page {
                                height: 100%;
                                width: 100%;
                                background-color: {$primaryBrand};
                            }

                            .left-side {
                                width: 70%;
                                height: 100%;
                                background-color: {$primaryBrand};
                                border-right: solid 3px {$thirdBrand};
                            }

                            .left-side-top {
                                width: 100%;
                                height: 75%;
                                border-bottom: solid 3px {$thirdBrand};
                                padding-left: 10px;
                                padding-right: 10px;
                            }

                            .left-side-bottom {
                                width: 100%;
                                height: calc(25% - 1px);
                                padding-left: 10px;
                                padding-right: 10px;
                            }

                            .right-side {
                                width: 30%;
                                height: 100%;
                                background-color: {$primaryBrand};
                                position: absolute;
                                left: 70%;
                                top: 0;
                                z-index: 10;
                                border-left: solid 3px {$thirdBrand};
                            }

                            .company-logo-front {
                                width: 60px;
                                height: 60px;
                                border-radius: 6px;
                                margin-top: 8px;
                                margin-bottom: 5px;
                                border: solid 3px {$thirdBrand};
                                position: relative;
                                top: 15px;
                            }

                            .gift-indicator {
                                width: 60px;
                                height: 60px;
                                border-radius: 6px;
                                margin-bottom: 5px;
                                position: relative;
                                left: 145px;
                                top: 50px;
                            }

                            .point-of-sale-front {
                                font-weight: bold;
                                font-size: 18px;
                                color: {$thirdBrand};
                                width: 100%;
                                text-align: center;
                                font-family: 'monospace';
                                text-transform: uppercase;
                            }

                            .voucher {
                                font-weight: bold;
                                font-size: 22px;
                                color: {$secondaryBrand};
                                width: 100%;
                                text-align: center;
                                font-family: 'wallpoet';
                                font-style: italic;
                            }

                            .customer-name {
                                font-weight: bold;
                                font-size: 15px;
                                width: 100%;
                                color: {$thirdBrand};
                                font-family: 'give you glory';
                                text-transform: uppercase;
                            }

                            .scan-info {
                                margin-top: 8px;
                                font-size: 10px;
                                font-weight: bold;
                                color: {$thirdBrand};
                                width: 100%;
                                text-align: center;
                                font-family: 'fixed';
                                font-style: italic;
                            }

                            .voucher-bar-code {
                                width: 80px;
                                height: 35px;
                            }

                            .code-info {
                                width: 100%;
                                margin-top: 5px;
                                text-align: center;
                                font-size: 10px;
                                font-weight: bold;
                                font-family: 'fixed';
                                color: {$thirdBrand};
                            }

                            .scan-info-bis {
                                margin-top: 5px;
                                color: {$thirdBrand};
                                width: 100%;
                                text-align: center;
                            }

                            .voucher-qr-code {
                                width: 80px;
                                height: 75px;
                            }

                            .company-logo-back {
                                width: 60px;
                                height: 60px;
                                border-radius: 6px;
                                margin-top: 8px;
                                margin-bottom: 5px;
                                border: solid 3px {$thirdBrand};
                            }

                            .point-of-sale-back {
                                font-weight: bold;
                                font-size: 18px;
                                color: {$thirdBrand};
                                width: 100%;
                                text-align: center;
                                font-family: 'monospace';
                                text-transform: uppercase;
                            }

                            .term-conditions {
                                font-weight: bold;
                                font-size: 16px;
                                color: {$secondaryBrand};
                            }

                            .term-condition-line {
                                font-weight: bold;
                                font-size: 13px;
                                color: {$thirdBrand};
                            }

                            .term-dot {
                                font-weight: bold;
                                font-size: 20px;
                                color: {$thirdBrand};
                                position: relative;
                                bottom: 3px;
                            }
                        </style>
                        <div class='main'>
                            <div class='first-page'>
                                <div class='left-side'>
                                    <div class='left-side-top'>
                                        <div style='width: 100%;'>
                                            <img class='company-logo-front' src='{$companyLogoLink}' />
                                            <img class='gift-indicator' src='{$giftVoucherIndicatorLink}' />
                                        </div>
                                        <div style='width: 100%;'>
                                            <span class='point-of-sale-front'>{$customerVoucher->getIdPointVente()->getNomPointVente()}</span>
                                        </div>
                                        <div style='width: 100%;'>
                                            <span class='voucher'>{$this->translator->_("report.printVoucher.voucher")} <i style='color: {$thirdBrand}'>({$voucherAmount} {$this->translator->_("shared.xaf")})<i></span>
                                        </div>
                                    </div>
                                    <div class='left-side-bottom'>
                                        <div style='margin-top: 10px;'>
                                            <span class='customer-name'>{$customerVoucher->getIdPersonne()->getNom()} {$customerVoucher->getIdPersonne()->getPrenom()}</span>
                                        </div>
                                    </div>
                                </div>
                                <div class='right-side'>
                                    <div class='scan-info'>
                                        {$this->translator->_("report.printVoucher.scanHere")}
                                    </div>
                                    <div style='width: 100%; margin-top: 5px; text-align: center;'>
                                        <img class='voucher-bar-code' src='{$barcodeImageLink}' />
                                    </div>
                                    <div class='code-info'>
                                        {$customerVoucher->getCodeBonAchatClient()}
                                    </div>
                                    <div class='scan-info-bis'>
                                        <img class='voucher-qr-code' src='{$qrCodeImage}' />
                                    </div>
                                    <div style='width: 100%; text-align: center; font-size: 10px; font-weight: bold; color: {$thirdBrand}; padding-top: 3px;'>
                                        Exp: {$customerVoucher->getDateExpiration()->format(DATE_FORMAT)}
                                    </div>
                                </div>
                            </div>
                            <div class='second-page'>
                                <div style='width: 100%; text-align: center;'>
                                    <img class='company-logo-back' src='{$companyLogoLink}' />
                                </div>
                                <div style='width: 100%; text-align: center; border-bottom: solid 3px {$thirdBrand};'>
                                    <span class='point-of-sale-back'>{$customerVoucher->getIdPointVente()->getNomPointVente()}</span>
                                </div>
                                <div style='width: 100%; text-align: center; margin-top: 5px;'>
                                    <span class='term-conditions'>{$this->translator->_("report.printVoucher.termsAndConditions")}</span>
                                </div>
                                <div style='width: 100%; text-align: center;'>
                                    <div>
                                        <span class='term-condition-line'><i class='term-dot'>.</i> {$this->translator->_("report.printVoucher.thisVoucherCanOnlyBeUsedUntilItsExpiryDate")}</span>
                                    </div>
                                    <div>
                                        <span class='term-condition-line'><i class='term-dot'>.</i> {$this->translator->_("report.printVoucher.thisVoucherCanOnlyBeUsedOnce")}</span>
                                    </div>
                                </div>
                            </div>
                        </div>
                    </body>
                </html>
            ";
            $domPdfOptions = new \Dompdf\Options();
            $domPdfOptions->set("isRemoteEnabled", true);
            $domPdf = new \Dompdf\Dompdf($domPdfOptions);
            $domPdf->setPaper(VOUCHER_DIMENSIONS, "portrait");
            $domPdf->loadHtml($documentHtml);
            $domPdf->setBasePath($_SERVER["DOCUMENT_ROOT"]);
            $domPdf->render();
            $output = $domPdf->output();
            $currentDate = new DateTime();
            $randomName = $currentDate->format("YmdHis");
            $filename = "voucher_{$_SESSION[SESSION_USER_ID]}_{$customerVoucher->getIdBonAchatClient()}_{$randomName}.pdf";
            $fileLocation = PUBLIC_PATH . "/documents/vouchers/{$filename}";
            file_put_contents($fileLocation, $output);
            $link = "{$this->getRequest()->getHttpHost()}{$this->view->baseUrl()}/documents/vouchers/{$filename}";
            $currentDate = new DateTime();
            $printCode = Utility::generatePrintCode($currentDate);
            $printTitle = "{$this->translator->_("report.printVoucher.voucherCode")} {$customerVoucher->getCodeBonAchatClient()}({$customerVoucher->getIdBonAchatClient()})";
            $printFilters = $this->translator->_("report.noFilter");
            PrintLogDAO::registerPrintForCurrentUser($printCode, $printTitle, $printFilters, $currentDate);
            echo ApiResponse::respond(REQUEST_SUCCESS_CODE, null, ["link" => $link]);
        } catch (Exception $ex) {
            echo ApiResponse::respond(101, $ex->getMessage());
        }
        exit();
    }
}
